Storage = class("Storage")

Storage.MAX_VOLUME = 10000 -- 容量上限

Storage.ITEM_TEMPLATE = {  -- item 数据结构定义
	
	s_tag = "storage_item",
	s_storage_id = nil,
	s_item_id = nil,
	n_amount = nil,
	s_category = nil,
	t_custom_data = nil,	

} 

Storage.INIT_CATEGORY = {
	
	"bag",
	"role",
	"skin",
	"card",

}

function Storage:get_instance()

	if not Storage.static_instance then

		Storage.static_instance = Storage:create()

	end

	return Storage.static_instance

end

function Storage:ctor()

	self.__t_storage = {} -- 存放所有的item，

	self.__t_category_list = {} -- 分类存放的item，

	self.__t_dirty_flags = {} -- dirty标记标记了经过修改的category 在save的过程中只有设置的dirty标记的category才会被存储

	self.__t_category_list["default"] = {} -- 添加默认分类
	self.__t_dirty_flags["default"] = false

	self.__n_list_dirty_flag = false

end

function Storage:get_item_by_storage_id(s_storage_id)

	return self.__t_storage[s_storage_id]

end

function Storage:remove_item_by_storage_id(s_storage_id)

	ERROR_ASSERT(self.__t_storage[s_storage_id] , "[Storage][change_item_at] : item does not exist")

	local item = self.__t_storage[s_storage_id]

	self.__t_storage[s_storage_id] = nil
	self.__t_category_list[item.s_category][item] = nil -- 从对应的category移除

	self.__t_dirty_flags[item.s_category] = true -- 设置category的dirty标记

end

function Storage:add_item(t_item, s_category)

	ERROR_ASSERT((t_item and t_item.s_tag == "storage_item"), "[Storage][add_item] : item nil or wrong type")
	ERROR_ASSERT(self:is_item_legal(t_item) , "[Storage][add_item] : illegal item")

	if s_category then

		ERROR_ASSERT(self.__t_category_list[s_category], "[Storage][add_item] : category not found")

		t_item.s_category = s_category

	else

		t_item.category = "default"  -- 如果没有指定分类则分配至默认的defualt分类 

	end

	for i = 1, Storage.MAX_VOLUME do    

		if not self.__t_storage[i] then  -- 为新加入的item分配一个唯一的id，分配方法为选择storage中第一个非空的数字键值

			t_item.s_storage_id = i
			self.__t_storage[i] = t_item

			self.__t_category_list[t_item.s_category][t_item] = t_item -- 加入对应的category

			self.__t_dirty_flags[t_item.s_category] = true  -- 设置category的dirty标记

			return

		end

	end

	ERROR_ASSERT(false, "[Storage][add_item] : storage overflow")  -- 如果所有的数字键值都被分配则返回错误

end

function Storage:change_item_at(s_storage_id)

	local item = self.__t_storage[s_storage_id]

	ERROR_ASSERT(item , "[Storage][change_item_at] : item does not exist")
	ERROR_ASSERT(self:is_item_legal(item) , "[Storage][change_item_at] : illegal item")

	self.__t_dirty_flags[item.s_category] = true

end

function Storage:is_item_legal(t_item)

	return true

end

function Storage:get_category(s_category)

	ERROR_ASSERT(self.__t_category_list[s_category], "[Storage][get_category] : category already exists")

	return self.__t_category_list -- 返回该分类下的所有item

end

function Storage:add_category(s_category)

	ERROR_ASSERT(self.__t_category_list[s_category] == nil, "[Storage][get_category] : category already exists")

	self.__t_category_list[s_category] = {}

	self.__n_list_dirty_flag = true -- 设置分类列表的dirty标记
	self.__t_dirty_flags[s_category] = true -- 设置新加分类的dirty标记

end

function Storage:remove_category(s_category)

	local category = self.__t_category_list[s_category]

	if category then

		for k, v in pairs(category) do

			self.__t_storage[v.s_storage_id] = nil

		end

	end

	self.__t_category_list[s_category] = nil 
	self.__t_dirty_flags[s_category] = nil -- 移除分类的dirty标记

	self.__n_list_dirty_flag = true -- 设置分类列表的dirty标记

end

function Storage:has_category(s_category)

	if self.__t_category_list[s_category] then

		return true

	else

		return false

	end

end

function Storage:clear_category(s_category)

	ERROR_ASSERT(self.__t_category_list[s_category], "[Storage][clear_category] : category does not exists")

	local category = self.__t_category_list[s_category]

	if category then

		for k, v in pairs(category) do

			self.__t_storage[v.s_storage_id] = nil

		end

	end

	self.__t_dirty_flags[s_category] = true

end

function Storage:create_item(t_item_data)

	local new_item = {

		s_tag = "storage_item",
		s_storage_id = nil,
		s_item_id = nil,
		n_amount = nil,
		s_category = nil,
		t_custom_data = nil,

	}

	ERROR_ASSERT((t_item_data.s_item_id == nil or not type(t_item_data.s_item_id) == "string"), "[Storage][create_item] : item_id nil or wrong type")

	new_item.s_item_id = t_item_data.s_item_id
	new_item.n_amount = t_item_data.n_amount or 1
	new_item.t_custom_data = t_item_data.t_custom_data

end

function Storage:query_item(t_query_config)



end

function Storage:first_time_init()

	self:add_category("bag")
	self:add_category("role")
	self:add_category("skin")
	self:add_category("card")
	self:add_category("mission")
	self:add_category("section")

end

function Storage:load()

	if cc.UserDefault:getInstance():getBoolForKey("storage_saved") then

		self:load_category_list()
		self:load_category()

	else

		self:first_time_init()

		self:save()

	end

end

function Storage:save()

	cc.UserDefault:getInstance():setBoolForKey("storage_saved", true)

	self:save_category_list()
	self:save_category()

end

function Storage:load_category_list()

	local categories = require "category_list"

	for i = 1, #categories do

		self:add_category(categories[i])

	end

end

function Storage:save_category_list()

	if self.__t_dirty_flags["category_list"] then

		local categories = {}

		for k, _ in pairs(self.__t_category_list) do

			categories[#categories + 1] = k

		end

		FileManipulator:getInstance():writeToFile("category_list", "tbl", categories)

	end

end

function Storage:load_category()

	for s_category, category in pairs(self.__t_category_list) do

		local items = require(s_category)

		for i = 1, #items do

			local item = items[i]

			category[item] = item

			self.__t_storage[item.s_storage_id] = item

		end

	end

end

function Storage:save_category()

	for s_category, flag in self.__t_dirty_flags do

		if flag then

			local category = self.__t_category_list[s_category]
			local items = {}

			for _, v in pairs(category) do

				items[#items + 1] = v

			end

			FileManipulator:getInstance():writeToFile("s_category", "tbl", items)

		end

	end

end

return Storage 