-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"

-- ## Imports ## --

local BoxCollider = class("BoxCollider", function() return GameObject:create() end)

function BoxCollider:ctor()

	self.__c_body = cc.PhysicsBody:createBox(cc.size(1, 1), cc.PhysicsMaterial(0.1, 0.0, 0.0))
	self.__c_body:retain()

	self:register_listeners_with_config(BoxCollider.LISTENER_CONFIG)
	self:register_properties_with_config(BoxCollider.PROPERTY_CONFIG)

end

function BoxCollider:on_assemble(c_msg)

	self.__c_parent:addComponent(self.__c_body)

end

function BoxCollider:on_disassemble(c_msg)

	self.__c_parent:removeComponent(self.__c_body)

end

function BoxCollider:on_destroy()

	self.__c_body:release()

end

function BoxCollider:after_set_width(c_msg)

	self.__c_body:setDefaultScale(c_msg.x_param.x_new_value, self:get("height") or 1)

end

function BoxCollider:after_set_height(c_msg)

	self.__c_body:setDefaultScale(self:get("width") or 1, c_msg.x_param.x_new_value)

end

function BoxCollider:after_set_material(c_msg)

	local shape = self.__c_body:getShapes()[1]

	if shape then

		shape:setMaterial(c_msg.x_param.x_new_value)

	end

end

function BoxCollider:after_set_dynamic(c_msg)

	self.__c_body:setDynamic(c_msg.x_param.x_new_value)

end

function BoxCollider:after_set_velocity(c_msg)

	self.__c_body:setVelocity(c_msg.x_param.x_new_value)

end

-- function BoxCollider:after_set_moment(c_msg)

-- 	self.__c_body:setMoment(c_msg.x_param.x_new_value)

-- end

function BoxCollider:on_apply_impulse(c_msg)

	self.__c_body:applyImpulse(c_msg.x_param)

end

BoxCollider.LISTENER_CONFIG = {

	{"assemble", {before = BoxCollider.on_assemble}},
	{"disassemble", {before = BoxCollider.on_disassemble}},
	{"apply_impluse", {before = BoxCollider.on_apply_impulse}},
	{"destroy", {before = BoxCollider.on_destroy}},

}

BoxCollider.PROPERTY_CONFIG = {

	{"width", 100},
	{"height", 100},
	{"material", cc.PhysicsMaterial(0.1, 0.0, 0.0)},
	{"dynamic", false},
	{"velocity", cc.p(0, 0)},
	{"moment", PHYSICS_INFINITY},

}

return BoxCollider