-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"

-- ## Imports ## --

local CircleCollider = class("CircleCollider", function() return GameObject:create() end)

function CircleCollider:ctor()

	self.__c_body = cc.PhysicsBody:createCircle(1, cc.PhysicsMaterial(0.1, 0.0, 0.0))
	-- self.__c_body:setLinearDamping(0.6)
	self.__c_body:retain()

	self:register_listeners_with_config(CircleCollider.LISTENER_CONFIG)
	self:register_properties_with_config(CircleCollider.PROPERTY_CONFIG)

end

function CircleCollider:on_assemble(c_msg)

	self.__c_parent:addComponent(self.__c_body)

end

function CircleCollider:on_disassemble(c_msg)

	self.__c_parent:removeComponent(self.__c_body)

end

function CircleCollider:on_destroy()

	self.__c_body:release()

end

function CircleCollider:after_set_radius(c_msg)

	self.__c_body:setDefaultScale(c_msg.x_param.x_new_value, c_msg.x_param.x_new_value)

end

function CircleCollider:after_set_material(c_msg)

	local shape = self.__c_body:getShapes()[1]

	if shape then

		shape:setMaterial(c_msg.x_param.x_new_value)

	end

end

function CircleCollider:after_set_dynamic(c_msg)

	self.__c_body:setDynamic(c_msg.x_param.x_new_value)

end

function CircleCollider:after_set_velocity(c_msg)

	self.__c_body:setVelocity(c_msg.x_param.x_new_value)

end

function CircleCollider:after_set_moment(c_msg)

	self.__c_body:setMoment(c_msg.x_param.x_new_value)

end

function CircleCollider:on_apply_impulse(c_msg)
	
	-- self.__c_body:applyImpulse(c_msg.x_param)

end

CircleCollider.LISTENER_CONFIG = {

	{"assemble", {before = CircleCollider.on_assemble}},
	{"disassemble", {before = CircleCollider.on_disassemble}},
	{"apply_impulse", {before = CircleCollider.on_apply_impulse}},
	{"destroy", {before = CircleCollider.on_destroy}},

}

CircleCollider.PROPERTY_CONFIG = {

	{"radius", 60},
	{"material", cc.PhysicsMaterial(0.1, 0.0, 0.0)},
	{"dynamic", true},
	{"velocity", cc.p(0, 0)},
	{"moment", PHYSICS_INFINITY}

}

return CircleCollider