-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"

-- ## Imports ## --




local EdgeChainCollider = class("EdgeChainCollider", function() return GameObject:create() end)

function EdgeChainCollider:ctor()

	self.__c_body = cc.PhysicsBody:createEdgeChain({cc.p(0, 0)}, cc.PhysicsMaterial(0.1, 0.0, 0.0))
	self.__c_body:retain()

	self:register_listeners_with_config(EdgeChainCollider.LISTENER_CONFIG)
	self:register_properties_with_config(EdgeChainCollider.PROPERTY_CONFIG)

end

function EdgeChainCollider:on_assemble(c_msg)

	self.__c_parent:addComponent(self.__c_body)

end

function EdgeChainCollider:on_disassemble(c_msg)

	self.__c_parent:removeComponent(self.__c_body)

end

function EdgeChainCollider:on_destroy()

	self.__c_body:release()

end

function EdgeChainCollider:after_set_rotation(c_msg)

	self.__c_body:setRotationOffset(c_msg.x_param.x_new_value)

end

function EdgeChainCollider:after_set_material(c_msg)

	local shape = self.__c_body:getShapes()[1]

	if shape then

		shape:setMaterial(c_msg.x_param.x_new_value)

	end

end

function EdgeChainCollider:after_set_edges(c_msg)

	if self.__c_parent then

		self.__c_parent:removeComponent(self.__c_body)

	end

	self.__c_body:release()
	self.__c_body = cc.PhysicsBody:createEdgeChain(c_msg.x_param.x_new_value, self:get("material"))
	self.__c_body:setRotationOffset(self:get("rotation"))
	self.__c_body:retain()


	if self.__c_parent then

		self.__c_parent:addComponent(self.__c_body)

	end	

	print(#c_msg.x_param.x_new_value)

end

EdgeChainCollider.LISTENER_CONFIG = {

	{"assemble", {before = EdgeChainCollider.on_assemble}},
	{"disassemble", {before = EdgeChainCollider.on_disassemble}},
	{"destroy", {before = EdgeChainCollider.on_destroy}},

}

EdgeChainCollider.PROPERTY_CONFIG = {
	
	{"rotation", 0},
	{"material", cc.PhysicsMaterial(0.1, 0.1, 0.3)},
	{"edges", {cc.p(0, 0)}}

}

return EdgeChainCollider