-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"
local ConfigManager = require "GameEx.Foundations.ConfigManager"

-- ## Imports ## --




local AdvancedEmitter = class("AdvancedEmitter", function() return GameObject:create() end)


function AdvancedEmitter:ctor()

	self.__c_type_getter = ProbGetter:create(GameGlobals.RandomGenerator)
	self.__c_prob_getter_normal = ProbGetter:create(GameGlobals.RandomGenerator)
	self.__c_prob_getter_special = ProbGetter:create(GameGlobals.RandomGenerator)

	self.__n_task = 0

	self.__f_schedule_func = function()

		self:update_func()

	end

	self:get_category():add("emitter")

	self:register_listeners_with_config(AdvancedEmitter.LISTENER_CONFIG)
	self:register_properties_with_config(AdvancedEmitter.PROPERTY_CONFIG)

	self.__n_phase_index = 1
	self.__t_current_phase = nil
	self.__n_special_counter = 0
	self.__n_round_counter = 0
	self.__n_total_remove = 0
	self.__n_current_special_limit = 0

end

function AdvancedEmitter:on_destroy()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
	self.__n_schedule_entry = nil

end

function AdvancedEmitter:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

end

function AdvancedEmitter:on_enter()

	self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, self:get("emit_interval"), false)

end

function AdvancedEmitter:on_exit()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
	self.__n_schedule_entry = nil

end

function AdvancedEmitter:emit()

	local current_phase = self:get("span")[self.__n_phase_index]

	if current_phase then

		local special_phase = current_phase["special"]
		local drop_special = self.__n_enable_special_drop and self.__n_special_counter < self.__n_current_special_limit

		self.__c_type_getter:modify("special", drop_special and special_phase["special_prob"] or 0)
		
		local draw_func = self.__c_type_getter:get()

		if draw_func then

			self:send_to_nozzle(draw_func(self))

		end

	end

end

function AdvancedEmitter:draw_normal()

	local rand_color = self.__c_prob_getter_normal:get()

	if rand_color then

		local sugar = ObjectFactory:get_instance():get("NormalSugar")
		sugar:set("color", rand_color)

		return sugar

	end

end

function AdvancedEmitter:draw_special()

	local rand_config = self.__c_prob_getter_special:get()

	if rand_config then

		self.__n_special_counter = self.__n_special_counter + 1

		local sugar = ObjectFactory:get_instance():get(rand_config["type"])
		local config = ConfigManager:get_instance():get_config(rand_config["type"])
		local properties = {}

		for _, tile in ipairs(config.tiles) do

			if tile.id == rand_config["config_index"] then

				properties = tile.properties

			end

		end 

		for key, value in pairs(properties) do

			sugar:set(key, value)

		end

		return sugar

	end

end

function AdvancedEmitter:send_to_nozzle(sugar)

	self.__t_nozzles = self.__t_nozzles or self:find_nozzles()

	if sugar and #self.__t_nozzles > 0 then

		self.__t_nozzles[GameGlobals.RandomGenerator:rand(1, #self.__t_nozzles)]:send_message(MessagePool:get_free_ex("launch", sugar))

	end

end

function AdvancedEmitter:find_nozzles()

	local t_filter = {}
	t_filter.t_category_filter = {
		type = "and",
		{"advanced_emitter_nozzle", "=", true},	
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,
		no_pos_limitation = true,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	return msg.x_result

end

function AdvancedEmitter:update_func()

	if self.__n_task > 0 then

		self:emit()

		self.__n_task = self.__n_task - 1

	end

end

function AdvancedEmitter:after_set_emit_interval(c_msg)

	if self.__n_schedule_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
		self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, c_msg.x_param.x_new_value, false)

	end

end

function AdvancedEmitter:on_assign_task(c_msg)
print("advanced emitter assign task!!")
	self.__n_task = self.__n_task + c_msg.x_param
	self.__n_total_remove = self.__n_total_remove + c_msg.x_param

	self:change_drop_limit()

end


function AdvancedEmitter:change_drop_limit()

	local span = span or self:get("span")
	local current_phase = span[self.__n_phase_index]

	if current_phase then

		for _, config in ipairs(current_phase["special"]["special_number_config"]) do

			if config.threshold == "INFINITY" or self.__n_total_remove <= config.threshold then

				self.__n_current_special_limit = config.max_number
print("change limit!") print(self.__n_total_remove) print(config.max_number)
				break

			end

		end

	end	

end


function AdvancedEmitter:on_round_start(c_msg)
print("advanced emitter round start!!")
	if c_msg.x_param == "player" or c_msg.x_param == "item" then

		self.__n_enable_special_drop = true
		self.__n_special_counter = 0
		self.__n_current_special_limit = 0
		self.__n_total_remove = 0
print(self.__n_total_remove)
		self:phase_advance()
		self:change_drop_limit()

	elseif c_msg.x_param == "game_start" or c_msg.x_param == "bonus" then
print("advanced emitter disable!!")
		self.__n_enable_special_drop = false

	end

end

function AdvancedEmitter:phase_advance()

	local current_phase = self:get("span")[self.__n_phase_index]

	if current_phase then

		if not (current_phase.duration == "INFINITY") then

			self.__n_round_counter = self.__n_round_counter + 1

			if current_phase.duration < self.__n_round_counter then

				self:go_to_next_phase()
				self:phase_advance()

			end

		end

	end	

end

function AdvancedEmitter:go_to_next_phase(span)

	self.__n_phase_index = self.__n_phase_index + 1
	self.__n_round_counter = 0

	local span = span or self:get("span")
	local current_phase = span[self.__n_phase_index]

	if current_phase then

		self.__c_type_getter:clear()
		self.__c_prob_getter_normal:clear()
		self.__c_prob_getter_special:clear()

		self.__c_type_getter:add("normal", self.draw_normal, current_phase["normal"]["normal_prob"])
		self.__c_type_getter:add("special", self.draw_special, current_phase["special"]["special_prob"])

		for key, value in pairs(current_phase["normal"]["normal_config"]) do

			self.__c_prob_getter_normal:add(key, AdvancedEmitter.prob_key_map[key], value)

		end

		for index, config in ipairs(current_phase["special"]["special_config"]) do

			self.__c_prob_getter_special:add(index, config, config["prob"])

		end		

	end		

end

function AdvancedEmitter:after_set_span(c_msg)

	self.__n_phase_index = 0

	self:go_to_next_phase(c_msg.x_param.x_new_value)

end

function AdvancedEmitter:on_draw_color(c_msg)

	c_msg.x_result = self.__c_prob_getter_normal:get()

end

AdvancedEmitter.prob_key_map = {
	
	["prob_red"] = "red",
	["prob_green"] = "green",
	["prob_blue"] = "blue",
	["prob_yellow"] = "yellow",
	["prob_pink"] = "pink",
	["prob_white"] = "white",

}

AdvancedEmitter.LISTENER_CONFIG = {

	{"enter", {before = AdvancedEmitter.on_enter}},
	{"exit", {before = AdvancedEmitter.on_exit}},
	{"assign_task", {before = AdvancedEmitter.on_assign_task}},
	{"reset", {before = AdvancedEmitter.on_reset}},
	{"destroy", {before = AdvancedEmitter.on_destroy}},
	{"round_start", {before = AdvancedEmitter.on_round_start}},
	{"round_finish", {before = AdvancedEmitter.on_round_finish}},
	{"draw_color", {before = AdvancedEmitter.on_draw_color}},

}	

AdvancedEmitter.PROPERTY_CONFIG = {
	
	{"span", {}},
	{"emit_interval", 0.033},
	{"enable", true},

}


return AdvancedEmitter