-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"

-- ## Imports ## --




local AdvancedEmitterNozzle = class("AdvancedEmitterNozzle", function() return GameObject:create() end)


function AdvancedEmitterNozzle:ctor()

	self:get_category():add("advanced_emitter_nozzle")

	self:register_listeners_with_config(AdvancedEmitterNozzle.LISTENER_CONFIG)
	self:register_properties_with_config(AdvancedEmitterNozzle.PROPERTY_CONFIG)

end


function AdvancedEmitterNozzle:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

end

function AdvancedEmitterNozzle:on_launch(c_msg)

	local sugar = c_msg.x_param
	sugar:setPosition(cc.p(self:getPosition()))
	sugar:set("velocity", cc.p(self:get("emit_speed_x"), -self:get("emit_speed_y")))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar_without_count", sugar, Message.N_TUNNEL))

end

AdvancedEmitterNozzle.LISTENER_CONFIG = {

	{"launch", {before = AdvancedEmitterNozzle.on_launch}},
	{"reset", {before = AdvancedEmitterNozzle.on_reset}},

}	

AdvancedEmitterNozzle.PROPERTY_CONFIG = {
	
	{"emit_speed_x", 10},
	{"emit_speed_y", 300},

}


return AdvancedEmitterNozzle