-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"

-- ## Imports ## --




local KeyEmitter = class("KeyEmitter", function() return GameObject:create() end)


function KeyEmitter:ctor()

	self.__n_task = 0

	self.__f_schedule_func = function()

		self:update_func()

	end

	self:get_category():add("key_emitter")

	self:register_listeners_with_config(KeyEmitter.LISTENER_CONFIG)
	self:register_properties_with_config(KeyEmitter.PROPERTY_CONFIG)

end

function KeyEmitter:on_destroy()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
	self.__n_schedule_entry = nil

end

function KeyEmitter:on_enter()

	self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, self:get("emit_interval"), false)

end

function KeyEmitter:on_exit()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
	self.__n_schedule_entry = nil

end

function KeyEmitter:emit()

	local sugar = ObjectFactory:get_instance():get("KeySugar")
	sugar:setPosition(cc.p(self:getPosition()))
	sugar:set("velocity", cc.p(self:get("emit_speed_x"), -self:get("emit_speed_y")))
	sugar:set("material", cc.PhysicsMaterial(1, 0.0, 0.0))
	sugar:set("radius", 60)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", sugar))

end

function KeyEmitter:update_func()

	if self.__n_task > 0 then

		self:emit()

		self.__n_task = self.__n_task - 1

	end

end

function KeyEmitter:on_assign_task(c_msg)

	self.__n_task = self.__n_task + c_msg.x_param

end


function KeyEmitter:after_set_emit_interval(c_msg)

	if self.__n_schedule_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
		self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, c_msg.x_param.x_new_value, false)

	end

end


KeyEmitter.LISTENER_CONFIG = {

	{"enter", {before = KeyEmitter.on_enter}},
	{"exit", {before = KeyEmitter.on_exit}},
	{"assign_task", {before = KeyEmitter.on_assign_task}},
	{"destroy", {before = KeyEmitter.on_destroy}},
}	

KeyEmitter.PROPERTY_CONFIG = {
	
	{"emit_interval", 0.033},
	{"emit_speed_x", 10},
	{"emit_speed_y", 300},

}

return KeyEmitter