-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameScene = require "GameEx.Components.Scene.GameScene"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local KeyEmitterManager = class("KeyEmitterManager", function() return GameObject:create() end)


function KeyEmitterManager:ctor()

	self.__n_wave_index = 1
	self.__n_wave_count = 0

	self:register_listeners_with_config(KeyEmitterManager.LISTENER_CONFIG)
	self:register_properties_with_config(KeyEmitterManager.PROPERTY_CONFIG)

	self.__n_task_number = 0

end

function KeyEmitterManager:on_destroy()



end

function KeyEmitterManager:on_reset()



end

function KeyEmitterManager:on_direct_assign(c_msg)

	self.__n_task_number = self.__n_task_number + c_msg.x_param

end

function KeyEmitterManager:on_dispatch_task(c_msg)

	self:on_round_finish()

end

function KeyEmitterManager:on_remove_sugar(c_msg)

	local sugar = c_msg.x_param
	local category = sugar:get_category()

	if category:has("key") then

		self.__n_wave_count = self.__n_wave_count - 1

		if self.__n_wave_count == 0 then
print("CAKE REMOVE !!!!")
			self.__n_wave_index = self.__n_wave_index + 1
			local waves = self:get("waves")

			if self.__n_wave_index <= #waves then

				self.__n_wave_count = waves[self.__n_wave_index]
				self.__n_task_number = self.__n_task_number + self.__n_wave_count
print("TASK NUMBER !!!!" .. tostring(self.__n_task_number))
			end

		end

	end

end

function KeyEmitterManager:on_round_finish()

	local t_filter = {}
	t_filter.t_category_filter = {
		type = "and",
		{"key_emitter", "=", true},	
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,
		no_pos_limitation = true,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local emitters = msg.x_result

	if #emitters > 0 then

		local random_draw = RandomDraw:create(GameGlobals.RandomGenerator)
		random_draw:set_target(#emitters)	
print("EMIT NUMBER !!!!" .. tostring(self.__n_task_number))
		for i = 1, self.__n_task_number do

			local emitter_index = random_draw:draw_next_repeat()
			emitters[emitter_index]:send_message(MessagePool:get_free_ex("assign_task", 1))

		end

		self.__n_task_number = 0

	end

end

function KeyEmitterManager:after_set_waves(c_msg)

	local waves = c_msg.x_param.x_new_value

	if waves and #waves > 0 then

		self.__n_wave_index = 1
		self.__n_wave_count = waves[1]
		self.__n_task_number = self.__n_wave_count

	end

end

KeyEmitterManager.LISTENER_CONFIG = {

	{"round_finish", {before = KeyEmitterManager.on_round_finish}},
	{"remove_sugar", {before = KeyEmitterManager.on_remove_sugar}},	
	{"remove_sugar_silent", {before = KeyEmitterManager.on_remove_sugar}},	
	{"direct_assign", {before = KeyEmitterManager.on_direct_assign}},
	{"dispatch_task", {before = KeyEmitterManager.on_dispatch_task}},		

}

KeyEmitterManager.PROPERTY_CONFIG = {
	
	{"waves", {}},

}

return KeyEmitterManager