-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"

-- ## Imports ## --




local TopEmitter = class("TopEmitter", function() return GameObject:create() end)


function TopEmitter:ctor()

	self.__c_prob_getter = ProbGetter:create(GameGlobals.RandomGenerator)

	self.__n_task = 0

	self.__f_schedule_func = function()

		self:update_func()

	end

	self:get_category():add("emitter")
	self:get_category():add("normal_emitter")

	self:register_listeners_with_config(TopEmitter.LISTENER_CONFIG)
	self:register_properties_with_config(TopEmitter.PROPERTY_CONFIG)

end

function TopEmitter:on_destroy()

	if self.__n_schedule_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
		self.__n_schedule_entry = nil

	end

end

function TopEmitter:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

end

function TopEmitter:on_enter()

	self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, self:get("emit_interval"), false)

end

function TopEmitter:on_exit()

	if self.__n_schedule_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
		self.__n_schedule_entry = nil

	end

end

function TopEmitter:emit()

	local sugar = ObjectFactory:get_instance():get("NormalSugar")
	sugar:setPosition(cc.p(self:getPosition()))
	sugar:set("velocity", cc.p(self:get("emit_speed_x"), -self:get("emit_speed_y")))
	sugar:set("color", self.__c_prob_getter:get())

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar_without_count", sugar, Message.N_TUNNEL))

end

function TopEmitter:update_func()

	if self.__n_task > 0 then

		self:emit()

		self.__n_task = self.__n_task - 1

	end

end

function TopEmitter:after_set_emit_interval(c_msg)

	if self.__n_schedule_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)
		self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(self.__f_schedule_func, c_msg.x_param.x_new_value, false)

	end

end

function TopEmitter:on_assign_task(c_msg)

	self.__n_task = self.__n_task + c_msg.x_param

end

function TopEmitter:after_set_prob_red(c_msg)

	self:change_prob("red", c_msg.x_param.x_new_value)

end

function TopEmitter:after_set_prob_green(c_msg)

	self:change_prob("green", c_msg.x_param.x_new_value)

end

function TopEmitter:after_set_prob_yellow(c_msg)

	self:change_prob("yellow", c_msg.x_param.x_new_value)

end

function TopEmitter:after_set_prob_pink(c_msg)

	self:change_prob("pink", c_msg.x_param.x_new_value)

end

function TopEmitter:after_set_prob_blue(c_msg)

	self:change_prob("blue", c_msg.x_param.x_new_value)

end

function TopEmitter:after_set_prob_white(c_msg)

	self:change_prob("white", c_msg.x_param.x_new_value)

end

function TopEmitter:change_prob(s_key, n_prob)

	if self.__c_prob_getter:has(s_key) then

		self.__c_prob_getter:modify(s_key, n_prob)

	else

		self.__c_prob_getter:add(s_key, s_key, n_prob)

	end	

end

function TopEmitter:on_draw_color(c_msg)

	c_msg.x_result = self.__c_prob_getter:get()

end

TopEmitter.LISTENER_CONFIG = {

	{"enter", {before = TopEmitter.on_enter}},
	{"exit", {before = TopEmitter.on_exit}},
	{"assign_task", {before = TopEmitter.on_assign_task}},
	{"reset", {before = TopEmitter.on_reset}},
	{"draw_color", {before = TopEmitter.on_draw_color}},
	{"destroy", {before = TopEmitter.on_destroy}},

}	

TopEmitter.PROPERTY_CONFIG = {
	
	{"emit_interval", 0.033},
	{"emit_speed_x", 10},
	{"emit_speed_y", 300},
	{"prob_red", 0},
	{"prob_green", 10},
	{"prob_yellow", 0},
	{"prob_pink", 0},
	{"prob_blue", 0},
	{"prob_white", 0},
	{"enable", true},

}


return TopEmitter