-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameScene = require "GameEx.Components.Scene.GameScene"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local TopEmitterManager = class("TopEmitterManager", function() return GameObject:create() end)


function TopEmitterManager:ctor()

	self:register_listeners_with_config(TopEmitterManager.LISTENER_CONFIG)
	self:register_properties_with_config(TopEmitterManager.PROPERTY_CONFIG)

	self.__n_task_number = 0

	self.__t_emitters = {}

end

function TopEmitterManager:on_direct_assign(c_msg)

	self.__n_task_number = c_msg.x_param

end


function TopEmitterManager:on_dispatch_task(c_msg)

	local emitters = {}

	for _, emitter in pairs(self.__t_emitters) do

		if emitter:get("enable") then

			emitters[#emitters + 1] = emitter

		end

	end

	if #emitters > 0 then
print("dispatch !@#$!@#$@#") print(#emitters)
		local random_draw = RandomDraw:create(GameGlobals.RandomGenerator)
		random_draw:set_target(#emitters)	

		for i = 1, self.__n_task_number do

			local emitter_index = random_draw:draw_next_repeat()
			emitters[emitter_index]:send_message(MessagePool:get_free_ex("assign_task", 1))

		end

		self.__n_task_number = 0

	end

end

function TopEmitterManager:on_round_start(c_msg)

	for _, emitter in pairs(self.__t_emitters) do

		emitter:send_message(c_msg)

	end

end

function TopEmitterManager:on_round_finish(c_msg)

	for _, emitter in pairs(self.__t_emitters) do

		emitter:send_message(c_msg)

	end

end

function TopEmitterManager:on_add_sugar(c_msg)

	if c_msg.x_param:get_category():has("emitter") then

		self.__t_emitters[c_msg.x_param] = c_msg.x_param

	end	

end

function TopEmitterManager:on_remove_sugar(c_msg)

	self.__t_emitters[c_msg.x_param] = nil

end

TopEmitterManager.LISTENER_CONFIG = {

	{"add_sugar", {before = TopEmitterManager.on_add_sugar}},
	{"add_sugar_without_count", {before = TopEmitterManager.on_add_sugar}},
	{"remove_sugar", {before = TopEmitterManager.on_remove_sugar}},
	{"remove_sugar_silent", {before = TopEmitterManager.on_remove_sugar}},

	{"direct_assign", {before = TopEmitterManager.on_direct_assign}},
	{"dispatch_task", {before = TopEmitterManager.on_dispatch_task}},
	{"round_start", {before = TopEmitterManager.on_round_start}},
	{"round_finish", {before = TopEmitterManager.on_round_finish}},			

}

TopEmitterManager.PROPERTY_CONFIG = {
	


}

return TopEmitterManager