-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"

-- ## Imports ## --




local IdleFinder = class("IdleFinder", function() return GameObject:create() end)


function IdleFinder:ctor()

	self.__c_select_circle = cc.PhysicsShapeCircle:create(1)
	self.__c_select_circle:retain()

	self:register_listeners_with_config(IdleFinder.LISTENER_CONFIG)
	self:register_properties_with_config(IdleFinder.PROPERTY_CONFIG)

	self.__n_find_times = 0

end

function IdleFinder:on_destroy()

	self.__c_select_circle:release()

end

function IdleFinder:on_find_link(c_msg)

	self.__n_find_times = self.__n_find_times + 1

	local result = self:find_special()

	if result == "no_available_link" then

		result = self:find_normal_link()

	end

	if result == "no_available_link" then
print(self.__n_find_times)
		if self.__n_find_times > self:get("game_over_threshold") then

			result = "game_over"

		end		

	elseif result == "game_over" then

	else

		self.__n_find_times = 0

	end

	c_msg.x_result = result

end

function IdleFinder:find_normal_link()

	local filter = {}

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	filter.t_category_filter = {
		type = "and",
		{
			type = "or",
			{"normal", "=", true},
			{"random", "=", true},
			{"timing_bomb", "=", true},
			{"colorful", "=", true},
		},
		{
			type = "and",
		},
	}

	local msg = MessagePool:get_free_ex("query_filter", filter)
	GameGlobals.Scene:send_message(msg)

	local query_result = msg.x_result

	if #query_result < 3 then
print("less than 3")
		return "game_over"

	end

	for i = 1, #query_result do 

		query_result[i].n_search_flag = false

	end

	for i = 1, #query_result do

		if not query_result[i].n_search_flag then

			filter.t_property_filter[2] = {"color", "=", query_result[i]:get("color")}

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_defined", query_result[i]:get("color"), Message.N_TUNNEL))

			local result = {}

			self:search_link(query_result[i], filter, result)

			local result_size = 0

			for sugar, _ in pairs(result) do

				result_size = result_size + 1

			end

			if result_size > 2 then

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))
				return result

			else

				for sugar, _ in pairs(result) do

					sugar.n_search_flag = true

				end

			end

		end

	end

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))

	return "no_available_link"

end

function IdleFinder:find_special()

	local filter = {}

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	filter.t_category_filter = {
		type = "and",
		{"special", "=", true},
	}

	local msg = MessagePool:get_free_ex("query_filter", filter)
	GameGlobals.Scene:send_message(msg)

	local query_result = msg.x_result

	if #query_result > 0 then

		local result = {}
		result[query_result[1]] = query_result[1]

		return result

	else

		return "no_available_link"

	end

end

function IdleFinder:search_link(c_sugar, t_filter, t_result)

	local search_radius = c_sugar:get("radius") + self:get("select_range")

	local param = {

		c_shape = self.__c_select_circle,
		t_p = cc.p(c_sugar:getPosition()),
		n_rotation = 0,
		n_scale_x = search_radius,
		n_scale_y = search_radius,
		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_shape"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local query_result = msg.x_result

	local result_size = #query_result

	for i = 1, result_size do

		local sugar = query_result[i]

		if not t_result[sugar] then

			t_result[sugar] = true

			self:search_link(sugar, t_filter, t_result)

		end

	end

end


function IdleFinder:on_reset_counter()

	self.__n_find_times = 0

end

function IdleFinder:on_check_available(c_msg)

	local hint = c_msg.x_param

	local available_flag = true

	for i = 1, #hint - 1 do

		local sugar_1 = hint[i]
		local sugar_2 = hint[i + 1]

		local distance = UtilFunction:caculate_distance(cc.p(sugar_1:getPosition()), cc.p(sugar_2:getPosition()))

		if distance - sugar_1:get("radius") * 2 > self:get("select_range") then

			available_flag = false

			break

		end

	end

	c_msg.x_result = available_flag	

end

IdleFinder.LISTENER_CONFIG = {

	{"find_link", {before = IdleFinder.on_find_link}},
	{"reset_counter", {before = IdleFinder.on_reset_counter}},
	{"destroy", {before = IdleFinder.on_destroy}},
	{"check_available", {before = IdleFinder.on_check_available}},

}

IdleFinder.PROPERTY_CONFIG = {
	
	{"select_range", 40},
	{"game_over_threshold", 3},

}


return IdleFinder