-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"

-- ## Imports ## --




local LinkFinder = class("LinkFinder", function() return GameObject:create() end)


function LinkFinder:ctor()

	self.__c_select_circle = cc.PhysicsShapeCircle:create(1)
	self.__c_select_circle:retain()

	self.__t_find_result = {}

	self:register_listeners_with_config(LinkFinder.LISTENER_CONFIG)
	self:register_properties_with_config(LinkFinder.PROPERTY_CONFIG)

end

function LinkFinder:on_destroy()

	self.__c_select_circle:release()

end

function LinkFinder:on_find_link(c_msg)

	local sugar = c_msg.x_param
	local filter = self:build_normal_filter(sugar)
	local result = {}

	self:search_link(sugar, filter, result)

	c_msg.x_result = result

end

function LinkFinder:search_link(c_sugar, t_filter, t_result)

	local search_radius = c_sugar:get("radius") + self:get("select_range")

	local param = {

		c_shape = self.__c_select_circle,
		t_p = cc.p(c_sugar:getPosition()),
		n_rotation = 0,
		n_scale_x = search_radius,
		n_scale_y = search_radius,
		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_shape"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local query_result = msg.x_result

	local result_size = #query_result

	for i = 1, result_size do

		local sugar = query_result[i]

		if not t_result[sugar] and sugar:get("state") == "normal" then

			t_result[sugar] = true

			-- self:search_link(sugar, t_filter, t_result)

		end

	end

end

function LinkFinder:build_normal_filter(c_sugar)

	local filter = {}

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
		{"color", "=", c_sugar:get("color")},
	}

	filter.t_category_filter = {
		type = "or",
		{"normal", "=", true},
		{"random", "=", true},
		{"timing_bomb", "=", true},
	}

	return filter

end


LinkFinder.LISTENER_CONFIG = {

	{"find_link", {before = LinkFinder.on_find_link}},
	{"destroy", {before = LinkFinder.on_destroy}},

}

LinkFinder.PROPERTY_CONFIG = {
	
	{"select_range", 40},

}


return LinkFinder