-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local RandomDrawArray = require "GameEx.Foundations.Utils.RandomDrawArray"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local CombineGenerator = class("CombineGenerator", function() return GameObject:create() end)


function CombineGenerator:ctor()

	self.__t_choosed = {}
	self.__c_random_draw = RandomDrawArray:create()

	self:register_listeners_with_config(CombineGenerator.LISTENER_CONFIG)
	self:register_properties_with_config(CombineGenerator.PROPERTY_CONFIG)

end

function CombineGenerator:on_generate(c_msg)

	if self:get("enable") and self.__t_choosed[c_msg.x_param] then

		local sugar = self:make_new_sugar()
		sugar:setPosition(c_msg.x_param:getPosition())

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", sugar, Message.N_TUNNEL))

		self.__c_parent:send_message(MessagePool:get_free_ex("sugar_generate", sugar))

		sugar:send_message(MessagePool:get_free_ex("ignite"))

	end

end

function CombineGenerator:after_set_candidate_list(c_msg)

	self:draw_from_candidate(c_msg.x_param.x_new_value, self:get("generate_number"))

end

function CombineGenerator:draw_from_candidate(canditate_list, choose_number)
-- if #canditate_list > 0 then ERROR_ASSERT(false, "string") end
	self.__t_choosed = {}

	self.__c_random_draw:set_target(#canditate_list)

	for i = 1, choose_number do

		local draw_result = self.__c_random_draw:draw_next()

		if draw_result then

			local sugar = canditate_list[draw_result]

			self.__t_choosed[sugar] = sugar

		else

			break

		end

	end

end

function CombineGenerator:make_new_sugar()

	local generate_type = self:get("generate_type")

	if generate_type == "bomb" then

		return ObjectFactory:get_instance():get("BombSugar")

	elseif generate_type == "ten" then

		return ObjectFactory:get_instance():get("CrossSugar")

	elseif generate_type == "line" then

		return ObjectFactory:get_instance():get("LineSugar")

	end

end

CombineGenerator.LISTENER_CONFIG = {
	
	{"generate", {before = CombineGenerator.on_generate}},

}

CombineGenerator.PROPERTY_CONFIG = {
	
	{"generate_type", "bomb"},
	{"generate_number", 2},
	{"candidate_list", {}},
	{"enable", false},

}


return CombineGenerator