-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --



local LinkGenerator = class("LinkGenerator", function() return GameObject:create() end)


function LinkGenerator:ctor()

	self.__s_generate_type = "None"

	self:register_listeners_with_config(LinkGenerator.LISTENER_CONFIG)
	self:register_properties_with_config(LinkGenerator.PROPERTY_CONFIG)

end

function LinkGenerator:on_condition(c_msg)

	local condition = c_msg.x_param

	if (#condition == 4) then

		if self:reach_cross_condition(condition) then

			self.__s_generate_type = "CrossSugar"

		else

			self.__s_generate_type = "None"

		end

	elseif (#condition > 5) and (#condition < 8) then

		self.__s_generate_type = "BombSugar"

	elseif (#condition >= 8) then

		self.__s_generate_type = "LightningSugar"

	else

		self.__s_generate_type = "None"

	end

	self.__c_parent:send_message(MessagePool:get_free_ex("indicate_generate", self.__s_generate_type, Message.N_TUNNEL))

	if (#condition == 4) then

		self.__t_last_condition = condition
		self:start_cross_checker()

	else

		self:stop_cross_checker()

	end

end

function LinkGenerator:cross_checker()

	if self:reach_cross_condition(self.__t_last_condition) then

		self.__s_generate_type = "CrossSugar"

	else

		self.__s_generate_type = "None"

	end

	self.__c_parent:send_message(MessagePool:get_free_ex("indicate_generate", self.__s_generate_type, Message.N_TUNNEL))

end

function LinkGenerator:start_cross_checker()

	self:stop_cross_checker()
print("link generator start checker")
	self.__n_check_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:cross_checker() end, 0.1, false)

end

function LinkGenerator:stop_cross_checker()

	if self.__n_check_entry then
print("link generator stop checker")
		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_check_entry)
		self.__n_check_entry = nil

	end

end

function LinkGenerator:reach_cross_condition(condition)

	local result = true

	for i = 0, 3 do

		local i_1 = math.mod(i, 4) + 1
		local i_2 = math.mod(i + 1, 4) + 1

		local p_1 = cc.p(condition[i_1]:getPosition())
		local p_2 = cc.p(condition[i_2]:getPosition())

		local distance = UtilFunction:caculate_distance(p_1, p_2)

		if distance > self:get("cross_distance_threshold") then

			result = false
			break

		end

	end

	if not result then

		result = true

		local condition_exchange = {}
		condition_exchange[1] = condition[1]
		condition_exchange[2] = condition[2]
		condition_exchange[3] = condition[4]
		condition_exchange[4] = condition[3]

		for i = 0, 3 do

			local i_1 = math.mod(i, 4) + 1
			local i_2 = math.mod(i + 1, 4) + 1

			local p_1 = cc.p(condition_exchange[i_1]:getPosition())
			local p_2 = cc.p(condition_exchange[i_2]:getPosition())

			local distance = UtilFunction:caculate_distance(p_1, p_2)

			if distance > self:get("cross_distance_threshold") then

				result = false
				break

			end

		end		

	end

	if not result then

		result = true

		local condition_exchange = {}
		condition_exchange[1] = condition[1]
		condition_exchange[2] = condition[3]
		condition_exchange[3] = condition[2]
		condition_exchange[4] = condition[4]

		for i = 0, 3 do

			local i_1 = math.mod(i, 4) + 1
			local i_2 = math.mod(i + 1, 4) + 1

			local p_1 = cc.p(condition_exchange[i_1]:getPosition())
			local p_2 = cc.p(condition_exchange[i_2]:getPosition())

			local distance = UtilFunction:caculate_distance(p_1, p_2)

			if distance > self:get("cross_distance_threshold") then

				result = false
				break

			end

		end		

	end

	return result

end

function LinkGenerator:on_generate(c_msg)

	if not (self.__s_generate_type == "None") then

		local result = c_msg.x_param
		local rear_sugar = result[#result]
		local generate_position = cc.p(rear_sugar:getPosition())

		local sugar = ObjectFactory:get_instance():get(self.__s_generate_type)
		sugar:setPosition(generate_position)

		if self.__s_generate_type == "LightningSugar" then

			sugar:set("select_color", result[1]:get("color"))

		end

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", sugar, Message.N_TUNNEL))

		self.__c_parent:send_message(MessagePool:get_free_ex("sugar_generate", sugar))

		self.__s_generate_type = "None"

		local event = cc.EventCustom:new("link_generate")
		event.param = {}
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)	

		TRIGGER_EVENT("LINK_GENERATE")

		if sugar:get_category():has("special") then

		    local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
		    event.param = {task_id = "task_04", target_type = sugar:get("type"), finish_count = 1, func = function() end }
		    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		end

		if sugar:get_category():has("special") then

			local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
			event.param = {achv_id = "achv_10", finish_count = 1, func = function() end }
			cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		end

	end

end

function LinkGenerator:on_link_finish()

	self:stop_cross_checker()

end

function LinkGenerator:on_destroy()

	self:stop_cross_checker()

end

LinkGenerator.LISTENER_CONFIG = {

	{"condition", {before = LinkGenerator.on_condition}},
	{"generate", {before = LinkGenerator.on_generate}},
	{"select_finish", {before = LinkGenerator.on_link_finish}},
	{"select_cancel", {before = LinkGenerator.on_link_finish}},
	{"destroy", {before = LinkGenerator.on_destroy}},

}

LinkGenerator.PROPERTY_CONFIG = {
	
	{"cross_distance_threshold", 150},

}


return LinkGenerator