-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local CircleSelector = require "GameEx.Components.Selectors.CircleSelector"

-- ## Imports ## --

local CircleImpulser = class("CircleImpulser", function() return GameObject:create() end)

function CircleImpulser:ctor()

	self:build_filter()

	self:add_component(CircleSelector:create(), "selector")

	self:register_listeners_with_config(CircleImpulser.LISTENER_CONFIG)
	self:register_properties_with_config(CircleImpulser.PROPERTY_CONFIG)

end

function CircleImpulser:on_apply_impulse(c_msg)

	local selector = self:get_component("selector")

	selector:set("select_radius", 240)
	selector:set("filter", self.__t_impulse_select_filter)

	local msg = MessagePool:get_free()
	msg.s_name = "select"
	msg.x_param = {ban_message = true, position = c_msg.x_param}

	self:get_component("selector"):send_message(msg)

	local result = msg.x_result

	for i = 1, #result do 

		-- result[i]:send_message(MessagePool:get_free_ex("apply_impulse", cc.p(0, 1500000), Message.N_TUNNEL))

		local start_position = c_msg.x_param
		local target_position = cc.p(result[i]:getPosition())
		local direction = UtilFunction:caculate_direction(target_position, start_position)
		direction = UtilFunction:normalize(direction)

		result[i]:send_message(MessagePool:get_free_ex("apply_impulse", cc.p(direction.x * 500000, direction.y * 500000), Message.N_TUNNEL))

		-- local move = cc.MoveBy:create(0.1, cc.p(direction.x * 100, direction.y * 100))
		-- result[i]:runAction(move)

	end

end

function CircleImpulser:build_filter()

	self.__t_impulse_select_filter = {
		-- t_property_filter = {
		-- 	type = "and",
		-- 	{"dynamic", "=", true}
		-- }
	}

end


CircleImpulser.LISTENER_CONFIG = {

	{"apply_impulse", {before = CircleImpulser.on_apply_impulse}},

}

CircleImpulser.PROPERTY_CONFIG = {
	
	{"impulse_force", 1000}

}

return CircleImpulser