-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local FractureIndicator = class("FractureIndicator", function() return GameObject:create() end)


function FractureIndicator:ctor()

	self.__c_indicate_sprite = cc.Sprite:create()
	self.__c_indicate_sprite:setOpacity(180)
	self.__c_indicate_sprite:setScale(1.7)
	self.__c_indicate_sprite:retain()

	self:register_listeners_with_config(FractureIndicator.LISTENER_CONFIG)
	self:register_properties_with_config(FractureIndicator.PROPERTY_CONFIG)

	self.__s_last_indicate_type = "none"
	self.__c_last_indicate_node = nil
	self.__t_last_indicate_targets = {}

end

function FractureIndicator:on_destroy()

	self.__c_indicate_sprite:release()

	self:stop_reindicate()

end

function FractureIndicator:on_indicate_fracture(c_msg)
print("fracture indicate start")
	local task = c_msg.x_param
print(task.type)
	if task.type == "normal" then

		self.__t_last_task = task
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_indicate", task, Message.N_TUNNEL))

		self:start_reindicate()

	else

		self:stop_reindicate()

	end

end

function FractureIndicator:on_stop_indicate(c_msg)

	
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_indicate", {}, Message.N_TUNNEL))

	self:stop_reindicate()

end


function FractureIndicator:reindicate()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_indicate", self.__t_last_task, Message.N_TUNNEL))

end

function FractureIndicator:start_reindicate()

	self:stop_reindicate()

	self.__n_check_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:reindicate() end, 0.1, false)

end

function FractureIndicator:stop_reindicate()

	if self.__n_check_entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_check_entry)
		self.__n_check_entry = nil

	end

end


FractureIndicator.LISTENER_CONFIG = {

	{"select_continue", {before = FractureIndicator.on_indicate_fracture}},
	{"select_finish", {before = FractureIndicator.on_stop_indicate}},
	{"select_cancel", {before = FractureIndicator.on_stop_indicate}},
	{"special_link_triggered", {before = FractureIndicator.on_indicate_special}},
	{"destroy", {before = FractureIndicator.on_destroy}},

}

FractureIndicator.PROPERTY_CONFIG = {
	
	{"enable", true},

}


return FractureIndicator