-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local GenerateIndicator = class("GenerateIndicator", function() return GameObject:create() end)


function GenerateIndicator:ctor()

	self.__c_indicate_sprite = cc.Sprite:create()
	self.__c_indicate_sprite:setOpacity(180)
	self.__c_indicate_sprite:setScale(1.7)
	self.__c_indicate_sprite:retain()

	self:add_component(ObjectFactory:get_instance():get("TouchComponent"), "touch_handler")

	self:register_listeners_with_config(GenerateIndicator.LISTENER_CONFIG)
	self:register_properties_with_config(GenerateIndicator.PROPERTY_CONFIG)

end

function GenerateIndicator:on_destroy()

	self.__c_indicate_sprite:release()

end

function GenerateIndicator:on_indicate_generate(c_msg)
print("indicate generate!!")
print(c_msg.x_param)
	if c_msg.x_param == "BombSugar" then

		self.__c_indicate_sprite:setTexture("res/images/barrier/boom.png");

	elseif c_msg.x_param == "CrossSugar" then

		self.__c_indicate_sprite:setTexture("res/images/barrier/cross.png");

	elseif c_msg.x_param == "LightningSugar" then

		self.__c_indicate_sprite:setTexture("res/images/barrier/flash.png");

	else

		self.__c_indicate_sprite:setTexture("");

	end

end

function GenerateIndicator:offset_position(position)

	position.x = position.x + 50
	position.y = position.y + 120

	return position
-- 
end

function GenerateIndicator:on_touch_down(c_msg)

	self.__c_indicate_sprite:setTexture("");

	if not self.__c_indicate_sprite:getParent() then

		GameGlobals.Scene.__c_scene:addChild(self.__c_indicate_sprite, 999)

		local scale_1 = cc.ScaleTo:create(0.8, 1.5)
		local scale_2 = cc.ScaleTo:create(0.8, 1.7)
		local sequence = cc.Sequence:create(scale_1, scale_2)
		local rep = cc.RepeatForever:create(sequence)
		self.__c_indicate_sprite:runAction(rep)

	end

	self.__c_indicate_sprite:setPosition(self:offset_position(c_msg.x_param))

end

function GenerateIndicator:on_touch_up(c_msg)

	self.__c_indicate_sprite:setTexture("");
	self.__c_indicate_sprite:removeFromParent()

end

function GenerateIndicator:on_touch_move(c_msg)

	self.__c_indicate_sprite:setPosition(self:offset_position(c_msg.x_param))

end

GenerateIndicator.LISTENER_CONFIG = {

	{"indicate_generate", {before = GenerateIndicator.on_indicate_generate}},
	{"touch_move", {before = GenerateIndicator.on_touch_move}},
	{"touch_down", {before = GenerateIndicator.on_touch_down}},
	{"touch_up", {before = GenerateIndicator.on_touch_up}},
	{"touch_cancel", {before = GenerateIndicator.on_touch_up}},
	{"select_cancel", {before = GenerateIndicator.on_touch_up}},
	{"destroy", {before = GenerateIndicator.on_destroy}},

}

GenerateIndicator.PROPERTY_CONFIG = {
	
	{"enable", true},

}


return GenerateIndicator