-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"

-- ## Imports ## --




local LinkIndicator = class("LinkIndicator", function() return GameObject:create() end)


function LinkIndicator:ctor()

	self.__t_last_indications = {}

	self:register_listeners_with_config(LinkIndicator.LISTENER_CONFIG)
	self:register_properties_with_config(LinkIndicator.PROPERTY_CONFIG)

end

function LinkIndicator:on_destroy()

	self:on_stop_indicate()

end

function LinkIndicator:on_start_indicate(c_msg)

	local current_indications = c_msg.x_param

	for sugar, _ in pairs(self.__t_last_indications) do

		self.__t_last_indications[sugar] = false

		sugar:release()

	end

	for sugar, _ in pairs(current_indications) do

		if self.__t_last_indications[sugar] == false then

			self.__t_last_indications[sugar] = true

		else

			self:high_light_on(sugar)

		end

		sugar:retain()

	end

	for sugar, _ in pairs(self.__t_last_indications) do

		if self.__t_last_indications[sugar] == false then

			self:high_light_off(sugar)

		end

	end

	self.__t_last_indications = current_indications

end

function LinkIndicator:on_stop_indicate()

	for sugar, _ in pairs(self.__t_last_indications) do

		self:high_light_off(sugar)

		sugar:release()

	end

	self.__t_last_indications = {}

end

function LinkIndicator:high_light_on(c_sugar)

	c_sugar:send_message(MessagePool:get_free_ex("highlight_on", "", Message.N_TUNNEL))

end

function LinkIndicator:high_light_off(c_sugar)

	c_sugar:send_message(MessagePool:get_free_ex("highlight_off", "", Message.N_TUNNEL))

end


LinkIndicator.LISTENER_CONFIG = {

	{"start_indicate", {before = LinkIndicator.on_start_indicate}},
	{"stop_indicate", {before = LinkIndicator.on_stop_indicate}},
	{"destroy", {before = LinkIndicator.on_destroy}},

}

LinkIndicator.PROPERTY_CONFIG = {
	


}


return LinkIndicator