-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local SpecialIndicator = class("SpecialIndicator", function() return GameObject:create() end)


function SpecialIndicator:ctor()

	self.__c_indicate_sprite = cc.Sprite:create()
	self.__c_indicate_sprite:setOpacity(180)
	self.__c_indicate_sprite:setScale(1.7)
	self.__c_indicate_sprite:retain()

	self:register_listeners_with_config(SpecialIndicator.LISTENER_CONFIG)
	self:register_properties_with_config(SpecialIndicator.PROPERTY_CONFIG)

	self.__s_last_indicate_type = "none"
	self.__c_last_indicate_node = nil
	self.__t_last_indicate_targets = {}

end

function SpecialIndicator:on_destroy()

	self.__c_indicate_sprite:release()

	self:stop_reindicate()

end

function SpecialIndicator:on_indicate_special(c_msg)

	local task = c_msg.x_param

	if self:check_special(task) and not self:check_duplicate_indicate(task) then

		self:clear_previous_indicate()
		self:trigger_new_indicate(task)

	end

end

function SpecialIndicator:on_stop_indicate(c_msg)
print("special indicate finish")
	self:clear_previous_indicate()

	self:stop_reindicate()

end

function SpecialIndicator:check_special(task)

	if task.type == "special" or task.type == "special_link_sort" then
print(string.format("%s checked", task.type))
		return true

	else

		return false

	end

end

function SpecialIndicator:check_duplicate_indicate(task)

	if task.type == self.__s_last_indicate_type and task[#task] == self.__c_last_indicate_node then
print(string.format("duplicate indicate checked"))
		return true

	else

		return false

	end

end

function SpecialIndicator:trigger_new_indicate(task)

	local sugar = task[#task]

	local select_result = self:do_select(sugar, task.type) or {}

	self:turn_on(select_result)

	self.__s_last_indicate_type = task.type
	self.__c_last_indicate_node = sugar
	self.__t_last_indicate_targets = select_result

	if task.type == "special" and sugar:get_category():has("lightning") then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_indicate", select_result, Message.N_TUNNEL))

	end

	self:start_reindicate()

end

function SpecialIndicator:do_select(sugar, type)

	local position = {x = 0, y = 0}

	if type == "special" then

		position = cc.p(sugar:getPosition())

	elseif type == "special_link_sort" then

		position = cc.p(sugar.c_position_getter:getPosition())

	end

	local msg = MessagePool:get_free()
	msg.s_name = "trigger_selector"
	msg.x_param = position

	sugar:send_message(msg)

	return msg.x_result or {}

end

function SpecialIndicator:turn_on(sugars)

	if not sugars then return end

	local sugars = sugars or {}

	for i = 1, #sugars do

		sugars[i]:send_message(MessagePool:get_free_ex("highlight_on", "", Message.N_TUNNEL))

	end	

end

function SpecialIndicator:turn_off(sugars)

	if not sugars then return end

	for i = 1, #sugars do

		sugars[i]:send_message(MessagePool:get_free_ex("highlight_off", "", Message.N_TUNNEL))

	end	

end


function SpecialIndicator:clear_previous_indicate()

	self:turn_off(self.__t_last_indicate_targets)

	self.__s_last_indicate_type = "none"
	self.__c_last_indicate_node = nil
	self.__t_last_indicate_targets = {}

end

function SpecialIndicator:reindicate()

	local new_select_result = self:do_select(self.__c_last_indicate_node, self.__s_last_indicate_type)

	local check_list = {}

	for i = 1, #self.__t_last_indicate_targets do

		local sugar = self.__t_last_indicate_targets[i]

		check_list[sugar] = false

	end

	for i = 1, #new_select_result do

		local sugar = new_select_result[i]

		check_list[sugar] = true

	end

	local on_list = {}
	local off_list = {}

	for key, value in pairs(check_list) do

		if value then

			on_list[#on_list + 1] = key

		else

			off_list[#off_list + 1] = key

		end

	end

	self:turn_off(off_list)
	self:turn_on(on_list)

	self.__t_last_indicate_targets = on_list

	if self.__s_last_indicate_type == "special" and self.__c_last_indicate_node:get_category():has("lightning") then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_indicate", self.__t_last_indicate_targets, Message.N_TUNNEL))

	end

end

function SpecialIndicator:start_reindicate()

	self:stop_reindicate()
print("special indicator start reindicate")
	self.__n_check_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:reindicate() end, 0.1, false)

end

function SpecialIndicator:stop_reindicate()

	if self.__n_check_entry then
print("special indicator stop reindicate")
		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_check_entry)
		self.__n_check_entry = nil

	end

end


SpecialIndicator.LISTENER_CONFIG = {

	{"special_triggered", {before = SpecialIndicator.on_indicate_special}},
	{"select_finish", {before = SpecialIndicator.on_stop_indicate}},
	{"select_cancel", {before = SpecialIndicator.on_stop_indicate}},
	{"special_link_triggered", {before = SpecialIndicator.on_indicate_special}},
	{"destroy", {before = SpecialIndicator.on_destroy}},

}

SpecialIndicator.PROPERTY_CONFIG = {
	
	{"enable", true},

}


return SpecialIndicator