-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local SkillTouchHandler = class("SkillTouchHandler", function() return GameObject:create() end)


function SkillTouchHandler:ctor()

	self.__c_listener_node = cc.Node:create()
	self.__c_listener_node:setLocalZOrder(99)
	self.__c_listener_node:retain()

	self:create_event_handler()

    self:register_listeners_with_config(SkillTouchHandler.LISTENER_CONFIG)
	self:register_properties_with_config(SkillTouchHandler.PROPERTY_CONFIG)

	self.stopped = false

end

function SkillTouchHandler:on_run()

	GameGlobals.Scene.__c_scene:addChild(self.__c_listener_node, 1)

end

function SkillTouchHandler:on_disassemble()

	self.__c_listener_node:removeFromParent()

end

function SkillTouchHandler:on_destroy()

	self.__c_listener_node:release()

end

function SkillTouchHandler:create_event_handler()

	function touch_down(event)

		if not self:get("enable") then

			return false

		end

		self.stopped = false

		local msg = MessagePool:get_free()
		msg.c_sender = self
		msg.x_param = event:getLocation()
		msg.s_name = "touch_down"

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_down"))
		self.__c_parent:send_message(msg)

		return true

	end

	function touch_move(event)

		if not self.stopped then

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = event:getLocation()
			msg.s_name = "touch_move"

			self.__c_parent:send_message(msg)

		end

	end

	function touch_up(event)

		if not self.stopped then

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = event:getLocation()
			msg.s_name = "touch_up"

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_up"))
			self.__c_parent:send_message(msg)

		else

			self.stopped = false

		end

	end

	function touch_cancel(event)

		if not self.stopped then

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = event:getLocation()
			msg.s_name = "touch_cancel"

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_cancel"))
			self.__c_parent:send_message(msg)

		else

			self.stopped = false

		end

	end

	local touch_listener = cc.EventListenerTouchOneByOne:create()
    touch_listener:setSwallowTouches(true)
    touch_listener:registerScriptHandler(touch_down,cc.Handler.EVENT_TOUCH_BEGAN )
    touch_listener:registerScriptHandler(touch_move,cc.Handler.EVENT_TOUCH_MOVED )
    touch_listener:registerScriptHandler(touch_up,cc.Handler.EVENT_TOUCH_ENDED )
    touch_listener:registerScriptHandler(touch_cancel,cc.Handler.EVENT_TOUCH_CANCELLED )
    self.__c_listener = touch_listener
    self.__c_listener_node:getEventDispatcher():addEventListenerWithSceneGraphPriority(touch_listener, self.__c_listener_node)

end

function SkillTouchHandler:on_stop()

	self.stopped = true

end

function SkillTouchHandler:on_raise_priority()

	self.__c_listener_node:setLocalZOrder(99999999)

end

function SkillTouchHandler:on_resume_priority()

	self.__c_listener_node:setLocalZOrder(1)

end

SkillTouchHandler.LISTENER_CONFIG = {

	{"assemble", {before = SkillTouchHandler.on_assemble}},
	{"disassemble", {before = SkillTouchHandler.on_disassemble}},
	{"stop_touch", {before = SkillTouchHandler.on_stop}},
	{"run", {before = SkillTouchHandler.on_run}},
	{"destroy", {before = SkillTouchHandler.on_destroy}},
	{"raise_priority", {before = SkillTouchHandler.on_raise_priority}},
	{"resume_priority", {before = SkillTouchHandler.on_resume_priority}},

}

SkillTouchHandler.PROPERTY_CONFIG = {
	
	{"enable", true}

}

return SkillTouchHandler