-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local TouchComponent = class("TouchComponent", function() return GameObject:create() end)


function TouchComponent:ctor()

	self.__c_listener_node = cc.Node:create()
	self.__c_listener_node:setLocalZOrder(1)
	self.__c_listener_node:retain()

	self:create_event_handler()

    self:register_listeners_with_config(TouchComponent.LISTENER_CONFIG)
	self:register_properties_with_config(TouchComponent.PROPERTY_CONFIG)

	self.stopped = false

end

function TouchComponent:on_run()

	GameGlobals.Scene.__c_scene:addChild(self.__c_listener_node, 1)

end

function TouchComponent:on_disassemble()

	self.__c_listener_node:removeFromParent()

end

function TouchComponent:on_destroy()

	self.__c_listener_node:release()

end

function TouchComponent:create_event_handler()

	function touch_down(touch, event)

		if not self:get("enable") then

			return false

		end

		local move_range = self:get("move_range")

		if not move_range.enable or
			(move_range.enable and self:is_touch_in_range(touch:getLocation(), move_range)) then

			self.stopped = false

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = touch:getLocation()
			msg.s_name = "touch_down"

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_down"))
			self.__c_parent:send_message(msg)

			return true

		else

			return false

		end

	end

	function touch_move(touch, event)

		if not self.stopped then

			local move_range = self:get("move_range")

			if not move_range.enable or
				(move_range.enable and self:is_touch_in_range(touch:getLocation(), move_range)) then

				local msg = MessagePool:get_free()
				msg.c_sender = self
				msg.x_param = touch:getLocation()
				msg.s_name = "touch_move"

				self.__c_parent:send_message(msg)

			end

		end

	end

	function touch_up(event)

		if not self.stopped then

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = event:getLocation()
			msg.s_name = "touch_up"

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_up"))
			self.__c_parent:send_message(msg)

		else

			self.stopped = false

		end

	end

	function touch_cancel(event)

		if not self.stopped then

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.x_param = event:getLocation()
			msg.s_name = "touch_cancel"

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("touch_cancel"))
			self.__c_parent:send_message(msg)

		else

			self.stopped = false

		end

	end

	local touch_listener = cc.EventListenerTouchOneByOne:create()
    touch_listener:setSwallowTouches(false)
    touch_listener:registerScriptHandler(touch_down,cc.Handler.EVENT_TOUCH_BEGAN )
    touch_listener:registerScriptHandler(touch_move,cc.Handler.EVENT_TOUCH_MOVED )
    touch_listener:registerScriptHandler(touch_up,cc.Handler.EVENT_TOUCH_ENDED )
    touch_listener:registerScriptHandler(touch_cancel,cc.Handler.EVENT_TOUCH_CANCELLED )
    self.__c_listener = touch_listener
    self.__c_listener_node:getEventDispatcher():addEventListenerWithSceneGraphPriority(touch_listener, self.__c_listener_node)

    local function onNodeEvent(event)

        if "enter" == event then
print("enter touch component")
        	function modify_touch_range(event)
print("modifiy range")
        		self:set("move_range", event.param)
dump(event.param)
        	end

	        self.__c_handler = cc.EventListenerCustom:create("modify_touch_range", modify_touch_range)
	        cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__c_handler, self.__c_listener_node)        	

        elseif "exit" == event then

        	cc.Director:getInstance():getEventDispatcher():removeEventListener(self.__c_handler)

        end

    end

    self.__c_listener_node:registerScriptHandler(onNodeEvent)

end

function TouchComponent:on_stop()

	self.stopped = true

end

function TouchComponent:on_disable()
print("disable_touch")
	self:set("enable", false)

end

function TouchComponent:on_enable()
print("enable_touch")
	self:set("enable", true)

end

function TouchComponent:is_touch_in_range(touch_pos, touch_range)

	if touch_pos.x > touch_range.x - touch_range.width / 2 and
		touch_pos.x < touch_range.x + touch_range.width / 2 and
		touch_pos.y > touch_range.y - touch_range.height / 2 and
		touch_pos.y < touch_range.y + touch_range.height / 2 then

		return true
	else

		return false

	end

end

function TouchComponent:on_raise_priority()

	self.__c_listener_node:setLocalZOrder(99999999)

end

function TouchComponent:on_resume_priority()

	self.__c_listener_node:setLocalZOrder(1)

end

TouchComponent.LISTENER_CONFIG = {

	{"assemble", {before = TouchComponent.on_assemble}},
	{"disassemble", {before = TouchComponent.on_disassemble}},
	{"stop_touch", {before = TouchComponent.on_stop}},
	{"round_start", {before = TouchComponent.on_disable}},
	{"round_finish", {before = TouchComponent.on_enable}},
	{"run", {before = TouchComponent.on_run}},
	{"destroy", {before = TouchComponent.on_destroy}},
	{"raise_priority", {before = TouchComponent.on_raise_priority}},
	{"resume_priority", {before = TouchComponent.on_resume_priority}}

}

TouchComponent.PROPERTY_CONFIG = {
	
	{"enable", true},
	{"move_range", {x = 500, y = 800, width = 200, height = 200, enable = false}}

}

return TouchComponent