-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"

-- ## Imports ## --


local Rearranger = class("Rearranger", function() return GameObject:create() end)


function Rearranger:ctor()

	self:register_listeners_with_config(Rearranger.LISTENER_CONFIG)
	self:register_properties_with_config(Rearranger.PROPERTY_CONFIG)

end

function Rearranger:on_destroy()



end

function Rearranger:on_rearrange()

	local filter = {}

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	filter.t_category_filter = {
		type = "and",
		{"normal", "=", true},
		{"fenced", "=", false},
		{"add_move", "=", false},
	}

	local msg = MessagePool:get_free_ex("query_filter", filter)
	GameGlobals.Scene:send_message(msg)

	local result = msg.x_result

	local random_draw = RandomDraw:create()
	random_draw:set_target(#result)

	local swap_1 = random_draw:draw_next()

	while true do

		local swap_2 = random_draw:draw_next()

		if swap_2 then

			local sugar_1 = result[swap_1]
			local sugar_2 = result[swap_2]

			local pos_1 = cc.p(sugar_1:getPosition())
			local pos_2 = cc.p(sugar_2:getPosition())

			sugar_2:setPosition(pos_1)
			sugar_1:setPosition(pos_2)

			swap_1 = swap_2

		else

			break

		end

	end

end

Rearranger.LISTENER_CONFIG = {

	{"rearrange", {before = Rearranger.on_rearrange}},

}

Rearranger.PROPERTY_CONFIG = {
	
	{"select_range", 40},
	{"game_over_threshold", 3},

}


return Rearranger