-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local BombRemover = class("BombRemover", function() return GameObject:create() end)

function BombRemover:ctor()

	self:register_listeners_with_config(BombRemover.LISTENER_CONFIG)
	self:register_properties_with_config(BombRemover.PROPERTY_CONFIG)

end


function BombRemover:on_assign(c_msg)

	local task = c_msg.x_param

	for i = 1, #task do

		task[i]:retain()

	end

	TRIGGER_EVENT("REMOVE_START")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", task, Message.N_TUNNEL))

	local removed = {}

	for i = 1, #task do

		local sugar = task[i]

		if sugar:get("state") == "normal" then

			removed[#removed + 1] = sugar

		end

		local category = sugar:get_category()

		if category:has("special") then

			self:trigger_ignite(sugar)

		else

			self:trigger_remove(sugar)

		end

	end

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", removed, Message.N_TUNNEL))

	for i = 1, #task do

		task[i]:release()

	end

end

function BombRemover:trigger_remove(sugar)

	self.__c_parent:send_message(MessagePool:get_free_ex("sugar_remove", sugar))
	sugar:send_message(MessagePool:get_free_ex("start_remove", "", Message.N_TUNNEL))

end

function BombRemover:trigger_ignite(sugar)

	sugar:send_message(MessagePool:get_free_ex("ignite"))

end


BombRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = BombRemover.on_assign}},

}

BombRemover.PROPERTY_CONFIG = {
	


}

return BombRemover