-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local LightningRemover = class("LightningRemover", function() return GameObject:create() end)

function LightningRemover:ctor()

	self:register_listeners_with_config(LightningRemover.LISTENER_CONFIG)
	self:register_properties_with_config(LightningRemover.PROPERTY_CONFIG)

end


function LightningRemover:on_assign(c_msg)

	local task = c_msg.x_param

	local removed = {}

	for i = 1, #task do

		local sugar = task[i]

		if sugar:get("state") == "normal" or "locked" then

			removed[#removed + 1] = sugar

		end

		sugar:retain()
		sugar:send_message(MessagePool:get_free_ex("lock"))

	end

	local assign_index = 1
	local finish_index = 0
	local perform_schedule_entry = 0

	function remove_performer()

		if finish_index == #task then

			self.__c_parent:runAction(cc.Sequence:create(cc.DelayTime:create(0.2), cc.CallFunc:create(function()

				for i = 1, #task do

					task[i]:send_message(MessagePool:get_free_ex("start_remove", "", Message.N_TUNNEL))
					self.__c_parent:send_message(MessagePool:get_free_ex("sugar_remove", task[i]))

				end

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("lightning_remove_finish", "", Message.N_TUNNEL))
				-- GameGlobals.Scene:send_message_to_sugars(MessagePool:get_free_ex("lightning_remove_finish", "", Message.N_TUNNEL))

				self.__c_parent:send_message(MessagePool:get_free_ex("finish_remove", removed))

				for i = 1, #task do

					task[i]:release()

				end

			end)))

			cc.Director:getInstance():getScheduler():unscheduleScriptEntry(perform_schedule_entry)

		end

		if assign_index <= #task then

			local target = task[assign_index]

			local homing_entry = 0
			local homing_particle = ResourceManager:get_instance():get_resource_by_key("lightning_flying_particle")
			homing_particle:setDuration(-1)
			homing_particle:setPosition(cc.p(self.__c_parent:getPosition()))
			homing_particle:setLocalZOrder(11)
			homing_particle:setAutoRemoveOnFinish(true)
			homing_particle:setSpeed(350)
			homing_particle:setLife(0.6)
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

			local homing_performer = function()

				local target_position = target:getParent():convertToWorldSpace(cc.p(target:getPosition()))
				local particle_position = cc.p(homing_particle:getPosition())

				local distance = UtilFunction:caculate_distance(target_position, particle_position)

				if distance < self:get("homing_speed") then

					self:trigger_remove(target)

					homing_particle:setSpeed(0)
					homing_particle:setDuration(0)

					finish_index = finish_index + 1

					cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

				else

					local speed = self:get("homing_speed")
					local direction = UtilFunction:caculate_direction(target_position, particle_position)
					local angle = UtilFunction:caculate_angle(direction)
					local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
					particle_position.x = particle_position.x + offset.x
					particle_position.y = particle_position.y + offset.y

					homing_particle:setAngle(-angle)
					homing_particle:setPosition(particle_position)

				end

			end

			homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

			assign_index = assign_index + 1

		end

	end

	perform_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(remove_performer, self:get("remove_interval"), false)

end

function LightningRemover:trigger_remove(sugar)

	sugar:send_message(MessagePool:get_free_ex("select", "", Message.N_TUNNEL))

end

function LightningRemover:trigger_ignite(sugar)

	sugar:send_message(MessagePool:get_free_ex("ignite"))

end


LightningRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = LightningRemover.on_assign}},

}

LightningRemover.PROPERTY_CONFIG = {
	
	{"remove_interval", 0.1},
	{"homing_speed", 40}

}

return LightningRemover