-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local SpecialLinkRemover = require "GameEx.Components.Removers.SpecialLinkRemover"
local SpecialRemover = require "GameEx.Components.Removers.SpecialRemover"
local NormalRemover = require "GameEx.Components.Removers.NormalRemover"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local LinkRemover = class("LinkRemover", function() return GameObject:create() end)

function LinkRemover:ctor()

	local factory = ObjectFactory:get_instance()

	self:add_component(factory:get("NormalRemover"), "normal_remover")
	self:add_component(factory:get("SpecialRemover"), "special_remover")
	self:add_component(factory:get("SpecialLinkRemover"), "special_link_remover")

	self:register_listeners_with_config(LinkRemover.LISTENER_CONFIG)
	self:register_properties_with_config(LinkRemover.PROPERTY_CONFIG)

end

function LinkRemover:on_assign(c_msg)

	local task = c_msg.x_param

	if task.type == "normal" then

		self:get_component("normal_remover"):send_message(MessagePool:get_free_ex("assign_remove_task", task))

	elseif task.type == "special" then

		self:get_component("special_remover"):send_message(MessagePool:get_free_ex("assign_remove_task", task))

	elseif task.type == "special_link" then

		self:get_component("special_link_remover"):send_message(MessagePool:get_free_ex("assign_remove_task", task))

	end

	c_msg.e_transmit = Message.N_DIRECT

end

LinkRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = LinkRemover.on_assign}},

}

LinkRemover.PROPERTY_CONFIG = {
	


}

return LinkRemover