-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"

-- ## Imports ## --




local NormalRemover = class("NormalRemover", function() return GameObject:create() end)


function NormalRemover:ctor()

	self.__t_task_cache = {}

	self:register_listeners_with_config(NormalRemover.LISTENER_CONFIG)
	self:register_properties_with_config(NormalRemover.PROPERTY_CONFIG)

end

function NormalRemover:on_destroy()

	for task_list, entry in pairs(self.__t_task_cache) do

		self:finish_task(task_list)

	end

end

function NormalRemover:on_assemble()



end

function NormalRemover:on_disassemble()

	self:on_destroy()

end

function NormalRemover:on_assign(c_msg)

	local task_list = c_msg.x_param

	if self:remove_condition_reached(task_list) then

		TRIGGER_EVENT("REMOVE_START")
		
		self:perform_task(task_list)

	else

		self:cancel_task(task_list)

	end

end

function NormalRemover:remove_condition_reached(task_list)

	if #task_list < self:get("remove_trigger_threshold") then

		return false

	else

		return true

	end

end

function NormalRemover:cancel_task(task_list)

	for i = 1, #task_list do

		self:cancel_remove(task_list[i])

	end		

end

function NormalRemover:trigger_remove(sugar)

	sugar:send_message(MessagePool:get_free_ex("start_remove", "", Message.N_TUNNEL))

end

function NormalRemover:cancel_remove(sugar)

	sugar:send_message(MessagePool:get_free_ex("unselect"))

end

function NormalRemover:perform_task(task_list)

	for i = 1, #task_list do

		task_list[i]:retain()

	end

	local index = 1
	local timer = 0

	local schedule_func = function(dt)

		timer = timer + dt;

		local threshold = self:caculate_threshold(index)

		while timer >= threshold do 

			if not task_list[index] then

				self:finish_task(task_list)
				return

			else

				self:trigger_remove(task_list[index])
				index = index + 1
				timer = timer - threshold
				threshold = self:caculate_threshold(index)

			end

		end

	end

	local scheduler = cc.Director:getInstance():getScheduler()
	scheduleEntry = scheduler:scheduleScriptFunc(schedule_func, 0, false) 

	self.__t_task_cache[task_list] = scheduleEntry

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("link_remove_start", task_list, Message.N_TUNNEL))
	-- GameGlobals.Scene:send_message_to_sugars(MessagePool:get_free_ex("link_remove_start", task_list))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", task_list, Message.N_TUNNEL))


-- broadcast event to game

	local event = cc.EventCustom:new("link_remove")
	event.param = {}
	cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function NormalRemover:caculate_threshold(index)

	local threshold = 0

	if index < 4 then threshold = self:get("remove_interval") else threshold = self:get("remove_interval") * (1.0 - (index - 2) / 10) end

	threshold = threshold < 0.033 and 0.033 or threshold

	return threshold	

end

function NormalRemover:finish_task(task_list)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_finish", task_list, Message.N_TUNNEL))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("link_remove_finish", task_list, Message.N_TUNNEL))

	for i = 1, #task_list do

		task_list[i]:release()

	end

	local scheduler = cc.Director:getInstance():getScheduler()
	scheduler:unscheduleScriptEntry(self.__t_task_cache[task_list])

	self.__t_task_cache[task_list] = nil
print("normal link remove finish")
	-- GameGlobals.Scene:send_message_to_sugars(MessagePool:get_free_ex("link_remove_finish"))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task_list, Message.N_TUNNEL))

end

NormalRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = NormalRemover.on_assign}},
	{"assemble", {before = NormalRemover.on_assemble}},
	{"disassemble", {before = NormalRemover.on_disassemble}},
	{"destroy", {before = NormalRemover.on_destroy}},

}

NormalRemover.PROPERTY_CONFIG = {
	
	{"remove_interval", 0.12},
	{"remove_trigger_threshold", 3},

}


return NormalRemover