-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local SpecialLinkRemover = class("SpecialLinkRemover", function() return GameObject:create() end)


function SpecialLinkRemover:ctor()

	self:register_listeners_with_config(SpecialLinkRemover.LISTENER_CONFIG)
	self:register_properties_with_config(SpecialLinkRemover.PROPERTY_CONFIG)

end

function SpecialLinkRemover:on_assign(c_msg)

	TRIGGER_EVENT("REMOVE_START")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_link_start", "", Message.N_TUNNEL))

	local task_list = c_msg.x_param

	self:sort_task(task_list)

	local combine_type = task_list[1]:get("color")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", task_list[1]))

	task_list[2]:send_message(MessagePool:get_free_ex("combine", combine_type))

-- broadcast event to game

	local event = cc.EventCustom:new("link_combine")
	event.param = {}
	cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function SpecialLinkRemover:trigger_remove(sugar)

	sugar:send_message(MessagePool:get_free_ex("start_remove"))

end

function SpecialLinkRemover:on_select_continue(c_msg)

	if c_msg.x_param.type == "special_link" then

		local task_list = {}
		task_list.type = "special_link_sort"
		task_list[1] = c_msg.x_param[1]
		task_list[2] = c_msg.x_param[2]

		self:sort_task_append_position(task_list)

		for i = 1, #task_list do

			task_list[i]:send_message(MessagePool:get_free_ex("special_link_triggered", task_list, Message.N_TUNNEL))

		end

		self.__c_parent.__c_parent:send_message(MessagePool:get_free_ex("special_link_triggered", task_list, Message.N_TUNNEL))

	end

end

function SpecialLinkRemover:sort_task(task_list)

	local rank_1 = SpecialLinkRemover.COLOR_RANKS[task_list[2]:get("color")]
	local rank_2 = SpecialLinkRemover.COLOR_RANKS[task_list[1]:get("color")]

	if rank_1 < rank_2 then

		local temp_position = cc.p(task_list[1]:getPosition())
		task_list[1]:setPosition(cc.p(task_list[2]:getPosition()))
		task_list[2]:setPosition(temp_position)

		local temp = task_list[1]
		task_list[1] = task_list[2]
		task_list[2] = temp

	end

end

function SpecialLinkRemover:sort_task_append_position(task_list)

	local rank_1 = SpecialLinkRemover.COLOR_RANKS[task_list[2]:get("color")]
	local rank_2 = SpecialLinkRemover.COLOR_RANKS[task_list[1]:get("color")]

	if rank_1 < rank_2 then
print("exchange")
		task_list[1].c_position_getter = task_list[2]
		task_list[2].c_position_getter = task_list[1]

		local temp = task_list[1]
		task_list[1] = task_list[2]
		task_list[2] = temp

	else

		task_list[1].c_position_getter = task_list[1]
		task_list[2].c_position_getter = task_list[2]

	end



end

SpecialLinkRemover.COLOR_RANKS = {
	
	["bomb"] = 1,
	["line"] = 2,
	["ten"] = 3,
	["lightning"] = 4,

}

SpecialLinkRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = SpecialLinkRemover.on_assign}},
	{"select_continue", {before = SpecialLinkRemover.on_select_continue}},

}

SpecialLinkRemover.PROPERTY_CONFIG = {
	
	

}


return SpecialLinkRemover