-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local SpecialRemover = class("SpecialRemover", function() return GameObject:create() end)


function SpecialRemover:ctor()

	self:register_listeners_with_config(SpecialRemover.LISTENER_CONFIG)
	self:register_properties_with_config(SpecialRemover.PROPERTY_CONFIG)

end

function SpecialRemover:on_assign(c_msg)

	if self:get("enable_remove") then

		TRIGGER_EVENT("REMOVE_START")

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_touch_start", "", Message.N_TUNNEL))

		self.__c_parent:send_message(MessagePool:get_free_ex("remove_perform"))

		local task_list = c_msg.x_param

		for i = 1, #task_list do

			self:trigger_remove(task_list[i])

		end

		local event = cc.EventCustom:new("special_remove")
		event.param = {}
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	else

		local task_list = c_msg.x_param

		for i = 1, #task_list do

			task_list[i]:send_message(MessagePool:get_free_ex("unselect"))

		end

	end

end

function SpecialRemover:on_select_continue(c_msg)

	if c_msg.x_param.type == "special" then

		local task_list = c_msg.x_param

		for i = 1, #task_list do

			task_list[i]:send_message(MessagePool:get_free_ex("special_triggered", task_list, Message.N_TUNNEL))

		end

		self.__c_parent.__c_parent:send_message(MessagePool:get_free_ex("special_triggered", task_list, Message.N_TUNNEL))

	end

end

function SpecialRemover:trigger_remove(sugar)

	sugar:send_message(MessagePool:get_free_ex("start_remove"))

end


SpecialRemover.LISTENER_CONFIG = {

	{"assign_remove_task", {before = SpecialRemover.on_assign}},
	{"select_continue", {before = SpecialRemover.on_select_continue}},

}

SpecialRemover.PROPERTY_CONFIG = {
	
	{"enable_remove", true}

}


return SpecialRemover