-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UIComponent = require "GameEx.Components.Scene.UIComponent"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local SkillPerformer = require "GameEx.Components.Scene.SkillPerformer"
local GameGlobals = require "GameEx.GameGlobals"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local AnimationGenerator = require "GameEx.Foundations.Utils.AnimationGenerator"
require "core.TalkingDataManager"

-- ## Imports ## --




local GameScene = class("GameScene", function() return GameObject:create() end)


function GameScene:ctor()

	GameGlobals.Scene = self

	self.__c_scene = cc.Scene:create()
	self.__c_scene:initWithPhysics()
	self.__c_scene:addChild(self, 4)
	self.__c_scene:retain()

	self.__c_game_layer = cc.Layer:create()
	self.__c_game_layer:setName("game_board")
	self.__c_game_layer:retain()
	self.__c_scene:addChild(self.__c_game_layer)

	self.__c_child_layer = cc.Layer:create()
	self.__c_child_layer:setName("animation_layer")
	self.__c_child_layer:retain()
	self.__c_scene:addChild(self.__c_child_layer, 7)

	self.__c_physics_world = self.__c_scene:getPhysicsWorld()
	self.__c_physics_world:setAutoStep(true)

	self.__t_sugars = {}

	self.__n_action_count = 0

	self.__n_sugar_count = 0
	self.__n_fill_count = 0

	self:init_state_machine()

	self:add_component(ObjectFactory:get_instance():get("TopEmitterManager"), "normal_emitter_manager")
	self:add_component(ObjectFactory:get_instance():get("ChocolateSugarManager"), "chocolate_manager")
	self:add_component(ObjectFactory:get_instance():get("TransformSugarManager"), "transform_manager")
	self:add_component(ObjectFactory:get_instance():get("KeyEmitterManager"), "key_emitter_manager")
	self:add_component(ObjectFactory:get_instance():get("LinkSugar"), "link_handler")
	self:add_component(ObjectFactory:get_instance():get("TargetHandler"), "target_handler")
	self:add_component(ObjectFactory:get_instance():get("IdleFinder"), "finder")
	self:add_component(ObjectFactory:get_instance():get("LinkIndicator"), "indicator")
	self:add_component(ObjectFactory:get_instance():get("Rearranger"), "rearranger")
	self:add_component(ObjectFactory:get_instance():get("VictorySkill"), "bonus_performer")
	self:add_component(ObjectFactory:get_instance():get("RankHandler"), "rank_handler")
	self:add_component(ObjectFactory:get_instance():get("ScoreHandler"), "score_handler")
	self:add_component(ObjectFactory:get_instance():get("FractureManager"), "fracture_manager")
	self:add_component(ObjectFactory:get_instance():get("TimingBombManager"), "timing_bomb_manager")
	self:add_component(ObjectFactory:get_instance():get("ColorfulManager"), "colorful_manager")

	self:add_component(ObjectFactory:get_instance():get("RewardBagSugarManager"), "reward_bag_manager")

	self:register_listeners_with_config(GameScene.LISTENER_CONFIG)
	self:register_properties_with_config(GameScene.PROPERTY_CONFIG)

	self.__s_round_type = "game_start"

	self:create_level_progress()

end

function GameScene:create_level_progress()

	self.__t_level_progress = {}

	local meta = {

		__index = function(table, key)
-- ERROR_ASSERT(false, "string")
			if key == "n_move_left" then

				local secured = rawget(table, key)

				if secured then

					return tonumber(secured)

				else
					
					return 0

				end

			else
				
				return rawget(table, key)

			end

		end,

		__newindex = function(table, key, value)

			if key == "n_move_left" then

				if type(value == "number") then

					local secure = CSecuredNumber:create_instance(value)
					rawset(table, key, secure)

				end

			else
				
				rawset(table, key, value)

			end		

		end

	}

	setmetatable(self.__t_level_progress, meta)

	self.__t_level_progress.n_move_left = 0
	self.__t_level_progress.n_score = 0
	self.__t_level_progress.t_req_remain = {}

end

function GameScene:on_destroy()

	self.__c_game_layer:removeFromParent()
	self.__c_game_layer:release()

	self.__c_child_layer:removeFromParent()
	self.__c_child_layer:release()

	self.__c_scene:release()

	self:stop_find_timer()

end

function GameScene:on_remove_from_scene(c_msg)

	sugar = c_msg.x_param

	if self.__t_sugars[sugar] then

		local msg = MessagePool:get_free()
		msg.c_sender = self
		msg.s_name = "exit"
		msg.e_transmit = Message.N_TUNNEL

		sugar:send_message(msg)

		self.__t_sugars[sugar] = nil
		self.__c_game_layer:removeChild(sugar)

		if self:is_sugar_counted(sugar) then

			self.__n_sugar_count = self.__n_sugar_count - 1
			print(sugar:get("color"))
			print(self.__n_sugar_count)
		end
-- print(self.__n_sugar_count)
		ERROR_ASSERT(self.__n_sugar_count >= 0, "GameScene: REDUNDENT REMOVE")

	end

	if self.__s_round_type ~= "bonus" then

		if sugar:get_category():has("normal") then

		    local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
		    event.param = {task_id = "task_03", target_type = sugar:get("color"), finish_count = 1, func = function() end }
		    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		   	local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
	        event.param = {achv_id = "achv_04", finish_count = 1, func = function() end }
	        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		end

		if sugar:get_category():has("conch") then

	        

	    end

		if sugar:get_category():has("ice") then

	        local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
	        event.param = {achv_id = "achv_03", finish_count = 1, func = function() end }
	        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	    end

		if sugar:get_category():has("moveable_ice") then

	        local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
	        event.param = {achv_id = "achv_05", finish_count = 1, func = function() end }
	        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	    end

	end

end

function GameScene:on_add_to_scene(c_msg)

	self:on_add_to_scene_without_count(c_msg)

	if self:is_sugar_counted(sugar) then

		self.__n_sugar_count = self.__n_sugar_count + 1

		-- print(self.__n_sugar_count)
	end

end

function GameScene:on_add_to_scene_without_count(c_msg)

	sugar = c_msg.x_param

	self.__t_sugars[sugar] = sugar
	self.__c_game_layer:addChild(sugar)

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "enter"
	msg.e_transmit = Message.N_TUNNEL

	sugar:send_message(msg)

end

function GameScene:on_query_shape(c_msg)

	local shape = c_msg.x_param.c_shape
	local position = c_msg.x_param.t_p
	local angle = c_msg.x_param.n_rotation
	local scale_x = c_msg.x_param.n_scale_x
	local scale_y = c_msg.x_param.n_scale_y
	local category_filter = c_msg.x_param.t_category_filter
	local property_filter = c_msg.x_param.t_property_filter
	local result = {}
	local result_size = 0

	function query_callback(world, shape)

		local sugar = shape:getBody():getNode()

		if self:is_sugar_visible(sugar) and sugar:category_filter(category_filter) and sugar:property_filter(property_filter) then

			result_size = result_size + 1
			result[result_size] = sugar

		end

		return true

	end

	self.__c_physics_world:queryShape(query_callback, shape, position, angle, scale_x, scale_y)

	c_msg.x_result = result

end

function GameScene:on_query_point(c_msg)

	local position = c_msg.x_param.t_p
	local category_filter = c_msg.x_param.t_category_filter
	local property_filter = c_msg.x_param.t_property_filter
	local result = {}
	local result_size = 0

	function query_callback(world, shape)

		local sugar = shape:getBody():getNode()

		if self:is_sugar_visible(sugar) and sugar:category_filter(category_filter) and sugar:property_filter(property_filter) then

			result_size = result_size + 1
			result[result_size] = sugar

		end

		return true

	end

	self.__c_physics_world:queryPoint(query_callback, position)

	c_msg.x_result = result

end

function GameScene:on_query_line(c_msg)

	local p1 = c_msg.x_param.t_p1
	local p2 = c_msg.x_param.t_p2
	local category_filter = c_msg.x_param.t_category_filter
	local property_filter = c_msg.x_param.t_property_filter
	local result = {}
	local result_size = 0

	function query_callback(world, info)

		local sugar = info.shape:getBody():getNode()

		if self:is_sugar_visible(sugar) and sugar:category_filter(category_filter) and sugar:property_filter(property_filter) then

			result_size = result_size + 1
			result[result_size] = sugar

		end

		return true

	end

	self.__c_physics_world:rayCast(query_callback, p1, p2)

	c_msg.x_result = result

end

function GameScene:on_query_filter(c_msg)

	local category_filter = c_msg.x_param.t_category_filter
	local property_filter = c_msg.x_param.t_property_filter
	local result = {}
	local result_size = 0

	for _, sugar in pairs(self.__t_sugars) do

		if sugar:category_filter(category_filter) and sugar:property_filter(property_filter) then

			if c_msg.x_param.no_pos_limitation or self:is_sugar_visible(sugar) then

				result_size = result_size + 1
				result[result_size] = sugar

			end

		end

	end

	c_msg.x_result = result

end

function GameScene:is_sugar_visible(c_sugar)

	local position = cc.p(c_sugar:getPosition())
	local visible_size = cc.Director:getInstance():getVisibleSize()

	if position.x >= visible_size.width / 2 - 500 and
		position.x <= visible_size.width / 2 + 500 and
		position.y <= 1480 then

		return true

	else

		return false

	end

end

function GameScene:send_message_to_sugars(c_msg)

	for sugar, _ in pairs(self.__t_sugars) do

		sugar:send_message(c_msg)

	end

end

function GameScene:on_add_child(c_msg)

	self.__c_child_layer:addChild(c_msg.x_param)

end

function GameScene:on_remove_child(c_msg)

	self.__c_child_layer:removeChild(c_msg.x_param)

end

function GameScene:on_run()

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", self:get("sugar_volume") - self.__n_sugar_count))

	self.__n_sugar_count = self:get("sugar_volume")

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))
	self:get_component("key_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

	self:init_level_progress()
	self:start_find_timer()

	self.__c_state_machine:do_transition("start_section")

	if self:get("has_boss") then

		local boss = self:get_component("boss")
		local hp = boss:get("max_hp")
		local ui = self:get_component("ui_component")

		ui:set_boss_full_hp(hp)

	end

	display.runScene(self.__c_scene)

end

function GameScene:init_level_progress()

	local level_config = self:get("level_config")

	self.__t_level_progress.n_move_left = level_config.move_limit
	self.__t_level_progress.t_req_remain = level_config.reqs

end

function GameScene:check_level_progress()

	if self:get("has_boss") then

		local progress = self.__t_level_progress
		local boss = self:get_component("boss")

		if boss:get("state") == "disappeared" then

			return "win"

		else

			if progress.n_move_left == 0 then

				return "lose"

			end

		end

	else

		local progress = self.__t_level_progress
		local req_fulfilled = true

		for key, amount in pairs(progress.t_req_remain) do

			if amount > 0 then

				req_fulfilled = false

			end

		end

		if req_fulfilled then

			return "win"

		else

			if progress.n_move_left == 0 then

				return "lose"

			end

		end

	end

end

function GameScene:on_stop()



end

function GameScene:on_pause()



end

function GameScene:after_set_debug_draw(c_msg)

	self.__c_physics_world:setDebugDrawMask(c_msg.x_param.x_new_value)

end

function GameScene:after_set_frame_rate(c_msg)

	cc.Director:getInstance():setAnimationInterval(1.0 / c_msg.x_param.x_new_value)

end

function GameScene:after_set_gravity(c_msg)

	self.__c_physics_world:setGravity(cc.p(0, -c_msg.x_param.x_new_value))

end

function GameScene:after_set_collision_slop(c_msg)

	self.__c_physics_world:setCollisionSlop(c_msg.x_param.x_new_value)

end

function GameScene:after_set_collision_bias(c_msg)

	-- self.__c_physics_world:setCollision_bias(c_msg.x_param.x_new_value)

end

function GameScene:after_set_iteration(c_msg)

	self.__c_physics_world:setIteration(c_msg.x_param.x_new_value)

end

function GameScene:after_set_idle_speed_threshold(c_msg)

	self.__c_physics_world:setIdleSpeedThreshold(c_msg.x_param.x_new_value)

end

function GameScene:after_set_con_move(c_msg)

	local req = self:get("level_config")

	req.steps = c_msg.x_param.x_new_value

end

function GameScene:after_set_has_boss(c_msg)

	if c_msg.x_param.x_new_value then

		self:set("type", "boss")

	end

end

function GameScene:on_add_action_count()

	self.__n_action_count = self.__n_action_count + 1

end

function GameScene:on_reduce_action_count()

	self.__n_action_count = self.__n_action_count - 1

	if self.__n_action_count == 0 then

		self:send_message(MessagePool:get_free_ex("section_end"))

	end

end

function GameScene:on_section_end()

	self.__c_state_machine:do_event("section_end")

end


function GameScene:after_set_level_config(c_msg)

	local config = c_msg.x_param.x_new_value

	if config then

		self:get_component("key_emitter_manager"):set("waves", config.key_waves)

		if config["boss"] then

			local boss = ObjectFactory:get_instance():get("BossSugar")
			self:add_component(boss, "boss")			

		end

	end

end

function GameScene:on_start_animation_finish()

	self.__c_state_machine:do_event("start_animation_finish")

end

function GameScene:init_state_machine()

	function on_enter_game_start_section()

		self:send_message(MessagePool:get_free_ex("disable_item", self.__s_round_type, Message.N_TUNNEL))

		self.__s_round_type = "game_start"
		self:send_message(MessagePool:get_free_ex("round_start", self.__s_round_type, Message.N_TUNNEL))

	end

	function on_enter_start_skill_section()
		print("start skill section")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("start_skill_start", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	function on_enter_hold_section()
-- broadcast event to game
print("enter hold section")
		TRIGGER_EVENT("GAME_READY")
		
		self:send_message(MessagePool:get_free_ex("enable_item", self.__s_round_type, Message.N_TUNNEL))

		self:send_message(MessagePool:get_free_ex("round_finish", self.__s_round_type, Message.N_TUNNEL))
		

	end

	function on_enter_item_remove_section()

		print("enter_item")
		self.__s_round_type = "item"

		self:send_message(MessagePool:get_free_ex("disable_item", self.__s_round_type, Message.N_TUNNEL))

		self:send_message(MessagePool:get_free_ex("round_start", self.__s_round_type, Message.N_TUNNEL))

		Player:getInstance():set_is_player_handle_in_game(true)

	end

	function on_enter_player_remove_section()

		print("enter_link")
		self.__s_round_type = "player"

		self:send_message(MessagePool:get_free_ex("disable_item", self.__s_round_type, Message.N_TUNNEL))
		
		self:send_message(MessagePool:get_free_ex("round_start", self.__s_round_type, Message.N_TUNNEL))

		Player:getInstance():set_is_player_handle_in_game(true)

	end

	function on_remove_leave_check()
		print("check_end")
		local result = self:check_level_progress()

		if result == "win" then

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

			self:send_message(MessagePool:get_free_ex("round_finish", "break", Message.N_TUNNEL))
			-- self:send_message_to_sugars(MessagePool:get_free_ex("round_finish", "break"))

			self.__c_state_machine:do_transition("clear_section")

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_win", "", Message.N_TUNNEL))

			if self.__t_level_progress.n_move_left == 0 then

		        local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
		        event.param = {achv_id = "achv_09", finish_count = 1, func = function() end }
		        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

    		end

    		GameGlobals.open_reward_bag = nil

			return false

		elseif result == "lose" then

			local recheck = function()

				local result = self:check_level_progress()

				if result == "win" then

					GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

					self:send_message(MessagePool:get_free_ex("round_finish", "break", Message.N_TUNNEL))
					-- self:send_message_to_sugars(MessagePool:get_free_ex("round_finish", "break"))

					self.__c_state_machine:do_transition("clear_section")

					GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_win", "", Message.N_TUNNEL))

					if self.__t_level_progress.n_move_left == 0 then

				        local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
				        event.param = {achv_id = "achv_09", finish_count = 1, func = function() end }
				        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		    		end

		    		GameGlobals.open_reward_bag = nil

					return false

				elseif result == "lose" then

					GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

					self:send_message(MessagePool:get_free_ex("round_finish", "break", Message.N_TUNNEL))
					-- self:send_message_to_sugars(MessagePool:get_free_ex("round_finish", "break"))

					self.__c_state_machine:do_transition("clear_section")

					GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_lose", "", Message.N_TUNNEL))

				end

			end

			local call = cc.CallFunc:create(recheck)
			local seq = cc.Sequence:create(cc.DelayTime:create(1.5), call)

			self:runAction(seq)

			GameGlobals.open_reward_bag = nil

			return false

		end

		if GameGlobals.open_reward_bag then
			GameGlobals.open_reward_bag = nil
			cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_reward_bag_enven)
			cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.open_reward_bag_enven)

		end

		return true

	end

	function on_enter_rank_section()
		print("enter_rank")
		local x_param = self.__t_level_progress.n_move_left

		self:send_message(MessagePool:get_free_ex("start_action"))
		-- self:send_message(MessagePool:get_free_ex("rank_start", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("drop_bag", x_param, Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	function on_enter_affect_section()
		print("enter_affect")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("affect_start", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	function on_enter_skill_section()
		print("enter_skill")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("player_skill_start", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	function on_enter_boss_section()
		print("enter_boss")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("boss_move_start", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	function on_enter_clear_section()
		print("enter_clear")
		self:stop_find_timer()

		self:runAction(cc.Sequence:create(cc.DelayTime:create(0), cc.CallFunc:create(function()

			self.__s_round_type = "clear"
			self:send_message(MessagePool:get_free_ex("round_start", self.__s_round_type, Message.N_TUNNEL))	

		end)))		

	end

	function on_leave_clear_section()
		print("leave_clear")

		self.__s_round_type = "end"
		self:send_message(MessagePool:get_free_ex("round_finish", self.__s_round_type, Message.N_TUNNEL))

	end

	function on_enter_bonus_remove_section()
		print("enter_bonus")
		self.__s_round_type = "bonus"
		self:send_message(MessagePool:get_free_ex("round_start", self.__s_round_type, Message.N_TUNNEL))

	end

	function on_enter_bonus_finish_section()

		print("leave_bonus")
		self:send_message(MessagePool:get_free_ex("round_finish", self.__s_round_type, Message.N_TUNNEL))

	end

	function on_leave_start_skill_section()
		print("leave start")
		local event = cc.EventCustom:new("game_play_start")
		event.param = {}
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	end

	function on_enter_transform_section()
		print("enter_transform")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("enter_transform", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "start_section", enter = on_enter_game_start_section},
		{name = "start_skill_section", enter = on_enter_start_skill_section, leave = on_leave_start_skill_section},
		{name = "hold_section", enter = on_enter_hold_section},
		{name = "player_remove_section", enter = on_enter_player_remove_section},
		{name = "item_remove_section", enter = on_enter_item_remove_section},
		{name = "bonus_remove_section", enter = on_enter_bonus_remove_section},
		{name = "bonus_finish_section", enter = on_enter_bonus_finish_section},
		{name = "rank_section", enter = on_enter_rank_section},
		{name = "rank_section_item", enter = on_enter_rank_section},
		{name = "affect_section", enter = on_enter_affect_section},
		{name = "transform_section", enter = on_enter_transform_section},
		{name = "skill_section", enter = on_enter_skill_section, check = on_remove_leave_check},
		{name = "skill_section_item", enter = on_enter_skill_section, check = on_remove_leave_check},
		{name = "boss_section", enter = on_enter_boss_section},
		{name = "clear_section", enter = on_enter_clear_section},

	})
	self.__c_state_machine:add_transition({

		{event = "start_animation_finish", from = "start_section", to = "start_skill_section"},
		{event = "section_end", from = "start_skill_section", to = "hold_section"},
		{event = "player_remove", from = "hold_section", to = "player_remove_section"},
		{event = "section_end", from = "player_remove_section", to = "rank_section"},
		{event = "section_end", from = "rank_section", to = "affect_section"},
		-- {event = "section_end", from = "affect_section", to = "skill_section"},
		{event = "section_end", from = "affect_section", to = "transform_section"},
		{event = "section_end", from = "transform_section", to = "skill_section"},
		{event = "section_end", from = "skill_section", to = "boss_section"},
		{event = "section_end", from = "boss_section", to = "hold_section"},

		{event = "item_remove", from = "hold_section", to = "item_remove_section"},
		{event = "section_end", from = "item_remove_section", to = "rank_section_item"},
		{event = "section_end", from = "rank_section_item", to = "skill_section_item"},
		{event = "section_end", from = "skill_section_item", to = "hold_section"},

		{event = "bonus_remove", from = "clear_section", to = "bonus_remove_section"},
		{event = "section_end", from = "bonus_remove_section", to = "bonus_finish_section"},
		{event = "bonus_remove", from = "bonus_finish_section", to = "bonus_remove_section"},

	})

	self.__c_state_machine:do_transition("start_section")

end

function GameScene:on_link_remove_start()

	self.__c_state_machine:do_event("player_remove")

end

function GameScene:on_trigger_bonus()
print("trigger bonus")
-- ERROR_ASSERT(false, "string")
	local msg = MessagePool:get_free_ex("perform", self.__t_level_progress.n_move_left)

	self:get_component("bonus_performer"):send_message(msg)

end

function GameScene:on_bonus_apply_start()

	self.__c_state_machine:do_event("bonus_remove")

end

function GameScene:on_bonus_remove_finish()

	self.__c_state_machine:do_transition("clear_section")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_clear", "", Message.N_TUNNEL))

end

function GameScene:on_bonus_skip()

	self.__c_state_machine:do_transition("clear_section")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_clear", "", Message.N_TUNNEL))

end

function GameScene:on_item_remove_start()
print("item remove")
	self.__c_state_machine:do_event("item_remove")

end

function GameScene:start_find_timer()

	self:stop_find_timer()

	local delay = cc.DelayTime:create(self:get("available_link_check_delay"))
	local call = cc.CallFunc:create(function()

		local idle_finder = self:get_component("finder")

		local msg = MessagePool:get_free_ex("find_link")

		idle_finder:send_message(msg)

		local find_result = msg.x_result

		if find_result == "game_over" then

			if self.__c_state_machine:get_current_state() == "hold_section" then

				self.__c_state_machine:do_transition("clear_section")

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_end", "", Message.N_TUNNEL))

				
				local player = Player:getInstance()
				local current_level = player:get_player_current_level()
		        local str_level = "关卡"..current_level
		        TalkingDataManager:getInstance():send("关卡失败原因", {[str_level] = "没有可消除的糖果三次"})
		        TalkingDataManager:getInstance():sendMissionInfo("failed", str_level, "没有可消除的糖果三次")

		        
	            local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
	            player:add_player_mission_fail_times(mission_info.mission_id)

			end

		elseif find_result == "no_available_link" then

			local delay = cc.DelayTime:create(1.0)
			local call = cc.CallFunc:create(function()

				local rearranger = self:get_component("rearranger")
				rearranger:send_message(MessagePool:get_free_ex("rearrange"))

				self:start_find_timer()

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

			end)

			local sequence = cc.Sequence:create(delay, call)
			GameGlobals.Scene:runAction(sequence)

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("rearrange_start", "", Message.N_TUNNEL))

		else

			local indicator = self:get_component("indicator")
			indicator:send_message(MessagePool:get_free_ex("start_indicate", find_result))

			local idle_finder = self:get_component("finder")
			idle_finder:send_message(MessagePool:get_free_ex("reset_counter"))

			self.__t_last_hint = find_result

			self:start_hint_check()

		end

	end)

	local sequence = cc.Sequence:create(delay, call)
	sequence:setTag(999)
	self:runAction(sequence)

end

function GameScene:stop_find_timer()

	self:stopActionByTag(999)

	self:stop_hint_check()

end

function GameScene:hint_checker()

	local msg = MessagePool:get_free()
	msg.x_param = self.__t_last_hint
	msg.s_name = "check_available"

	self:get_component("finder"):send_message(msg)

	if not msg.x_result then

		self:stop_hint_check()

		self:get_component("indicator"):send_message(MessagePool:get_free_ex("stop_indicate", ""))

		self:start_find_timer()

	end

end

function GameScene:start_hint_check()

	self:stop_hint_check()
print("game scene start hint check")
	self.__n_check_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:hint_checker() end, 0.1, false)	

end

function GameScene:stop_hint_check()

	if self.__n_check_entry then
print("game scene stop hint check")
		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_check_entry)
		self.__n_check_entry = nil

	end

end

function GameScene:on_touch_down()

	local indicator = self:get_component("indicator")
	indicator:send_message(MessagePool:get_free_ex("stop_indicate", ""))

	self:stop_find_timer()

end

function GameScene:on_touch_up()

	if self:get("state") == "hold_section" then

		self:start_find_timer()

	end

end

function GameScene:on_touch_cancel()

	self:start_find_timer()

end

function GameScene:on_round_finish(c_msg)
print(string.format("round finish : %s", self.__s_round_type))
	local idle_finder = self:get_component("finder")
	idle_finder:send_message(MessagePool:get_free_ex("reset_counter"))

	self:start_find_timer()

end

function GameScene:on_player_round_start(c_msg)

	self:stop_find_timer()

	if c_msg.x_param == "player" then
print("!!!!! reduce move")
		self.__t_level_progress.n_move_left = self.__t_level_progress.n_move_left - 1

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("move_change", self.__t_level_progress.n_move_left, Message.N_TUNNEL))

	end

end

function GameScene:on_target(c_msg)

	local target_color = c_msg.x_param

	if not self:get("has_boss") then

		self.__t_level_progress.t_req_remain[target_color] = self.__t_level_progress.t_req_remain[target_color] - 1

	end

	-- print(string.format("%s:%d", target_color, self.__t_level_progress.t_req_remain[target_color]))

end

function GameScene:on_add_score(c_msg)

	self.__t_level_progress.n_score = self.__t_level_progress.n_score + c_msg.x_param

	self:send_message(MessagePool:get_free_ex("update_score", self.__t_level_progress.n_score, Message.N_TUNNEL))

end

function GameScene:on_bonus_replace()

	self.__t_level_progress.n_move_left = self.__t_level_progress.n_move_left - 1

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("move_change", self.__t_level_progress.n_move_left, Message.N_TUNNEL))

end

function GameScene:on_perform_item_skill(c_msg)
print(self.__n_sugar_count)
	if self:get("state") == "hold_section" then

		local link_sugar = self:get_component("link_handler")
		link_sugar:set("enable", false)

		local skill_component = self:get_component(c_msg.x_param)

		if skill_component then

			skill_component:send_message(MessagePool:get_free_ex("perform"))

		else

			link_sugar:set("enable", true)

		end

	end

end

function GameScene:on_cancel_item_skill(c_msg)

	print("item canceled !!@@")

	local link_sugar = self:get_component("link_handler")
	link_sugar:set("enable", true)

	local skill_component = self:get_component(c_msg.x_param)

	if skill_component then

		skill_component:send_message(MessagePool:get_free_ex("cancel"))

	end

	self.__c_state_machine:do_set("hold_section")

end

function GameScene:is_sugar_counted(sugar)

	local category = sugar:get_category()

	if category:has("normal") or 
		category:has("chocolate") or
		category:has("special") or 
		category:has("random") or 
		category:has("moveable_ice") or 
		category:has("panda") or 
		category:has("colorful") or 
		category:has("timing_bomb") then

		return true

	else

		return false

	end

end

function GameScene:on_add_move(c_msg)

	self.__t_level_progress.n_move_left = self.__t_level_progress.n_move_left + c_msg.x_param

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("move_change", self.__t_level_progress.n_move_left, Message.N_TUNNEL))

end

function GameScene:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end

function GameScene:on_emit_link(c_msg)

	-- print(self.__n_sugar_count)

	local task_number = self:get("sugar_volume") - self.__n_sugar_count

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", task_number))
	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

	self.__n_sugar_count = self:get("sugar_volume")


end

function GameScene:on_emit_special(c_msg)

	-- print(self.__n_sugar_count)

	local task_number = self:get("sugar_volume") - self.__n_sugar_count
print("task number")
print(task_number)
	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", task_number))
	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

	self.__n_sugar_count = self:get("sugar_volume")

end

function GameScene:on_emit_self(c_msg)

	-- print(self.__n_sugar_count)

	local task_number = self:get("sugar_volume") - self.__n_sugar_count

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", task_number))
	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

	self.__n_sugar_count = self:get("sugar_volume")

end

function GameScene:on_cake_remove(c_msg)

	if self.__c_state_machine:get_current_state() == "hold_section" then

		local result = self:check_level_progress()

		if result == "win" then

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

			self.__c_state_machine:do_transition("clear_section")

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_win", "", Message.N_TUNNEL))

			return false

		elseif result == "lose" then

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_passive", "", Message.N_TUNNEL))

			self.__c_state_machine:do_transition("clear_section")

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_lose", "", Message.N_TUNNEL))

			return false

		end

	end

end

function GameScene:on_relive(c_msg)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", 5))

	self.__c_state_machine:do_transition("hold_section")

end

function GameScene:on_bomb_relive(c_msg)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", 5))

	local t_filter = {}
	t_filter.t_category_filter = {
		type = "and",
		{"timing_bomb", "=", true},	
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local result = msg.x_result 

	for i = 1, #result do

		local bomb = result[i]

		local current_count_down = bomb:get("count_down")

		bomb:set("count_down", current_count_down + 5)

	end

	self.__c_state_machine:do_transition("hold_section")

end

function GameScene:on_timing_bomb_explode()

	if self.__c_state_machine:get_current_state() == "hold_section" then

		self.__c_state_machine:do_transition("clear_section")

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_lose_bomb", "", Message.N_TUNNEL))

	end	

end

function GameScene:on_die_bomb()

	local bomb_spine = ResourceManager:get_instance():get_resource_by_key("timing_bomb_sugar_spine")
	bomb_spine:setScale(1)
	bomb_spine:setAnimation(0, "zhadan_bomb", false)
	bomb_spine:setPosition(cc.p(540, 960))
	bomb_spine:setSkin("blue")

	cc.Director:getInstance():getRunningScene():addChild(bomb_spine, 99)

	local seq_shake = cc.Sequence:create(cc.DelayTime:create(0.033 * 30), cc.CallFunc:create(function()

		cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_1))

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("timingbomb_explode", "", Message.N_TUNNEL))

	end))

	cc.Director:getInstance():getRunningScene():runAction(seq_shake)

	local seq_end = cc.Sequence:create(cc.DelayTime:create(0.033 * 10), cc.CallFunc:create(function()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_end", "", Message.N_TUNNEL))
		
	end))

	cc.Director:getInstance():getRunningScene():runAction(seq_end)

end

function GameScene:on_boss_dead()

	local seq = cc.Sequence:create(cc.DelayTime:create(0.3), cc.CallFunc:create(function()

		cc.Director:getInstance():runTimeScaleChangeAction(1.0, 0.2, 0.9)

	end))

	self.__c_scene:runAction(seq)

end

function GameScene:get_sugar_by_category(category)

	for _, sugar in pairs(self.__t_sugars) do

		if sugar:get_category():has(category) then

			return sugar

		end

	end

end


function GameScene:on_get_valid_color_random(c_msg)

	local emitter = self:get_sugar_by_category("emitter")

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "draw_color"
	msg.x_param = {}

	emitter:send_message(msg)

	c_msg.x_result = msg.x_result

end

GameScene.LISTENER_CONFIG = {

	{"add_sugar", {before = GameScene.on_add_to_scene}},
	{"add_sugar_without_count", {before = GameScene.on_add_to_scene_without_count}},
	{"remove_sugar", {after = GameScene.on_remove_from_scene}},
	{"remove_sugar_silent", {after = GameScene.on_remove_from_scene}},
	{"query_shape", {before = GameScene.on_query_shape}},
	{"query_point", {before = GameScene.on_query_point}},
	{"query_line", {before = GameScene.on_query_line}},
	{"query_filter", {before = GameScene.on_query_filter}},
	{"run", {before = GameScene.on_run}},
	{"pause", {before = GameScene.on_pause}},
	{"stop", {before = GameScene.on_stop}},
	{"destroy", {before = GameScene.on_destroy}},
	{"normal_add_child", {before = GameScene.on_add_child}},
	{"start_action", {before = GameScene.on_add_action_count}},
	{"finish_action", {before = GameScene.on_reduce_action_count}},
	{"link_remove_start", {before = GameScene.on_link_remove_start}},
	{"special_link_start", {before = GameScene.on_link_remove_start}},
	{"special_touch_start", {before = GameScene.on_item_remove_start}},
	{"section_end", {before = GameScene.on_section_end}},
	{"round_finish", {before = GameScene.on_round_finish}},
	{"touch_down", {before = GameScene.on_touch_down}},
	{"touch_up", {before = GameScene.on_touch_up}},
	{"touch_cancel", {before = GameScene.on_touch_cancel}},
	{"trigger_bonus", {before = GameScene.on_trigger_bonus}},
	{"bonus_apply_start", {before = GameScene.on_bonus_apply_start}},
	{"bonus_finish", {before = GameScene.on_bonus_remove_finish}},
	{"bonus_skip", {before = GameScene.on_bonus_skip}},
	{"bonus_replace", {before = GameScene.on_bonus_replace}},
	{"round_start", {before = GameScene.on_player_round_start}},
	{"target", {before = GameScene.on_target}},
	{"add_score", {before = GameScene.on_add_score}},
	{"perform_item_skill", {before = GameScene.on_perform_item_skill}},
	{"cancel_item_skill", {before = GameScene.on_cancel_item_skill}},
	{"item_remove_start", {before = GameScene.on_item_remove_start}},
	{"add_move", {before = GameScene.on_add_move}},
	{"link_remove_finish", {before = GameScene.on_emit_link}},	
	{"special_remove_finish", {before = GameScene.on_emit_special}},	
	{"self_remove_finish", {before = GameScene.on_emit_self}},	
	{"start_animation_finish", {before = GameScene.on_start_animation_finish}},		
	{"cake_remove", {before = GameScene.on_cake_remove}},
	{"relive", {before = GameScene.on_relive}},	
	{"relive_bomb", {before = GameScene.on_bomb_relive}},
	{"die_bomb", {before = GameScene.on_die_bomb}},	
	{"timing_bomb_explode", {before = GameScene.on_timing_bomb_explode}},
	{"boss_dead", {before = GameScene.on_boss_dead}},
	{"get_valid_color_random", {before = GameScene.on_get_valid_color_random}},					

}

GameScene.PROPERTY_CONFIG = {
	
	{"debug_draw", cc.PhysicsWorld.DEBUGDRAW_NONE},
	{"frame_rate", 40},
	{"gravity", 2000},
	{"iteration", 2},
	-- {"collision_slop", 0},
	-- {"collision_bias", 1},
	-- {"idle_speed_threshold", 200},
	{"level_config", {}},
	{"sugar_volume", 50},
	{"available_link_check_delay", 2},
	{"has_boss", false},
	{"state", "hold_section"},
	{"type", "normal"},
}


return GameScene