local GameScene = require "GameEx.Components.Scene.GameScene"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UIComponent = require "GameEx.Components.Scene.UIComponent"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager_Core = require "core.ResourceManager"
local LevelBuilder = require "GameEx.Foundations.LevelBuilderTournament"
local ConfigManager = require "GameEx.Foundations.ConfigManager"
local TournamentProgresser = require "GameEx.Components.Scene.TournamentProgresser"


local GameSceneTournament = class("GameSceneTournament", function() return GameScene:create() end)

function GameSceneTournament:ctor()

	self:extend_state_machine()

	self:add_component(ObjectFactory:get_instance():get("TargetHandlerTournament"), "target_handler")

	self:add_component(ObjectFactory:get_instance():get("GiftSugarManager"), "gift_sugar_manager")

	self:add_component(ObjectFactory:get_instance():get("ConchSkill"), "conch_skill")

	self:add_component(ObjectFactory:get_instance():get("TournamentRewardSkill"), "tournament_reward_skill")

	self:register_listeners_with_config(GameSceneTournament.LISTENER_CONFIG)
	self:register_properties_with_config(GameSceneTournament.PROPERTY_CONFIG)

	self:set("type", "tournament")

	self.__n_current_level = 1

end

function GameSceneTournament:on_reset(c_msg)

	self.__ctc = __ctc or 0
	self.__ctc = self.__ctc + 1

	if self.__ctc > 1 then

		ERROR_ASSERT(false, "string")

	end

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

end

function GameSceneTournament:extend_state_machine()

	function on_enter_gift_check_section()
		print("enter_gift_check")
		self:send_message(MessagePool:get_free_ex("start_action"))
		self:send_message(MessagePool:get_free_ex("gift_check", "", Message.N_TUNNEL))
		self:send_message(MessagePool:get_free_ex("finish_action"))

	end

	self.__c_state_machine:add_state({

		{name = "gift_check_section", enter = on_enter_gift_check_section},
		{name = "gift_check_section_item", enter = on_enter_gift_check_section},

	})

	self.__c_state_machine:add_transition({

		{event = "section_end", from = "rank_section", to = "gift_check_section"},
		{event = "section_end", from = "gift_check_section", to = "affect_section"},

		{event = "section_end", from = "rank_section_item", to = "gift_check_section_item"},
		{event = "section_end", from = "gift_check_section_item", to = "skill_section_item"},

	})

end
 
function GameSceneTournament:on_target(c_msg)

	GameScene.on_target(self, c_msg)

	if c_msg.x_param == "conch" then

		local current = self:get("conch_collected")
		self:set("conch_collected", current + 1)

	end

end

function GameSceneTournament:on_run()

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", self:get("sugar_volume") - self.__n_sugar_count))

	self.__n_sugar_count = self:get("sugar_volume")

	self:init_level_progress()
	self:start_find_timer()

	self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))
	self:get_component("key_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

	self.__c_state_machine:do_transition("start_section")

	if self:get("has_boss") then

		local boss = self:get_component("boss")
		local hp = boss:get("max_hp")
		local ui = self:get_component("ui_component")

		ui:set_boss_full_hp(hp)

	end

	display.runScene(self.__c_scene)

	MusicManager:getInstance().play_effect(ResourceManager_Core:getInstance():getAudioPathByKey("Water_Audio"), false)

	local water_spine = ResourceManager:get_instance():get_resource_by_key("before_game_water")
	water_spine:setScale(1.0)
	water_spine:setAnimation(0, "water_up", false)
	water_spine:addAnimation(0, "water_loop", true)
	water_spine:setPosition(cc.p(540, 960))

	self.__c_scene:addChild(water_spine, 9)

	local water_spine = ResourceManager:get_instance():get_resource_by_key("before_game_water")
	water_spine:setScale(1.0)
	water_spine:setAnimation(0, "popo_loop_1", true)
	water_spine:setAnimation(1, "popo_loop_2", true)
	water_spine:setPosition(cc.p(540, 960))

	self.__c_scene:addChild(water_spine, 9)

end

function GameSceneTournament:on_target_animation_finish(c_msg)
-- ERROR_ASSERT(false, "string")
	self:get_component("ui_component"):send_message(MessagePool:get_free_ex("update_conch", self:get("conch_collected"), Message.N_TUNNEL))

end

function GameSceneTournament:on_remove_all_sugars(c_msg)

	for sugar, _ in pairs(self.__t_sugars) do

		self:send_message(MessagePool:get_free_ex("remove_sugar_silent", sugar, Message.N_TUNNEL))

	end

	self.__t_sugars = {}

end

function GameSceneTournament:on_to_next_phase()
	MusicManager:getInstance().play_effect(ResourceManager_Core:getInstance():getAudioPathByKey("Refresh1_Audio"), false)
	
	self.__n_current_level = self.__n_current_level + 1
	self:get_component("ui_component"):send_message(MessagePool:get_free_ex("refresh_board", self.__n_current_level))

	local delay = cc.DelayTime:create(0.6)
	local call = cc.CallFunc:create(function()

	self:on_remove_all_sugars()

		local progresser = TournamentProgresser:get_instance()
		local next_level = progresser:get_next_level()

		LevelBuilder:get_instance():load_level_design(next_level)
		LevelBuilder:get_instance():build_current_level(self)

		self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("direct_assign", self:get("sugar_volume") - self.__n_sugar_count))

		self.__n_sugar_count = self:get("sugar_volume")

		self:get_component("normal_emitter_manager"):send_message(MessagePool:get_free_ex("dispatch_task"))

		self.__c_state_machine:do_transition("hold_section")

		self:get_component("ui_component"):send_message(MessagePool:get_free_ex("relive", ""))

		local config = ConfigManager:get_instance():get_config("GameScene")
		local parse_result = LevelBuilder:get_instance():parse_level_config(config.properties)
		local original = self:get("level_config")
		original.reqs = parse_result.reqs

		self:set("level_config", original)
		self.__t_level_progress.t_req_remain = original.reqs

	end)

	local seq = cc.Sequence:create(delay, call)

	GameGlobals.Scene.__c_scene:runAction(seq)

end

function GameSceneTournament:on_tournament_reward()

	self:get_component("tournament_reward_skill"):send_message(MessagePool:get_free_ex("tournament_reward_start", ""))

end

function GameSceneTournament:on_reward_conch(c_msg)

	local current = self:get("conch_collected")
	self:set("conch_collected", current + c_msg.x_param)

	self:get_component("ui_component"):send_message(MessagePool:get_free_ex("add_target", c_msg.x_param))

end

function GameSceneTournament:check_level_progress()

	local progress = self.__t_level_progress
	local req_fulfilled = true

	for key, amount in pairs(progress.t_req_remain) do

		if amount > 0 then

			req_fulfilled = false

		end

	end

	if progress.n_move_left == 0 then

		return "lose"

	else

		if req_fulfilled then

			return "win"

		end

	end

end

function GameSceneTournament:is_sugar_visible(c_sugar)

	local position = cc.p(c_sugar:getPosition())
	local visible_size = cc.Director:getInstance():getVisibleSize()

	if position.x >= visible_size.width / 2 - 500 and
		position.x <= visible_size.width / 2 + 500 and
		position.y <= 1480 and
		position.y >= 250 then

		return true

	else

		return false

	end

end

function GameSceneTournament:on_relive(c_msg)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", 5))

	local result = self:check_level_progress()

	if result == "win" then

		self:send_message(MessagePool:get_free_ex("round_finish", "break", Message.N_TUNNEL))

		self.__c_state_machine:do_transition("clear_section")

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("game_win", "", Message.N_TUNNEL))

		return false

	elseif result == "lose" then
	else

		self.__c_state_machine:do_transition("hold_section")

	end

	return true

end

function GameSceneTournament:is_sugar_visible(c_sugar)

	local position = cc.p(c_sugar:getPosition())
	local visible_size = cc.Director:getInstance():getVisibleSize()

	if position.x >= visible_size.width / 2 - 500 and
		position.x <= visible_size.width / 2 + 500 and
		position.y <= 1480 and
		position.y >= 250 then

		return true

	else

		return false

	end

end

function GameSceneTournament:is_goal_reached()

	local progress = self.__t_level_progress
	local req_fulfilled = true

	for key, amount in pairs(progress.t_req_remain) do

		if amount > 0 then

			req_fulfilled = false

		end

	end

	if req_fulfilled then

		return true

	else

		return false

	end

end

GameSceneTournament.LISTENER_CONFIG = {

	{"target", {before = GameSceneTournament.on_target}},	
	{"sugar_target_animation_finish", {before = GameSceneTournament.on_target_animation_finish}},	
	{"to_next_phase", {before = GameSceneTournament.on_to_next_phase}},
	{"remove_all_sugars", {before = GameSceneTournament.on_remove_all_sugars}},
	{"run", {before = GameSceneTournament.on_run}},	
	{"tournament_reward", {before = GameSceneTournament.on_tournament_reward}},
	{"reward_conch", {before = GameSceneTournament.on_reward_conch}},
	{"reward_skill_finish", {before = GameSceneTournament.on_to_next_phase}},
	{"reset", {before = GameSceneTournament.on_reset}},
	{"relive", {before = GameSceneTournament.on_relive}},

}

GameSceneTournament.PROPERTY_CONFIG = {
	
	{"conch_collected", 0},
	{"is_tournament", true}

}

return GameSceneTournament