-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager1 = require "core.ResourceManager"

-- ## Imports ## --

local RankHandler = class("RankHandler", function() return GameObject:create() end)

function RankHandler:ctor()

	self.__c_remove_counter = 0

	self:register_listeners_with_config(RankHandler.LISTENER_CONFIG)
	self:register_properties_with_config(RankHandler.PROPERTY_CONFIG)

end

function RankHandler:on_round_start(c_msg)

	self.__c_remove_counter = 0

end

function RankHandler:on_round_finish(t_pos)
print("RankHandler: rank_start !!@@@@@")
	if self.__c_remove_counter > 0 then

		local pos = {}
		pos.x = t_pos.x
		pos.y = t_pos.y + 200

		if pos.x < 200 then

			pos.x = 200

		elseif pos.x > 880 then

			pos.x = 880 

		end

		if self.__c_remove_counter < 10 then

			return

		end

		local spine = ResourceManager:get_instance():get_resource_by_key("rank_spine")
		spine:setPosition(pos)
		spine:setLocalZOrder(999)
		spine:registerSpineEventHandler(function (event)

			spine:removeFromParent()

		end, sp.EventType.ANIMATION_END)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local rank = "none"

		if self.__c_remove_counter >= 10  and self.__c_remove_counter < 20 then
print("Good") 
			spine:setAnimation(0, "Good", false)
			rank = "good"

		elseif self.__c_remove_counter >= 20 and self.__c_remove_counter < 33 then
print("Great") 
			spine:setAnimation(0, "Great", false)
			rank = "great"

		elseif self.__c_remove_counter >= 33 and self.__c_remove_counter < 45 then
print("Perfect") 
			spine:setAnimation(0, "Perfect", false)
			rank = "perfect"

		elseif self.__c_remove_counter >= 45 and self.__c_remove_counter < 55 then
print("Excellent") 
			spine:setAnimation(0, "Excellent", false)
			rank = "excellent"

		elseif self.__c_remove_counter >= 55 then
print("Wonderfull") 		 	
			spine:setAnimation(0, "Wonderfull", false)
			rank = "wonderful"

		end

		if not GameGlobals.bonus then

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("do_rank", rank, Message.N_TUNNEL))

		end

		local function empty_func()



		end

		RUN_ACTION_WITH_DELAY(1.5, empty_func)

	end

end

function RankHandler:on_sugar_remove(c_msg)

	local sugar = c_msg.x_param
	local category = sugar:get_category()

	if category:has("normal") or category:has("timing_bomb") or category:has("random") then

		self.__c_remove_counter = self.__c_remove_counter + 1

	end

end

function RankHandler:on_link_remove_finish(c_msg)

	local remove_list = c_msg.x_param

	self:on_round_finish(cc.p(remove_list[#remove_list]:getPosition()))

end

function RankHandler:on_special_remove_finish(c_msg)

	self:on_round_finish(c_msg.x_param.t_pos)

end


RankHandler.LISTENER_CONFIG = {

	{"remove_sugar", {before = RankHandler.on_sugar_remove}},
	{"round_start", {before = RankHandler.on_round_start}},
	-- {"rank_start", {before = RankHandler.on_round_finish}},
	{"special_remove_finish", {before = RankHandler.on_special_remove_finish}},
	{"link_remove_finish", {before = RankHandler.on_link_remove_finish}},

}

RankHandler.PROPERTY_CONFIG = {
	
	

}

return RankHandler