-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local ScoreHandler = class("ScoreHandler", function() return GameObject:create() end)

function ScoreHandler:ctor()

	self.__c_remove_counter = 0

	self:register_listeners_with_config(ScoreHandler.LISTENER_CONFIG)
	self:register_properties_with_config(ScoreHandler.PROPERTY_CONFIG)

end

function ScoreHandler:on_link_remove_finish(c_msg)

	self:caculate_normal_score(c_msg)
	
end

function ScoreHandler:on_special_remove_finish(c_msg)

	self:caculate_special_score(c_msg)

end

function ScoreHandler:on_target_finale(c_msg)

	self:caculate_normal_damage(c_msg)

end

function ScoreHandler:on_blast_finale(c_msg)
print("blast")
	c_msg.x_param.apd_n_damage = 1
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_damage", 1, Message.N_TUNNEL))

end

function ScoreHandler:caculate_normal_damage(c_msg)

	local a = self:get("player_damage_bonus")
	local b = self:get("normal_damage_b")
	local c = self:get("normal_damage_c")

	local n = #c_msg.x_param

	if n > 0 then

		local position = cc.p(c_msg.x_param[n]:getPosition())
		local result = math.floor((a + (n - b) * n * c))

		for i = 1, n do

			c_msg.x_param[i].apd_n_damage = math.floor(result / n)

		end

		local remainder = result - math.floor(result / n) * n
		c_msg.x_param[n].apd_n_damage = c_msg.x_param[n].apd_n_damage + remainder

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_damage", result, Message.N_TUNNEL))

	end

end

function ScoreHandler:caculate_normal_score(c_msg)

	local a = self:get("normal_score_a")
	local b = self:get("normal_score_b")
	local c = self:get("normal_score_c")
	local bonus = self:get("player_score_bonus")

	local n = #c_msg.x_param

	local position = cc.p(c_msg.x_param[n]:getPosition())
	local result = math.floor(((a * n ^ b) * ( 1 + n / c)) * (1 + bonus))

	if result > 0 then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_score", result, Message.N_TUNNEL))
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("score_animation", {score = result, position = position}, Message.N_TUNNEL))

	end	

end

function ScoreHandler:caculate_special_score(c_msg)

	local a = self:get("special_score_a")
	local b = self:get("special_score_b")
	local c = self:get("special_score_c")
	local bonus = self:get("player_score_bonus")

	local n = #c_msg.x_param.c_list

	local result = math.floor(((a * n ^ b) * ( 1 + n / c)) * (1 + bonus))
	local position = c_msg.x_param.t_pos

	if result > 0 then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_score", result, Message.N_TUNNEL))
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("score_animation", {score = result, position = position}, Message.N_TUNNEL))

	end		

end

function ScoreHandler:on_bonus_remove(c_msg)

	local a = self:get("bonus_score_a")

	local position = cc.p(c_msg.x_param:getPosition())

	local result = a

	-- self:play_score_animation(result, position)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_score", result, Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("score_animation", {score = result, position = position}, Message.N_TUNNEL))	

end

function ScoreHandler:on_skill_remove_finish(c_msg)

	local a = self:get("normal_score_a")
	local b = self:get("normal_score_b")
	local c = self:get("normal_score_c")
	local bonus = self:get("player_score_bonus")

	local n = #c_msg.x_param
	local result = 0

	if n < 3 then

		result = math.floor(((a * 3 ^ b) * ( 1 + 3 / c)) * (1 + bonus) / 3 * n)

	else

		result = math.floor(((a * n ^ b) * ( 1 + n / c)) * (1 + bonus))

	end



	local position = cc.p(c_msg.x_param[n]:getPosition())

	if result > 0 then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_score", result, Message.N_TUNNEL))
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("score_animation", {score = result, position = position}, Message.N_TUNNEL))

	end		

end

ScoreHandler.LISTENER_CONFIG = {

	{"link_remove_finish", {before = ScoreHandler.on_link_remove_finish}},
	{"special_remove_finish", {before = ScoreHandler.on_special_remove_finish}},
	{"skill_remove_finish", {before = ScoreHandler.on_skill_remove_finish}},
	{"target_finale", {before = ScoreHandler.on_target_finale}},
	{"bonus_replace", {before = ScoreHandler.on_bonus_remove}},
	{"special_target_finale", {before = ScoreHandler.on_blast_finale}},

}

ScoreHandler.PROPERTY_CONFIG = {
	
	{"normal_score_a", 10},
	{"normal_score_b", 2},
	{"normal_score_c", 28},
	{"special_score_a", 25},
	{"special_score_b", 1.8},
	{"special_score_c", 54},
	{"bonus_score_a", 2000},
	{"player_score_bonus", 0.01},
	{"player_damage_bonus", 300},
	{"normal_damage_b", 3},
	{"normal_damage_c", 5},

}

return ScoreHandler