-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local SkillPerformer = class("SkillPerformer", function() return GameObject:create() end)

function SkillPerformer:ctor()

	self:register_listeners_with_config(SkillPerformer.LISTENER_CONFIG)
	self:register_properties_with_config(SkillPerformer.PROPERTY_CONFIG)

end

function SkillPerformer:on_perform_skill(c_msg)

	local t_filter = {}
	t_filter.t_category_filter = {
		type = "and",
		{"normal", "=", true},	
	}
	t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	if #msg.x_result > 0 then

		local target_sugar = msg.x_result[1]

		target_sugar:send_message(MessagePool:get_free_ex("lock"))

		local position = cc.p(target_sugar:getPosition())

		local new_sugar = ObjectFactory:get_instance():get("BombSugar")
		new_sugar:setPosition(position)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", target_sugar, Message.N_TUNNEL))
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", new_sugar, Message.N_TUNNEL))

	end

end


SkillPerformer.LISTENER_CONFIG = {

	{"perform_skill", {before = SkillPerformer.on_perform_skill}},

}

SkillPerformer.PROPERTY_CONFIG = {
	
	{"target_list", {}}

}

return SkillPerformer