-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManagerEx = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local SoundPerformer = class("SoundPerformer", function() return GameObject:create() end)

function SoundPerformer:ctor()

	self.__n_link_started = false
	self.__n_link_counter = 0

	self:register_listeners_with_config(SoundPerformer.LISTENER_CONFIG)
	self:register_properties_with_config(SoundPerformer.PROPERTY_CONFIG)

	ccexp.AudioEngine:setMaxAudioInstance(8)

end

function SoundPerformer:on_destroy()

	-- self.__c_schedule_node:removeFromParent()

end

function SoundPerformer:on_link_remove_start(c_msg)

	self.__n_link_started = true

end

function SoundPerformer:on_link_remove_finish(c_msg)

	self.__n_link_started = false
	self.__n_link_counter = 0

end

function SoundPerformer:on_sugar_remove(c_msg)

	local category = c_msg.x_param:get_category()

	if	category:has("normal") or
		category:has("colorful") then

		self:play_sound("Combo_Voice_Audio")
		-- MusicManager:getInstance().play_2d(ResourceManager:getInstance():getAudioPathByKey("Combo_Voice_Audio"))

	elseif category:has("ice") then

		self:play_sound("Ice_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ice_Audio"), false)

	elseif category:has("panda") then

		self:play_sound("Panda_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Panda_Audio"), false)

	elseif category:has("fenced") then

		self:play_sound("Fence_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Fence_Audio"), false)

	elseif category:has("fan") then

		self:play_sound("Flower_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Flower_Audio"), false)

	elseif category:has("chocolate") then

		self:play_sound("Cotton_Candy_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Cotton_Candy_Audio"), false)

	elseif category:has("moveable_ice") then

		self:play_sound("Cookies_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Cookies_Audio"), false)

	elseif category:has("key") or category:has("reward_bag") then

		self:play_sound("Cake_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Cake_Audio"), false)

	elseif category:has("random") then

		self:play_sound("Pots_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Pots_Audio"), false)

	elseif category:has("coco") then

		self:play_sound("Coconut1_Audio")
		-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Pots_Audio"), false)

	end

end

function SoundPerformer:on_link_selected(c_msg)

	local index = c_msg.x_param

	if index > 10 then 

		index = 10

	end

	self:play_sound(string.format("Combo%d_Audio", index))

end

function SoundPerformer:on_target_collect_animation()

	self:play_sound("Goal_Audio")
	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Goal_Audio", index)), false)

end

function SoundPerformer:on_bomb_remove()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Bomb_Audio", index)), false)
	self:play_sound("Bomb_Audio")

end

function SoundPerformer:on_bomb_remove_ignite()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Bombpang_Audio", index)), false)
	self:play_sound("Bombpang_Audio")

end

function SoundPerformer:on_cross_remove()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Cross_Audio", index)), false)
	self:play_sound("Cross_Audio")

end

function SoundPerformer:on_cross_remove_ignite()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Cross_Audio", index)), false)
	self:play_sound("Cross_Audio")

end

function SoundPerformer:on_lightning_remove()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Light_Audio", index)), false)
	self:play_sound("Light_Audio")

end

function SoundPerformer:on_lightning_remove_ignite()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Light_Audio", index)), false)
	self:play_sound("Light_Audio")

end

function SoundPerformer:on_lightning_remove_begin()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Light_Audio", index)), false)
	self:play_sound("Light_Audio")

end

function SoundPerformer:on_bonus_replace()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Reward_Audio", index)), false)
	self:play_sound("Reward_Audio")

end

function SoundPerformer:on_hammer_skill_hit()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Hammer_Audio", index)), false)
	self:play_sound("Hammer_Audio")

end

function SoundPerformer:on_candy_skill_hit()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Reward_Audio", index)), false)
	self:play_sound("Reward_Audio")

end

function SoundPerformer:on_move_skill_hit()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Addstep_Audio", index)), false)
	self:play_sound("Addstep_Audio")

end

function SoundPerformer:on_bomb_skill_hit()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Bombuse_Audio", index)), false)
	self:play_sound("Bombuse_Audio")

end

function SoundPerformer:on_ice_break()

	self:play_sound("Ice_Audio")
	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Ice_Audio", index)), false)

end

function SoundPerformer:on_moveable_break()

	self:play_sound("Cookies_Audio")
	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Ice_Audio", index)), false)

end

function SoundPerformer:on_coco_break()

	self:play_sound("Coconut_Audio")
	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Ice_Audio", index)), false)

end


function SoundPerformer:on_soil_break()

	self:play_sound("Soil_Audio")
	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Ice_Audio", index)), false)

end

function SoundPerformer:on_chocolate_spread()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("Spread_Audio", index)), false)
	self:play_sound("Spread_Audio")

end

function SoundPerformer:on_timing_bomb_ready()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("TimingBomb_Ready", index)), false)
	self:play_sound("TimingBomb_Ready")

end

function SoundPerformer:on_timing_bomb_explode()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("TimingBomb_Explode", index)), false)
	self:play_sound("TimingBomb_Explode")

end

function SoundPerformer:on_flower_fill()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("TimingBomb_Explode", index)), false)
	self:play_sound("Flower_Audio")

end

function SoundPerformer:on_total_remove_start()

	-- MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(string.format("TimingBomb_Explode", index)), false)
	self:play_sound("Light_Audio")

end

function SoundPerformer:on_skill()

	self:play_sound("Skill_Audio")

end

function SoundPerformer:on_do_rank(c_msg)

	local rank = c_msg.x_param
	local voice_key = string.format("VOICE_%s", rank)

	if not self.__n_rank_playing then

		self:play_sound(voice_key)
		self.__n_rank_playing = true

		local func = cc.CallFunc:create(function() self.__n_rank_playing = false end)
		local sequence = cc.Sequence:create(cc.DelayTime:create(0.2), func)

		GameGlobals.Scene:runAction(sequence)

	end

end

function SoundPerformer:play_sound(sound_key, conflict_window)

	MusicManager:getInstance():play_sound(sound_key, conflict_window)

end


SoundPerformer.LISTENER_CONFIG = {

	{"link_remove_start", {before = SoundPerformer.on_link_remove_start}},
	{"link_remove_finish", {before = SoundPerformer.on_link_remove_finish}},
	{"remove_sugar", {before = SoundPerformer.on_sugar_remove}},
	{"link_selected", {before = SoundPerformer.on_link_selected}},
	{"target_collect_animation", {before = SoundPerformer.on_target_collect_animation}},
	{"bomb_remove", {before = SoundPerformer.on_bomb_remove}},
	{"bomb_remove_ignite", {before = SoundPerformer.on_bomb_remove_ignite}},
	{"cross_remove", {before = SoundPerformer.on_cross_remove}},
	{"cross_remove_ignite", {before = SoundPerformer.on_cross_remove_ignite}},
	{"lightning_remove", {before = SoundPerformer.on_lightning_remove}},
	{"lightning_remove_ignite", {before = SoundPerformer.on_lightning_remove_ignite}},
	{"lightning_remove_begin", {before = SoundPerformer.on_lightning_remove_begin}},
	{"bonus_replace", {before = SoundPerformer.on_bonus_replace}},
	{"bonus_replace_start", {before = SoundPerformer.on_bonus_replace}},
	{"bonus_replace_gift", {before = SoundPerformer.on_bonus_replace}},
	{"hammer_skill_hit", {before = SoundPerformer.on_hammer_skill_hit}},
	{"candy_skill_hit", {before = SoundPerformer.on_candy_skill_hit}},
	{"move_skill_hit", {before = SoundPerformer.on_move_skill_hit}},
	{"bomb_skill_hit", {before = SoundPerformer.on_bomb_skill_hit}},
	{"ice_break", {before = SoundPerformer.on_ice_break}},
	{"soil_break", {before = SoundPerformer.on_soil_break}},
	{"moveable_break", {before = SoundPerformer.on_moveable_break}},
	{"coco_break", {before = SoundPerformer.on_coco_break}},
	{"chocolate_spread", {before = SoundPerformer.on_chocolate_spread}},
	{"timingbomb_ready", {before = SoundPerformer.on_timing_bomb_ready}},
	{"timingbomb_explode", {before = SoundPerformer.on_timing_bomb_explode}},
	{"flower_fill", {before = SoundPerformer.on_flower_fill}},
	{"total_remove_start", {before = SoundPerformer.on_total_remove_start}},
	{"destroy", {before = SoundPerformer.on_destroy}},
	{"do_rank", {before = SoundPerformer.on_do_rank}},
	{"player_skill_cast", {before = SoundPerformer.on_skill}},

}

SoundPerformer.PROPERTY_CONFIG = {
	
	

}

return SoundPerformer