-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local TargetHandler = class("TargetHandler", function() return GameObject:create() end)

function TargetHandler:ctor()

	local factory = ObjectFactory:get_instance()

	self:add_component(factory:get("TargetVisual"), "visual")

	self:register_listeners_with_config(TargetHandler.LISTENER_CONFIG)
	self:register_properties_with_config(TargetHandler.PROPERTY_CONFIG)

end

function TargetHandler:on_remove(c_msg)

	local sugar = c_msg.x_param
	local goal_color = self:get_goal_color(sugar)

	if goal_color then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("target", goal_color, Message.N_TUNNEL))

		self:get_component("visual"):send_message(MessagePool:get_free_ex("target_animation", sugar))

	end

end

function TargetHandler:get_goal_color(c_sugar)

	if GameGlobals.Scene:get("has_boss") then

		local boss = GameGlobals.Scene:get_component("boss")

		if boss:get("state") == "disappeared" then

			return 

		end

	end

	local goals = GameGlobals.Scene:get("level_config").reqs

	for color, number in pairs(goals) do

		if number > 0 then

			if c_sugar:get_category():has(color) then

				return color

			end

		end

	end

	return nil

end

function TargetHandler:on_normal_remove_finish(c_msg)

	local removed_targets = {}
	local total_removed = c_msg.x_param
	local goals = GameGlobals.Scene:get("level_config").reqs

	for i = 1, #total_removed do

		for color, number in pairs(goals) do

			if total_removed[i]:get_category():has(color) then

				if total_removed[i]:get_category():has("normal") 
					and not total_removed[i]:get_category():has("fenced") 
					and not total_removed[i].repeatitive then

					removed_targets[#removed_targets + 1] = total_removed[i]
					total_removed[i].repeatitive = true

				end

				break

			end

		end

	end

	for i = 1, #total_removed do

		total_removed[i].repeatitive = false

	end	

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("target_finale", removed_targets, Message.N_TUNNEL))

end

function TargetHandler:on_special_remove_finish(c_msg)

	local removed_targets = {}
	local total_removed = c_msg.x_param.c_list
	local goals = GameGlobals.Scene:get("level_config").reqs

	for i = 1, #total_removed do

		for color, number in pairs(goals) do

			if total_removed[i]:get_category():has(color) then

				if total_removed[i]:get_category():has("normal") 
					and not total_removed[i]:get_category():has("fenced")
					and not total_removed[i].repeatitive then

					removed_targets[#removed_targets + 1] = total_removed[i]
					total_removed[i].repeatitive = true

				end

				break

			end

		end

	end

	for i = 1, #total_removed do

		total_removed[i].repeatitive = false

	end	

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("target_finale", removed_targets, Message.N_TUNNEL))

end

function TargetHandler:on_chocolate_affect()

	local goals = GameGlobals.Scene:get("level_config").reqs

	for color, number in pairs(goals) do

		if color == "chocolate" and goals[color] then

			goals[color] = goals[color] + 1
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("goal_increase", "chocolate", Message.N_TUNNEL))	

		end

	end

end

function TargetHandler:on_blast_finale(c_msg)

	local sugar = c_msg.x_param
	local goal_color = self:get_goal_color(sugar)

	if goal_color then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_target_finale", sugar, Message.N_TUNNEL))

	end

end


TargetHandler.LISTENER_CONFIG = {

	{"remove_sugar", {before = TargetHandler.on_remove}},
	{"link_remove_start", {before = TargetHandler.on_normal_remove_finish}},
	{"special_remove_start", {before = TargetHandler.on_special_remove_finish}},
	{"skill_remove_start", {before = TargetHandler.on_normal_remove_finish}},
	{"chocolate_affect", {before = TargetHandler.on_chocolate_affect}},
	{"blast_finale",{before = TargetHandler.on_blast_finale}}

}

TargetHandler.PROPERTY_CONFIG = {
	
	{"target_list", {}}

}

return TargetHandler