-- ## Imports ## --

require "core.Section_Data"
require "UI.Layer.GameBackgroundLayer"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameGlobals = require "GameEx.GameGlobals"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"

-- ## Imports ## --


local UIComponent = class("UIComponent", function() return GameObject:create() end)


function UIComponent:ctor()

	self.__c_remove_counter = 0

	self:init_state_machine()

	self:register_listeners_with_config(UIComponent.LISTENER_CONFIG)
	self:register_properties_with_config(UIComponent.PROPERTY_CONFIG)

	self.__n_animation_count = 0
	self.__s_game_finish_flag = "not_finish"

end

function UIComponent:on_assemble()

	local visible_size = cc.Director:getInstance():getVisibleSize()
	local scene = self.__c_parent
	local cc_scene = scene.__c_scene

	local section_info = Section_Data:getInstance():get_section_info_by_id(Player:getInstance():get_player_current_section())
	local game_bgk_node = cc.CSLoader:createNode("game_bkg.csb")

	game_bgk_node:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(game_bgk_node) 
    
	game_bgk_node:getChildByName("game_bkg"):setTexture(section_info.game_bg_res)
	self.__c_bg = game_bgk_node
    cc_scene:addChild(self.__c_bg, 0);

	local back_board = cc.Sprite:create("res/images/game/game_area_dia_2.png")
	back_board:setPosition(cc.p(visible_size.width / 2, visible_size.height / 2 - 110))
	back_board:setScale(2)
	-- cc_scene:addChild(back_board, 1)

	--
	scene.__c_game_layer:addChild(back_board)

	-- local clipping_mask = cc.Sprite:create("res/images/background/game_bkg_1.jpg")
	local clipping_mask = cc.Sprite:create("res/images/game/game_area_dia_2.png")
	clipping_mask:setScale(2)
	clipping_mask:setPosition(cc.p(visible_size.width / 2, visible_size.height / 2 - 110))

	local function CaptureBoard()
		local render = cc.RenderTexture:create(visible_size.width, 1500)
        render:begin()
        scene.__c_game_layer:visit()
        render:endToLua()
        -- render:saveToFile("images/board.jpg")
        render:retain()
    
	    performWithDelay(scene.__c_game_layer,function()
	        local image = render:newImage()
	        local fileName = global_get_screenshot_path().."board.jpg"
	        image:saveToFile(fileName)
	        render:release()

	    end, 0.016)
	end
	local listener1 = cc.EventListenerCustom:create(NotificationUtil.EventType.capture_board_event, CaptureBoard)
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(listener1, cc.Director:getInstance():getNotificationNode())

	local clipping_node = cc.ClippingNode:create()
	clipping_node:setStencil(clipping_mask)
	clipping_node:setAlphaThreshold(0.1)
	clipping_node:setName("game_board_frame")

	scene.__c_game_layer:removeFromParent()
	clipping_node:addChild(scene.__c_game_layer)

	cc_scene:addChild(clipping_node, 3)

	self.__c_ui = GameBackgroundLayer:create() 
	self.__c_ui:init(self.__c_parent:get("type"))
	self.__c_ui:retain()
	cc_scene:addChild(self.__c_ui, 5)

	self.__c_ui:set_item_click_callback(function(skill_name)
		print("skill:"..skill_name)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("perform_item_skill", skill_name))

	end)

	self.__c_ui:set_item_cancel_callback(function(skill_name)
		print(skill_name)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("cancel_item_skill", skill_name))

	end)

	self.__c_ui:setGameStartFinishCallback(function()
-- ERROR_ASSERT(false, "string")
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_animation_finish", skill_name, Message.N_TUNNEL))

	end)

	self.__c_ui:set_relive_callback(function()

		self:game_relive()

	end)

	self.__c_ui:set_relive_bomb_callback(function()

		self:game_bomb_relive()
		
	end)

	self.__c_ui:set_die_bomb_callback(function()

		self:game_bomb_die()
		
	end)

end

function UIComponent:on_destroy()

	self.__c_ui:removeFromParent()
	self.__c_ui:release()

end

function UIComponent:on_disassemble()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("perform_skill", "", Message.N_TUNNEL))

end

function UIComponent:game_relive()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("relive", "", Message.N_TUNNEL))

end

function UIComponent:game_bomb_relive()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("relive_bomb", "", Message.N_TUNNEL))

end

function UIComponent:game_bomb_die()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("die_bomb", "", Message.N_TUNNEL))

end

function UIComponent:on_run()

	local level_config = self.__c_parent:get("level_config")
	self.__c_ui:setGoal(level_config)

end

function UIComponent:fully_charged()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("perform_skill", "", Message.N_TUNNEL))

end

function UIComponent:on_target_remove(c_msg)

	self.__n_animation_count = self.__n_animation_count + 1

end

function UIComponent:on_target_animation_finish(c_msg)

	if not GameGlobals.Scene:get("has_boss") then

		self.__c_ui:play_candy_collect_animation(c_msg.x_param)

	end

	self.__n_animation_count = self.__n_animation_count - 1
-- print(string.format("UI animation count: %d", self.__n_animation_count))
	
	self:go_to_end_section()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("target_collect_animation", "", Message.N_TUNNEL))

	local function empty_func()



	end

	RUN_ACTION_WITH_DELAY(0.2, empty_func)

end

function UIComponent:go_to_end_section()
-- print(self.__s_game_finish_flag, self.__c_state_machine:get_current_state())
	if self.__n_animation_count == 0 then
		-- 延迟一帧发送消息，解决boss死亡动画播放顺序出现的问题

		TRIGGER_EVENT("COLLECT_FINISH")

		GameGlobals.Scene:runAction(cc.Sequence:create(cc.DelayTime:create(0), cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("all_target_animation_finish", "", Message.N_TUNNEL)) end)))

		if self.__s_game_finish_flag == "win" then

			self.__c_state_machine:do_event("win")

		elseif self.__s_game_finish_flag == "lose" then

			self.__c_state_machine:do_event("lose")

		elseif self.__s_game_finish_flag == "lose_bomb" then

			self.__c_state_machine:do_event("lose_bomb")

		elseif self.__s_game_finish_flag == "end" then

			self.__c_state_machine:do_transition("end")

		elseif self.__s_game_finish_flag == "bonus_end" then

			self.__c_state_machine:do_event("game_over")

		end

	end

end

function UIComponent:on_rearrange_start()

	self.__c_ui:resetGame()

end

function UIComponent:on_game_win()
print("win")
	self.__s_game_finish_flag = "win"

	self:go_to_end_section()

end

function UIComponent:on_game_lose()
print("lose")
	self.__s_game_finish_flag = "lose"

	self:go_to_end_section()

end

function UIComponent:on_game_lose_bomb()
print("lose_bomb")
	self.__s_game_finish_flag = "lose_bomb"

	self:go_to_end_section()

end

function UIComponent:on_game_end()
print("end")
	self.__s_game_finish_flag = "end"

	self:go_to_end_section()

end

function UIComponent:on_bonus_end()
print("bonus_end")
	self.__s_game_finish_flag = "bonus_end"

	self:go_to_end_section()

end

function UIComponent:on_update_score(c_msg)

	self.__c_ui:update_score(c_msg.x_param)

end

function UIComponent:on_update_step(c_msg)

	self.__c_ui:setStep(c_msg.x_param)

end

function UIComponent:on_boss_dead(c_msg)

	-- self.__c_ui:set_boss_blood(0)

end

function UIComponent:on_boss_hp_change(c_msg)

	self.__c_ui:set_boss_blood(c_msg.x_param)

end

function UIComponent:on_boss_hp_offset(c_msg)

	self.__c_ui:offset_boss_blood(c_msg.x_param)

end

function UIComponent:set_boss_full_hp(n_full_hp)

	self.__c_ui:set_boss_full_hp(n_full_hp)

end

function UIComponent:on_boss_cooldown_change(c_msg)

	self.__c_ui:set_boss_step(c_msg.x_param)

end

function UIComponent:on_boss_attack_change(c_msg)

	self.__c_ui:set_boss_attack(c_msg.x_param.attack_type, c_msg.x_param.attack_number)

end

function UIComponent:on_link_remove_finish()
	-- print("on_link_remove_finish::"..self.__c_remove_counter)

	local action = GameBackgroundLayer.Role_Action.Bag

	if self.__c_remove_counter >= 3 and self.__c_remove_counter < 8 then

		action = GameBackgroundLayer.Role_Action.Star

	elseif self.__c_remove_counter >= 8 then

		action = GameBackgroundLayer.Role_Action.Shop
		
	end

	if not GameGlobals.bonus then
		self.__c_ui:play_role_action(action)
	end

	self.__c_remove_counter = 0

end

function UIComponent:on_special_remove_finish()
	-- print("on_special_remove_finish::"..self.__c_remove_counter)
	
	local action = GameBackgroundLayer.Role_Action.Bag

	if self.__c_remove_counter >= 3 and self.__c_remove_counter < 8 then

		action = GameBackgroundLayer.Role_Action.Star

	elseif self.__c_remove_counter >= 8 then

		action = GameBackgroundLayer.Role_Action.Shop

	end

	if not GameGlobals.bonus then
		self.__c_ui:play_role_action(action)
	end

	self.__c_remove_counter = 0

end

function UIComponent:on_update_skill_fill(c_msg)

	self.__c_ui:set_skill_percent(c_msg.x_param)

end

function UIComponent:on_item_used(c_msg)

	self.__c_ui:use_item(c_msg.x_param)

end

function UIComponent:on_bonus_perform()

	self.__c_ui:play_bonus_animation()
	self.__n_with_bonus = true

	GameGlobals.bonus = true

end

function UIComponent:win()

	self.__c_ui:disable_touch()
	
	local function after_animation()

		local delay = cc.DelayTime:create(0.5)
		local call = cc.CallFunc:create(function() self.__c_state_machine:do_event("bonus") end)
		local sequence = cc.Sequence:create(delay, call)

		GameGlobals.Scene:runAction(sequence)

	end

	self.__c_ui:play_battle_win_animation(after_animation)	

end

function UIComponent:lose()

	print("battle relive")
	local delay = cc.DelayTime:create(1.5)
	local call = cc.CallFunc:create(function() self.__c_ui:battle_relive() end)
	local sequence = cc.Sequence:create(delay, call)

	GameGlobals.Scene:runAction(sequence)

end

function UIComponent:init_state_machine()

	function on_enter_play()
		
		print("enter play")

	end

	function on_enter_win()

		self:win()

	end

	function on_enter_lose()
		
		self:lose()

	end

	function on_enter_lose_bomb()
		print("battle bomb relive")
		local delay = cc.DelayTime:create(1.5)
		local call = cc.CallFunc:create(function() self.__c_ui:battle_relive_bomb() end)
		local sequence = cc.Sequence:create(delay, call)

		GameGlobals.Scene:runAction(sequence)

	end

	function on_enter_end()
		print("battle end")
		local delay = cc.DelayTime:create(1.5)
		local call = cc.CallFunc:create(function() self.__c_ui:battle_lost() end)
		local sequence = cc.Sequence:create(delay, call)

		GameGlobals.Scene:runAction(sequence)

	end

	function on_enter_bonus()
-- ERROR_ASSERT(false, "string")
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("trigger_bonus"))

	end

	function on_enter_game_over_win()
		
		local delay_time = 0

		if self.__n_with_bonus then

			delay_time = 1.5

		end

		local delay = cc.DelayTime:create(delay_time)
		local call = cc.CallFunc:create(function() self.__c_ui:win_jiesuan() end)
		local sequence = cc.Sequence:create(delay, call)

		GameGlobals.Scene:runAction(sequence)

	end

	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "play", enter = on_enter_play},
		{name = "win", enter = on_enter_win},
		{name = "lose", enter = on_enter_lose},
		{name = "lose_bomb", enter = on_enter_lose_bomb},
		{name = "end", enter = on_enter_end},
		{name = "bonus", enter = on_enter_bonus},
		{name = "game_over_win", enter = on_enter_game_over_win},

	})
	self.__c_state_machine:add_transition({

		{event = "relive", from = "lose", to = "play"},
		{event = "win", from = "play", to = "win"},
		{event = "win", from = "lose", to = "win"},
		{event = "lose", from = "play", to = "lose"},
		{event = "lose_bomb", from = "play", to = "lose_bomb"},
		{event = "bonus", from = "win", to = "bonus"},
		{event = "game_over", from = "bonus", to = "game_over_win"},
		{event = "game_over", from = "lose", to = "game_over_lose"},

	})

	self.__c_state_machine:do_transition("play")

end

function UIComponent:on_disable_item()
print("disable item !!!!@@@@@@@s")
	self.__c_ui:disable_touch()

end

function UIComponent:on_enable_item()
print("enable item !!!!@@@@@@@s")
	self.__c_ui:enable_touch()

end

function UIComponent:on_relive()

	self.__c_state_machine:do_transition("play")
	self.__s_game_finish_flag = "not_finish"

end

function UIComponent:on_skill_add_move()

	self.__c_ui:play_skill_add_step_indication()

end

function UIComponent:on_item_add_move()

	self.__c_ui:play_item_add_step_indication()

end

function UIComponent:on_score_animation(c_msg)

	self.__c_ui:play_score_animation(c_msg.x_param.score, c_msg.x_param.position)

end

function UIComponent:on_goal_increase(c_msg)

	self.__c_ui:increase_goal(c_msg.x_param)

end

function UIComponent:on_start_item_tutorial(c_msg)

	self.__c_ui:set_item_tutorial(true)

end

function UIComponent:on_finish_item_tutorial(c_msg)

	self.__c_ui:set_item_tutorial(false)

end

function UIComponent:on_cancel_item_passive()
-- ERROR_ASSERT(false, "string")
	self.__c_ui:cancel_item_passive()

end

function UIComponent:on_sugar_remove(c_msg)

	local sugar = c_msg.x_param
	local category = sugar:get_category()

	if category:has("normal") or category:has("timing_bomb") or category:has("random") then

		self.__c_remove_counter = self.__c_remove_counter + 1
		
	end

end

UIComponent.LISTENER_CONFIG = {

	{"assemble", {before = UIComponent.on_assemble}},
	{"remove_sugar", {before = UIComponent.on_sugar_remove}},
	{"link_remove_finish", {before = UIComponent.on_link_finish}},
	{"target",{before = UIComponent.on_target_remove}},
	{"sugar_target_animation_finish",{before = UIComponent.on_target_animation_finish}},
	{"run", {before = UIComponent.on_run}},
	{"rearrange_start", {before = UIComponent.on_rearrange_start}},
	{"game_win", {before = UIComponent.on_game_win}},
	{"game_lose", {before = UIComponent.on_game_lose}},
	{"game_lose_bomb", {before = UIComponent.on_game_lose_bomb}},
	{"game_end", {before = UIComponent.on_game_end}},
	{"update_score", {before = UIComponent.on_update_score}},
	{"bonus_clear", {before = UIComponent.on_bonus_end}},
	{"move_change", {before = UIComponent.on_update_step}},
	{"boss_hp_change", {before = UIComponent.on_boss_hp_change}},
	{"boss_hp_offset", {before = UIComponent.on_boss_hp_offset}},
	{"boss_cooldown_change", {before = UIComponent.on_boss_cooldown_change}},
	{"boss_attack_change", {before = UIComponent.on_boss_attack_change}},
	{"link_remove_finish", {before = UIComponent.on_link_remove_finish}},
	{"special_remove_finish", {before = UIComponent.on_special_remove_finish}},
	{"update_skill_fill", {before = UIComponent.on_update_skill_fill}},
	{"item_used", {before = UIComponent.on_item_used}},
	{"disable_item", {before = UIComponent.on_disable_item}},
	{"enable_item", {before = UIComponent.on_enable_item}},
	{"relive", {before = UIComponent.on_relive}},
	{"relive_bomb", {before = UIComponent.on_relive}},
	{"skill_add_move", {before = UIComponent.on_skill_add_move}},
	{"item_add_move", {before = UIComponent.on_item_add_move}},
	{"bonus_perform", {before = UIComponent.on_bonus_perform}},
	{"score_animation", {before = UIComponent.on_score_animation}},
	{"goal_increase", {before = UIComponent.on_goal_increase}},
	{"destroy", {before = UIComponent.on_destroy}},
	{"boss_dead", {before = UIComponent.on_boss_dead}},
	{"start_item_tutorial", {before = UIComponent.on_start_item_tutorial}},
	{"finish_item_tutorial", {before = UIComponent.on_finish_item_tutorial}},
	{"cancel_item_passive", {before = UIComponent.on_cancel_item_passive}},

}

UIComponent.PROPERTY_CONFIG = {
	
	{"scene", 0}

}

return UIComponent