-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local BoxSelector = class("BoxSelector", function() return GameObject:create() end)

function BoxSelector:ctor()

	self.__c_select_box = cc.PhysicsShapeBox:create(cc.size(1, 1))
	self.__c_select_box:retain()
	
	self:register_listeners_with_config(BoxSelector.LISTENER_CONFIG)
	self:register_properties_with_config(BoxSelector.PROPERTY_CONFIG)

end

function BoxSelector:on_destroy()

	self.__c_select_box:release()

end


function BoxSelector:on_select(c_msg)

	local filter = self:get("filter")
	local size = self:get("select_size")
	local rotation = self:get("select_rotation")
	local position = self:get("position")

	local param = {

		c_shape = self.__c_select_box,
		t_p = position,
		n_rotation = rotation,
		n_scale_x = size.width,
		n_scale_y = size.height,
		t_category_filter = filter.t_category_filter,
		t_property_filter = filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_shape"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	if not c_msg.x_param or not c_msg.x_param.ban_message then

		for i = 1, #msg.x_result do

			msg.x_result[i]:send_message(MessagePool:get_free_ex("select"))

		end

	end

	c_msg.x_result = msg.x_result

end

BoxSelector.LISTENER_CONFIG = {

	{"select", {before = BoxSelector.on_select}},
	{"destroy", {before = BoxSelector.on_destroy}},

}

BoxSelector.PROPERTY_CONFIG = {
	
	{"select_rotation", 0},
	{"select_size", cc.size(1, 1)},
	{"position", cc.p(0, 0)},
	{"filter", {}},

}

return BoxSelector