-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local CircleSelector = class("CircleSelector", function() return GameObject:create() end)

function CircleSelector:ctor()

	self.__c_select_circle = cc.PhysicsShapeCircle:create(1)
	self.__c_select_circle:retain()

	self:register_listeners_with_config(CircleSelector.LISTENER_CONFIG)
	self:register_properties_with_config(CircleSelector.PROPERTY_CONFIG)

end

function CircleSelector:on_destroy()

	self.__c_select_circle:release()

end


function CircleSelector:on_select(c_msg)

	local filter = self:get("filter")

	local param = {

		c_shape = self.__c_select_circle,
		t_p = c_msg.x_param.position or cc.p(self.__c_parent:getPosition()),
		n_rotation = 0,
		n_scale_x = self:get("select_radius"),
		n_scale_y = self:get("select_radius"),
		t_category_filter = filter.t_category_filter,
		t_property_filter = filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_shape"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	if not c_msg.x_param or not c_msg.x_param.ban_message then

		for i = 1, #msg.x_result do

			msg.x_result[i]:send_message(MessagePool:get_free_ex("select"))

		end

	end

	c_msg.x_result = msg.x_result

end

CircleSelector.LISTENER_CONFIG = {

	{"select", {before = CircleSelector.on_select}},
	{"destroy", {before = CircleSelector.on_destroy}},

}

CircleSelector.PROPERTY_CONFIG = {
	
	{"select_radius", 120},
	{"filter", {}}

}

return CircleSelector