-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local ColorSelector = class("ColorSelector", function() return GameObject:create() end)

function ColorSelector:ctor()

	self.__c_select_circle = cc.PhysicsShapeCircle:create(1)
	self.__c_select_circle:retain()

	self:register_listeners_with_config(ColorSelector.LISTENER_CONFIG)
	self:register_properties_with_config(ColorSelector.PROPERTY_CONFIG)

	self.__t_filter = {}
	self:build_filter()

end

function ColorSelector:on_destroy()

	self.__c_select_circle:release()

end


function ColorSelector:on_select(c_msg)

	self:build_filter()

	local param = {

		t_category_filter = self.__t_filter.t_category_filter,
		t_property_filter = self.__t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	if not c_msg.x_param or not c_msg.x_param.ban_message then

		for i = 1, #msg.x_result do

			msg.x_result[i]:send_message(MessagePool:get_free_ex("select"))

		end

	end

	c_msg.x_result = msg.x_result

end

function ColorSelector:build_filter()

	self.__t_filter = {}
	self.__t_filter.t_category_filter = {
		type = "or",
		{"normal", "=", true},
		{"random", "=", true},
		{"timing_bomb", "=", true},	
	}
	self.__t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
		{"color", "=", self:get("select_color")}	
	}

end

ColorSelector.LISTENER_CONFIG = {

	{"select", {before = ColorSelector.on_select}},
	{"destroy", {before = ColorSelector.on_destroy}},

}

ColorSelector.PROPERTY_CONFIG = {
	
	{"select_color", "red"}

}

return ColorSelector