-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local KeySelector = class("KeySelector", function() return GameObject:create() end)

function KeySelector:ctor()

	self:register_listeners_with_config(KeySelector.LISTENER_CONFIG)
	self:register_properties_with_config(KeySelector.PROPERTY_CONFIG)

	self:build_filter()

end


function KeySelector:on_select(c_msg)

	local filter = self:get("filter")

	local param = {

		t_category_filter = self.__t_filter.t_category_filter,
		t_property_filter = self.__t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	for i = 1, #msg.x_result do

		msg.x_result[i]:send_message(MessagePool:get_free_ex("select"))

	end

	c_msg.x_result = msg.x_result

end

function KeySelector:build_filter()

	self.__t_filter = {}
	self.__t_filter.t_category_filter = {
		type = "or",
		{"normal", "=", true},
		{"special", "=", true},	
		{"fracture", "=", true},		
	}
	self.__t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},	
	}

end

KeySelector.LISTENER_CONFIG = {

	{"select", {before = KeySelector.on_select}},
	{"destroy", {before = KeySelector.on_destroy}},

}

KeySelector.PROPERTY_CONFIG = {
	

}

return KeySelector