-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local TouchComponent = require "GameEx.Components.InputHandlers.TouchComponent"
local Message = require "GameEx.Foundations.Message"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local LinkSelector = class("LinkSelector", function() return GameObject:create() end)


function LinkSelector:ctor()

	self.__c_select_circle = cc.PhysicsShapeCircle:create(1)
	self.__c_select_circle:retain()

	self.__c_select_box = cc.PhysicsShapeBox:create(cc.size(1, 1))
	self.__c_select_box:retain()

	self.__t_last_position = cc.p(0, 0)
	self.__t_current_position = cc.p(0, 0)

	self.__t_select_result = {}
	self.__n_select_result_size = 0

	self.__t_filter = self:build_begin_filter()

	self:add_component(ObjectFactory:get_instance():get("TouchComponent"), "touch_handler")

	self:register_listeners_with_config(LinkSelector.LISTENER_CONFIG)
	self:register_properties_with_config(LinkSelector.PROPERTY_CONFIG)

end

function LinkSelector:on_destroy()

	self.__c_select_box:release()
	self.__c_select_circle:release()

	for i = 1, self.__n_select_result_size do

		self:unselect(self.__t_select_result[i])

	end

	self:stop_linked_check()	

end

function LinkSelector:on_touch_down(c_msg)

	self:begin(c_msg.x_param)

end

function LinkSelector:on_touch_up(c_msg)

	if not self:check_sequence_finish() then

		self:cancel()
		return

	end

	self:finish()

end

function LinkSelector:on_touch_move(c_msg)

	self:continue(c_msg.x_param)

end

function LinkSelector:on_touch_cancel(c_msg)

	self:cancel()

end

function LinkSelector:begin(t_position)

	local param = {

		t_p = t_position,

	}

	local msg = MessagePool:get_free()
	msg.s_name = "query_point"
	msg.x_param = param
	GameGlobals.Scene:send_message(msg)

	self:apply_result(msg.x_result)
	self.__t_start_position = t_position
	self.__t_current_position = t_position

	self:start_linked_check()

end

function LinkSelector:continue(t_position)

	self.__t_current_position = t_position

	local position = {x = (self.__t_current_position.x + self.__t_start_position.x) / 2, y = (self.__t_current_position.y + self.__t_start_position.y) / 2}
	local direction = UtilFunction:caculate_direction(self.__t_current_position, self.__t_start_position)
	local rotation = UtilFunction:caculate_angle(direction)
	local length = UtilFunction:caculate_distance(self.__t_current_position, self.__t_start_position)

	if length < 1 then 

		return 

	end

	local width = 20

	local param = {

		c_shape = self.__c_select_box,
		t_p = position,
		n_rotation = rotation,
		n_scale_x = length,
		n_scale_y = width,
		t_p1 = self.__t_start_position,
		t_p2 = self.__t_current_position,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_shape"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	self:apply_result(self:sort_result(msg.x_result))

	if self:cancel_condition_reached(self.__t_current_position) then

		self:cancel()

	end

	self.__t_start_position = self.__t_current_position

end

function LinkSelector:finish()

	local return_result = self.__t_select_result
	return_result.type = self.__t_filter.type

	for i = 1, self.__n_select_result_size do

		self.__t_select_result[i]:release()

	end

	self.__t_select_result = {}
	self.__n_select_result_size = 0
	self.__t_filter = self:build_begin_filter()

	self.__c_parent:send_message(MessagePool:get_free_ex("select_finish", return_result, Message.N_TUNNEL))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))

	self:stop_linked_check()

end

function LinkSelector:cancel()

	local return_result = self.__t_select_result

	for i = 1, self.__n_select_result_size do

		self:unselect(self.__t_select_result[i])

	end

	self.__t_select_result = {}
	self.__n_select_result_size = 0
	self.__t_filter = self:build_begin_filter()

	self.__c_parent:send_message(MessagePool:get_free_ex("select_cancel", return_result, Message.N_TUNNEL))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))

	self:send_message(MessagePool:get_free_ex("stop_touch", "", Message.N_TUNNEL))

	self:stop_linked_check()

end

function LinkSelector:sort_result(t_query_result)

	local direction = UtilFunction:caculate_direction(self.__t_current_position, self.__t_start_position)

	for i = 1, #t_query_result do

		local position = cc.p(t_query_result[i]:getPosition())
		position.x = position.x - self.__t_start_position.x
		position.y = position.y - self.__t_start_position.y

		local projection = UtilFunction:caculate_projection(position, direction)

		t_query_result[i].projection = projection

	end

	for beacon = 1, #t_query_result do

		local min_pos = beacon

		for i = beacon, #t_query_result do
			
			if t_query_result[i].projection < t_query_result[min_pos].projection then

				min_pos = i

			end

		end

		local temp = t_query_result[beacon]
		t_query_result[beacon] = t_query_result[min_pos]
		t_query_result[min_pos] = temp

	end

	return t_query_result

end

function LinkSelector:apply_result(t_query_result)

	local original_size = self.__n_select_result_size

	local result_size = #t_query_result

	for i = 1, result_size do
		
		local sugar = t_query_result[i]

		if sugar:property_filter(self.__t_filter.t_property_filter) and sugar:category_filter(self.__t_filter.t_category_filter) then

			if sugar:get("state") == "select" then

				self:remove_top(sugar)

			elseif sugar:get("state") == "normal" then

				self:apply_rear(sugar)

			end

		end

		self:change_filter()

	end

	if not (original_size == self.__n_select_result_size) then

		self.__t_select_result.type = self.__t_filter.type
		self.__c_parent:send_message(MessagePool:get_free_ex("select_continue", self.__t_select_result, Message.N_TUNNEL))
		self.__n_select_changed = true

	end

end

function LinkSelector:change_filter()

	if self.__t_filter.type == "empty" then

		if self.__n_select_result_size == 1 then

			local rear_sugar = self.__t_select_result[1]
			local category = rear_sugar:get_category()

			if not category:has("special") then

				self.__t_filter = self:build_normal_filter(rear_sugar)

			else

				self.__t_filter = self:build_special_filter()

			end

		end

	elseif self.__t_filter.type == "special" then

		if self.__n_select_result_size == 2 then

			self.__t_filter = self:build_special_link_filter()

		end

	elseif self.__t_filter.type == "special_link" then

		if self.__n_select_result_size == 1 then

			self.__t_filter = self:build_special_filter()

		end

	end

end

function LinkSelector:apply_rear(c_sugar)

	if self:in_select_range(c_sugar) and self:check_sequence(c_sugar) then

		self:select(c_sugar)

		self.__t_select_result[self.__n_select_result_size + 1] = c_sugar
		self.__n_select_result_size = self.__n_select_result_size + 1

		local index = self.__n_select_result_size

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("link_selected", index, Message.N_TUNNEL))

	end

end

function LinkSelector:check_sequence(c_sugar)

	if self:get("mode") == "fixed" then

		local seq = self:get("sequence")

		local next_index = self.__n_select_result_size + 1

		if c_sugar:get("id") == seq[next_index] then

			return true

		else

			return false

		end

	else

		return true

	end

end

function LinkSelector:check_sequence_finish()

	if (self:get("mode") == "fixed" and #self:get("sequence") == self.__n_select_result_size) or
		self:get("mode") == "arbitary" then

		return true

	else

		return false

	end

end

function LinkSelector:remove_top(c_sugar)

	local current_rear = self.__t_select_result[self.__n_select_result_size - 1]

	if current_rear == c_sugar then

		self:unselect(self.__t_select_result[self.__n_select_result_size])

		self.__t_select_result[self.__n_select_result_size] = nil
		self.__n_select_result_size = self.__n_select_result_size - 1

	end

end

function LinkSelector:cancel_condition_reached(t_position)

	if self.__n_select_result_size == 0 then

		return false

	else

		local rear_sugar = self.__t_select_result[self.__n_select_result_size]
		local rear_position = cc.p(rear_sugar:getPosition())

		if UtilFunction:caculate_distance(t_position, rear_position) > self:get("cancel_threshold") then

			return true

		else

			return false

		end

	end

end


function LinkSelector:select(c_sugar)

	c_sugar:retain()

	c_sugar:send_message(MessagePool:get_free_ex("select"))

end

function LinkSelector:unselect(c_sugar)

	c_sugar:release()

	c_sugar:send_message(MessagePool:get_free_ex("unselect"))

end

function LinkSelector:in_select_range(c_sugar)

	if self.__n_select_result_size == 0 then

		return true

	else

		local search_radius = c_sugar:get("radius") + self:get("select_range")
		local rear_sugar = self.__t_select_result[self.__n_select_result_size]

		local param = {

			c_shape = self.__c_select_circle,
			t_p = cc.p(rear_sugar:getPosition()),
			n_rotation = 0,
			n_scale_x = search_radius,
			n_scale_y = search_radius,

		}

		local msg = MessagePool:get_free()
		msg.c_sender = self
		msg.s_name = "query_shape"
		msg.x_param = param

		GameGlobals.Scene:send_message(msg)

		local query_result = msg.x_result

		for i = 1, #query_result do

			if query_result[i] == c_sugar then

				return true

			end

		end

		return false		

	end

end

function LinkSelector:build_begin_filter()

	local filter = {}

	filter.type = "empty"

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"}
	}

	filter.t_category_filter = {
		type = "or",
		{"normal", "=", true},
		{"special", "=", true},
		{"random", "=", true},
		{"timing_bomb", "=", true},
	}

	return filter

end

function LinkSelector:build_normal_filter(c_sugar)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_defined", c_sugar:get("color"), Message.N_TUNNEL))

	local filter = {}

	filter.type = "normal"

	filter.t_property_filter = {
		type = "and",
		{"state", "~=", "remove"},
		{"state", "~=", "ignite"},
		{"color", "=", c_sugar:get("color")},
	}

	filter.t_category_filter = {
		type = "or",
		{"normal", "=", true},
		{"random", "=", true},
		{"timing_bomb", "=", true},
		{"colorful", "=", true},
	}

	return filter

end

function LinkSelector:build_special_filter(c_sugar)

	local filter = {}

	filter.type = "special"

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	filter.t_category_filter = {
		type = "and",
		{"special", "=", true},
	}

	return filter

end

function LinkSelector:build_special_link_filter(c_sugar)

	local filter = {}

	filter.type = "special_link"

	filter.t_property_filter = {
		type = "and",
		{"state", "=", "select"},
	}

	filter.t_category_filter = {
		type = "and",
		{"special", "=", true},
	}

	return filter

end

function LinkSelector:check_linked()
-- print("do link check")
	for i = 1, #self.__t_select_result - 1 do

		local sugar_1 = self.__t_select_result[i]
		local sugar_2 = self.__t_select_result[i + 1]

		local distance = UtilFunction:caculate_distance(cc.p(sugar_1:getPosition()), cc.p(sugar_2:getPosition()))

		if distance - sugar_1:get("radius") * 2 > self:get("select_range") then

			self:cancel()

			break

		end

	end

end

function LinkSelector:start_linked_check()
print("start link check")

	self:stop_linked_check()

	self.__n_check_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:check_linked() end, 0.1, false)

end

function LinkSelector:stop_linked_check()

	if self.__n_check_entry then
print("stop link check")
		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_check_entry)
		self.__n_check_entry = nil

	end

end

LinkSelector.LISTENER_CONFIG = {

	{"touch_down", {before = LinkSelector.on_touch_down}},
	{"touch_up", {before = LinkSelector.on_touch_up}},
	{"touch_move", {before = LinkSelector.on_touch_move}},
	{"touch_cancel", {before = LinkSelector.on_touch_cancel}},
	{"destroy", {before = LinkSelector.on_destroy}},

}

LinkSelector.PROPERTY_CONFIG = {
	
	{"select_range", 40},
	{"cancel_threshold", 800},
	{"enable", true},
	{"mode", "arbitary"},
	{"sequence", {}},

}


return LinkSelector