-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --


local AhDaiSkill = class("AhDaiSkill", function() return GameObject:create() end)


function AhDaiSkill:ctor()

    self.__n_current_remove_count = 0

	self:register_listeners_with_config(AhDaiSkill.LISTENER_CONFIG)
	self:register_properties_with_config(AhDaiSkill.PROPERTY_CONFIG)

end

function AhDaiSkill:on_skill_start(c_msg)

    if self.__n_current_remove_count >= self:get("perform_threshold") then

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("player_skill_cast", "", Message.N_TUNNEL))

        local t_filter = {}

        t_filter.t_category_filter = {
            type = "and",
            {"normal", "=", true},
            {"fenced", "=", false},
            {"add_move", "=", false},   
        }
        t_filter.t_property_filter = {
            type = "and",
            {"state", "=", "normal"},
        }

        local param = {

            t_category_filter = t_filter.t_category_filter,
            t_property_filter = t_filter.t_property_filter,

        }

        local msg = MessagePool:get_free()
        msg.c_sender = self
        msg.s_name = "query_filter"
        msg.x_param = param

        GameGlobals.Scene:send_message(msg)

        if #msg.x_result > 0 then

            local visible_size = cc.Director:getInstance():getVisibleSize()

            self.__n_current_remove_count = 0

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("skill_add_move", self:get("change_number"), Message.N_TUNNEL))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

            local position1 = cc.p(visible_size.width / 2 - 420, 1700)
            local position2 = cc.p(visible_size.width / 2 - 370, 1550)
            local light_particle = ResourceManager:get_instance():get_resource_by_key("skill_flying_particle")
            light_particle:setPosition(position1)
            light_particle:setLocalZOrder(99)
            light_particle:setDuration(-1)
            light_particle:setAutoRemoveOnFinish(false)
            light_particle:setVisible(false)
            light_particle:runAction(cc.Sequence:create(cc.MoveTo:create(29 * 0.033, cc.p(position2.x, position2.y)), cc.CallFunc:create(function()

                    local light2_particle = ResourceManager:get_instance():get_resource_by_key("bonus_remove_particle")
                    light2_particle:setAutoRemoveOnFinish(true)
                    light2_particle:setPosition(cc.p(visible_size.width / 2 - 360, 1580))
            
                    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", light2_particle))

                    GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", self:get("change_number")))

                    light_particle:setDuration(0.1)
                    light_particle:setAutoRemoveOnFinish(true)

                    GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

            end)))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", light_particle))  

        end         

    end

end

function AhDaiSkill:on_sugar_remove(c_msg)

    local category = c_msg.x_param:get_category()

    if category:has("normal") or category:has("colorful") or category:has("random") or category:has("timing_bomb")  then

        self.__n_current_remove_count = self.__n_current_remove_count + 1

    end

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))
-- print(string.format("skill: %d [count]", self.__n_current_remove_count))
end

AhDaiSkill.LISTENER_CONFIG = {

	{"player_skill_start", {before = AhDaiSkill.on_skill_start}},
    {"remove_sugar", {before = AhDaiSkill.on_sugar_remove}},

}

AhDaiSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"perform_threshold", 30},
    {"change_number", 1},

}


return AhDaiSkill