-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --




local BombSkill = class("BombSkill", function() return GameObject:create() end)


function BombSkill:ctor()

    self:add_component(ObjectFactory:get_instance():get("SkillTouchHandler"), "touch_handler")
    self:add_component(ObjectFactory:get_instance():get("CircleSelector"), "selector")
    self:add_component(ObjectFactory:get_instance():get("BombRemover"), "remover")

	self:register_listeners_with_config(BombSkill.LISTENER_CONFIG)
	self:register_properties_with_config(BombSkill.PROPERTY_CONFIG)

    self:set("select_radius", 160)

    local filter = {}

    filter.t_property_filter = {
        type = "and",
        {"state", "=", "normal"},
    }

    filter.t_category_filter = {
        type = "or",
        {"special", "=", true},
        {"normal", "=", true},
        {"fracture", "=", true},
        {"fenced", "=", true},
        {"panda", "=", true},
        {"random", "=", true},
        {"timing_bomb", "=", true},
        {"colorful", "=", true},
    }

    self:get_component("selector"):set("filter", filter)

end

function BombSkill:on_perform_skill(c_msg)

	self:set("enable", true)

end

function BombSkill:on_touch_down(c_msg)

    local position = c_msg.x_param

    if position.x < 40 or position.x > 1000 or position.y < 220 or position.y > 1470 then

        return

    end 

    TRIGGER_EVENT("ITEM_SKILL_START")

    self:set("enable", false)

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_used", "BombSkill", Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_remove_start"))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", "", Message.N_TUNNEL))

    local position = c_msg.x_param
    self:setPosition(position)

    local msg = MessagePool:get_free()
    msg.s_name = "select"
    msg.x_param = {ban_message = true}

    self:get_component("selector"):send_message(msg)

    local select_result = msg.x_result

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_start", {c_list = select_result, t_pos = position}, Message.N_TUNNEL))

    local msg = MessagePool:get_free()
    msg.s_name = "assign_remove_task"
    msg.x_param = select_result

    self:get_component("remover"):send_message(msg)

    local world_position = position
    local scale_modify = 1.5

    local particle_1 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_1")
    particle_1:setLocalZOrder(10)
    particle_1:setPosition(world_position)
    particle_1:setAutoRemoveOnFinish(true)
    particle_1:setScale(scale_modify)
    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

    local particle_2 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_2")
    particle_2:setLocalZOrder(10)
    particle_2:setPosition(world_position)
    particle_2:setAutoRemoveOnFinish(true)
    -- particle_2:setScale(scale_modify)
    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

    local particle_3 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_3")
    particle_3:setLocalZOrder(10)
    particle_3:setPosition(world_position)
    particle_3:setAutoRemoveOnFinish(true)
    particle_3:setScale(scale_modify)
    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_finish", {c_list = select_result, t_pos = position}, Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("bomb_skill_hit", "", Message.N_TUNNEL))

    local event = cc.EventCustom:new("item_consume")
    event.param = {}   
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function BombSkill:on_cancel(c_msg)

    self:set("enable", false)

end

BombSkill.LISTENER_CONFIG = {

	{"perform", {before = BombSkill.on_perform_skill}},
    {"touch_down", {before = BombSkill.on_touch_down}},
    {"cancel", {before = BombSkill.on_cancel}},

}

BombSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"enable", false}

}


return BombSkill