-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --




local BossSkill = class("BossSkill", function() return GameObject:create() end)


function BossSkill:ctor()

	self:register_listeners_with_config(BossSkill.LISTENER_CONFIG)
	self:register_properties_with_config(BossSkill.PROPERTY_CONFIG)

end

function BossSkill:on_perform_skill(c_msg)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", task, Message.N_TUNNEL))

	local number = c_msg.x_param.number
	local config = c_msg.x_param.config

	local t_filter = {}

	t_filter.t_category_filter = {
		type = "and",
		{"normal", "=", true},
		{"fenced", "=", false},  	
	}
	t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local count = 0

	if #msg.x_result > 0 then

		number = number > #msg.x_result and #msg.x_result or number

		for i = 1, number do 

			local target = msg.x_result[i]

			target:send_message(MessagePool:get_free_ex("lock"))

			local visible_size = cc.Director:getInstance():getVisibleSize()

			local homing_entry = 0
			local homing_particle = ResourceManager:get_instance():get_resource_by_key("boss_attack_particle")
			homing_particle:setDuration(-1)
			homing_particle:setPosition(cc.p(self.__c_parent:getPosition()))
			homing_particle:setLocalZOrder(11)
			homing_particle:setAutoRemoveOnFinish(true)
			homing_particle:setSpeed(200)
			homing_particle:setPosition(cc.p(visible_size.width / 2 - 140 + 15, 1500))
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

			local homing_performer = function()

				local target_position = target:getParent():convertToWorldSpace(cc.p(target:getPosition()))
				local particle_position = cc.p(homing_particle:getPosition())

				local distance = UtilFunction:caculate_distance(target_position, particle_position)

				if distance < self:get("homing_speed") then

					local config = self:get("attack_config")

					local new_sugar = ObjectFactory:get_instance():get(config.type)

					for key, value in pairs(config.properties) do

						new_sugar:set(key, value)

					end

					new_sugar:setPosition(target_position)

			        local particle = ResourceManager:get_instance():get_resource_by_key("boss_remove_particle")
			        particle:setAutoRemoveOnFinish(true)
			        particle:setPosition(target_position)
			        
			        GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))					

					GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar_silent", target, Message.N_TUNNEL))
					GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", new_sugar, Message.N_TUNNEL))

					homing_particle:setSpeed(0)
					homing_particle:setDuration(0)

					cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

					count = count + 1

					if count == number then

						self.__c_parent:send_message(MessagePool:get_free_ex("boss_skill_finish"))
						GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

					end

				else

					local speed = self:get("homing_speed")
					local direction = UtilFunction:caculate_direction(target_position, particle_position)
					local angle = UtilFunction:caculate_angle(direction)
					local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
					particle_position.x = particle_position.x + offset.x
					particle_position.y = particle_position.y + offset.y

					homing_particle:setAngle(-angle)
					homing_particle:setPosition(particle_position)

				end

			end

			homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

		end

	end

end


BossSkill.LISTENER_CONFIG = {

	{"perform", {before = BossSkill.on_perform_skill}},

}

BossSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
	{"attack_config", { type = "IceSugar", properties = {}}}

}


return BossSkill