-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --


local ConchSkill = class("ConchSkill", function() return GameObject:create() end)


function ConchSkill:ctor()

    self.__n_current_fill = 0

    local visual = ObjectFactory:get_instance():get("ConchSkillVisual")
    self:add_component(visual, "visual")

    local selector = ObjectFactory:get_instance():get("TotalSelector")
    self:add_component(selector, "selector")

	self:register_listeners_with_config(ConchSkill.LISTENER_CONFIG)
	self:register_properties_with_config(ConchSkill.PROPERTY_CONFIG)

end

function ConchSkill:on_reset()

    self.__n_current_fill = 0
    self:send_message(MessagePool:get_free_ex("update_fill", 0, Message.N_TUNNEL))

end

function ConchSkill:on_skill_start(c_msg)

    if self.__n_current_fill == self:get("conch_skill_volume") then

        self.__n_current_fill = 0
        self:send_message(MessagePool:get_free_ex("update_fill", 0, Message.N_TUNNEL))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_remove_start", "", Message.N_TUNNEL))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", "", Message.N_TUNNEL))

        local spine = ResourceManager:get_instance():get_resource_by_key("total_remove_spine")
        spine:setAnimation(0, "powerful", false)
        spine:setLocalZOrder(11)
        spine:setPosition(cc.p(540, 1000))
        spine:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
        GameGlobals.Scene.__c_scene:addChild(spine)

        local delay_1 = cc.DelayTime:create(1)
        local remove_call = cc.CallFunc:create(function()

            local msg = MessagePool:get_free()
            msg.s_name = "select"
            msg.x_param = {}
            msg.x_param.position = cc.p(self:getPosition())
            msg.x_param.ban_message = true

            self:get_component("selector"):send_message(msg)

            local select_result = msg.x_result

            for i = 1, #select_result do

                select_result[i]:send_message(MessagePool:get_free_ex("lock", "", Message.N_TUNNEL))

            end

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_start", {c_list = select_result, t_pos = cc.p(540, 1000)}, Message.N_TUNNEL))

            local removed = {}

            for i = 1, #select_result do

                local sugar = select_result[i]

                if sugar:get("state") == "normal" or "locked" then

                    removed[#removed + 1] = sugar

                end

                sugar:send_message(MessagePool:get_free_ex("start_remove", "", Message.N_TUNNEL))

            end

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_finish", {c_list = removed, t_pos = cc.p(cc.p(540, 1000))}, Message.N_TUNNEL))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

        end)
        local delay_2 = cc.DelayTime:create(1)
        local remove_self = cc.RemoveSelf:create()
        local sequence = cc.Sequence:create(delay_1, remove_call, delay_2, remove_self)
        spine:runAction(sequence)

    end

end

function ConchSkill:on_sugar_remove(c_msg)

    if self.__n_current_fill == self:get("conch_skill_volume") then

        return

    end

    local category = c_msg.x_param:get_category()

    if category:has("conch") then

        self.__n_current_fill = self.__n_current_fill + 1
        self:send_message(MessagePool:get_free_ex("update_fill", self.__n_current_fill / self:get("conch_skill_volume"), Message.N_TUNNEL))

    end   

end

ConchSkill.LISTENER_CONFIG = {

	{"conch_skill_start", {before = ConchSkill.on_skill_start}},
    {"remove_sugar", {before = ConchSkill.on_sugar_remove}},
    {"reset", {before = ConchSkill.on_reset}},

}

ConchSkill.PROPERTY_CONFIG = {
	
	{"conch_skill_volume", 2},

}


return ConchSkill