-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManagerEx = require "GameEx.Foundations.ResourceManager"

local GameItemLayer = require "UI.Layer.GameItemLayer"


-- ## Imports ## --


local GameStartSkill = class("GameStartSkill", function() return GameObject:create() end)


function GameStartSkill:ctor()

    self.__n_current_remove_count = 0

	self:register_listeners_with_config(GameStartSkill.LISTENER_CONFIG)
	self:register_properties_with_config(GameStartSkill.PROPERTY_CONFIG)

end

function GameStartSkill:perform_move()

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

    local perform = function()

        local visible_size = cc.Director:getInstance():getVisibleSize()
        local target_position = cc.p(visible_size.width / 2 - 370, 1550)

        local item_sprite = ResourceManager:getInstance():getSpriteByKey("BEFORE_GAME_MOVE")
        item_sprite:setPosition(cc.p(540, 1280))
        item_sprite:setOpacity(0)
        item_sprite:setScale(0.3)
        GameGlobals.Scene.__c_scene:addChild(item_sprite, 100)

        local scale = cc.ScaleTo:create(0.4, 1.0)
        local ease = cc.EaseSineIn:create(scale)
        local fade = cc.FadeTo:create(0.4, 255)
        local spawn = cc.Spawn:create(ease, fade)
        item_sprite:runAction(spawn)

        local effect_spine = ResourceManagerEx:get_instance():get_resource_by_key("before_game_effect")
        effect_spine:setAnimation(0, "zhanqiandaoju", false)
        effect_spine:setPosition(cc.p(560, 1280))
        GameGlobals.Scene.__c_scene:addChild(effect_spine, 99)

        local delay = cc.DelayTime:create(0.5)
        local remove = cc.RemoveSelf:create()
        local sequence = cc.Sequence:create(delay, remove)
        effect_spine:runAction(sequence)


        local move = cc.MoveTo:create(0.4, target_position)
        local ease = cc.EaseSineInOut:create(move)
        local call = cc.CallFunc:create(function()

            local particle = ResourceManagerEx:get_instance():get_resource_by_key("bonus_remove_particle")
            particle:setLocalZOrder(99)
            particle:setPosition(target_position)
            particle:setAutoRemoveOnFinish(true)

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", 2))

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

        end)

        item_sprite:runAction(cc.Sequence:create(cc.DelayTime:create(0.4), ease, call, cc.RemoveSelf:create()))

    end

    local call = cc.CallFunc:create(perform)
    local delay = cc.DelayTime:create(0.9)
    local seq = cc.Sequence:create(call, delay)

    return seq

end

function GameStartSkill:perform_change(type)

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

    local perform = function() end

    local t_filter = {}

    t_filter.t_category_filter = {
        type = "and",
        {"normal", "=", true},
        {"fenced", "=", false},
        {"tagged", "=", false},  
    }
    t_filter.t_property_filter = {
        type = "and",
        {"state", "=", "normal"},
    }

    local param = {

        t_category_filter = t_filter.t_category_filter,
        t_property_filter = t_filter.t_property_filter,

    }

    local msg = MessagePool:get_free()
    msg.c_sender = self
    msg.s_name = "query_filter"
    msg.x_param = param

    GameGlobals.Scene:send_message(msg)

    local result = msg.x_result

    if #result > 0 then

        local target = result[1]

        target:send_message(MessagePool:get_free_ex("lock"))

        local perform = function()

            local target_position = cc.p(target:getPosition())

            local item_sprite = ResourceManager:getInstance():getSpriteByKey(GameStartSkill.IMAGE_KEY_MAP[type])
            item_sprite:setPosition(cc.p(540, 1280))
            item_sprite:setOpacity(0)
            item_sprite:setScale(0.3)
            GameGlobals.Scene.__c_scene:addChild(item_sprite, 100)

            local scale = cc.ScaleTo:create(0.4, 1.0)
            local ease = cc.EaseSineIn:create(scale)
            local fade = cc.FadeTo:create(0.4, 255)
            local spawn = cc.Spawn:create(ease, fade)
            item_sprite:runAction(spawn)

            local effect_spine = ResourceManagerEx:get_instance():get_resource_by_key("before_game_effect")
            effect_spine:setAnimation(0, "zhanqiandaoju", false)
            effect_spine:setPosition(cc.p(560, 1280))
            GameGlobals.Scene.__c_scene:addChild(effect_spine, 99)

            local delay = cc.DelayTime:create(0.5)
            local remove = cc.RemoveSelf:create()
            local sequence = cc.Sequence:create(delay, remove)
            effect_spine:runAction(sequence)

            local rand_type = type

            local move = cc.MoveTo:create(0.4, target_position)
            local ease = cc.EaseSineInOut:create(move)
            local call = cc.CallFunc:create(function()

                local particle = ResourceManagerEx:get_instance():get_resource_by_key("bonus_remove_particle")
                particle:setLocalZOrder(99)
                particle:setPosition(target_position)
                particle:setAutoRemoveOnFinish(true)

                GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

                local sugar = ObjectFactory:get_instance():get(rand_type)
                sugar:set("ban_ignite", false)
                sugar:setPosition(target_position)

                if rand_type == "LightningSugar" then

                    local msg = MessagePool:get_free()
                    msg.c_sender = self
                    msg.s_name = "get_valid_color_random"
                    msg.x_param = {}

                    GameGlobals.Scene:send_message(msg)

                    sugar:set("select_color", msg.x_result)


                end

                target:send_message(MessagePool:get_free_ex("remove_sugar_silent", ""))
                GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar_silent", target, Message.N_TUNNEL))

                GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_replace_start", target, Message.N_TUNNEL))

                GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", sugar, Message.N_TUNNEL))

                GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

            end)

            item_sprite:runAction(cc.Sequence:create(cc.DelayTime:create(0.4), ease, call, cc.RemoveSelf:create()))

        end

        local call = cc.CallFunc:create(perform)
        local delay = cc.DelayTime:create(0.9)
        local seq = cc.Sequence:create(call, delay)

        return seq

    end 

    return cc.DelayTime:create(0)

end

function GameStartSkill:on_skill_start(c_msg)

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))


    local action_seq = {}    
    
    if (not self:get("add_move")) and (not self:get("add_bomb")) and (not self:get("add_lightning")) then
        local function run_skill_action()
            local call = cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action")) end)
            action_seq[#action_seq + 1] = call
            GameGlobals.Scene:runAction(cc.Sequence:create(action_seq))
        end

        local function step_gameItemLayer()
            local gameItemLayer = GameItemLayer:create()
            gameItemLayer:init_ui("step")
            gameItemLayer:set_buy_callback(function()
                action_seq[#action_seq + 1] = self:perform_move()
                run_skill_action()
                end)
            gameItemLayer:set_close_callback(function()
                run_skill_action()
                end)
            LayerManager:getInstance():add_layer_to_scene(gameItemLayer)
        end
        local function boom_gameItemLayer()
            local gameItemLayer = GameItemLayer:create()
            gameItemLayer:init_ui("boom")
            gameItemLayer:set_buy_callback(function()
                action_seq[#action_seq + 1] = self:perform_change("BombSugar")
                run_skill_action()
            end)
            gameItemLayer:set_close_callback(function()
                run_skill_action()
            end)
            LayerManager:getInstance():add_layer_to_scene(gameItemLayer)
        end
        local function flash_gameItemLayer()
            local gameItemLayer = GameItemLayer:create()
            gameItemLayer:init_ui("flash")
            gameItemLayer:set_buy_callback(function()
                action_seq[#action_seq + 1] = self:perform_change("LightningSugar")
                run_skill_action()
            end)
            gameItemLayer:set_close_callback(function()
                run_skill_action()
            end)
            LayerManager:getInstance():add_layer_to_scene(gameItemLayer)
        end

        local player = Player:getInstance()
        local gold_num = player:get_player_gold()
        local newest_mission_id = player:get_player_newest_mission_id()
        local mission_info = Mission_Data:getInstance():get_mission_info_by_id(newest_mission_id)
        local newest_level = mission_info.mission_level
        local index = 0
        local item_cost = {}
        for i=1,3 do
            local prop_info = Prop_Data:getInstance():get_prop_info_by_id(string.format("item_20%d", i))
            if newest_level >= prop_info.Unlock_level then
                index = index + 1
            end
            local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(string.format("item_20%d", i))
            item_cost[i] = shop_item_info.cost
        end
        table.sort(item_cost)

        local current_level = player:get_player_current_level()
        local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
        local current_mission_fail_times = player:get_player_mission_fail_times(mission_info.mission_id) or 0

        if index > 0 and current_mission_fail_times>=3 and GameGlobals.Scene:get("type") == "normal" then
            if gold_num >= item_cost[1] and gold_num < item_cost[2] then
                if index > 1 then
                    boom_gameItemLayer()
                end
            elseif gold_num >= item_cost[2] and gold_num < item_cost[3] then
                step_gameItemLayer()
            elseif gold_num >= item_cost[3] then
                if index < 3 then
                    step_gameItemLayer()
                else
                    flash_gameItemLayer()
                end
            else
                run_skill_action()
            end
            
        else
            run_skill_action()
        end
    
    else

        if self:get("add_lightning") then

            action_seq[#action_seq + 1] = self:perform_change("LightningSugar")

        end

        if self:get("add_bomb") then

            action_seq[#action_seq + 1] = self:perform_change("BombSugar")

        end

        if self:get("add_move") then

            action_seq[#action_seq + 1] = self:perform_move()

        end

        local call = cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action")) end)
        action_seq[#action_seq + 1] = call

        GameGlobals.Scene:runAction(cc.Sequence:create(action_seq))

    end

end

GameStartSkill.IMAGE_KEY_MAP = {
    
    ["BombSugar"] = "BEFORE_GAME_BOMB",
    ["LightningSugar"]  = "BEFORE_GAME_LIGHTNING",

}

GameStartSkill.LISTENER_CONFIG = {

	{"start_skill_start", {before = GameStartSkill.on_skill_start}},

}

GameStartSkill.PROPERTY_CONFIG = {
	
	{"add_bomb", false},
    {"add_move", false},
    {"add_lightning", false},

}


return GameStartSkill