-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --




local HammerSkill = class("HammerSkill", function() return GameObject:create() end)


function HammerSkill:ctor()

    self:add_component(ObjectFactory:get_instance():get("SkillTouchHandler"), "touch_handler")
    self:add_component(ObjectFactory:get_instance():get("CircleSelector"), "selector")

	self:register_listeners_with_config(HammerSkill.LISTENER_CONFIG)
	self:register_properties_with_config(HammerSkill.PROPERTY_CONFIG)

    self:set("select_radius", 10)

    local filter = {}

    filter.t_property_filter = {
        type = "and",
        {"state", "=", "normal"},
    }

    filter.t_category_filter = {
        type = "or",
        {"special", "=", true},
        {"normal", "=", true},
        {"fracture", "=", true},
        {"fenced", "=", true},
        {"panda", "=", true},
        {"random", "=", true},
        {"timing_bomb", "=", true},
        {"colorful", "=", true},
    }

    self:get_component("selector"):set("filter", filter)

end

function HammerSkill:on_perform_skill(c_msg)

	self:set("enable", true)

end

function HammerSkill:check_sequence(c_sugar)

    if self:get("mode") == "fixed" then

        local seq = self:get("sequence")

        for _, id in ipairs(seq) do

            if c_sugar:get("id") == id then

                return true

            end            

        end

print("hammer check false")
        return false

    else

        return true

    end

end

function HammerSkill:on_touch_down(c_msg)

    local position = c_msg.x_param
    self:setPosition(position)

    local msg = MessagePool:get_free()
    msg.s_name = "select"
    msg.x_param = {ban_message = true}

    self:get_component("selector"):send_message(msg)

    local select_result = msg.x_result

    if #select_result == 0 or not self:check_sequence(select_result[1])  then

        return

    end

    TRIGGER_EVENT("ITEM_SKILL_START")

    self:set("enable", false)

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_used", "HammerSkill", Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_remove_start"))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_start", {c_list = select_result, t_pos = position}, Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", "", Message.N_TUNNEL))

    local sugar = select_result[1]

    local world_position = cc.p(sugar:getPosition())
    local scale_modify = 1.5

    local spine_position = world_position
    spine_position.x = spine_position.x + 35
    spine_position.y = spine_position.y + 20

    local spine = ResourceManager:get_instance():get_resource_by_key("hammer_remove_spine")
    spine:setPosition(world_position)
    spine:setAnimation(0, "chuizi", false)
    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

    local function perform_remove()

        local particle_position = world_position
        particle_position.x = particle_position.x - 20
        particle_position.y = particle_position.y + 25

        local particle_1 = ResourceManager:get_instance():get_resource_by_key("hammer_remove_particle_1")
        particle_1:setLocalZOrder(10)
        particle_1:setPosition(world_position)
        particle_1:setAutoRemoveOnFinish(true)
        particle_1:setScale(scale_modify)
        GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

        local particle_2 = ResourceManager:get_instance():get_resource_by_key("hammer_remove_particle_2")
        particle_2:setLocalZOrder(10)
        particle_2:setPosition(world_position)
        particle_2:setAutoRemoveOnFinish(true)
        GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

        local particle_3 = ResourceManager:get_instance():get_resource_by_key("hammer_remove_particle_3")
        particle_3:setLocalZOrder(10)
        particle_3:setPosition(world_position)
        particle_3:setAutoRemoveOnFinish(true)
        particle_3:setScale(scale_modify)
        GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

        sugar:send_message(MessagePool:get_free_ex("start_remove", "", Message.N_TUNNEL))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("special_remove_finish", {c_list = {sugar}, t_pos = position}, Message.N_TUNNEL))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("hammer_skill_hit", "", Message.N_TUNNEL))

    end

    RUN_ACTION_WITH_DELAY(0.5, perform_remove)

    local delay = cc.DelayTime:create(1.5)
    local remove = cc.RemoveSelf:create()
    local sequence = cc.Sequence:create(delay, remove)
    spine:runAction(sequence)

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

    local event = cc.EventCustom:new("item_consume")
    event.param = {}   
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function HammerSkill:on_cancel(c_msg)

    self:set("enable", false)

end

HammerSkill.LISTENER_CONFIG = {

	{"perform", {before = HammerSkill.on_perform_skill}},
    {"touch_down", {before = HammerSkill.on_touch_down}},
    {"cancel", {before = HammerSkill.on_cancel}},

}

HammerSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"enable", false},
    {"mode", "arbitary"},
    {"sequence", {}},

}


return HammerSkill