-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --




local LineSkill = class("LineSkill", function() return GameObject:create() end)


function LineSkill:ctor()

    self:add_component(ObjectFactory:get_instance():get("SkillTouchHandler"), "touch_handler")
    self:add_component(ObjectFactory:get_instance():get("LineSkillVisual"), "visual")
    self:add_component(ObjectFactory:get_instance():get("NormalRemover"), "remover")
    self:add_component(ObjectFactory:get_instance():get("LinkGenerator"), "generator")
    self:add_component(ObjectFactory:get_instance():get("GenerateIndicator"), "generate_indicator")
    self:add_component(ObjectFactory:get_instance():get("FractureIndicator"), "fracture_indicator")

    self:get_component("remover"):set("remove_trigger_threshold", 0)

	self:register_listeners_with_config(LineSkill.LISTENER_CONFIG)
	self:register_properties_with_config(LineSkill.PROPERTY_CONFIG)

    self:set("select_radius", 10)

    self.__t_begin_filter = {}
    self.__t_begin_filter.t_property_filter = {
        type = "and",
        {"state", "=", "normal"}
    }

    self.__t_begin_filter.t_category_filter = {
        type = "or",
        {"normal", "=", true},
        {"random", "=", true},
        {"timing_bomb", "=", true},
    }
    self.__t_begin_filter.type = "empty"

    self.__t_continue_filter = {}
    self.__t_continue_filter.t_property_filter = {
        type = "and",
        {"state", "=", "normal"}
    }

    self.__t_continue_filter.t_category_filter = {
        type = "or",
        {"normal", "=", true},
        {"random", "=", true},
        {"timing_bomb", "=", true},
        {"colorful", "=", true},
    }
    self.__t_continue_filter.type = "normal"

    self.__t_last_position = cc.p(0, 0)
    self.__t_current_position = cc.p(0, 0)

    self.__t_select_result = {}
    self.__n_select_result_size = 0

    self.__t_filter = self.__t_begin_filter

end

function LineSkill:on_perform_skill(c_msg)

	self:set("enable", true)

    self.__t_last_position = cc.p(0, 0)
    self.__t_current_position = cc.p(0, 0)

    self.__t_select_result = {}
    self.__n_select_result_size = 0

    self.__t_filter = self.__t_begin_filter

end

function LineSkill:on_touch_down(c_msg)

    local param = {

        t_p = c_msg.x_param,

    }

    local msg = MessagePool:get_free()
    msg.s_name = "query_point"
    msg.x_param = param
    GameGlobals.Scene:send_message(msg)

    self:apply_result(msg.x_result)

    self.__t_start_position = c_msg.x_param
    self.__t_current_position = c_msg.x_param    

end

function LineSkill:on_touch_move(c_msg)

    self.__t_current_position = c_msg.x_param

    local position = {x = (self.__t_current_position.x + self.__t_start_position.x) / 2, y = (self.__t_current_position.y + self.__t_start_position.y) / 2}
    local direction = UtilFunction:caculate_direction(self.__t_current_position, self.__t_start_position)
    local rotation = UtilFunction:caculate_angle(direction)
    local length = UtilFunction:caculate_distance(direction, t_position_2)
    local width = 10

    local param = {

        c_shape = self.__c_select_box,
        t_p = position,
        n_rotation = rotation,
        n_scale_x = length,
        n_scale_y = width,
        t_p1 = self.__t_start_position,
        t_p2 = self.__t_current_position,

    }

    local msg = MessagePool:get_free()
    msg.c_sender = self
    msg.s_name = "query_line"
    msg.x_param = param

    GameGlobals.Scene:send_message(msg)

    self:apply_result(msg.x_result)

    self.__t_start_position = self.__t_current_position

end

function LineSkill:on_touch_up(c_msg)

    if not self:check_sequence_finish() then

        self:cancel()
        return

    end

    self:finish()

end

function LineSkill:apply_result(t_query_result)

    local original_size = self.__n_select_result_size

    local result_size = #t_query_result

    for i = 1, result_size do
        
        local sugar = t_query_result[i]

        if sugar:property_filter(self.__t_filter.t_property_filter) and sugar:category_filter(self.__t_filter.t_category_filter) then

            self:apply_rear(sugar)
            self:change_filter()

        end

    end

    if not (original_size == self.__n_select_result_size) then

        self.__t_select_result.type = "normal"
        self:send_message(MessagePool:get_free_ex("select_continue", self.__t_select_result, Message.N_TUNNEL))

    end

    local msg = MessagePool:get_free()
    msg.s_name = "update_visual"
    msg.x_param = self.__t_select_result
    msg.e_transmit = Message.N_TUNNEL

    self:send_message(msg)

    local msg = MessagePool:get_free()
    msg.s_name = "condition"
    msg.x_param = self.__t_select_result
    msg.e_transmit = Message.N_TUNNEL

    self:send_message(msg)

end

function LineSkill:change_filter()

    if self.__t_filter.type == "empty" then

        if self.__n_select_result_size == 1 then

            local rear_sugar = self.__t_select_result[1]

            self.__t_continue_filter.t_property_filter[2] = {"color", "=", rear_sugar:get("color")}

            self.__t_filter = self.__t_continue_filter

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_defined", rear_sugar:get("color"), Message.N_TUNNEL))

        end

    end

end

function LineSkill:apply_rear(c_sugar)

    if self:check_sequence(c_sugar) then

        self:select(c_sugar)

        self.__t_select_result[self.__n_select_result_size + 1] = c_sugar
        self.__n_select_result_size = self.__n_select_result_size + 1

        local index = self.__n_select_result_size

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("link_selected", index, Message.N_TUNNEL))

    end

end

function LineSkill:check_sequence(c_sugar)

    if self:get("mode") == "fixed" then

        local seq = self:get("sequence")

        local next_index = self.__n_select_result_size + 1

        if c_sugar:get("id") == seq[next_index] then

            return true

        else

            return false

        end

    else

        return true

    end

end

function LineSkill:select(c_sugar)

    c_sugar:send_message(MessagePool:get_free_ex("select"))

end

function LineSkill:check_sequence_finish()

    if (self:get("mode") == "fixed" and #self:get("sequence") == self.__n_select_result_size) or
        self:get("mode") == "arbitary" then

        return true

    else

        return false

    end

end

function LineSkill:on_link_remove_finish(c_msg)

    local msg = MessagePool:get_free()
    msg.s_name = "generate"
    msg.x_param = c_msg.x_param
    msg.e_transmit = Message.N_TUNNEL

    self:send_message(msg)

    TRIGGER_EVENT("ITEM_SKILL_FINISH")

end

function LineSkill:on_cancel(c_msg)

    self:cancel()
    self:set("enable", false)

end

function LineSkill:cancel()

    local return_result = self.__t_select_result

    for i = 1, self.__n_select_result_size do

        return_result[i]:send_message(MessagePool:get_free_ex("unselect"))

    end

    self.__t_select_result = {}
    self.__n_select_result_size = 0
    self.__t_filter = self.__t_begin_filter

    self.__c_parent:send_message(MessagePool:get_free_ex("select_cancel", return_result, Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))

    local msg = MessagePool:get_free()
    msg.s_name = "update_visual"
    msg.x_param = self.__t_select_result
    msg.e_transmit = Message.N_TUNNEL

    self:send_message(msg)

    self:set("enable", true)

end

function LineSkill:finish()

    local return_result = self.__t_select_result
    return_result.type = self.__t_filter.type

    local msg = MessagePool:get_free()
    msg.s_name = "update_visual"
    msg.x_param = {}
    msg.e_transmit = Message.N_TUNNEL

    self:send_message(msg)

    if #return_result > 0 then

        TRIGGER_EVENT("ITEM_SKILL_START")

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_used", "LineSkill", Message.N_TUNNEL))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("item_remove_start"))

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", "", Message.N_TUNNEL))

        local msg = MessagePool:get_free()
        msg.s_name = "assign_remove_task"
        msg.x_param = return_result
        msg.e_transmit = Message.N_TUNNEL

        self:send_message(msg)

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

    end

    self:send_message(MessagePool:get_free_ex("select_finish", self.__t_select_result, Message.N_TUNNEL))

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("color_undefined", "", Message.N_TUNNEL))

    self:set("enable", false)    

end

LineSkill.LISTENER_CONFIG = {

	{"perform", {before = LineSkill.on_perform_skill}},
    {"touch_down", {before = LineSkill.on_touch_down}},
    {"touch_up", {before = LineSkill.on_touch_up}},
    {"touch_cancel", {before = LineSkill.on_touch_up}},
    {"touch_move", {before = LineSkill.on_touch_move}},
    {"cancel", {before = LineSkill.on_cancel}},
    {"remove_finish", {before = LineSkill.on_link_remove_finish}},

}

LineSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"enable", false},
    {"mode", "arbitary"},
    {"sequence", {}},

}


return LineSkill