-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --


local NiNiSkill = class("NiNiSkill", function() return GameObject:create() end)


function NiNiSkill:ctor()

    self.__n_current_remove_count = 0

	self:register_listeners_with_config(NiNiSkill.LISTENER_CONFIG)
	self:register_properties_with_config(NiNiSkill.PROPERTY_CONFIG)

end

function NiNiSkill:on_skill_start(c_msg)

    if self.__n_current_remove_count >= self:get("perform_threshold") then

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("player_skill_cast", "", Message.N_TUNNEL))

        local t_filter = {}

        t_filter.t_category_filter = {
            type = "and",
            {
                type = "or",
                {"ice", "=", true},
                {"chocolate", "=", true}, 
                {"moveable_ice", "=", true},  
                {"fenced", "=", true},
                {"soil", "=", true},
                {"conch", "=", true},
                {"gift", "=", true},
                {"coco", "=", true},
                {"timing_bomb", "=", true},
                {"random", "=", true},
                {"reward_bag", "=", true},
            },
            {"tagged", "=", false},
        }
        t_filter.t_property_filter = {
            type = "and",
            {"state", "=", "normal"},
        }

        local param = {

            t_category_filter = t_filter.t_category_filter,
            t_property_filter = t_filter.t_property_filter,

        }

        local msg = MessagePool:get_free()
        msg.c_sender = self
        msg.s_name = "query_filter"
        msg.x_param = param

        GameGlobals.Scene:send_message(msg)

        if #msg.x_result == 0 then

            local t_filter = {}

            t_filter.t_category_filter = {
                type = "and",
                {
                    type = "or",
                    {"normal", "=", true},
                    {"transform", "=", true},
                    {"add_move", "=", false},
                },
                {"panda", "=", false},
                {"special", "=", false},
                {"tagged", "=", false}, 
            }
            t_filter.t_property_filter = {
                type = "and",
                {"state", "=", "normal"},
            }

            local param = {

                t_category_filter = t_filter.t_category_filter,
                t_property_filter = t_filter.t_property_filter,

            }

            local msg2 = MessagePool:get_free()
            msg2.c_sender = self
            msg2.s_name = "query_filter"
            msg2.x_param = param

            GameGlobals.Scene:send_message(msg2)

            msg.x_result = msg2.x_result

        end

        if #msg.x_result > 0 then

            local counter = self:get("change_number")
            local count = 0 

            if #msg.x_result < counter then counter = #msg.x_result end

            local actual_result = {}

            for i = 1, counter do

                actual_result[i] = msg.x_result[i]

            end

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("skill_remove_start", actual_result, Message.N_TUNNEL))

            for i = 1, counter do 

                local target = msg.x_result[i]

                local visible_size = cc.Director:getInstance():getVisibleSize()

                local homing_entry = 0
                local homing_particle = ResourceManager:get_instance():get_resource_by_key("skill_flying_particle")
                homing_particle:setDuration(-1)
                homing_particle:setPosition(cc.p(self.__c_parent:getPosition()))
                homing_particle:setLocalZOrder(11)
                homing_particle:setAutoRemoveOnFinish(false)
                homing_particle:setSpeed(200)
                homing_particle:setPosition(cc.p(visible_size.width / 2 - 470, 1500))
                GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

                local homing_performer = function()

                    local target_position = target:getParent():convertToWorldSpace(cc.p(target:getPosition()))
                    local particle_position = cc.p(homing_particle:getPosition())

                    local distance = UtilFunction:caculate_distance(target_position, particle_position)

                    if distance < self:get("homing_speed") then

                        local sugar = target

                        sugar:send_message(MessagePool:get_free_ex("start_remove", "nini", Message.N_TUNNEL))

                        homing_particle:setDuration(0.1)
                        homing_particle:setAutoRemoveOnFinish(true)

                        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

                        count = count + 1

                        if count == counter then

                            GameGlobals.Scene:send_message(MessagePool:get_free_ex("skill_remove_finish", actual_result, Message.N_TUNNEL))

                            GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

                        end

                    else

                        local speed = self:get("homing_speed")
                        local direction = UtilFunction:caculate_direction(target_position, particle_position)
                        local angle = UtilFunction:caculate_angle(direction)
                        local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
                        particle_position.x = particle_position.x + offset.x
                        particle_position.y = particle_position.y + offset.y

                        homing_particle:setAngle(-angle)
                        homing_particle:setPosition(particle_position)

                    end

                end

                homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

            end

            self.__n_current_remove_count = 0
            
            GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))
            GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

        end         

    end

end

function NiNiSkill:on_sugar_remove(c_msg)

    local category = c_msg.x_param:get_category()

    if category:has("normal") or category:has("colorful") or category:has("random") or category:has("timing_bomb")  then

        self.__n_current_remove_count = self.__n_current_remove_count + 1

    end

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))
print(string.format("skill: %d [count]", self.__n_current_remove_count))
end

function NiNiSkill:on_fill_skill(c_msg)

    self.__n_current_remove_count = self:get("perform_threshold")

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))

end


NiNiSkill.LISTENER_CONFIG = {

	{"player_skill_start", {before = NiNiSkill.on_skill_start}},
    {"remove_sugar", {before = NiNiSkill.on_sugar_remove}},
    {"fill_skill", {before = NiNiSkill.on_fill_skill}},

}

NiNiSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"perform_threshold", 30},
    {"change_number", 1},

}


return NiNiSkill