-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager_Core = require "core.ResourceManager"

-- ## Imports ## --


local TournamentRewardSkill = class("TournamentRewardSkill", function() return GameObject:create() end)


function TournamentRewardSkill:ctor()

    self.__n_current_remove_count = 0

	self:register_listeners_with_config(TournamentRewardSkill.LISTENER_CONFIG)
	self:register_properties_with_config(TournamentRewardSkill.PROPERTY_CONFIG)

end

function TournamentRewardSkill:on_skill_start(c_msg)

    local visible_size = cc.Director:getInstance():getVisibleSize()

    local spine = ResourceManager:get_instance():get_resource_by_key("tournament_reward")
    spine:setAnimation(0, "weekgame_reward", false)
    spine:setPosition(cc.p(visible_size.width / 2, visible_size.height / 2))

    GameGlobals.Scene.__c_scene:addChild(spine, 10)

    local delay_1 = cc.DelayTime:create(1.8)
    local delay_2 = cc.DelayTime:create(0.3)
    local call_1 = cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("reward_conch", self:get("conch_reward_number"))) end)
    local remove = cc.RemoveSelf:create()
    local call_2 = cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("reward_skill_finish", "")) end)
    local seq = cc.Sequence:create(delay_1, call_1, delay_2, call_2, remove)

    spine:runAction(seq)

    local delay_1 = cc.DelayTime:create(0.6)
    local call_1 = cc.CallFunc:create(function() MusicManager:getInstance().play_effect(ResourceManager_Core:getInstance():getAudioPathByKey("Chest_Audio"), false) end)
    local seq = cc.Sequence:create(delay_1, call_1)

    spine:runAction(seq)

end

function TournamentRewardSkill:dispatch_reward(c_msg)

    local visible_size = cc.Director:getInstance():getVisibleSize()

    local spine = ResourceManager:get_instance():get_resource_by_key("tournament_reward")
    spine:setAnimation(0, "weekgame_reward", false)
    spine:setPosition(cc.p(visible_size.width / 2, visible_size.height / 2))

    GameGlobals.Scene.__c_scene:addChild(spine, 10)

    local delay_1 = cc.DelayTime:create(1.8)
    local delay_2 = cc.DelayTime:create(0.3)
    local call_1 = cc.CallFunc:create(function() GameGlobals.Scene:send_message(MessagePool:get_free_ex("reward_conch", self:get("conch_reward_number"))) end)
    local remove = cc.RemoveSelf:create()
    local call_2 = cc.CallFunc:create(function() GameGlobals.Scene:get_component("ui_component").__c_state_machine:do_transition("bonus") end)
    local seq = cc.Sequence:create(delay_1, call_1, delay_2, call_2, remove)

    spine:runAction(seq)    

end


TournamentRewardSkill.LISTENER_CONFIG = {

	{"tournament_reward_start", {before = TournamentRewardSkill.on_skill_start}},

}

TournamentRewardSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"conch_reward_number", 8},

}


return TournamentRewardSkill