-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local RandomGenerator = require "GameEx.Foundations.Utils.RandomGenerator"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"

-- ## Imports ## --




local VictorySkill = class("VictorySkill", function() return GameObject:create() end)


function VictorySkill:ctor()

	self:register_listeners_with_config(VictorySkill.LISTENER_CONFIG)
	self:register_properties_with_config(VictorySkill.PROPERTY_CONFIG)

	self.__n_task_number = 0
	self.__n_original_runned = false

	self.__t_random_list = {

		"BombSugar",
		"CrossSugar",
		"LineSugar",

	}

	self.__c_random_generator = GameGlobals.RandomGenerator
	self.__n_first_time_flag = true	

end

function VictorySkill:on_assemble()

	self.__c_parent:addChild(self)

end

function VictorySkill:on_disassemble()

	self.__c_parent:removeChild(self)

end

function VictorySkill:on_perform_skill(c_msg)

	self.__n_task_number = c_msg.x_param

	if self:will_perform() then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_perform", "", Message.N_TUNNEL))

		self:apply_bonus()

	else

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_skip"))

	end

end

function VictorySkill:on_bonus_round_finish(c_msg)

	if c_msg.x_param == "bonus" then

		self:apply_bonus()

	end

end

function VictorySkill:will_perform()

	if self:has_task() or self:has_original() then

		return true

	else

		return false

	end

end

function VictorySkill:has_task()

	if self.__n_task_number > 0 then

		return true

	else

		return false

	end

end

function VictorySkill:has_original()

	local t_filter = {}

	t_filter.t_category_filter = {
		type = "or",
		{"special", "=", true},	
		{"random", "=", true},	
	}
	t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local result = msg.x_result

	if #result > 0 and not self.__n_original_runned then

		return true

	else
			
		return false

	end	

end

function VictorySkill:ignite_original()

	local t_filter = {}

	t_filter.t_category_filter = {
		type = "or",
		{"special", "=", true},	
		{"random", "=", true},	
	}
	t_filter.t_property_filter = {
		type = "and",
		{"state", "=", "normal"},
	}

	local param = {

		t_category_filter = t_filter.t_category_filter,
		t_property_filter = t_filter.t_property_filter,

	}

	local msg = MessagePool:get_free()
	msg.c_sender = self
	msg.s_name = "query_filter"
	msg.x_param = param

	GameGlobals.Scene:send_message(msg)

	local result = msg.x_result

	if #result > 0 then

		local function ignite()

			for i = 1, #result do 

				result[i]:set("ban_ignite", true)

			end

			for i = 1, #result do 

				result[i]:send_message(MessagePool:get_free_ex("start_remove"))

			end

		end

		RUN_ACTION_WITH_DELAY(2.0, ignite)

		self.__n_original_runned = true

		self.__n_first_time_flag = false

	end

end

function VictorySkill:do_task()

		local function select_and_replace()

			local t_filter = {}

			t_filter.t_category_filter = {
				type = "and",
				{"normal", "=", true},
				{"fenced", "=", false},	
			}
			t_filter.t_property_filter = {
				type = "and",
				{"state", "=", "normal"},
			}

			local param = {

				t_category_filter = t_filter.t_category_filter,
				t_property_filter = t_filter.t_property_filter,

			}

			local msg = MessagePool:get_free()
			msg.c_sender = self
			msg.s_name = "query_filter"
			msg.x_param = param

			GameGlobals.Scene:send_message(msg)

			local result = msg.x_result

			local result_size = #result
			local per_time_number = self:get("per_time_number")

			local remove_number = result_size > per_time_number and per_time_number or result_size
			local remove_number = remove_number > self.__n_task_number and self.__n_task_number or remove_number

			local per_round_ignite_list = {}

			local random_draw = RandomDraw:create()
  			random_draw:set_target(remove_number)


			for i = 1, remove_number do

				local index = random_draw:draw_next_repeat()

				local function replace()

					local target = result[index]
					local target_position = cc.p(target:getPosition())

					local visible_size = cc.Director:getInstance():getVisibleSize()

					local homing_entry = 0
	                local homing_particle = ResourceManager:get_instance():get_resource_by_key("skill_flying_particle")
	                homing_particle:setDuration(-1)
	                homing_particle:setLocalZOrder(11)
	                homing_particle:setAutoRemoveOnFinish(false)
	                homing_particle:setSpeed(100)
	                homing_particle:setPosition(cc.p(visible_size.width / 2 - 360, 1580))
	                -- homing_particle:setPositionType(1)
	                GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

	                local dist = UtilFunction:caculate_distance(target_position, cc.p(visible_size.width / 2 - 360, 1580))
	                local speed = dist / 0.1 / 60

	                local homing_performer = function()

	                    local target_position = target:getParent():convertToWorldSpace(cc.p(target:getPosition()))
	                    local particle_position = cc.p(homing_particle:getPosition())

	                    local distance = UtilFunction:caculate_distance(target_position, particle_position)

	                    if distance < speed then

	                        local particle = ResourceManager:get_instance():get_resource_by_key("bonus_remove_particle")
							particle:setLocalZOrder(99)
							particle:setPosition(target_position)
							particle:setAutoRemoveOnFinish(true)
							particle:setScale(1.6)

							GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

							local rand_type = self.__t_random_list[self.__c_random_generator:rand(1, 3)]
							local sugar = ObjectFactory:get_instance():get(rand_type)
							sugar:set("ban_ignite", true)
							sugar:setPosition(target_position)
							per_round_ignite_list[#per_round_ignite_list + 1] = sugar

							GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_replace", target, Message.N_TUNNEL))

							GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", target, Message.N_TUNNEL))
							GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", sugar, Message.N_TUNNEL))

							homing_particle:setDuration(0)
                        	homing_particle:setAutoRemoveOnFinish(true)


                        	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

	                    else

	                        -- local speed = self:get("homing_speed")
	                        local direction = UtilFunction:caculate_direction(target_position, particle_position)
	                        local angle = UtilFunction:caculate_angle(direction)
	                        local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
	                        particle_position.x = particle_position.x + offset.x
	                        particle_position.y = particle_position.y + offset.y

	                        homing_particle:setAngle(-angle)
	                        homing_particle:setPosition(particle_position)

	                    end

	                end

	                homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

				end

				RUN_ACTION_WITH_DELAY(0.3 * i, replace)

			end

			local function do_ignite()

				for i = 1, #per_round_ignite_list do

					local index = i

					local function ignite_single()

						per_round_ignite_list[index]:send_message(MessagePool:get_free_ex("start_remove"))

					end

					RUN_ACTION_WITH_DELAY(0.3 * (i - 1), ignite_single) 

				end

			end

			RUN_ACTION_WITH_DELAY(0.3 * remove_number + 0.6, do_ignite)  

			self.__n_task_number = self.__n_task_number - remove_number

		end

		if self.__n_first_time_flag then

			RUN_ACTION_WITH_DELAY(2.0, select_and_replace)
			self.__n_first_time_flag = false

		else

			RUN_ACTION_WITH_DELAY(0.4, select_and_replace)

		end

end

function VictorySkill:apply_bonus()

	if self:will_perform() then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_apply_start"))

		if self:has_original() then

			self:ignite_original()

		else

			self:do_task()

		end

	else

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("bonus_finish", "", Message.N_TUNNEL))

	end

end

VictorySkill.LISTENER_CONFIG = {

	{"perform", {before = VictorySkill.on_perform_skill}},
	{"round_finish", {before = VictorySkill.on_bonus_round_finish}},
	{"assemble", {before = VictorySkill.on_assemble}},
	{"disassemble", {before = VictorySkill.on_disassemble}},

}

VictorySkill.PROPERTY_CONFIG = {
	
	{"per_time_number", 20},
	{"homing_speed", 80}

}


return VictorySkill