-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --


local ZhengNanSkill = class("ZhengNanSkill", function() return GameObject:create() end)


function ZhengNanSkill:ctor()

    self.__n_current_remove_count = 0

	self:register_listeners_with_config(ZhengNanSkill.LISTENER_CONFIG)
	self:register_properties_with_config(ZhengNanSkill.PROPERTY_CONFIG)

end

function ZhengNanSkill:on_skill_start(c_msg)

    if self.__n_current_remove_count >= self:get("perform_threshold") then

        GameGlobals.Scene:send_message(MessagePool:get_free_ex("player_skill_cast", "", Message.N_TUNNEL))

        local t_filter = {}

        t_filter.t_category_filter = {
            type = "and",
            {"normal", "=", true},
            {"fenced", "=", false},
            {"add_move", "=", false},   
        }
        t_filter.t_property_filter = {
            type = "and",
            {"state", "=", "normal"},
        }

        local param = {

            t_category_filter = t_filter.t_category_filter,
            t_property_filter = t_filter.t_property_filter,

        }

        local msg = MessagePool:get_free()
        msg.c_sender = self
        msg.s_name = "query_filter"
        msg.x_param = param

        GameGlobals.Scene:send_message(msg)

        if #msg.x_result > 0 then

            local role_level = Player:getInstance():get_player_skill_level("role_301")
            local counter = 0

            if role_level < 3 then

                counter = 1;

            elseif role_level >= 3 and role_level < 5 then

                local rand = GameGlobals.RandomGenerator:rand(1, 100)

                if rand < 36 then

                    counter = 2

                else

                    counter = 1

                end

            elseif role_level >= 5 then

                counter = 2

            end

            local actual_result = {}

            for i = 1, counter do

                actual_result[i] = msg.x_result[i]

            end

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("skill_remove_start", actual_result, Message.N_TUNNEL))

            local count = 0 

            if #msg.x_result < counter then counter = #msg.x_result end

            for i = 1, counter do 

                local target = msg.x_result[i]

                target:send_message(MessagePool:get_free_ex("lock"))

                local visible_size = cc.Director:getInstance():getVisibleSize()

                local homing_entry = 0
                local homing_particle = ResourceManager:get_instance():get_resource_by_key("skill_flying_particle")
                homing_particle:setDuration(-1)
                homing_particle:setPosition(cc.p(self.__c_parent:getPosition()))
                homing_particle:setLocalZOrder(11)
                homing_particle:setAutoRemoveOnFinish(false)
                homing_particle:setSpeed(200)
                homing_particle:setPosition(cc.p(visible_size.width / 2 - 470, 1500))
                GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

                local homing_performer = function()

                    local target_position = target:getParent():convertToWorldSpace(cc.p(target:getPosition()))
                    local particle_position = cc.p(homing_particle:getPosition())

                    local distance = UtilFunction:caculate_distance(target_position, particle_position)

                    if distance < self:get("homing_speed") then

                        local sugar = target

                        local position = cc.p(sugar:getPosition())

                        sugar:send_message(MessagePool:get_free_ex("start_remove", ""))

                        local line = ObjectFactory:get_instance():get("BombSugar")
                        line:setPosition(position)
                        GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar", line, Message.N_TUNNEL))

                        homing_particle:setDuration(0.1)
                        homing_particle:setAutoRemoveOnFinish(true)

                        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

                        count = count + 1

                        if count == counter then

                            GameGlobals.Scene:send_message(MessagePool:get_free_ex("skill_remove_finish", actual_result, Message.N_TUNNEL))

                            GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

                        end

                    else

                        local speed = self:get("homing_speed")
                        local direction = UtilFunction:caculate_direction(target_position, particle_position)
                        local angle = UtilFunction:caculate_angle(direction)
                        local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
                        particle_position.x = particle_position.x + offset.x
                        particle_position.y = particle_position.y + offset.y

                        homing_particle:setAngle(-angle)
                        homing_particle:setPosition(particle_position)

                    end

                end

                homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

            end

            self.__n_current_remove_count = 0

            GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))
            GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))


        end         

    end

end

function ZhengNanSkill:on_sugar_remove(c_msg)

    local category = c_msg.x_param:get_category()

    if category:has("normal") or category:has("colorful") or category:has("random") or category:has("timing_bomb")  then

        self.__n_current_remove_count = self.__n_current_remove_count + 1

    end

    GameGlobals.Scene:send_message(MessagePool:get_free_ex("update_skill_fill", self.__n_current_remove_count / self:get("perform_threshold") * 100, Message.N_TUNNEL))
print(string.format("skill: %d [count]", self.__n_current_remove_count))
end

ZhengNanSkill.LISTENER_CONFIG = {

	{"player_skill_start", {before = ZhengNanSkill.on_skill_start}},
    {"remove_sugar", {before = ZhengNanSkill.on_sugar_remove}},

}

ZhengNanSkill.PROPERTY_CONFIG = {
	
	{"homing_speed", 30},
    {"perform_threshold", 30},
    {"change_number", 1},

}


return ZhengNanSkill