-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local CircleCollider = require "GameEx.Components.Colliders.CircleCollider"
local TimingBombSugarVisual = require "GameEx.Components.Visuals.TimingBombSugarVisual"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"
local RandomGenerator = require "GameEx.Foundations.Utils.RandomGenerator"
local ConfigManager = require "GameEx.Foundations.ConfigManager"

-- ## Imports ## --




local AddMoveSugar = class("AddMoveSugar", function() return GameObject:create() end)



function AddMoveSugar:ctor()

	self:init_state_machine()

	local visual = ObjectFactory:get_instance():get("AddMoveSugarVisual")
	self:add_component(visual, "visual")

	local collider = ObjectFactory:get_instance():get("CircleCollider")
	self:add_component(collider, "collider")

	local category = self:get_category()
	category:add("add_move")
	category:add("normal")

	self.__n_random_flag = false

	self.__c_random_generator = RandomGenerator:create()
	self.__c_color_prob_getter = ProbGetter:create(GameGlobals.RandomGenerator)

	self:register_listeners_with_config(AddMoveSugar.LISTENER_CONFIG)
	self:register_properties_with_config(AddMoveSugar.PROPERTY_CONFIG)

end

function AddMoveSugar:on_destroy()



end

function AddMoveSugar:on_enter()

	if self.__n_random_flag then

		self:set("color", self.__c_color_prob_getter:get())

	end

	self.__n_random_flag = false

end

function AddMoveSugar:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

	self:set("state", "normal")

end

function AddMoveSugar:on_lock()
	
	self.__c_state_machine:do_event("lock")

end

function AddMoveSugar:on_select()
	
	self.__c_state_machine:do_event("select")

end

function AddMoveSugar:on_unselect()

	self.__c_state_machine:do_event("unselect")

end

function AddMoveSugar:on_start_remove(c_msg)

	if not c_msg.n_handled then

		self.__c_state_machine:do_event("remove")

	end

end

function AddMoveSugar:on_remove_started(c_msg)



end

function AddMoveSugar:on_finish_remove()

	self:send_message(MessagePool:get_free_ex("remove_finished", "", Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", self, Message.N_TUNNEL))

end

function AddMoveSugar:add_move()
-- ERROR_ASSERT(false, "string")

	if GameGlobals.Scene:get("state") == "player_remove_section" or GameGlobals.Scene:get("state") == "item_remove_section" then

		local visible_size = cc.Director:getInstance():getVisibleSize()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", "", Message.N_TUNNEL))

		local target_position = cc.p(visible_size.width / 2 - 360, 1580)

		local visible_size = cc.Director:getInstance():getVisibleSize()

		local homing_entry = 0
	    local homing_particle = ResourceManager:get_instance():get_resource_by_key("skill_flying_particle")
	    homing_particle:setDuration(-1)
	    homing_particle:setLocalZOrder(11)
	    homing_particle:setAutoRemoveOnFinish(false)
	    homing_particle:setSpeed(100)
	    homing_particle:setPosition(cc.p(self:getPosition()))
	    GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", homing_particle))

	    local dist = UtilFunction:caculate_distance(target_position, cc.p(self:getPosition()))
	    local speed = dist / 0.1 / 60

	    local homing_performer = function()

	        local target_position = cc.p(visible_size.width / 2 - 360, 1580)
	        local particle_position = cc.p(homing_particle:getPosition())

	        local distance = UtilFunction:caculate_distance(target_position, particle_position)

	        if distance < speed then

	            local particle = ResourceManager:get_instance():get_resource_by_key("bonus_remove_particle")
				particle:setLocalZOrder(99)
				particle:setPosition(target_position)
				particle:setAutoRemoveOnFinish(true)
				particle:setScale(1.6)

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_move", 5, Message.N_TUNNEL))

				homing_particle:setDuration(0)
	        	homing_particle:setAutoRemoveOnFinish(true)


	        	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(homing_entry)

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", "", Message.N_TUNNEL))

	        else

	            -- local speed = self:get("homing_speed")
	            local direction = UtilFunction:caculate_direction(target_position, particle_position)
	            local angle = UtilFunction:caculate_angle(direction)
	            local offset = cc.p(speed * math.cos(math.rad(-angle)), speed * math.sin(math.rad(-angle)))
	            particle_position.x = particle_position.x + offset.x
	            particle_position.y = particle_position.y + offset.y

	            homing_particle:setAngle(-angle)
	            homing_particle:setPosition(particle_position)

	        end

	    end

	    homing_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(homing_performer, 0.016, false)

    end	

end

function AddMoveSugar:after_set_state(c_msg)

	self.__c_state_machine:do_transition(c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_id(c_msg)

	self:setName(c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end

function AddMoveSugar:before_set_color(c_msg)

	if c_msg.x_param.x_new_value == "random" then

		self.__n_random_flag = true

	else

		self.__n_random_flag = false

	end

end

function AddMoveSugar:after_set_prob_red(c_msg)

	self:change_prob("red", c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_prob_green(c_msg)

	self:change_prob("green", c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_prob_yellow(c_msg)

	self:change_prob("yellow", c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_prob_pink(c_msg)

	self:change_prob("pink", c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_prob_blue(c_msg)

	self:change_prob("blue", c_msg.x_param.x_new_value)

end

function AddMoveSugar:after_set_prob_white(c_msg)

	self:change_prob("white", c_msg.x_param.x_new_value)

end

function AddMoveSugar:change_prob(s_key, n_prob)

	if self.__c_color_prob_getter:has(s_key) then

		self.__c_color_prob_getter:modify(s_key, n_prob)

	else

		self.__c_color_prob_getter:add(s_key, s_key, n_prob)

	end	

end

function AddMoveSugar:init_state_machine()

	function on_enter_remove()

		self:send_message(MessagePool:get_free_ex("remove_started", "", Message.N_TUNNEL))

	end

	function on_enter_select()

		self:send_message(MessagePool:get_free_ex("selected", "", Message.N_TUNNEL))

	end

	function on_leave_select()
		
		self:send_message(MessagePool:get_free_ex("unselected", "", Message.N_TUNNEL))

	end


	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "normal", enter = on_enter_normal},
		{name = "select", enter = on_enter_select, leave = on_leave_select},
		{name = "remove", enter = on_enter_remove},
		{name = "lock"},

	})
	self.__c_state_machine:add_transition({

		{event = "lock", from = "normal", to = "lock"},
		{event = "remove", from = "normal", to = "remove"},
		{event = "remove", from = "select", to = "remove"},
		{event = "remove", from = "lock", to = "remove"},
		{event = "select", from = "normal", to = "select"},
		{event = "select", from = "lock", to = "select"},
		{event = "unselect", from = "select", to = "normal"},

	})
	self.__c_state_machine:do_set("normal")

end


AddMoveSugar.LISTENER_CONFIG = {

	{"lock", {before = AddMoveSugar.on_lock}},
	{"select", {before = AddMoveSugar.on_select}},
	{"unselect", {before = AddMoveSugar.on_unselect}},
	{"start_remove", {after = AddMoveSugar.on_start_remove}},
	{"finish_remove", {before = AddMoveSugar.on_finish_remove}},
	{"remove_finished", {before = AddMoveSugar.add_move}},
	{"remove_started", {before = AddMoveSugar.on_remove_started}},
	{"remove_sugar_silent", {before = AddMoveSugar.add_move}},
	{"target", {before = AddMoveSugar.on_target}},
	{"reset", {before = AddMoveSugar.on_reset}},
	{"enter", {before = AddMoveSugar.on_enter}},

}

AddMoveSugar.PROPERTY_CONFIG = {
	
	{"dynamic", true},
	{"prob_red", 0},
	{"prob_green", 0},
	{"prob_blue", 10},
	{"prob_yellow", 10},
	{"prob_pink", 0},
	{"prob_white", 0},
	{"color", "random"},
	{"radius", 60},
	{"state", "normal"},
	{"id", "random"}

}


return AddMoveSugar