-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local BossSkill = require "GameEx.Components.Skills.BossSkill"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local BossSugar = class("BossSugar", function() return GameObject:create() end)


function BossSugar:ctor()

	local visual = ObjectFactory:get_instance():get("BossSugarVisual")
	self:add_component(visual, "visual")

	local skill = BossSkill:create()
	self:add_component(skill, "skill")

	self.__n_round_counter = 0
	self.__n_damage_accumulator = 0
	self.__n_current_hp = 100
	self.__n_current_hp_threshold = 101

	self.__f_damge_caculator = function() return 10 end
	self.__f_damage_manipulator = function() end
	self.__f_boss_move = function() end
	self.__f_boss_finish = function() end

	self:register_listeners_with_config(BossSugar.LISTENER_CONFIG)
	self:register_properties_with_config(BossSugar.PROPERTY_CONFIG)

	self:init_state_machine()

	self:setName("boss")

end

function BossSugar:on_destroy()


end

function BossSugar:on_reset(c_msg)

	local config = c_msg.x_param

	for key, value in pairs(config) do

		self:set(key, value)

	end

	self.__n_round_counter = self:get("attack_cooldown")
	self.__n_damage_accumulator = 0
	self.__n_current_hp = self:get("max_hp")
	self.__n_current_hp_threshold = 999
	self:change_boss_stage()

	self.__c_state_machine:do_transition("idle")

end

function BossSugar:on_assemble()

	local visible_size = cc.Director:getInstance():getVisibleSize()

	self:setPosition(cc.p(visible_size.width / 2 + 15, 1515))
	self:setLocalZOrder(1)

	GameGlobals.Scene.__c_scene:addChild(self)

end

function BossSugar:on_disassemble()

	self:removeFromParent()

end

-- dizzle state handlers

function BossSugar:dizzle_damage_manipulator(damage)

	self:apply_damage(damage)

end

function BossSugar:dizzle_move()
print("dizzle move !!!!")
	self.__n_round_counter = self.__n_round_counter - 1
print(self.__n_round_counter)
	local dizzle_str = string.format("晕 x%d", self.__n_round_counter)
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", dizzle_str, Message.N_TUNNEL))

end

function BossSugar:dizzle_finish()

	if self.__n_round_counter == 0 then
print("dizzle finish !!!!")
		self.__c_state_machine:do_event("idle")

	end	

end

-- normal state handlers

function BossSugar:normal_damage_manipulator(damage)

	self:apply_damage(damage)

end

function BossSugar:normal_move()

	self.__n_round_counter = self.__n_round_counter - 1

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", tostring(self.__n_round_counter), Message.N_TUNNEL))

end

function BossSugar:normal_finish()

	if self.__n_round_counter == 0 then

		self.__c_state_machine:do_event("attack")

	end

end

-- prepare state handlers

function BossSugar:attack_damage_manipulator(damage)

	self:apply_damage(damage)

end

function BossSugar:attack_move()

	self.__c_parent:send_message(MessagePool:get_free_ex("attack_move", "", Message.N_TUNNEL))

	local function skill_animation()

		local function perform_skill()

			local msg = MessagePool:get_free()
			msg.s_name = "perform"
			msg.x_param = { number = self:get("attack_number"), config = self:get("attack_config")}
			msg.e_transmit = Message.N_TUNNEL
		 
			self:send_message(msg)

		end

		RUN_ACTION_WITH_DELAY(0.2, perform_skill)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", "喝！", Message.N_TUNNEL))

	end

	RUN_ACTION_WITH_DELAY(0.3, skill_animation)


end

function BossSugar:attack_finish()

	-- self.__c_state_machine:do_event("idle")

end

function BossSugar:on_boss_skill_finish()

	self.__c_state_machine:do_event("idle")

end

-- dead state handlers

function BossSugar:dead_damage_manipulator(damage)



end

function BossSugar:dead_move()



end

function BossSugar:dead_finish()



end

function BossSugar:apply_damage(damage)
print("apply_damage")
print(damage)
	self.__n_current_hp = self.__n_current_hp - damage
	self.__n_damage_accumulator = self.__n_damage_accumulator + damage

	self:change_boss_stage() 

	if self.__n_current_hp <= 0 then

		self.__c_state_machine:do_event("dead")

	else

		if self.__n_damage_accumulator / self:get("max_hp") * 100 >= self:get("dizzle_threshold") then
print("dizzle triggered !!!!")
			self.__c_state_machine:do_event("dizzle")

		end

	end	

end

function BossSugar:change_boss_stage()

	local percentage = math.floor(self.__n_current_hp / self:get("max_hp") * 100)


	if percentage <= self.__n_current_hp_threshold then

		local stage_config = self:get("hp_stage_config")

		for i = 1, #stage_config do

			local stage = stage_config[i]

			if percentage >= stage.percentage then

				self.__n_current_hp_threshold = stage.percentage

				local config = stage.config

				for key, value in pairs(config) do

					self:set(key, value)

				end

				if i == 3 then

					self:get_component("visual").__s_idle_animation_name = "anger"

				else

					self:get_component("visual").__s_idle_animation_name = "idle"

				end

				break

			end

		end

	end

end

function BossSugar:on_enter_dizzle()
print("dizzle enter !!!")
	self.__n_damage_accumulator = 0
	self.__n_round_counter = self:get("dizzle_duration")

	self.__f_damage_manipulator = BossSugar.dizzle_damage_manipulator
	self.__f_boss_move = BossSugar.dizzle_move
	self.__f_boss_finish = BossSugar.dizzle_finish

	-- local dizzle_str = string.format("晕 x%d", self.__n_round_counter)
	-- GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", dizzle_str, Message.N_TUNNEL))

end

function BossSugar:on_enter_idle()

	self.__n_round_counter = self:get("attack_cooldown")

	self.__f_damage_manipulator = BossSugar.normal_damage_manipulator
	self.__f_boss_move = BossSugar.normal_move
	self.__f_boss_finish = BossSugar.normal_finish

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", tostring(self.__n_round_counter), Message.N_TUNNEL))

end

function BossSugar:on_enter_dead()

	self.__f_damage_manipulator = BossSugar.dead_damage_manipulator
	self.__f_boss_move = BossSugar.dead_move
	self.__f_boss_finish = BossSugar.dead_finish

end

function BossSugar:on_enter_celebrate()

	self.__f_damage_manipulator = BossSugar.dead_damage_manipulator
	self.__f_boss_move = BossSugar.dead_move
	self.__f_boss_finish = BossSugar.dead_finish

end

function BossSugar:on_enter_attack()

	self.__f_damage_manipulator = BossSugar.attack_damage_manipulator
	self.__f_boss_move = BossSugar.attack_move
	self.__f_boss_finish = BossSugar.attack_finish

	local attack_type = self:get("attack_config").type
	local attack_number = self:get("attack_number")

	-- GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", attack_number, Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_attack_change", {attack_type = attack_type, attack_number = attack_number}, Message.N_TUNNEL))	

end

function BossSugar:on_add_score(c_msg)
print("boss_damage")

	self.__f_damage_manipulator(self, c_msg.x_param)

end

function BossSugar:on_player_round_begin(c_msg)

	if c_msg.x_param == "player" or c_msg.x_param == "item" then

		self.__n_damage_accumulator = 0

	end

end

function BossSugar:on_game_lose()

	self.__c_state_machine:do_transition("celebrate")

end

function BossSugar:boss_move_start()

	self.__f_boss_move(self)

	self:__f_boss_finish(self)

end

function BossSugar:on_config(c_msg)

	local config = c_msg.x_param

	for key, value in pairs(config) do

		self:set(key, value)

	end

end

function BossSugar:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end

function BossSugar:on_run()

	self.__c_state_machine:do_transition("idle")

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_cooldown_change", self.__n_round_counter, Message.N_TUNNEL))

end

function BossSugar:on_hit_boss(c_msg)
	print("boss damage@!!!!")
	print(c_msg.x_param)
	-- ERROR_ASSERT(c_msg.x_param > 0, "fuck@@")
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_hp_offset", -c_msg.x_param, Message.N_TUNNEL))

end

function BossSugar:on_boss_disappeared()

	self.__c_state_machine:do_event("disappear")

end

function BossSugar:init_state_machine()

	function on_enter_idle()

		self:send_message(MessagePool:get_free_ex("enter_idle", "", Message.N_TUNNEL))

	end

	function on_enter_dizzle()
		
		self:send_message(MessagePool:get_free_ex("enter_dizzle", "", Message.N_TUNNEL))

	end

	function on_enter_attack()

		self:send_message(MessagePool:get_free_ex("enter_attack", "", Message.N_TUNNEL))

	end

	function on_enter_dead()
	
		self:send_message(MessagePool:get_free_ex("enter_dead", "", Message.N_TUNNEL))

	end

	function on_enter_celebrate()
		
		self:send_message(MessagePool:get_free_ex("enter_celebrate", "", Message.N_TUNNEL))

	end

	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "idle", enter = on_enter_idle},
		{name = "dizzle", enter = on_enter_dizzle},
		{name = "attack", enter = on_enter_attack},
		{name = "dead", enter = on_enter_dead},
		{name = "celebrate", enter = on_enter_celebrate},
		{name = "disappeared"},

	})
	self.__c_state_machine:add_transition({

		{event = "dizzle", from = "idle", to = "dizzle"},
		{event = "dizzle", from = "attack", to = "dizzle"},
		{event = "dizzle", from = "dizzle", to = "dizzle"},
		{event = "attack", from = "idle", to = "attack"},
		{event = "idle", from = "dizzle", to = "idle"},
		{event = "idle", from = "attack", to = "idle"},
		{event = "dead", from = "idle", to = "dead"},
		{event = "dead", from = "dizzle", to = "dead"},
		{event = "dead", from = "attack", to = "dead"},
		{event = "disappear", from = "dead", to = "disappeared"},

	})

	self.__c_state_machine:do_set("idle")

end

function BossSugar:on_remove_finish()

	if self.__c_state_machine:get_current_state() == "dead" and not self.__n_alerted then

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_dead", "", Message.N_TUNNEL))

		self.__n_alerted = true

	end

end


BossSugar.LISTENER_CONFIG = {

	{"add_damage", {before = BossSugar.on_add_score}},
	{"round_start", {before = BossSugar.on_player_round_begin}},
	{"boss_move_start", {before = BossSugar.boss_move_start}},
	{"enter_dizzle", {before = BossSugar.on_enter_dizzle}},
	{"enter_dead", {before = BossSugar.on_enter_dead}},
	{"on_enter_celebrate", {before = BossSugar.on_enter_celebrate}},
	{"enter_idle", {before = BossSugar.on_enter_idle}},
	{"enter_attack", {before = BossSugar.on_enter_attack}},
	{"config", {before = BossSugar.on_config}},
	{"reset", {before = BossSugar.on_reset}},
	{"skill_finish", {before = BossSugar.on_skill_finish}},
	{"game_lose", {before = BossSugar.game_lose}},
	{"run", {before = BossSugar.on_run}},
	{"sugar_target_animation_finish", {before = BossSugar.on_hit_boss}},
	{"boss_skill_finish", {before = BossSugar.on_boss_skill_finish}},
	{"boss_disappeared", {before = BossSugar.on_boss_disappeared}},

	{"link_remove_finish", {before = BossSugar.on_remove_finish}},
	{"special_remove_finish", {before = BossSugar.on_remove_finish}},
	{"destroy", {before = BossSugar.on_destroy}},
	{"assemble", {before = BossSugar.on_assemble}},
	{"disassemble", {before = BossSugar.on_disassemble}},

}

BossSugar.PROPERTY_CONFIG = {
	
	{"state", "idle"},
	{"max_hp", 100},
	{"dizzle_threshold", 0.4},
	{"dizzle_duration", 1},
	{"attack_cooldown", 10},
	{"attack_number", 0},
	{"attack_config", {}},
	{"hp_stage_config", {}}

}


return BossSugar