-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameScene = require "GameEx.Components.Scene.GameScene"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local ChocolateSugarManager = class("ChocolateSugarManager", function() return GameObject:create() end)


function ChocolateSugarManager:ctor()

	self:register_listeners_with_config(ChocolateSugarManager.LISTENER_CONFIG)
	self:register_properties_with_config(ChocolateSugarManager.PROPERTY_CONFIG)

	self.__c_will_spread = true

end

function ChocolateSugarManager:on_destroy()



end

function ChocolateSugarManager:on_reset()



end

function ChocolateSugarManager:on_remove_sugar(c_msg)

	local sugar = c_msg.x_param
	local category = sugar:get_category()

	if category:has("chocolate") then

		self.__c_will_spread = false

	end

end

function ChocolateSugarManager:on_affect()

	if self.__c_will_spread then

		local param = {

			t_category_filter = {
				type = "and",
				{"chocolate", "=", true},	
			},

		}

		local msg = MessagePool:get_free()
		msg.c_sender = self
		msg.s_name = "query_filter"
		msg.x_param = param

		GameGlobals.Scene:send_message(msg)

		local select_result = msg.x_result

		local random_draw = RandomDraw:create()
		random_draw:set_target(#select_result)

		local index = random_draw:draw_next()

		while index do

			local sugar = select_result[index]

			local msg = MessagePool:get_free_ex("affect")
			sugar:send_message(msg)

			if msg.x_return_value then

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("chocolate_affect", "", Message.N_TUNNEL))
				break

			else

				index = random_draw:draw_next()

			end

		end

	end

end


function ChocolateSugarManager:on_round_start(c_msg)

	self.__c_will_spread = true

end

ChocolateSugarManager.LISTENER_CONFIG = {

	{"affect_start", {before = ChocolateSugarManager.on_affect}},
	{"remove_sugar", {before = ChocolateSugarManager.on_remove_sugar}},	
	{"remove_sugar_silent", {before = ChocolateSugarManager.on_remove_sugar}},
	{"round_start", {before = ChocolateSugarManager.on_round_start}}		

}

ChocolateSugarManager.PROPERTY_CONFIG = {
	


}

return ChocolateSugarManager