-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local CircleCollider = require "GameEx.Components.Colliders.CircleCollider"
local ColorfulSugarVisual = require "GameEx.Components.Visuals.ColorfulSugarVisual"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local ProbGetter = require "GameEx.Foundations.Utils.ProbGetter"
local RandomGenerator = require "GameEx.Foundations.Utils.RandomGenerator"

-- ## Imports ## --




local ColorfulSugar = class("ColorfulSugar", function() return GameObject:create() end)


function ColorfulSugar:ctor()

	self.__c_prob_getter = ProbGetter:create(GameGlobals.RandomGenerator)

	self:init_state_machine()

	local visual = ObjectFactory:get_instance():get("ColorfulSugarVisual")
	self:add_component(visual, "visual")

	local collider = ObjectFactory:get_instance():get("CircleCollider")
	self:add_component(collider, "collider")

	local category = self:get_category()
	category:add("colorful")

	self.__n_random_flag = false

	self:register_listeners_with_config(ColorfulSugar.LISTENER_CONFIG)
	self:register_properties_with_config(ColorfulSugar.PROPERTY_CONFIG)

end

function ColorfulSugar:on_destroy(c_msg)


end

function ColorfulSugar:on_enter()


end

function ColorfulSugar:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

	self:set("state", "normal")

end

function ColorfulSugar:on_lock()
	
	self.__c_state_machine:do_event("lock")

end

function ColorfulSugar:on_select()
	
	self.__c_state_machine:do_event("select")

end

function ColorfulSugar:on_unselect()

	self.__c_state_machine:do_event("unselect")

end

function ColorfulSugar:on_start_remove(c_msg)

	if not c_msg.n_handled then

		self.__c_state_machine:do_event("remove")

	end

end

function ColorfulSugar:on_color_defined(c_msg)

	self:get_category():remove(self:get("color"))
	self:get_category():add(c_msg.x_param)

	self:set("color", c_msg.x_param)

end

function ColorfulSugar:on_color_undefined(c_msg)
print("color undefined")

	self:get_category():remove(self:get("color"))

	self:set("color", "undefined")

end



function ColorfulSugar:on_remove_started(c_msg)

	self:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

end

function ColorfulSugar:on_finish_remove()
-- 
	self:send_message(MessagePool:get_free_ex("remove_finished", "", Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", self, Message.N_TUNNEL))

end

function ColorfulSugar:after_set_state(c_msg)

	self.__c_state_machine:do_transition(c_msg.x_param.x_new_value)

end

function ColorfulSugar:after_set_id(c_msg)

	self:setName(c_msg.x_param.x_new_value)

end

function ColorfulSugar:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end

function ColorfulSugar:init_state_machine()

	function on_enter_remove()

		self:send_message(MessagePool:get_free_ex("remove_started", "", Message.N_TUNNEL))

	end

	function on_enter_select()

		self:send_message(MessagePool:get_free_ex("selected", "", Message.N_TUNNEL))

	end

	function on_leave_select()
		
		self:send_message(MessagePool:get_free_ex("unselected", "", Message.N_TUNNEL))

	end


	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "normal", enter = on_enter_normal},
		{name = "select", enter = on_enter_select, leave = on_leave_select},
		{name = "remove", enter = on_enter_remove},
		{name = "lock"},

	})
	self.__c_state_machine:add_transition({

		{event = "lock", from = "normal", to = "lock"},
		{event = "remove", from = "normal", to = "remove"},
		{event = "remove", from = "select", to = "remove"},
		{event = "remove", from = "lock", to = "remove"},
		{event = "select", from = "normal", to = "select"},
		{event = "select", from = "lock", to = "select"},
		{event = "unselect", from = "select", to = "normal"},

	})
	self.__c_state_machine:do_set("normal")

end

ColorfulSugar.LISTENER_CONFIG = {

	{"lock", {before = ColorfulSugar.on_lock}},
	{"select", {before = ColorfulSugar.on_select}},
	{"unselect", {before = ColorfulSugar.on_unselect}},
	{"start_remove", {after = ColorfulSugar.on_start_remove}},
	{"finish_remove", {before = ColorfulSugar.on_finish_remove}},
	{"remove_started", {before = ColorfulSugar.on_remove_started}},
	{"target", {before = ColorfulSugar.on_target}},
	{"reset", {before = ColorfulSugar.on_reset}},
	{"enter", {before = ColorfulSugar.on_enter}},
	{"color_defined", {before = ColorfulSugar.on_color_defined}},
	{"color_undefined", {before = ColorfulSugar.on_color_undefined}},

}

ColorfulSugar.PROPERTY_CONFIG = {
	
	{"dynamic", true},
	{"radius", 60},
	{"state", "normal"},
	{"color", "undefined"},
	{"id", "colorful"},

}


return ColorfulSugar