-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameScene = require "GameEx.Components.Scene.GameScene"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local EdgeChainCollider = require "GameEx.Components.Colliders.EdgeChainCollider"
local CircleCollider = require "GameEx.Components.Colliders.CircleCollider"
local FenceSugarVisual = require "GameEx.Components.Visuals.FenceSugarVisual"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --




local FenceSugar = class("FenceSugar", function() return GameObject:create() end)


function FenceSugar:ctor()

	self:get_category():add("fenced")

	self:add_component(FenceSugarVisual:create(), "visual")

	self:register_listeners_with_config(FenceSugar.LISTENER_CONFIG)
	self:register_properties_with_config(FenceSugar.PROPERTY_CONFIG)

end

function FenceSugar:on_assemble(c_msg)

	self.__c_parent:setLocalZOrder(99)
	self.__c_parent:set("dynamic", false)
	self.__c_parent:addChild(self, 1)

	self.__t_position = cc.p(self.__c_parent:getPosition())

	local parent_category = self.__c_parent:get_category()
	parent_category:add("fenced")

	self:set("radius", self.__c_parent:get("radius"))
	self:set("width", self.__c_parent:get("width"))
	self:set("height", self.__c_parent:get("height"))

end

function FenceSugar:on_disassemble(c_msg)

	self.__c_parent:removeChild(self)

end

function FenceSugar:on_append_to(c_msg)

	c_msg.x_param:add_component(self, "fence")

end

function FenceSugar:on_start_remove(c_msg)

	local parent_category = self.__c_parent:get_category()
	parent_category:remove("fenced")	

	local world = self.__c_parent:getParent()
	self.__c_parent:removeFromParent()
	self.__c_parent:set("dynamic", true)
	world:addChild(self.__c_parent)

	self.__c_parent:send_message(MessagePool:get_free_ex("unselect"))
	self.__c_parent:set("state", "normal")
	self.__c_parent:remove_component("fence")

	c_msg.n_handled = true

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", self, Message.N_TUNNEL))

end

function FenceSugar:getPosition()

	return self.__t_position.x, self.__t_position.y

end

FenceSugar.LISTENER_CONFIG = {

	{"start_remove", {after = FenceSugar.on_start_remove}},
	{"append_to", {before = FenceSugar.on_append_to}},
	{"assemble", {before = FenceSugar.on_assemble}},
	{"disassemble", {before = FenceSugar.on_disassemble}},

}

FenceSugar.PROPERTY_CONFIG = {
	


}


return FenceSugar