-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local CircleCollider = require "GameEx.Components.Colliders.CircleCollider"
local KeySugarVisual = require "GameEx.Components.Visuals.KeySugarVisual"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local KeySugar = class("KeySugar", function() return GameObject:create() end)


function KeySugar:ctor()

	self:init_state_machine()

	local visual = ObjectFactory:get_instance():get("KeySugarVisual")
	self:add_component(visual, "visual")

	local collider = ObjectFactory:get_instance():get("BoxCollider")
	self:add_component(collider, "collider")

	local selector = ObjectFactory:get_instance():get("CircleSelector")
	self:add_component(selector, "selector")

	local category = self:get_category()
	category:add("key")

	self:register_listeners_with_config(KeySugar.LISTENER_CONFIG)
	self:register_properties_with_config(KeySugar.PROPERTY_CONFIG)

end

function KeySugar:on_enter()

	local entry = 0

	local function check_scheduler()

		local msg = MessagePool:get_free_ex("select", {ban_message = true}, Message.N_TUNNEL)
		self:send_message(msg)

		local select_result = msg.x_result

		for i = 1, #select_result do

			if select_result[i]:get_category():has("bottom") then

				cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.entry)
				self:send_message(MessagePool:get_free_ex("start_remove"))
				break

			end

		end

	end

	self.entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(check_scheduler, 0.1, false)

end

function KeySugar:on_exit()



end

function KeySugar:on_destroy()
	if self.entry then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.entry)
		self.entry = nil

	end

end

function KeySugar:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

	self:set("state", "normal")

end

function KeySugar:on_select()
	
	self.__c_state_machine:do_event("select")

end

function KeySugar:on_unselect()

	self.__c_state_machine:do_event("unselect")

end

function KeySugar:on_start_remove(c_msg)

	if not c_msg.n_handled then

		self.__c_state_machine:do_event("remove")

	end

end

function KeySugar:on_remove_started()

	self:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

end

function KeySugar:on_finish_remove()

	self:send_message(MessagePool:get_free_ex("remove_finished", "", Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", self, Message.N_TUNNEL))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("cake_remove", self, Message.N_TUNNEL))
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("self_remove_finish"))

end

function KeySugar:after_set_state(c_msg)

	self.__c_state_machine:do_transition(c_msg.x_param.x_new_value)

end

function KeySugar:after_set_id(c_msg)

	self:setName(c_msg.x_param.x_new_value)

end

function KeySugar:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end


function KeySugar:init_state_machine()

	function on_enter_remove()

		self:send_message(MessagePool:get_free_ex("remove_started", "", Message.N_TUNNEL))

	end

	function on_enter_select()

		self:send_message(MessagePool:get_free_ex("selected", "", Message.N_TUNNEL))

	end

	function on_leave_select()
		
		self:send_message(MessagePool:get_free_ex("unselected", "", Message.N_TUNNEL))

	end


	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "normal", enter = on_enter_normal},
		{name = "select", enter = on_enter_select, leave = on_leave_select},
		{name = "remove", enter = on_enter_remove},

	})
	self.__c_state_machine:add_transition({

		{event = "remove", from = "normal", to = "remove"},
		{event = "remove", from = "select", to = "remove"},
		{event = "select", from = "normal", to = "select"},
		{event = "unselect", from = "select", to = "normal"},

	})
	self.__c_state_machine:do_set("normal")

end


KeySugar.LISTENER_CONFIG = {

	{"start_remove", {after = KeySugar.on_start_remove}},
	{"finish_remove", {before = KeySugar.on_finish_remove}},
	{"remove_started", {before = KeySugar.on_remove_started}},
	{"enter", {before = KeySugar.on_enter}},
	{"exit", {before = KeySugar.on_exit}},
	{"reset", {before = KeySugar.on_reset}},
	{"destroy", {before = KeySugar.on_destroy}}

}

KeySugar.PROPERTY_CONFIG = {
	
	{"dynamic", true},
	{"state", "normal"},
	{"material", cc.PhysicsMaterial(0.15, 0.0, 0.0)},
	{"id", "key"},

}


return KeySugar