-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local LinkSelector = require "GameEx.Components.Selectors.LinkSelector"
local LinkRemover = require "GameEx.Components.Removers.LinkRemover"
local LinkGenerator = require "GameEx.Components.Generators.LinkGenerator"
local LinkVisual = require "GameEx.Components.Visuals.LinkVisual"
local LinkFinder = require "GameEx.Components.Finders.LinkFinder"
local LinkIndicator = require "GameEx.Components.Indicators.LinkIndicator"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --



local LinkSugar = class("LinkSugar", function() return GameObject:create() end)


function LinkSugar:ctor()

	local factory = ObjectFactory:get_instance()

	self:add_component(factory:get_instance():get("GenerateIndicator"), "generate_indicator")
	self:add_component(factory:get_instance():get("SpecialIndicator"), "special_indicator")
	self:add_component(factory:get_instance():get("FractureIndicator"), "fracture_indicator")
	self:add_component(factory:get_instance():get("LinkSelector"), "selector")
	self:add_component(factory:get_instance():get("LinkGenerator"), "generator")
	self:add_component(factory:get_instance():get("LinkVisual"), "visual")
	self:add_component(factory:get_instance():get("LinkFinder"), "finder")
	self:add_component(factory:get_instance():get("LinkIndicator"), "indicator")
	self:add_component(factory:get_instance():get("LinkRemover"), "remover")

	self:register_listeners_with_config(LinkSugar.LISTENER_CONFIG)
	self:register_properties_with_config(LinkSugar.PROPERTY_CONFIG)

end

function LinkSugar:on_select_continue(c_msg)
print("select continue")
	local result = c_msg.x_param

	local msg = MessagePool:get_free()
	msg.s_name = "update_visual"
	msg.x_param = result
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

	if #result > 0 then

		local msg = MessagePool:get_free()
		msg.s_name = "find_link"
		msg.x_param = result[#result]
		msg.e_transmit = Message.N_TUNNEL

		self:send_message(msg)

		local find_result = msg.x_result

		local msg = MessagePool:get_free()
		msg.s_name = "start_indicate"
		msg.x_param = find_result
		msg.e_transmit = Message.N_TUNNEL

		self:send_message(msg)

		local msg = MessagePool:get_free()
		msg.s_name = "condition"
		msg.x_param = result
		msg.e_transmit = Message.N_TUNNEL

		self:send_message(msg)

	end

end

function LinkSugar:on_select_finish(c_msg)

	local msg = MessagePool:get_free()
	msg.s_name = "update_visual"
	msg.x_param = {}
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

	local msg = MessagePool:get_free()
	msg.s_name = "stop_indicate"
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

	local msg = MessagePool:get_free()
	msg.s_name = "assign_remove_task"
	msg.x_param = c_msg.x_param
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

end

function LinkSugar:on_select_cancel(c_msg)

	local msg = MessagePool:get_free()
	msg.s_name = "stop_indicate"
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

	local result = c_msg.x_param

	local msg = MessagePool:get_free()
	msg.s_name = "update_visual"
	msg.x_param = {}
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

end

function LinkSugar:on_link_remove_finish(c_msg)

	local msg = MessagePool:get_free()
	msg.s_name = "generate"
	msg.x_param = c_msg.x_param
	msg.e_transmit = Message.N_TUNNEL

	self:send_message(msg)

end


LinkSugar.LISTENER_CONFIG = {

	{"select_continue", {before = LinkSugar.on_select_continue}},
	{"select_finish", {before = LinkSugar.on_select_finish}},
	{"select_cancel", {before = LinkSugar.on_select_cancel}},
	{"remove_finish", {before = LinkSugar.on_link_remove_finish}},

}

LinkSugar.PROPERTY_CONFIG = {
	


}


return LinkSugar