-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local CircleCollider = require "GameEx.Components.Colliders.CircleCollider"
local KeySugarVisual = require "GameEx.Components.Visuals.KeySugarVisual"
local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"

-- ## Imports ## --




local PandaSugar = class("PandaSugar", function() return GameObject:create() end)


function PandaSugar:ctor()

	self:init_state_machine()

	local visual = ObjectFactory:get_instance():get("PandaSugarVisual")
	self:add_component(visual, "visual")

	local collider = ObjectFactory:get_instance():get("CircleCollider")
	self:add_component(collider, "collider")

	local category = self:get_category()
	category:add("panda")

	self:register_listeners_with_config(PandaSugar.LISTENER_CONFIG)
	self:register_properties_with_config(PandaSugar.PROPERTY_CONFIG)

end

function PandaSugar:on_destroy()


end

function PandaSugar:on_reset(c_msg)

	local default_config = c_msg.x_param

	for key, value in pairs(default_config) do

		self:set(key, value)

	end

	self:set("state", "normal")

end


function PandaSugar:on_start_remove(c_msg)

	if not c_msg.n_handled then

		self.__c_state_machine:do_event("remove")

	end

end

function PandaSugar:on_remove_started()

	self:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

end

function PandaSugar:on_finish_remove()

	self:send_message(MessagePool:get_free_ex("remove_finished", "", Message.N_TUNNEL))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("blast_finale", self, Message.N_TUNNEL))
	
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("remove_sugar", self, Message.N_TUNNEL))

end

function PandaSugar:after_set_state(c_msg)

	self.__c_state_machine:do_transition(c_msg.x_param.x_new_value)

end

function PandaSugar:after_set_id(c_msg)

	self:setName(c_msg.x_param.x_new_value)

end

function PandaSugar:after_get_state(c_msg)

	c_msg.x_param.x_return_value = self.__c_state_machine:get_current_state()

end


function PandaSugar:init_state_machine()

	function on_enter_remove()

		self:send_message(MessagePool:get_free_ex("remove_started", "", Message.N_TUNNEL))

	end

	function on_enter_select()

		self:send_message(MessagePool:get_free_ex("selected", "", Message.N_TUNNEL))

	end

	function on_leave_select()
		
		self:send_message(MessagePool:get_free_ex("unselected", "", Message.N_TUNNEL))

	end


	self.__c_state_machine = StateMachine:create()
	self.__c_state_machine:add_state({

		{name = "normal", enter = on_enter_normal},
		{name = "select", enter = on_enter_select, leave = on_leave_select},
		{name = "remove", enter = on_enter_remove},

	})
	self.__c_state_machine:add_transition({

		{event = "remove", from = "normal", to = "remove"},
		{event = "remove", from = "select", to = "remove"},
		{event = "select", from = "normal", to = "select"},
		{event = "unselect", from = "select", to = "normal"},

	})
	self.__c_state_machine:do_set("normal")

end


PandaSugar.LISTENER_CONFIG = {

	{"start_remove", {after = PandaSugar.on_start_remove}},
	{"finish_remove", {before = PandaSugar.on_finish_remove}},
	{"remove_started", {before = PandaSugar.on_remove_started}},
	{"reset", {before = PandaSugar.on_reset}},


}

PandaSugar.PROPERTY_CONFIG = {
	
	{"dynamic", true},
	{"state", "normal"},
	{"id", "panda"}

}


return PandaSugar