-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local GameScene = require "GameEx.Components.Scene.GameScene"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local GameGlobals = require "GameEx.GameGlobals"
local RandomDraw = require "GameEx.Foundations.Utils.RandomDrawArray"

-- ## Imports ## --




local RewardBagSugarManager = class("RewardBagSugarManager", function() return GameObject:create() end)


function RewardBagSugarManager:ctor()

	self:register_listeners_with_config(RewardBagSugarManager.LISTENER_CONFIG)
	self:register_properties_with_config(RewardBagSugarManager.PROPERTY_CONFIG)


	local current_level = Player:getInstance():get_player_current_level()
	self.__c_will_drop = false

	local bag_config_vector = nil
	self.bag_config = nil

	-- pcall(function() bag_config_vector = require("GameEx.Resources.Levels.bag_config") end)
	bag_config_vector = require("GameEx.Resources.Levels.bag_config")

	if bag_config_vector then

		self.bag_config = bag_config_vector[current_level]

	end

end

function RewardBagSugarManager:on_destroy()



end

function RewardBagSugarManager:on_reset()



end

function RewardBagSugarManager:on_remove_sugar(c_msg)

	local sugar = c_msg.x_param
	local category = sugar:get_category()

	if category:has("reward_bag") and not GameGlobals.bonus then
		-- RUN_ACTION_WITH_DELAY(0.033, function()

		-- 	cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_reward_bag_enven)
		-- 	cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.open_reward_bag_enven)
		-- end)
		
		
		GameGlobals.open_reward_bag = true
		-- cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_reward_bag_enven)
		-- cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.open_reward_bag_enven)

	end

end

function RewardBagSugarManager:on_eat_candy_start()

	

end


function RewardBagSugarManager:on_round_start(c_msg)


end

function RewardBagSugarManager:on_drop_bag(c_msg)

	if self.bag_config and not self.__c_will_drop then

		math.randomseed(os.time())
	    local random_num = math.random()
	    print(random_num) --(0,1)
	    local random_table = {
	    	[5] = 0.5,
	    	[4] = 0.65,
	    	[3] = 0.8,
	    	[2]	= 0.9,
	    	[1] = 1.05,
	    }

	 	if c_msg.x_param <= self.bag_config.left_step then
	 	
	 		if random_num <= random_table[self.bag_config.left_step] then

				self.__c_will_drop = true
			
				GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

		        local visible_size = cc.Director:getInstance():getVisibleSize()
		        local target_position = cc.p(visible_size.width/2,visible_size.height)

		        local new_sugar = ObjectFactory:get_instance():get("RewardBagSugar")
		        new_sugar:setPosition(target_position)
		        GameGlobals.Scene:send_message(MessagePool:get_free_ex("add_sugar_without_count", new_sugar, Message.N_TUNNEL))
			        
			    GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))
			end

		end

	end

end

RewardBagSugarManager.LISTENER_CONFIG = {

	{"eat_candy_start", {before = RewardBagSugarManager.on_eat_candy_start}},
	{"remove_sugar", {before = RewardBagSugarManager.on_remove_sugar}},	
	{"remove_sugar_silent", {before = RewardBagSugarManager.on_remove_sugar}},
	{"round_start", {before = RewardBagSugarManager.on_round_start}},

	{"drop_bag", {before = RewardBagSugarManager.on_drop_bag}},

}

RewardBagSugarManager.PROPERTY_CONFIG = {
	


}

return RewardBagSugarManager