-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local AddMoveSugarVisual = class("AddMoveSugarVisual", function() return GameObject:create() end)

function AddMoveSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("add_move_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_106_2", true)
	self:addChild(self.__c_spine)


	self:register_listeners_with_config(AddMoveSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(AddMoveSugarVisual.PROPERTY_CONFIG)

end

function AddMoveSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function AddMoveSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function AddMoveSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function AddMoveSugarVisual:on_select()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, "barrier_106_touch", false)
	self.__c_spine:addAnimation(0, "barrier_106_hold", true)

end

function AddMoveSugarVisual:on_unselect()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, "barrier_106_2", true)

end

function AddMoveSugarVisual:on_start_remove()

	self:set_skin(self:get("color"), "")

	local node_position = cc.p(self:getPosition())
	local world_position = self:getParent():convertToWorldSpace(node_position)

   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_1")
   	particle_1:setLocalZOrder(9)
   	particle_1:setPosition(world_position)
   	particle_1:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_2")
   	particle_2:setLocalZOrder(9)
   	particle_2:setPosition(world_position)
   	particle_2:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_3")
   	particle_3:setLocalZOrder(9)
   	particle_3:setPosition(world_position)
   	particle_3:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

   	self.__c_parent:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

end

function AddMoveSugarVisual:on_target()



end

function AddMoveSugarVisual:on_finish_remove()



end


function AddMoveSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:addAnimation(1, "barrier_106_tishi", true)

end

function AddMoveSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:addAnimation(1, "barrier_106_tishi_end", false)

end

function AddMoveSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 64)

end

function AddMoveSugarVisual:after_set_color(c_msg)

	self:set_skin(c_msg.x_param.x_new_value, "")

end

function AddMoveSugarVisual:set_skin(color, suffix)

	local str = string.format("%s%s", color, suffix)

	self.__c_spine:setSkin(str)

end

AddMoveSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = AddMoveSugarVisual.on_assemble}},
	{"disassemble", {before = AddMoveSugarVisual.on_disassemble}},
	{"selected", {before = AddMoveSugarVisual.on_select}},
	{"unselected", {before = AddMoveSugarVisual.on_unselect}},
	{"remove_started", {before = AddMoveSugarVisual.on_start_remove}},
	{"remove_finished", {before = AddMoveSugarVisual.on_finish_remove}},
	{"highlight_on", {before = AddMoveSugarVisual.on_highlight_on}},
	{"highlight_off", {before = AddMoveSugarVisual.on_highlight_off}},
	{"target", {before = AddMoveSugarVisual.on_target}},
	{"destroy", {before = AddMoveSugarVisual.on_destroy}},

}

AddMoveSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 1},
	{"color", "red"},

}

return AddMoveSugarVisual