-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local BarrierSugarVisual = class("BarrierSugarVisual", function() return GameObject:create() end)

function BarrierSugarVisual:ctor()

	self.__c_sprite = ResourceManager:get_instance():get_resource_by_key("barrier_60", true)
	self.__c_sprite:retain()

	self:register_listeners_with_config(BarrierSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(BarrierSugarVisual.PROPERTY_CONFIG)

end

function BarrierSugarVisual:on_destroy()

	self.__c_sprite:release()

end

function BarrierSugarVisual:on_reset()



end

function BarrierSugarVisual:on_assemble()

	self.__c_parent:addChild(self.__c_sprite)

end

function BarrierSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self.__c_sprite)

end

function BarrierSugarVisual:after_set_texture(c_msg)

	local sprite_key = c_msg.x_param.x_new_value

	local new_sprite = ResourceManager:get_instance():get_resource_by_key(sprite_key)
	new_sprite:retain()

	if self.__c_parent then 

		self.__c_parent:removeChild(self.__c_sprite)
		self.__c_sprite:release()
		self.__c_parent:addChild(new_sprite)

	end

	self.__c_sprite = new_sprite

end


BarrierSugarVisual.LISTENER_CONFIG = {

	{"destroy", {before = BarrierSugarVisual.on_destroy}},

}

BarrierSugarVisual.PROPERTY_CONFIG = {
	
	{"texture", "barrier_60"},

}

return BarrierSugarVisual