-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManagerEx = require "GameEx.Foundations.ResourceManager"
local ResourceManager = require "core.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
-- ## Imports ## --

local BossSugarVisual = class("BossSugarVisual", function() return GameObject:create() end)

function BossSugarVisual:ctor()

	self.__s_idle_animation_name = "idle"

	self.__c_spine = ResourceManagerEx:get_instance():get_resource_by_key("boss_1", true)
	self.__c_spine:setAnimation(0, __s_idle_animation_name, true)
	self.__c_spine:setScale(2)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(BossSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(BossSugarVisual.PROPERTY_CONFIG)

	self.__s_last_animation_name = self.__s_idle_animation_name

	self.__n_dead = false

	self.__s_hurt_action_name = "hurt"

	self:setVisible(false)

end

function BossSugarVisual:on_destroy()
print("boss visual remove")
	self.__c_spine:removeFromParent()

end

function BossSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function BossSugarVisual:on_disassemble()

	self:removeFromParent()

end

function BossSugarVisual:on_start_remove()

	

end

function BossSugarVisual:on_finish_remove()



end

function BossSugarVisual:on_hit_boss()
print("hit boss")	

	if not self.__n_dead then

		self.__c_spine:setAnimation(0, self.__s_hurt_action_name, false)

	else

		self.__c_spine:setAnimation(0, self.__s_hurt_action_name, false)

	end

end

function BossSugarVisual:after_set_width(c_msg)

	self:setScaleX(c_msg.x_param.x_new_value / 200)

end

function BossSugarVisual:after_set_height(c_msg)

	self:setScaleY(c_msg.x_param.x_new_value / 200)

end

function BossSugarVisual:on_enter_dizzle()

	self.__s_last_animation_name = "halo"

end

function BossSugarVisual:on_enter_dead()

	self.__n_dead = true

end

function BossSugarVisual:on_animation_end(c_msg)

	if self.__n_dead and not self.__n_dead_animation_finish then
print("adnimation end !!!!!!!!!!!!!")
		MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Bossdead_Audio"), false)
		self.__c_spine:addAnimation(0, "lose", false)
		self.__n_dead_animation_finish = true

		local action = cc.Sequence:create(cc.DelayTime:create(0.2), cc.CallFunc:create(function()

			local smoke = ResourceManagerEx:get_instance():get_resource_by_key("boss_smoke_spine")
			smoke:setPosition(cc.p(0, 0))
			smoke:setAnimation(0, "smoke_1", false)
			smoke:setScale(2)

			self:addChild(smoke, 7)

			local sequence = cc.Sequence:create(cc.DelayTime:create(20 * 0.033), cc.RemoveSelf:create())

			smoke:runAction(sequence)

		end))

		self:runAction(action)

		RUN_ACTION_WITH_DELAY(1.2, function()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_disappeared", self, Message.N_TUNNEL))

		end)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("boss_disappeared", self, Message.N_TUNNEL))

	elseif not self.__n_dead then

		self.__c_spine:addAnimation(0, self.__s_last_animation_name, true)

		if self.__s_last_animation_name == "halo" then
			MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Xuanyun_Audio"), false)
		end

	end

end

function BossSugarVisual:on_enter_idle()

	self.__c_spine:setAnimation(0, self.__s_idle_animation_name, true)
	self.__s_last_animation_name = self.__s_idle_animation_name

end

function BossSugarVisual:on_enter_attack()

	self.__c_spine:setAnimation(0, "attack_1", false)
	self.__c_spine:addAnimation(0, "attack_2", true)
	self.__s_last_animation_name = "attack_2"

	TRIGGER_EVENT("BOSS_ATTACK_PREPARE")

end

function BossSugarVisual:on_attack_move()
print("visual move")
	local delay = cc.DelayTime:create(0.3)
	local call = cc.CallFunc:create(function()

		MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Bossskill_Audio"), false)
		self.__c_spine:setAnimation(0, "attack_3", false)
		self.__c_spine:addAnimation(0, self.__s_idle_animation_name, true)

	end)
	local sequence = cc.Sequence:create(delay, call)

	GameGlobals.Scene:runAction(sequence)

end

function BossSugarVisual:after_set_texture(c_msg)

	self.__c_spine:removeFromParent()

	self.__c_spine = ResourceManagerEx:get_instance():get_resource_by_key(c_msg.x_param.x_new_value, true)
	self.__c_spine:setAnimation(0, self.__s_idle_animation_name, true)
	self.__c_spine:setScale(2)
	self:addChild(self.__c_spine)	

end

function BossSugarVisual:on_link_remove_start()

	self.__s_hurt_action_name = "hurt"

end

function BossSugarVisual:on_special_remove_finish()

	self.__s_hurt_action_name = "hurt_heavy"

end

function BossSugarVisual:on_start_animation_finish()

	local smoke = ResourceManagerEx:get_instance():get_resource_by_key("boss_smoke_spine")
	smoke:setPosition(cc.p(0, 0))
	smoke:setAnimation(0, "smoke", false)
	smoke:setScale(2)

	self:addChild(smoke, 7)

	local function show()

		self:setVisible(true)

	end	

	smoke:runAction(cc.Sequence:create(cc.DelayTime:create(6 * 0.033), cc.CallFunc:create(show), cc.DelayTime:create(20 * 0.033), cc.RemoveSelf:create()))

end

BossSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = BossSugarVisual.on_assemble}},
	{"disassemble", {before = BossSugarVisual.on_disassemble}},
	{"remove_started", {before = BossSugarVisual.on_start_remove}},
	{"remove_finished", {before = BossSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = BossSugarVisual.on_layer_reduce}},
	{"all_target_animation_finish", {before = BossSugarVisual.on_animation_end}},
	{"sugar_target_animation_finish", {before = BossSugarVisual.on_hit_boss}},
	{"enter_dizzle", {before = BossSugarVisual.on_enter_dizzle}},
	{"boss_move_start", {before = BossSugarVisual.boss_move_start}},
	{"link_remove_start", {before = BossSugarVisual.on_link_remove_start}},
	{"special_remove_finish", {before = BossSugarVisual.on_special_remove_finish}},
	{"start_animation_finish", {before = BossSugarVisual.on_start_animation_finish}},		

	{"enter_dead", {before = BossSugarVisual.on_enter_dead}},
	{"enter_idle", {before = BossSugarVisual.on_enter_idle}},
	{"enter_attack", {before = BossSugarVisual.on_enter_attack}},
	{"attack_move", {before = BossSugarVisual.on_attack_move}},
	{"destroy", {before = BossSugarVisual.on_destroy}},

}

BossSugarVisual.PROPERTY_CONFIG = {
	
	{"width", 100},
	{"height", 100},
	{"texture", "boss_1"},

}

return BossSugarVisual