-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local ChocolateSugarVisual = class("ChocolateSugarVisual", function() return GameObject:create() end)

function ChocolateSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("chocolate_sugar_spine", true)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(ChocolateSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(ChocolateSugarVisual.PROPERTY_CONFIG)

end

function ChocolateSugarVisual:on_reset()

	self.__c_spine:setAnimation(0, "barrier_102_2", false)
	self.__c_spine:addAnimation(0, "barrier_102", true)
	self.__c_spine:setAnimation(1, "barrier_102_tishi_end", false)

end

function ChocolateSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function ChocolateSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function ChocolateSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function ChocolateSugarVisual:on_start_remove()

	local particle = ResourceManager:get_instance():get_resource_by_key("chocolate_remove_particle")
	particle:setAutoRemoveOnFinish(true)
	particle:setPosition(cc.p(self.__c_parent:getPosition()))
	particle:setScale(1.2)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

end

function ChocolateSugarVisual:on_finish_remove()



end


function ChocolateSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 65)

end


function ChocolateSugarVisual:after_set_layers(c_msg)



end

function ChocolateSugarVisual:on_layer_reduce(c_msg)



end

function ChocolateSugarVisual:on_enter()

	self.__c_spine:setAnimation(0, "barrier_102_2", false)
	self.__c_spine:addAnimation(0, "barrier_102", true)

end

function ChocolateSugarVisual:on_affect_animation(c_msg)

	local sugar = c_msg.x_param
	local sugar_position = cc.p(sugar:getPosition())
	local chocolate_position = cc.p(self.__c_parent:getPosition())
	local direction = UtilFunction:caculate_direction(sugar_position, chocolate_position)
	local angle = UtilFunction:caculate_angle(direction)

	self:setRotation(angle)

	self.__c_spine:setAnimation(0, "barrier_102_1", false)
	self.__c_spine:addAnimation(0, "barrier_102", true)

	local spine = nil

	if sugar:get_category():has("transform") then

		spine = ResourceManager:get_instance():get_resource_by_key("transform_sugar_spine")

	else

		spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine")

	end

	spine:setSkin(sugar:get("color"))
	spine:setScale(60 / 64)
	spine:setPosition(cc.p(sugar:getPosition()))
	spine:setLocalZOrder(-1)

	sugar:getParent():addChild(spine)

	local delay = cc.DelayTime:create(22 * 0.033)
	local remove_self = cc.RemoveSelf:create()
	local sequence = cc.Sequence:create(delay, remove_self)
	spine:runAction(sequence)

end

function ChocolateSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(1, "barrier_102_tishi", true)

end

function ChocolateSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(1, "barrier_102_tishi_end", false)

end

ChocolateSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = ChocolateSugarVisual.on_assemble}},
	{"disassemble", {before = ChocolateSugarVisual.on_disassemble}},
	{"remove_started", {before = ChocolateSugarVisual.on_start_remove}},
	{"remove_finished", {before = ChocolateSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = ChocolateSugarVisual.on_layer_reduce}},
	{"affect_animation", {before = ChocolateSugarVisual.on_affect_animation}},
	{"enter", {before = ChocolateSugarVisual.on_enter}},
	{"destroy", {before = ChocolateSugarVisual.on_destroy}},
	{"highlight_on", {before = ChocolateSugarVisual.on_highlight_on}},
	{"highlight_off", {before = ChocolateSugarVisual.on_highlight_off}},
	{"reset", {before = ChocolateSugarVisual.on_reset}}

}

ChocolateSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 60},
	{"layers", 1},

}

return ChocolateSugarVisual