-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager_Core = require "core.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local CocoSugarVisual = class("CocoSugarVisual", function() return GameObject:create() end)

function CocoSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("coco_sugar_spine", true)
	self:addChild(self.__c_spine, 1)

	self:register_listeners_with_config(CocoSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(CocoSugarVisual.PROPERTY_CONFIG)

	self.__n_reduce_count = 0
	self.__n_animation_counter = 0
	self.__n_sound_counter = 0

end

function CocoSugarVisual:on_reset()

	self.__n_reduce_count = 0
	self.__c_spine:setAnimation(0, "normal", false)
	self.__n_animation_counter = 0
	self.__n_animation_timer = nil
	self.__n_remove_flag = false

end

function CocoSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()
	self:finish_animation_timer()

end


function CocoSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function CocoSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function CocoSugarVisual:on_start_remove()
	
	self.__c_spine:addAnimation(0, "empty", false)
	-- self.__n_animation_counter = self.__n_animation_counter + 0.3

	self.__n_remove_flag = true

end

function CocoSugarVisual:on_finish_remove()



end


function CocoSugarVisual:after_set_width(c_msg)

	-- self:setScaleX(c_msg.x_param.x_new_value / 120)

end

function CocoSugarVisual:after_set_height(c_msg)

	-- self:setScaleY(c_msg.x_param.x_new_value / 120)

end

function CocoSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 132)

end

function CocoSugarVisual:on_accumulate(c_msg)



end

function CocoSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi", true)

end

function CocoSugarVisual:on_highlight_off()

	self.on = false
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi_end", false)

end

function CocoSugarVisual:on_layer_reduce(c_msg)
	

	local original_layers = c_msg.x_param
	local current_layers = self:get("layers")

	for animation_index = original_layers, current_layers + 1, -1 do

		if animation_index > 0 then

			self.__c_spine:addAnimation(0, tostring(animation_index), false)
			self.__n_animation_counter = self.__n_animation_counter + 0.3

		end

	end

	self:start_animation_timer()

end

function CocoSugarVisual:start_animation_timer()

	if not self.__n_animation_timer then

		self.__n_sound_counter = 0

		local schedule_func = function(dt)

			self.__n_animation_counter = self.__n_animation_counter - 0.033
			self.__n_sound_counter = self.__n_sound_counter - 0.033

			if self.__n_animation_counter <= 0 then

				self.__n_animation_counter = 0
				self:finish_animation_timer()

			end

			if self.__n_sound_counter <= 0 then
print("sound")
				self.__n_sound_counter = 0.3
				GameGlobals.Scene:send_message(MessagePool:get_free_ex("coco_break", "", Message.N_TUNNEL))

			end

		end

		self.__n_animation_timer = cc.Director:getInstance():getScheduler():scheduleScriptFunc(schedule_func, 0.033, false)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	end

end

function CocoSugarVisual:finish_animation_timer()

	if self.__n_animation_timer then

		cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_animation_timer)
		self.__n_animation_timer = nil

		if self.__n_remove_flag then

			self.__n_remove_flag = false
			self.__c_parent:send_message(MessagePool:get_free_ex("finish_remove"))

		else

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

	end

end

function CocoSugarVisual:on_exit()

	self:finish_animation_timer()

end

CocoSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = CocoSugarVisual.on_assemble}},
	{"disassemble", {before = CocoSugarVisual.on_disassemble}},
	{"remove_started", {before = CocoSugarVisual.on_start_remove}},
	{"remove_finished", {before = CocoSugarVisual.on_finish_remove}},
	{"reset", {before = CocoSugarVisual.on_reset}},
	{"destroy", {before = CocoSugarVisual.on_destroy}},
	{"highlight_on", {before = CocoSugarVisual.on_highlight_on}},
	{"highlight_off", {before = CocoSugarVisual.on_highlight_off}},
	{"accumulate", {before = CocoSugarVisual.on_accumulate}},
	{"layer_reduce", {before = CocoSugarVisual.on_layer_reduce}},
	{"exit", {before = CocoSugarVisual.on_exit}},

}

CocoSugarVisual.PROPERTY_CONFIG = {
	
	{"width", 120},
	{"height", 120},
	{"radius", 124},
	{"layers", 10}

}

return CocoSugarVisual