-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
-- ## Imports ## --

local ConchSkillVisual = class("ConchSkillVisual", function() return GameObject:create() end)

function ConchSkillVisual:ctor()

	self:setPosition(cc.p(880, 1630))

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("conch_skill_spine", true)
	self.__c_spine:setAnimation(0, "shell_1", true)
	self.__c_spine:setPosition(cc.p(self.__c_spine:getContentSize().width / 2, self.__c_spine:getContentSize().height / 2))
	self.__c_spine:setScale(1)
	self.__c_spine:setPosition(cc.p(-50, -50))
	self:addChild(self.__c_spine, 1)

	local widget = ccui.Widget:create()
	widget:setContentSize(cc.size(400, 300))
	self:addChild(widget, 1)

	self.__c_wrapper = ButtonWrap:create()
	self.__c_wrapper:wrap(widget, ButtonWrap.Action_Type.NORMAL)
	self.__c_wrapper:addTouchEndCallback(function()

		self.__c_parent:send_message(MessagePool:get_free_ex("conch_skill_start", ""))

	end)

    local progress_bar = cc.ProgressTimer:create(new_sprite)
    progress_bar:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    progress_bar:setPosition(cc.p(-45, -145))
    progress_bar:setSprite(ResourceManager:get_instance():get_resource_by_key("skill_progress_bar", true))
    progress_bar:setMidpoint(cc.p(0, 0.5))
    progress_bar:setBarChangeRate(cc.p(1.0, 0))
    self:addChild(progress_bar, 2)
    progress_bar:setPercentage(0)

    self.progress_bar = progress_bar

    local progress_background = ResourceManager:get_instance():get_resource_by_key("skill_progress_frame", true)
    progress_background:setPosition(cc.p(-45, -150))
    progress_background:setScaleX(1.03)
    progress_background:setScaleY(1.07)
    progress_background:setOpacity(180)
    self:addChild(progress_background, 1)

	self:register_listeners_with_config(ConchSkillVisual.LISTENER_CONFIG)
	self:register_properties_with_config(ConchSkillVisual.PROPERTY_CONFIG)

end

function ConchSkillVisual:on_destroy()

	self.__c_spine:removeFromParent()

end

function ConchSkillVisual:on_assemble()

	GameGlobals.Scene.__c_scene:addChild(self, 6)

end

function ConchSkillVisual:on_disassemble()

	self:removeFromParent()

end

function ConchSkillVisual:on_update_fill(c_msg)

	local phase = c_msg.x_param * 100
	
	if phase >= 0 and phase < 25 then

		self.__c_spine:setAnimation(0, "shell_1", false)
		self.__n_full = false

	elseif phase >= 25 and phase < 50 then

		self.__c_spine:setAnimation(0, "shell_2", false)
		self.__n_full = false

	elseif phase >= 50 and phase < 75 then

		self.__c_spine:setAnimation(0, "shell_3", false)
		self.__n_full = false

	elseif phase >= 75 and phase < 100 then

		self.__c_spine:setAnimation(0, "shell_4", false)
		self.__n_full = false

	elseif phase >= 100 and not self.__n_full then

		self.__c_spine:setAnimation(0, "shell_5", true)
		self.__n_full = true

	end

	self.__c_spine:setAnimation(1, "pop", false)

	self.__c_spine:setAnimation(0, "shell", false)
	self.progress_bar:runAction(cc.ProgressTo:create(0.1, c_msg.x_param * 100))

end

function ConchSkillVisual:on_enable()

	self.__c_wrapper:enable()

end

function ConchSkillVisual:on_disable()

	self.__c_wrapper:disable()

end


function ConchSkillVisual:on_round_start(c_msg)

    self:on_disable()

end

function ConchSkillVisual:on_round_finish(c_msg)

    self:on_enable()

end

ConchSkillVisual.LISTENER_CONFIG = {

	{"assemble", {before = ConchSkillVisual.on_assemble}},
	{"disassemble", {before = ConchSkillVisual.on_disassemble}},
	{"update_fill", {before = ConchSkillVisual.on_update_fill}},
	{"destroy", {before = ConchSkillVisual.on_destroy}},
	{"enable", {before = ConchSkillVisual.on_enable}},
	{"disable", {before = ConchSkillVisual.on_disable}},
	{"round_start", {before = ConchSkillVisual.on_round_start}},
	{"round_finish", {before = ConchSkillVisual.on_round_finish}},

}

ConchSkillVisual.PROPERTY_CONFIG = {
	


}

return ConchSkillVisual