-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local FenceSugarVisual = class("FenceSugarVisual", function() return GameObject:create() end)

function FenceSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("fence_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_103", true)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(FenceSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(FenceSugarVisual.PROPERTY_CONFIG)

end

function FenceSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end

function FenceSugarVisual:on_append_to()

	self.__c_spine:setAnimation(0, "barrier_103", false)

end

function FenceSugarVisual:on_start_remove(c_msg)

	local node_position = cc.p(self:getPosition())
	local world_position = self:getParent():getParent():convertToWorldSpace(node_position)

   	local effect = ResourceManager:get_instance():get_resource_by_key("fence_sugar_spine")
   	effect:setPosition(world_position)
   	effect:setAnimation(0, "barrier_103_2", false)
   	effect:setScale(self:getScale())
	effect:registerSpineEventHandler(function (event)

		effect:registerSpineEventHandler(function(event) end, sp.EventType.ANIMATION_END)

		local sequence = cc.Sequence:create(cc.DelayTime:create(0), cc.RemoveSelf:create())

		effect:runAction(sequence)

	end, sp.EventType.ANIMATION_END)

   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", effect))

end

function FenceSugarVisual:on_assemble()

	self.__c_parent:addChild(self, 1)

end

function FenceSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function FenceSugarVisual:after_set_radius(c_msg)

	if c_msg.x_param.x_new_value then

		self:setScale(c_msg.x_param.x_new_value / 64)

	end

end

function FenceSugarVisual:after_set_width(c_msg)

	if c_msg.x_param.x_new_value then

		self:setScale(c_msg.x_param.x_new_value / 128)

	end

end

function FenceSugarVisual:after_set_height(c_msg)

	if c_msg.x_param.x_new_value then

		self:setScale(c_msg.x_param.x_new_value / 128)

	end

end

FenceSugarVisual.LISTENER_CONFIG = {

	{"start_remove", {before = FenceSugarVisual.on_start_remove}},
	{"append_to", {before = FenceSugarVisual.on_append_to}},
	{"assemble", {before = FenceSugarVisual.on_assemble}},
	{"disassemble", {before = FenceSugarVisual.on_disassemble}},
	{"destroy", {before = FenceSugarVisual.on_destroy}},

}

FenceSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 60}

}

return FenceSugarVisual