-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager_Core = require "core.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local GiftSugarVisual = class("GiftSugarVisual", function() return GameObject:create() end)

function GiftSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("gift_sugar_spine", true)
	self.__c_spine:setAnimation(1, "barrier_112", true)
	self:addChild(self.__c_spine, 1)

	self:register_listeners_with_config(GiftSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(GiftSugarVisual.PROPERTY_CONFIG)

	self.__n_reduce_count = 0

    local progress_bar = cc.ProgressTimer:create(new_sprite)
    progress_bar:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    progress_bar:setPosition(cc.p(0, -80))
    progress_bar:setSprite(ResourceManager:get_instance():get_resource_by_key("conch_progress_bar", true))
    progress_bar:setScale(0.8)
    progress_bar:setMidpoint(cc.p(0, 0.5))
    progress_bar:setBarChangeRate(cc.p(1.0, 0))
    self:addChild(progress_bar, 2)
    -- progress_bar:setPercentage(100)

    self.progress_bar = progress_bar

    local progress_background = ResourceManager:get_instance():get_resource_by_key("conch_progress_frame", true)
    progress_background:setPosition(cc.p(0, -80))
    progress_background:setScale(0.82)
    progress_background:setOpacity(180)
    self:addChild(progress_background, 1)

    self.progress_background = progress_background

end

function GiftSugarVisual:on_reset()

	self.__n_reduce_count = 0

	self.progress_bar:setVisible(true)
	self.progress_background:setVisible(true)

end

function GiftSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function GiftSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function GiftSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function GiftSugarVisual:on_start_remove()
	MusicManager:getInstance().play_effect(ResourceManager_Core:getInstance():getAudioPathByKey("Baiskill_Audio"), false)
	
	self.__c_spine:setAnimation(1, "barrier_112_disappear", false)

	self.progress_bar:setVisible(false)
	self.progress_background:setVisible(false)

end

function GiftSugarVisual:on_finish_remove()



end


function GiftSugarVisual:after_set_width(c_msg)

	-- self:setScaleX(c_msg.x_param.x_new_value / 120)

end

function GiftSugarVisual:after_set_height(c_msg)

	-- self:setScaleY(c_msg.x_param.x_new_value / 120)

end

function GiftSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 132)

end

function GiftSugarVisual:on_accumulate(c_msg)

	-- self.progress_bar:setPercentage(c_msg.x_param)

	self.progress_bar:runAction(cc.ProgressTo:create(0.1, c_msg.x_param))

	self.__c_spine:setAnimation(1, "barrier_hit", false)
	self.__c_spine:addAnimation(1, "barrier_112", true)

end

function GiftSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi", true)

end

function GiftSugarVisual:on_highlight_off()

	self.on = false
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi_end", false)

end

GiftSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = GiftSugarVisual.on_assemble}},
	{"disassemble", {before = GiftSugarVisual.on_disassemble}},
	{"remove_started", {before = GiftSugarVisual.on_start_remove}},
	{"remove_finished", {before = GiftSugarVisual.on_finish_remove}},
	{"reset", {before = GiftSugarVisual.on_reset}},
	{"destroy", {before = GiftSugarVisual.on_destroy}},
	{"highlight_on", {before = GiftSugarVisual.on_highlight_on}},
	{"highlight_off", {before = GiftSugarVisual.on_highlight_off}},
	{"accumulate", {before = GiftSugarVisual.on_accumulate}},

}

GiftSugarVisual.PROPERTY_CONFIG = {
	
	{"width", 120},
	{"height", 120},
	{"radius", 124},

}

return GiftSugarVisual