-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local IceSugarVisual = class("IceSugarVisual", function() return GameObject:create() end)

function IceSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("ice_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_101", true)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(IceSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(IceSugarVisual.PROPERTY_CONFIG)

	self.__n_reduce_count = 0

end

function IceSugarVisual:on_reset()

	self.__n_reduce_count = 0

end

function IceSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function IceSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function IceSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function IceSugarVisual:on_start_remove()

	

end

function IceSugarVisual:on_finish_remove()



end


function IceSugarVisual:after_set_width(c_msg)

	self:setScaleX(c_msg.x_param.x_new_value / 120)

end

function IceSugarVisual:after_set_height(c_msg)

	self:setScaleY(c_msg.x_param.x_new_value / 120)

end

function IceSugarVisual:after_set_layers(c_msg)

	local layer_number = c_msg.x_param.x_new_value

	if layer_number > 0 then

		local skin_name = string.format("%d", 4 - layer_number)
		self.__c_spine:setSkin(skin_name)

	end

end

function IceSugarVisual:on_layer_reduce(c_msg)

	self.__n_reduce_count = self.__n_reduce_count + 1
print("reduce_count" .. tostring(self.__n_reduce_count))
	local crack_spine = ResourceManager:get_instance():get_resource_by_key("ice_sugar_spine")
	crack_spine:clearTracks()
	crack_spine:setScale(self:get("width") / 120)
	crack_spine:setAnimation(0, string.format("barrier_101_%d", self.__n_reduce_count), false)
	crack_spine:setPosition(self.__c_parent:getPosition())
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", crack_spine))

	crack_spine:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(function() crack_spine:removeFromParent() end)))

	local particle = ResourceManager:get_instance():get_resource_by_key("ice_remove_particle")
	particle:setScale(self:get("width") / 120)
	particle:setAutoRemoveOnFinish(true)
	particle:setPosition(self.__c_parent:getPosition())
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

end

function IceSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(0, "barrier_101_tishi", true)

end

function IceSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(0, "barrier_101_tishi_end", false)

end

IceSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = IceSugarVisual.on_assemble}},
	{"disassemble", {before = IceSugarVisual.on_disassemble}},
	{"remove_started", {before = IceSugarVisual.on_start_remove}},
	{"remove_finished", {before = IceSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = IceSugarVisual.on_layer_reduce}},
	{"reset", {before = IceSugarVisual.on_reset}},
	{"destroy", {before = IceSugarVisual.on_destroy}},
	{"highlight_on", {before = IceSugarVisual.on_highlight_on}},
	{"highlight_off", {before = IceSugarVisual.on_highlight_off}},

}

IceSugarVisual.PROPERTY_CONFIG = {
	
	{"width", 120},
	{"height", 120},
	{"layers", 3}

}

return IceSugarVisual