-- ## Imports ## --

local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local LineSkillVisual = class("LineSkillVisual", function() return GameObject:create() end)

function LineSkillVisual:ctor()

	self.__t_lines = {}
	self.__t_results = {}

	self.__n_current_animation_index = 0
	self.__n_animation_timer = 0

	self:register_listeners_with_config(LineSkillVisual.LISTENER_CONFIG)
	self:register_properties_with_config(LineSkillVisual.PROPERTY_CONFIG)

end

function LineSkillVisual:on_destroy()


end

function LineSkillVisual:on_reset()



end

function LineSkillVisual:on_assemble()

	local function line_updater()

		self:refresh_lines()

	end

	self.__n_line_updater_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(line_updater, 0.033, false)

	-- local function animation_updater(dt)

	-- 	self:line_animator(dt)

	-- end

	-- self.__n_line_animator_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(animation_updater, 0.033, false)

end

function LineSkillVisual:on_disassemble()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_line_updater_entry)
	-- cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_line_animator_entry)

end

function LineSkillVisual:on_update_visual(c_msg)

	self.__t_results = c_msg.x_param

	self:refresh_lines()

end

function LineSkillVisual:refresh_lines()

	for i = 1, #self.__t_results - 1 do

		if not self.__t_lines[i] then

			local line_spine = ResourceManager:get_instance():get_resource_by_key("link_lightning_spine")
			line_spine:setLocalZOrder(99)
			line_spine:setAnimation(0, "connect", true)
			line_spine:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", line_spine))
			self.__t_lines[i] = line_spine

		end

		local line_spine = self.__t_lines[i]
		local start_position = cc.p(self.__t_results[i]:getPosition())
		local end_position = cc.p(self.__t_results[i + 1]:getPosition())

		local spine_position = cc.p((start_position.x + end_position.x) / 2, (start_position.y + end_position.y) / 2)
		local spine_angle = UtilFunction:caculate_angle(UtilFunction:caculate_direction(end_position, start_position)) + 90

		line_spine:setPosition(spine_position)
		line_spine:setRotation(spine_angle)
		line_spine:setScaleY(UtilFunction:caculate_distance(end_position, start_position) / 160)

	end

	local over_flow_start = #self.__t_results

	if over_flow_start == 0 then

		over_flow_start = 1

	end

	for i = over_flow_start, #self.__t_lines do

		self.__t_lines[i]:removeFromParent()
		self.__t_lines[i] = nil

	end

end

function LineSkillVisual:line_animator(dt)

	self.__n_animation_timer = self.__n_animation_timer - dt

	if self.__n_animation_timer <= 0 then

		self.__n_current_animation_index = self.__n_current_animation_index + 1

		if self.__n_current_animation_index <= #self.__t_lines then

			self.__t_lines[self.__n_current_animation_index]:setAnimation(0, "connect", false)

		else

			self.__n_current_animation_index = 0

		end

		self.__n_animation_timer = self:get("animation_move_interval")

	end

end


LineSkillVisual.LISTENER_CONFIG = {

	{"update_visual", {before = LineSkillVisual.on_update_visual}},
	{"assemble", {before = LineSkillVisual.on_assemble}},
	{"disassemble", {before = LineSkillVisual.on_disassemble}},
	{"destroy", {before = LineSkillVisual.on_destroy}},

}

LineSkillVisual.PROPERTY_CONFIG = {
	
	{"animation_move_interval", 0.2},

}

return LineSkillVisual