-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local MoveableIceSugarVisual = class("MoveableIceSugarVisual", function() return GameObject:create() end)

function MoveableIceSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("moveable_ice_sugar_spine", true)
	self.__c_spine:setAnimation(0, "cookie", true)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(MoveableIceSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(MoveableIceSugarVisual.PROPERTY_CONFIG)

end

function MoveableIceSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function MoveableIceSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function MoveableIceSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function MoveableIceSugarVisual:on_start_remove()

	local particle = ResourceManager:get_instance():get_resource_by_key("moveable_ice_remove_particle")
	particle:setPosition(self.__c_parent:getPosition())
	particle:setAutoRemoveOnFinish(true)
	particle:setScale(self:get("radius") / 64)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

end

function MoveableIceSugarVisual:on_finish_remove()



end


function MoveableIceSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 60)

end


function MoveableIceSugarVisual:after_set_layers(c_msg)

	local layer_number = c_msg.x_param.x_new_value

	if layer_number > 0 then

		if layer_number == 1 then

			local skin_name = string.format("cookie", layer_number)
			self.__c_spine:setSkin(skin_name)

		else
-- ERROR_ASSERT(false, "string")
			local skin_name = string.format("cookie2", layer_number)
			self.__c_spine:setSkin(skin_name)

		end

	end

end

function MoveableIceSugarVisual:on_layer_reduce(c_msg)

	local particle = ResourceManager:get_instance():get_resource_by_key("moveable_ice_remove_particle")
	particle:setPosition(self.__c_parent:getPosition())
	particle:setAutoRemoveOnFinish(true)
	particle:setScale(self:get("radius") / 64)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))	

end

function MoveableIceSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(0, "barrier_107_tishi", true)

end

function MoveableIceSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(0, "barrier_107_tishi_end", false)

end

MoveableIceSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = MoveableIceSugarVisual.on_assemble}},
	{"disassemble", {before = MoveableIceSugarVisual.on_disassemble}},
	{"remove_started", {before = MoveableIceSugarVisual.on_start_remove}},
	{"remove_finished", {before = MoveableIceSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = MoveableIceSugarVisual.on_layer_reduce}},
	{"destroy", {before = MoveableIceSugarVisual.on_destroy}},
	{"highlight_on", {before = MoveableIceSugarVisual.on_highlight_on}},
	{"highlight_off", {before = MoveableIceSugarVisual.on_highlight_off}},

}

MoveableIceSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 60},
	{"layers", 1}

}

return MoveableIceSugarVisual