-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local NormalSugarVisual = class("NormalSugarVisual", function() return GameObject:create() end)

function NormalSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine", true)
	self.__c_spine:setAnimation(0, "idle", false)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(NormalSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(NormalSugarVisual.PROPERTY_CONFIG)

end

function NormalSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end

function NormalSugarVisual:on_reset()

	self.__c_spine:setAnimation(0, "untouch", false)
	self.__c_spine:addAnimation(0, "idle", false)

end

function NormalSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function NormalSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function NormalSugarVisual:on_select()

	self:set_skin(self:get("color"), "_face")

	self.__c_spine:setAnimation(0, "touch", false)
	self.__c_spine:addAnimation(0, "hold", true)

	self.__n_original_z = self.__c_parent:getLocalZOrder()
	self.__c_parent:setLocalZOrder(999)

	self:setScale(1.0)

end

function NormalSugarVisual:on_unselect()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, "untouch", false)

	self.__c_parent:setLocalZOrder(self.__n_original_z)

	self:setScale(0.95)

end

function NormalSugarVisual:on_start_remove()

	self:set_skin(self:get("color"), "")

	local node_position = cc.p(self:getPosition())
	local world_position = self:getParent():convertToWorldSpace(node_position)

   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_1")
   	particle_1:setLocalZOrder(9)
   	particle_1:setPosition(world_position)
   	particle_1:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_2")
   	particle_2:setLocalZOrder(9)
   	particle_2:setPosition(world_position)
   	particle_2:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_3")
   	particle_3:setLocalZOrder(9)
   	particle_3:setPosition(world_position)
   	particle_3:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

end


function NormalSugarVisual:on_finish_remove()



end

function NormalSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(2, "tishi", true)

end

function NormalSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(2, "tishi_end", false)

end

function NormalSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 64)

end

function NormalSugarVisual:after_set_color(c_msg)

	self:set_skin(c_msg.x_param.x_new_value, "")

end

function NormalSugarVisual:set_skin(color, suffix)

	local str = string.format("%s%s", color, suffix)

	self.__c_spine:setSkin(str)

end

NormalSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = NormalSugarVisual.on_assemble}},
	{"disassemble", {before = NormalSugarVisual.on_disassemble}},
	{"selected", {before = NormalSugarVisual.on_select}},
	{"unselected", {before = NormalSugarVisual.on_unselect}},
	{"remove_started", {before = NormalSugarVisual.on_start_remove}},
	{"remove_finished", {before = NormalSugarVisual.on_finish_remove}},
	{"highlight_on", {before = NormalSugarVisual.on_highlight_on}},
	{"highlight_off", {before = NormalSugarVisual.on_highlight_off}},
	{"reset", {before = NormalSugarVisual.on_reset}},
	{"destroy", {before = NormalSugarVisual.on_destroy}},

}

NormalSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 1},
	{"color", "red"}

}

return NormalSugarVisual