-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local PandaSugarVisual = class("PandaSugarVisual", function() return GameObject:create() end)

function PandaSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("panda_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_109", true)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(PandaSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(PandaSugarVisual.PROPERTY_CONFIG)

end

function PandaSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function PandaSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function PandaSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function PandaSugarVisual:on_start_remove()

	local particle = ResourceManager:get_instance():get_resource_by_key("panda_remove_particle")
	particle:setPosition(self.__c_parent:getPosition())
	particle:setAutoRemoveOnFinish(true)
	particle:setScale(self:get("radius") / 64)
	particle:setLocalZOrder(9)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

end

function PandaSugarVisual:on_finish_remove()



end

function PandaSugarVisual:on_target()



end

function PandaSugarVisual:after_set_width(c_msg)

	self:setScaleX(c_msg.x_param.x_new_value / 60)

end

function PandaSugarVisual:after_set_height(c_msg)

	self:setScaleY(c_msg.x_param.x_new_value / 60)

end

function PandaSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(0, "barrier_109_tishi", true)

end

function PandaSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(0, "barrier_109_tishi_end", false)

end


PandaSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = PandaSugarVisual.on_assemble}},
	{"disassemble", {before = PandaSugarVisual.on_disassemble}},
	{"remove_started", {before = PandaSugarVisual.on_start_remove}},
	{"remove_finished", {before = PandaSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = PandaSugarVisual.on_layer_reduce}},
	{"target", {before = PandaSugarVisual.on_target}},
	{"destroy", {before = PandaSugarVisual.on_destroy}},
	{"highlight_on", {before = on_highlight_on}},
	{"highlight_off", {before = PandaSugarVisual.on_highlight_off}},

}

PandaSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 64},

}

return PandaSugarVisual