-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local RandomSugarVisual = class("RandomSugarVisual", function() return GameObject:create() end)

function RandomSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("random_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_106_2", true)
	self:addChild(self.__c_spine)


	self:register_listeners_with_config(RandomSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(RandomSugarVisual.PROPERTY_CONFIG)

end

function RandomSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function RandomSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function RandomSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function RandomSugarVisual:on_reset()

	self.__c_spine:setAnimation(0, "barrier_106_2", true)

end

function RandomSugarVisual:on_select()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, "barrier_106_touch", false)
	self.__c_spine:addAnimation(0, "barrier_106_hold", true)

	self.__n_original_z = self.__c_parent:getLocalZOrder()
	self.__c_parent:setLocalZOrder(999)

end

function RandomSugarVisual:on_unselect()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, "barrier_106_2", true)

	self.__c_parent:setLocalZOrder(self.__n_original_z)

end

function RandomSugarVisual:on_start_remove()

	self:set_skin(self:get("color"), "")

   	self.__c_spine:setAnimation(0, "barrier_106", false)

	RUN_ACTION_WITH_DELAY(7 * 0.033, function()

		local node_position = cc.p(self:getPosition())
		local world_position = self:getParent():convertToWorldSpace(node_position)

	   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_1")
	   	particle_1:setLocalZOrder(9)
	   	particle_1:setPosition(world_position)
	   	particle_1:setAutoRemoveOnFinish(true)
	   	particle_1:setLocalZOrder(9)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

	   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_2")
	   	particle_2:setLocalZOrder(9)
	   	particle_2:setPosition(world_position)
	   	particle_2:setAutoRemoveOnFinish(true)
	   	particle_2:setLocalZOrder(9)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

	   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_3")
	   	particle_3:setLocalZOrder(9)
	   	particle_3:setPosition(world_position)
	   	particle_3:setAutoRemoveOnFinish(true)
	   	particle_3:setLocalZOrder(9)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

	   	local particle_4 = ResourceManager:get_instance():get_resource_by_key("random_remove_particle")
	   	particle_4:setLocalZOrder(9)
	   	particle_4:setPosition(world_position)
	   	particle_4:setAutoRemoveOnFinish(true)
	   	particle_4:setScale(1.3)
	   	particle_4:setLocalZOrder(10)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_4))

	   	self.__c_parent:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

	end)

end

function RandomSugarVisual:on_target()



end

function RandomSugarVisual:on_finish_remove()



end


function RandomSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:addAnimation(1, "barrier_106_tishi", true)

end

function RandomSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:addAnimation(1, "barrier_106_tishi_end", false)

end

function RandomSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 64)

end

function RandomSugarVisual:after_set_color(c_msg)

	self:set_skin(c_msg.x_param.x_new_value, "")

end

function RandomSugarVisual:set_skin(color, suffix)

	local str = string.format("%s%s", color, suffix)

	self.__c_spine:setSkin(str)

end

RandomSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = RandomSugarVisual.on_assemble}},
	{"disassemble", {before = RandomSugarVisual.on_disassemble}},
	{"selected", {before = RandomSugarVisual.on_select}},
	{"unselected", {before = RandomSugarVisual.on_unselect}},
	{"remove_started", {before = RandomSugarVisual.on_start_remove}},
	{"remove_finished", {before = RandomSugarVisual.on_finish_remove}},
	{"highlight_on", {before = RandomSugarVisual.on_highlight_on}},
	{"highlight_off", {before = RandomSugarVisual.on_highlight_off}},
	{"target", {before = RandomSugarVisual.on_target}},
	{"destroy", {before = RandomSugarVisual.on_destroy}},
	{"reset", {before = RandomSugarVisual.on_reset}}

}

RandomSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 1},
	{"color", "red"},

}

return RandomSugarVisual