-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local RewardBagSugarVisual = class("RewardBagSugarVisual", function() return GameObject:create() end)

function RewardBagSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("reward_bag_spine", true)
	self.__c_spine:setAnimation(0, "reward_bag_tishi_end", false)
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(RewardBagSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(RewardBagSugarVisual.PROPERTY_CONFIG)

end

function RewardBagSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end

function RewardBagSugarVisual:on_reset()

	-- self.__c_spine:setAnimation(0, "cookie", false)

end

function RewardBagSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function RewardBagSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function RewardBagSugarVisual:on_start_remove()

	local particle = ResourceManager:get_instance():get_resource_by_key("reward_bag_remove_particle")
	particle:setPosition(self.__c_parent:getPosition())
	particle:setAutoRemoveOnFinish(true)
	particle:setScale(self:get("radius") / 64)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

	self.__c_spine:setAnimation(0, "animation", false)

	RUN_ACTION_WITH_DELAY(20*0.033, function()

		self.__c_parent:send_message(MessagePool:get_free_ex("finish_remove", "", Message.N_TUNNEL))

	end)

end

function RewardBagSugarVisual:on_finish_remove()



end


function RewardBagSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 60)

end

function RewardBagSugarVisual:on_layer_reduce(c_msg)

end

function RewardBagSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(0, "reward_bag_tishi", true)

end

function RewardBagSugarVisual:on_highlight_off()

	self.on = false
	self.__c_spine:setAnimation(0, "reward_bag_tishi_end", false)

end

RewardBagSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = RewardBagSugarVisual.on_assemble}},
	{"disassemble", {before = RewardBagSugarVisual.on_disassemble}},
	{"remove_started", {before = RewardBagSugarVisual.on_start_remove}},
	{"remove_finished", {before = RewardBagSugarVisual.on_finish_remove}},
	{"destroy", {before = RewardBagSugarVisual.on_destroy}},
	{"highlight_on", {before = RewardBagSugarVisual.on_highlight_on}},
	{"highlight_off", {before = RewardBagSugarVisual.on_highlight_off}},
	{"reset", {before = RewardBagSugarVisual.on_reset}},

}

RewardBagSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 60},

}

return RewardBagSugarVisual