-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager_Core = require "core.ResourceManager"
local GameGlobals = require "GameEx.GameGlobals"

-- ## Imports ## --

local SoilSugarVisual = class("SoilSugarVisual", function() return GameObject:create() end)

function SoilSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("soil_sugar_spine", true)
	self.__c_spine:setAnimation(0, "barrier_111_A", true)
	self:addChild(self.__c_spine, 2)

	self:register_listeners_with_config(SoilSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(SoilSugarVisual.PROPERTY_CONFIG)

	self.__n_reduce_count = 0

end

function SoilSugarVisual:on_reset()

	self.__n_reduce_count = 0

end

function SoilSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

end


function SoilSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function SoilSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function SoilSugarVisual:on_start_remove()
	-- MusicManager:getInstance().play_effect(ResourceManager_Core:getInstance():getAudioPathByKey("Soil_Audio"), false)
	

end

function SoilSugarVisual:on_finish_remove()



end


function SoilSugarVisual:after_set_width(c_msg)

	self:setScaleX(c_msg.x_param.x_new_value / 130)

end

function SoilSugarVisual:after_set_height(c_msg)

	self:setScaleY(c_msg.x_param.x_new_value / 130)

end

function SoilSugarVisual:after_set_layers(c_msg)

	local layer_number = c_msg.x_param.x_new_value

	if layer_number > 0 then

		local skin_name = string.format("%d", layer_number)
		self.__c_spine:setSkin(skin_name)

	end

end

function SoilSugarVisual:on_layer_reduce(c_msg)

	self.__n_reduce_count = self.__n_reduce_count + 1

	if self.__n_reduce_count == 1 then

	   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("soil_particle_yellow")
	   	particle_3:setLocalZOrder(9)
	   	particle_3:setPosition(cc.p(self.__c_parent:getPosition()))
	   	particle_3:setAutoRemoveOnFinish(true)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))	

	else

	   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("soil_particle_blue")
	   	particle_3:setLocalZOrder(9)
	   	particle_3:setPosition(cc.p(self.__c_parent:getPosition()))
	   	particle_3:setAutoRemoveOnFinish(true)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))	

	end

	-- self.__c_spine:setAnimation(1, "barrier_111_B", false)

	-- local crack_spine = ResourceManager:get_instance():get_resource_by_key("soil_sugar_spine")
	-- crack_spine:clearTracks()
	-- crack_spine:setScale(self:get("width") / 120)
	-- crack_spine:setAnimation(0, "barrier_111_B", false)
	-- crack_spine:setPosition(self.__c_parent:getPosition())
	-- GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", crack_spine))

	-- crack_spine:runAction(cc.Sequence:create(cc.DelayTime:create(0.4), cc.CallFunc:create(function() crack_spine:removeFromParent() end)))

end

function SoilSugarVisual:on_highlight_on()

	if self.on then return end
	self.on = true
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi", true)

end

function SoilSugarVisual:on_highlight_off()

	self.on = false
	-- self.__c_spine:setAnimation(0, "barrier_101_tishi_end", false)

end


SoilSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = SoilSugarVisual.on_assemble}},
	{"disassemble", {before = SoilSugarVisual.on_disassemble}},
	{"remove_started", {before = SoilSugarVisual.on_start_remove}},
	{"remove_finished", {before = SoilSugarVisual.on_finish_remove}},
	{"layer_reduce", {before = SoilSugarVisual.on_layer_reduce}},
	{"reset", {before = SoilSugarVisual.on_reset}},
	{"destroy", {before = SoilSugarVisual.on_destroy}},
	{"highlight_on", {before = SoilSugarVisual.on_highlight_on}},
	{"highlight_off", {before = SoilSugarVisual.on_highlight_off}},

}

SoilSugarVisual.PROPERTY_CONFIG = {
	
	{"width", 120},
	{"height", 120},
	{"layers", 3},

}

return SoilSugarVisual