-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local AnimationGenerator = require "GameEx.Foundations.Utils.AnimationGenerator"

-- ## Imports ## --

local SpecialSugarVisual = class("SpecialSugarVisual", function() return GameObject:create() end)

function SpecialSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine", true)
	self.__c_spine:setAnimation(0, "idle", false)
	self.__c_spine:setOpacity(255)
	self.__c_spine:setVisible(true)
	self.__c_spine:setPosition(cc.p(0, 0))
	self.__c_spine:setScale(1)
	self.__c_spine:clearTracks()
	self:addChild(self.__c_spine)

	self:register_listeners_with_config(SpecialSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(SpecialSugarVisual.PROPERTY_CONFIG)

end

function SpecialSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()

	if self.__c_direction_sprite then

		self.__c_direction_sprite:release()

	end

end

function SpecialSugarVisual:on_reset(c_msg)

	self.__c_spine:setAnimation(0, "untouch", false)
	self.__c_spine:addAnimation(0, "idle", false)
	self:set("ignited", false)

end

function SpecialSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function SpecialSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function SpecialSugarVisual:on_select()

	self.__c_spine:setAnimation(0, "touch", false)
	self.__c_spine:addAnimation(0, "hold", true)

	self.__n_original_z = self.__c_parent:getLocalZOrder()
	self.__c_parent:setLocalZOrder(999)

end

function SpecialSugarVisual:on_unselect()

	self.__c_spine:setAnimation(0, "untouch", false)

	self.__c_parent:setLocalZOrder(self.__n_original_z)

end

function SpecialSugarVisual:on_start_remove()

	local special_type = self:get("type")

	if special_type == "ten" then

		if self:get("ignited") then

			local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))
			local multiplier = self:get("select_multiplier")
			local interval = self:get("select_interval")
			local total_distance = (multiplier - 1) * interval
			local current_position_x = world_position.x
			local current_position_y = world_position.y

			local start_pos_x = current_position_x - total_distance / 2
			local start_pos_y = current_position_y - total_distance / 2

			for i = 1, multiplier do

				local horizonal = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
				horizonal:setPosition(cc.p(start_pos_x + (i - 1) * interval, start_pos_y))
				horizonal:setRotation(90)
				horizonal:setLocalZOrder(10)
				horizonal:setAnimation(0, "one", false)
				horizonal:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
				horizonal:registerSpineEventHandler(function (event)

					horizonal:removeFromParent()

				end, sp.EventType.ANIMATION_END)

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", horizonal))

			end

			for i = 1, multiplier do

				local vertical = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
				vertical:setPosition(cc.p(start_pos_x, start_pos_y + (i - 1) * interval))
				vertical:setRotation(0)
				vertical:setLocalZOrder(10)
				vertical:setAnimation(0, "one", false)
				vertical:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
				vertical:registerSpineEventHandler(function (event)

					vertical:removeFromParent()

				end, sp.EventType.ANIMATION_END)
				GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", vertical))

			end

			cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_1))

		else

			local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))

			local horizonal = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
			horizonal:setRotation(0)
			horizonal:setPosition(world_position)
			horizonal:setLocalZOrder(10)
			horizonal:registerSpineEventHandler(function (event)

				horizonal:removeFromParent()

			end, sp.EventType.ANIMATION_END)
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", horizonal))

			local vertical = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
			vertical:setPosition(world_position)
			vertical:setRotation(90)
			vertical:setLocalZOrder(10)
			vertical:registerSpineEventHandler(function (event)

				vertical:removeFromParent()

			end, sp.EventType.ANIMATION_END)
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", vertical))

			horizonal:setAnimation(0, "one", false)
			vertical:setAnimation(0, "one", false)

		   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("cross_remove_particle")
		   	particle_1:setLocalZOrder(10)
		   	particle_1:setRotation(0)
		   	particle_1:setPosition(world_position)
		   	particle_1:setAutoRemoveOnFinish(true)
		   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

		   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("cross_remove_particle")
		   	particle_2:setLocalZOrder(10)
		   	particle_2:setRotation(90)
		   	particle_2:setPosition(world_position)
		   	particle_2:setAutoRemoveOnFinish(true)
		   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

		   	cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_2))

		end

	elseif special_type == "line" then

		if self:get("ignited") then

			local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))
			local multiplier = self:get("select_multiplier")
			local interval = self:get("select_interval")
			local total_distance = (multiplier - 1) * interval
			local current_position_x = world_position.x
			local current_position_y = world_position.y

			local start_pos_x = current_position_x - total_distance / 2 * math.cos(math.rad(-self:get("line_angle")))
			local start_pos_y = current_position_y - total_distance / 2 * math.sin(math.rad(-self:get("line_angle")))

			for i = 1, multiplier do

				local pos = {}
				pos.x = start_pos_x + (i - 1) * interval * math.cos(math.rad(-self:get("line_angle")))
				pos.y = start_pos_y + (i - 1) * interval * math.sin(math.rad(-self:get("line_angle")))

				local vertical = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
				vertical:setPosition(pos)
				vertical:setRotation(self:get("line_angle") + 90)
				vertical:setLocalZOrder(10)
				vertical:setAnimation(0, "one", false)
				vertical:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
				vertical:registerSpineEventHandler(function (event)

					vertical:removeFromParent()

				end, sp.EventType.ANIMATION_END)

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", vertical))

			end

			cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_1))		

		else

			local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))

			local horizonal = ResourceManager:get_instance():get_resource_by_key("cross_effect_spine")
			horizonal:setRotation(self:get("line_angle") + 90)
			horizonal:setPosition(world_position)
			horizonal:setLocalZOrder(10)
			horizonal:registerSpineEventHandler(function (event)

				horizonal:removeFromParent()

			end, sp.EventType.ANIMATION_END)
			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", horizonal))

			horizonal:setAnimation(0, "one", false)

		   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("cross_remove_particle")
		   	particle_1:setLocalZOrder(10)
		   	particle_1:setRotation(self:get("line_angle"))
		   	particle_1:setPosition(world_position)
		   	particle_1:setAutoRemoveOnFinish(true)
		   	particle_1:setRotation(self:get("line_angle"))
		   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

		   	cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_2))

		end

	elseif special_type == "bomb" then

		local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))
		local scale_modify = 1.5

		if self:get("ignited") then

			scale_modify = 2.5

			cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_1))

		else

			cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_2))

		end

	   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_1")
	   	particle_1:setLocalZOrder(10)
	   	particle_1:setPosition(world_position)
	   	particle_1:setAutoRemoveOnFinish(true)
	   	particle_1:setScale(scale_modify)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

	   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_2")
	   	particle_2:setLocalZOrder(10)
	   	particle_2:setPosition(world_position)
	   	particle_2:setAutoRemoveOnFinish(true)
	   	-- particle_2:setScale(scale_modify)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

	   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("bomb_remove_particle_3")
	   	particle_3:setLocalZOrder(10)
	   	particle_3:setPosition(world_position)
	   	particle_3:setAutoRemoveOnFinish(true)
	   	particle_3:setScale(scale_modify)
	   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

	elseif special_type == "lightning" then

		if not self:get("total") then

		   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("lightning_remove_particle_1")
		   	particle_1:setLocalZOrder(10)
		   	particle_1:setDuration(-1)
		   	particle_1:setName("lightning_particle_1")
		   	self:addChild(particle_1)

		   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("lightning_remove_particle_2")
		   	particle_2:setLocalZOrder(10)
		   	particle_2:setDuration(-1)
		   	particle_2:setName("lightning_particle_2")
		   	self:addChild(particle_2)

	   else

	   		local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))

	   		local spine = ResourceManager:get_instance():get_resource_by_key("total_remove_spine")
	   		spine:setAnimation(0, "powerful", false)
	   		spine:setLocalZOrder(11)
	   		spine:setPosition(cc.p(00, 30))
	   		spine:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
	   		self:addChild(spine, 3)

	   		self.__c_total_spine = spine

	   end

	end

end

function SpecialSugarVisual:on_finish_remove()

	if self:get("ignited") then

		self:removeChildByName("particle_1")
		self:removeChildByName("particle_2")

	end

	if self:get("type") == "lightning" then

		self:removeChildByName("lightning_particle_1")
		self:removeChildByName("lightning_particle_2")

		cc.Director:getInstance():getRunningScene():runAction(AnimationGenerator:get_instance():get_shake_action(AnimationGenerator.SHAKE_ANIMATION_LEVEL_1))

		if self:get("total") then

			local world_position = self:getParent():convertToWorldSpace(cc.p(self:getPosition()))

			self.__c_total_spine:removeFromParent()
			self.__c_total_spine:setPosition(world_position)
			self.__c_total_spine:setLocalZOrder(99)

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", self.__c_total_spine))

			local delay = cc.DelayTime:create(0.9)
			local remove = cc.RemoveSelf:create()
			local sequence = cc.Sequence:create(delay, remove)

			self.__c_total_spine:runAction(sequence)


		end

	end

end

function SpecialSugarVisual:on_ignite()

   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("ignite_particle_1")
   	particle_1:resetSystem()
   	particle_1:setLocalZOrder(10)
   	particle_1:setDuration(-1)
   	particle_1:setName("particle_1")
   	self:addChild(particle_1)

   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("ignite_particle_2")
   	particle_2:resetSystem()
   	particle_2:setLocalZOrder(10)
   	particle_2:setDuration(-1)
   	particle_2:setName("particle_2")
   	self:addChild(particle_2)

	self.__c_spine:setAnimation(0, "set off", true)

	self:set("ignited", true)

end

function SpecialSugarVisual:on_highlight_on()
	
	if self.on then return end
	self.on = true
	self.__c_spine:setAnimation(1, "tishi", true)

end

function SpecialSugarVisual:on_highlight_off()
	
	self.on = false
	self.__c_spine:setAnimation(1, "tishi_end", false)

end

function SpecialSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 64)

end

function SpecialSugarVisual:after_set_color(c_msg)

	self:set_skin(c_msg.x_param.x_new_value, "")

end

function SpecialSugarVisual:after_set_select_color(c_msg)

	local special_type = self:get("type")

	if special_type == "lightning" then

		self.__c_spine:setSkin(string.format("lightning_%s", c_msg.x_param.x_new_value))

	end

end

function SpecialSugarVisual:after_set_type(c_msg)

	if c_msg.x_param.x_new_value == "line" then

		self.__c_direction_sprite = ResourceManager:get_instance():get_resource_by_key("direction_sprite")
		self:addChild(self.__c_direction_sprite, 2)

	end

end

function SpecialSugarVisual:after_set_line_angle(c_msg)

	if self.__c_direction_sprite then

		self.__c_direction_sprite:setRotation(c_msg.x_param.x_new_value + 90)

	end

end

function SpecialSugarVisual:set_skin(color, suffix)

	local str = string.format("%s", color)

	self.__c_spine:setSkin(str)

end

function SpecialSugarVisual:on_enter()

	local create_effect_2 = ResourceManager:get_instance():get_resource_by_key("create_special_spine_2")
	self:addChild(create_effect_2)

	create_effect_2:setBlendFunc(cc.blendFunc(gl.SRC_ALPHA , gl.ONE))
	create_effect_2:setSkin(self:get("type"))
	create_effect_2:setAnimation(0, "create", false)

	local delay = cc.DelayTime:create(1.5)
	local remove = cc.RemoveSelf:create()
	local sequence = cc.Sequence:create(delay, remove)

	create_effect_2:runAction(sequence)

end

SpecialSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = SpecialSugarVisual.on_assemble}},
	{"disassemble", {before = SpecialSugarVisual.on_disassemble}},
	{"selected", {before = SpecialSugarVisual.on_select}},
	{"unselected", {before = SpecialSugarVisual.on_unselect}},
	{"remove_started", {before = SpecialSugarVisual.on_start_remove}},
	{"remove_finished", {before = SpecialSugarVisual.on_finish_remove}},
	{"highlight_on", {before = SpecialSugarVisual.on_highlight_on}},
	{"highlight_off", {before = SpecialSugarVisual.on_highlight_off}},
	{"enter", {before = SpecialSugarVisual.on_enter}},
	{"ignited", {before = SpecialSugarVisual.on_ignite}},
	{"reset", {before = SpecialSugarVisual.on_reset}},
	{"destroy", {before = SpecialSugarVisual.on_destroy}},

}

SpecialSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 1},
	{"color", "red"},
	{"type", "lightning"},
	{"total", false},
	{"ignited", false},
	{"select_multiplier", 1},
	{"select_interval", 240},
	{"select_color", "red"},
	{"line_angle", 0}

}

return SpecialSugarVisual