-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local TargetVisual = class("TargetVisual", function() return GameObject:create() end)

function TargetVisual:ctor()

	self:register_listeners_with_config(TargetVisual.LISTENER_CONFIG)
	self:register_properties_with_config(TargetVisual.PROPERTY_CONFIG)

end

function TargetVisual:on_target(c_msg)

	local sugar = c_msg.x_param

	local category = sugar:get_category()

	if category:has("key") then

		self:play_key_target_animation(sugar)

	elseif category:has("ice") then

		self:play_ice_target_animation(sugar)

	elseif category:has("conch") then

		self:play_conch_target_animation(sugar)

	elseif (category:has("normal") 
		or category:has("colorful")) 
		and (not category:has("random")) 
		and (not category:has("timing_bomb")) 
		and (not category:has("transform")) then

		self:play_normal_target_animation(sugar)

	elseif category:has("transform") then

		self:play_transform_target_animation(sugar)

	elseif category:has("special") then

		self:play_normal_target_animation(sugar)

	else

		self:play_common_target_animation(sugar)

	end

end

function TargetVisual:play_key_target_animation(sugar)

	local visible_size = cc.Director:getInstance():getVisibleSize()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	local spine = ResourceManager:get_instance():get_resource_by_key("key_sugar_spine")
	spine:setAnimation(0, "barrier_104po", false)
	spine:setScale(sugar:get("width") / 150)
	spine:setPosition(sugar:getPosition())

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

	local node_position = cc.p(sugar:getPosition())
	local world_position = node_position

	local function move_spine()

	 	function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", "key", Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

		local distance = UtilFunction:caculate_distance(world_position, cc.p(540, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 + 80, 1550),
	        cc.p(visible_size.width / 2 + 330, 1650),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	end

	local function add_particle()

		local particle = ResourceManager:get_instance():get_resource_by_key("key_remove_particle")
		particle:setAutoRemoveOnFinish(true)
		particle:setPosition(world_position)

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle))

	end

	local delay_particle = cc.DelayTime:create(10 * 0.033)
	local delay_spine = cc.DelayTime:create(30 * 0.033)
	local call_func_spine = cc.CallFunc:create(move_spine)
	local call_func_particle = cc.CallFunc:create(add_particle)
	local sequence_spine = cc.Sequence:create(delay_spine, call_func_spine)
	local sequence_particle = cc.Sequence:create(delay_particle, call_func_particle)
	local spawn = cc.Spawn:create(sequence_spine, sequence_particle)

	spine:runAction(spawn)

end

function TargetVisual:play_normal_target_animation(sugar)

	if GameGlobals.Scene:get("has_boss") then

		local visible_size = cc.Director:getInstance():getVisibleSize()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", task, Message.N_TUNNEL))

		function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", sugar.apd_n_damage, Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

			local spine = ResourceManager:get_instance():get_resource_by_key("boss_hurt_spine")
			spine:setPosition(cc.p(visible_size.width / 2 + 50, 1550))
			spine:setLocalZOrder(7)
			spine:setScale(1.2)
			spine:setAnimation(0, "hurt", false)

			GameGlobals.Scene.__c_scene:addChild(spine)

			local sequence = cc.Sequence:create(cc.DelayTime:create(10 * 0.033), cc.RemoveSelf:create())
			spine:runAction(sequence)

		end

		local node_position = cc.p(sugar:getPosition())
		local world_position = sugar:getParent():convertToWorldSpace(node_position)

		local spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine")
		spine:setScale(sugar:getScale())
		spine:setSkin(sugar:get("color"))
		spine:setAnimation(0, "untouch", false)

		spine:setPosition(world_position)
		spine:setLocalZOrder(6)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2 + 50, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 - 360, 1500),
	        cc.p(visible_size.width / 2, 1600),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.7)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	else

		local visible_size = cc.Director:getInstance():getVisibleSize()

	    local color = sugar:get("color")

	    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	 	function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", color, Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

		local node_position = cc.p(sugar:getPosition())
		local world_position = sugar:getParent():convertToWorldSpace(node_position)

		local spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine")
		spine:setScale(sugar:getScale())
		spine:setSkin(sugar:get("color"))
		spine:setAnimation(0, "untouch", false)

		spine:setPosition(world_position)
		spine:setLocalZOrder(9)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 + 80, 1550),
	        cc.p(visible_size.width / 2 + 330, 1650),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	end

end

function TargetVisual:play_transform_target_animation(sugar)

	if GameGlobals.Scene:get("has_boss") then

		local visible_size = cc.Director:getInstance():getVisibleSize()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action", task, Message.N_TUNNEL))

		function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", sugar.apd_n_damage, Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

			local spine = ResourceManager:get_instance():get_resource_by_key("boss_hurt_spine")
			spine:setPosition(cc.p(visible_size.width / 2 + 50, 1550))
			spine:setLocalZOrder(7)
			spine:setScale(1.2)
			spine:setAnimation(0, "hurt", false)

			GameGlobals.Scene.__c_scene:addChild(spine)

			local sequence = cc.Sequence:create(cc.DelayTime:create(10 * 0.033), cc.RemoveSelf:create())
			spine:runAction(sequence)

		end

		local node_position = cc.p(sugar:getPosition())
		local world_position = sugar:getParent():convertToWorldSpace(node_position)

		local spine = ResourceManager:get_instance():get_resource_by_key("normal_sugar_spine")
		spine:setScale(sugar:getScale())
		spine:setSkin(sugar:get("color"))
		spine:setAnimation(0, "untouch", false)

		spine:setPosition(world_position)
		spine:setLocalZOrder(6)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2 + 50, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 - 360, 1500),
	        cc.p(visible_size.width / 2, 1600),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.7)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	else

		local visible_size = cc.Director:getInstance():getVisibleSize()

	    local color = sugar:get("color")

	    GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	 	function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", "transform_" .. color, Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

		local node_position = cc.p(sugar:getPosition())
		local world_position = sugar:getParent():convertToWorldSpace(node_position)

		local spine = ResourceManager:get_instance():get_resource_by_key("transform_sugar_spine")
		spine:setScale(sugar:getScale())
		spine:setSkin(sugar:get("color"))
		spine:setAnimation(0, "untouch", false)

		spine:setPosition(world_position)
		spine:setLocalZOrder(9)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 + 80, 1550),
	        cc.p(visible_size.width / 2 + 330, 1650),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	end

end

function TargetVisual:play_ice_target_animation(sugar)

	if GameGlobals.Scene:get("has_boss") then
print("start ice visual")

		local visible_size = cc.Director:getInstance():getVisibleSize()

		local spine = ResourceManager:get_instance():get_resource_by_key("ice_sugar_spine")
		spine:clearTracks()
		spine:setAnimation(0, "barrier_101_4", false)
		spine:setAnimation(1, "barrier_101_tishi_end", false)
		spine:setScale(sugar:get("width") / 150)
		spine:setPosition(sugar:getPosition())

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

		local node_position = cc.p(sugar:getPosition())
		local world_position = node_position

		local function move_spine()

		 	function trigger_hit()

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", sugar.apd_n_damage, Message.N_TUNNEL))

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

				local spine = ResourceManager:get_instance():get_resource_by_key("boss_hurt_spine")
				spine:setPosition(cc.p(visible_size.width / 2 + 50, 1550))
				spine:setLocalZOrder(7)
				spine:setScale(1.2)
				spine:setAnimation(0, "hurt", false)

				GameGlobals.Scene.__c_scene:addChild(spine)

				local sequence = cc.Sequence:create(cc.DelayTime:create(10 * 0.033), cc.RemoveSelf:create())
				spine:runAction(sequence)

			end

			local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
			local time = 0.6

			local control_point = cc.p(world_position)
			control_point.x = control_point.x - 300
			control_point.y = control_point.y - 100 

			local bezierConfig ={
				control_point,
	        	cc.p(visible_size.width / 2 - 360, 1500),
	        	cc.p(visible_size.width / 2, 1600),
		    }

		    local bezier_move = cc.BezierTo:create(time, bezierConfig)
		    local scale_delay = cc.DelayTime:create(time / 2)
		    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
		    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
		    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
		    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


		    local ease = cc.EaseSineIn:create(sequence)

		    spine:runAction(ease)

		end

		local delay_spine = cc.DelayTime:create(30 * 0.033)
		local call_func_spine = cc.CallFunc:create(move_spine)
		local sequence_spine = cc.Sequence:create(delay_spine, call_func_spine)
		local spawn = cc.Spawn:create(sequence_spine)

		spine:runAction(spawn)

		return		

	end

	local visible_size = cc.Director:getInstance():getVisibleSize()

	local spine = ResourceManager:get_instance():get_resource_by_key("ice_sugar_spine")
	spine:clearTracks()
	spine:setAnimation(0, "barrier_101_4", false)
	spine:setScale(sugar:get("width") / 150)
	spine:setPosition(sugar:getPosition())

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	local node_position = cc.p(sugar:getPosition())
	local world_position = node_position

	local function move_spine()

	 	function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", "ice", Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 + 80, 1550),
	        cc.p(visible_size.width / 2 + 330, 1650),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	end

	local delay_spine = cc.DelayTime:create(30 * 0.033)
	local call_func_spine = cc.CallFunc:create(move_spine)
	local sequence_spine = cc.Sequence:create(delay_spine, call_func_spine)
	local spawn = cc.Spawn:create(sequence_spine)

	spine:runAction(spawn)

end

function TargetVisual:play_conch_target_animation(sugar)

	local visible_size = cc.Director:getInstance():getVisibleSize()

	local spine = ResourceManager:get_instance():get_resource_by_key("soil_sugar_spine")
	spine:clearTracks()
	spine:setAnimation(0, "barrier_111_A", true)
	spine:setScale(1.1)
	spine:setSkin("shell")
	spine:setPosition(sugar:getPosition())

	local scale_1 = cc.ScaleTo:create(0.2, 1.5)
	local scale_2 = cc.ScaleTo:create(0.2, 1.05)
	local scale_3 = cc.ScaleTo:create(0.2, 1.2)
	local scale_4 = cc.ScaleTo:create(0.2, 1.1)
	local seq = cc.Sequence:create(scale_1, scale_2, scale_3, scale_4)
	spine:runAction(seq)

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	local node_position = cc.p(sugar:getPosition())
	local world_position = node_position

	local function move_spine()

	 	function trigger_hit()

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", "conch", Message.N_TUNNEL))

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

		end

		local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
		local time = 0.6

		local control_point = cc.p(world_position)
		control_point.x = control_point.x - 300
		control_point.y = control_point.y - 100 

		local bezierConfig ={
			control_point,
	        cc.p(visible_size.width / 2 + 80, 1550),
	        cc.p(visible_size.width / 2 + 330, 1650),
	    }

	    local bezier_move = cc.BezierTo:create(time, bezierConfig)
	    local scale_delay = cc.DelayTime:create(time / 2)
	    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
	    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
	    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
	    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


	    local ease = cc.EaseSineIn:create(sequence)

	    spine:runAction(ease)

	end

	local delay_spine = cc.DelayTime:create(30 * 0.033)
	local call_func_spine = cc.CallFunc:create(move_spine)
	local sequence_spine = cc.Sequence:create(delay_spine, call_func_spine)
	local spawn = cc.Spawn:create(sequence_spine)

	spine:runAction(spawn)

end

function TargetVisual:play_common_target_animation(sugar)

	local color = 0
	local spine = 0

	local category = sugar:get_category()

	if category:has("fenced") then

		color = "fenced"

		spine = ResourceManager:get_instance():get_resource_by_key("fence_sugar_spine")
		spine:clearTracks()
		spine:setAnimation(0, "barrier_103", false)

	elseif category:has("chocolate") then

		color = "chocolate"

		spine = ResourceManager:get_instance():get_resource_by_key("chocolate_sugar_spine")
		spine:setAnimation(0, "barrier_102", false)
		spine:setAnimation(1, "barrier_102_tishi_end", false)

	elseif category:has("moveable_ice") then

		color = "moveable_ice"

		spine = ResourceManager:get_instance():get_resource_by_key("moveable_ice_sugar_spine")
		spine:setSkin("cookie")
		spine:setAnimation(0, "barrier_107", false)
		spine:setAnimation(1, "barrier_107_tishi_end", false)

	elseif category:has("random") then

		color = "random"

		spine = ResourceManager:get_instance():get_resource_by_key("random_sugar_spine")
		spine:setSkin("color")

	elseif category:has("panda") then

		color = "panda"

		spine = ResourceManager:get_instance():get_resource_by_key("panda_sugar_spine")
		spine:setAnimation(0, "barrier_109", false)
		spine:setAnimation(1, "barrier_109_tishi_end", false)

	elseif category:has("fan") then

		color = "fan"

		spine = ResourceManager:get_instance():get_resource_by_key("fan_sugar_spine")
		spine:setAnimation(0, "roll", false)
		spine:setSkin("all")

	elseif category:has("coco") then

		color = "coco"

		spine = ResourceManager:get_instance():get_resource_by_key("coco_sugar_spine")
		spine:setAnimation(0, "empty", false)

	end

	if GameGlobals.Scene:get("has_boss") then

		local visible_size = cc.Director:getInstance():getVisibleSize()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

		local node_position = cc.p(sugar:getPosition())
		local world_position = node_position

		spine:setPosition(world_position)
		spine:setLocalZOrder(5)
		GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

		local function move_spine()

		 	function trigger_hit()

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", 1, Message.N_TUNNEL))

				GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action", task, Message.N_TUNNEL))

				local spine = ResourceManager:get_instance():get_resource_by_key("boss_hurt_spine")
				spine:setPosition(cc.p(visible_size.width / 2 + 50, 1550))
				spine:setLocalZOrder(7)
				spine:setScale(1.2)
				spine:setAnimation(0, "hurt", false)

				GameGlobals.Scene.__c_scene:addChild(spine)

				local sequence = cc.Sequence:create(cc.DelayTime:create(10 * 0.033), cc.RemoveSelf:create())
				spine:runAction(sequence)

			end

			local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
			local time = 0.6

			local control_point = cc.p(world_position)
			control_point.x = control_point.x - 300
			control_point.y = control_point.y - 100 

			local bezierConfig ={
				control_point,
	        	cc.p(visible_size.width / 2 - 360, 1500),
	        	cc.p(visible_size.width / 2, 1600),
		    }

		    local bezier_move = cc.BezierTo:create(time, bezierConfig)
		    local scale_delay = cc.DelayTime:create(time / 2)
		    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
		    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
		    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
		    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())


		    local ease = cc.EaseSineIn:create(sequence)

		    spine:runAction(ease)

		end

		local delay_spine = cc.DelayTime:create(00 * 0.033)
		local call_func_spine = cc.CallFunc:create(move_spine)
		local sequence_spine = cc.Sequence:create(delay_spine, call_func_spine)
		local spawn = cc.Spawn:create(sequence_spine)

		spine:runAction(spawn)

		return		

	end

	local visible_size = cc.Director:getInstance():getVisibleSize()

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

 	function trigger_hit()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("sugar_target_animation_finish", color, Message.N_TUNNEL))

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

	end

	local node_position = cc.p(sugar:getPosition())
	local world_position = node_position

	spine:setPosition(world_position)
	spine:setLocalZOrder(5)
	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", spine))

	local distance = UtilFunction:caculate_distance(world_position, cc.p(visible_size.width / 2, 1600))
	local time = 0.6

	local control_point = cc.p(world_position)
	control_point.x = control_point.x - 300
	control_point.y = control_point.y - 100 

	local bezierConfig ={
		control_point,
        cc.p(visible_size.width / 2 + 80, 1550),
        cc.p(visible_size.width / 2 + 330, 1650),
    }

    local bezier_move = cc.BezierTo:create(time, bezierConfig)
    local scale_delay = cc.DelayTime:create(time / 2)
    local scale_to = cc.ScaleTo:create(time / 2, 0.63)
    local scale_sequence = cc.Sequence:create(scale_delay, scale_to)
    local spawn = cc.Spawn:create(bezier_move, scale_sequence)
    local sequence = cc.Sequence:create(spawn, cc.CallFunc:create(trigger_hit), cc.RemoveSelf:create())

    local ease = cc.EaseSineIn:create(sequence)

    spine:runAction(ease)

end


TargetVisual.LISTENER_CONFIG = {

	{"target_animation", {before = TargetVisual.on_target}},

}

TargetVisual.PROPERTY_CONFIG = {
	


}

return TargetVisual