-- ## Imports ## --

local GameGlobals = require "GameEx.GameGlobals"
local GameObject = require "GameEx.Foundations.GameObject"
local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local UtilFunction = require "GameEx.Foundations.Utils.Function"
local ResourceManager = require "GameEx.Foundations.ResourceManager"

-- ## Imports ## --

local TimingBombSugarVisual = class("TimingBombSugarVisual", function() return GameObject:create() end)

function TimingBombSugarVisual:ctor()

	self.__c_spine = ResourceManager:get_instance():get_resource_by_key("timing_bomb_sugar_spine", true)
	self.__c_spine:setAnimation(0, "zhadan", true)
	self:addChild(self.__c_spine)

	self.__t_numbers = {}

	self.__t_numbers[1] = ResourceManager:get_instance():get_resource_by_key("timing_bomb_number_spine")
	self.__t_numbers[1]:setSkin(0)
	self.__t_numbers[1]:setLocalZOrder(1)
	self.__t_numbers[1]:setPosition(cc.p(12, 0))
	self.__t_numbers[1]:setScale(0.85)
	self:addChild(self.__t_numbers[1])

	self.__t_numbers[2] = ResourceManager:get_instance():get_resource_by_key("timing_bomb_number_spine")
	self.__t_numbers[2]:setSkin(0)
	self.__t_numbers[2]:setLocalZOrder(1)
	self.__t_numbers[2]:setPosition(cc.p(-12, 0))
	self.__t_numbers[2]:setScale(0.85)
	self:addChild(self.__t_numbers[2])

	self.__s_state = "zhadan"

	self:register_listeners_with_config(TimingBombSugarVisual.LISTENER_CONFIG)
	self:register_properties_with_config(TimingBombSugarVisual.PROPERTY_CONFIG)

end

function TimingBombSugarVisual:on_destroy()

	self.__c_spine:removeFromParent()
	self.__t_numbers[1]:removeFromParent()
	self.__t_numbers[2]:removeFromParent()

end


function TimingBombSugarVisual:on_assemble()

	self.__c_parent:addChild(self)

end

function TimingBombSugarVisual:on_disassemble()

	self.__c_parent:removeChild(self)

end

function TimingBombSugarVisual:on_reset()
print("ZhaDanReset !!!!!!!!!")
	self.__s_state = "zhadan"
	self.__c_spine:setAnimation(0, "zhadan", true)

end

function TimingBombSugarVisual:on_select()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, string.format("%s_touch", self.__s_state), false)
	self.__c_spine:addAnimation(0, string.format("%s_hold", self.__s_state), true)

	self.__n_original_z = self.__c_parent:getLocalZOrder()
	self.__c_parent:setLocalZOrder(999)

	for i = 1, #self.__t_numbers do

		self.__t_numbers[i]:setAnimation(0, "number_touch", false)
		self.__t_numbers[i]:addAnimation(0, "number_hold", true)

	end


end

function TimingBombSugarVisual:on_unselect()

	self:set_skin(self:get("color"), "")

	self.__c_spine:setAnimation(0, string.format("%s", self.__s_state), true)

	for i = 1, #self.__t_numbers do

		self.__t_numbers[i]:setAnimation(0, "number", true)

	end

	self.__c_parent:setLocalZOrder(self.__n_original_z)

end

function TimingBombSugarVisual:on_start_remove()

	self:set_skin(self:get("color"), "")

	local node_position = cc.p(self:getPosition())
	local world_position = self:getParent():convertToWorldSpace(node_position)

   	local particle_1 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_1")
   	particle_1:setLocalZOrder(9)
   	particle_1:setPosition(world_position)
   	particle_1:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_1))

   	local particle_2 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_2")
   	particle_2:setLocalZOrder(9)
   	particle_2:setPosition(world_position)
   	particle_2:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_2))

   	local particle_3 = ResourceManager:get_instance():get_resource_by_key("normal_remove_particle_3")
   	particle_3:setLocalZOrder(9)
   	particle_3:setPosition(world_position)
   	particle_3:setAutoRemoveOnFinish(true)
   	GameGlobals.Scene:send_message(MessagePool:get_free_ex("normal_add_child", particle_3))

end

function TimingBombSugarVisual:on_finish_remove()



end

function TimingBombSugarVisual:after_set_count_down(c_msg)

	self.__n_current_count = c_msg.x_param.x_new_value

	self:update_number()

end

function TimingBombSugarVisual:on_reduce_count()

	self.__n_current_count = self.__n_current_count - 1 

	if self.__n_current_count < 0 then

		self.__n_current_count = 0

	end

	if self.__n_current_count == 5 then

		if self.__s_state == "zhadan" then

			self.__s_state = "shan"
			self.__c_spine:setAnimation(0, string.format("%s", self.__s_state), true)

			GameGlobals.Scene:send_message(MessagePool:get_free_ex("timingbomb_ready", "", Message.N_TUNNEL))

		end

	end

	self:update_number()

end

function TimingBombSugarVisual:update_number()

	local number = self.__n_current_count

	for i = 1, #self.__t_numbers do

		local result = number % 10

		self.__t_numbers[i]:setSkin(tostring(result))

		number = math.floor(number / 10)

		local scale_1 = cc.ScaleTo:create(0.1, 1.0)
		local scale_2 = cc.ScaleTo:create(0.2, 0.85)
		local sequence = cc.Sequence:create(scale_1, scale_2)
		self.__t_numbers[i]:runAction(sequence)

		local move_1 = cc.MoveBy:create(0.05, cc.p(0, 4))
		local move_2 = cc.MoveBy:create(0.1, cc.p(0, -4))
		local delay = cc.DelayTime:create(0.15)
		local sequence_2 = cc.Sequence:create(move_1, delay, move_2)
		self.__t_numbers[i]:runAction(sequence_2)

		local rotate_1 = cc.RotateBy:create(0.1, -5)
		local rotate_2 = cc.RotateBy:create(0.1, 10)
		local rotate_3 = cc.RotateBy:create(0.1, -10)
		local rotate_4 = cc.RotateBy:create(0.1, 10)
		local rotate_5 = cc.RotateBy:create(0.1, -5)
		local sequence_3 = cc.Sequence:create(rotate_1, rotate_2, rotate_3, rotate_4, rotate_5)
		self.__t_numbers[i]:runAction(sequence_3)

	end

end

function TimingBombSugarVisual:on_highlight_on()
print("timing bomb highlight")
	self.__c_spine:setAnimation(1, string.format("%s_shan", self.__s_state), true)

end

function TimingBombSugarVisual:on_highlight_off()

	self.__c_spine:setAnimation(1, string.format("%s_shan_end", self.__s_state), false)

end

function TimingBombSugarVisual:after_set_radius(c_msg)

	self:setScale(c_msg.x_param.x_new_value / 64)

end

function TimingBombSugarVisual:after_set_color(c_msg)

	self:set_skin(c_msg.x_param.x_new_value, "")

end

function TimingBombSugarVisual:set_skin(color, suffix)

	local str = string.format("%s%s", color, suffix)

	self.__c_spine:setSkin(str)

end

TimingBombSugarVisual.LISTENER_CONFIG = {

	{"assemble", {before = TimingBombSugarVisual.on_assemble}},
	{"disassemble", {before = TimingBombSugarVisual.on_disassemble}},
	{"selected", {before = TimingBombSugarVisual.on_select}},
	{"unselected", {before = TimingBombSugarVisual.on_unselect}},
	{"remove_started", {before = TimingBombSugarVisual.on_start_remove}},
	{"remove_finished", {before = TimingBombSugarVisual.on_finish_remove}},
	{"highlight_on", {before = TimingBombSugarVisual.on_highlight_on}},
	{"highlight_off", {before = TimingBombSugarVisual.on_highlight_off}},
	{"round_finish", {before = TimingBombSugarVisual.on_player_round_finish}},
	{"reduce_count", {before = TimingBombSugarVisual.on_reduce_count}},
	{"destroy", {before = TimingBombSugarVisual.on_destroy}},
	{"reset", {before = TimingBombSugarVisual.on_reset}},

}

TimingBombSugarVisual.PROPERTY_CONFIG = {
	
	{"radius", 1},
	{"color", "red"},
	{"count_down", 10}

}

return TimingBombSugarVisual