-- ## Imports ## --

local ListPool = require "GameEx.Foundations.Utils.ListPool"

-- ## Imports ## --




local AutoReleasePool = class("AutoReleasePool", {})

function AutoReleasePool:ctor(n_init_size, n_refill_amount, t_funcs, s_object_name)
	
	self.__t_freeList = {}
	self.__n_freeSize = 0
	self.__t_usedList = ListPool:create()
	self.__n_refill_amount = n_refill_amount
	self.__t_funcs = t_funcs

	self:set_object_name(s_object_name)
	self:refill(n_init_size)

end

function AutoReleasePool:get_free()

	if self.__n_freeSize == 0 then

		self:refill(self.__n_refill_amount)

	end

	local return_object = self.__t_freeList[self.__n_freeSize]
	self.__n_freeSize = self.__n_freeSize - 1
	self.__t_funcs.reset(return_object, self.__t_funcs.reset_param)
	self.__t_usedList:pushBack(return_object)

	return return_object

end

function AutoReleasePool:rob()

	return self.__t_funcs.create(self.__t_funcs.create_param)

end

function AutoReleasePool:refill(n_refill_amount)
print(string.format("Autorelease refill !!! : %s, [%d]", self:get_object_name(), n_refill_amount))
	for i = 1, n_refill_amount do

		local object = self.__t_funcs.create(self.__t_funcs.create_param)
		object:retain()

		self.__n_freeSize = self.__n_freeSize + 1
		self.__t_freeList[self.__n_freeSize] = object

	end

end

function AutoReleasePool:recycle()

	local used_list = self.__t_usedList

	local iter = used_list.head[1]
-- local recycle_count = 0
	while not (iter == used_list.head) do

		local object = iter[3]

		if object:getReferenceCount() == 1 then

			self.__n_freeSize = self.__n_freeSize + 1
			self.__t_freeList[self.__n_freeSize] = object

			iter = self.__t_usedList:remove(iter)
-- recycle_count = recycle_count + 1
		end
		
		iter = iter[1]

	end
-- if recycle_count > 0 then print(string.format("%s: %d [recycled]  %d [free_size]", self:get_object_name(), recycle_count, self.__n_freeSize)) end

end

function AutoReleasePool:clear()

	if self.__n_freeSize > 0 then
print(string.format("%s:[%d]destroyed", self.__s_object_name, self.__n_freeSize))
		for i = 1, self.__n_freeSize do

			local object = self.__t_freeList[i]
			self.__t_funcs.destroy(object)
			object:release()
			self.__t_freeList[i] = nil

		end

	end

	self.__n_freeSize = 0

end

function AutoReleasePool:set_object_name(s_name)

	self.__s_object_name = s_name

end

function AutoReleasePool:get_object_name()

	return self.__s_object_name

end

return AutoReleasePool