-- ## Imports ## --

local UtilFunction = require "GameEx.Foundations.Utils.Function"

-- ## Imports ## --



ConfigManager = class("ConfigManager", {})

function ConfigManager:ctor()

	self.__t_available_config = {}
	self.__t_default_config = {}

end

function ConfigManager:load_default_config()

	local config_list = require "GameEx.Resources.ConfigRegistry"

	for key, config in pairs(config_list) do

		if type(config) == "table" then

			self.__t_default_config[key] = config

		else

			self.__t_default_config[key] = require(config)

		end

		self.__t_default_config[key].properties = self.__t_default_config[key].properties or {}
		self.__t_default_config[key].resources = self.__t_default_config[key].resources or {}
		self.__t_default_config[key].class_name = self.__t_default_config[key].class_name or key
		self.__t_default_config[key].actived = false
		self.__t_default_config[key].tiles = {}

	end

	self:reset_to_default()

end

function ConfigManager:clear()

	self:reset_to_default()

end

function ConfigManager:reset_to_default()

	self.__t_available_config = {}

end

function ConfigManager:get_config(s_config_key)

	local config = self.__t_available_config[s_config_key] or self.__t_default_config[s_config_key]
ERROR_ASSERT(config, string.format("config key[%s] not found !", s_config_key))
	return config

end

function ConfigManager:set_tiles(s_config_key, t_tiles)

	if not self.__t_available_config[s_config_key] then

		self:copy_default(s_config_key)

	end

	self.__t_available_config[s_config_key]["tiles"] = t_tiles	

end

function ConfigManager:get_tiles(s_config_key)

	return self:get_config(s_config_key).tiles

end

function ConfigManager:set_actived(s_config_key)

	if not self.__t_available_config[s_config_key] then

		self:copy_default(s_config_key)

	end

	self.__t_available_config[s_config_key]["actived"] = true	

end

function ConfigManager:get_all_actived_config()

	local actived_list = {}

	for key, value in pairs(self.__t_available_config) do

		if value.actived == true then

			actived_list[key] = value

		end

	end

	return actived_list

end

function ConfigManager:get_properties(s_config_key)

	return self:get_config(s_config_key).properties

end

function ConfigManager:change_properties(s_config_key, x_new_properties)

	if not self.__t_available_config[s_config_key] then

		self:copy_default(s_config_key)

	end

	self.__t_available_config[s_config_key]["properties"] = x_new_properties

end

function ConfigManager:change_property_by_key(s_config_key, s_key, x_new_value)

	if not self.__t_available_config[s_config_key] then

		self:copy_default(s_config_key)

	end

	self.__t_available_config[s_config_key]["properties"][s_key] = x_new_value

end

function ConfigManager:copy_default(s_config_key)

	local config = self:get_config(s_config_key)

	local copy = UtilFunction:copy(config, { resources = "ref" })

	self.__t_available_config[s_config_key] = copy

end

function ConfigManager:get_instance()

	if not ConfigManager.instance then

		ConfigManager.instance = ConfigManager:create()

	end

	return ConfigManager.instance

end

function ConfigManager:new_instance()


	ConfigManager.instance = ConfigManager:create()


	return ConfigManager.instance

end

return ConfigManager