-- ## Imports ## --

local TileMapParser = require "GameEx.Foundations.TileMapParser"
local ConfigManager = require "GameEx.Foundations.ConfigManager"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local FileManipulator = require "GameEx.Foundations.Utils.FileManipulator"
local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local GameScene = require "GameEx.Components.Scene.GameScene"
local GameGlobals = require "GameEx.GameGlobals"
local RandomGenerator = require "GameEx.Foundations.Utils.RandomGenerator"
local MusicResourceManager = require "core.ResourceManager"
local UtilFunction = require "GameEx.Foundations.Utils.Function"

-- ## Imports ## --




local LevelBuilder = class("LevelBuilder", {})

function LevelBuilder:ctor()


end

function LevelBuilder:pre_build()

	ResourceManager:new_instance()
	ObjectFactory:new_instance()

	ResourceManager:get_instance():load_key_path_map()
	ConfigManager:get_instance():load_default_config()

	ConfigManager:get_instance():reset_to_default()

end

function LevelBuilder:load_level_design(n_level_id)
print("level_id:")
print(n_level_id)
	local file_path = string.format("GameEx.Resources.Levels.level_%d", n_level_id)
	local parser = TileMapParser:create()
	local parse_result = parser:parse(file_path, n_level_id)

	self.__t_parse_result = parse_result
	self.__n_current_level_id = n_level_id

	self:apply_config()

end

function LevelBuilder:apply_config()

	local parsed_configs = self.__t_parse_result.config
	local config_manager = ConfigManager:get_instance()
	config_manager:reset_to_default()

	for config_key, config in pairs(parsed_configs) do

		config_manager:set_actived(config_key)
		config_manager:set_tiles(config_key, config.tiles)

		local properties = config.properties

		for property_key, value in pairs(properties) do
			
			config_manager:change_property_by_key(config_key, property_key, value)

		end

	end

	if not GameGlobals.RandomGenerator then	GameGlobals.RandomGenerator = RandomGenerator:create() end

end

function LevelBuilder:preload_current_level()

	self.__n_preload_progress = 0

	local function preload()

		self:start_resource_preload()
		coroutine.yield()
		self:start_object_preload()
		coroutine.yield()
		self:preload_finish()

	end

	self.__c_preload_coroutine = coroutine.wrap(preload)

	self.__c_preload_coroutine()

end

function LevelBuilder:preload_game_sound()

	for i = 1, 10 do 

		MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey(string.format("Combo%d_Audio", i)))

	end

	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Combo_Voice_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Ice_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Cookies_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Cake_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Fence_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Bomb_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Bombpang_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Cross_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Light_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Goal_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Hammer_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Spread_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Cotton_Candy_Audio"))
	MusicManager.preloadEffect(MusicResourceManager:getInstance():getAudioPathByKey("Reward_Audio"))


end

function LevelBuilder:preload_finish()

	self:preload_game_sound()

	if self.__f_preload_finish_callback then

		self.__f_preload_finish_callback()

	end

end

function LevelBuilder:start_resource_preload()

	local resource_manager = ResourceManager:get_instance()
	local object_factory = ObjectFactory:get_instance()

	local actived_config = ConfigManager:get_instance():get_all_actived_config()

	for config_key, config in pairs(actived_config) do

		if config.resources then

			for _, key in ipairs(config.resources) do

				resource_manager:add_keys_to_preload_list(key)

			end

		end

	end	

	resource_manager:set_finish_preload_callback(function() self.__c_preload_coroutine() end)
	resource_manager:start_preload()

end

function LevelBuilder:start_object_preload()

	local configs = self.__t_parse_result.config
	local load_list = {}
	local load_index = 1

	for config_key, config in pairs(configs) do

		load_list[#load_list + 1] = config_key

	end

	local scheduler_entry = 0

	local function load_performer()

		if load_index <= #load_list then

			self.__n_preload_progress = math.floor(load_index / #load_list * 50)

			ObjectFactory:get_instance():lazy_require(load_list[load_index])

			if load_list[load_index] == "GameScene" then  -- 为什么要放在这，因为有个TM的BUG导致放在后面会出问题，然后这个TM的BUG是无解的

				local config = ConfigManager:get_instance():get_config("GameScene")
				local scene = ObjectFactory:get_instance():get("GameScene")

				local parse_result = self:parse_level_config(config.properties)

				scene:set("level_config", parse_result)

				for key, value in pairs(config.properties) do

					scene:set(key, value)

				end

				local ui = ObjectFactory:get_instance():get("UIComponent")
				scene:add_component(ui, "ui_component")

				local sound_performer = ObjectFactory:get_instance():get("SoundPerformer")
				scene:add_component(sound_performer, "SoundPerformer")

				scene:set("player_damage_bonus", Player:getInstance():get_player_total_atk())
				scene:set("player_score_bonus", Player:getInstance():get_player_score_add())

				self.__c_ret_scene = scene			

			end

			load_index = load_index + 1

		else

			self.__n_preload_progress = 50

			cc.Director:getInstance():getScheduler():unscheduleScriptEntry(scheduler_entry)
			self.__c_preload_coroutine()

		end

	end

	scheduler_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(load_performer, 0.033, false)

end

function LevelBuilder:get_preload_progress()

	local resource_preload_progress = ResourceManager:get_instance():get_preload_progress()
	resource_preload_progress = math.floor(resource_preload_progress / 100 * 50)

	return self.__n_preload_progress + resource_preload_progress

end

function LevelBuilder:set_preload_finish_callback(f_callback)

	self.__f_preload_finish_callback = f_callback

end

function LevelBuilder:build_current_level(c_scene, scene_start_param)

	for i = 1, #self.__t_parse_result.canvas do

		local config = self.__t_parse_result.canvas[i]
		local object = self:common_assembler(config)

		c_scene:send_message(MessagePool:get_free_ex("add_sugar", object, Message.N_TUNNEL))

	end

	if self.__t_parse_result.has_span then

		local emitter = ObjectFactory:get_instance():get("AdvancedEmitter")
		c_scene:send_message(MessagePool:get_free_ex("add_sugar", emitter, Message.N_TUNNEL))

	end

end

function LevelBuilder:common_assembler(config)

	local object = ObjectFactory:get_instance():get(config.name)

	if config.properties then

		for key, value in pairs(config.properties) do

			object:set(key, value)

		end

	end

	if config.name == "FenceSugar" then
dump(config.properties)
		local ref = config.properties["ref"][1]

		local hostage_config = ConfigManager:get_instance():get_config(ref.type)
		local hostage = ObjectFactory:get_instance():get(ref.type)
		local hostage_properties = {}

		for _, tile in ipairs(hostage_config.tiles) do

			if tile.id == ref.tile_index then

				hostage_properties = tile.properties

			end

		end 

		for key, value in pairs(hostage_properties) do

			hostage:set(key, value)

		end

		local pos = config.position
		local visible_size = cc.Director:getInstance():getVisibleSize()
		pos.x = pos.x + 10 + visible_size.width / 2 - 540
		pos.y = pos.y + visible_size.height / 2 - 960

		hostage:setPosition(pos)

		object:send_message(MessagePool:get_free_ex("append_to", hostage, Message.N_TUNNEL))

		object = hostage

	else

		local pos = config.position
		local visible_size = cc.Director:getInstance():getVisibleSize()
		pos.x = pos.x + 10 + visible_size.width / 2 - 540
		pos.y = pos.y + visible_size.height / 2 - 960

		object:setPosition(pos)

	end

	return object

end

function LevelBuilder:create_scene()

	return self.__c_ret_scene

end

function LevelBuilder:get_level_config(n_level_id)

	self:load_level_design(n_level_id)

	return self:parse_level_config(ConfigManager:get_instance():get_config("GameScene").properties)

end

function LevelBuilder:parse_level_config(config)

	local level_config = {}

	level_config.reqs = self:parse_req(config)
	level_config.key_waves = self:parse_wave(config.key_wave)
	level_config.score_wave = self:parse_wave(config.score_wave)
	level_config.boss = self:parse_boss(config)
	level_config.move_limit = config.con_move

	return level_config

end

function LevelBuilder:parse_boss(config)

	if config.has_boss then

		local boss = require(string.format("GameEx.Resources.Levels.boss_%d", self.__n_current_level_id))
		return boss

	else

		return nil

	end

end

function LevelBuilder:parse_req(config)

	local reqs = {}

	local req_number = config.req_number or 0
	-- print(string.format("level_builder:%d, [req_number]", req_number))
	for i = 1, req_number do

		local req_type = config[string.format("req_%d_type", i)] or "red"
		local req_amount = config[string.format("req_%d_amount", i)] or 1

		reqs[req_type] = req_amount

	end

	return reqs

end

function LevelBuilder:parse_wave(wave_str)

	local waves = {}

	if wave_str then

		for number_str in string.gmatch(wave_str, "%d+") do

			waves[#waves + 1] = tonumber(number_str)

		end

	end

	return waves

end

function LevelBuilder:prepare_skill(scene, scene_start_param)

	local skill_info = scene_start_param.skill_info

	local skill = ObjectFactory:get_instance():get(skill_info.name)
	skill:set("change_number", skill_info.change_number)
	skill:set("perform_threshold", skill_info.fill_threshold)
	scene:add_component(skill, skill_info.name)


	local move_skill = ObjectFactory:get_instance():get("MoveSkill")
	scene:add_component(move_skill, "MoveSkill")

	local bomb_skill = ObjectFactory:get_instance():get("BombSkill")
	scene:add_component(bomb_skill, "BombSkill")

	local hammer_skill = ObjectFactory:get_instance():get("HammerSkill")
	scene:add_component(hammer_skill, "HammerSkill")

	local line_skill = ObjectFactory:get_instance():get("LineSkill")
	scene:add_component(line_skill, "LineSkill")

	local candy_skill = ObjectFactory:get_instance():get("CandySkill")
	scene:add_component(candy_skill, "CandySkill")

	local item_info = scene_start_param.item_info

	local start_skill = ObjectFactory:get_instance():get("StartSkill")
	start_skill:set("add_lightning", item_info.lightning)
	start_skill:set("add_bomb", item_info.bomb)
	start_skill:set("add_move", item_info.move)
	scene:add_component(start_skill, "StartSkill")


	local player = Player:getInstance()
	local atk = player:get_player_total_atk()
	local score = player:get_player_score_add()

	local scoreHandler = scene:get_component("score_handler")
	scoreHandler:set("player_damage_bonus", atk)
	scoreHandler:set("player_score_bonus", score)

end


function LevelBuilder:get_instance()

	if not LevelBuilder.instance then

		LevelBuilder.instance = LevelBuilder:create()

	end

	return LevelBuilder.instance

end

function LevelBuilder:new_instance()

	LevelBuilder.instance = LevelBuilder:create()

	return LevelBuilder.instance

end


return LevelBuilder