-- ## Imports ## --

local Message = require "GameEx.Foundations.Message"
local MessagePool = require "GameEx.Foundations.MessagePool"
local ConfigManager = require "GameEx.Foundations.ConfigManager"
local AutoReleasePool = require "GameEx.Foundations.AutoReleasePool"

-- ## Imports ## --




local ObjectFactory = class("ObjectFactory", {})

function ObjectFactory:ctor()

	self.__t_classes = {}
	self.__t_recycle = {}

end

function ObjectFactory:lazy_require(s_class_name)

	local class_config = ConfigManager:get_instance():get_config(s_class_name)
	local class_script_path = class_config.script_path
	local cache_count = class_config.cache_count or 1
	local class_struct = {}
	class_struct.name = s_class_name
	class_struct.class = require(class_script_path)
	class_struct.funcs = {create = ObjectFactory.create_object, reset = ObjectFactory.reset_object, destroy = ObjectFactory.destroy_object, create_param = class_struct, reset_param = class_struct}
	class_struct.pool = AutoReleasePool:create(cache_count, cache_count, class_struct.funcs, s_class_name)
	self.__t_classes[s_class_name] = class_struct
print(string.format("%s:%d, [required]", s_class_name, cache_count))

	if class_config.will_recycle then

		self.__t_recycle[s_class_name] = class_struct

	end

end

function ObjectFactory:get(s_class_name)

	if not self.__t_classes[s_class_name] then

		self:lazy_require(s_class_name)

	end

	return self.__t_classes[s_class_name].pool:get_free()

end

function ObjectFactory:get_instance()
	
	if ObjectFactory.instance == nil then

		ObjectFactory.instance = ObjectFactory:create()

	end

	return ObjectFactory.instance

end

function ObjectFactory:new_instance()

	ObjectFactory.instance = ObjectFactory:create()
	return ObjectFactory.instance

end

function ObjectFactory:clear()

	for i = 1, 10 do 

		cc.Director:getInstance():clearPool()

		for key, struct in pairs(self.__t_classes) do

			struct.pool:recycle()

		end

		for key, struct in pairs(self.__t_classes) do

			struct.pool:clear()

		end

	end

	self.__t_classes = {}
	self.__t_recycle = {}

end

function ObjectFactory:recycle()

	for key, struct in pairs(self.__t_recycle) do

		struct.pool:recycle()

	end

end

function ObjectFactory.create_object(class_struct)

	return class_struct.class:create()

end

function ObjectFactory.reset_object(c_object, class_struct)

	local config = ConfigManager:get_instance():get_config(class_struct.name)

	c_object:send_message(MessagePool:get_free_ex("reset", config.properties, Message.N_TUNNEL))

end

function ObjectFactory.destroy_object(c_object)

	c_object:destroy()

end

return ObjectFactory