-- ## Imports ## --

local AutoReleasePool = require "GameEx.Foundations.AutoReleasePool"

-- ## Imports ## --




ResourceManagerEx = class("ResourceManagerEx", {})

function ResourceManagerEx:ctor()

	self.__t_keys = {}

	self.__t_preload_textures = {};
	self.__t_preload_keys = {};

	self.__t_loaded_textures = {};

	self.__n_finish_count = 0
	self.__n_preload_progress = 0

	self.__f_preload_finish_callback = function() end

end

function ResourceManagerEx:load_key_path_map()

	local map = require "GameEx.Resources.ResourceRegistry"

	for key, config in pairs(map.spines) do

		self:load_spine_key(key, config)

	end

	for key, config in pairs(map.sprites) do

		self:load_sprite_key(key, config)

	end

	for key, config in pairs(map.particles) do

		self:load_particle_key(key, config)

	end

	for key, config in pairs(map.sounds) do

		self:load_sound_key(key, config)

	end

end

function ResourceManagerEx:clear()

	for key, info in pairs(self.__t_keys) do

		if info.loaded then

			info.pool:clear()

		end

	end

end

function ResourceManagerEx:load_spine_key(s_key, c_config)

	local info = {}
	info.type = "spine"
	info.texture = string.format("%s.png", c_config[1])
	info.json = string.format("%s.json", c_config[1])
	info.atlas = string.format("%s.atlas", c_config[1])
	info.cache_number = c_config[2] or 1
	info.loaded = false

	function create_spine(param)

		local spine = sp.SkeletonAnimation:create(param[1], param[2])
ERROR_ASSERT(spine, string.format("ResourceManager: Particle file [%s] not found !", param[1]))
		return spine

	end

	function reset_spine(object)

		object:setScale(1)

	end

	function destroy_spine()

	end


	local funcs = {

		create_param = {info.json, info.atlas},
		create = create_spine,
		reset = reset_spine,
		destroy = destroy_spine,

	}

	info.pool = AutoReleasePool:create(0, info.cache_number, funcs, s_key)

	self.__t_keys[s_key] = info

end

function ResourceManagerEx:load_sprite_key(s_key, c_config)

	local info = {}
	info.type = "sprite"
	info.texture = string.format("%s.png", c_config[1])
	info.cache_number = c_config[2] or 1

	function create_sprite(param)
		
		local sprite = cc.Sprite:create(param)
ERROR_ASSERT(sprite, string.format("ResourceManager: Sprite file [%s] not found !", param))
		return sprite

	end

	function reset_sprite()

	end

	function destroy_sprite()

	end


	local funcs = {

		create_param = info.texture,
		create = create_sprite,
		reset = reset_sprite,
		destroy = destroy_sprite,

	}

	info.pool = AutoReleasePool:create(0, info.cache_number, funcs, s_key)

	self.__t_keys[s_key] = info

end

function ResourceManagerEx:load_particle_key(s_key, c_config)

	local info = {}
	info.type = "sprite"
	info.plist = string.format("%s.plist", c_config[1])
	info.cache_number = c_config[2] or 1

	function create_particle(param)

		local particle = cc.ParticleSystemQuad:create(param)
ERROR_ASSERT(particle, string.format("ResourceManager: Particle file [%s] not found !", param))
		return particle

	end

	function reset_particle(particle)
		particle:resetSystem()
	end

	function destroy_particle()

	end


	local funcs = {

		create_param = info.plist,
		create = create_particle,
		reset = reset_particle,
		destroy = destroy_particle,

	}

	info.pool = AutoReleasePool:create(0, info.cache_number, funcs, s_key)

	self.__t_keys[s_key] = info

end

function ResourceManagerEx:load_sound_keys(s_key, c_config)



end

function ResourceManagerEx:add_textures_to_preload_list(s_texture_file_path)

	self.__t_preload_textures[#self.__t_preload_textures + 1] = s_texture_file_path

end

function ResourceManagerEx:add_keys_to_preload_list(s_key)

	local info = self.__t_keys[s_key]
ERROR_ASSERT(info, string.format("resource key[%s] not found", s_key))
	if info and not info.loaded then

		info.loaded = true

		if info.texture and not self.__t_loaded_textures[info.texture] then

			self.__t_preload_textures[#self.__t_preload_textures + 1] = info.texture

		end

		self.__t_preload_keys[#self.__t_preload_keys + 1] = s_key

	end

end

function ResourceManagerEx:start_preload()

	self.__n_finish_count = 0
	self.__n_preload_progress = 0

	local checker_entry = 0
	local total = #self.__t_preload_textures + #self.__t_preload_keys

	function preload_checker()

		self.__n_preload_progress = math.floor(self.__n_finish_count * 100 / total)

		if self.__n_finish_count == total then

			cc.Director:getInstance():getScheduler():unscheduleScriptEntry(checker_entry)

			self:finish_preload()

		end

	end

	checker_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(preload_checker, 0.016, false) 

	function preload()

		self:start_texture_preload()
		coroutine.yield()
		self:start_object_preload()
		coroutine.yield()

	end

	self.__c_preload_coroutine = coroutine.wrap(preload)
	self.__c_preload_coroutine()

end

function ResourceManagerEx:start_texture_preload()

	local texture_load_end = #self.__t_preload_textures
	local texture_load_count = 0
	local performer_entry = 0

	function finish_callback()

		texture_load_count = texture_load_count + 1
		self.__n_finish_count = self.__n_finish_count + 1

	end

	for i = 1, #self.__t_preload_textures do

		cc.Director:getInstance():getTextureCache():addImageAsync(self.__t_preload_textures[i], finish_callback)	

	end

	function texture_load_checker()

		if texture_load_count == texture_load_end then

			cc.Director:getInstance():getScheduler():unscheduleScriptEntry(performer_entry)
			self.__c_preload_coroutine()

		end

	end

	performer_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(texture_load_checker, 0.033, false)

end

function ResourceManagerEx:start_object_preload()

	local object_load_end = #self.__t_preload_keys
	local object_load_count = 0
	local performer_entry = 0

	function object_preload_performer()

		if object_load_count == object_load_end then

			cc.Director:getInstance():getScheduler():unscheduleScriptEntry(performer_entry)
			self.__c_preload_coroutine()

		else

			object_load_count = object_load_count + 1

			local key = self.__t_preload_keys[object_load_count]
			local info = self.__t_keys[key]

			info.pool:refill(info.cache_number)

			self.__n_finish_count = self.__n_finish_count + 1
print(string.format("%s:%d, [loaded]", key, info.cache_number))
		end

	end

	performer_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(object_preload_performer, 0.033, false)

end

function ResourceManagerEx:finish_preload()
	
	self.__t_loaded_textures = self.__t_preload_textures
	self.__t_preload_textures = {}
	self.__t_preload_keys = {}
	self:__f_preload_finish_callback()

end

function ResourceManagerEx:set_finish_preload_callback(f_callback)

	self.__f_preload_finish_callback = f_callback

end

function ResourceManagerEx:get_preload_progress()

	return self.__n_preload_progress

end

function ResourceManagerEx:clear_loaded_objects()

	for key, info in pairs(self.__t_keys) do

		info.pool:clear()
		info.loaded = false

	end

end

function ResourceManagerEx:clear_loaded_textures()

	for index, key in ipairs(self.__t_loaded_textures) do

		cc.Director:getInstance():getTextureCache():removeTextureForKey(key)

	end

	self.__t_loaded_textures = {}

end

function ResourceManagerEx:remove_loaded_texture_by_path(s_path)

	for i = 1, #self.__t_loaded_textures do
		
		if self.__t_loaded_textures[i] == s_path then

			table.remove(self.__t_loaded_textures, i)
			break

		end
		
	end

end

function ResourceManagerEx:clear()

	self:clear_loaded_objects()
	self:clear_loaded_textures()

end

function ResourceManagerEx:get_resource_by_key(s_key, rob)

	local info = self:get_resource_info_by_key(s_key)
	info.loaded = true

	if rob then

		return info.pool:rob()

	else

		return info.pool:get_free()

	end

end

function ResourceManagerEx:get_resource_info_by_key(s_key)

	local info = self.__t_keys[s_key]
ERROR_ASSERT(info, string.format("resource key[%s] not found", s_key))
	return info

end

function ResourceManagerEx:recycle()

	for key, info in pairs(self.__t_keys) do

		if info.loaded then

			info.pool:recycle()

		end

	end

end

function ResourceManagerEx:get_instance()

	if ResourceManagerEx.instance == nil then

		ResourceManagerEx.instance = ResourceManagerEx:create()

	end

	return ResourceManagerEx.instance

end

function ResourceManagerEx:new_instance()

	ResourceManagerEx.instance = ResourceManagerEx:create()
	return ResourceManagerEx.instance

end

return ResourceManagerEx