local DataObjectPool = class("DataObjectPool")

function DataObjectPool:ctor(...)

	self.defaultCreate = function() return {} end
	self.defaultRecycle = function() end

	self.template = {create = self.defaultCreate, recycle = self.defaultRecycle}
	self.maxVolume = 1
	self.refillStep = 1
	self.pool = {}

end

function DataObjectPool:init(template, initSize, maxVolume, refillStep)

	local _template = template or {}
	_template.create = _template.create or self.defaultCreate
	_template.recycle = _template.recycle or self.defaultRecycle

	local _maxVolume = maxVolume or 1
	maxVolume = _maxVolume > 0 and _maxVolume or 1

	local _refillStep = refillStep or 1
	_refillStep = _refillStep > 0 and _refillStep or 1

	local _initSize = initSize or 1
	_initSize = _initSize > 0 and _initSize or 1

	self.template = _template
	self.maxVolume = _maxVolume
	self.refillStep = _refillStep

	self:refill(_initSize)

end

function DataObjectPool:refill(size)

	local currentPoolSize = #self.pool

	for i = 1, size or self.refillStep do
		
		if currentPoolSize < self.maxVolume then
			
			self.pool[currentPoolSize + 1] = self:copyTemplate()
			currentPoolSize = currentPoolSize + 1

		end

	end

end

function DataObjectPool:copyTemplate()

	return self.template.create()

end

function DataObjectPool:getFree()

	local currentPoolSize = #self.pool

	if currentPoolSize > 0 then

		local object = self.pool[currentPoolSize]
		self.pool[currentPoolSize] = nil
		return object

	else
		
		self:refill()
		return self:getFree()

	end

end

function DataObjectPool:recycle(object)

	self.template.recycle(object)

	local currentPoolSize = #self.pool

	if currentPoolSize < self.maxVolume then

		self.pool[currentPoolSize + 1] = object

	end

end

return DataObjectPool