-- ## Imports ## --

local ListPool = require "GameEx.Utils.ListPool"

-- ## Imports ## --




local EventDispatcher = class("EventDispatcher")

function EventDispatcher:ctor()

	self.listener = {}
	self.eventCache = ListPool:create()

end

function EventDispatcher:initCacheNodePool()

	local cacheNodePool = DataObjectPool:create()

	local cacheNodeTemplate = {}

	cacheNodeTemplate.create = function() 

		return {event = 0, param = 0} 

	end

	cacheNodeTemplate.recycle = function(object) 

		object.event = 0
		object.param = 0

	end

	cacheNodePool:init(cacheNodeTemplate, 100, 100, 10)

end

function EventDispatcher:addListener(event, listener, target)

	self.listener[event] = self.listener[event] or {}
	
	self.listener[event][listener] = target or Constant.TARGET_TYPE.NO_TARGET

end

function EventDispatcher:removeListener(event, listener)

	self.listener[event][listener] = nil

end

function EventDispatcher:fireEvent(event, param, mode)

	mode = mode or Constant.EVENT_FIRE_TYPE.IMMEDIATE
	param = param or Constant.PARAM_TYPE.NO_PARAM

	if mode == Constant.EVENT_FIRE_TYPE.IMMEDIATE then

		local listeners = self.listener[event]

		if listeners then

			for listener, target in pairs(listeners) do

				if not target == Constant.TARGET_TYPE.NO_TARGET then

					listener(target, param)

				else

					listener(param)

				end

			end

		end

	else

		local cacheNode = self.cacheNodePool:getFree()
		cacheNode.event = event
		cacheNode.param = param

		self.eventCache:pushBack(cacheNode)

	end

end

function EventDispatcher:popCachedEvent()

	for iter, cacheNode in self.eventCache:iterate() do

		local listeners = self.listener[event]

		if listeners then

			for listener, target in pairs(listeners) do

				if not target == Constant.TARGET_TYPE.NO_TARGET then

					listener(target, param)

				else

					listener(param)

				end

			end

		end

		self.eventCache:popFront()
		self.cacheNodePool:recycle(cacheNode)

	end

end