JavaUtils = JavaUtils or {
	
	["log"] = function(tag, content)

		print(string.format("//%s//  %s", tag, content))

	end,

	["get_device_id"] = function()

		return string.format("-%d-%d", os.time(), math.random(1, 1000000))

	end,

	["get_imei"] = function()

		return string.format("%d%d", os.time(), math.random(10000, 100000))

	end,

}

function SAFE_REQUIRE(module_name)

	if pcall(function() require(module_name) end) then

		return require(module_name)

	else

		return {}

	end

end

function SAFE_REQUIRE_WITH_ID(module_name)
	local _module_name = PLAYER_ID..module_name
	if pcall(function() require(_module_name) end) then
	
		package.loaded[_module_name] = nil
		return require(_module_name)

	else
		return {}

	end

end

function SAFE_REQUIRE_NIL(module_name)

	if pcall(function() require(module_name) end) then

		return require(module_name)

	else

		return nil

	end

end

function ERROR_ASSERT(condition, string)

	if not condition then

		print(string)
		assert(false)

	end

end

function RUN_ACTION_WITH_DELAY(delay, func)

	local GameGlobals = require "GameEx.GameGlobals"
	local MessagePool = require "GameEx.Foundations.MessagePool"

	GameGlobals.Scene:send_message(MessagePool:get_free_ex("start_action"))

	local delay_action = cc.DelayTime:create(delay)
	local call_action = cc.CallFunc:create(func)
	local sequence_action = cc.Sequence:create(delay_action, cc.CallFunc:create(function()

		func()

		GameGlobals.Scene:send_message(MessagePool:get_free_ex("finish_action"))

	end))

	GameGlobals.Scene:runAction(sequence_action)

end

function ACTION_FOR_TARGET(action, target)
	action:retain()
	local begin_call = function()

		target:runAction(action)
		action:release()
	end

	local call_action = cc.CallFunc:create(begin_call)
	local delay_action = cc.DelayTime:create(action:getDuration())

	local spawn = cc.Spawn:create(call_action, delay_action)

	return spawn

end

function HOOKABLE_CALL(event_config, call_info)

	local will_hook = true

	for _, config in ipairs(event_config) do

		config.param = config.param or {}
		config.param.hitcher_info = call_info

		TRIGGER_EVENT(config.name, config.param)

		will_hook = will_hook and call_info.added

	end

	if not will_hook then

		CALL(call_info)

	end

end

function CALL(call_info)

	if not call_info.func then return end

	call_info.param = call_info.param or {}

	if call_info.target then

		call_info.func(call_info.target, unpack(call_info.param))

	else

		call_info.func(unpack(call_info.param))

	end	

end

function TRIGGER_EVENT(event_name, event_param)

	local event = cc.EventCustom:new(event_name)
	event.name = event_name
	event.param = event_param

	cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	TutorialManager:get_instance():trigger_event(event)

end

function SPLIT_STRING(str, delimiter)

	if str==nil or str=='' or delimiter==nil then
		return nil
	end
	
    local result = {}
    for match in (str..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match)
    end
    return result	

end
