-- ## Imports ## --

local DataObjectPool = require "GameEx.Foundations.Utils.DataObjectPool"

-- ## Imports ## --




local ListPool = class("ListPool")

function ListPool:ctor()

	self.head = {}
	self.head[1] = self.head
	self.head[2] = self.head
	self.head[3] = nil

	self.front = self.head
	self.back = self.head

	self.size = 0

	self.nodePool = DataObjectPool:create()

	local template = {}

	template.create = function() 
		return {[1] = 0, [2] = 0, [3] = 0} 
	end

	template.recycle = function(object) 
		-- object[1] = 0
		-- object[2] = 0
		-- object[3] = 0
	end

	self.nodePool:init(template, 100, 100, 10)

end

function ListPool:pushBack(element)

	self:insertBefore(self.head, element)

end

function ListPool:pushFront(element)

	self:insertAfter(self.head, element)

end

function ListPool:clear()

	while self.size > 0 do
		
		self:remove(self.head[1])

	end

end

function ListPool:getSize()

	return self.size

end

function ListPool:getFront()

	return self.front[3]

end

function ListPool:getBack()

	return self.back[3]

end

function ListPool:popFront()

	if self.size > 0 then

		self:remove(self.front)

	end

end

function ListPool:popBack()

	if self.size > 0 then

		self:remove(self.back)

	end

end

function ListPool:remove(iter)

	local next = iter[1]
	local prev = iter[2]

	prev[1] = next
	next[2] = prev

	self.front = self.head[1]
	self.back = self.head[2]

	self.size = self.size - 1

	self.nodePool:recycle(iter)

	return prev

end

function ListPool:removeData(data)

	for key, value in self:iterate() do

		if value == data then

			self:remove(key)
			
			break

		end

	end

end

function ListPool:insertAfter(iter, element)

	local newNode = self.nodePool:getFree()
	newNode[2] = iter
	newNode[1] = iter[1]
	newNode[3] = element

	iter[1][2] = newNode
	iter[1] = newNode

	self.front = self.head[1]
	self.back = self.head[2]

	self.size = self.size + 1

end

function ListPool:insertBefore(iter, element)

	if self.insertCallback then

		self.insertCallback(element)

	end

	local newNode = self.nodePool:getFree()
	newNode[2] = iter[2]
	newNode[1] = iter
	newNode[3] = element

	iter[2][1] = newNode
	iter[2] = newNode

	self.front = self.head[1]
	self.back = self.head[2]

	self.size = self.size + 1

end

function ListPool:iterate()

	return self.iterFunc, self, self.head

end

function ListPool:iterFunc(iter)

	local next = iter[1]

	if next == self.head then

		return nil, nil

	else
	
		return next, next[3]

	end

end

function ListPool:begin()

	return self.head[1]

end

function ListPool:next(iter)

	return iter[1]

end

function ListPool:finish()

	return self.head

end

function ListPool:getData(iter)

	return iter[3]

end

return ListPool