local StateMachine = class("StateMachine")

function StateMachine:ctor(...)

	self.stateInfo = {}
	self.currentState = "idle"
	self.defaultState = "idle"
	self.inTransition = false

	self:addState("idle")

end

function StateMachine:addState(stateName, stateBehaviour)

	stateBehaviour = stateBehaviour or {}

	local stateInfo = {

		transition = {},
		behaviour = {

			beforeLeave = stateBehaviour.beforeLeave or StateMachine.beforeLeave,
			beforeEnter = stateBehaviour.beforeEnter or StateMachine.beforeEnter,
			leave = stateBehaviour.leave or StateMachine.leave,
			enter = stateBehaviour.enter or StateMachine.enter,
			run = stateBehaviour.run or StateMachine.run,

		}

	}

	self.stateInfo[stateName] = stateInfo

end

function StateMachine:addTransition(eventName, stateNameFrom, stateNameTo, transitionBehaviour)

	transitionBehaviour = transitionBehaviour or {}

	local transitionInfo = {}
	transitionInfo.dstState = stateNameTo
	transitionInfo.behaviour = {

		beforeLeave = transitionBehaviour.beforeLeave or self.stateInfo[stateNameFrom].behaviour.beforeLeave,
		beforeEnter = transitionBehaviour.beforeEnter or self.stateInfo[stateNameTo].behaviour.beforeEnter,
		leave = transitionBehaviour.leave or self.stateInfo[stateNameFrom].behaviour.leave,
		enter = transitionBehaviour.enter or self.stateInfo[stateNameTo].behaviour.enter,

	}

	self.stateInfo[stateNameFrom].transition[eventName] = transitionInfo

end

function StateMachine:doTransition(eventName, eventParam)

	local currentStateInfo = self.stateInfo[self.currentState]

	if not self.inTransition then 

		local transition = currentStateInfo.transition[eventName]

		if transition then

			self.inTransition = true

			local transitionBehaviour = transition.behaviour
			local dstState = transition.dstState

			local beforeLeave = transitionBehaviour.beforeLeave
			local beforeEnter = transitionBehaviour.beforeEnter
			local leave = transitionBehaviour.leave 
			local enter = transitionBehaviour.enter

			if beforeLeave(eventName, eventParam) and beforeEnter(eventName, eventParam) then

				leave(eventName, eventParam)

				self.currentState = dstState
				self.inTransition = false

				enter(eventName, eventParam)

			end

			self.inTransition = false

		end

	end

end

function StateMachine:getCurrentState()

	return self.currentState

end

function StateMachine:forceTransition(stateName, param)

	self.currentState = stateName
	self.stateInfo[self.currentState].behaviour.enter(param)

end

function StateMachine:setDefaultState(stateName)

	self.defaultState = stateName

end


StateMachine.beforeLeave = function()

	return true

end

StateMachine.beforeEnter = function()

	return true

end

StateMachine.leave = function()

end

StateMachine.enter = function()

end

StateMachine.run = function()

end

return StateMachine