local StateMachine = class("StateMachine")

function StateMachine:ctor()

	self.__s_current_state = "idle"
	self.__t_transitions = {}
	self.__t_states = {}

	self:add_state({

		{name = "idle"}

	})

end

function StateMachine:add_state(t_state_config)

	local config_size = #t_state_config

	for i = 1, config_size do

		local config = t_state_config[i]

		self.__t_states[config.name] = {
			leave = config.leave or StateMachine.default_leave, 
			enter = config.enter or StateMachine.default_enter, 
			check = config.check or StateMachine.default_check
		}

	end

end

function StateMachine:add_transition(t_transition_config)

	local config_size = #t_transition_config

	for i = 1, config_size do

		local config = t_transition_config[i]

		self.__t_transitions[config.event] = self.__t_transitions[config.event] or {}

		self.__t_transitions[config.event][config.from] = config.to

	end

end

function StateMachine:do_event(event, param)

	local transitions = self.__t_transitions[event]

	if transitions then

		local from_state = self.__s_current_state
		local target_state = transitions[from_state]

		if from_state then

			local from_state_func = self.__t_states[from_state]
			local target_state_func = self.__t_states[target_state]

			if from_state_func.check(target_state, event, param) and target_state_func then

				self.__s_current_state = nil
				from_state_func.leave()
				self.__s_current_state = target_state
				target_state_func.enter()

			end

		end

	end

end

function StateMachine:do_transition(state)

	local from_state_func = self.__t_states[self.__s_current_state]
	local target_state_func = self.__t_states[state]

	if target_state_func then

		from_state_func.leave()
		self.__s_current_state = state
		target_state_func.enter()		

	end

end

function StateMachine:do_set(state)

	self.__s_current_state = state

end

function StateMachine:get_current_state()

	return self.__s_current_state

end

StateMachine.default_enter = function() end
StateMachine.default_leave = function() end
StateMachine.default_check = function() return true end

return StateMachine