-- ## Imports ## --

local MessagePool = require "GameEx.Foundations.MessagePool"
local Message = require "GameEx.Foundations.Message"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ConfigManager = require "GameEx.Foundations.ConfigManager"
local ObjectFactory = require "GameEx.Foundations.ObjectFactory"
local LevelBuilder = require "GameEx.Foundations.LevelBuilder"
local LevelBuilderTournament = require "GameEx.Foundations.LevelBuilderTournament"
local RandomGenerator = require "GameEx.Foundations.Utils.RandomGenerator"



local SceneBuilder = {}

function SceneBuilder:build(scene_start_param)

  GameGlobals.RandomGenerator = RandomGenerator:create()

  if scene_start_param.random_seed then

    GameGlobals.RandomGenerator:seed(scene_start_param.random_seed)

  end

	local level_builder = LevelBuilder:get_instance()
	level_builder:pre_build()
	level_builder:load_level_design(scene_start_param.level_index)
	level_builder:preload_current_level()
	level_builder:set_preload_finish_callback(function()

	  local scene = LevelBuilder:get_instance():create_scene()
    scene.tutorial_config = scene_start_param.tutorial_config

    level_builder:prepare_skill(scene, scene_start_param)

    GameGlobals.bonus = false
    GameGlobals.Scene = scene

    LevelBuilder:get_instance():build_current_level(scene)

    local msg = {}
    msg.s_name = "run"
    msg.e_transmit = Message.N_TUNNEL
    scene:send_message(msg)

    function recycle_peformer()

    	ObjectFactory:get_instance():recycle()
    	ResourceManager:get_instance():recycle()

    end

    SceneBuilder.recycle_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(recycle_peformer, 0.2, false)

    local function final_texture_clean_up()

        cc.SpriteFrameCache:getInstance():removeUnusedSpriteFrames()
        cc.Director:getInstance():getTextureCache():removeUnusedTextures()

    end

    local delay = cc.DelayTime:create(0.1)
    local call = cc.CallFunc:create(final_texture_clean_up)
    scene:runAction(cc.Sequence:create(delay, call))

    TRIGGER_EVENT("GAME_START", { level = scene_start_param.level_index })

	end)

end

function SceneBuilder:build_tournament(scene_start_param)

  GameGlobals.RandomGenerator = RandomGenerator:create()

  if scene_start_param.random_seed then

    GameGlobals.RandomGenerator:seed(scene_start_param.random_seed)

  end

  local level_builder = LevelBuilderTournament:get_instance()
  level_builder:pre_build()
  level_builder:load_level_design(scene_start_param.level_index)
  level_builder:preload_current_level()
  level_builder:set_preload_finish_callback(function()

      local scene = LevelBuilderTournament:get_instance():create_scene()
        scene.tutorial_config = scene_start_param.tutorial_config

        level_builder:prepare_skill(scene, scene_start_param)

        GameGlobals.bonus = false
        GameGlobals.Scene = scene

        LevelBuilderTournament:get_instance():build_current_level(scene)

        local msg = {}
        msg.s_name = "run"
        msg.e_transmit = Message.N_TUNNEL
        scene:send_message(msg)

        function recycle_peformer()

          ObjectFactory:get_instance():recycle()
          ResourceManager:get_instance():recycle()

        end

        SceneBuilder.recycle_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(recycle_peformer, 0.2, false)

        local function final_texture_clean_up()

            cc.SpriteFrameCache:getInstance():removeUnusedSpriteFrames()
            cc.Director:getInstance():getTextureCache():removeUnusedTextures()

        end

        local delay = cc.DelayTime:create(0.1)
        local call = cc.CallFunc:create(final_texture_clean_up)
        scene:runAction(cc.Sequence:create(delay, call))

        TRIGGER_EVENT("GAME_START", { level = scene_start_param.level_index })

  end)

end

function SceneBuilder:clearGameScene()
print("clear game scene")

  GameGlobals.Scene:removeFromParent()

  ObjectFactory:get_instance():clear()
  ResourceManager:get_instance():clear()
  ConfigManager:get_instance():clear()

  cc.Director:getInstance():getScheduler():unscheduleScriptEntry(SceneBuilder.recycle_entry)

end

return SceneBuilder