require "core.Bag_Data"
require "UI.Layer.ItemInfoLayer"
require "UI.Layer.CommonPopupLayer"
require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "core.TalkingDataManager"

BagLayer = class("BagLayer", function() return cc.Layer:create() end)

function BagLayer:ctor()

    self:setName("BagLayer")

	local widget = cc.CSLoader:createNode("bag.csb")
    self:addChild(widget, 1);
    self.widget = widget

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("bag.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    self.action:play("bag_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "bag_in_end" then

        elseif str == "bag_out_end" then
            self:removeFromParent()

        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

    local panel_item = image_dia:getChildByName("panel_item")
    self.panel_item = panel_item
    local panel_1 = panel_item:getChildByName(" panel_1")
    panel_item:setItemModel(panel_1:clone())
    panel_item:removeAllItems()

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    self:update_bag_info()

    --9.1
    local function handle_remove_limited_time_item(event)
        self:update_bag_info()

        local warningLayer = WarningLayer:create()
        warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("GameBackgroundLayer_2"))
        LayerManager:getInstance():add_layer_to_scene(warningLayer)
    end
    local listener1 = cc.EventListenerCustom:create(NotificationUtil.EventType.remove_limited_time_item_from_bag, handle_remove_limited_time_item)
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener1, self)
    --9.1

    self:add_role_spine()
end

function BagLayer:add_role_spine()

    local current_role_id = Player:getInstance():get_player_cur_role_id()
    local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)

    local role_spine = ResourceManager:getInstance():getSpineByKey(role_info.ui_spine_name, 1.0)
    role_spine:setAnimation(0, "bag", true)
    -- role_spine:setTimeScale(2)

    local node = self.widget:getChildByName("node")
    local node_role = node:getChildByName("node_role")
    node_role:addChild(role_spine)

    -- local rep = cc.RepeatForever:create(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function() role_spine:setAnimation(0, "bag", false) end)))

    -- self:runAction(rep)

end

function BagLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("bag_out", false)
    set_list_fade_out(self.panel_item)

    --9.1
    cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.remove_limited_time_item_from_bag)
end

function BagLayer:update_bag_info()
    local gameKernel = GameKernel:getInstance()
    local player = Player:getInstance()
    local player_bag_info_show_vector = player:get_player_bag_info_show_vector()
    local bag_Data = Bag_Data:getInstance()
    local bag_info_vector = bag_Data:get_bag_info_vector()

    local bag_count = get_table_length(player_bag_info_show_vector)
    local row = math.max(5, math.ceil(bag_count/4.0))
    
    for i=1,row do
        self.panel_item:pushBackDefaultItem()
    end


    local items_count = table.getn(self.panel_item:getItems())
    for i=1,items_count do
        local item = self.panel_item:getItem(i -1)
        for j=1,4 do
            local cur_index = (i-1) * 4 + j
            local item_child = item:getChildByName(string.format("item_%d", j))
            local image_item = item_child:getChildByName("image_item")
            local text_num = item_child:getChildByName("text_num")
            if bag_count >= cur_index then
                image_item:setVisible(true)
                text_num:setVisible(true)
                local bag_info = bag_Data:get_bag_info_by_id(player_bag_info_show_vector[cur_index].item_id)
                image_item:loadTexture(bag_info.res_key)
                text_num:setString(string.format("%d", player_bag_info_show_vector[cur_index].item_num))

                local item_Btn = ButtonWrap:create()
                item_Btn:wrap(image_item, 1)
                item_Btn:addTouchEndCallback(function()
                    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                    if bag_info.can_use then
                        local commonPopupLayer = CommonPopupLayer:create()
                        commonPopupLayer:disable_close()
                        commonPopupLayer:showTips(bag_info.item_name, bag_info.description, ResourceManager:getInstance():getTextStringByKey("BagLayer_4"), function()
                            if Player:getInstance():is_full_power() then
                                local commonPopupLayer_line = CommonPopupLayer:create()
                                commonPopupLayer_line:disable_close()
                                local str1 = ResourceManager:getInstance():getTextStringByKey("BagLayer_1")
                                local str2 = ResourceManager:getInstance():getTextStringByKey("BagLayer_2")
                                local str3 = ResourceManager:getInstance():getTextStringByKey("BagLayer_3")
                                commonPopupLayer_line:showMessage(str1, str2, str3, function()
                                    commonPopupLayer:close_self()
                                    commonPopupLayer_line:close_self()
                                end, nil)
                            else
                                
                                TalkingDataManager:getInstance():send("道具的使用次数统计", {["道具使用的item_id"] = bag_info.item_id})
                                TalkingDataManager:getInstance():sendOnUse(bag_info.item_id,1)
                                Player:getInstance():use_player_bag_info_by_id(bag_info.item_id, -1)
                                if player_bag_info_show_vector[cur_index].item_num == 0 then
                                    commonPopupLayer:close_self()
                                end
                                self:update_bag_info()
                            end
                        end, nil)
                        commonPopupLayer:setPosition(image_item:getParent():convertToWorldSpace(cc.p(image_item:getPosition())))
                    else
                        local commonPopupLayer = CommonPopupLayer:create()
                        commonPopupLayer:disable_close()
                        commonPopupLayer:disable_get()
                        if bag_info.is_limited_time then
                            local diff = os.time() - player_bag_info_show_vector[cur_index].limited_time_item_vector[1].get_time
                            local hour = math.floor((24*3600-diff)/3600)
                            local min = math.floor(((24*3600-diff)%3600)/60)
                            commonPopupLayer:showTips(bag_info.item_name,
                            string.format("%s\n剩余时间:%d时%d分",bag_info.description,hour,min), "", nil, nil)

                        else
                            commonPopupLayer:showTips(bag_info.item_name, bag_info.description, "", nil, nil)
                        end
                        
                        commonPopupLayer:setPosition(image_item:getParent():convertToWorldSpace(cc.p(image_item:getPosition())))
                    end

                end)

            else
                image_item:setVisible(false)
                text_num:setVisible(false)
            end
        end
    end

    if row <= 5 then
        self.panel_item:setTouchEnabled(false)
    else
        self.panel_item:setTouchEnabled(true)
    end
end

return BagLayer