require "UI.Layer.CardLayer"
require "core.Book_Data"
require "core.Mission_Data"
require "UI.Layer.CommonPopupLayer"
require "core.LayerManager"
require "UI.Widget.MaskLayer"

BookLayer = class("BookLayer", function() return cc.Layer:create() end)

function BookLayer:ctor()

    self:setName("book_layer")

	local widget = cc.CSLoader:createNode("book.csb")
    self:addChild(widget, 1);
    self.widget = widget

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("book.csb")
    widget:runAction(self.action)

    self.action:setCurrentFrame(0)
    self.action:play("book_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "book_in_end" then

        elseif str == "book_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

    local panel_book = image_dia:getChildByName("panel_book")
    self.panel_book = panel_book
    panel_book:setScrollBarEnabled(false)
    local book_list = panel_book:getChildByName("list")
    panel_book:setItemModel(book_list:clone())
    panel_book:removeAllItems()

    local player_book_info_vector = Player:getInstance():get_player_book_info_vector()
    local book_count = #player_book_info_vector

    for i=1,math.ceil(book_count/3.0) do
        panel_book:pushBackDefaultItem()
    end

    self.image_new = {}

    --延时加载
    local items_count = table.getn(panel_book:getItems())
    local function pageLoad(num)
        local count = num*3
        if count > items_count then
            count = items_count
        end
        for i=1+(num-1)*3,count do
            local item = panel_book:getItem(i -1)
            for j=1,3 do
                local cur_index = (i-1) * 3 + j
                local card = item:getChildByName(string.format("card_%d", j))
                if cur_index <= book_count then
                    local book_info = Book_Data:getInstance():get_book_info_by_id(player_book_info_vector[cur_index].book_id)

                    -- 角色和产生的关卡
                    local image_role = card:getChildByName("image_role")
                    image_role:loadTexture(book_info.res_key)

                    self.image_new[cur_index] = card:getChildByName("image_new")
                    if player_book_info_vector[cur_index].is_see then
                        self.image_new[cur_index]:setVisible(false)
                    end
                    if not player_book_info_vector[cur_index].is_get then
                        self.image_new[cur_index]:setVisible(false)
                    end

                    if player_book_info_vector[cur_index].is_get then
                        image_role:setVisible(true)
                        card:loadTexture(Book_Data:getInstance():get_rank_res_by_res(book_info.books_rank))
                    else
                        image_role:setVisible(false)
                        card:loadTexture(Book_Data:getInstance():get_book_unlock_res())
                    end

                    local mission_level = Mission_Data:getInstance():get_mission_info_by_id(book_info.generate_mission_id).mission_level
                    local text_num = card:getChildByName("text_num")
                    local no_str = string.format("NO.%d", mission_level)
                    text_num:setString(no_str)

                    local card_Btn = ButtonWrap:create()
                    card_Btn:wrap(card, 1)
                    card_Btn:addTouchEndCallback(function()
                        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                        if player_book_info_vector[cur_index].is_get then
                            player_book_info_vector[cur_index].is_see = true
                            local cardLayer = CardLayer:create()
                            cardLayer:set_cur_card(player_book_info_vector[cur_index].book_id)
                            cardLayer:set_show_type(1)
                            LayerManager:getInstance():add_layer_to_scene(cardLayer)
                            cardLayer:set_show_finish_callback(function()
                                --8.9   遍历self.image_new
                                for i=1,#self.image_new do
                                    if player_book_info_vector[i].is_see then
                                        self.image_new[i]:setVisible(false)
                                    end
                                end
                            end)
                        else
                            local commonPopupLayer = CommonPopupLayer:create()
                            commonPopupLayer:disable_close()
                            commonPopupLayer:setPosition(card:getParent():convertToWorldSpace(cc.p(card:getPosition())))
                            local content_text = string.format(ResourceManager:getInstance():getTextStringByKey("BookLayer_1"), mission_level)
                            commonPopupLayer:show_card_level(no_str, content_text, ResourceManager:getInstance():getTextStringByKey("BookLayer_2"), function()
                                local is_section_layer = cc.UserDefault:getInstance():getBoolForKey("is_section_layer", true)
                                local mission_info = Mission_Data:getInstance():get_mission_info_by_id(book_info.generate_mission_id)
                                if is_section_layer then
                                    cc.Director:getInstance():getRunningScene():switch_to_level_info(mission_info.mission_section, mission_info.mission_level)
                                else

                                    Player:getInstance():set_player_current_section(mission_info.mission_section)
                                    Player:getInstance():set_player_current_level(mission_info.mission_level)
                                    local levelInfoLayer2 = LevelInfoLayer:create()
                                    LayerManager:getInstance():add_layer_to_scene(levelInfoLayer2)
                                end
                                self:removeFromParent()
                                commonPopupLayer:removeFromParent()
                            end, nil)
                            local player_mission_info = Player:getInstance():get_player_mission_info_by_id(book_info.generate_mission_id)
                            if player_mission_info and player_mission_info.is_enable then
                                commonPopupLayer:set_get_dark(true)
                            else
                                commonPopupLayer:set_get_dark(false)
                            end

                        end
                    end)
                else
                    card:setVisible(false)
                end
            end
        end
    end
    local page = math.ceil(book_count/9)
    pageLoad(1)
    performWithDelay(self,function() 
        for i=2,page do
            performWithDelay(self, function() pageLoad(i) end, 0.016)
        end 
        end,0.25)

    performWithDelay(self,function() 
        local size = panel_book:getInnerContainerSize()
        panel_book:setInnerContainerSize(cc.size(size.width,size.height+50))
    end, 0.3)
    --

    -- 标题
    local text_title = image_dia:getChildByName("text_title")
    text_title:setPositionY(text_title:getPositionY()+40)
    local ui_text = ccui.Text:create("某些关卡满星通关必获得纪念册卡片", "images/font/zh_CN.TTF", 40)
    local width = cc.Director:getInstance():getVisibleSize().width
    ui_text:setPosition(cc.p(490, 1220))
    ui_text:setColor(ccc3(120,120,120))
    image_dia:addChild(ui_text)

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)



    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    self:add_role_spine()

    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.3), cc.CallFunc:create(function()
        
        TRIGGER_EVENT("BOOK_LAYER_SHOW")

    end)))

end

function BookLayer:add_role_spine()

    local current_role_id = Player:getInstance():get_player_cur_role_id()
    local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)

    local role_spine = ResourceManager:getInstance():getSpineByKey(role_info.ui_spine_name, 1.0)
    role_spine:setAnimation(0, "card", true)
    -- role_spine:setTimeScale(2)

    local node = self.widget:getChildByName("node")
    local node_role = node:getChildByName("node_role")
    node_role:addChild(role_spine)

    -- local rep = cc.RepeatForever:create(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function() role_spine:setAnimation(0, "card", false) end)))

    -- self:runAction(rep)

end

function BookLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("book_out",false)
    
    set_list_fade_out(self.panel_book)    
    self:set_all_card_is_see()
end

function BookLayer:set_all_card_is_see()
    local player_book_info_vector = Player:getInstance():get_player_book_info_vector()
    local book_count = #player_book_info_vector
    for i=1,book_count do
        if player_book_info_vector[i].is_get then
            player_book_info_vector[i].is_see = true
        end
        
    end
    GameKernel:getInstance():write_player_book_data()
end

return BookLayer