require "core.Book_Data"
require "core.LayerManager"
require "UI.Widget.MaskLayer"

CardLayer = class("CardLayer", function() return cc.Layer:create() end)

function CardLayer:ctor()

    self:setName("CardLayer")

    self.book_id = ""
    self.show_type = 1
    self.finish_callback = nil
    self.show_finish_callback = nil
    self.is_can_close = false
	local widget = cc.CSLoader:createNode("card.csb")
    self:addChild(widget, 1);

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("card.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "card_in_end" then

        elseif str == "card_out_end" then
            if self.show_finish_callback then
                self.show_finish_callback()
            end
            self:removeFromParent()
        elseif str == "get_card_in_end" then
            self.is_can_close = true
            self.action:play("get_card_show", true)

            self:send_card_aquire_msg()

        elseif str == "get_card_out_end" then
            if self.finish_callback then
                self.finish_callback()
            end
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_card = node:getChildByName("image_card")
    local text_num = image_card:getChildByName("text_num")
    local text_stage = node:getChildByName("text_stage")

    self.image_card = image_card
    self.text_num = text_num
    self.text_stage = text_stage


    local right_image = node:getChildByName("button_right")
    self.right_image = right_image
    local right_Btn = ButtonWrap:create()
    right_Btn:wrap(right_image, 1)
    right_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:next_card(true)
    end)

    local left_image = node:getChildByName("button_left")
    self.left_image = left_image
    local left_Btn = ButtonWrap:create()
    left_Btn:wrap(left_image, 1)
    left_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:next_card(false)
    end)

    image_card:setTouchEnabled(true)
    image_card:setSwallowTouches(true)
    local function touchEventCallback(sender,eventType)
        return true
    end
    image_card:addTouchEventListener(touchEventCallback)

    node:getChildByName("image_black"):setTouchEnabled(false)
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        if self.show_type == 1 then
            self:close_self()
        elseif self.show_type == 2 then
            if self.is_can_close then
                self:close_self()
            end
        end
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = widget:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, widget)

end

function CardLayer:send_card_aquire_msg()

    local event = cc.EventCustom:new("card_aquired")
    event.param = {}   
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function CardLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    if self.show_type == 1 then
        self.action:play("card_out", false)
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
    elseif self.show_type == 2 then
        self.action:play("get_card_out", false)
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
    end
end

function CardLayer:set_cur_card(book_id)
    self.book_id  = book_id
    local book_info = Book_Data:getInstance():get_book_info_by_id(book_id)
    self.image_card:loadTexture(Book_Data:getInstance():get_rank_res_by_res(book_info.books_rank))

    local image_role  = self.image_card:getChildByName("image_role")
    image_role:loadTexture(book_info.res_key)

    local mission_info = Mission_Data:getInstance():get_mission_info_by_id(book_info.generate_mission_id)
    self.text_num:setString(string.format(ResourceManager:getInstance():getTextStringByKey("CardLayer_1"), mission_info.mission_level))
    self.text_stage:setString(string.format(ResourceManager:getInstance():getTextStringByKey("CardLayer_2"), mission_info.mission_level))
end

function CardLayer:set_show_type(show_type)
    self.show_type = show_type
    if show_type == 1 then
        self.action:play("card_in", false)
        self.image_card:setTouchEnabled(true)
    elseif show_type == 2 then
        self.action:play("get_card_in", false)
        self.image_card:setTouchEnabled(false)

        self.right_image:setVisible(false)
        self.left_image:setVisible(false)
    end
end

function CardLayer:set_finish_callback(finish_callback)
    self.finish_callback = finish_callback
end
--8.9
function CardLayer:set_show_finish_callback(show_finish_callback)
    self.show_finish_callback = show_finish_callback
end
--8.9

function CardLayer:next_card(is_next)
    local player_book_info_vector = Player:getInstance():get_player_book_info_vector()
    local index = 1
    for i=1,#player_book_info_vector do
        index = i
        if self.book_id == player_book_info_vector[i].book_id then
            break
        end
    end
    if is_next then
        index = math.max((index + 1)%(#player_book_info_vector+1), 1)
        while not(player_book_info_vector[index].is_get) do
            index = math.max((index + 1)%(#player_book_info_vector+1), 1)
        end
    else
        index = index - 1
        if index == 0 then
            index = #player_book_info_vector
        end
        while not(player_book_info_vector[index].is_get) do
            index = index - 1
            if index == 0 then
                index = #player_book_info_vector
            end
        end
    end
    self:set_cur_card(player_book_info_vector[index].book_id)
    player_book_info_vector[index].is_see = true

end

return CardLayer