require "core.LayerManager"
require "UI.Widget.MaskLayer"

CdkeyLayer = class("CdkeyLayer", function() return cc.Layer:create() end)

function CdkeyLayer:ctor()

    self:setName("CdkeyLayer")
    self.__s_code = ""

    local widget = cc.CSLoader:createNode("cdkey.csb")
    self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("cdkey.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    self.action:play("setting_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "setting_in_end" then

        elseif str == "setting_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

    local cdkey_input = image_dia:getChildByName("cdkey_input")
    cdkey_input:setPlaceHolderColor(cc.c3b(156,136,112))
    cdkey_input:setMaxLength(3)
    cdkey_input:addEventListener(function(sender, eventType)
        --print(eventType)
        if eventType == ccui.TextFiledEventType.attach_with_ime then
            cdkey_input:setPlaceHolder("")
        elseif eventType == ccui.TextFiledEventType.detach_with_ime then
            print(sender:getString())
            self.__s_code = sender:getString()
        end
    end)

    local get_image = image_dia:getChildByName("button_get")
    local get_Btn = ButtonWrap:create()
    get_Btn:wrap(get_image, 1)
    get_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:request_redeem_code()
    end)

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function CdkeyLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("setting_out", false)
end

function CdkeyLayer:request_redeem_code()
    --request 兑换码
    if not self.__c_mask_layer then
        self.__c_mask_layer = CommunicatingMask:create()
        self:addChild(self.__c_mask_layer, 999)
    end
    self.__c_mask_layer:setEnabled(true)
    self.__c_mask_layer:setVisible(true)
    local rpc_param = {

        PLAYER_ID,
        self.__s_code,

    }

    RPC:call_bind("SHIN/code/redeem_2", rpc_param, self, function(status, response)
        self.__c_mask_layer:setEnabled(false)
        self.__c_mask_layer:setVisible(false)

        print("err_code"..status)
        if status == RPC.RPC_CALL_SUCCESS then

            local redeem_type = response.type
            self:dispatch_gift(tonumber(redeem_type))
        elseif status == RPC.RPC_CALL_FAILURE then
            local warningLayer = WarningLayer:create()
            warningLayer:set_text_string("兑换码无效")
            LayerManager:getInstance():add_layer_to_scene(warningLayer)
        else

            print("request error")
            print(status)
            local warningLayer = WarningLayer:create()
            warningLayer:set_text_string("请检查你的网络连接！")
            LayerManager:getInstance():add_layer_to_scene(warningLayer)

        end

    end)
end

function CdkeyLayer:dispatch_gift(gift_type)

    local player = Player:getInstance()
    local gift_title_str = ""
    local gift_list_str = ""
    local redeem_code_reward_info_vector = require "script.redeem_code_reward_info"
    if type(gift_type) == "number" then

        local info = redeem_code_reward_info_vector[gift_type]
        gift_title_str = info.reward_name
        local item_vector = info.item_vector
        for k,v in pairs(item_vector) do
            player:add_player_bag_info_by_id(v.item_id, v.item_num)
            local bag_info = Bag_Data:getInstance():get_bag_info_by_id(v.item_id)
            gift_list_str = string.format("%s%s*%d\n", gift_list_str, bag_info.item_name, v.item_num)
        end

    end

    local commonPopupLayer = CommonPopupLayer:create()
    commonPopupLayer:disable_close()
    local popup_title = gift_title_str
    local popup_content = gift_list_str
    local btn_text = "确定"
    commonPopupLayer:showMessage(gift_title_str, popup_content, btn_text, function()
        commonPopupLayer:close_self()
    end, nil)

end

function CdkeyLayer:onEnter()
    RPC:register_target(self)
end

function CdkeyLayer:onExit()
    RPC:unregister_target(self)
end

return CdkeyLayer