require "core.LayerManager"
require "UI.Widget.MaskLayer"

CommonDialogLayer = class("CommonDialogLayer", function() return cc.Layer:create() end)

function CommonDialogLayer:ctor()

    self:setName("CommonDialogLayer")

	local widget = cc.CSLoader:createNode("common_dialog.csb")
	self:addChild(widget)
    self.widget = widget

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("common_dialog.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    self.action:play("dia_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "dia_in_end" then

        elseif str == "dia_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local panel_1 = node:getChildByName("panel_1")
    local box_des = panel_1:getChildByName("box_des")
    self.box_des = box_des

    self.panel = {}
    for i=1,4 do
        self.panel[i] = panel_1:getChildByName(string.format("panel_%d", i-1))
        self.panel[i]:setVisible(false)
    end

	local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        self:close_self()
        
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    LayerManager:getInstance():add_layer_to_scene(self)
end

function CommonDialogLayer:show_on_right(table)
    self.box_des:setFlippedX(true)
    self.widget:setPosition(cc.p(100,30))
    self.widget:setScale(0.68)

    self:init_panel(table)
end

function CommonDialogLayer:show_on_left(table)
    self.box_des:setFlippedX(false)
    self.widget:setPosition(cc.p(-150,120))
    self.widget:setScale(1.0)

    self:init_panel(table)
end

function CommonDialogLayer:init_panel(table)
    local count = get_table_length(table)
    self.panel[count]:setVisible(true)
    for k,v in pairs(table) do
        local item_id = v.item_id
        local item_num = v.item_num
        local item_image = self.panel[count]:getChildByName(string.format("item_%d", k))
        local text_num = item_image:getChildByName("text_num")

        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(item_id)
        if item_id == "item_02" then
            item_image:loadTexture("images/item/item_8.png")
        elseif item_id == "item_03" then
            item_image:loadTexture("images/item/item_9.png")
        else
            item_image:loadTexture(bag_info.res_key)
        end
        if item_num > 1000 then
            local num = math.floor(item_num/1000)
            text_num:setString(string.format("x%dk",num))
        else
            text_num:setString(string.format("x%d",item_num))
        end
    end
end

function CommonDialogLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("dia_out", false)
end

function CommonDialogLayer:onEnter()

end

function CommonDialogLayer:onExit()
end

return CommonDialogLayer