require "core.LayerManager"
require "UI.Widget.MaskLayer"

DeadLineLayer = class("DeadLineLayer", function() return cc.Layer:create() end)

function DeadLineLayer:ctor()

    self.__s_show_type = "SHOW_NONE"
    self.close_callback = nil

end

function DeadLineLayer:init_with_time_stamp(time_stamp, days_available)

    self.__n_start_time = time_stamp
    self.__n_duration = days_available

    if cc.UserDefault:getInstance():getBoolForKey("dead_line_reached", false) or self:check_dead_line() then

        cc.UserDefault:getInstance():setBoolForKey("dead_line_reached", true)

        self.__s_show_type = "SHOW_DEAD_LINE"

    else

        -- if not cc.UserDefault:getInstance():getBoolForKey("welcome_showed", false) then

            cc.UserDefault:getInstance():setBoolForKey("welcome_showed", true)

            self.__s_show_type = "SHOW_WELCOME"

        -- end

    end

    

end

function DeadLineLayer:create_notice_layer()
    self:setName("DeadLineLayer")

    local widget = cc.CSLoader:createNode("notice.csb")
    self:addChild(widget, 1);
    self.widget = widget

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("notice.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "notice_in_end" then

        elseif str == "notice_out_end" then

            self:removeFromParent()

        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local image_dia_1 = image_dia:getChildByName("image_dia_1")
    self.text_text = image_dia_1:getChildByName("text_text")
    self.text_text_end = image_dia_1:getChildByName("text_text_end")
    self.text_text:setString("《蜡笔小新糖果世界》内测火爆开启中，\n蜡笔小新正版IP授权，创新玩法物理连线\n消除。\n\n欢迎各位萌友加入内测qq群，多多吐槽，\n祝大家游戏愉快。\n\n\n\n\n\n测试时间：4月19日0点 至  4月30日24点\n内测qq群：20855149\n\n\n\n《蜡笔小新糖果世界》运营团队")

end

function DeadLineLayer:check_dead_line()

    local current_time_stamp = os.time()

    local days_past = math.floor((current_time_stamp - self.__n_start_time) / (3600 * 24))

    if days_past >= 0 and days_past < self.__n_duration then

        return false

    else

        return true

    end

end

function DeadLineLayer:show()

    

    if self.__s_show_type == "SHOW_WELCOME" then

        self:create_notice_layer()
        self:show_welcome()
        self:append_mask()


    elseif self.__s_show_type == "SHOW_DEAD_LINE" then
        self:create_notice_layer()
        self:show_dead_line()
        self:append_mask()

    end 

end

function DeadLineLayer:show_welcome()
    self.text_text_end:setVisible(false)
    self.action:play("notice_in", false)
    
end

function DeadLineLayer:show_dead_line()
    self.text_text:setVisible(false)
    self.action:play("notice_in", false)

    self.close_callback = nil
end

function DeadLineLayer:set_close_callback(callback)
    self.close_callback = callback

end

function DeadLineLayer:close_self()
    if self.__s_show_type == "SHOW_DEAD_LINE" then
        return
    end
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("notice_out", false)

    if self.close_callback then
        self.close_callback()
        self.close_callback = nil
    end

end

function DeadLineLayer:append_mask()

    self.__c_mask = cc.LayerColor:create(cc.c4b(0, 0, 0, 0))

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    listenner:registerScriptHandler(function(touch, event)
        TRIGGER_EVENT("TUTORIAL_OK_PRESSED")
    end, cc.Handler.EVENT_TOUCH_ENDED )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self.__c_mask)

    self:addChild(self.__c_mask)    

end

return DeadLineLayer

