local Dialog = require "UI.Widget.Dialog"


DialogLayer = class("DialogLayer", function() return cc.Layer:create() end)

function DialogLayer:ctor()

	self:setName("DialogLayer")

	self.__c_bg_layer = cc.LayerColor:create(cc.c4b(0, 0, 0, 0))
	self:addChild(self.__c_bg_layer)

	self.__n_touch_enabled = false
	self.__n_current_progress = 0
	self.__t_dialog_alive = {}
	self.__t_config = {}
	self.__f_finish_callback = function() end

	self:register_touch_handler()

end

function DialogLayer:enable_touch()

	self.__n_touch_enabled = true

end

function DialogLayer:disable_touch()

	self.__n_touch_enabled = false

end

function DialogLayer:register_touch_handler()

	function touch_down()

		return true

	end

	function touch_up()

		if self.__n_touch_enabled then

			self:touch_event_handler()

		end

	end

	local touch_listener = cc.EventListenerTouchOneByOne:create()
    touch_listener:setSwallowTouches(true)
    touch_listener:registerScriptHandler(touch_down,cc.Handler.EVENT_TOUCH_BEGAN)
    touch_listener:registerScriptHandler(touch_up,cc.Handler.EVENT_TOUCH_ENDED)
    self:getEventDispatcher():addEventListenerWithSceneGraphPriority(touch_listener, self)

end

function DialogLayer:init_with_config(config)

	self.__t_config = config or {}

end

function DialogLayer:start()

	self:enter_with_animation()

end

function DialogLayer:touch_event_handler()

	self:cost_alive_dialog()
	self:withdraw_dying_dialog()
	self:carry_on_dialog()

end

function DialogLayer:carry_on_dialog()

	self.__n_current_progress = self.__n_current_progress + 1

	if self.__n_current_progress > #self.__t_config then

		self:terminate()

	else

		self:pop_new_dialog(self.__t_config[self.__n_current_progress])

	end

end

function DialogLayer:cost_alive_dialog()

	for key, value in pairs(self.__t_dialog_alive) do

		value.life_span = value.life_span - 1

	end	

end

function DialogLayer:withdraw_dying_dialog()

	for key, value in pairs(self.__t_dialog_alive) do

		if value.life_span <= 0 then

			value.dialog:with_draw()
			self.__t_dialog_alive[key] = nil

		end

	end

end

function DialogLayer:pop_new_dialog(config)

	local dialog = self.__t_dialog_alive[config.id] and self.__t_dialog_alive[config.id].dialog or Dialog:create()
	dialog:append_to(self)
	dialog:set_config(config)
	dialog:pop()

	self.__t_dialog_alive[config.id] = {life_span = config.life_span, dialog = dialog}

end

function DialogLayer:terminate()

	self:withdraw_all_dialog()
	self:exit_with_animation()

end

function DialogLayer:withdraw_all_dialog()

	for key, value in pairs(self.__t_dialog_alive) do

		value.dialog:with_draw()
		self.__t_dialog_alive[key] = nil

	end

end

function DialogLayer:enter_with_animation()

	local fade_in = cc.FadeTo:create(0.15, 200)
	self.__c_bg_layer:runAction(fade_in)

	local delay = cc.DelayTime:create(0.35)
	local sequence = cc.Sequence:create(delay, cc.CallFunc:create(function() self:carry_on_dialog() self:enable_touch() end))
	self:runAction(sequence)

end

function DialogLayer:exit_with_animation()

	local fade_out = cc.FadeTo:create(0.35, 0)
	self.__c_bg_layer:runAction(fade_out)

	local delay = cc.DelayTime:create(0.35)
	local sequence = cc.Sequence:create(delay, cc.CallFunc:create(function() self.__f_finish_callback() end), cc.RemoveSelf:create())
	self:runAction(sequence)

end

function DialogLayer:set_finish_callback(call_back)

	self.__f_finish_callback = call_back

end

return DialogLayer

