require "UI.Layer.BuyItemLayer"
require "UI.Widget.SweepLightLayer"
require "core.LayerManager"
require "UI.Layer.WinLayer"
require "UI.Layer.LoseLayer"
require "UI.Layer.ReliveLayer"
require "UI.Layer.PauseLayer"
require "UI.Layer.WarningLayer"
require "UI.Layer.LuckyMoveLayer"
require "core.Prop_Data"
require "core.TalkingDataManager"

local StateMachine = require "GameEx.Foundations.Utils.StateMachineRevised"
local GameGlobals = require "GameEx.GameGlobals"
local ResourceManagerEx = require "GameEx.Foundations.ResourceManager"

GameBackgroundLayer = class("GameBackgroundLayer", function() return cc.Layer:create() end)

GameBackgroundLayer.Role_Action = {
    Bag = 1, --bag待机, star普通消除, shop15个以上高级消除, card超级奖励, HP步数不足
    Star = 2,
    Shop = 3,
    Card = 4,
    HP = 5,
}



function GameBackgroundLayer:ctor()

    self:setName("game_ui_layer")
    -- 读取关卡内容，关卡步数以及目标
    -- self.total_step = GameGlobals.Gameplay.Config.global_setting.move_limit
    self.total_step  = 20
    self.count_step = 1
    self.surplus_step = 1
    self.game_time = 0

    self.can_open_bag = true
    local function open_reward_bag()
        
        if self.can_open_bag then
            self.can_open_bag = false

            --打开福袋逻辑
            -- local pauseLayer = PauseLayer:create()
            -- LayerManager:getInstance():add_layer_to_scene(pauseLayer)
            --113 宝藏礼包
            local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Treasure)
            LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)

        end

    end

    local listener = cc.EventListenerCustom:create(NotificationUtil.EventType.open_reward_bag_enven, open_reward_bag)
    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, self)
    
    local function update_game_time()
        self.game_time = self.game_time + 1
    end
    self.scheduleScriptEntryID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update_game_time, 1.0, false)

    self.goal = {}
    self.total_goal = {}

    self.item_click_callback = nil
    self.item_cancel_callback = nil
    self.is_using = false

    self.relive_times = 1
    self.relive_bomb_times = 1

    -- local goal_data = GameGlobals.Gameplay.Config.global_setting.req
    local goal_data = {
    }
    for key, value in pairs(goal_data) do
        if not (value == 0) then
            self.goal[key] = value
            self.total_goal[key] = value
            if not (type(value) == "number") then
                print(ResourceManager:getInstance():getTextStringByKey("GameBackgroundLayer_1"))
            end
        end

    end

    -- 先初始化大小，减少消耗
    -- 目标糖果内容
    self.goal_text = {nil, nil, nil, nil, nil}
    self.goal_image = {nil, nil, nil, nil, nil}

    -- 道具内容
    self.item_text = {nil, nil, nil, nil, nil}
    self.item_acount = {[1] = 5, [2] = 5, [3] = 5, [4] = 5, [5] = 5}
    self.item_image = {nil, nil, nil, nil, nil}
    self.item_btn_vecotr = {nil, nil, nil, nil, nil}
    -- self.item_tips1 = {nil, nil, nil, nil, nil}
    -- self.item_tips2 = {nil, nil, nil, nil, nil}

    -- 游戏提示
    self.game_lose = false
    self.step_tips = false 
    self.game_win = false
    self.role_is_acting = false

    -- 游戏分数以及星星
    self.score = 0
    self.star_score = {}
    

    local mission_info = Mission_Data:getInstance():get_mission_info_by_level(Player:getInstance():get_player_current_level())
    local star_score = mission_info.star_goal
    for key, value in ipairs(star_score) do
        if not (value == 0) then
            self.star_score[key] = value
        end

    end

    self.empty_star = {nil, nil, nil}
    self.full_star = {nil, nil, nil}

    -- boss 血量、技能步数
    self.boss_hp_percent = 0
    self.boss_skill_step = ""
    self.boss_progress_blood = nil
    self.boss_fnt_step = nil
    self.boss_hp_state = 1

    -- 游戏变量
    self.widget = nil
    self.action = nil
    self.fnt_step = nils
    self.text_score = nil
    self.score_loading = nil
    self.image_role = nil
    self.role_spine = nil

    -- 游戏资源 
    self.rm = ResourceManager:getInstance()
    self.candy_res = { 
        ["red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RED"),
        ["green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_GREEN"),
        ["blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BLUE"),
        ["yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_YELLOW"),
        ["pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PINK"),
        ["white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_WHITE"),
        ["ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_ICE"),
        ["key"] = self.rm:getPicturePathByKey("IMAGES_CANDY_KEY"),
        ["fan"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FAN"),
        ["random"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RANDOM"),
        ["panda"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PANDA"),
        ["moveable_ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_MOVEABLE_ICE"),
        ["fenced"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FENCE"),
        ["chocolate"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CHOCOLATE"),
        ["bomb"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BOMB"),
        ["ten"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CROSS"),
        ["lightning"] = self.rm:getPicturePathByKey("IMAGES_CANDY_LIGHTNING"),
        ["conch"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CONCH"),
        ["coco"] = self.rm:getPicturePathByKey("IMAGES_CANDY_COCO"),
        ["transform_red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_RED"),
        ["transform_green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_GREEN"),
        ["transform_blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_BLUE"),
        ["transform_yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_YELLOW"),
        ["transform_pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_PINK"),
        ["transform_white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_WHITE"),
    }

    self.attack_img_map = {
        ["ChocolateSugar"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CHOCOLATE"),
        ["MoveableIceSugar"] = self.rm:getPicturePathByKey("IMAGES_CANDY_MOVEABLE_ICE"),
        ["IceSugar"] = self.rm:getPicturePathByKey("IMAGES_CANDY_ICE"),
        ["PandaSugar"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PANDA"),
        ["TimingBombSugar"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_RED"),
    }

    -- 开始游戏动画判断
    self.can_play = false
    self.can_touch = false
    self.game_aim_ing = false

    self.game_start_finish_callback = function() end
    self.relive_callback = function() end
    self.relive_bomb_callback = function() end
    self.die_bomb_callback = function() end

    self.__n_full_hp = 0

    self.role_state = "normal"

    self.__t_score_pool = {}
    self.__n_free_score = 0

    for i = 1, 30 do

        local score_text = cc.CSLoader:createNode("score_num.csb")
        -- score_text:setPosition(pos)
        score_text:setLocalZOrder(99)
        score_text:getChildByName("node"):getChildByName("text_1"):setVisible(false)
        -- score_text:getChildByName("node"):getChildByName("text_5"):setString(tostring(score))
        score_text:getChildByName("node"):getChildByName("text_3"):setVisible(false)
        score_text:getChildByName("node"):getChildByName("text_5"):setVisible(true)
        score_text:retain()

        local action = cc.CSLoader:createTimeline("score_num.csb")
        action:retain()
        score_text.action = action
        score_text.tag = i

        self.__t_score_pool[i] = score_text
        self.__n_free_score = i

    end

end

function GameBackgroundLayer:setGameStartFinishCallback(callback)

    self.game_start_finish_callback = callback

end

function GameBackgroundLayer:setGoal(t_level_config)

    self.total_step = t_level_config.move_limit

    self.count_step = t_level_config.move_limit

    for key, value in pairs(t_level_config.reqs) do

        if not (value == 0) then
            self.goal[key] = value
            self.total_goal[key] = value
            if not (type(value) == "number") then
                print(ResourceManager:getInstance():getTextStringByKey("GameBackgroundLayer_1"))
            end
        end

    end

    self:showGoal()
    self.fnt_step:setString(tostring(self.total_step))

end


function GameBackgroundLayer:init(game_type)

    self.add_step_node = cc.CSLoader:createNode("game_step_add.csb")
    self.add_step_node:setContentSize(cc.Director:getInstance():getVisibleSize())
    self.add_step_node:setPosition(cc.p(cc.Director:getInstance():getVisibleSize().width / 2 + 380, 120))
    self:addChild(self.add_step_node, 3);
    ccui.Helper:doLayout(self.add_step_node)

    self.add_step_action = cc.CSLoader:createTimeline("game_step_add.csb")
    self.add_step_node:runAction(self.add_step_action)

    self.indication_node = cc.CSLoader:createNode("game_step_add.csb")
    self.indication_node:setContentSize(cc.Director:getInstance():getVisibleSize())
    self.indication_node:setPosition(cc.p(950, 90))
    self:addChild(self.indication_node, 3);
    ccui.Helper:doLayout(self.indication_node)

    self.indication_action = cc.CSLoader:createTimeline("game_step_add.csb")
    self.indication_node:runAction(self.indication_action)

    self.is_boss = is_boss

    if game_type == "boss" then

        self.is_boss = true

        self.widget = cc.CSLoader:createNode("game_boss.csb")
        self:addChild(self.widget, 1);
        self.widget:setContentSize(cc.Director:getInstance():getVisibleSize())
        ccui.Helper:doLayout(self.widget)

        -- 开场动画
        self.action = cc.CSLoader:createTimeline("game_boss.csb")
        self.widget:runAction(self.action)
        self.action:gotoFrameAndPlay(0, 80, false)

    elseif game_type == "normal" then

        self.widget = cc.CSLoader:createNode("game.csb")
        self:addChild(self.widget, 1);
        self.widget:setContentSize(cc.Director:getInstance():getVisibleSize())
        ccui.Helper:doLayout(self.widget)

        -- 开场动画
        self.action = cc.CSLoader:createTimeline("game.csb")
        self.widget:runAction(self.action)

        self:runAction(cc.Sequence:create(cc.DelayTime:create(2), cc.CallFunc:create(function()

            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_cheer"), false)

        end)))

        self.widget:getChildByName("panel_score"):getChildByName("image_score_dia"):setLocalZOrder(3)

    elseif game_type == "tournament" then

        self.widget = cc.CSLoader:createNode("game_weekgame.csb")
        self:addChild(self.widget, 1);
        self.widget:setContentSize(cc.Director:getInstance():getVisibleSize())
        ccui.Helper:doLayout(self.widget)

        -- 开场动画
        self.action = cc.CSLoader:createTimeline("game_weekgame.csb")
        self.widget:runAction(self.action)

        self:runAction(cc.Sequence:create(cc.DelayTime:create(2), cc.CallFunc:create(function()

            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_cheer"), false)

        end)))

        self.widget:getChildByName("panel_score"):getChildByName("image_score_dia"):setLocalZOrder(3)

    end

    self.game_type = game_type

    local game_aim = cc.Sequence:create(cc.DelayTime:create(3.5), cc.CallFunc:create(function(sender)
        self.game_aim_ing = true
        self.action:gotoFrameAndPlay(80, 110, false)
    end))
    game_aim:setTag(234)
    self.widget:runAction(game_aim)

    -- 散出物体
    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()

        if str == "candy" then
            local size = cc.Director:getInstance():getVisibleSize()
            local effect = self.rm:getSpineByKey("SPINE_DISPERSE", 1.0)
            effect:setAnimation(0, "disperse", false)
            effect:registerSpineEventHandler(function (event)
                effect:retain()
                effect:removeFromParent()
                effect:autorelease()
            end, sp.EventType.ANIMATION_END)
            effect:setPosition(cc.p(size.width/2, size.height/2))
            self:addChild(effect, 3)
        elseif str == "game_in_end" then

            self.can_touch = true
            cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.capture_board_event)
            cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.capture_board_event)
        elseif str == "game_aim_end" then

            local delay = cc.DelayTime:create(0.0)
            local call = cc.CallFunc:create(function()

                self.can_play = true
                self:setAllButtonEnabel(true)
                self:game_start_finish_callback()

                if is_boss then

                    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(function()

                        self.action:play("boss_progress_in", false)

                        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_boss_1"), false)

                    end)))

                else

                    self:play_sub_step_animation()

                end

            end)
            local sequence = cc.Sequence:create(delay, call)
            self:runAction(sequence)

            local image_bottle = self.widget:getChildByName("panel_score"):getChildByName("image_bottle")
            if image_bottle then
                self.__t_bottle_original_position = cc.p(image_bottle:getPosition())
            end

        elseif str == "resetting_end" then
            local visible_size = cc.Director:getInstance():getVisibleSize()
            local bling_particle = ResourceManager:getInstance():getParticleByKey("PARTICLE_BLING")
            bling_particle:setAutoRemoveOnFinish(true)
            bling_particle:setPosition(cc.p(visible_size.width/2, visible_size.height/2))
            self:addChild(bling_particle, 5)

            if self.total_step <= 5 then
                self.step_tips = true
                self.action:play("step_less", true)
                self:play_sub_step_animation()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("5step_Audio"), false)
            end
        elseif str == "bottle_out_end" then
            self:add_win_bottle()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    -- 玩家角色
    if not self.is_boss then

        self.image_role = self.widget:getChildByName("image_role")

        local current_role_id = Player:getInstance():get_player_cur_role_id()
        local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)

        local clipping_mask = cc.Sprite:create("images/game/in_game/upper_mask.png")
        clipping_mask:setPosition(cc.p(0, 190))

        local clipping_node = cc.ClippingNode:create()
        clipping_node:setStencil(clipping_mask)
        clipping_node:setAlphaThreshold(0.1)
        self.image_role:addChild(clipping_node, 1)
        self.clipping_node = clipping_node

        self.role_spine = self.rm:getSpineByKey(role_info.ui_spine_name, 0.6)
        self.role_spine:setScale(1.5)
        self.role_spine:setAnimation(0, "bag", true)
        self.role_spine:setPosition(cc.p(get_node_width(self.image_role)/2 , 0))
        self.role_spine:setTimeScale(0.6)
        clipping_node:addChild(self.role_spine, 1)

    end

    --玩家的行走步数
    local image_dia = self.widget:getChildByName("image_dia")
    local image_step_dia = image_dia:getChildByName("image_step_dia")
    self.fnt_step =  image_step_dia:getChildByName("fnt_step")
    self.fnt_step:setString(tostring(self.total_step))

    -- image_dia:getChildByName("image_dia_1"):setVisible(false)

    -- 玩家当前分数
    local panel_score = self.widget:getChildByName("panel_score")
    if self.is_boss == false then
        local image_bottle = panel_score:getChildByName("image_bottle")
    end
    local image_boss_goal = panel_score:getChildByName("image_boss_goal")

    -- 技能
    local image_skill_dia = panel_score:getChildByName("image_skill_dia")
    local image_skill = image_skill_dia:getChildByName("image_skill")
    local skill_res = "images/role_info/skill.png"

    image_skill:setTexture(skill_res)

    --进度条
    local new_sprite = cc.Sprite:create(skill_res)
    new_sprite:setLocalZOrder(1)
    new_sprite:setColor(cc.c3b(150, 150, 150))

    local skill_progress = cc.ProgressTimer:create(new_sprite)
    skill_progress:setType(cc.PROGRESS_TIMER_TYPE_RADIAL)
    skill_progress:setPosition(cc.p(get_node_width(image_skill)/2, get_node_height(image_skill)/2))
    skill_progress:setReverseDirection(true)
    image_skill:addChild(skill_progress, 2)
    self.skill_progress = skill_progress
    self.energy_point = 0
    self.total_energy_point = 5
    skill_progress:setPercentage(100)

    -- 分数进度条
    local image_score_dia = panel_score:getChildByName("image_score_dia")
    local image_progress_bar = image_score_dia:getChildByName("image_progress_bar")
    self.score_loading = image_progress_bar:getChildByName("progress_score")
    self.score_loading:setPercent(0)

    -- 分数
    self.text_score = image_progress_bar:getChildByName("text_score")
    self.text_score:setString(self.score)

    -- 目标星星
    local panel_star = image_progress_bar:getChildByName("panel_star")
    local empty_star = panel_star:getChildByName("image_star_empty"):setVisible(false)
    local full_star = panel_star:getChildByName("image_star_full"):setVisible(false)
    self.empty_star[1] = cc.Sprite:create("game/star_1.png")
    self.full_star[1] = cc.Sprite:create("game/star_0.png")

    self.empty_star[2] = cc.Sprite:create("game/star_1.png")
    self.full_star[2] = cc.Sprite:create("game/star_0.png")

    self.empty_star[3] = cc.Sprite:create("game/star_1.png")
    self.full_star[3] = cc.Sprite:create("game/star_0.png")

    for key, value in ipairs(self.star_score) do
        image_progress_bar:addChild(self.empty_star[key])
        image_progress_bar:addChild(self.full_star[key])
        
        local progress_width = image_progress_bar:getContentSize().width
        self.empty_star[key]:setPosition(cc.p(progress_width * self.star_score[key] / self.star_score[3], 0))
        self.full_star[key]:setPosition(cc.p(self.empty_star[key]:getPosition()))

        self.full_star[key]:setVisible(false)
    end

    -- boss 血量进度条
    if  self.is_boss then
        local boss_progress = image_dia:getChildByName("boss_progress")
        self.boss_progress_blood = boss_progress:getChildByName("progress_blood")
        self.boss_progress_blood_text = boss_progress:getChildByName("text_blood")

        local image_turn_based = image_dia:getChildByName("image_turn_based")
        self.boss_fnt_step = image_turn_based:getChildByName("text_turn_based")
        self.attack_number = image_turn_based:getChildByName("attack_text")
        self.attack_image = image_turn_based:getChildByName("attack_img")
    end

    -- 暂停按钮
    local pauseImage = image_dia:getChildByName("button_pause")
    local pauseBtn = ButtonWrap:create()
    pauseBtn:wrap(pauseImage, 1)
    pauseBtn:addTouchEndCallback(function(sender)
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        local pauseLayer = PauseLayer:create()
        LayerManager:getInstance():add_layer_to_scene(pauseLayer)
        
    end)
    self.pauseBtn = pauseBtn
    
    -- 游戏道具按钮
    local panel_item = self.widget:getChildByName("panel_item")
    --传入的道具参数
    self.item_map = {
        [1] = "item_301", [2] = "item_302", [3] = "item_303", [4] = "item_304", [5] = "item_305",
    }

    self.limited_time_item_map = {
        [1] = "item_501", [2] = "item_502", [3] = "item_503", [4] = "item_504",
    }
    for i=1,4 do
        local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.limited_time_item_map[i])
        
        if player_bag_info.item_num > 0 then
            self.item_map[i] = self.limited_time_item_map[i]
        end
    end

    --9.1
    local function handle_remove_limited_time_item(event)
        local item_map = {
            [1] = "item_301", [2] = "item_302", [3] = "item_303", [4] = "item_304",
        }
        for i=1,4 do
            local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.limited_time_item_map[i])
            
            if player_bag_info.item_num <= 0 then
                self.item_map[i] = item_map[i]
            end
            local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.item_map[i])
            self.item_acount[i] = player_bag_info.item_num
            print("···::"..self.item_acount[i])
            self.item_text[i]:setString(self.item_acount[i])

            local warningLayer = WarningLayer:create()
            warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("GameBackgroundLayer_2"))
            LayerManager:getInstance():add_layer_to_scene(warningLayer)
        end
    end
    local listener2 = cc.EventListenerCustom:create(NotificationUtil.EventType.remove_limited_time_item_from_game, handle_remove_limited_time_item)
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener2, self)
    --9.1

    self.item_skill_map = {}
    self.in_game_item = {}

    --7.29  获得最新的最高通关的关卡号，判断是否解锁道具
    local newest_mission_id = Player:getInstance():get_player_newest_mission_id()
    local mission_info = Mission_Data:getInstance():get_mission_info_by_id(newest_mission_id)
    local newest_level = mission_info.mission_level

    for i=1,5 do
        local prop_info = Prop_Data:getInstance():get_prop_info_by_id(self.item_map[i])
        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(prop_info.item_id)
        self.item_skill_map[i] = bag_info.skill_name
        if newest_level >= prop_info.Unlock_level then
            self.in_game_item[i] = bag_info.skill_name
        else
            self.in_game_item[i] = "LOCKED"
        end
    end    

    -- self.item_skill_map = {
    -- [1] = "HammerSkill",[2] = "BombSkill",[3] = "CandySkill",[4] = "LineSkill",[5] = "MoveSkill",
    -- }   --7.14

    --7.18  判断是否锁道具
    -- local mission_info = Mission_Data:getInstance():get_mission_info_by_level(Player:getInstance():get_player_current_level())
    -- self.in_game_item = nil
    -- if mission_info.in_game_item then
    --     self.in_game_item = mission_info.in_game_item
    -- else
    --     self.in_game_item = { "HammerSkill", "BombSkill", "CandySkill", "LineSkill", "MoveSkill" }
    -- end
    --7.18
    self.use_times = {"3","3","3","3","3"}

    for i=1, 5 do
        -- 道具image
        local itemBtn = panel_item:getChildByName(string.format("button_item_%d", i - 1))
        self.item_image[i] = itemBtn
        -- 道具按钮
        local itemBtnWrap = ButtonWrap:create()
        itemBtnWrap:wrap(itemBtn, 1)
        itemBtnWrap:addTouchEndCallback(function()
            if self.item_image[i].original_position then
                self:stop_item_highlight_animation(i)
            end            

            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
       
            local event = cc.EventCustom:new("item_pressed")
            event.param = {}   
            cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

            --7.19
            if self.use_times[i] == 0 and self.item_acount[i] > 0 then
                local warningLayer = WarningLayer:create()
                warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("itemUseTimes"))
                LayerManager:getInstance():add_layer_to_scene(warningLayer)
            else
                self:onPropEventCallback(i)
            end
            --7.19

            TRIGGER_EVENT("INGAME_ITEM_SELECT", {id = tostring(i)})

        end)
        self.item_btn_vecotr[i] = itemBtnWrap

        -- 按钮图片
        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[i])
        local image_item = itemBtn:getChildByName(string.format("image_item_%d", i - 1))
        image_item:setTexture(bag_info.res_key)

        local image_mark = itemBtn:getChildByName("image_mark")

        
        local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.item_map[i])
        self.item_acount[i] = player_bag_info.item_num
        

        -- 按钮文本
        self.item_text[i] = image_mark:getChildByName("text_num") 
        self.item_text[i]:setString(self.item_acount[i])

        if self.item_acount[i] == 0 then
            self.item_text[i]:setVisible(false)
            self.item_image[i]:getChildByName("image_mark"):setTexture("images/game/item_buy.png")
        end
        local prop_info = Prop_Data:getInstance():get_prop_info_by_id(self.item_map[i])
        self.use_times[i] = prop_info.use_times

        -- 道具没有解锁状态
        if self.in_game_item[i] == "LOCKED" then
            itemBtn:loadTexture("images/item/item_lock.png")
            image_item:setVisible(false)
            image_mark:setVisible(false)
            self.item_text[i]:setVisible(false)
            itemBtnWrap:disable()
        end

    end

    -- 吃掉点击事件
    self:setAllButtonEnabel(false)
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        if self.can_play then
            return false
        else
            if self.can_touch then
                self.widget:stopActionByTag(234)
                if self.game_aim_ing == false then
                    self.game_aim_ing = true
                    self.action:gotoFrameAndPlay(80, 110, false)
                end
            end
            return true
        end
    end, cc.Handler.EVENT_TOUCH_BEGAN )

    local function onKeyReleased(keyCode, event)
        if keyCode == cc.KeyCode.KEY_BACK then
            local top_layer = LayerManager:getInstance():get_top_layer()
            if top_layer then
                if top_layer.close_self then
                    top_layer:close_self()
                end
                return
            else
                local pauseLayer = PauseLayer:create()
                LayerManager:getInstance():add_layer_to_scene(pauseLayer)
            end
        end
    end
    local listener1 = cc.EventListenerKeyboard:create()
    listener1:registerScriptHandler(onKeyReleased, cc.Handler.EVENT_KEYBOARD_RELEASED )

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener1, self)

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    self:init_game_state_machine()

    if self.is_boss then
        MusicManager:getInstance().play_bg_music(ResourceManager:getInstance():getAudioPathByKey("Game_Boss_Music"), true)
    else
        MusicManager:getInstance().play_bg_music(ResourceManager:getInstance():getAudioPathByKey("Game_Normal_Music"), true)
    end
    return true
end

function GameBackgroundLayer:play_role_action(action_type)

    local ResourceManagerEx = require "GameEx.Foundations.ResourceManager"
    ResourceManagerEx = ResourceManagerEx:get_instance()

    if self.is_boss then

        return 

    end

    -- local action_name_map = {

    --     [GameBackgroundLayer.Role_Action.Normal] = "common",
    --     [GameBackgroundLayer.Role_Action.Skill] = "kill",
    --     [GameBackgroundLayer.Role_Action.Special] = "special",

    -- }

    if action_type == GameBackgroundLayer.Role_Action.Bag then

        -- self.role_state = "bag"
        self.role_spine:setToSetupPose()
        self.role_spine:setAnimation(0, "bag", true)

    elseif action_type == GameBackgroundLayer.Role_Action.Star then

        --超级奖励时不播放普通消除
        if not GameGlobals.bonus then

            -- self.role_state = "star"
            self.role_spine:setToSetupPose()
            self.role_spine:setAnimation(0, "star",false)

            local action = "bag"
            if self.total_step <= 5 then
                action = "HP"
            end
            -- self.role_spine:addAnimation(0, action, true)

            performWithDelay(self,function ()
                self.role_spine:setToSetupPose() 
                self.role_spine:setAnimation(0, action, true)
                end,0.665*2)
        end

    elseif action_type == GameBackgroundLayer.Role_Action.Shop then

        --超级奖励时不播放高级消除
        if not GameGlobals.bonus then

            -- self.role_state = "shop"
            self.role_spine:setToSetupPose()
            self.role_spine:setAnimation(0, "shop", false)

            local action = "bag"
            if self.total_step <= 5 then
                action = "HP"
            end
            -- self.role_spine:addAnimation(0, action, true)

            performWithDelay(self,function ()
                self.role_spine:setToSetupPose() 
                self.role_spine:setAnimation(0, action, true)
                end, 0.801*2) 
        end

    elseif action_type == GameBackgroundLayer.Role_Action.Card then
        
        -- self.role_state =  "card"
        self.role_spine:setToSetupPose()
        self.role_spine:setAnimation(0, "card", true)
        -- self.role_spine:addAnimation(0, "bag", true)
        -- performWithDelay(self,function ()
        -- self.role_spine:setToSetupPose() end, 0.4)

    elseif action_type == GameBackgroundLayer.Role_Action.HP then

        -- self.role_state = "HP"
        self.role_spine:setToSetupPose()
        self.role_spine:setAnimation(0, "HP", true)
    end

end


function GameBackgroundLayer:init_game_state_machine()

    local function on_enter_normal()



    end

    local function on_enter_no_step()

        self:battle_relive()

    end

    local function on_enter_win()

        self:battle_win()

    end

    local function on_enter_relive()

        

    end

    local function on_enter_lose()



    end

    self.__c_state_machine = StateMachine:create()
    self.__c_state_machine:add_state({

        {name = "normal", enter = on_enter_normal},
        {name = "no_step", enter = on_enter_no_step},
        {name = "win", enter = on_enter_win},
        {name = "relive", enter = on_enter_relive},
        {name = "lose", enter = on_enter_lose},

    })
    self.__c_state_machine:add_transition({

        {event = "no_step", from = "normal", to = "no_step"},
        {event = "win", from = "normal", to = "win"},
        {event = "win", from = "no_step", to = "win"},
        {event = "lose", from = "no_step", to = "relive"},
        {event = "lose", from = "relive", to = "lose"},
        {event = "relive", from = "relive", to = "normal"},

    })
    self.__c_state_machine:do_transition("normal")

end

function GameBackgroundLayer:setAllButtonEnabel(flag)
    if flag then
        self.pauseBtn:enable()
        for i=1,5 do
            if self.in_game_item[i] == "LOCKED" then    --7.18
                -- 某个道具没有解锁
                self.item_btn_vecotr[i]:disable()
            else
                self.item_btn_vecotr[i]:enable()
            end
        end
    else
        self.pauseBtn:disable()
        for i=1,5 do
            self.item_btn_vecotr[i]:disable()
        end
    end
end

function GameBackgroundLayer:disable_other_button(item_type)
    self.pauseBtn:disable()
    for i=1,5 do
        self.item_btn_vecotr[i]:disable()
        if i == item_type then
            self.item_image[i]:setLocalZOrder(5)
            self.item_btn_vecotr[i]:enable()
        end
    end
end

function GameBackgroundLayer:enable_all_button()
    self.pauseBtn:enable()
    for i=1,5 do
        self.item_btn_vecotr[i]:enable()
        self.item_image[i]:setLocalZOrder(0)
    end
end

function GameBackgroundLayer:useItemByType(item_type)
    if self.item_acount[item_type] > 0 then
        self.item_acount[item_type] = self.item_acount[item_type] - 1
        self.item_text[item_type]:setString(self.item_acount[item_type])

        --7.14 相应道具数-1
        Player:getInstance():use_player_bag_info_by_id(self.item_map[item_type],-1)

        --9.1
        local item_map = {
            [1] = "item_301", [2] = "item_302", [3] = "item_303", [4] = "item_304",
        }
        for i=1,4 do
            local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.limited_time_item_map[i])
            
            if player_bag_info.item_num <= 0 then
                self.item_map[i] = item_map[i]
                local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.item_map[i])
                self.item_acount[i] = player_bag_info.item_num
                self.item_text[i]:setString(self.item_acount[i])

                for i=1, 5 do
                    -- 道具按钮图片
                    local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[i])
                    local image_item = self.item_image[i]:getChildByName(string.format("image_item_%d", i - 1))
                    image_item:setTexture(bag_info.res_key)
                end
                
            end
        end
        --9.1

        if self.item_acount[item_type] == 0 then
            self.item_text[item_type]:setVisible(false)
            self.item_image[item_type]:getChildByName("image_mark"):setTexture("images/game/item_buy.png")
        end
    else

    end
end

function GameBackgroundLayer:add_prop_tips(item_type)

    local visible_size = cc.Director:getInstance():getVisibleSize()

    local gray_layer = cc.LayerColor:create(cc.c4b(255, 255, 255, 0))
    self.widget:getChildByName("panel_item"):addChild(gray_layer, 4, 1005)

    local node = cc.Node:create()
    gray_layer:addChild(node)

    local back = ccui.ImageView:create("images/game/mask_layer.png")
    back:setScale(2)
    back:setPosition(cc.p(0, visible_size.height/2))
    node:addChild(back)

    local visible_size = cc.Director:getInstance():getVisibleSize()
    local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[item_type])
    local image = ccui.ImageView:create(bag_info.res_key)
    node:addChild(image)

    --7.29
    local prop_info = Prop_Data:getInstance():get_prop_info_by_id(self.item_map[item_type])
    local text = ccui.Text:create("", "images/font/zh_CN.TTF", 40)
    text:setString(prop_info.description)--bag_info.description)
    node:addChild(text)

    local width1 = image:getContentSize().width
    local width2 = text:getContentSize().width
    image:setPosition(cc.p( - (width1 + width2)/2 + width1/2, visible_size.height * 0.87))
    text:setPosition(cc.p(  (width1 + width2)/2 - width2/2, visible_size.height * 0.87))

end

function GameBackgroundLayer:remove_prop_tips()

    self.widget:getChildByName("panel_item"):removeChildByTag(1005)

end

function GameBackgroundLayer:set_item_click_callback(callback)
    self.item_click_callback = callback
end

function GameBackgroundLayer:set_item_cancel_callback(callback)
    self.item_cancel_callback = callback 
end

function GameBackgroundLayer:use_item(skill_name)
    self.is_using = false
    self:remove_prop_tips()
    self:enable_all_button()
    --7.14
    for i=1,5 do
        if self.item_skill_map[i] == skill_name then

            --每日任务事件发送
            local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
            event.param = {task_id = "task_06", finish_count = 1, func = function() end }
            cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
            
            TalkingDataManager:getInstance():send("道具的使用次数统计", {["道具使用的item_id"] = self.item_map[i]})
            TalkingDataManager:getInstance():sendOnUse(self.item_map[i],1)
            local current_level = Player:getInstance():get_player_current_level()
            local str_level = "关卡"..current_level
            TalkingDataManager:getInstance():send("每关每个道具使用次数统计", {[str_level] = self.item_map[i]})

            if not self.item_tutorial_flag then

                self.use_times[i] = self.use_times[i] - 1
                self:useItemByType(i);

            end

            return
        end
    end
    --7.14
end

function GameBackgroundLayer:cancel_item_passive()

    if self.is_using then

        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[self.current_active_item])
        self.item_cancel_callback(bag_info.skill_name)
        self:remove_prop_tips()
        self.is_using = false
        self:enable_all_button()
        self:setAllButtonEnabel(true)   --7.19

    end    

end

function GameBackgroundLayer:onPropEventCallback(item_type)
    if self.item_acount[item_type] <= 0 and not self.item_tutorial_flag then
        local buyItemLayer = BuyItemLayer:create()
        buyItemLayer:set_item(self.item_map[item_type])
        buyItemLayer:set_last_layer("GameBackgroundLayer")
        --7.14
        buyItemLayer:set_buy_callback(function()
            local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.item_map[item_type])
            local item_num = player_bag_info.item_num
            self:addItemByType(item_type, item_num)
            end)
        --7.14
        LayerManager:getInstance():add_layer_to_scene(buyItemLayer)
        return
    end

    if self.is_using then
        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[item_type])
        self.item_cancel_callback(bag_info.skill_name)
        self:remove_prop_tips()
        self.is_using = false
        self:enable_all_button()
        self:setAllButtonEnabel(true)   --7.19

        self.current_active_item = nil
    else
        self:disable_other_button(item_type)
        self.is_using = true
        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_map[item_type])
        self:add_prop_tips(item_type)
        self.item_click_callback(bag_info.skill_name)

        local event = cc.EventCustom:new("item_use")
        event.param = {}   
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

        self.current_active_item = item_type

    end

    -- local position1 = self.item_image[item_type]:getParent():convertToWorldSpace(cc.p(self.item_image[item_type]:getPosition()))
    -- local position2 = self.fnt_step:getParent():convertToWorldSpace(cc.p(self.fnt_step:getPosition()))
    -- local light_particle = self.rm:getParticleByKey("PARTICLE_LIGHT")
    -- light_particle:setAutoRemoveOnFinish(true)
    -- light_particle:setPosition(position1)
    -- light_particle:runAction(cc.Sequence:create(cc.MoveTo:create(0.5, cc.p(position2.x, position2.y)), cc.CallFunc:create(function()
    --         local light2_particle = self.rm:getParticleByKey("PARTICLE_LIGHT2")
    --         light2_particle:setAutoRemoveOnFinish(true)
    --         light2_particle:setPosition(self.fnt_step:getPosition())
    --         self.fnt_step:getParent():addChild(light2_particle, 2)

    --         -- 增加步数道具使用的效果
    --         self:useItemByType(item_type)
    --         self:addStep(5)

    --         light_particle:removeFromParent()
    -- end)))
    -- self:addChild(light_particle, 5)
end

function GameBackgroundLayer:addItemByType(item_type, item_num)
    if self.item_acount[item_type] == 0 and item_num > 0 then
        self.item_text[item_type]:setVisible(true)
        self.item_image[item_type]:getChildByName("image_mark"):setTexture("images/game/item_mark.png")
    end
    self.item_acount[item_type] = self.item_acount[item_type] + item_num
    self.item_text[item_type]:setString(self.item_acount[item_type])
end

function GameBackgroundLayer:disable_touch()
-- print("disable")
    self.can_play = false
    self:setAllButtonEnabel(false)

end

function GameBackgroundLayer:enable_touch()
-- print("enable")
    self.can_play = true
    self:setAllButtonEnabel(true)

end

function GameBackgroundLayer:set_item_tutorial(flag)

    self.item_tutorial_flag = flag

end

function GameBackgroundLayer:showGoal()
    local goal_number = 0
    for key, value in pairs(self.goal) do
        goal_number = goal_number + 1
    end

    -- 目标糖果
    local panel_score = self.widget:getChildByName("panel_score")
    if self.is_boss == false then
        local image_bottle = panel_score:getChildByName("image_bottle")
    end
    local panel_candy = panel_score:getChildByName("panel_candy")
    panel_candy:getChildByName("one"):setVisible(false)
    panel_candy:getChildByName("two"):setVisible(false)
    panel_candy:getChildByName("three"):setVisible(false)
    panel_candy:getChildByName("four"):setVisible(false)
    if goal_number == 1 then
        panel_candy:getChildByName("one"):setVisible(true)
        self.panel_candy = panel_candy:getChildByName("one")
        self:setGoalCandy(panel_candy:getChildByName("one"))
    elseif goal_number == 2 then
        panel_candy:getChildByName("two"):setVisible(true)
        self.panel_candy = panel_candy:getChildByName("two")
        self:setGoalCandy(panel_candy:getChildByName("two"))
    elseif goal_number == 3 then
        panel_candy:getChildByName("three"):setVisible(true)
        self.panel_candy = panel_candy:getChildByName("three")
        self:setGoalCandy(panel_candy:getChildByName("three"))
    elseif goal_number == 4 then
        panel_candy:getChildByName("four"):setVisible(true)
        self.panel_candy = panel_candy:getChildByName("four")
        self:setGoalCandy(panel_candy:getChildByName("four"))
    end
    panel_score:getChildByName("panel_goal_score"):setVisible(false)

    -- 刚进场时的目标糖果
    local panel_goal = self.widget:getChildByName("panel_goal")
    local panel_candy_1 = panel_goal:getChildByName("panel_candy")
    panel_candy_1:getChildByName("one"):setVisible(false)
    panel_candy_1:getChildByName("two"):setVisible(false)
    panel_candy_1:getChildByName("three"):setVisible(false)
    panel_candy_1:getChildByName("four"):setVisible(false)
    if goal_number == 1 then
        panel_candy_1:getChildByName("one"):setVisible(true)
        self:setInGoalCandy(panel_candy_1:getChildByName("one"))
    elseif goal_number == 2 then
        panel_candy_1:getChildByName("two"):setVisible(true)
        self:setInGoalCandy(panel_candy_1:getChildByName("two"))
    elseif goal_number == 3 then
        panel_candy_1:getChildByName("three"):setVisible(true)
        self:setInGoalCandy(panel_candy_1:getChildByName("three"))
    elseif goal_number == 4 then
        panel_candy_1:getChildByName("four"):setVisible(true)
        self:setInGoalCandy(panel_candy_1:getChildByName("four"))
    end
    panel_goal:getChildByName("panel_goal_score"):setVisible(false)
end

-- 初始目标糖果
function GameBackgroundLayer:setGoalCandy(widget)
    local index = 1
    self.candy_origin_scale = widget:getChildByName("image_candy_1"):getScale()
    for key, value in pairs(self.goal) do
        self.goal_text[key] = widget:getChildByName(string.format("text_candy_num_%d", index))
        self.goal_text[key]:setString(value)
        self.goal_image[key] = widget:getChildByName(string.format("image_candy_%d", index))
        self.goal_image[key]:setTexture(self.candy_res[key])
        if self.is_boss then
            self.goal_text[key]:setVisible(false)
        end
        index = index + 1

        if self.game_type == "tournament" then

            self.goal_text[key]:setString(tostring(0))

        end

    end
end

-- 初始进入的糖果
function GameBackgroundLayer:setInGoalCandy(widget)
        local index = 1
    for key, value in pairs(self.goal) do
        widget:getChildByName(string.format("text_candy_num_%d", index)):setString(value)
        local candy = widget:getChildByName(string.format("image_candy_%d", index))
        candy:setTexture(self.candy_res[key])
        if self.is_boss or self.game_type == "tournament" then
            widget:getChildByName(string.format("text_candy_num_%d", index)):setVisible(false)
        end
        index = index + 1
    end

end

function GameBackgroundLayer:addItemEffect(item_type)
    local itemSize = self.item_image[item_type]:getContentSize()

    local sprite = cc.Sprite:create()
    self.item_image[item_type]:addChild(sprite)
    sprite:setPosition(cc.p(itemSize.width/2, itemSize.height/2))
    sprite:runAction(cc.RepeatForever:create(cc.RotateBy:create(0.7, 60)))
end


-- 获得相应糖果后的动画
function GameBackgroundLayer:candy_animation(candy_node, candy_type)

    if self.is_boss == false then

        local image_bottle = self.widget:getChildByName("panel_score"):getChildByName("image_bottle")

        image_bottle:setScale(1)
        image_bottle:setPosition(self.__t_bottle_original_position)

        image_bottle:stopAllActions()
        image_bottle:runAction(cc.Sequence:create(cc.ScaleTo:create(0.033 * 3, 1.01, 0.98), cc.ScaleTo:create(0.033 * 2, 1.0, 1.0)))
        image_bottle:runAction(cc.Sequence:create(cc.MoveBy:create(0.033 * 3, cc.p(0,4)), cc.MoveBy:create(0.033 * 2, cc.p(0, -4))))

        local original_scale = self.panel_candy:getChildByName("image_candy_1"):getScale()

        local sprite_position = cc.p(self.__t_bottle_original_position.x, self.__t_bottle_original_position.y - 150)
        local texture_path = self.candy_res[candy_type]
        local animation_sprite = cc.Sprite:create(texture_path)
        animation_sprite:setScale(1.5)
        animation_sprite:setLocalZOrder(image_bottle:getLocalZOrder() - 1)
        animation_sprite:setPosition(sprite_position)
        image_bottle:getParent():addChild(animation_sprite)

        local parent_position = cc.p(candy_node:getParent():getParent():getPosition())
        local position = cc.p(candy_node:getPosition())
        position.x = position.x + parent_position.x
        position.y = position.y + parent_position.y
        local move = cc.MoveTo:create(0.1, position)
        local scale = cc.ScaleTo:create(0.1, self.candy_origin_scale)
        local spawn = cc.Spawn:create(move, scale)
        local remove = cc.RemoveSelf:create()
        local sequence = cc.Sequence:create(spawn, remove)

        animation_sprite:runAction(sequence)

    end

    candy_node:stopAllActions()
    candy_node:setScale(self.candy_origin_scale)
    candy_node:runAction(cc.Sequence:create(cc.ScaleTo:create(0.033 * 3 , self.candy_origin_scale + 0.2), cc.ScaleTo:create(0.033 * 3, self.candy_origin_scale - 0.05), cc.ScaleTo:create(0.033 * 2, self.candy_origin_scale)))
end
--检测是否 结束
function GameBackgroundLayer:terminate_condition_check()

    local win = true;

    for key, value in pairs(self.goal) do
        if self.goal[key] > 0 then
            win = false
        end
    end

    return win

end

function GameBackgroundLayer:set_boss_full_hp(n_full_hp)

    self.__n_full_hp = n_full_hp
    self.__n_current_hp = n_full_hp

    if self.boss_progress_blood_text then

        self.boss_progress_blood_text:setString(string.format("%d/%d", self.__n_full_hp, self.__n_full_hp))

    end

end

function GameBackgroundLayer:set_boss_blood(boss_hp_percent)
    
    if self.blood_updater_entry then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.blood_updater_entry)
    end

    if boss_hp_percent < 0 then

        boss_hp_percent = 0 

    end

    self.boss_hp_percent = boss_hp_percent

    if self.boss_progress_blood then

        self.boss_progress_blood:setPercent(boss_hp_percent)

        if self.__n_current_hp < 0 then

            self.__n_current_hp = 0

        end

        self.boss_progress_blood_text:setString(string.format("%d/%d", self.__n_current_hp, self.__n_full_hp))

    end
end

function GameBackgroundLayer:offset_boss_blood(hp_offset)
    -- print(hp_offset)
    self.__n_current_hp = self.__n_current_hp + hp_offset

    if self.__n_current_hp < 0 then

        self.__n_current_hp = 0

    end

    self:set_boss_blood(self.__n_current_hp / self.__n_full_hp * 100)

end

function GameBackgroundLayer:set_boss_blood_state(boss_hp_percent)
    local init_player_data = GameKernel:getInstance():get_init_player_data()
    if self.boss_progress_blood then
        if boss_hp_percent < init_player_data.boss_blood_state_rate_2 and self.boss_hp_state == 1 then
            self.boss_hp_state = 2
            self.boss_progress_blood:loadTexture("images/game/boss_2_progress.png")
        elseif boss_hp_percent < init_player_data.boss_blood_state_rate_3 and self.boss_hp_state == 2 then
            self.boss_hp_state = 3
            self.boss_progress_blood:loadTexture("images/game/boss_3_progress.png")
        end
    end
end

function GameBackgroundLayer:set_boss_step(step)

    self.boss_fnt_step:setVisible(true)
    self.attack_number:setVisible(false)
    self.attack_image:setVisible(false)

    self.boss_skill_step = step
    self.boss_fnt_step:setString(step)

end

function GameBackgroundLayer:set_boss_attack(attack_type, attack_number)

    self.boss_fnt_step:setVisible(false)
    self.attack_number:setVisible(true)
    self.attack_image:setVisible(true)

    self.attack_image:loadTexture(self.attack_img_map[attack_type])
    self.attack_number:setString(string.format("x%d", attack_number))

end

function GameBackgroundLayer:subCandy(candy_type, account)
-- print(self.goal[candy_type])
    self.goal[candy_type] = math.max(0, self.goal[candy_type] - account)

    self:play_candy_collect_animation(candy_type)

    if self:terminate_condition_check() then
        self.__c_state_machine:do_event("win")
    else
        self.__c_state_machine:do_event("lose")
    end

end

function GameBackgroundLayer:subStep(account)

    self.total_step = self.total_step - account

    self:play_sub_step_animation()

    if self.total_step == 0 then

        self.__c_state_machine:do_event("no_step")

    end

end


function GameBackgroundLayer:setStep(step)

    self.total_step = step
    self:play_sub_step_animation()
    if not self.game_win then
        self:highlight_item()
    end    

end

function GameBackgroundLayer:addScore(score)

    self.score = self.score + score

end

function GameBackgroundLayer:play_item_add_step_indication()
print("item add step $%&^^")
    self.add_step_action:play("step_add", false)

    local light_particle = ResourceManagerEx:get_instance():get_resource_by_key("skill_flying_particle")
    light_particle:setPosition(cc.p(105, 105))
    light_particle:setDuration(-1)
    light_particle:setLocalZOrder(99)
    light_particle:setScale(1.2)
    light_particle:setVisible(false)

    local star_node = self.add_step_node:getChildByName("node"):getChildByName("image_step_dia")
    star_node:addChild(light_particle)

    local remove_action = cc.Sequence:create(cc.DelayTime:create(25 * 0.033), cc.CallFunc:create(function()

        light_particle:setAutoRemoveOnFinish(true)
        light_particle:setDuration(0)

    end))

    light_particle:runAction(remove_action)

end

function GameBackgroundLayer:play_skill_add_step_indication()
print("skill add step $%&^^")
    self.add_step_action:play("skill_step_add", false)

    local light_particle = ResourceManagerEx:get_instance():get_resource_by_key("skill_flying_particle")
    light_particle:setPosition(cc.p(105, 105))
    light_particle:setDuration(-1)
    light_particle:setLocalZOrder(99)
    light_particle:setScale(1.2)
    light_particle:setVisible(false)

    local star_node = self.add_step_node:getChildByName("node"):getChildByName("image_step_dia")
    star_node:addChild(light_particle)

    local remove_action = cc.Sequence:create(cc.DelayTime:create(25 * 0.033), cc.CallFunc:create(function()

        light_particle:setAutoRemoveOnFinish(true)
        light_particle:setDuration(0)

    end))

    light_particle:runAction(remove_action)

end

function GameBackgroundLayer:play_alert_step_indication()

    self.indication_action:play("shiro_step_less", false)

end

function GameBackgroundLayer:play_candy_collect_animation(candy_type)

    self:candy_animation(self.goal_image[candy_type], candy_type)

    if self.game_type == "tournament" then

        self.__n_shell_number = self.__n_shell_number or 0
        self.__n_shell_number = self.__n_shell_number + 1

        self.goal_text[candy_type]:setString(tostring(self.__n_shell_number))

    else

        self.goal[candy_type] = math.max(0, self.goal[candy_type] - 1)

        self.goal_text[candy_type]:setString(tostring(self.goal[candy_type]))
        -- 当糖果为零的时候打钩并且隐藏计数
        if self.goal[candy_type] <= 0 and self.goal_text[candy_type]:isVisible() then
            self.goal_text[candy_type]:setVisible(false)

            local effect = self.rm:getSpineByKey("SPINE_NIKE", 1.0)
            effect:setScale(0.5)
            effect:setAnimation(0, "nike", false)
            local candy_size = self.goal_image[candy_type]:getContentSize()
            effect:setPosition(cc.p(candy_size.width/2, candy_size.height * 0.2))
            self.goal_image[candy_type]:addChild(effect)

            local nike_particle = self.rm:getParticleByKey("PARTICLE_NIKE")
            nike_particle:setAutoRemoveOnFinish(true)
            nike_particle:setPosition(cc.p(candy_size.width/2, candy_size.height * 0.5))
            self.goal_image[candy_type]:addChild(nike_particle)
        end

    end

end

function GameBackgroundLayer:play_sub_step_animation()

    self.fnt_step:setString(tostring(self.total_step))

    if self.total_step <= 5 and self.step_tips == false and not self.game_win  then
        
        -- local bushu_spine = self.rm:getSpineByKey("SPINE_BUSHU", 1.0)
        -- bushu_spine:setAnimation(0, "bushu", false)
        -- bushu_spine:registerSpineEventHandler(function (event)
        --     self.fnt_step:setFntFile("images/font/step_less.fnt")
        --     MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("5step_Audio"), false)
        --     self.action:gotoFrameAndPlay(166, 192, true)
        --     bushu_spine:retain()
        --     bushu_spine:removeFromParent()
        --     bushu_spine:autorelease()
        -- end, sp.EventType.ANIMATION_END)
        -- bushu_spine:setPosition(cc.p(self.fnt_step:getPositionX() + 30, self.fnt_step:getPositionY()))
        -- self.fnt_step:getParent():addChild(bushu_spine, 3)

        if not self.is_boss and not GameGlobals.bonus then

            self:play_role_action(GameBackgroundLayer.Role_Action.HP)

        end

        self.fnt_step:setFntFile("images/font/step_less.fnt")
        self.action:play("step_less", true)
        self:play_alert_step_indication()
        self.step_tips = true

    elseif self.total_step > 5 then

        if not self.is_boss and not GameGlobals.bonus then

            self:play_role_action(GameBackgroundLayer.Role_Action.Bag)

        end

        self.fnt_step:setFntFile("images/font/step.fnt")
        self.action:gotoFrameAndPlay(166, 166, false)
        self.step_tips = false

    elseif self.total_step == 0 then
        -- print("self.can_open_bag = false")
        self.can_open_bag = false
        self.action:gotoFrameAndPlay(166, 166, false)

    end

end

function GameBackgroundLayer:update_score(score)

    -- self.score = score

    -- self:play_add_score_animation()

end

function GameBackgroundLayer:play_score_animation(score, position)

    local pos = {}
    pos.x = position.x
    pos.y = position.y + 100

    if pos.x < 200 then

        pos.x = 200

    elseif pos.x > 880 then

        pos.x = 880 

    end

print(string.format("score_test:%d", self.__n_free_score))

    local score_text = self.__t_score_pool[self.__n_free_score]
    self.__t_score_pool[self.__n_free_score] = nil
    self.__n_free_score = self.__n_free_score - 1
print(score_text.tag)
    score_text:setPosition(pos)
    score_text:setScale(1.0)
    score_text:getChildByName("node"):getChildByName("text_5"):setString(tostring(score))

    self:getParent():addChild(score_text)

    score_text:runAction(score_text.action)
    score_text.action:play("text_5", false)

    local target_position = self.score_loading:getParent():convertToWorldSpace(cc.p(self.score_loading:getPosition()))
    target_position.x = target_position.x + (self.score_loading:getPercent() - 50) / 100 * 393

    local delay = cc.DelayTime:create(20 * 0.033)
    local scale = cc.ScaleTo:create(0.2, 0.5)
    local delay_seq = cc.Sequence:create(cc.DelayTime:create(0.2), scale)
    local move = cc.MoveTo:create(0.4, target_position)
    local spawn = cc.Spawn:create(move, delay_seq)
    local ease = cc.EaseSineIn:create(spawn)
    local remove = cc.RemoveSelf:create()
    local reach_callback = cc.CallFunc:create(function()

        score_text:removeFromParent()
        self.__n_free_score = self.__n_free_score + 1
        self.__t_score_pool[self.__n_free_score] = score_text
        self:play_add_score_animation(score)

    end)

    local sequence = cc.Sequence:create(delay, ease, reach_callback)

    score_text:runAction(sequence)  

end

function GameBackgroundLayer:play_add_score_animation(score)

    self.score = self.score + score
    self.text_score:setString(self.score)

    local update_target = self.score < self.star_score[3] and self.score or self.star_score[3]
    local score_updater_entry = 0

    local function score_updater()

        local addPercent = 0.033 / 3.0 * 100

        local lastPercent = self.score_loading:getPercent() + addPercent

        if lastPercent >= 100 * update_target / self.star_score[3] then

            lastPercent = 100 * update_target / self.star_score[3]
            cc.Director:getInstance():getScheduler():unscheduleScriptEntry(score_updater_entry)

        end

        self.score_loading:setPercent(lastPercent)

        for i=1,3 do

            if not(self.full_star[i]:isVisible()) and lastPercent * self.star_score[3] / 100 >= self.star_score[i] then
                self.full_star[i]:setVisible(true)

                local particle = self.rm:getParticleByKey("PARTICLE_STAR")
                particle:setAutoRemoveOnFinish(true)
                particle:setPosition(cc.p(self.full_star[i]:getContentSize().width/2, self.full_star[i]:getContentSize().height * 0.5))
                self.full_star[i]:addChild(particle)
            end
            
        end

    end

    score_updater_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(score_updater, 0.033, false)
    
end

-- 技能接口百分比
function GameBackgroundLayer:set_skill_percent(percent)

    self.skill_progress:setPercentage(100 - percent)

    if percent >= 100 and not self.__n_is_ready then

        ERROR_ASSERT(not self.skill_progress:getChildByName("skill_particle"), "DUPLICATE PARTICLE !!!!!")

        local ReosurceManagerEx = require "GameEx.Foundations.ResourceManager"

        local particle = ReosurceManagerEx:get_instance():get_resource_by_key("skill_ready_particle")
        particle:setDuration(-1)
        particle:setScale(0.7)
        particle:setTag(99)
        particle:setPosition(cc.p(60, 60))
        particle:setAutoRemoveOnFinish(false)
        particle:setName("skill_particle")
        self.skill_progress:addChild(particle, 9)

        self.__n_is_ready = true

    elseif percent < 100 then

        if self.__n_is_ready then

            local particle = self.skill_progress:getChildByTag(99)
            particle:removeFromParent()

            self.__n_is_ready = false

        end

    end

end

-- 没有可消除的糖果时，游戏重置
function GameBackgroundLayer:resetGame()

    self.action:play("resetting", false)

    self.widget:getChildByName("panel_goal"):setVisible(true)

    local stage_num = self.widget:getChildByName("image_stage_num")

    if stage_num then

        stage_num:setVisible(false)

    end

end

function GameBackgroundLayer:resetGameTournament(level)
-- ERROR_ASSERT(false, "string")
    self.action:play("resetting", false)

    self.widget:getChildByName("panel_goal"):setVisible(false)

    local stage_num = self.widget:getChildByName("image_stage_num")

    if stage_num then

        stage_num:getChildByName("font_stage_num"):setString(string.format("第%d关", level))
        stage_num:setVisible(true)

    end

end

function GameBackgroundLayer:updateTournamentLevel()

    local stage_num = selt.widget:getChildByName("image_state_num")

    if stage_num then



    end

end

-- 添加胜利瓶子
function GameBackgroundLayer:add_win_bottle( )
    local bottle_node = cc.CSLoader:createNode("bottle.csb")
    self:addChild(bottle_node, 5)
    bottle_node:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(bottle_node)

        -- 开场动画
    local bottle_action = cc.CSLoader:createTimeline("bottle.csb")
    bottle_action:play("bottle", false)
    bottle_node:runAction(bottle_action)

    -- 散出物体
    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "bottle_end" then
            self:win_jiesuan()
        end
    end
    bottle_action:setFrameEventCallFunc(onFrameEvent)
end

function GameBackgroundLayer:win_jiesuan()

    if self.game_type == "tournament" then

        local weekgameWinLayer = WeekgameWinLayer:create()
        weekgameWinLayer:setConchNumber(GameGlobals.Scene:get("conch_collected"))
        cc.Director:getInstance():getRunningScene():addChild(weekgameWinLayer, 9)

    else

        local starCount = 0
        local curScore = self.score
        for key, value in ipairs(self.star_score) do
            if curScore >= value then
                starCount = starCount + 1
            end
        end

        -- if self.surplus_step == 0 then
        --     local luckyMoveLayer = LuckyMoveLayer:create()
        --     luckyMoveLayer:set_close_callback(function()
        --         local winLayer = WinLayer:create()
        --         winLayer:set_game_score(self.score)
        --         winLayer:setStarGoal(starCount)
        --         LayerManager:getInstance():add_layer_to_scene(winLayer)
        --     end)
        --     LayerManager:getInstance():add_layer_to_scene(luckyMoveLayer)

        -- else
            local winLayer = WinLayer:create()
            winLayer:set_game_score(self.score)
            winLayer:setStarGoal(starCount)
            LayerManager:getInstance():add_layer_to_scene(winLayer)
        -- end

    end

end

-- 游戏胜利
function GameBackgroundLayer:play_battle_win_animation(finish_callback)
    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_win"), false)
    self.game_win = true
    -- 游戏结束 spine特效加粒子
    local size = cc.Director:getInstance():getVisibleSize()
    local success_spine = self.rm:getSpineByKey("SPINE_SUCCESS", 1.0)
    success_spine:setAnimation(0, "success", false)
    success_spine:registerSpineEventHandler(function (event)
        if finish_callback then

            finish_callback()

        end

        success_spine:retain()
        success_spine:removeFromParent()
        success_spine:autorelease()
    end, sp.EventType.ANIMATION_END)
    success_spine:setPosition(cc.p(size.width/2, size.height/2))
    cc.Director:getInstance():getRunningScene():addChild(success_spine, 100)

    local spine_size = success_spine:getContentSize()
    local success_particle = self.rm:getParticleByKey("PARTICLE_SUCCESS")
    success_particle:setAutoRemoveOnFinish(true)
    success_particle:setPosition(cc.p(spine_size.width/2, spine_size.height * 0.5))
    success_spine:addChild(success_particle)

    --TalkingData   self.total_step当前剩余步数 self.count_step总步数 self.game_time 游戏时间
    local current_level = Player:getInstance():get_player_current_level()
    TalkingDataManager:getInstance():send_for_use_step(current_level, self.count_step-self.total_step)
    TalkingDataManager:getInstance():send_for_surplus_step(current_level, self.total_step)
    TalkingDataManager:getInstance():send_for_finish_time(current_level, self.game_time)
    
    self.surplus_step = self.total_step

    if self.__n_last_highlight_index then
        self:stop_item_highlight_animation(self.__n_last_highlight_index)
    end
end

function GameBackgroundLayer:play_bonus_animation()
    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_superreward_1"), false)
    MusicManager:getInstance().play_bg_music(ResourceManager:getInstance():getAudioPathByKey("Super_Reward_Music"), true)

    local size = cc.Director:getInstance():getVisibleSize()
    local super_reward = self.rm:getSpineByKey("SPINE_SUPERREWARD", 1.0)
    super_reward:setAnimation(0, "super_reward", false)
    super_reward:setPosition(cc.p(size.width/2, size.height/2))
    super_reward:registerSpineEventHandler(function (event)
        
        super_reward:retain()
        super_reward:removeFromParent()
        super_reward:autorelease()

    end, sp.EventType.ANIMATION_END)
    self:addChild(super_reward, 5)

    self:play_role_action(GameBackgroundLayer.Role_Action.Card)

end

function GameBackgroundLayer:set_relive_callback(callback)

    self.relive_callback = callback

end

function GameBackgroundLayer:set_relive_bomb_callback(callback)

    self.relive_bomb_callback = callback

end

function GameBackgroundLayer:set_die_bomb_callback(callback)

    self.die_bomb_callback = callback

end

-- 游戏失败
function GameBackgroundLayer:battle_lost()
    local starCount = 0
    local curScore = self.score
    for key, value in ipairs(self.star_score) do
        if curScore >= value then
            starCount = key
        end
    end

    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_lose"), false)
    local loseLayer = LoseLayer:create()
    loseLayer:setStarGoal(starCount)
    LayerManager:getInstance():add_layer_to_scene(loseLayer)
end

-- 游戏失败
function GameBackgroundLayer:battle_lost_tournament()
    
    if GameGlobals.Scene:is_goal_reached() then

        GameGlobals.Scene:get_component("tournament_reward_skill"):dispatch_reward()

    else
         
         GameGlobals.Scene:get_component("ui_component").__c_state_machine:do_transition("bonus")

    end 

end

-- 游戏复活
function GameBackgroundLayer:battle_relive()

    self.can_open_bag = false

    if self.game_type == "tournament" then

        local relieveLayer = ReliveLayer:create(ReliveLayer.Relive_Type.Tournament)
        relieveLayer:set_relive_times(self.relive_times)
        self.relive_times = self.relive_times + 1
        relieveLayer:set_die_callback(function()

            self:battle_lost_tournament()

        end)
        relieveLayer:set_relive_callback(function()

            self:relive_callback()

        end)
        LayerManager:getInstance():add_layer_to_scene(relieveLayer)        

    else

        local relieveLayer = ReliveLayer:create(1)
        relieveLayer:set_is_boss(self.is_boss)
        relieveLayer:setCurrentStage(self.goal, self.total_goal)
        relieveLayer:set_relive_times(self.relive_times)
        self.relive_times = self.relive_times + 1
        relieveLayer:set_die_callback(function()
            self:battle_lost()

            local player = Player:getInstance()
            local current_level = player:get_player_current_level()
            local str_level = "关卡"..current_level
            TalkingDataManager:getInstance():send("关卡失败原因", {[str_level] = "步数不足"})
            TalkingDataManager:getInstance():sendMissionInfo("failed", str_level, "步数不足")

            
            local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
            player:add_player_mission_fail_times(mission_info.mission_id)

            --
            if self.is_boss then
                local blood = self.__n_full_hp * self.boss_hp_percent / 100
                TalkingDataManager:getInstance():send_for_boss_blood_remain(current_level, blood)
            end
        end)
        relieveLayer:set_relive_callback(function()

            self:relive_callback()

        end)
        LayerManager:getInstance():add_layer_to_scene(relieveLayer)

    end

end

function GameBackgroundLayer:battle_relive_bomb()

    self.can_open_bag = true

    local relieveLayer = ReliveLayer:create(ReliveLayer.Relive_Type.Bomb)
    relieveLayer:set_is_boss(self.is_boss)
    relieveLayer:setCurrentStage(self.goal, self.total_goal)
    relieveLayer:set_relive_bomb_times(self.relive_bomb_times)
    self.relive_bomb_times = self.relive_bomb_times + 1
    relieveLayer:set_die_callback(function()
        self.die_bomb_callback()
        local player = Player:getInstance()
        local current_level = player:get_player_current_level()
        local str_level = "关卡"..current_level
        TalkingDataManager:getInstance():send("关卡失败原因", {[str_level] = "定时炸弹爆炸"})
        TalkingDataManager:getInstance():sendMissionInfo("failed", str_level, "定时炸弹爆炸")

        
        local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
        player:add_player_mission_fail_times(mission_info.mission_id)
    end)
    relieveLayer:set_relive_callback(function()

        self:relive_bomb_callback()

    end)
    LayerManager:getInstance():add_layer_to_scene(relieveLayer)
end

function GameBackgroundLayer:onEnter()
    self.action:gotoFrameAndPlay(0, 80, false)
end

function GameBackgroundLayer:onExit()

    for i = 1, self.__n_free_score do

        self.__t_score_pool[i].action:release()
        self.__t_score_pool[i]:release()

    end
    --9.1
    cc.Director:getInstance():getEventDispatcher():removeCustomEventListeners(NotificationUtil.EventType.remove_limited_time_item_from_game)

    cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleScriptEntryID)

end

function GameBackgroundLayer:increase_goal(type)

    self.total_goal[type] = self.total_goal[type] + 1 

    self.goal[type] = self.goal[type] + 1 

    self:candy_animation(self.goal_image[type], type)

    self.goal_text[type]:setString(tostring(self.goal[type]))

end

function GameBackgroundLayer:highlight_item()

    if self.total_step == 1 then
        if self.item_image[3].original_position then
            self:stop_item_highlight_animation(3)
        end   
    elseif self.total_step == 2 then        
        if self.use_times[3] > 0 and self.in_game_item[3] ~= "LOCKED" then
            self:play_item_highlight_animation(3)
        end

    elseif self.total_step == 3 then
        math.randomseed(os.time())
        local random_num = math.random()
        if random_num > 0.5 then
            if self.use_times[2] > 0 and self.in_game_item[2] ~= "LOCKED" then
                self:play_item_highlight_animation(2)
            end
        else
            if self.use_times[1] > 0 and self.in_game_item[1] ~= "LOCKED" then
                self:play_item_highlight_animation(1)
            end
        end

    elseif self.total_step == 4 then
        if self.use_times[4] > 0 and self.in_game_item[4] ~= "LOCKED" then
            self:play_item_highlight_animation(4)
        end

    elseif self.total_step == 5 then
        if self.use_times[5] > 0 and self.in_game_item[5] ~= "LOCKED" then
            self:play_item_highlight_animation(5)
        end

    end

end

function GameBackgroundLayer:play_item_highlight_animation(item_index)

    self:stop_item_highlight_animation(self.__n_last_highlight_index)

    local move_1 = cc.MoveBy:create(0.2, cc.p(0, 30))
    local scale_1 = cc.ScaleTo:create(0.2, 1.05)
    local span_1 = cc.Spawn:create(move_1, scale_1)

    local delay_1 = cc.DelayTime:create(0.25)
    local rotate_1 = cc.RotateBy:create(0.066, -5)
    local rotate_2 = cc.RotateBy:create(0.066, 10)
    local rotate_3 = cc.RotateBy:create(0.066, -5)
    local rotate_4 = cc.RotateBy:create(0.066, -5)
    local rotate_5 = cc.RotateBy:create(0.066, 10)
    local rotate_6 = cc.RotateBy:create(0.066, -5)
    local delay_2 = cc.DelayTime:create(0.2)
    local seq_1 = cc.Sequence:create(delay_1, rotate_1, rotate_2, rotate_3, rotate_4, rotate_5, rotate_6, delay_2)

    local move_2 = cc.MoveBy:create(0.2, cc.p(0, -30))
    local scale_2 = cc.ScaleTo:create(0.2, 1.0)
    local span_2 = cc.Spawn:create(move_2, scale_2)

    local seq_2 = cc.Sequence:create(seq_1, span_2)
    local span_3 = cc.Spawn:create(span_1, seq_2)
    local delay_3 = cc.DelayTime:create(0.5)
    local seq_3 = cc.Sequence:create(span_3, delay_3)
    local rep = cc.RepeatForever:create(seq_3)

    -- local scale_1 = cc.ScaleTo:create(0.3, 1.1)
    -- local scale_2 = cc.ScaleTo:create(0.3, 1.0)
    -- local seq = cc.Sequence:create(scale_1, scale_2)
    -- local rep = cc.RepeatForever:create(seq)
    rep:setTag(999)

    self.item_image[item_index]:runAction(rep)
    self.item_image[item_index].original_position = cc.p(self.item_image[item_index]:getPosition())

    self.__n_last_highlight_index = item_index

end

function GameBackgroundLayer:stop_item_highlight_animation(item_index)

    if item_index then

        self.item_image[item_index]:stopActionByTag(999)
        self.item_image[item_index]:setScale(1.0)
        self.item_image[item_index]:setPosition(self.item_image[item_index].original_position)

    end

end

return GameBackgroundLayer
