require "core.LayerManager"
require "core.Shop_Item_Data"
require "UI.Widget.MaskLayer"

GameItemLayer = class("GameItemLayer", function() return cc.Layer:create() end)

function GameItemLayer:ctor()

end

function GameItemLayer:init_ui(item_name)

    self:setName("GameItemLayer")
    self.is_buy = false
    self.buy_callback = nil
    self.close_callback = nil

    local current_level = Player:getInstance():get_player_current_level()
    local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
    Player:getInstance():set_player_mission_fail_times(mission_info.mission_id, 0)

	local widget = cc.CSLoader:createNode(string.format("game_item_%s.csb",item_name))
	self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline(string.format("game_item_%s.csb",item_name))
    widget:runAction(self.action)
    self.action:play("flash_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "item_info_in_end" then
        	self.action:play("display", true)
        elseif str == "item_info_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")
    local image_price = image_dia:getChildByName("image_price")
    local text_price = image_dia:getChildByName("text_price")
    local buy_image = image_dia:getChildByName("button_buy")
    local buy_text = buy_image:getChildByName("text_price")

    local cost = 0
    local item_num = 0
    local item_id = "item_02"
    local player = Player:getInstance()
    if item_name == "step" then
    	local player_bag_info = player:get_player_bag_info_by_id("item_201")
    	item_id = "item_201"
    	item_num = player_bag_info.item_num

        local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(item_id)
        cost = shop_item_info.cost
		text_price:setString(cost)
    elseif item_name == "boom" then
    	local player_bag_info = player:get_player_bag_info_by_id("item_202")
    	item_id = "item_202"
    	item_num = player_bag_info.item_num

        local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(item_id)
        cost = shop_item_info.cost
    	text_price:setString(cost)
    elseif item_name == "flash" then
    	local player_bag_info = player:get_player_bag_info_by_id("item_203")
    	item_id = "item_203"
    	item_num = player_bag_info.item_num

        local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(item_id)
        cost = shop_item_info.cost
    	text_price:setString(cost)
    end
    if item_num > 0 then
    	image_price:setVisible(false)
    	text_price:setString(string.format("数量:%d", item_num))
    	text_price:setPositionX(text_price:getPositionX()-20)
    	buy_text:setString("使用")
    end
    
    local buy_Btn = ButtonWrap:create()
    buy_Btn:wrap(buy_image, 1)
    buy_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        if item_num > 0 then
        	player:add_player_bag_info_by_id(item_id, -1)
        else
        	player:add_player_gold(-cost)
        end
        
        if self.buy_callback then
        	self.buy_callback()
        end
        self.is_buy = true
        self:close_self()
    end)

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

	local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function GameItemLayer:close_self()
    if self.close_callback and self.is_buy == false then
        self.close_callback()
    end
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("flash_out", false)
end

function GameItemLayer:set_buy_callback(buy_callback)
	self.buy_callback = buy_callback
end

function GameItemLayer:set_close_callback(close_callback)
    self.close_callback = close_callback
end

function GameItemLayer:onEnter()

end

function GameItemLayer:onExit()
end

return GameItemLayer