require "UI.Widget.MaskLayer"
require "UI.Layer.ItemInfoLayer"
require "core.Shop_Item_Data"
require "core.LayerManager"
require "UI.Scene.LoadingScene"
require "core.Prop_Data"
require "core.TalkingDataManager"

local LevelBuilder = require "GameEx.Foundations.LevelBuilder"
LevelInfoLayer = class("LevelInfoLayer", function() return cc.Layer:create() end)

function LevelInfoLayer:ctor()

    self:setName("level_info_layer")

    self:setTag(1001)
    self.close_callback = nil
    self.is_fish_guide = false
    -- 游戏资源
    self.rm = ResourceManager:getInstance()
    self.candy_res = { 
        ["red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RED"),
        ["green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_GREEN"),
        ["blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BLUE"),
        ["yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_YELLOW"),
        ["pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PINK"),
        ["white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_WHITE"),
        ["ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_ICE"),
        ["key"] = self.rm:getPicturePathByKey("IMAGES_CANDY_KEY"),
        ["fan"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FAN"),
        ["random"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RANDOM"),
        ["panda"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PANDA"),
        ["moveable_ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_MOVEABLE_ICE"),
        ["fenced"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FENCE"),
        ["chocolate"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CHOCOLATE"),
        ["bomb"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BOMB"),
        ["ten"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CROSS"),
        ["lightning"] = self.rm:getPicturePathByKey("IMAGES_CANDY_LIGHTNING"),
        ["coco"] = self.rm:getPicturePathByKey("IMAGES_CANDY_COCO"),
        ["transform_red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_RED"),
        ["transform_green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_GREEN"),
        ["transform_blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_BLUE"),
        ["transform_yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_YELLOW"),
        ["transform_pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_PINK"),
        ["transform_white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_WHITE"),
    }

    -- 顶部栏
    local topBarLayer = TopBarLayer:create()
    self:addChild(topBarLayer, 1)

    local rm = ResourceManager:getInstance()
	local widget = cc.CSLoader:createNode("game_set_out.csb")
	self:addChild(widget)

    local visible_size = cc.Director:getInstance():getVisibleSize()
    widget:setContentSize(visible_size)
    ccui.Helper:doLayout(widget)

    -- 动画
    self.action = cc.CSLoader:createTimeline("game_set_out.csb")
    widget:runAction(self.action)
    
    self.action:setCurrentFrame(0)
    self.action:play("item_info_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "item_info_in_end" then
            self.action:play("light", true)
            --print("TimeSpeed::"..self.action:getTimeSpeed())
            --self.action:setTimeSpeed(0.1)
        elseif str == "item_info_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local level_index = Player:getInstance():get_player_current_level()

    local mask = MaskLayer:create()
    self:addChild(mask, 99999)

    local after_enter = function()

        mask:removeFromParent()

        TRIGGER_EVENT("LEVEL_INFO_SHOW", {level = level_index})

        local event = cc.EventCustom:new("level_info_show")
        event.param = {
            ["level_index"] = level_index
        }
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event) 

    end

    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.7), cc.CallFunc:create(after_enter)))

    local node = widget:getChildByName("node")
    self.node = node
    local image_dia = node:getChildByName("image_dia")

    -- 关卡标题
    local image_stage_dia = image_dia:getChildByName("image_stage_dia")
    local text_stage = image_stage_dia:getChildByName("text_stage")
    text_stage:setString(string.format(ResourceManager:getInstance():getTextStringByKey("LevelInfoLayer_1"), level_index))

    -- 人物
    self.image_role = image_dia:getChildByName("image_role")
    self.text_role_name = image_dia:getChildByName("text_role_name")
    self.text_role_level = image_dia:getChildByName("text_role_level")
    self:update_skin_image()

    -- 关卡目标
    local image_goal_dia = image_dia:getChildByName("image_goal_dia")
    self:show_goal_candy(image_goal_dia)

    local image_item = image_dia:getChildByName("image_item")
    -- 三个道具按钮
    self.is_buy_item = {}
    self.is_use_item_from_bag = {}   --7.13
    self.item_id = {[1]= "item_201", [2] = "item_202", [3] = "item_203",}
    for i=1,#self.item_id do
        self.is_buy_item[self.item_id[i]] = false
        self.is_use_item_from_bag[self.item_id[i]] = false  --7.13
    end

    --7.18  读mission_info表，判断out_game_item字段，为LOCKED则变成锁图images/item/item_lock_1.png
    -- local mission_info = Mission_Data:getInstance():get_mission_info_by_level(level_index)
    -- local out_game_item = nil
    -- if mission_info.out_game_item then
    --     out_game_item = mission_info.out_game_item
    -- else
    --     out_game_item = { "Move", "Bomb", "Lightning" }
    -- end    
    --7.18

    --7.29  获得最新的最高通关的关卡号，判断是否解锁道具
    local newest_mission_id = Player:getInstance():get_player_newest_mission_id()
    local mission_info = Mission_Data:getInstance():get_mission_info_by_id(newest_mission_id)
    local newest_level = mission_info.mission_level

    local out_game_item = {}
    for i=1,3 do
        local prop_info = Prop_Data:getInstance():get_prop_info_by_id(self.item_id[i])
        if newest_level >= prop_info.Unlock_level then
            out_game_item[i] = "can_use"
        else
            out_game_item[i] = "LOCKED"
        end
    end
    --7.29

    for i=1,3 do
        local item = image_item:getChildByName(string.format("item_%d", i))
        local image_item = item:getChildByName("image_item")
        local text_price = item:getChildByName("text_price")
        local image_gold = item:getChildByName("image_gold")
        local image_get = item:getChildByName("image_get")
        image_get:setVisible(false)
        local text_unlock_num = item:getChildByName("text_unlock_num")
        local image_mark = item:getChildByName("image_mark")
        local text_num = image_mark:getChildByName("text_num")

        local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(self.item_id[i])
        local bag_info = Bag_Data:getInstance():get_bag_info_by_id(self.item_id[i])
        image_item:setTexture(bag_info.res_key)
        image_gold:setTexture(Shop_Item_Data:getInstance():get_res_by_currency(shop_item_info.currency))
        text_price:setString(string.format("%d", shop_item_info.cost))

        --7.29
        local prop_info = Prop_Data:getInstance():get_prop_info_by_id(self.item_id[i])
        local lock_text = prop_info.lock_text
        text_unlock_num:setString(lock_text)
        --7.29
        --7.13
        local player_bag_info = Player:getInstance():get_player_bag_info_by_id(self.item_id[i])
        local item_num = player_bag_info.item_num
        local item_name = bag_info.item_name
        --print(item_name..":"..item_num)
        if item_num>0 then
            text_num:setString(string.format("%d", item_num))
            text_price:setString(ResourceManager:getInstance():getTextStringByKey("LevelInfoLayer_3"))
            text_price:setPositionX(text_price:getPositionX()-10)
            image_gold:setVisible(false)
        else
            image_mark:setVisible(false)
        end
        --7.13

        local item_Btn = ButtonWrap:create()
        item_Btn:wrap(item, 1)
        item_Btn:addTouchEndCallback(function()

            TRIGGER_EVENT("OUT_GAME_ITEM_SELECTED")
            
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
            if self.is_buy_item[self.item_id[i]] then
                --取消
                self.is_buy_item[self.item_id[i]] = false
                if self.is_use_item_from_bag[self.item_id[i]] then
                    self.is_use_item_from_bag[self.item_id[i]] = false
                    Player:getInstance():add_player_bag_info_by_id(self.item_id[i], 1)
                else
                    self:cancel_item(self.item_id[i])   --返还金币
                end
                image_get:setVisible(false)
                image_item:setColor(cc.c3b(255, 255, 255))
               
            else
                --购买
                if item_num>0 then--增加self.is_use_item_from_bag变量来判断是用金币购买的还是背包中的道具
                    --对应道具-1
                    self.is_use_item_from_bag[self.item_id[i]] = true
                    Player:getInstance():add_player_bag_info_by_id(self.item_id[i], -1)
                    self.is_buy_item[self.item_id[i]] = true
                    image_get:setVisible(true)
                    image_item:setColor(cc.c3b(255 * 0.5, 255 * 0.5, 255 * 0.5))

                else
                    if self.is_fish_guide then
                        self.is_fish_guide = false
                        self.is_buy_item[self.item_id[i]] = true
                        image_get:setVisible(true)
                        image_item:setColor(cc.c3b(255 * 0.5, 255 * 0.5, 255 * 0.5))
                    elseif self:is_enought_currency_by_add_item(self.item_id[i]) then  --扣除金币
                        self.is_buy_item[self.item_id[i]] = true
                        image_get:setVisible(true)
                        image_item:setColor(cc.c3b(255 * 0.5, 255 * 0.5, 255 * 0.5))
                    end

                end

                local event = cc.EventCustom:new("battle_item_hooked")
                event.param = {}
                cc.Director:getInstance():getEventDispatcher():dispatchEvent(event) 
                
            end
        end)

        --7.18
        if out_game_item[i] == "LOCKED" then
            item:loadTexture("images/item/item_lock_1.png")
            image_item:setVisible(false)
            text_price:setVisible(false)
            image_gold:setVisible(false)
            image_mark:setVisible(false)
            item_Btn:disable()
        else
            text_unlock_num:setVisible(false)
        end
        --7.18

    end

    --  道具解释按钮
    local info_image = image_item:getChildByName("button_info")
    if out_game_item[1] == "LOCKED" then
        info_image:setVisible(false)
    else
        local info_Btn = ButtonWrap:create()
        info_Btn:wrap(info_image, 1)
        info_Btn:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
            local itemInfoLayer = ItemInfoLayer:create()
            itemInfoLayer:hide_item(out_game_item)
            LayerManager:getInstance():add_layer_to_scene(itemInfoLayer)
        end)
    end
    

    -- 按钮
    -- 皮肤按钮
    local skin_image = image_dia:getChildByName("button_skin")
    local skin_Btn = ButtonWrap:create()
    skin_Btn:wrap(skin_image, 1)
    skin_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        if Player:getInstance():get_player_cur_role_id() == "role_101" then
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_xin"), false)
        end
        local roleInfoLayer = RoleInfoLayer:create()
        LayerManager:getInstance():add_layer_to_scene(roleInfoLayer)
    end)

    -- 角色切换按钮
    local right_image = image_dia:getChildByName("button_right")
    local right_Btn = ButtonWrap:create()
    right_Btn:wrap(right_image, 1)
    right_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:select_next_role(true)
    end)

    local left_image = image_dia:getChildByName("button_left")
    local left_Btn = ButtonWrap:create()
    left_Btn:wrap(left_image, 1)
    left_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:select_next_role(false)
    end)

    --排行榜
    self:init_rank()

    local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(level_index)
    local player_mission_info = Player:getInstance():get_player_mission_info_by_id(mission_info.mission_id)

    self:update_Chapter(level_index, player_mission_info.star_count, player_mission_info.max_score)

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()

        --7.13
        for i=1,#self.item_id do
            if self.is_buy_item[self.item_id[i]] then
                if self.is_use_item_from_bag[self.item_id[i]] then
                    self.is_use_item_from_bag[self.item_id[i]] = false
                    Player:getInstance():add_player_bag_info_by_id(self.item_id[i], 1)
                else
                    self:cancel_item(self.item_id[i])   --返还金币
                end
                
            end
        end        
        --7.13

        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click_Audio"), false)
        -- if self.close_callback then
        --     self.close_callback()
        -- end
        self:close_self()
    end)

    -- 开始按钮
    local start_image = image_dia:getChildByName("button_start")
    local start_scale = start_image:getScale()
    move_cycle_action(start_image)
    local start_Btn = ButtonWrap:create()
    start_Btn:wrap(start_image, 1)
    start_Btn:addTouchBeginCallback(function()
        start_image:stopAllActions()
    end)
    start_Btn:addTouchCancelCallback(function()
        move_cycle_action(start_image)
    end)
    start_Btn:addTouchEndCallback(function()

        TRIGGER_EVENT("GAME_START_PRESSED")

        local maskLayer = MaskLayer:create()
        self:addChild(maskLayer, 5)

        Player:getInstance():set_is_player_handle_in_game(false)    --8.26

        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        
        if Player:getInstance():get_player_power() < Player:getInstance():get_play_once_need_power() then
            -- local shopEnergyLayer = ShopEnergyLayer:create()
            -- LayerManager:getInstance():add_layer_to_scene(shopEnergyLayer)
            --113
            local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Energy)
            LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)
            
            maskLayer:removeFromParent()
            return
        end
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_ks"), false)

        
        start_image:setScale(start_scale)
        start_image:stopAllActions()
        local effect = rm:getSpineByKey("SPINE_XIAOHAO", 1.0)
        effect:setAnimation(0, "xiaohao", false)
        effect:registerSpineEventHandler(function (event)
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Staruse_Audio"), false)
            local level_index = Player:getInstance():get_player_current_level()
            local section_index = Player:getInstance():get_player_current_section()
            local player = Player:getInstance()
            --无限体力判断
            local is_Unlimit_Power = cc.UserDefault:getInstance():getBoolForKey("is_Unlimit_Power", false)
            if is_Unlimit_Power then
                
            else
                player:add_player_power(-player:get_play_once_need_power())
            end
            
            --

            self:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(function()

                -- 开始游戏

                local current_role_id = Player:getInstance():get_player_cur_role_id()
                local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)
                local skill_info = Skill_Data:getInstance():get_skill_info_by_id(role_info.skill_id)
                local skill_level = Player:getInstance():get_player_skill_level(current_role_id)
                local mission_info = Mission_Data:getInstance():get_mission_info_by_level(level_index)

                local current_skin_id = Player:getInstance():get_player_cur_skin_id()

                local skill = {}
                skill.name = skill_info.class_name
                skill.change_number = skill_info.change_num[skill_level]
                skill.fill_threshold = skill_info.trigger_condition[skill_level]

                local item = {}
                item.move = self.is_buy_item["item_201"]
                item.bomb = self.is_buy_item["item_202"]
                item.lightning = self.is_buy_item["item_203"]

                
                local str_level = "关卡"..level_index
                --遍历self.is_buy_item，有使用战前道具则发送消息
                for i=1,#self.item_id do
                    if self.is_buy_item[self.item_id[i]] then
                        TalkingDataManager:getInstance():send("每关每个道具使用次数统计" ,{[str_level] = self.item_id[i]})
                        TalkingDataManager:getInstance():send("道具的使用次数统计", {["道具使用的item_id"] = self.item_id[i]})
                        --TalkingDataManager:getInstance():sendOnUse(self.item_id[i],1)
                    end
                end
                --

                local scene_start_param = {}
                scene_start_param.skill_info = skill
                scene_start_param.item_info = item
                scene_start_param.level_index = level_index
                scene_start_param.random_seed = mission_info.random_seed
                scene_start_param.tutorial_config = mission_info.tutorial_config

                local loadingScene = LoadingScene:create(2)
                loadingScene:load_game_scene_resource(scene_start_param)
                LayerManager:getInstance():clear_layer()
                cc.Director:getInstance():replaceScene(loadingScene)


            end)))
        end, sp.EventType.ANIMATION_END)
        local size = cc.Director:getInstance():getVisibleSize()
        effect:setPosition(cc.p(size.width/2, size.height/2))
        self:addChild(effect, 3)

        self:runAction(cc.Sequence:create(cc.DelayTime:create(0.7), cc.CallFunc:create(function()
            local pos = start_image:getParent():convertToWorldSpace(cc.p(start_image:getPosition()))
            local xiaohao_particle = rm:getParticleByKey("PARTICLE_XIAOHAO")
            xiaohao_particle:setAutoRemoveOnFinish(false)
            xiaohao_particle:setPosition(cc.p(pos))
            self:addChild(xiaohao_particle, 4)
        end)))

        local event = cc.EventCustom:new("battle_start_button_pressed")
        event.param = {}
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event) 

    end)                        

	local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function LevelInfoLayer:update_Chapter(level, star, score)
    local rpc_param = {

        PLAYER_ID,
        level,
        star,
        score,
    }

    RPC:call_bind("SHIN/chapter/updateChapter", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            global_check_remote_archive(function()end)

            self:request_getfriendRanklist(self.friend_async_callback)
            -- self:request_getRanklist(self.world_async_callback)
        else

            print("request error")

        end

    end)
end

function LevelInfoLayer:close_self()
    if self.close_callback then
        self.close_callback()
    end
        
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("item_info_out", false)
    --self:removeFromParent()

    set_list_fade_out(self.rank_friends)
    set_list_fade_out(self.rank_world)
end

function LevelInfoLayer:init_rank()
    local image_rank_dia = self.node:getChildByName("image_rank_dia")
    local panel_friends = image_rank_dia:getChildByName("panel_friends")
    local panel_world = image_rank_dia:getChildByName("panel_world")
    local text_my_score = image_rank_dia:getChildByName("text_my_score")
    text_my_score:setVisible(false)
    local invite_image = image_rank_dia:getChildByName("button_invite")
    local text_friends = panel_friends:getChildByName("text_friends")
    local text_world = panel_world:getChildByName("text_world")
    local world_image = panel_friends:getChildByName("button_world")
    local text_world_ = world_image:getChildByName("text_world")
    local friends_image = panel_world:getChildByName("button_friends")
    local text_friends_ = friends_image:getChildByName("text_friends")
    
    text_friends_:setColor(cc.c3b(21, 102, 164))
    text_world_:setColor(cc.c3b(21, 102, 164))
    panel_friends:setVisible(true)
    panel_world:setVisible(false)
    text_friends:setColor(cc.c3b(148, 55, 51))
    text_world:setColor(cc.c3b(148, 55, 51))

    local invite_Btn = ButtonWrap:create()
    invite_Btn:wrap(invite_image, 1)
    invite_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        local friendAddLayer = FriendAddLayer:create()
        LayerManager:getInstance():add_layer_to_scene(friendAddLayer) 
    end)
    local friends_Btn = ButtonWrap:create()
    friends_Btn:wrap(friends_image, 1)
    friends_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        panel_friends:setVisible(true)
        panel_world:setVisible(false)
    end)
    local world_Btn = ButtonWrap:create()
    world_Btn:wrap(world_image, 1)
    world_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        panel_friends:setVisible(false)
        panel_world:setVisible(true)

        if not self.is_request_ranklist then
            self.is_request_ranklist = true
            self:request_getRanklist(self.world_async_callback)
        end
    end)
    self.is_request_ranklist = false

    local rank_res = {
        [1] = "images/section/friend/rank_first.png",
        [2] = "images/section/friend/rank_second.png",
        [3] = "images/section/friend/rank_third.png",
    }

    local rank_friends = panel_friends:getChildByName("rank_friends")
    self.rank_friends = rank_friends
    local panel_1 = rank_friends:getChildByName("panle_1")
    rank_friends:setClippingEnabled(true)
    --rank_friends:setScrollBarEnabled(false)
    rank_friends:setItemModel(panel_1:clone())
    rank_friends:removeAllItems()
    local function friend_async_callback(list)
        local friendRanklist = list
        local count = get_table_length(friendRanklist)
        for i=1,count do
            rank_friends:pushBackDefaultItem()
        end
        for i=1,count do
            local item = rank_friends:getItem(i -1)
            local image_head = item:getChildByName("image_id")
            local text_name = item:getChildByName("text_name")
            local text_score = item:getChildByName("text_score")
            local image_rank_num = item:getChildByName("image_rank_num")
            local text_rank_num = item:getChildByName("text_rank_num")

            if i < 4 then
                text_rank_num:setVisible(false)
                image_rank_num:setVisible(true)
                image_rank_num:loadTexture(rank_res[i])
            else
                image_rank_num:setVisible(false)
                text_rank_num:setVisible(true)
                text_rank_num:setString(i)
            end
            local friendRank_info = friendRanklist[i]
            --text_rank_num:setString(friendRank_info.rank)
            text_name:setString(friendRank_info.name)
            text_score:setString(friendRank_info.score)

            if friendRank_info.sex == "0" and friendRank_info.headUrl and friendRank_info.headUrl ~= "" then
                
                local function on_create_finish()
                    local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friendRank_info.guid.."head_00"
                    if cc.FileUtils:getInstance():isFileExist(headPath) then
                        image_head:loadTexture(headPath)
                    end
                end
                global_create_social_head(friendRank_info.guid, friendRank_info.headUrl, on_create_finish)

            else
                local head_id = string.format("head_%02d", friendRank_info.sex+1)
                local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                image_head:loadTexture(head_info.res_key)
            end

            if friendRank_info.guid == PLAYER_ID then
                text_my_score:setVisible(true)
                text_my_score:setString(friendRank_info.score)

                -- local player_head_id = Player:getInstance():get_player_head_id()
                -- if player_head_id == "head_00" then
                --     local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
                --     image_head:loadTexture(headPath)
                -- end
            end
        end
    end
    self.friend_async_callback = friend_async_callback
    -- self:request_getfriendRanklist(friend_async_callback)

    local rank_world = panel_world:getChildByName("rank_world")
    self.rank_world = rank_world
    local panel_1 = rank_world:getChildByName("panle_1")
    rank_world:setClippingEnabled(true)
    --rank_world:setScrollBarEnabled(false)
    rank_world:setItemModel(panel_1:clone())
    rank_world:removeAllItems()
    local function world_async_callback(list)
        local worldRanklist = list
        local count = get_table_length(worldRanklist)
        for i=1,count do
            rank_world:pushBackDefaultItem()
        end
        for i=1,count do
            local item = rank_world:getItem(i -1)
            local image_head = item:getChildByName("image_id")
            local text_name = item:getChildByName("text_name")
            local text_score = item:getChildByName("text_score")
            local image_rank_num = item:getChildByName("image_rank_num")
            local text_rank_num = item:getChildByName("text_rank_num")

            if i < 4 then
                text_rank_num:setVisible(false)
                image_rank_num:setVisible(true)
                image_rank_num:loadTexture(rank_res[i])
            else
                image_rank_num:setVisible(false)
                text_rank_num:setVisible(true)
                text_rank_num:setString(i)
            end
            local worldRank_info = worldRanklist[i]
            --text_rank_num:setString(worldRank_info.rank)
            text_name:setString(worldRank_info.name)
            text_score:setString(worldRank_info.score)

            if worldRank_info.sex == "0" and worldRank_info.headUrl and worldRank_info.headUrl ~= "" then
                
                local function on_create_finish()
                    local headPath = FileManipulator:getInstance():getPath().."head_cache/"..worldRank_info.guid.."head_00"
                    if cc.FileUtils:getInstance():isFileExist(headPath) then
                        image_head:loadTexture(headPath)
                    end
                end
                global_create_social_head(worldRank_info.guid, worldRank_info.headUrl, on_create_finish)

            else
                local head_id = string.format("head_%02d", worldRank_info.sex+1)
                local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                image_head:loadTexture(head_info.res_key)
            end

            -- local head_id = string.format("head_%02d", worldRank_info.sex+1)
            -- local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
            -- image_head:loadTexture(head_info.res_key)

            -- if worldRank_info.guid == PLAYER_ID then
            --     local player_head_id = Player:getInstance():get_player_head_id()
            --     if player_head_id == "head_00" then
            --         local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
            --         image_head:loadTexture(headPath)
            --     end
            -- end
        end
    end
    self.world_async_callback = world_async_callback
    -- self:request_getRanklist(world_async_callback)

    rank_friends:setVisible(false)
    rank_world:setVisible(false)
    rank_friends:runAction(cc.Sequence:create(cc.FadeIn:create(0.5),cc.CallFunc:create(function()rank_friends:setVisible(true) end)))
    rank_world:runAction(cc.Sequence:create(cc.FadeIn:create(0.5),cc.CallFunc:create(function()rank_world:setVisible(true) end)))
end

function LevelInfoLayer:request_getfriendRanklist(async_callback)
    --request getfriendRanklist 好友排行
    local level_index = Player:getInstance():get_player_current_level()
    local rpc_param = {

        PLAYER_ID,
        level_index,
    }

    RPC:call_bind("SHIN/chapter/getfriendRanklist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            -- dump(response)
            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function LevelInfoLayer:request_getRanklist(async_callback)
    --request getRanklist 世界排行
    local level_index = Player:getInstance():get_player_current_level()
    local rpc_param = {
        level_index,
    }

    RPC:call_bind("SHIN/chapterMgr/getRanklist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function LevelInfoLayer:show_goal_candy(image_goal_dia)

    local texture_path_map = {

        boss_1 = "images/boss/boss_1.png",
        boss_2 = "images/boss/boss_2.png",
        boss_3 = "images/boss/boss_3.png",
        boss_4 = "images/boss/boss_4.png",
        boss_5 = "images/boss/boss_5.png",
        boss_6 = "images/boss/boss_6.png",
        boss_7 = "images/boss/boss_7.png",

    }

    local mission_level = Player:getInstance():get_player_current_level()
    local level_config = LevelBuilder:get_instance():get_level_config(mission_level)
    local goal_map = {}
    local index = 1
    local count = 0
    for key, value in pairs(level_config.reqs) do
        if not (value == 0) then
            goal_map[index] = {["type"] = key, ["num"] = value}
            if not (type(value) == "number") then
                print(ResourceManager:getInstance():getTextStringByKey("LevelInfoLayer_2"))
            end
        end
        index = index + 1
        count = count + 1
    end

    local is_boss = false
    if level_config.boss then
        is_boss = true
    end

    local panel_candy = image_goal_dia:getChildByName("panel_candy")
    local text_candy_num = panel_candy:getChildByName("text_candy_num")
    local image_candy = panel_candy:getChildByName("image_candy")
    text_candy_num:setVisible(false)
    image_candy:setVisible(false)

    local panel_boss = image_goal_dia:getChildByName("panel_boss")
    local image_boss = panel_boss:getChildByName("image_boss")
    local text_boss = panel_boss:getChildByName("text_boss")
    image_boss:setVisible(false)
    text_boss:setVisible(false)
    if is_boss then
        image_boss:setVisible(true)
        image_boss:setTexture(texture_path_map[level_config.boss.texture])
        text_boss:setVisible(true)
    else
        for i=1, count do
            print(goal_map[i].type)
            local candy = cc.Sprite:create(self.candy_res[goal_map[i].type])
            local text = cc.Label:createWithTTF("12", "font/zh_CN.TTF", 36)
            candy:setPosition(cc.p(image_candy:getPositionX() + (100 + (4- count) * 15) * (i - 1- (count -1)/2.0), image_candy:getPositionY()))
            text:setPosition(cc.p(text_candy_num:getPositionX() + (100 + (4- count) * 15) * (i - 1- (count -1)/2.0), text_candy_num:getPositionY() - (4-count) *10))
            text:setTextColor(text_candy_num:getTextColor())
            text:setString(string.format("%d", goal_map[i].num))
            panel_candy:addChild(candy)
            panel_candy:addChild(text)
            candy:setScale(1.0 + (4 - count)/4.0)
            text:setScale(1.0 + (4 - count)/4.0)
        end
    end
end

function LevelInfoLayer:select_next_role(is_next)
    local player = Player:getInstance()
    local player_role_info_vector = player:get_player_role_info_vector()
    local player_cur_role_id = player:get_player_cur_role_id()
    local index = 1
    for i=1,#player_role_info_vector do
        if player_cur_role_id == player_role_info_vector[i].role_id then
            index = i
        end
    end
    if is_next then
        index = math.max(1, (index + 1)%(#player_role_info_vector+1))
        while not(player_role_info_vector[index].is_unlock) do
            index = math.max(1, (index + 1)%(#player_role_info_vector+1))
        end
    else
        index = index - 1
        if index == 0 then
            index = #player_role_info_vector
        end
        while not(player_role_info_vector[index].is_unlock) do
            index = index - 1
            if index == 0 then
                index = #player_role_info_vector
            end
        end
    end
    
    player:set_player_cur_role_id(player_role_info_vector[index].role_id)
    player:set_player_cur_skin_id(player_role_info_vector[index].skin_id)

end

function LevelInfoLayer:is_enought_currency_by_add_item(item_id)
    local player = Player:getInstance()
    local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(item_id)
    if shop_item_info.currency == "gold" then
        if Player:getInstance():get_player_gold() < shop_item_info.cost then
            -- local shopLayer = ShopLayer:create()
            -- shopLayer:show_goods_by_type(ShopLayer.SHOP_TYPE.GOLD)
            -- LayerManager:getInstance():add_layer_to_scene(shopLayer) --113

            local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Gold)
            LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)

            local warningLayer = WarningLayer:create()
            warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("needGold"))
            LayerManager:getInstance():add_layer_to_scene(warningLayer)
            return false
        else
            player:add_player_gold(-shop_item_info.cost)
        end

    elseif shop_item_info.currency == "jewel" then
        if Player:getInstance():get_player_jewel() < shop_item_info.cost then
            -- local shopLayer = ShopLayer:create()
            -- shopLayer:show_goods_by_type(ShopLayer.SHOP_TYPE.JEWEL)
            -- LayerManager:getInstance():add_layer_to_scene(shopLayer) --113

            local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Jewel)
            LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)
            
            local warningLayer = WarningLayer:create()
            warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("needJewel"))
            LayerManager:getInstance():add_layer_to_scene(warningLayer)
            
            return false
        else
            player:add_player_jewel(-shop_item_info.cost)
            TalkingDataManager:getInstance():sendOnPurchase(item_id,1,shop_item_info.cost)
        end
    end
    return true
end

function LevelInfoLayer:cancel_item(item_id)
    local player = Player:getInstance()
    local shop_item_info = Shop_Item_Data:getInstance():get_shop_item_info_by_id(item_id)
    if shop_item_info.currency == "gold" then
        player:add_player_gold(shop_item_info.cost)
    elseif shop_item_info.currency == "jewel" then
        player:add_player_jewel(shop_item_info.cost)
    end
end

function LevelInfoLayer:update_skin_image()
    local player = Player:getInstance()
    local role_info = Role_Data:getInstance():get_role_info_by_id(player:get_player_cur_role_id())
    local player_role_info = player:get_player_role_info_by_id(role_info.role_id)
    local skin_info = Skin_Data:getInstance():get_skin_info_by_id(player:get_player_cur_skin_id())
    self.image_role:setTexture(skin_info.res_key)
    self.text_role_name:setString(role_info.role_name)
    self.text_role_level:setString(string.format("lv.%d", player_role_info.role_level))
end

function LevelInfoLayer:set_close_callback(close_callback)
    self.close_callback = close_callback
end

function LevelInfoLayer:get_close_callback()
    return self.close_callback
end

function LevelInfoLayer:onEnter()
    local function update_skin(event) self:update_skin_image() end
    local listener1 = cc.EventListenerCustom:create(NotificationUtil.EventType.player_skin_change, update_skin)
    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener1, self)

    RPC:register_target(self)
end

function LevelInfoLayer:onExit()

    RPC:unregister_target(self)

end

return LevelInfoLayer