require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "core.TalkingDataManager"
LimitGiftLayer = class("LimitGiftLayer", function() return cc.Layer:create() end)

function LimitGiftLayer:ctor()

    self:setName("LimitGiftLayer")
    self.close_callback = nil

	local widget = cc.CSLoader:createNode("limit_gift.csb")
    self:addChild(widget, 1);

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("limit_gift.csb")
    widget:runAction(self.action)

    self.action:setCurrentFrame(0)
    self.action:play("springsale_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "springsale_in_end" then

        elseif str == "springsale_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

	
    local player = Player:getInstance()
    local gift_1 = image_dia:getChildByName("gift_1")
    local springsale_info_vector = require "script.springsale_info"
    local springsale_info = springsale_info_vector[2]
    self.springsale_info = springsale_info
    for i=1,4 do
    	local item = gift_1:getChildByName(i)
    	local image_item = item:getChildByName("image_item")
    	local text_num = item:getChildByName("text_num")

    	local bag_info = Bag_Data:getInstance():get_bag_info_by_id(springsale_info.item_vector[i].item_id)

        if springsale_info.item_vector[i].item_id == "item_02" then
            image_item:loadTexture("images/item/item_8.png")
        elseif springsale_info.item_vector[i].item_id == "item_03" then
            image_item:loadTexture("images/item/item_9.png")
        else
            image_item:loadTexture(bag_info.res_key)
        end
    	text_num:setString(string.format("x%d",springsale_info.item_vector[i].item_num))
    end

    local buy_Image = image_dia:getChildByName("button_buy")
    local text_price = buy_Image:getChildByName("text_price")
    local image_price = buy_Image:getChildByName("image_price")
    text_price:setString(springsale_info.cost)
    if springsale_info.currency == "jewel" then
    	image_price:loadTexture("images/shop/jewel.png")
    elseif springsale_info.currency == "rmb" then
    	image_price:loadTexture("images/shop/rmb.png")
    end

    local buy_Btn = ButtonWrap:create()
    buy_Btn:wrap(buy_Image, 1)
    buy_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        if springsale_info.currency == "jewel" then
	    	if springsale_info.cost <= player:get_player_jewel() then
                player:add_player_jewel(-springsale_info.cost)
                TalkingDataManager:getInstance():sendOnPurchase(springsale_info.sale_name,1,springsale_info.cost)

                self:buy_springsale_success()
            else
                local shopLayer = ShopLayer:create()
                shopLayer:show_goods_by_type(ShopLayer.SHOP_TYPE.JEWEL)
                LayerManager:getInstance():add_layer_to_scene(shopLayer)
                
                local warningLayer = WarningLayer:create()
                warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("needJewel"))
                LayerManager:getInstance():add_layer_to_scene(warningLayer)
            end
	    elseif springsale_info.currency == "rmb" then
	    	TradeManager:getInstance():pay_buy_currency(springsale_info.sale_name, springsale_info.cost,
	    		function() self:buy_springsale_success() end, nil)
	    end
    end)

    -- 关闭按钮
    local closeImage = image_dia:getChildByName("button_close")
    local closeBtn = ButtonWrap:create()
    closeBtn:wrap(closeImage, 1)
    closeBtn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

end

function LimitGiftLayer:buy_springsale_success()
    local player = Player:getInstance()
    local item_vector = self.springsale_info.item_vector
    for k,v in pairs(item_vector) do
        local item_id = v.item_id
        local item_num = v.item_num
        if item_id == "item_02" or item_id == "item_03" then
            player:add_player_currency_by_id(item_id, item_num)
            if item_id == "item_03" then
                TalkingDataManager:getInstance():sendOnReward(item_num,self.springsale_info.sale_name)
            end
        else
            player:add_player_bag_info_by_id(item_id, item_num)
        end
    end
    player:set_is_buy_limit_gift(true)
    require "core.TalkingDataManager"
    TalkingDataManager:getInstance():send("礼包", {["限购礼包购买次数"] = "次数"})

    local commonPopupLayer1 = CommonPopupLayer:create()

    commonPopupLayer1:disable_close()
    local str1 = "购买成功"
    local str2 = string.format("购买%s成功",self.springsale_info.sale_name)
    local str3 = "确定"
    commonPopupLayer1:showMessage(str1, str2, str3, function()
        commonPopupLayer1:close_self()
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.limit_gift_event)
        self:close_self()
    end, nil)

end

function LimitGiftLayer:close_self()
    if self.close_callback then
        self.close_callback()
    end
    
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("springsale_out", false)
end

function LimitGiftLayer:set_close_callback(callback)
    self.close_callback = callback
end

function LimitGiftLayer:onExit()
end

return LimitGiftLayer