require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "UI.Layer.FriendAddLayer"
require "UI.Layer.RequestEnergyLayer"

MailLayer = class("MailLayer", function() return cc.Layer:create() end)

function MailLayer:ctor()

    self:setName("MailLayer")
    self.reqFriendsList = nil
    self.sendFriendsList = nil
    self.sendgiftnum_limit = 20
    self.gotgiftnum_limit = 10
    self.sendgiftnum = 0
    self.gotgiftnum = 0

    local widget = cc.CSLoader:createNode("mail.csb")
    self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("mail.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    self.action:play("mail_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "mail_in_end" then

        elseif str == "mail_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")
    self.image_dia = image_dia

    local all_agree_image = image_dia:getChildByName("button_all_agree")
    local all_ignore_image = image_dia:getChildByName("button_all_ignore")
    self.all_agree_image = all_agree_image
    self.all_ignore_image = all_ignore_image
    all_agree_image:setVisible(false)
    all_ignore_image:setVisible(false)

    --全部同意
    local all_agree_Btn = ButtonWrap:create()
    all_agree_Btn:wrap(all_agree_image, 1)
    all_agree_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        --体力全同意
        for k,v in pairs(self.energy_agree_Btn_vector) do
            if v:isImageVisible() and v:isDarkButton() == false then
                v:triggerTouchEndCallback()
            end
        end
        for k,v in pairs(self.energy_get_Btn_vector) do
            if v:isImageVisible() and v:isDarkButton() == false then
                v:triggerTouchEndCallback()
            end
        end
    end)
    --全部忽略
    local all_ignore_Btn = ButtonWrap:create()
    all_ignore_Btn:wrap(all_ignore_image, 1)
    all_ignore_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        --好友全忽略
        for k,v in pairs(self.friend_close_Btn_vector) do
            if v:isImageVisible() then
                v:triggerTouchEndCallback()
            end
        end
    end)
    
    local friend_image = image_dia:getChildByName("button_friend")
    local energy_image = image_dia:getChildByName("button_energy")
    local mail_image = image_dia:getChildByName("button_mail")
    self.button_image = {}
    self.button_image[1] = friend_image
    self.button_image[2] = energy_image
    self.button_image[3] = mail_image
    local friend_image_mark = friend_image:getChildByName("image_mark")
    local energy_image_mark = energy_image:getChildByName("image_mark")
    local mail_image_mark = mail_image:getChildByName("image_mark")
    friend_image_mark:setVisible(false)
    energy_image_mark:setVisible(false)
    mail_image_mark:setVisible(false)
    self.friend_mark_count = 0
    self.energy_mark_count = 0
    self.mail_mark_count = 0

    local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    local function update_friend_mark_count()
        local count = 0
        count = self.friend_mark_count
        if count > 0 then
            friend_image_mark:setVisible(true)
            friend_image_mark:getChildByName("text_1"):setString(count)
        else
            friend_image_mark:setVisible(false)
        end
    end
    local listener1 = cc.EventListenerCustom:create(NotificationUtil.EventType.mail_friend_mark_count, update_friend_mark_count)
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener1, self)

    local function update_energy_mark_count()
        local count = 0
        count = self.energy_mark_count
        if count > 0 then
            energy_image_mark:setVisible(true)
            energy_image_mark:getChildByName("text_1"):setString(count)
        else
            energy_image_mark:setVisible(false)
        end
    end
    local listener2 = cc.EventListenerCustom:create(NotificationUtil.EventType.mail_energy_mark_count, update_energy_mark_count)
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener2, self)

    local function update_mail_mark_count()
        local count = 0
        count = self.mail_mark_count
        if count > 0 then
            mail_image_mark:setVisible(true)
            mail_image_mark:getChildByName("text_1"):setString(count)
        else
            mail_image_mark:setVisible(false)
        end
    end
    local listener3 = cc.EventListenerCustom:create(NotificationUtil.EventType.mail_mail_mark_count, update_mail_mark_count)
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener3, self)


    local friend_layout = image_dia:getChildByName("friend")
    local energy_layout = image_dia:getChildByName("energy")
    local mail_layout = image_dia:getChildByName("mail")
    friend_layout:setVisible(false)
    energy_layout:setVisible(false)
    mail_layout:setVisible(false)
    self.layout_vector = {}


    self:init_layout()
    self:show_layout_by_type(1)

    local friend_Btn = ButtonWrap:create()
    friend_Btn:wrap(friend_image, 1)
    friend_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:show_layout_by_type(1)
    end)
    local energy_Btn = ButtonWrap:create()
    energy_Btn:wrap(energy_image, 1)
    energy_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:show_layout_by_type(2)
    end)
    local mail_Btn = ButtonWrap:create()
    mail_Btn:wrap(mail_image, 1)
    mail_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:show_layout_by_type(3)
    end)
    

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    --bag_get_reward
    self.bag_reward = BagRewardWidget:create()
    self:addChild(self.bag_reward, 5)

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function MailLayer:show_layout_by_type(mail_type)
    
    if mail_type == 1 then
        self.all_agree_image:setVisible(false)
        if self.friend_mark_count > 0 then
            self.all_ignore_image:setVisible(true)
        end
        
    elseif mail_type == 2 then
        if self.energy_mark_count > 0 then
            self.all_agree_image:setVisible(true)
        end
        self.all_ignore_image:setVisible(false)
    else
        self.all_agree_image:setVisible(false)
        self.all_ignore_image:setVisible(false)
    end
    for key, value in pairs(self.button_image) do
        if key == mail_type then
            self.button_image[key]:setColor(cc.c3b(255, 255, 255))
        else
            self.button_image[key]:setColor(cc.c3b(255 * 0.6, 255 * 0.6, 255 * 0.6))
        end
    end

    for key,value in pairs(self.layout_vector) do
        if key == mail_type then
            self.layout_vector[key]:setVisible(true)
        else
            self.layout_vector[key]:setVisible(false)
        end
    end

end

function MailLayer:init_layout()
    self:init_friend_layout()
    self:init_energy_layout()
    self:init_mail_layout()

    self:request_sendgiftnum_and_gotgiftnum()
end

function MailLayer:init_friend_layout()
    local friend_layout = self.image_dia:getChildByName("friend")
    self.layout_vector[1] = friend_layout
    local image_no_add_friend = friend_layout:getChildByName("image_no_add_friend")
    image_no_add_friend:setVisible(false)
    local add_image = image_no_add_friend:getChildByName("button_add")
    local add_Btn = ButtonWrap:create()
    add_Btn:wrap(add_image, 1)
    add_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        local friendAddLayer = FriendAddLayer:create()
        LayerManager:getInstance():add_layer_to_scene(friendAddLayer) 
    end)

    local friend_listview = friend_layout:getChildByName("panel_friend")
    self.friend_listview = friend_listview
    local panel_1 = friend_listview:getChildByName("panel_1")
    --friend_listview:setScrollBarEnabled(false)
    friend_listview:setItemModel(panel_1:clone())
    friend_listview:removeAllItems()
    self.friend_close_Btn_vector = {}

    local function async_callback(list)
        local addFriendsList = list

        local count = get_table_length(addFriendsList)

        self.friend_mark_count = count
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_friend_mark_count)
        if count <= 0 then
            image_no_add_friend:setVisible(true)
            friend_listview:setVisible(false)
            self.all_ignore_image:setVisible(false)
        else
            image_no_add_friend:setVisible(false)
            friend_listview:setVisible(true)
            self.all_ignore_image:setVisible(true)
        end

        for i=1,count do
            friend_listview:pushBackDefaultItem()
        end
        for i=1,count do
            local item = friend_listview:getItem(i -1)
            local text_nick_name = item:getChildByName("text_name")
            local image_head = item:getChildByName("image_id")
            local text_des = item:getChildByName("text_des")
            local agree_image = item:getChildByName("button_agree")
            local close_image = item:getChildByName("close")
            local image_get = item:getChildByName("image_get")
            local text_ignored = item:getChildByName("text_ignored")
            image_get:setVisible(false)
            text_ignored:setVisible(false)

            local agree_Btn = ButtonWrap:create()
            agree_Btn:wrap(agree_image, 1)
            agree_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                agree_image:setVisible(false)
                close_image:setVisible(false)
                image_get:setVisible(true)
                
                self:send_agreeFriendAdd(addFriendsList[i].id)
                self.friend_mark_count = self.friend_mark_count - 1
                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_friend_mark_count)
            end)
            local close_Btn = ButtonWrap:create()
            self.friend_close_Btn_vector[i] = close_Btn
            close_Btn:wrap(close_image, 1)
            close_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                agree_image:setVisible(false)
                close_image:setVisible(false)
                text_ignored:setVisible(true)
                
                self:send_ignoreFriendAdd(addFriendsList[i].id)
                self.friend_mark_count = self.friend_mark_count - 1
                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_friend_mark_count)
            end)

            local friend_info = addFriendsList[i].senderinfo
            
            if friend_info.sex == "0" and friend_info.headUrl and friend_info.headUrl ~= "" then
                
                local function on_create_finish()
                    local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friend_info.guid.."head_00"
                    if cc.FileUtils:getInstance():isFileExist(headPath) then
                        image_head:loadTexture(headPath)
                    end
                end
                global_create_social_head(friend_info.guid, friend_info.headUrl, on_create_finish)

            else
                local head_id = string.format("head_%02d", friend_info.sex+1)
                local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                image_head:loadTexture(head_info.res_key)
            end
            
            text_nick_name:setString(friend_info.name)
            text_des:setString(ResourceManager:getInstance():getTextStringByKey("MailLayer1"))
        end
    end

    self.friend_async_callback = async_callback

end

function MailLayer:init_energy_layout()
    local energy_layout = self.image_dia:getChildByName("energy")
    self.layout_vector[2] = energy_layout
    local image_no_demand = energy_layout:getChildByName("image_no_demand")
    image_no_demand:setVisible(false)
    local demand_image = image_no_demand:getChildByName("button_demand")
    local demand_Btn = ButtonWrap:create()
    demand_Btn:wrap(demand_image, 1)
    demand_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        local requestEnergyLayer = RequestEnergyLayer:create()
        LayerManager:getInstance():add_layer_to_scene(requestEnergyLayer) 
    end)

    local energy_listview = energy_layout:getChildByName("panel_energy")
    self.energy_listview = energy_listview
    local panel_1 = energy_listview:getChildByName("panel_1")
    --energy_listview:setScrollBarEnabled(false)
    energy_listview:setItemModel(panel_1:clone())
    energy_listview:removeAllItems()

    self.energy_agree_Btn_vector = {}
    self.energy_get_Btn_vector = {}

    local function async_callback(list1, list2)
        local reqFriendsList = list1
        local sendFriendsList = list2
        
        local count_req = get_table_length(reqFriendsList)
        local count_send = get_table_length(sendFriendsList)
        local count = count_req + count_send
        self.energy_mark_count = count
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_energy_mark_count)
        
        if count <= 0 then
            image_no_demand:setVisible(true)
            energy_listview:setVisible(false)
        else
            image_no_demand:setVisible(false)
            energy_listview:setVisible(true)
        end

        for i=1,count do
            energy_listview:pushBackDefaultItem()
        end
        for i=1,count do
            local item = energy_listview:getItem(i -1)
            local text_nick_name = item:getChildByName("text_name")
            local image_head = item:getChildByName("image_id")
            local agree_image = item:getChildByName("button_agree")
            -- local text_agree = agree_image:getChildByName("text_agree")
            -- text_agree:setLocalZOrder(1)
            local button_get_image = item:getChildByName("button_get")
            -- local text_get = button_get_image:getChildByName("text_get")
            -- text_get:setLocalZOrder(1)
            local close_image = item:getChildByName("close")
            local image_get = item:getChildByName("image_get")
            local text_des = item:getChildByName("text_des")
            local text_des_0 = item:getChildByName("text_des_0")
            image_get:setVisible(false)
            text_des_0:setVisible(false)

            local agree_Btn = ButtonWrap:create()
            self.energy_agree_Btn_vector[i] = agree_Btn
            agree_Btn:wrap(agree_image, 1)
            agree_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                agree_image:setVisible(false)
                close_image:setVisible(false)
                button_get_image:setVisible(false)
                image_get:setVisible(true)

                --
                self.sendgiftnum = self.sendgiftnum + 1
                -- print("sendgiftnum_diff::"..self.sendgiftnum_limit - self.sendgiftnum)
                if (self.sendgiftnum_limit - self.sendgiftnum) <= 0 then
                    for k,v in pairs(self.energy_agree_Btn_vector) do
                        if v:isImageVisible() then
                            v:disable()
                            v:setDarkButtonWithText(false, "text_agree")
                        end
                    end
                end

                self:send_agreeSendEnergy(reqFriendsList[i].id, text_des_0)
                self.energy_mark_count = self.energy_mark_count - 1
                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_energy_mark_count)
            end)
            -- print("sendgiftnum_diff::"..self.sendgiftnum_limit - self.sendgiftnum)
            if (self.sendgiftnum_limit - self.sendgiftnum) <= 0 then
                agree_Btn:disable()
                agree_Btn:setDarkButtonWithText(false, "text_agree")
            end

            local get_Btn = ButtonWrap:create()
            self.energy_get_Btn_vector[i] = get_Btn
            get_Btn:wrap(button_get_image, 1)
            get_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                agree_image:setVisible(false)
                close_image:setVisible(false)
                button_get_image:setVisible(false)
                image_get:setVisible(true)

                --
                self.gotgiftnum = self.gotgiftnum + 1
                -- print("gotgiftnum_diff::"..self.gotgiftnum_limit - self.gotgiftnum)
                if (self.gotgiftnum_limit - self.gotgiftnum) <= 0 then
                    for k,v in pairs(self.energy_get_Btn_vector) do
                        if v:isImageVisible() then
                            v:disable()
                            v:setDarkButtonWithText(false, "text_get")
                        end
                    end
                end

                self:send_getEnergy(sendFriendsList[i - count_req].id, text_des_0)
                self.energy_mark_count = self.energy_mark_count - 1
                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_energy_mark_count)
            end)
            -- print("gotgiftnum_diff::"..self.gotgiftnum_limit - self.gotgiftnum)
            if (self.gotgiftnum_limit - self.gotgiftnum) <= 0 then
                get_Btn:disable()
                get_Btn:setDarkButtonWithText(false, "text_get")
            end

            local close_Btn = ButtonWrap:create()
            close_Btn:wrap(close_image, 1)
            close_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                agree_image:setVisible(false)
                close_image:setVisible(false)
                button_get_image:setVisible(false)
                
                if i <= count_req then
                    --print("忽略赠送:"..reqFriendsList[i].id)
                    self:send_ignoreSendEnergy(reqFriendsList[i].id)
                    self.energy_mark_count = self.energy_mark_count - 1
                    cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_energy_mark_count)
                else
                    --print("忽略领取:"..sendFriendsList[i - count_req].id)
                    self:send_ignoreGetEnergy(sendFriendsList[i - count_req].id)
                    self.energy_mark_count = self.energy_mark_count - 1
                    cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_energy_mark_count)
                end
            end)

            
            if i <= count_req then
                agree_image:setVisible(true)
                button_get_image:setVisible(false)
                text_des:setString(ResourceManager:getInstance():getTextStringByKey("MailLayer2"))
                local friend_info = reqFriendsList[i].senderinfo
                text_nick_name:setString(friend_info.name)

                if friend_info.sex == "0" and friend_info.headUrl and friend_info.headUrl ~= "" then
                
                    local function on_create_finish()
                        local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friend_info.guid.."head_00"
                        if cc.FileUtils:getInstance():isFileExist(headPath) then
                            image_head:loadTexture(headPath)
                        end
                    end
                    global_create_social_head(friend_info.guid, friend_info.headUrl, on_create_finish)

                else
                    local head_id = string.format("head_%02d", friend_info.sex+1)
                    local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                    image_head:loadTexture(head_info.res_key)
                end
            else
                agree_image:setVisible(false)
                button_get_image:setVisible(true)
                text_des:setString(ResourceManager:getInstance():getTextStringByKey("MailLayer3"))
                local friend_info = sendFriendsList[i - count_req].senderinfo
                text_nick_name:setString(friend_info.name)

                if friend_info.sex == "0" and friend_info.headUrl and friend_info.headUrl ~= "" then
                
                    local function on_create_finish()
                        local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friend_info.guid.."head_00"
                        if cc.FileUtils:getInstance():isFileExist(headPath) then
                            image_head:loadTexture(headPath)
                        end
                    end
                    global_create_social_head(friend_info.guid, friend_info.headUrl, on_create_finish)

                else
                    local head_id = string.format("head_%02d", friend_info.sex+1)
                    local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                    image_head:loadTexture(head_info.res_key)
                end
            end
        end
    end

    self.energy_async_callback = async_callback
end

function MailLayer:init_mail_layout()
    local mail_layout = self.image_dia:getChildByName("mail")
    self.layout_vector[3] = mail_layout
    local image_no_mail = mail_layout:getChildByName("image_no_mail")
    local text_tips = image_no_mail:getChildByName("text_tips")
    local tips_string_vector = require "script.tips_string"
    math.randomseed(os.time())
    text_tips:setString(ResourceManager:getInstance():getTipsStringByKey(math.random(1, #tips_string_vector)))
    -- image_no_mail:setVisible(false)

    local mail_listview = mail_layout:getChildByName("panel_mail")
    self.mail_listview = mail_listview
    local panel_1 = mail_listview:getChildByName("panel_1")
    --mail_listview:setScrollBarEnabled(false)
    mail_listview:setItemModel(panel_1:clone())
    mail_listview:removeAllItems()


    
    local function async_callback(list)
        
        --筛选isValid为1或者mail_type为2(告知类)的邮件
        local mailList = {}
        local count = 0
        local mark_count = 0
        for k,v in pairs(list) do
            
            if tonumber(v.isValid) == 1 then
                mark_count = mark_count + 1
            end

            if tonumber(v.isValid) == 1 or tonumber(v.mail_type) == 2 then
                count = count + 1
                mailList[count] = v
            end
        end

        self.mail_mark_count = mark_count
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_mail_mark_count)

        if count <= 0 then
            image_no_mail:setVisible(true)
            mail_listview:setVisible(false)
        else
            image_no_mail:setVisible(false)
            mail_listview:setVisible(true)
        end

        for i=1,count do
            mail_listview:pushBackDefaultItem()
        end

        self.info_vector = {}

        for i=1,count do
            local item = mail_listview:getItem(i -1)

            local panel_notice = item:getChildByName("panel_notice")
            local panel_reward = item:getChildByName("panel_reward")

            local mail_info = mailList[i]

            if tonumber(mail_info.mail_type) == 2 then
                --告知类
                panel_reward:setVisible(false)

                local notice_info = panel_notice:getChildByName("info")
                local notice_content = notice_info:getChildByName("text_des")
                local notice_title = panel_notice:getChildByName("text_des")
                local image_get = panel_notice:getChildByName("image_get")
                local button_check_image = panel_notice:getChildByName("button_get")
                local text_check = button_check_image:getChildByName("text_get")

                self.info_vector[i] = notice_info
                notice_info:setVisible(false)
                image_get:setVisible(false)

                notice_title:setString(mail_info.title)
                notice_content:setString(mail_info.content)

                local check_Btn = ButtonWrap:create()
                check_Btn:wrap(button_check_image, 1)
                check_Btn:addTouchEndCallback(function()
                    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)

                    if tonumber(mail_info.isValid) == 1 then
                        self:send_readMail(mail_info.mail_id)
                        self.mail_mark_count = self.mail_mark_count - 1
                        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_mail_mark_count)
                    end

                    -- button_check_image:setVisible(false)
                    -- image_get:setVisible(true)

                    self:move_items(i, count)
                    
                end)

            elseif tonumber(mail_info.mail_type) == 1 then
                --奖励类
                panel_notice:setVisible(false)

                local reward_title = panel_reward:getChildByName("text_des")
                local button_check_image = panel_reward:getChildByName("button_check")
                local text_check = button_check_image:getChildByName("text_check")
                local reward_info = panel_reward:getChildByName("info")
                local reward_content = reward_info:getChildByName("text_des")
                local button_get_0 = reward_info:getChildByName("button_get_0")
                local text_get = button_get_0:getChildByName("text_get")
                local image_get = reward_info:getChildByName("image_get")

                self.info_vector[i] = reward_info
                reward_info:setVisible(false)
                image_get:setVisible(false)

                reward_title:setString(mail_info.title)
                reward_content:setString(mail_info.content)

                local check_Btn = ButtonWrap:create()
                check_Btn:wrap(button_check_image, 1)
                check_Btn:addTouchEndCallback(function()
                    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)

                    self:move_items(i, count)
                    
                end)
                
                for j=1,4 do
                    reward_info:getChildByName(string.format("%d", j)):setVisible(false)
                end
                local index = get_table_length(mail_info.award)
                local reward_layout = reward_info:getChildByName(string.format("%d", index))
                reward_layout:setVisible(true)
                for j=1,index do

                    local image_reward = nil
                    local text_num = nil
                    if index == 1 then
                        
                        image_reward = reward_layout:getChildByName("image_reward")
                        text_num = reward_layout:getChildByName("text_num")
                    else
                        
                        image_reward = reward_layout:getChildByName(string.format("image_reward_%d", j))
                        text_num = reward_layout:getChildByName(string.format("text_num_%d", j))
                    end
                    
                    local bag_info = Bag_Data:getInstance():get_bag_info_by_id(mail_info.award[j].item_id)
                    if mail_info.award[j].item_id == "item_02" then
                        image_reward:loadTexture("images/item/item_8.png")
                    elseif mail_info.award[j].item_id == "item_03" then
                        image_reward:loadTexture("images/item/item_9.png")
                    else
                        image_reward:loadTexture(bag_info.res_key)
                    end
                    text_num:setString(string.format("x%d", mail_info.award[j].item_num))

                end

                local get_Btn = ButtonWrap:create()
                get_Btn:wrap(button_get_0, 1)
                get_Btn:addTouchEndCallback(function()
                    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                    print(mail_info.mail_id)
                    self:send_readMail(mail_info.mail_id)
                    self.mail_mark_count = self.mail_mark_count - 1
                    cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_mail_mark_count)

                    button_check_image:setVisible(false)
                    button_get_0:setVisible(false)
                    image_get:setVisible(true)

                    for k,v in pairs(mail_info.award) do
                        Player:getInstance():add_player_bag_info_by_id(v.item_id, tonumber(v.item_num))
                    end

                end)

            end

        end

        performWithDelay(self,function() 
            local size = mail_listview:getInnerContainerSize()
            mail_listview:setInnerContainerSize(cc.size(size.width,size.height+250))
        end, 1)
    end
    
    self:request_getMailList(async_callback)
end

function MailLayer:move_items(i, count)

    local is_have = false
    local index = 0
    for j=1,count do
        if self.info_vector[j]:isVisible() then
            is_have = true
            index = j
            break
        end
    end
    if is_have then
        if index == i then
            self.info_vector[i]:setVisible(false)
            self:move_items_behind(i, true, count)
        else
            self.info_vector[index]:setVisible(false)
            self:move_items_behind(index, true, count)

            self.info_vector[i]:setVisible(true)
            self:move_items_behind(i, false, count)
        end
    else
        self.info_vector[i]:setVisible(true)
        self:move_items_behind(i, false, count)
    end

end

function MailLayer:move_items_behind(index, flag, count)
    for i=index+1, count do
        local item = self.mail_listview:getItem(i -1)
        if flag then
            --上移
            item:runAction(cc.MoveTo:create(0.3, cc.p(item:getPositionX(), item:getPositionY()+215)))
        else
            --下移
            item:runAction(cc.MoveTo:create(0.3, cc.p(item:getPositionX(), item:getPositionY()-215)))
        end
    end
end

function MailLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("mail_out", false)

    if self.friend_listview then
        set_list_fade_out(self.friend_listview)
    end
    if self.energy_listview then
        set_list_fade_out(self.energy_listview)
    end
    if self.mail_listview then
        set_list_fade_out(self.mail_listview)
    end
    cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mail_image_mark_event)
end

function MailLayer:request_sendgiftnum_and_gotgiftnum()
    --request 请求已赠送次数、已领取次数
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/account/charinfo", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            -- print("request_sendgiftnum_and_gotgiftnum"..response.sendgiftnum, response.gotgiftnum)
            self.sendgiftnum = response.sendgiftnum
            self.gotgiftnum = response.gotgiftnum
            self:request_MsgList()
        else

            print("request error")

        end

    end)
end

function MailLayer:request_MsgList()
    --request 所有消息列表
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/message/getMsgList", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            if response == nil then
                response = {}
            end

            local index1, index2, index3 = 1, 1, 1
            local reqFriendlist = {}
            local sendEnergylist = {}
            local reqEnergylist = {}
            for k,v in pairs(response) do
                if v.type == "1" then
                    reqFriendlist[index1] = v
                    index1 = index1 + 1
                elseif v.type == "2" then
                    sendEnergylist[index2] = v
                    index2 = index2 + 1
                elseif v.type == "3" then
                    reqEnergylist[index3] = v
                    index3 = index3 + 1
                end
            end
            self.friend_async_callback(reqFriendlist)
            self.energy_async_callback(reqEnergylist, sendEnergylist)
        else

            print("request error")

        end

    end)
end

function MailLayer:request_getMailList(async_callback)
    --request  获取邮件列表
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/message/getMailList", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            async_callback(response)
            
        else

            print("request error")

        end

    end)
end

function MailLayer:send_agreeFriendAdd(msg_id)
    --send  同意好友申请
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        1,

    }

    RPC:call("SHIN/message/handleMsg", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
        else

            print("request error")

        end

    end)
end

function MailLayer:send_ignoreFriendAdd(msg_id)
    --send  忽略好友申请
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        0,

    }

    RPC:call("SHIN/message/handleMsg", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
        else

            print("request error")

        end

    end)
end

function MailLayer:send_agreeSendEnergy(msg_id, text_des_0)
    --send 同意赠送体力 每天限赠送20次
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        1,

    }

    RPC:call_bind("SHIN/message/handleMsg", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
            text_des_0:setVisible(true)
            self.sendgiftnum = response.sendgiftnum
            text_des_0:setString(string.format(ResourceManager:getInstance():getTextStringByKey("MailLayer4"),self.sendgiftnum_limit - self.sendgiftnum))

            -- print("sendgiftnum_diff::"..self.sendgiftnum_limit - self.sendgiftnum)
            -- if (self.sendgiftnum_limit - self.sendgiftnum) <= 0 then
            --     for k,v in pairs(self.energy_agree_Btn_vector) do
            --         if v:isImageVisible() then
            --             v:disable()
            --             v:setDarkButtonWithText(false, "text_agree")
            --         end
            --     end
            -- end
        else

            print("request error")

        end

    end)
end

function MailLayer:send_ignoreSendEnergy(msg_id)
    --send 忽略赠送体力
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        0,

    }

    RPC:call("SHIN/message/handleMsg", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
        else

            print("request error")

        end

    end)
end

function MailLayer:send_getEnergy(msg_id, text_des_0)
    --send 同意领取体力 每天限领取10次
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        1,

    }

    RPC:call_bind("SHIN/message/handleMsg", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            --dump(response)
            --体力+1
            Player:getInstance():add_player_bag_info_by_id("item_101", 1)
            local bag_info = Bag_Data:getInstance():get_bag_info_by_id("item_101")                    
            self.bag_reward:setItemTexture(bag_info.res_key)
            self.bag_reward:setTouchEnabled(true)
            self.bag_reward:runAction()

            text_des_0:setVisible(true)
            self.gotgiftnum = response.gotgiftnum
            text_des_0:setString(string.format(ResourceManager:getInstance():getTextStringByKey("MailLayer5"), self.gotgiftnum_limit - self.gotgiftnum))

            -- print("gotgiftnum_diff::"..self.gotgiftnum_limit - self.gotgiftnum)
            -- if (self.gotgiftnum_limit - self.gotgiftnum) <= 0 then
            --     for k,v in pairs(self.energy_get_Btn_vector) do
            --         if v:isImageVisible() then
            --             v:disable()
            --             v:setDarkButtonWithText(false, "text_get")
            --         end
            --     end
            -- end
        else

            print("request error")

        end

    end)
end

function MailLayer:send_ignoreGetEnergy(msg_id)
    --send 忽略领取体力
    local rpc_param = {

        PLAYER_ID,
        msg_id,
        0,

    }

    RPC:call("SHIN/message/handleMsg", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
        else

            print("request error")

        end

    end)
end

function MailLayer:send_readMail(mail_id)
    --send 阅读邮件
    local rpc_param = {

        PLAYER_ID,
        mail_id,

    }

    RPC:call_bind("SHIN/message/readMail", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then


        else

            print("request error")

        end

    end)
end

function MailLayer:test_mail_data()
    local list = {}
    for i=1,10 do
        list[i] = {
            ["mail_id"] = string.format("id%d", i),
            ["mail_type"] = i%2,
            ["title"] = string.format("周赛第%d名奖励", i),
            ["content"] = string.format("这是周赛第%d名奖励", i),
            ["reward"] = {
                [1] = {
                    ["item_id"] = "item_03",
                    ["item_num"] = 45,
                },
                [2] = {
                    ["item_id"] = "item_303",
                    ["item_num"] = 1,
                },
            }
        }
    end
    return list
end

-- function MailLayer:test_sendFriendsList_data()
--     local list = {}
--     for i=1,3 do
--         list[i] = {
--             ["type"] = 2,
--             ["senderinfo"] = {
--                     ["name"] = string.format("名字%d", i),
--                     ["sex"] = i,
--                 } ,
--         }
--     end
--     return list
-- end

function MailLayer:onEnter()
    RPC:register_target(self)
end

function MailLayer:onExit()
    RPC:unregister_target(self)
end

return MailLayer