require "core.LayerManager"
require "core.Mission_Data"
require "core.Section_Data"
require "UI.Widget.MaskLayer"
require "UI.Layer.FirstGiftLayer"
require "UI.Layer.GetNewSystemLayer"
require "core.Head_Data"
require "UI.Layer.LuckyGiftLayer"
require "core.LuckyGift_Data"

local LevelBuilder = require "GameEx.Foundations.LevelBuilder"

MapLayer = class("MapLayer", function() return cc.Layer:create() end)

KMapTag = 102
function MapLayer:ctor(mission_section)


end

function MapLayer:init_map(mission_section, new_section)

    self:setName("MapLayer")

    self.is_add_level_info_layer = false

    cc.UserDefault:getInstance():setBoolForKey("is_section_layer", false)
    self:setTag(KMapTag)
    self.image_role = nil
    local visible_size = cc.Director:getInstance():getVisibleSize()
    local section_info = Section_Data:getInstance():get_section_info_by_id(mission_section)
    self.section_info = section_info
    self.map = cc.CSLoader:createNode(section_info.map_csb)
    self:addChild(self.map)

    self.map:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(self.map)

    local node = self.map:getChildByName("node")
    local panel_node = node:getChildByName(string.format("panel_%d", mission_section))
    local map_size = panel_node:getContentSize()
    panel_node:setContentSize(cc.size(get_node_width(panel_node), visible_size.height))
    panel_node:setInnerContainerSize(cc.size(map_size.width, map_size.height))
    panel_node:jumpToBottom()
    panel_node:setScrollBarEnabled(false)
    
    self.panel_node = panel_node

    local panel_stage = panel_node:getChildByName("panel_stage")
    panel_stage:setLocalZOrder(1)
    self.panel_stage = panel_stage

    local player = Player:getInstance()
    player:set_player_current_section(mission_section)
    local mission_count = Mission_Data:getInstance():get_section_mission_count(mission_section)
    self.mission_map = {}

    -- 当前最新关卡角色动画
    local stage_dobber = cc.CSLoader:createNode("stage_dobber.csb")
    stage_dobber:setLocalZOrder(3)
    panel_stage:addChild(stage_dobber)
    stage_dobber:setVisible(false)

    local node = stage_dobber:getChildByName("node")
    local image_dobber = node:getChildByName("image_dobber")
    self.image_role = image_dobber:getChildByName("image_role")
    -- local role_info = Role_Data:getInstance():get_role_info_by_id(Player:getInstance():get_player_cur_role_id())
    -- self.image_role:loadTexture(role_info.res_key)
    
    local player_head_id = player:get_player_head_id()
    if player_head_id == "head_00" then
        local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
        self.image_role:loadTexture(headPath)
    else
        local head_info = Head_Data:getInstance():get_head_info_by_id(player_head_id)
        self.image_role:loadTexture(head_info.res_key)
    end

    local action = cc.CSLoader:createTimeline("stage_dobber.csb")
    action:play("stage_dobber", true)
    stage_dobber:runAction(action)

    -- 获得最新的最高通关的关卡号
    local newest_mission_id = Player:getInstance():get_player_newest_mission_id()
    local newest_mission_info = Mission_Data:getInstance():get_mission_info_by_id(newest_mission_id)
    local newest_level = newest_mission_info.mission_level

    -- 关卡内容
    local move_pos = cc.p(0,0)
    local is_head_move = false
    for i=1, mission_count do 
        local mission_level = Mission_Data:getInstance():get_mission_level(i, mission_section)

        local tag_image = panel_stage:getChildByName(string.format("button_%d", i))
        tag_image:setOpacity(0)

        local level_config = LevelBuilder:get_instance():get_level_config(mission_level)

        local is_boss = false
        if level_config.boss then
            is_boss = true
        end

        local level_image = nil
        if is_boss == false then
            level_image = ccui.ImageView:create(section_info.button_stage_res)
        else
            level_image = ccui.ImageView:create("images/stage_map/button_boss.png")
        end

        level_image:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()))
        tag_image:getParent():addChild(level_image, 1)

        self.mission_map[mission_level] = tag_image
        local text = tag_image:getChildByName("text_1")

        local new_text = ccui.Text:create()
        new_text:setString(string.format("%d", mission_level))
        new_text:setLocalZOrder(2)
        new_text:setPosition(cc.p(get_node_width(level_image)/2, get_node_height(level_image) * section_info.height_rate))
        new_text:enableShadow(cc.c4b(123, 52, 88, 255), cc.size(0, -3), 0)
        new_text:setFontSize(55)
        new_text:setFontName("images/font/zh_CN.TTF")
        level_image:addChild(new_text)

        if is_boss == false then
        else
            new_text:setVisible(false)
        end

        local button = ButtonWrap:create()
        button:wrap(level_image, 1)
        button:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
            self.panel_node:stopAllActions()

            local function show()

                player:set_player_current_level(mission_level)
                local levelInfoLayer = LevelInfoLayer:create()
                LayerManager:getInstance():add_layer_to_scene(levelInfoLayer)

            end

            local param = { constraint = mission_level, show = true, func = show }

            local event = cc.EventCustom:new("LEVEL_START->DIALOG")
            event.param = param
            cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

            if param.show then

                show()

            end


        end)

        local mission_info = Mission_Data:getInstance():get_mission_info_by_level(mission_level)
        local player_mission_info = player:get_player_mission_info_by_id(mission_info.mission_id)
        local is_move_action = false
        if player_mission_info and player_mission_info.is_enable then
            if player_mission_info.is_pass == false then
                level_image:setOpacity(0)
                -- 判断是否是boss关卡
                if is_boss == false then
                    local stage_button_animation = cc.CSLoader:createNode("stage_button_animation.csb")
                    stage_button_animation:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()))
                    stage_button_animation:setLocalZOrder(2)
                    stage_button_animation:setTag(1002)
                    tag_image:getParent():addChild(stage_button_animation)

                    local node = stage_button_animation:getChildByName("node")
                    local image_stage_button = node:getChildByName("image_stage_button")
                    image_stage_button:loadTexture(self.section_info.button_stage_res)

                    local text_1 = image_stage_button:getChildByName("text_1")
                    text_1:setLocalZOrder(1)
                    text_1:setString(string.format("%d", mission_level))

                    local button_action = cc.CSLoader:createTimeline("stage_button_animation.csb")
                    button_action:play("stage_button_ani", true)
                    stage_button_animation:runAction(button_action)
                else
                    local stage_button_boss = cc.CSLoader:createNode("stage_button_boss.csb")
                    stage_button_boss:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()))
                    stage_button_boss:setLocalZOrder(2)
                    stage_button_boss:setTag(1002)
                    tag_image:getParent():addChild(stage_button_boss)

                    local boss_action = cc.CSLoader:createTimeline("stage_button_boss.csb")
                    boss_action:play("stage_button_ani", true)
                    stage_button_boss:runAction(boss_action)
                end

                move_pos = cc.p(tag_image:getPositionX(), tag_image:getPositionY())
                stage_dobber:setVisible(true)
                if player_mission_info.is_move_action then
                    is_move_action  = true
                    -- 延时处理
                    tag_image:getParent():getChildByTag(1002):setVisible(false)
                    -- self:runAction(cc.Sequence:create(cc.DelayTime:create(0.05), cc.CallFunc:create(function()
                    --     player:set_player_mission_move_action(player_mission_info.mission_id, false)
                    -- end)))
                else
                    stage_dobber:setPosition(move_pos)
                end
            end
        else
            
            button:setDarkButton(false)
            button:disable()
        end

        if move_pos.x == 0 or i == 1 then
            stage_dobber:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()))
            if tag_image:getParent():getChildByTag(1002) then
                tag_image:getParent():getChildByTag(1002):setVisible(true)
            end
        end

        if is_move_action or new_section then
            is_head_move = true
            self:scroll_to_level(mission_level)

            button:setDarkButton(false)
            stage_dobber:runAction(cc.Sequence:create(cc.DelayTime:create(0), cc.CallFunc:create(function()

                tag_image:getParent():getChildByTag(1002):setVisible(true)
                button:setDarkButton(true)

                local function move()
                    stage_dobber:runAction(cc.Sequence:create(cc.DelayTime:create(0.1), cc.MoveTo:create(1.5, move_pos), 
                        cc.CallFunc:create(function()
                            player:set_player_mission_move_action(player_mission_info.mission_id, false)
                            if self.is_add_level_info_layer == false then
                                if mission_info.new_system then
                                    -- self.panel_stage:getChildByTag(mission_info.mission_level):setVisible(false)                                

                                    local getNewSystemLayer = GetNewSystemLayer:create()
                                    getNewSystemLayer:set_cur_system(mission_info.new_system)
                                    
                                    getNewSystemLayer:set_close_callback(function()
                                        if mission_info.new_system == "sign" then
                                            local maskLayer = MaskLayer:create()
                                            self:addChild(maskLayer, 5)

                                            local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                                            local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                                            local point = cc.p(-visible_size.width/2,-550)
                                            image_icon:runAction(cc.MoveTo:create(1.5,point))
                                            self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                                                icon_dobber:setVisible(false)
                                                maskLayer:removeFromParent()
                                                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_sign_system)

                                                local signBoardLayer = SignBoardLayer:create()
                                                LayerManager:getInstance():add_layer_to_scene(signBoardLayer)
                                            end)))
                                        elseif mission_info.new_system == "task" then
                                            local maskLayer = MaskLayer:create()
                                            self:addChild(maskLayer, 5)

                                            local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                                            local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                                            local point = cc.p(visible_size.width*0.2,350)
                                            image_icon:runAction(cc.MoveTo:create(1,point))
                                            self:runAction(cc.Sequence:create(cc.DelayTime:create(1), cc.CallFunc:create(function()
                                                icon_dobber:setVisible(false)
                                                maskLayer:removeFromParent()
                                                local taskManager = TaskManager:getInstance()
                                                taskManager:update_date()
                                                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_task_system)
                                                self:create_luckyGiftLayer()
                                            end)))
                                        elseif mission_info.new_system == "weekgame" then
                                            local maskLayer = MaskLayer:create()
                                            self:addChild(maskLayer, 5)

                                            local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                                            local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                                            local point = cc.p(visible_size.width*0.4,350)
                                            image_icon:runAction(cc.MoveTo:create(1.5,point))
                                            self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                                                icon_dobber:setVisible(false)
                                                maskLayer:removeFromParent()
                                                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_weekgame_system)
                                                self:create_luckyGiftLayer()
                                            end)))
                                        elseif mission_info.new_system == "invite_reward" then
                                            if LOGIN_PLATFROM ~= 0 then
                                                local maskLayer = MaskLayer:create()
                                                self:addChild(maskLayer, 5)

                                                local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                                                local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                                                local point = cc.p(visible_size.width*0.4,200)
                                                image_icon:runAction(cc.MoveTo:create(1.5,point))
                                                self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                                                    icon_dobber:setVisible(false)
                                                    maskLayer:removeFromParent()
                                                    cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_invite_reward_system)
                                                    self:create_luckyGiftLayer()
                                                end)))
                                            end
                                        else
                                            local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                                            icon_dobber:setVisible(false)
                                            self:create_luckyGiftLayer()
                                        end
                                    end)
                                    LayerManager:getInstance():add_layer_to_scene(getNewSystemLayer)
                                else
                                    self:create_luckyGiftLayer()
                                end                                
                            end
                            
                        end))
                    )

                    if self.show_level_info then

                        self:add_level_info_layer(self.show_section, self.show_level)

                    end

                end

                self.show_level = self.show_level or 1
                local param = { constraint = self.show_level - 1, show = true, func = move }

                local event = cc.EventCustom:new("LEVEL_FINISH->DIALOG")
                event.param = param
                cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

                if param.show then

                    move()

                end

            end)))

        end

        -- 8.23 解锁的系统
        if mission_info.new_system and newest_level <= mission_info.mission_level and player_mission_info.is_move_action then
            local icon_dobber = cc.CSLoader:createNode("icon_dobber.csb")
            icon_dobber:setLocalZOrder(2)
            icon_dobber:setTag(mission_info.mission_level)
            panel_stage:addChild(icon_dobber)
            icon_dobber:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()))

            local action = cc.CSLoader:createTimeline("icon_dobber.csb")
            icon_dobber:runAction(action)

            local node = icon_dobber:getChildByName("node")
            local image_dobber = node:getChildByName("image_dobber")
            local image_icon = image_dobber:getChildByName("image_icon")

            local text = ""
            if mission_info.new_system == "sign" then
                action:play("icon_dobber", true)
                local sign_in_pic = ccui.ImageView:create("images/section/sign_in.png")
                image_icon:addChild(sign_in_pic)

                text = ResourceManager:getInstance():getTextStringByKey("open_sign_tip")
            elseif mission_info.new_system == "task" then
                action:play("icon_dobber", true)
                local task_pic = ccui.ImageView:create("images/section/mission.png")
                image_icon:addChild(task_pic)

                text = ResourceManager:getInstance():getTextStringByKey("open_task_tip")
            elseif mission_info.new_system == "weekgame" then
                action:play("icon_dobber", true)
                local weekgame_pic = ccui.ImageView:create("images/section/weekgame.png")
                image_icon:addChild(weekgame_pic)

                text = ResourceManager:getInstance():getTextStringByKey("open_weekgame_tip")
            elseif mission_info.new_system == "invite_reward" then
                if LOGIN_PLATFROM ~= 0 then
                    action:play("icon_dobber", true)
                    local invite_pic = ccui.ImageView:create("images/section/invite_reward.png")
                    image_icon:addChild(invite_pic)

                    text = ResourceManager:getInstance():getTextStringByKey("open_invite_reward_tip")
                else
                    icon_dobber:removeFromParent()
                end
            else
                action:play("gift_star", true)
                local gift_info = Gift_Data:getInstance():get_gift_info_by_id(mission_info.new_system)
                local bag_info = Bag_Data:getInstance():get_bag_info_by_id(gift_info.item_id)
                local gift_pic = ccui.ImageView:create()

                if gift_info.item_id == "item_02" then
                    gift_pic:loadTexture("images/item/item_8.png")
                elseif gift_info.item_id == "item_03" then
                    gift_pic:loadTexture("images/item/item_9.png")
                else
                    gift_pic:loadTexture(bag_info.res_key)
                end                
                image_icon:addChild(gift_pic)
                
                local gift_info = Gift_Data:getInstance():get_gift_info_by_id(mission_info.new_system)
                text = gift_info.click_text
            end

            local dobber_Btn = ButtonWrap:create()
            dobber_Btn:wrap(image_dobber, 1)
            dobber_Btn:addTouchEndCallback(function()
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                
                local warningLayer = WarningLayer:create()
                warningLayer:set_text_string(text)
                LayerManager:getInstance():add_layer_to_scene(warningLayer)
            end)
        end
        --8.23

        -- 显示的星星数
        if player_mission_info.is_pass then
            local star_count = player_mission_info.star_count
            local star_spine = ResourceManager:getInstance():getSpineByKey("SPINE_STAGE_STAR", 1.0)
            star_spine:setPosition(cc.p(get_node_width(level_image)/2, get_node_height(level_image)/2))
            level_image:addChild(star_spine)
            if player_mission_info.is_max_star then
                -- 延时处理
                self:runAction(cc.Sequence:create(cc.DelayTime:create(0.05), cc.CallFunc:create(function()
                    Player:getInstance():set_player_mission_is_max_star(player_mission_info.mission_id, false)
                end)))
                if star_count == 1 then
                    star_spine:setAnimation(0, "star_1_get", false)
                elseif star_count == 2 then
                    star_spine:setAnimation(0, "star_1_get", false)
                    star_spine:addAnimation(0, "star_2_get", false)
                elseif star_count == 3 then
                    star_spine:setAnimation(0, "star_1_get", false)
                    star_spine:addAnimation(0, "star_2_get", false)
                    star_spine:addAnimation(0, "star_3_get", false)
                end
            else
                if star_count == 1 then
                    star_spine:setAnimation(0, "star_1", false)
                elseif star_count == 2 then
                    star_spine:setAnimation(0, "star_2", false)
                elseif star_count == 3 then
                    star_spine:setAnimation(0, "star_3", false)
                elseif star_count == 0 then
                    star_spine:setAnimation(0, "star_no", false)
                end
            end
        end

    end

    --好友头像
    local function async_callback(list)
        local friendList = list
        for i=1,#friendList do
            if friendList[i].guid == PLAYER_ID then
                -- friendList[i] = nil
                friendList[i].is_delete = true
            end
            if i>1 and friendList[i].maxChapter == friendList[i-1].maxChapter then
                friendList[i].is_delete = true
            end
        end
        for i=1,#friendList do
            if friendList[i].is_delete then
                friendList[i] = nil
            end
        end
        local temp_table = {}
        for k,v in pairs(friendList) do
            table.insert(temp_table, v)
        end
        friendList = temp_table

        -- for k,v in pairs(friendList) do
        --     print(k,v.name)
        -- end

        local count = get_table_length(friendList)
        self.is_friend_dobber_play_vector = {}
        for i=1,count do
            for j=1, mission_count do 
                local tag_image = panel_stage:getChildByName(string.format("button_%d", j))
                local mission_level = Mission_Data:getInstance():get_mission_level(j, mission_section)
                local mission_info = Mission_Data:getInstance():get_mission_info_by_level(mission_level)
                local player_mission_info = player:get_player_mission_info_by_id(mission_info.mission_id)
                
                local newest_chapter = friendList[i].maxChapter + 1
                if newest_chapter == mission_level and newest_chapter ~= newest_level and not 
                (mission_info.new_system and newest_level <= mission_info.mission_level and player_mission_info.is_move_action) then
                    
                    local stage_friend_dobber = cc.CSLoader:createNode("stage_friend_dobber.csb")
                    stage_friend_dobber:setLocalZOrder(2)
                    panel_stage:addChild(stage_friend_dobber)
                    stage_friend_dobber:setPosition(cc.p(tag_image:getPositionX(), tag_image:getPositionY()+45))

                    local action = cc.CSLoader:createTimeline("stage_friend_dobber.csb")
                    stage_friend_dobber:runAction(action)
                    self.is_friend_dobber_play_vector[i] = false

                    local node = stage_friend_dobber:getChildByName("node")
                    local right = node:getChildByName("right")
                    local right_text_name = right:getChildByName("text_name")
                    local right_text_star_num = right:getChildByName("text_star_num")
                    local left = node:getChildByName("left")
                    local left_text_name = left:getChildByName("text_name")
                    local left_text_star_num = left:getChildByName("text_star_num")
                    local image_head = node:getChildByName("image_id")
                    local image_dobber = node:getChildByName("image_dobber")

                    right_text_name:setString(friendList[i].name)
                    right_text_star_num:setString(friendList[i].totalStar)
                    left_text_name:setString(friendList[i].name)
                    left_text_star_num:setString(friendList[i].totalStar)

                    if friendList[i].sex == "0" and friendList[i].headUrl and friendList[i].headUrl ~= "" then
                
                        local function on_create_finish()
                            local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friendList[i].guid.."head_00"
                            if cc.FileUtils:getInstance():isFileExist(headPath) then
                                image_head:loadTexture(headPath)
                            end
                        end
                        global_create_social_head(friendList[i].guid, friendList[i].headUrl, on_create_finish)

                    else
                        local head_id = string.format("head_%02d", friendList[i].sex+1)
                        local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                        image_head:loadTexture(head_info.res_key)
                    end

                    local dobber_Btn = ButtonWrap:create()
                    dobber_Btn:wrap(image_dobber, 1)
                    dobber_Btn:addTouchEndCallback(function()
                        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                        
                        local posX = stage_friend_dobber:getPositionX()
                        if self.is_friend_dobber_play_vector[i] then
                            self.is_friend_dobber_play_vector[i] = false
                            if posX >= 720 then
                                action:play("left_out", false)
                            else
                                action:play("right_out", false)
                            end
                        else
                            self.is_friend_dobber_play_vector[i] = true
                            if posX >= 720 then
                                action:play("left_in", false)
                            else
                                action:play("right_in", false)
                            end
                        end
                        
                    end)
                end
            end
        end
    end
    self:request_getFriendlist(async_callback)



    performWithDelay(self,function() 
        if is_head_move == false and self.is_add_level_info_layer == false then
            self:create_luckyGiftLayer()
        end
    end,0.05)

    local particle_map = {
        [1] = "PARTICLE_YINGHUA",
        [2] = "PARTICLE_SNOW",
        [3] = "PARTICLE_LEAF",
        [4] = "PARTICLE_RED",
        [5] = "PARTICLE_SMOKE_1",
        [6] = "PARTICLE_SUNNY",
        [7] = "PARTICLE_SAVA",
    }

    local map_particle = ResourceManager:getInstance():getParticleByKey(particle_map[mission_section])
    map_particle:setAutoRemoveOnFinish(false)
    if mission_section == 5 then
        map_particle:setPosition(cc.p(-50, visible_size.height/2))
    elseif mission_section == 6 then
        map_particle:setPosition(cc.p(visible_size.width/2, visible_size.height/2))
    elseif mission_section == 7 then
        map_particle:setPosition(cc.p(visible_size.width/2, visible_size.height+100))
    else
        map_particle:setPosition(cc.p(visible_size.width/2, visible_size.height))
    end

    if mission_section == 2 then
        local snowflake = ResourceManager:getInstance():getParticleByKey("PARTICLE_SNOWFLAKE")
        snowflake:setPosition(cc.p(visible_size.width/2, visible_size.height))
        self.map:addChild(snowflake, 2)
    end

    if mission_section == 3 then
        panel_node:getChildByName("panel_map"):getChildByName("image_3"):setLocalZOrder(3)
    end

    self.map:addChild(map_particle, 2)

    local map_effect = GameKernel:getInstance():get_map_effect_config()
    local particle_map = map_effect[mission_section]["particle"]
    local spine_map = map_effect[mission_section]["spine"]

    for k,v in pairs(particle_map) do
        local particle = ResourceManager:getInstance():getParticleByKey(k)
        particle:setAutoRemoveOnFinish(false)
        particle:setPositionType(cc.POSITION_TYPE_GROUPED)
        particle:setPosition(cc.p(v.x, v.y))
        panel_node:addChild(particle, 0)
    end

    for k,v in pairs(spine_map) do
        local spine = ResourceManager:getInstance():getSpineByKey(k, 1.0)
        spine:setAnimation(0, v.name, true)
        spine:setPosition(cc.p(v.x, v.y))
        panel_node:getChildByName("panel_map"):addChild(spine)
    end

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

end

function MapLayer:request_getFriendlist(async_callback)
    --request getFriendlist 当前好友列表
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/friend/getFriendlist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function MapLayer:show_finish_dialog(mission_level, finish_func)

    local unlock = function() 

        finish_func()
                
    end

    local param = { constraint = mission_level, show = true, func = unlock }

    local event = cc.EventCustom:new("LEVEL_FINISH->DIALOG")
    event.param = param
    cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

    if param.show then

        unlock()

    end

end

function MapLayer:add_level_info_layer(mission_section, mission_level)
    print("add_level_info")
    local visible_size = cc.Director:getInstance():getVisibleSize()
    Player:getInstance():set_player_current_section(mission_section)
    Player:getInstance():set_player_current_level(mission_level)

    local mission_info = Mission_Data:getInstance():get_mission_info_by_level(mission_level)
    local player_mission_info = Player:getInstance():get_player_mission_info_by_id(mission_info.mission_id)

    local player_mission_info_last = nil
    if mission_level > 1 then    
        local mission_info_last = Mission_Data:getInstance():get_mission_info_by_level(mission_level-1)
        player_mission_info_last = Player:getInstance():get_player_mission_info_by_id(mission_info_last.mission_id)
    end

    local delay_time = 0.3


    local is_move_action = player_mission_info.is_move_action
    self.is_add_level_info_layer = true
    delay_time = delay_time + 1.6

    Player:getInstance():set_player_mission_move_action(player_mission_info.mission_id, false)

    if not self.show_level_info then
        delay_time = 0.3
        if mission_level > 1 then   
            if player_mission_info_last.is_max_star then    --获取上一关是否得到新的星星
                delay_time = delay_time + 0.6
            end
        end
    end


    self:getParent():add_mask_layer()
    self:runAction(cc.Sequence:create(cc.DelayTime:create(delay_time), cc.CallFunc:create(function()
        self:getParent():remove_mask_layer()
        local function show()

            if mission_info.new_system and is_move_action then
                self:getParent():remove_mask_layer()
                local getNewSystemLayer = GetNewSystemLayer:create()
                getNewSystemLayer:set_cur_system(mission_info.new_system)
                
                getNewSystemLayer:set_close_callback(function()
                    if mission_info.new_system == "sign" then
                        local maskLayer = MaskLayer:create()
                        self:addChild(maskLayer, 5)

                        local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                        local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                        local point = cc.p(-visible_size.width/2,-550)
                        image_icon:runAction(cc.MoveTo:create(1.5,point))
                        self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                            icon_dobber:setVisible(false)
                            maskLayer:removeFromParent()                            
                            cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_sign_system)

                            local signBoardLayer = SignBoardLayer:create()
                            signBoardLayer:set_close_callback(function() self:append_level_info_layer() end)
                            LayerManager:getInstance():add_layer_to_scene(signBoardLayer)                            
                        end)))
                    elseif mission_info.new_system == "task" then
                        local maskLayer = MaskLayer:create()
                        self:addChild(maskLayer, 5)

                        local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                        local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                        local point = cc.p(visible_size.width*0.2,350)
                        image_icon:runAction(cc.MoveTo:create(1,point))
                        self:runAction(cc.Sequence:create(cc.DelayTime:create(1), cc.CallFunc:create(function()
                            icon_dobber:setVisible(false)
                            maskLayer:removeFromParent()
                            local taskManager = TaskManager:getInstance()
                            taskManager:update_date()
                            cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_task_system)
                            self:create_luckyGiftLayer(function() self:append_level_info_layer() end)
                        end)))
                    elseif mission_info.new_system == "weekgame" then
                        local maskLayer = MaskLayer:create()
                        self:addChild(maskLayer, 5)

                        local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                        local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                        local point = cc.p(visible_size.width*0.4,350)
                        image_icon:runAction(cc.MoveTo:create(1.5,point))
                        self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                            icon_dobber:setVisible(false)
                            maskLayer:removeFromParent()
                            cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_weekgame_system)
                            self:create_luckyGiftLayer(function() self:append_level_info_layer() end)
                        end)))
                    elseif mission_info.new_system == "invite_reward" then
                        if LOGIN_PLATFROM ~= 0 then
                            local maskLayer = MaskLayer:create()
                            self:addChild(maskLayer, 5)

                            local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                            local image_icon = icon_dobber:getChildByName("node"):getChildByName("image_dobber"):getChildByName("image_icon")
                            local point = cc.p(visible_size.width*0.4,200)
                            image_icon:runAction(cc.MoveTo:create(1.5,point))
                            self:runAction(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function()
                                icon_dobber:setVisible(false)
                                maskLayer:removeFromParent()
                                cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.open_invite_reward_system)
                                self:create_luckyGiftLayer(function() self:append_level_info_layer() end)
                            end)))
                        end
                    else
                        local icon_dobber = self.panel_stage:getChildByTag(mission_info.mission_level)
                        icon_dobber:setVisible(false)
                        self:create_luckyGiftLayer(function() self:append_level_info_layer() end)
                    end
                end)
                LayerManager:getInstance():add_layer_to_scene(getNewSystemLayer)
            else
                self:create_luckyGiftLayer(function() self:append_level_info_layer() end)
                
            end 

        end

        local param = { constraint = mission_level, show = true, func = show }

        local event = cc.EventCustom:new("LEVEL_START->DIALOG")
        event.param = param
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

        if param.show then

            show()

        end       

    end)))

end

function MapLayer:append_level_info_layer()
    self:getParent():remove_mask_layer()
    local levelInfoLayer = LevelInfoLayer:create()
    LayerManager:getInstance():add_layer_to_scene(levelInfoLayer)

end

function MapLayer:create_luckyGiftLayer(callback)
    local player = Player:getInstance()
    
    local current_level = player:get_player_current_level()
    local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
    local player_mission_info = player:get_player_mission_info_by_id(mission_info.mission_id)
    local is_last_level = Mission_Data:getInstance():is_last_level(current_level)
    -- if is_last_level and player_mission_info.is_pass and player_mission_info.is_last_level_get == false then
    --     player:set_player_mission_is_last_level_get(mission_info.mission_id, true)
    --     performWithDelay(self,function()
    --         --率先通关
    --         local shareStageLayer = ShareStageLayer:create(2)
    --         LayerManager:getInstance():add_layer_to_scene(shareStageLayer)
    --     end, 0.12)
    -- elseif player:get_continue_pass_stage() >= 10 then
    --     player:set_continue_pass_stage(0)
    --     performWithDelay(self,function()
    --         --连续通关
    --         local shareStageLayer = ShareStageLayer:create(1)
    --         LayerManager:getInstance():add_layer_to_scene(shareStageLayer)
    --     end,0.12)
    -- end

    if GameKernel:getInstance():get_gift_visible() then
        
        if player:get_is_create_lucky_gift() and player:get_num_luckygift_buy() > 0 then
            player:set_is_create_lucky_gift(false)
            performWithDelay(self,function() 
                local luckyGiftLayer = LuckyGiftLayer:create()
                if callback then
                    luckyGiftLayer:set_close_callback(callback)
                end
                -- LayerManager:getInstance():add_layer_to_scene(luckyGiftLayer)
            end,0.1)
        elseif callback then
            callback()
        end
    elseif callback then
        callback()
    end
end

function MapLayer:scroll_to_level(mission_level)

    if self.mission_map[mission_level] then
        local pos_y = self.mission_map[mission_level]:getPositionY()
        local height = self.panel_node:getContentSize().height
        local container_height = self.panel_node:getInnerContainerSize().height
        local percent = ((container_height - pos_y) - height/2)/(container_height - height)
        self.panel_node:jumpToPercentVertical(math.min(math.max(0, percent * 100), 100))
    else
        self.panel_node:jumpToPercentVertical(0)
    end
end

function MapLayer:move_to_newest_level()
    self.panel_node:jumpToPercentVertical(100)
    local mission_id = Player:getInstance():get_player_newest_mission_id()
    local mission_info = Mission_Data:getInstance():get_mission_info_by_id(mission_id)
    local mission_level  = mission_info.mission_level
    local move_time = 0.0
    if self.mission_map[mission_level] then
        local pos_y = self.mission_map[mission_level]:getPositionY()
        local height = self.panel_node:getContentSize().height
        local container_height =  self.panel_node:getInnerContainerSize().height
        local percent = ((container_height - pos_y) - height/2)/(container_height - height)
        if percent > 0 then
            -- self.panel_node:scrollToPercentVertical(math.min(math.max(0, percent * 100), 100), 3.0 * math.pow(1.0 - percent/100.0, 0.5), true)
            move_time = 2.0 * (1.0 - percent/100.0)
        else
            -- self.panel_node:scrollToPercentVertical(0, 3.0, true)
        end
        self.panel_node:scrollToPercentVertical(math.min(math.max(0, percent * 100), 100), move_time, true)
    else
        self.panel_node:jumpToPercentVertical(0)
    end

    self.panel_node:setTouchEnabled(false)
    self:runAction(cc.Sequence:create(cc.DelayTime:create(move_time), cc.CallFunc:create(function()
        self.panel_node:setTouchEnabled(true)

    end)))
end

function MapLayer:onEnter()
    --8.9
    -- local function update_role(event) 
    --     if self.image_role then
    --         local role_info = Role_Data:getInstance():get_role_info_by_id(Player:getInstance():get_player_cur_role_id())
    --         self.image_role:loadTexture(role_info.res_key)
    --     end
    -- end

    -- local listener1 = cc.EventListenerCustom:create(NotificationUtil.EventType.player_skin_change, update_role)
    -- local listener2 = cc.EventListenerCustom:create(NotificationUtil.EventType.player_role_change, update_role)

    -- local eventDispatcher = cc.Director:getInstance():getEventDispatcher()
    -- eventDispatcher:addEventListenerWithSceneGraphPriority(listener1, self)
    -- eventDispatcher:addEventListenerWithSceneGraphPriority(listener2, self)

    local callFunc = cc.CallFunc:create(function()

        local player = Player:getInstance()
        local mission_info = player:get_player_mission_info_vector()

        local index = 0

        for i = 1, #mission_info do

            if mission_info[i].is_enable and not mission_info[i].is_pass and i > index then

                index = i

            end

        end

        local event = cc.EventCustom:new("enter_map")
        event.param = {
            ["current_level"] = index,
        }   
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

        print("enter_map") print(index)
    end)

    local sequence = cc.Sequence:create(cc.DelayTime:create(0), callFunc)
    self:runAction(sequence)

    RPC:register_target(self)
end

function MapLayer:onExit()
    RPC:unregister_target(self)
end

return MapLayer