require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "core.TalkingDataManager"

ReliveLayer = class("ReliveLayer", function() return cc.Layer:create() end)
ReliveLayer.Relive_Type = {
    Normal = 1,
    Boom = 2,
    Tournament = 3,
}

function ReliveLayer:ctor(relive_type)

    self:setName("ReliveLayer")

    self.relive_type = relive_type
    self.is_boss = false
	self.widget = cc.CSLoader:createNode("relive.csb")
    self:addChild(self.widget, 1);

    self.widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(self.widget)

    self.animation = cc.CSLoader:createTimeline("relive.csb")
    self.widget:runAction(self.animation)

    self.animation:setCurrentFrame(0)
    self.animation:play("continue_in", false)

    self.cost = 0
    self.currency = 0

    self.relive_callback = nil
    self.die_callback = nil
    self.index = 9
    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "continue_in_end" then
            if self.relive_type == ReliveLayer.Relive_Type.Normal then
                self.animation:play("time", false)
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Count_Audio"), false)
            end
        elseif str == "go_on_end" then
            if self.relive_callback then
                self.relive_callback()
            end
            self:removeFromParent()
        elseif str == "time_end" then
            if self.index == 0 then
                MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Timeup_Audio"), false)
                self.animation:play("give_up", false)
                return
            end
            self.index = self.index - 1
            self.text_countdown:setString(string.format("%d", self.index))
            self.animation:play("time", false)
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Count_Audio"), false)
        elseif str == "give_up_end" then
            if self.die_callback then
                self.die_callback()
                print("give_up_end")
            end
            self:removeFromParent()
        end
    end
    self.animation:setFrameEventCallFunc(onFrameEvent)

    local node = self.widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")

    local image_price = image_dia:getChildByName("image_price")
    local image_jewel = image_price:getChildByName("image_jewel")
    self.image_jewel = image_jewel
    -- image_jewel:setTexture("images/shop/rmb.png")
    self.text_price = image_price:getChildByName("text_price")


    local continueImage = image_dia:getChildByName("button_continue")
    local continueBtn = ButtonWrap:create()
    continueBtn:wrap(continueImage, 1)
    continueBtn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
       
        local player = Player:getInstance()
        if self.currency == "jewel" then
            if player:get_player_jewel() >= self.cost then
                self:add_player_currency(self.currency, -self.cost)

                self:buy_relive_success()
            else
                self.animation:pause()
                
                local shopLayer = ShopLayer:create()
                shopLayer:show_goods_by_type(ShopLayer.SHOP_TYPE.JEWEL)
                LayerManager:getInstance():add_layer_to_scene(shopLayer)
                shopLayer:set_close_callback(function()
                    self.animation:resume()
                end)

                local warningLayer = WarningLayer:create()
                warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("needJewel"))
                LayerManager:getInstance():add_layer_to_scene(warningLayer)
            end
        elseif self.currency == "rmb" then
            self.animation:pause()
            TradeManager:getInstance():pay_buy_currency("复活", self.cost,
                function()
                    self:buy_relive_success()
                    self.animation:resume()
                end,
                function()
                    print("asd")
                    self.animation:resume()
                end)
        end
        
    end)

    local panel_relive = image_dia:getChildByName("panel_relive")
    self.text_countdown = panel_relive:getChildByName("text_countdown")
    self.text_countdown:setString(string.format("%d", self.index))

    local panel_boom_relive = image_dia:getChildByName("panel_boom_relive")
    self.image_barrier = panel_boom_relive:getChildByName("image_barrier_108")
    self:set_current_boom()

    if self.relive_type == ReliveLayer.Relive_Type.Normal or self.relive_type == ReliveLayer.Relive_Type.Tournament then
        panel_relive:setVisible(true)
        panel_boom_relive:setVisible(false)
    else
        panel_relive:setVisible(false)
        panel_boom_relive:setVisible(true)
    end

    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        --113 宝藏礼包
        local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Treasure)
        SpecialGiftLayer:set_close_callback(
            self.die_callback
        )
        LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)
        self:close_self()

    end)


    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    if self.relive_type == ReliveLayer.Relive_Type.Tournament then

        local node = self.widget:getChildByName("node")
        local image_dia = node:getChildByName("image_dia")
        local panel_goal = image_dia:getChildByName("panel_goal")
        local image_candy = panel_goal:getChildByName("image_candy")
        local image_fill = panel_goal:getChildByName("image_fill")
        local text_candy_num = panel_goal:getChildByName("text_candy_num")

        image_candy:setVisible(true)
        image_candy:loadTexture(ResourceManager:getInstance():getPicturePathByKey("IMAGES_CANDY_CONCH"))

        image_fill:setVisible(false)

        local GameGlobals = require "GameEx.GameGlobals"
        text_candy_num:setString(GameGlobals.Scene:get("conch_collected"))

        image_dia:getChildByName("panel_relive"):getChildByName("text_word"):setString("再加5步继续收集贝壳吧！")        

    end

end

function ReliveLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.animation:play("give_up", false)
end

function ReliveLayer:get_res_by_currency(currency)
    if currency == "gold" then
        return "images/shop/gold.png"
    elseif currency == "jewel" then
        return "images/shop/jewel.png"
    elseif currency == "rmb" then
        return "images/shop/rmb.png"
    end
end

function ReliveLayer:add_player_currency(currency, num)
    local player = Player:getInstance()
    if currency == "gold" then
        player:add_player_gold(num)
    elseif currency == "jewel" then
        player:add_player_jewel(num)
        TalkingDataManager:getInstance():sendOnPurchase("复活",1,math.abs(num))
    elseif currency == "rmb" then
    end
end

function ReliveLayer:set_relive_times(relive_times)
    if relive_times > 15 then
        relive_times = 15
    end
    local relive_info_vector = require "script/relive_info.lua"
    local relive_info = relive_info_vector[relive_times]
    self.currency = relive_info.currency
    self.image_jewel:setTexture(self:get_res_by_currency(relive_info.currency))
    self.cost = relive_info.cost
    self.text_price:setString(self.cost)
end
function ReliveLayer:set_relive_bomb_times(relive_bomb_times)
    if relive_bomb_times > 15 then
        relive_bomb_times = 15
    end
    local relive_bomb_info_vector = require "script/relive_bomb_info.lua"
    local relive_bomb_info = relive_bomb_info_vector[relive_bomb_times]
    self.currency = relive_bomb_info.currency
    self.image_jewel:setTexture(self:get_res_by_currency(relive_bomb_info.currency))
    self.cost = relive_bomb_info.cost
    self.text_price:setString(self.cost)
end

function ReliveLayer:buy_relive_success()
    self.animation:play("go_on", false)
    MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_again"), false)
    
    local current_level = Player:getInstance():get_player_current_level()
    local str = "关卡"..current_level
    TalkingDataManager:getInstance():send("每关各价格复活次数", {[str] = self.cost})
end

function ReliveLayer:set_relive_callback(relive_callback)
    self.relive_callback = relive_callback
end

function ReliveLayer:set_die_callback(die_callback)
    self.die_callback = die_callback
end

function ReliveLayer:set_is_boss(flag)
    self.is_boss = flag 
end

function ReliveLayer:set_current_boom()
    local spine = sp.SkeletonAnimation:create("spine/game/barrier_108.json", "spine/game/barrier_108.atlas", 1.0)
    spine:setSkin("red")
    spine:setAnimation(0, "zhadan", true)
    self.image_barrier:addChild(spine)
end

function ReliveLayer:setCurrentStage(goal, total_goal)
        -- 游戏资源
    self.rm = ResourceManager:getInstance()
    candy_res = { 
        ["red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RED"),
        ["green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_GREEN"),
        ["blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BLUE"),
        ["yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_YELLOW"),
        ["pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PINK"),
        ["white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_WHITE"),
        ["ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_ICE"),
        ["key"] = self.rm:getPicturePathByKey("IMAGES_CANDY_KEY"),
        ["fan"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FAN"),
        ["random"] = self.rm:getPicturePathByKey("IMAGES_CANDY_RANDOM"),
        ["panda"] = self.rm:getPicturePathByKey("IMAGES_CANDY_PANDA"),
        ["moveable_ice"] = self.rm:getPicturePathByKey("IMAGES_CANDY_MOVEABLE_ICE"),
        ["fenced"] = self.rm:getPicturePathByKey("IMAGES_CANDY_FENCE"),
        ["chocolate"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CHOCOLATE"),
        ["bomb"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BOMB"),
        ["ten"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CROSS"),
        ["lightning"] = self.rm:getPicturePathByKey("IMAGES_CANDY_LIGHTNING"),
        ["conch"] = self.rm:getPicturePathByKey("IMAGES_CANDY_CONCH"),
        ["coco"] = self.rm:getPicturePathByKey("IMAGES_CANDY_COCO"),
        ["transform_red"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_RED"),
        ["transform_green"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_GREEN"),
        ["transform_blue"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_BLUE"),
        ["transform_yellow"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_YELLOW"),
        ["transform_pink"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_PINK"),
        ["transform_white"] = self.rm:getPicturePathByKey("IMAGES_CANDY_BALL_WHITE"),
    }

    local goal_number = 0
    for key, value in pairs(goal) do
        goal_number = goal_number + 1
    end

    local node = self.widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")
    local panel_goal = image_dia:getChildByName("panel_goal")
    local image_candy = panel_goal:getChildByName("image_candy")
    local image_fill = panel_goal:getChildByName("image_fill")
    local text_candy_num = panel_goal:getChildByName("text_candy_num")
    image_candy:setVisible(false)
    image_fill:setVisible(false)
    text_candy_num:setVisible(false)

    local index = 1
    for key, value in pairs(goal) do
        local candy = cc.Sprite:create(candy_res[key])
        local fill = nil
        if value > 0 then
            fill = cc.Sprite:create("images/common/undone.png")
        else
            fill = cc.Sprite:create("images/common/done.png")
        end
        local text = cc.Label:createWithTTF("12/13", "font/zh_CN.TTF", 36)

        candy:setScale(1.5)
        candy:setPosition(cc.p((index - (goal_number - 1)/2.0 - 1) * 160  , image_candy:getPositionY()))
        fill:setPosition(cc.p(candy:getPositionX() + 35, image_fill:getPositionY()))
        text:setPosition(cc.p(candy:getPositionX(), text_candy_num:getPositionY()))
        text:setString(string.format("%d/%d",total_goal[key] - goal[key], total_goal[key]))
        text:setTextColor(text_candy_num:getTextColor())
        if self.is_boss then
            text:setVisible(false)
            fill:setVisible(false)
        end

        panel_goal:addChild(candy)
        panel_goal:addChild(fill)
        panel_goal:addChild(text)
        index = index + 1
    end

end

return ReliveLayer