require "core.LayerManager"
require "UI.Widget.MaskLayer"

RequestEnergyLayer = class("RequestEnergyLayer", function() return cc.Layer:create() end)

function RequestEnergyLayer:ctor()

    self:setName("RequestEnergyLayer")

	local widget = cc.CSLoader:createNode("mail_free_energy.csb")
	self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("mail_free_energy.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)
    self.action:play("mail_free_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "mail_free_in_end" then

        elseif str == "mail_free_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")
    self.image_dia = image_dia

    self:init_listview()
    
    local text_time = image_dia:getChildByName("image_dia")

    local send_image = image_dia:getChildByName("button_send")
    local send_Btn = ButtonWrap:create()
    send_Btn:wrap(send_image, 1)
    send_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        self:close_self()
        
        for k,v in pairs(self.image_get_vector) do
            if v:isVisible() then
                
                print(self.friendList[k].guid)
                self:send_requestGift(self.friendList[k].guid)
            end
        end
    end)

    -- 关闭按钮
    local close_image = image_dia:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

	local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function RequestEnergyLayer:init_listview()

    local listview = self.image_dia:getChildByName("panel_1")
    self.listview = listview
    local panel_1 = listview:getChildByName("1")
    --listview:setScrollBarEnabled(false)
    listview:setItemModel(panel_1:clone())
    listview:removeAllItems()
    self.image_get_vector = {}

    local function async_callback(list)
        local friendList = list
        
        for k,v in pairs(friendList) do
            if v.isRequestGift or v.guid == PLAYER_ID then
                friendList[k] = nil
            end
        end
        local temp_table = {}
        for k,v in pairs(friendList) do
            table.insert(temp_table, v)
        end
        friendList = temp_table
        self.friendList = friendList

        local count_vector = get_table_length(friendList)
        local row = math.ceil(count_vector/4.0)
        for i=1, row do
            listview:pushBackDefaultItem()
        end        
        
        local items_count = get_table_length(listview:getItems())
        for i=1,items_count do
            local item = listview:getItem(i -1)
            for j=1,4 do
                local cur_index = (i - 1) * 4 + j
                local item_child = item:getChildByName(j)
                local image_get = item_child:getChildByName("image_get")
                local text_name = item_child:getChildByName("text_name")

                if cur_index <= count_vector then
                    -- print(cur_index, count_vector)
                    if cur_index > 12 then
                        image_get:setVisible(false)
                    end
                    self.image_get_vector[cur_index] = image_get

                    local friend_info = friendList[cur_index]
                    text_name:setString(friend_info.name)

                    if friend_info.sex == "0" and friend_info.headUrl and friend_info.headUrl ~= "" then
                
                        local function on_create_finish()
                            local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friend_info.guid.."head_00"
                            if cc.FileUtils:getInstance():isFileExist(headPath) then
                                item_child:loadTexture(headPath)
                            end
                        end
                        global_create_social_head(friend_info.guid, friend_info.headUrl, on_create_finish)

                    else
                        local head_id = string.format("head_%02d", friend_info.sex+1)
                        local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                        item_child:loadTexture(head_info.res_key)
                    end

                    local head_Btn = ButtonWrap:create()
                    head_Btn:wrap(item_child, 1)
                    head_Btn:addTouchEndCallback(function()
                        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
                        if image_get:isVisible() then
                            image_get:setVisible(false)
                        else
                            image_get:setVisible(true)
                        end
                    end)
                else
                    item_child:setVisible(false)
                end
            end
        end

        performWithDelay(self,function() 
            local size = listview:getInnerContainerSize()
            listview:setInnerContainerSize(cc.size(size.width,size.height+100))
            end, 0.1)
    end

    self:request_getFriendlist(async_callback)

end

function RequestEnergyLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("mail_free__out", false)

    set_list_fade_out(self.listview)
end

function RequestEnergyLayer:request_getFriendlist(async_callback)
    --request getFriendlist 当前好友列表
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/friend/getFriendlist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function RequestEnergyLayer:send_requestGift(friendid)
    --send requestGift 请求体力
    local rpc_param = {
        PLAYER_ID,
        friendid,
        3,
        0,

    }

    RPC:call("SHIN/message/newMsg", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            --dump(response)
        else

            print("request error")

        end

    end)
end

-- function RequestEnergyLayer:test_data()
--     local list = {}
--     for i=1,5 do
--         list[i] = {
--             ["guid"] = string.format("id%d", i),
--             ["name"] = string.format("nick_name%d", i),
--             ["maxChapter"] = i,
--             ["totalStar"] = i*2,
--             ["suo_yao"] = false,
--             ["zeng_song"] = false,
--             ["logintime"] = string.format("%d小时", i),
--         }
--     end
--     return list
-- end

function RequestEnergyLayer:onEnter()
    RPC:register_target(self)
end

function RequestEnergyLayer:onExit()
    RPC:unregister_target(self)
end

return RequestEnergyLayer