require "UI.Layer.CommonPopupLayer"
require "UI.Widget.SectionLockWidget"
require "core.Mission_Data"
require "core.Section_Data"
require "UI.Widget.MaskLayer"

SectionLayer = class("SectionLayer", function() return cc.Layer:create() end)

KSectionTag = 101
MapName = {
    Japan_Map = 1,
    Forest_Map = 2, 
    Desert_Map = 3,
}

function SectionLayer:ctor()

    self:setName("SectionLayer")

    self.enter_section_callback = nil
    cc.UserDefault:getInstance():setBoolForKey("is_section_layer", true)
    self:setTag(KSectionTag)
    local visible_size = cc.Director:getInstance():getVisibleSize()
    local section_map = cc.CSLoader:createNode("section_map.csb")
    self:addChild(section_map)

    section_map:setContentSize(visible_size)
    ccui.Helper:doLayout(section_map)

    local cloud_action = cc.CSLoader:createTimeline("section_map.csb")
    section_map:runAction(cloud_action)

    local play_cloud = function()

        cloud_action:play("cloud_screen", false)
        ccui.Helper:doLayout(section_map)

    end

    local rep = cc.RepeatForever:create(cc.Sequence:create(cc.CallFunc:create(play_cloud), cc.DelayTime:create(0.033 * 1200)))
    self:runAction(rep)

    -- ui
    local node = section_map:getChildByName("node")
    local panel_map = node:getChildByName("panel_map")

    local map_size =  cc.size(get_node_width(panel_map), get_node_height(panel_map))
    panel_map:setContentSize(cc.size(get_node_width(panel_map), visible_size.height))
    panel_map:setInnerContainerSize(cc.size(map_size.width, map_size.height))
    panel_map:jumpToBottom()
    panel_map:setScrollBarEnabled(false)
    panel_map:setBounceEnabled(true)
    self.panel_map = panel_map

    self.map_image_map = {}
    local section_info_vector = Section_Data:getInstance():get_section_info_vector()
    for i=1, 7 do
        local section_info = section_info_vector[i]
        local panel_map = panel_map:getChildByName(string.format("panel_%d", i))
        local map_image = panel_map:getChildByName(string.format("button_%d", i))
        local map_Btn = ButtonWrap:create()
        map_Btn:wrap(map_image, 1)
        map_Btn:addTouchEndCallback(function()
            Player:getInstance():set_player_current_section(section_info.section_id)
            self:getParent():move_map(section_info.section_id)
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        end)
        map_Btn:add_whiting()
        self.map_image_map[section_info.section_id] = map_image

        -- 地图相应的数据
        local image_board_map = panel_map:getChildByName(string.format("image_board_%d", i))
        local text_num = image_board_map:getChildByName("text_num")
        local player_star_count = Player:getInstance():get_player_section_star_count_by_section(section_info.section_id)
        local total_star_count = Mission_Data:getInstance():get_section_mission_count(section_info.section_id) * 3
        text_num:setString(string.format("%d/%d", player_star_count, total_star_count))

        if player_star_count >= total_star_count then
            self:runAction(cc.Sequence:create(cc.DelayTime:create(0.5), cc.CallFunc:create(function()
            image_board_map:setTexture("images/section/map_dialog_full.png")
            end)))
            text_num:setTextColor(cc.c3b(255, 255, 0))
            image_board_map:getChildByName("image_crown"):setVisible(true)
        end

        local player_section_info = Player:getInstance():get_player_section_info_by_id(section_info.section_id)
        local unlock_section = false
        if player_section_info then
            -- 解锁章节动画
            if player_section_info.is_enable then
                unlock_section = true
                if player_section_info.is_animation then
                    Player:getInstance():set_player_section_is_animation(player_section_info.section_id, false)
                    map_image:setColor(cc.c3b(255 * 0.6, 255 * 0.6, 255 * 0.6))
                    map_Btn:disable()
                    local sectionLockWidget = SectionLockWidget:create()
                    sectionLockWidget:setPosition(cc.p(get_node_width(map_image)/2, get_node_height(map_image)/2))
                    map_image:addChild(sectionLockWidget, 1)
                    local percent = self:get_move_percent()
                    local delay_time = 0.0
                    if percent > 0 then
                        delay_time = 3.0 * math.pow(percent/100.0, 0.5)
                    end
                    self:runAction(cc.Sequence:create(cc.DelayTime:create(delay_time/1.5), cc.CallFunc:create(function()
                        local mask_layer = MaskLayer:create()
                        cc.Director:getInstance():getRunningScene():addChild(mask_layer, 5)
                        sectionLockWidget:set_unlock_callback(function()
                            map_image:setColor(cc.c3b(255, 255, 255))
                            map_Btn:enable()
                            if self.enter_section_callback then
                                self:runAction(cc.Sequence:create(cc.DelayTime:create(0.3), cc.CallFunc:create(function()
                                    mask_layer:removeFromParent()
                                    self.enter_section_callback()
                                end)))
                            else
                                mask_layer:removeFromParent()
                            end
                        end)
                        sectionLockWidget:unlock()
                    end)))
                end
            end

            local player_star_count = Player:getInstance():get_player_section_star_count_by_section(section_info.section_id)
            local total_star_count = Mission_Data:getInstance():get_section_mission_count(section_info.section_id) * 3
            text_num:setString(string.format("%d/%d", player_star_count, total_star_count))
        else

        end

        if unlock_section == false then
            map_image:setColor(cc.c3b(255 * 0.6, 255 * 0.6, 255 * 0.6))
            map_Btn:disable()

            local sectionLockWidget = SectionLockWidget:create()
            sectionLockWidget:setPosition(cc.p(get_node_width(map_image)/2, get_node_height(map_image)/2))
            map_image:addChild(sectionLockWidget, 1)
        end
    end
    self:Scroll_To_Current_Section()

    self.move_finish_callback = function() end

end

function SectionLayer:Scroll_To_Current_Section()
    local section_id = Player:getInstance():get_player_current_section()
    if self.map_image_map[section_id]  then
        local pos_y = self.map_image_map[section_id]:getPositionY()
        local height = self.panel_map:getContentSize().height
        local container_height =  self.panel_map:getInnerContainerSize().height
        local percent = ((container_height - pos_y) - height/2)/(container_height - height)
        self.panel_map:jumpToPercentVertical(math.min(math.max(0, percent * 100), 100))
    else
        self.panel_map:jumpToPercentVertical(0)
    end
end

function SectionLayer:move_to_current_section()
    self.panel_map:jumpToPercentVertical(0)
    local percent = self:get_move_percent()
    local current_percent = self:get_current_move_percent()

    print(string.format("current: %d,target: %d", current_percent, percent))

    -- 速度非匀速变化
    if percent > 0 then
        -- 滚动界面，锁掉点击事件
        self:getParent():add_mask_layer()
        local time = 2.5 * math.pow(math.abs(100 - current_percent - percent)/100.0, 0.5)
        self.panel_map:scrollToPercentVertical(math.min(math.max(0, percent), 100), time, true)
        self:runAction(cc.Sequence:create(cc.DelayTime:create(time/1.5), cc.CallFunc:create(function()
            self:getParent():remove_mask_layer()
            self.move_finish_callback()
        end)))
    else
        self.move_finish_callback()
        self.panel_map:jumpToPercentVertical(0)
    end
end

function SectionLayer:move_and_enter_section(section_id, mission_id)
    self.enter_section_callback = function()
        Player:getInstance():set_player_current_section(section_id)
        self:getParent():switch_to_level_info(section_id, mission_id)
    end
    self:move_to_current_section()
end

function SectionLayer:get_move_percent()
    local section_id = Player:getInstance():get_player_newest_section()
    if self.map_image_map[section_id]  then
        local pos_y = self.map_image_map[section_id]:getPositionY()
        local height = self.panel_map:getContentSize().height
        local container_height =  self.panel_map:getInnerContainerSize().height
        local percent = ((container_height - pos_y) - height/2)/(container_height - height) * 100
        return percent
    end
    return 0
end

function SectionLayer:get_current_move_percent()

    local visible_height = self.panel_map:getContentSize().height
    local container_height =  self.panel_map:getInnerContainerSize().height

    local container = self.panel_map:getInnerContainer()
    local container_pos = cc.p(container:getPosition())
    local percent = -container_pos.y / (container_height - visible_height) * 100

    return percent

end

function SectionLayer:setScrollFinishCallback(callback)

    self.move_finish_callback = callback

end

return SectionLayer