require "core.LayerManager"
require "UI.Widget.BagRewardWidget"
require "UI.Layer.CommonPopupLayer"
require "UI.Widget.MaskLayer"
require "core.TalkingDataManager"

StarRewardLayer = class("StarRewardLayer", function() return cc.Layer:create() end)

function StarRewardLayer:ctor()

    self:setName("StarRewardLayer")

	local widget = cc.CSLoader:createNode("star_reward.csb")
    self:addChild(widget, 1);
    self.widget = widget

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("star_reward.csb")
    widget:runAction(self.action)

    self.action:setCurrentFrame(0)
    self.action:play("star_reward_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "star_reward_in_end" then

        elseif str == "star_reward_out_end" then
            self:removeFromParent()
        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    --bag_get_reward
    self.bag_reward = BagRewardWidget:create()
    self:addChild(self.bag_reward, 2)

    local node = widget:getChildByName("node")
    local image_dia = node:getChildByName("image_dia")
    self.image_dia = image_dia

    --text_word
    self.text_word = image_dia:getChildByName("text_word")

    -- 进度条
    self.text_num = image_dia:getChildByName("text_num")
    self.progress = image_dia:getChildByName("progress")

    -- 领取按钮
    self.image_dia_1 = image_dia:getChildByName("image_dia_1")
    local getImage = self.image_dia_1:getChildByName("button_get")
    self.text_get = getImage:getChildByName("text_get")
    self.text_get:setLocalZOrder(1)
    self.getBtn = ButtonWrap:create()
    self.getBtn:wrap(getImage, 1)
    self.getBtn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Buyitem_Audio"), false)
        --获取用户的单例，获得玩家星星信息，然后从星星信息那里返回
        local player = Player:getInstance()
        local player_star_reward_info = player:get_unget_player_star_reward_info()
        local star_reward_info = Star_Reward_Data:getInstance():get_star_reward_info_by_id(player_star_reward_info.reward_id)
        local reward_item = star_reward_info.reward_item

        local item_id = ""
        local item_num = 0
        for k,v in pairs(reward_item) do
            player:add_player_bag_info_by_id(k, v)
            item_id = k
            item_num = v
            break
        end
        player:set_player_star_reward_info_is_get(player_star_reward_info.reward_id)
        self:update_star_info()

        if item_id == "item_01" or item_id == "item_02" or item_id == "item_03" then
            if item_id == "item_03" then
                TalkingDataManager:getInstance():sendOnReward(item_num,"星星奖励")
            end
        else
            local bag_info = Bag_Data:getInstance():get_bag_info_by_id(item_id)
            self.bag_reward:setItemTexture(bag_info.res_key)
            self.bag_reward:setTouchEnabled(true)
            self.bag_reward:runAction()
        end
    end)

    self:update_star_info()

    -- 关闭按钮
    local closeImage = image_dia:getChildByName("button_close")
    local closeBtn = ButtonWrap:create()
    closeBtn:wrap(closeImage, 1)
    closeBtn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)

    self:add_role_spine()

    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.3), cc.CallFunc:create(function()
        
        TRIGGER_EVENT("STAR_REWARD_LAYER_SHOW")

    end)))

end

function StarRewardLayer:add_role_spine()

    local current_role_id = Player:getInstance():get_player_cur_role_id()
    local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)

    local role_spine = ResourceManager:getInstance():getSpineByKey(role_info.ui_spine_name, 1.0)
    role_spine:setAnimation(0, "star", true)
    -- role_spine:setTimeScale(2)

    local node = self.widget:getChildByName("node")
    local node_role = node:getChildByName("node_role")
    node_role:addChild(role_spine)

    -- local rep = cc.RepeatForever:create(cc.Sequence:create(cc.DelayTime:create(1.5), cc.CallFunc:create(function() role_spine:setAnimation(0, "star", false) end)))

    -- self:runAction(rep)

end

function StarRewardLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self.action:play("star_reward_out", false)
    --8.4
    self.getBtn:set_fade_out()
    --8.4
end

function StarRewardLayer:update_star_info()
    local player = Player:getInstance()
    local player_star_reward_info = player:get_unget_player_star_reward_info()
    local star_reward_info = Star_Reward_Data:getInstance():get_star_reward_info_by_id(player_star_reward_info.reward_id)

    local player_star_count = player:get_player_star_count()
    self.text_num:setString(string.format("%d/%d", player_star_count, star_reward_info.star_count))
    self.progress:setPercent(player_star_count * 100 / star_reward_info.star_count)

    local diff = star_reward_info.star_count - player_star_count
    if diff > 0 then
        self.text_word:setString(string.format(ResourceManager:getInstance():getTextStringByKey("StarRewardLayer_6"), diff))
    else
        self.text_word:setString(ResourceManager:getInstance():getTextStringByKey("StarRewardLayer_7"))
    end
    

    -- 奖励的物品
    local reward_item = star_reward_info.reward_item
    local image_reward = self.image_dia_1:getChildByName("image_reward")
    local text_reward = self.image_dia_1:getChildByName("text_reward")
    for k,v in pairs(reward_item) do
        local item = Bag_Data:getInstance():get_bag_info_by_id(k)
        image_reward:setTexture(item.res_key)
        text_reward:setString(string.format("x%d", v))
    end

    if player_star_count >= star_reward_info.star_count then
        self.getBtn:enable()
        self.getBtn:setDarkButton(true)
        self.text_get:setColor(cc.c3b(255, 255, 255))
        self.text_get:setOpacity(255)
    else
        self.getBtn:disable()
        self.getBtn:setDarkButton(false)
        self.text_get:setColor(cc.c3b(0, 0, 0))
        self.text_get:setOpacity(255 * 0.5)
    end

end

return StarRewardLayer