local TutorialInterface = class("TutorialInterface", function() return cc.Layer:create() end)

function TutorialInterface:ctor()

	self.__t_indication_info = {}
	self.__t_config = {}
	self.__t_traverse_list = {}

	self:create_mask_layer()

end

function TutorialInterface:init_with_config(config)

	self.__t_config = config

end

function TutorialInterface:start()

	function start_func()

		self:search_indication_target(self.__t_config.indication)
		self:create_indication()
		self:show_indication()

		self:create_dialog(self.__t_config.dialog)
		self:show_dialog()

		self:create_instruction(self.__t_config.instruction)
		self:show_instruction()

	end

	local call = cc.CallFunc:create(start_func)
	local sequence = cc.Sequence:create(cc.DelayTime:create(0), call)
	self:runAction(sequence)

	cc.Director:getInstance():getRunningScene():addChild(self, 99999999)

	self.__c_mask:setOpacity(self.__t_config.mask_opacity or 0)

end

function TutorialInterface:finish()

	self:remove_hand()
	self:remove_halo()
	self:remove_highlight()
	self:removeFromParent()

	--1113 
	-- local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Treasure)
 --    LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)

end

function TutorialInterface:create_dialog(dialog_config)

	if not dialog_config then return end

	self.__c_dialog_node = cc.Node:create()
	self:addChild(self.__c_dialog_node, 3)

	self.__c_role_sprite = cc.Sprite:create(ResourceManager:getInstance():getPicturePathByKey(dialog_config.role))
	self.__c_role_sprite:setScale(0.7)
	self.__c_role_sprite:setAnchorPoint(cc.p(0.5, 0))
	self.__c_role_sprite:setPosition(cc.p(220, dialog_config.position - 80))
	self.__c_role_sprite:setOpacity(255)
	self.__c_dialog_node:addChild(self.__c_role_sprite, 3)

	self.__c_frame_sprite = ccui.Scale9Sprite:create({x = 50, y = 50, width = 13, height = 35}, ResourceManager:getInstance():getPicturePathByKey("DIALOG_FRAME"))
	self.__c_frame_sprite:setContentSize(cc.size(700, 300))
	self.__c_frame_sprite:setOpacity(255)
	self.__c_frame_sprite:setPosition(cc.p(540, dialog_config.position))
	self.__c_dialog_node:addChild(self.__c_frame_sprite, 2)

	self.__c_text_label = cc.Label:createWithTTF(dialog_config.text, "font/zh_CN.TTF", 46)
	self.__c_text_label:setOpacity(255)
	self.__c_text_label:setMaxLineWidth(500)
	self.__c_text_label:setOverflow(cc.LabelOverflow.RESIZE_HEIGHT)
	self.__c_text_label:setDimensions(0, 0)
	self.__c_text_label:setPosition(cc.p(400, 150))
	self.__c_text_label:setColor(cc.c3b(117, 38, 7))
	self.__c_frame_sprite:addChild(self.__c_text_label, 4)

	-- if dialog_config.button then

		self.__c_ok_button = ccui.ImageView:create(ResourceManager:getInstance():getPicturePathByKey("TUTORIAL_BUTTON_OK_1"))
		self.__c_ok_button:setPosition(cc.p(400, 0))
		self.__c_frame_sprite:addChild(self.__c_ok_button, 1)

		self.__c_ok_button_wrapper = ButtonWrap:create() 
	    self.__c_ok_button_wrapper:wrap(self.__c_ok_button, 1)
	    self.__c_ok_button_wrapper:addTouchEndCallback(function()
	        
			TRIGGER_EVENT("TUTORIAL_OK_PRESSED")

	     end)

	-- end

end

function TutorialInterface:create_instruction(instruction_config)

	if not instruction_config then return end

	self.__c_instruction_node = cc.Node:create()
	self:addChild(self.__c_instruction_node, 3)

	self.__c_instruction_img = cc.Sprite:create(ResourceManager:getInstance():getPicturePathByKey(instruction_config.image))
	self.__c_instruction_img:setPosition(cc.p(540, instruction_config.position))
	self.__c_instruction_node:addChild(self.__c_instruction_img, 2)

	self.__c_ok_button = ccui.ImageView:create(ResourceManager:getInstance():getPicturePathByKey("TUTORIAL_BUTTON_OK"))
	self.__c_ok_button:setPosition(cc.p(540, instruction_config.position - 360))
	self.__c_instruction_node:addChild(self.__c_ok_button, 3)

	self.__c_ok_button_wrapper = ButtonWrap:create() 
    self.__c_ok_button_wrapper:wrap(self.__c_ok_button, 1)
    self.__c_ok_button_wrapper:addTouchEndCallback(function()
        
		TRIGGER_EVENT("TUTORIAL_OK_PRESSED")

    end)

end

function TutorialInterface:show_instruction()



end

function TutorialInterface:show_dialog()



end


function TutorialInterface:search_indication_target(indication_config)

	if not indication_config then return end

	for key, config in ipairs(indication_config) do

		local scene = cc.Director:getInstance():getRunningScene()
		local result = self:search_for_target(config.routine, 1, scene)

		if result then

			local info = {}
			info.target = result
			info.original_parent = result:getParent()
			info.original_position = cc.p(result:getPosition())
			info.world_position = info.original_parent:convertToWorldSpace(info.original_position)
			info.original_z_order = info.target:getLocalZOrder()
			info.hand_action = config.hand_action
			info.rune = config.rune
			info.spot = config.spot
			info.highlight = config.highlight

			self.__t_indication_info[#self.__t_indication_info + 1] = info

		end

	end

end

function TutorialInterface:create_indication()

	self:create_highlight()
	self:create_halo()
	self:create_hand()

end

function TutorialInterface:remove_indication()

	self:remove_hand()
	self:remove_halo()
	self:remove_highlight()

end

function TutorialInterface:show_indication()



end

function TutorialInterface:create_highlight()

	for index, info in ipairs(self.__t_indication_info) do

		if info.highlight then

			info.target:setPosition(info.world_position)
			info.target:retain()
			info.target:removeFromParentAndCleanup(false)
			self:addChild(info.target, 1)
			info.target:release()

		end

	end

end

function TutorialInterface:remove_highlight()

	for index, info in ipairs(self.__t_indication_info) do

		if info.highlight then

			info.target:setPosition(info.original_position)
			info.target:retain()
			info.target:removeFromParentAndCleanup(false)
			info.original_parent:addChild(info.target, info.original_z_order)
			info.target:release()

		end

	end

end

function TutorialInterface:create_hand()

	for _, info in ipairs(self.__t_indication_info) do

		if info.hand_action == "HAND_ACTION_CLICK" then

			local hand_sprite = ResourceManager:getInstance():getSpriteByKey("TUTORIAL_IMAGE_HAND")
			local content_size = info.target:getContentSize()
			hand_sprite:setAnchorPoint(cc.p(0.6, 0.1))
			hand_sprite:setPosition(cc.p(content_size.width / 2 + 80, content_size.height / 2 - 95))
			hand_sprite:setName("TUTORIAL_HAND")
			info.target:addChild(hand_sprite, 1000)

			local rotate_1 = cc.RotateBy:create(0.4, 10)
			local move_1 = cc.MoveBy:create(0.4, cc.p(10, 0))
			local scale_1 = cc.ScaleTo:create(0.4, 1.02)
			local spawn_1 = cc.Spawn:create(rotate_1, move_1, scale_1)
			local rotate_2 = cc.RotateBy:create(0.2, -10)
			local move_2 = cc.MoveBy:create(0.2, cc.p(-10, 0))
			local scale_2 = cc.ScaleTo:create(0.2, 1.0)
			local spawn_2 = cc.Spawn:create(rotate_2, move_2, scale_2)
			local sequence = cc.Sequence:create(cc.DelayTime:create(1.8), spawn_1, spawn_2)
			local req = cc.RepeatForever:create(sequence)
			hand_sprite:runAction(req)

		elseif info.hand_action == "HAND_ACTION_TRAVERSE" then

			self.__t_traverse_list[#self.__t_traverse_list + 1] = info

		end

	end

	if #self.__t_traverse_list > 0 then

		self.__c_hand_sprite = ResourceManager:getInstance():getSpriteByKey("TUTORIAL_IMAGE_HAND")
		self.__c_hand_sprite:setOpacity(0)
		self:addChild(self.__c_hand_sprite, 99999)

		local action_seq = {}

		local reset_position = cc.CallFunc:create(function()

			local hand_position = cc.p(self.__t_traverse_list[1].world_position.x + 70, self.__t_traverse_list[1].world_position.y - 60)
			self.__c_hand_sprite:setPosition(hand_position)

		end)
		action_seq[1] = reset_position

		local fade_1 = cc.FadeTo:create(0.4, 255)
		action_seq[2] = fade_1

		for i = 2, #self.__t_traverse_list do

			local hand_position = cc.p(self.__t_traverse_list[i].world_position.x + 70, self.__t_traverse_list[i].world_position.y - 60)
			local move = cc.MoveTo:create(0.5, hand_position)
			action_seq[#action_seq + 1] = move

		end

		local fade_2 = cc.FadeTo:create(0.4, 0)
		action_seq[#action_seq + 1] = fade_2

		local delay = cc.DelayTime:create(2.0)
		action_seq[#action_seq + 1] = delay



		local sequence = cc.Sequence:create(action_seq)
		local rep = cc.RepeatForever:create(sequence)

		self.__c_hand_sprite:runAction(rep)

	end	

end

function TutorialInterface:remove_hand()

	for _, info in ipairs(self.__t_indication_info) do

		info.target:removeChildByName("TUTORIAL_HAND", true)

	end	

	if #self.__t_traverse_list > 0 then

		self.__c_hand_sprite:removeFromParent()

	end

end

function TutorialInterface:create_halo()

	for _, info in ipairs(self.__t_indication_info) do

		if info.rune then

			local halo_spine = ResourceManager:getInstance():getSpineByKey("SPINE_SELECT_HALO", 1)
			local content_size = info.target:getContentSize()
			halo_spine:setPosition(cc.p(content_size.width / 2, content_size.height / 2))
			halo_spine:setName("TUTORIAL_HALO")
			halo_spine:setAnimation(0, "select", true)
			info.target:addChild(halo_spine, 999)

		end

	end

end

function TutorialInterface:remove_halo()

	for _, info in ipairs(self.__t_indication_info) do

		info.target:removeChildByName("TUTORIAL_HALO", true)

	end	

end


function TutorialInterface:create_mask_layer()

	self.__c_mask = cc.LayerColor:create(cc.c4b(0, 0, 0, 0))

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    listenner:registerScriptHandler(function(touch, event)
        -- TRIGGER_EVENT("TUTORIAL_OK_PRESSED") 1113
    end, cc.Handler.EVENT_TOUCH_ENDED )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self.__c_mask)

	self:addChild(self.__c_mask)

end

function TutorialInterface:search_for_target(routine, cur, start_node)

	if not start_node then return end

	local child = start_node:getChildByName(routine[cur])

	if not child then return end

	if #routine == cur then return child else return self:search_for_target(routine, cur + 1, child) end

end


return TutorialInterface
