local MaskLayer = require "UI/Widget/MaskLayer"


TutorialLayer = class("TutorialLayer", function() return cc.Layer:create() end)

function TutorialLayer:ctor()

	self.__s_enter_type = "full_animated"
	self.__s_leave_type = "full_animated"

	self.__f_enter_callback = function() end
	self.__f_leave_callback = function() end

    self.__c_clipping_node = cc.ClippingNode:create()
    self.__c_clipping_node:setInverted(true)
    self.__c_clipping_node:setAlphaThreshold(0.1)
  	self.__c_clipping_node:setStencil(cc.Sprite:create())
    self:addChild(self.__c_clipping_node, 1)

    self.__c_view_bg = cc.LayerColor:create(cc.c4b(0, 0, 0, 190))
    self.__c_clipping_node:addChild(self.__c_view_bg, 1)

    self.__c_mask_layer = MaskLayer:create()
    self.__c_mask_layer:init_background(cc.c4b(0, 0, 0, 0))
    self.__c_mask_layer:setEnabled(false)
    self:addChild(self.__c_mask_layer, 99)

    self.__n_touch_through = true


end

function TutorialLayer:add_dialog(t_param)

	if t_param.dialog_type == "1" or 
		t_param.dialog_type == "2" or
		t_param.dialog_type == "3" or
		t_param.dialog_type == "4" or 
		t_param.dialog_type == "5" then

		self.__c_guide_node = cc.CSLoader:createNode("guide.csb")
		self.__c_guide_node:setContentSize(cc.Director:getInstance():getVisibleSize())
		self.__c_guide_node.dialog_type = t_param.dialog_type
    	self:addChild(self.__c_guide_node, 2);
    	ccui.Helper:doLayout(self.__c_guide_node)

    	self.__c_guide_node:getChildByName(string.format("panel_%s", t_param.dialog_type)):getChildByName("image_dia"):getChildByName("text_2"):setVisible(false)
    	self.__c_guide_node:getChildByName(string.format("panel_%s", t_param.dialog_type)):getChildByName("image_dia"):getChildByName("text_1"):setString(t_param.dialog_content)

    	if t_param.no_portrait then

    		self.__c_guide_node:getChildByName(string.format("panel_%s", t_param.dialog_type)):getChildByName("panel_mask"):getChildByName("image_role"):setVisible(false)

    	end

    	if t_param.dialog_type == "5" then

    		self.__c_guide_node:getChildByName(string.format("panel_%s", t_param.dialog_type)):getChildByName("image_dia"):setTexture(string.format("res/images/guide/guide_%s.png", t_param.info_type))

    	end

    	self:add_skip_button()

	end

end

function TutorialLayer:add_skip_button()

	local wrapper = ButtonWrap:create()
	wrapper:wrap(self.__c_guide_node:getChildByName("text_skip"), ButtonWrap.Action_Type.NORMAL)
	wrapper:addTouchEndCallback(function()

		local event = cc.EventCustom:new("tutorial_skip")
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

	end)

end

function TutorialLayer:add_view(t_param)

	self.__c_view_mask = cc.Sprite:create("images/game/in_game/upper_mask.png")
    self.__c_view_mask:setPosition(cc.p(t_param.view_region.x, t_param.view_region.y))
    self.__c_view_mask:setScaleX(t_param.view_region.width / 1080)
    self.__c_view_mask:setScaleY(t_param.view_region.height / 408)
    self.__c_view_mask.view_region = t_param.view_region

    self.__c_clipping_node:setStencil(self.__c_view_mask)

end

function TutorialLayer:add_indicator(t_param)

	self.__c_indicator = cc.Sprite:create("images/guide/hand.png")
	self.__c_indicator:setPosition(cc.p(t_param.indicator_position.x + 80, t_param.indicator_position.y - 30))
	self:addChild(self.__c_indicator, 3)

	local move_1 = cc.MoveBy:create(0.5, cc.p(0, -15))
	local move_2 = cc.MoveBy:create(0.5, cc.p(-0, 15))
	local sequence_1 = cc.Sequence:create(move_1, move_2)

	local rotate_1 = cc.RotateBy:create(0.5, -20)
	local rotate_2 = cc.RotateBy:create(0.5, 20)
	local sequence_2 = cc.Sequence:create(rotate_1, rotate_2)

	local spawn = cc.Spawn:create(sequence_1, sequence_2)

	local rep = cc.RepeatForever:create(spawn)

	self.__c_indicator:runAction(rep)

end

function TutorialLayer:add_exit_with_touch()

	if self.__c_guide_node then

		self.__c_guide_node:getChildByName(string.format("panel_%s", self.__c_guide_node.dialog_type)):getChildByName("image_dia"):getChildByName("text_2"):setVisible(true)

	end

end

function TutorialLayer:set_enter_type(type)

	self.__s_enter_type = type

end

function TutorialLayer:set_leave_type(type)

	self.__s_leave_type = type

end

function TutorialLayer:set_enter_callback(callback)

	self.__f_enter_callback = callback

end

function TutorialLayer:set_leave_callback(callback)

	self.__f_leave_callback = callback

end

function TutorialLayer:set_touch_through(touch_through_flag)

	self.__n_touch_through = touch_through_flag

end

function TutorialLayer:set_background_invisible()

	self.__c_clipping_node:setVisible(false)

end

function TutorialLayer:set_no_skip()

	if self.__c_guide_node then

		self.__c_guide_node:getChildByName("text_skip"):setVisible(false)

	end

end

function TutorialLayer:enter()
print("enter")
	self.__c_mask_layer:setEnabled(true)

	if self.__s_enter_type == "full_animated" then

		local act_1 = self:build_dark_in_animation()
		local act_2 = self:build_dialog_enter_animation()
		local act_3 = self:build_view_enter_animation()
		local act_4 = self:build_indicator_enter_animation()
		local act_5 = cc.CallFunc:create(function() self:after_enter() self.__c_mask_layer:setEnabled(false) end)

		self:runAction(cc.Sequence:create(act_1, act_2, act_3, act_4, act_5))

	elseif self.__s_enter_type == "no_dark_in" then

		local act_2 = self:build_dialog_enter_animation()
		local act_3 = self:build_view_enter_animation()
		local act_4 = self:build_indicator_enter_animation()
		local act_5 = cc.CallFunc:create(function() self:after_enter() self.__c_mask_layer:setEnabled(false) end)

		self:runAction(cc.Sequence:create(act_2, act_3, act_4, act_5))

	elseif self.__s_enter_type == "direct" then

		local act_2 = self:build_dialog_enter_animation()
		local act_5 = cc.CallFunc:create(function() self:after_enter() self.__c_mask_layer:setEnabled(false) end)

		self:runAction(cc.Sequence:create(act_2, act_5))

	elseif self.__s_enter_type == "trigger_only" then
print("trigger_only")
		self.__f_enter_callback()

	end

end

function TutorialLayer:leave()

	self.__c_mask_layer:setEnabled(true)

	if self.__s_leave_type == "full_animated" then
print("leave")
		local act_1 = self:build_dark_out_animation()
		local act_2 = self:build_dialog_leave_animation()
		local act_3 = self:build_view_leave_animation()
		local act_4 = self:build_indicator_leave_animation()
		local act_5 = cc.CallFunc:create(function() self:after_leave() self.__c_mask_layer:setEnabled(false) end)

		self:runAction(cc.Sequence:create(cc.Spawn:create(act_1, act_2, act_3, act_4), act_5))

	elseif self.__s_leave_type == "no_dark_out" then

		local act_2 = self:build_dialog_leave_animation()
		local act_3 = self:build_view_leave_animation()
		local act_4 = self:build_indicator_leave_animation()
		local act_5 = cc.CallFunc:create(function() self:after_leave() self.__c_mask_layer:setEnabled(false) end)

		self:runAction(cc.Sequence:create(cc.Spawn:create(act_2, act_3, act_4), act_5))		

	elseif self.__s_leave_type == "direct" then

		local act_5 = cc.CallFunc:create(function() self:after_leave() self.__c_mask_layer:setEnabled(false) end)
		self:runAction(cc.Sequence:create(act_5))	

	end

end

function TutorialLayer:after_enter()

	self:register_touch_handler()
	self.__f_enter_callback()

end

function TutorialLayer:after_leave()

	self:removeFromParent()
	self.__f_leave_callback()

end

function TutorialLayer:register_touch_handler()
print("register touch handler")
	local function touch_down(touch, event)

		local event = cc.EventCustom:new("tutorial_layer_touch_anywhere")
		cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

		if self.__n_touch_through and self.__c_view_mask then

			local touch_position = touch:getLocation()
dump(self.__c_view_mask.view_region)
dump(touch_position)
			if touch_position.x > self.__c_view_mask.view_region.x - self.__c_view_mask.view_region.width / 2 and
				touch_position.x < self.__c_view_mask.view_region.x + self.__c_view_mask.view_region.width / 2 and
				touch_position.y > self.__c_view_mask.view_region.y - self.__c_view_mask.view_region.height / 2 and 
				touch_position.y < self.__c_view_mask.view_region.y + self.__c_view_mask.view_region.height / 2 then
print("touch in view")
				return false

			else
print("touch out view")
				return true

			end

		else

			return true

		end

	end

	local touch_listener = cc.EventListenerTouchOneByOne:create()
    touch_listener:setSwallowTouches(true)
    touch_listener:registerScriptHandler(touch_down,cc.Handler.EVENT_TOUCH_BEGAN)
    self:getEventDispatcher():addEventListenerWithSceneGraphPriority(touch_listener, self)

end

function TutorialLayer:build_indicator_enter_animation()

	if self.__c_indicator then

		self.__c_indicator:setOpacity(0)
		self.__c_indicator:setScale(1.5)

		local fade_action = cc.FadeTo:create(0.2, 255)
		local move_action_1 = cc.MoveBy:create(0, cc.p(80, -80))
		local move_action_2 = cc.MoveBy:create(0.2, cc.p(-80, 80))
		local scale = cc.ScaleTo:create(0.2, 1.0)
		local sequence = cc.Sequence:create(move_action_1, move_action_2)
		local spawn = cc.Spawn:create(scale, fade_action, sequence)

		return ACTION_FOR_TARGET(spawn, self.__c_indicator)

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_indicator_leave_animation()

	if self.__c_indicator then

		local fade_action = cc.FadeTo:create(0.2, 0)
		local move_action = cc.MoveBy:create(0.2, cc.p(-50, -50))
		local spawn = cc.Spawn:create(fade_action, move_action)

		return ACTION_FOR_TARGET(spawn, self.__c_indicator)

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_dialog_enter_animation()

	if self.__c_guide_node then

		self.__c_guide_node:setVisible(false)

		local begin_call = function()

			self.__c_guide_node:setVisible(true)

			self.__c_guide_action = cc.CSLoader:createTimeline("guide.csb")
    		self.__c_guide_node:runAction(self.__c_guide_action)

			self.__c_guide_action:play(string.format("panel_%s_in", self.__c_guide_node.dialog_type), false)

		end

		local call_action = cc.CallFunc:create(begin_call)
		local delay_action = cc.DelayTime:create(10 * 0.033)
		local spawn = cc.Spawn:create(call_action, delay_action)
		local sequence = cc.Sequence:create(spawn, cc.DelayTime:create(0.3))

		return sequence

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_dialog_leave_animation()

	if self.__c_guide_node then
print("dialog leave")
		local begin_call = function()

			self.__c_guide_action = cc.CSLoader:createTimeline("guide.csb")
    		self.__c_guide_node:runAction(self.__c_guide_action)

			self.__c_guide_action:play(string.format("panel_%s_out", self.__c_guide_node.dialog_type), false)

		end

		local call_action = cc.CallFunc:create(begin_call)
		local delay_action = cc.DelayTime:create(10 * 0.033)
		local spawn = cc.Spawn:create(call_action, delay_action)

		return spawn

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_view_enter_animation()

	if self.__c_view_mask then

		self.__c_view_mask:setScale(0)

		local scale_action_1 = cc.ScaleTo:create(3 * 0.033, self.__c_view_mask.view_region.width / 1080 * 1.05, self.__c_view_mask.view_region.height / 408 * 1.05)
		local scale_action_2 = cc.ScaleTo:create(2 * 0.033, self.__c_view_mask.view_region.width / 1080, self.__c_view_mask.view_region.height / 408)
		local sequence = cc.Sequence:create(scale_action_1, scale_action_2, cc.DelayTime:create(0.3))

		return ACTION_FOR_TARGET(sequence, self.__c_view_mask)

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_view_leave_animation()

	if self.__c_view_mask then

		local scale_action_1 = cc.ScaleTo:create(2 * 0.033, self.__c_view_mask.view_region.width / 1080 * 1.05, self.__c_view_mask.view_region.height / 408 * 1.05)
		local scale_action_2 = cc.ScaleTo:create(3 * 0.033, 0, 0)
		local sequence = cc.Sequence:create(scale_action_1, scale_action_2, cc.DelayTime:create(0.3))

		return ACTION_FOR_TARGET(sequence, self.__c_view_mask)

	else

		return cc.DelayTime:create(0)

	end

end

function TutorialLayer:build_dark_in_animation()

	self.__c_view_bg:setOpacity(0)

	local fade_action = cc.FadeTo:create(0.15, 190)

	return ACTION_FOR_TARGET(fade_action, self.__c_view_bg)

end

function TutorialLayer:build_dark_out_animation()

	local fade_action = cc.FadeTo:create(0.15, 0)

	return ACTION_FOR_TARGET(fade_action, self.__c_view_bg)

end

return TutorialLayer
