require "core.Role_Data"
require "core.Skin_Data"
require "core.Book_Data"
require "core.Mission_Data"
require "UI.Layer.CommonPopupLayer"
require "core.LayerManager"
require "UI.Widget.MaskLayer"

UnlockRoleLayer = class("UnlockRoleLayer", function() return cc.Layer:create() end)

function UnlockRoleLayer:ctor()

    self:setName("UnlockRoleLayer")

    self.switch_callback = nil
    self.card_enable = true
	local widget = cc.CSLoader:createNode("unlock_role.csb")
    self:addChild(widget, 1);

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    -- ui
    local text_word = widget:getChildByName("text_word")
    local text_word_0 = widget:getChildByName("text_word_0")
    self.image_role = widget:getChildByName("image_role")

    self.panel_book = widget:getChildByName("panel_book")

    -- 关闭按钮
    local close_image = widget:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)


end

function UnlockRoleLayer:set_cur_role(role_id)

    local role_info = Role_Data:getInstance():get_role_info_by_id(role_id)
    local skin_id = role_info.skin_id_list[1]
    local skin_info = Skin_Data:getInstance():get_skin_info_by_id(skin_id)
    self.image_role:setTexture(skin_info.res_key)

    local book_list = skin_info.unlock_condition

    for i=1,3 do
        local book_info = Book_Data:getInstance():get_book_info_by_id(book_list[i])
        local player_book_info = Player:getInstance():get_player_book_info_by_id(book_list[i])
        local mission_info = Mission_Data:getInstance():get_mission_info_by_id(book_info.generate_mission_id)

        local card = self.panel_book:getChildByName(string.format("card_%d", i))
        local card_Btn = ButtonWrap:create()
        card_Btn:wrap(card, 1)
        if self.card_enable == false then
            card_Btn:disable()
        end
        card_Btn:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
            local commonPopupLayer = CommonPopupLayer:create()
            commonPopupLayer:disable_close()
            commonPopupLayer:setPosition(card:getParent():convertToWorldSpace(cc.p(card:getPosition())))
            local content_text = string.format(ResourceManager:getInstance():getTextStringByKey("UnlockRoleLayer_1"), mission_info.mission_level)
            commonPopupLayer:show_card_level(no_str, content_text, ResourceManager:getInstance():getTextStringByKey("UnlockRoleLayer_2"), function()
                    local mission_info = Mission_Data:getInstance():get_mission_info_by_id(book_info.generate_mission_id)
                    local is_section_layer = cc.UserDefault:getInstance():getBoolForKey("is_section_layer", true)
                    if is_section_layer then
                        cc.Director:getInstance():getRunningScene():switch_to_level_info(mission_info.mission_section, mission_info.mission_level)
                    else
                        local close_callback = nil
                        local levelInfoLayer1 = cc.Director:getInstance():getRunningScene():getChildByTag(1001)
                        if levelInfoLayer1 then
                            close_callback = levelInfoLayer1:get_close_callback()
                            levelInfoLayer1:close_self()
                        end

                        Player:getInstance():set_player_current_section(mission_info.mission_section)
                        Player:getInstance():set_player_current_level(mission_info.mission_level)
                        local levelInfoLayer2 = LevelInfoLayer:create()
                        levelInfoLayer2:set_close_callback(close_callback)
                        LayerManager:getInstance():add_layer_to_scene(levelInfoLayer2)
                    end
                    if self.switch_callback then
                        self.switch_callback()
                    end
                    commonPopupLayer:removeFromParent()
                    self:close_self()
            end, nil)

            if player_book_info.is_get then
                commonPopupLayer:disable_get()
            end

            local player_mission_info = Player:getInstance():get_player_mission_info_by_id(book_info.generate_mission_id)
            if player_mission_info and player_mission_info.is_enable then
                commonPopupLayer:set_get_dark(true)
            else
                commonPopupLayer:set_get_dark(false)
            end
            
        end)

        local image_role = card:getChildByName("image_role")
        local text_num = card:getChildByName("text_num")
        if player_book_info.is_get then
            card:loadTexture(Book_Data:getInstance():get_rank_res_by_res(book_info.books_rank))
            image_role:loadTexture(book_info.res_key)
        else
            card:loadTexture(Book_Data:getInstance():get_book_unlock_res())
            image_role:setVisible(false)
        end
        text_num:setString(string.format(ResourceManager:getInstance():getTextStringByKey("UnlockRoleLayer_3"), mission_info.mission_level))

    end
end

function UnlockRoleLayer:set_go_on_enable(flag)
    self.card_enable = flag
end

function UnlockRoleLayer:close_self()
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()
    self:removeFromParent()
end

function UnlockRoleLayer:set_swtich_callback(switch_callback)
    self.switch_callback = switch_callback
end

return UnlockRoleLayer