local UpdateManager = require "core.UpdateManager"

local UpdateLayer = class("UpdateLayer", function() return cc.Layer:create() end)

function UpdateLayer:ctor()

    self:init_description()
    self:init_title()
    self:init_background()
    self:init_progress()
    self:start_update()

end

function UpdateLayer:init_title()

    self.title = {}
    self.title_word_count = 4
    self.font_size = 128

    for i = 1, self.title_word_count do

        self.title[i] = cc.Label:createWithTTF("X", "font/zh_CN.TTF", 128)
        self:addChild(self.title[i], 1)

        local visible_size = cc.Director:getInstance():getVisibleSize()
        local font_width = self.font_size * 1.0
        local offset = visible_size.width / 2 - font_width * self.title_word_count / 2 + font_width / 2

        self.title[i]:setPosition(cc.p(offset + font_width * (i - 1), 1280))

    end

    self.title[1]:setString("游")
    self.title[2]:setString("戏")
    self.title[3]:setString("更")
    self.title[4]:setString("新")

    self:runAction(cc.RepeatForever:create(cc.Sequence:create(cc.CallFunc:create(function()

        for i = 1, self.title_word_count do

            self.title[i]:runAction(cc.Sequence:create(cc.DelayTime:create(i * 0.2), cc.MoveBy:create(0.4 - i * 0.03, cc.p(0, 50 - i * 8)), cc.MoveBy:create(0.3 - i * 0.03, cc.p(0, -50 + i * 8))))

        end

    end),cc.DelayTime:create(1 + 0.3 * self.title_word_count))))

end

function UpdateLayer:init_description()

    self.description = cc.Label:createWithTTF("", "font/zh_CN.TTF", 36)

    self:addChild(self.description, 3)

    local visible_size = cc.Director:getInstance():getVisibleSize()
    self.description:setPosition(cc.p(visible_size.width / 2, 400))

end

function UpdateLayer:init_background()

    self.background = cc.LayerColor:create(cc.c4b(0, 0, 0, 255))
    self:addChild(self.background, 0)

end

function UpdateLayer:init_progress()

    self.progress = {}
    self.progress.background = ResourceManager:getInstance():getSpriteByKey("IMAGES_LOADING_BACKGROUND")
    self.progress.bar = cc.ProgressTimer:create(ResourceManager:getInstance():getSpriteByKey("IMAGES_LOADING_BAR"))

    self:addChild(self.progress.background, 1)
    self:addChild(self.progress.bar, 2)

    local visible_size = cc.Director:getInstance():getVisibleSize()
    self.progress.background:setPosition(cc.p(visible_size.width / 2, 300))
    self.progress.bar:setPosition(cc.p(visible_size.width / 2, 300))

    self.progress.bar:setType(cc.PROGRESS_TIMER_TYPE_BAR)
    self.progress.bar:setMidpoint(cc.p(0, 0.5))
    self.progress.bar:setBarChangeRate(cc.p(1.0, 0))
    self.progress.bar:setPercentage(70) 

end

function UpdateLayer:start_update()



end

function UpdateLayer:check_new_version()



end

function UpdateLayer:download_new_version()



end

function UpdateLayer:show_confirm_dialog()



end

function UpdateLayer:show_error_dialog()



end

return UpdateLayer