require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "UI.Widget.BagRewardWidget"
require "UI.Layer.WeekgameActivityLayer"
require "UI.Layer.WeekgameTimeLayer"
require "UI.Layer.RoleInfoLayer"
require "UI.Layer.WeekgameWinLayer"
require "core.TalkingDataManager"
require "UI.Layer.CommonDialogLayer"

local TournamentProgresser = require "GameEx.Components.Scene.TournamentProgresser"

WeeklyChallengeLayer = class("WeeklyChallengeLayer", function() return cc.Layer:create() end)

function WeeklyChallengeLayer:ctor(timeStamp)

    self:setName("WeeklyChallengeLayer")
    self.close_callback = nil
    self.scheduleScriptEntryID = nil
    self.timeStamp = timeStamp

	local widget = cc.CSLoader:createNode("weekgame.csb")
	self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("weekgame.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)

    local node = widget:getChildByName("node")
    self.node = node
    local image_dia = node:getChildByName("image_dia")
    self.image_dia = image_dia

    self:update_date()
    self:init_rank()

    --bag_get_reward
    self.bag_reward = BagRewardWidget:create()
    self:addChild(self.bag_reward, 2)

    --活动介绍界面
    local activity_image = image_dia:getChildByName("button_activity")
    local activity_Btn = ButtonWrap:create()
    activity_Btn:wrap(activity_image, 1)
    activity_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        local weekgameActivityLayer = WeekgameActivityLayer:create()
        LayerManager:getInstance():add_layer_to_scene(weekgameActivityLayer)
    end)

    --开始闯关
    local start_image = image_dia:getChildByName("button_start")
    self.text_start = start_image:getChildByName("text_start")
    self.image_mask = start_image:getChildByName("image_mask")
    self.today_challenge_num_text = self.image_mask:getChildByName("text_times")

    local start_Btn = ButtonWrap:create()
    start_Btn:wrap(start_image, 1)
    start_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        if self.today_challenge_num <= 0 then
            local weekgameTimeLayer = WeekgameTimeLayer:create()
            weekgameTimeLayer:set_add_challenge_num_callback(function()
                    self:add_challenge_num_callback()
                end)
            weekgameTimeLayer:set_go_to_play_callback(function()
                    self:close_self()
                end)
            LayerManager:getInstance():add_layer_to_scene(weekgameTimeLayer)
        else
            self.today_challenge_num = self.today_challenge_num - 1
            Player:getInstance():set_player_weekly_challenge_today_challenge_num(self.today_challenge_num)
            self.today_challenge_num_text:setString(self.today_challenge_num)
            Player:getInstance():set_player_weekly_challenge_conch_once_num(0)
            --每日任务事件发送
            local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
            event.param = {task_id = "task_07", finish_count = 1, func = function() end }
            cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

            TalkingDataManager:getInstance():send("周赛次数统计", {["参与周赛次数"] = "次数"})

            Player:getInstance():set_player_current_level(0)

            --进入周赛
            local progresser = TournamentProgresser:get_instance()
            progresser:start()
            local next_level = progresser:get_next_level()

            local current_role_id = Player:getInstance():get_player_cur_role_id()
            local role_info = Role_Data:getInstance():get_role_info_by_id(current_role_id)
            local skill_info = Skill_Data:getInstance():get_skill_info_by_id(role_info.skill_id)
            local skill_level = Player:getInstance():get_player_skill_level(current_role_id)

            local skill = {}
            skill.name = skill_info.class_name
            skill.change_number = skill_info.change_num[skill_level]
            skill.fill_threshold = skill_info.trigger_condition[skill_level]

            local item = {}
            item.move = false
            item.bomb = false
            item.lightning = false

            local scene_start_param = {}
            scene_start_param.skill_info = skill
            scene_start_param.item_info = item
            scene_start_param.level_index = next_level

            local loadingScene = LoadingScene:create(2)
            loadingScene:load_game_scene_tournament_resource(scene_start_param)
            LayerManager:getInstance():clear_layer()
            cc.Director:getInstance():replaceScene(loadingScene)

        end
        
    end)

    --角色界面
    local role_image = image_dia:getChildByName("button_role")
    local role_Btn = ButtonWrap:create()
    role_Btn:wrap(role_image, 1)
    role_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        if Player:getInstance():get_player_cur_role_id() == "role_101" then
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_xin"), false)
        end
        local roleInfoLayer = RoleInfoLayer:create()
        LayerManager:getInstance():add_layer_to_scene(roleInfoLayer)
    end)

    --今日奖励
    self:init_today_reward()

    --每周奖励
    self:init_week_reward()

    --倒计时
    local week_text_time = image_dia:getChildByName("text_time")
    local day_text_time = self.image_dia:getChildByName("panel_day_reward"):getChildByName("text_time")

    local function update_Challenge_countdown_time()
        local t1,t2 = self:get_electronic_clock()
        week_text_time:setString(t1)
        day_text_time:setString(t2)
    end
    update_Challenge_countdown_time()
    self.scheduleScriptEntryID = cc.Director:getInstance():getScheduler():scheduleScriptFunc(update_Challenge_countdown_time, 1.0, false)

    -- 关闭按钮
    local close_image = node:getChildByName("button_close")
    local close_Btn = ButtonWrap:create()
    close_Btn:wrap(close_image, 1)
    close_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
    end)

    self:update_reward()

    local function onNodeEvent(event)
        if "enter" == event then
            self:onEnter()
        elseif "exit" == event then
            self:onExit()
        end
    end
    self:registerScriptHandler(onNodeEvent)

    -- 吃掉点击事件
    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        -- if self.box_des:isVisible() then
        --     self.box_des:setVisible(false)
        --     for i=1,3 do
        --         local panel_ = self.box_des:getChildByName(string.format("panel_%d",i))
        --         panel_:setVisible(false)
        --     end
        -- end
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)
end

function WeeklyChallengeLayer:init_today_reward()
    local player = Player:getInstance()
    local panel_day_reward = self.image_dia:getChildByName("panel_day_reward")
    self.conch_once_num_text = panel_day_reward:getChildByName("text_num")
    local text_text = panel_day_reward:getChildByName("text_text")
    text_text:setString("单次收集")
    local day_reward_page_view = panel_day_reward:getChildByName("reward")
    day_reward_page_view:setBounceEnabled(true)
    day_reward_page_view:setCurrentPageIndex(0)

    local day_left_image = panel_day_reward:getChildByName("button_left")
    local day_left_Btn = ButtonWrap:create()
    day_left_Btn:wrap(day_left_image, 1)
    day_left_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        local index = day_reward_page_view:getCurrentPageIndex()
        if index <= 0 then
            index = get_table_length(self.today_reward_info_vector)
        end
        day_reward_page_view:scrollToPage(index-1)
    end)

    local day_right_image = panel_day_reward:getChildByName("button_right")
    local day_right_Btn = ButtonWrap:create()
    day_right_Btn:wrap(day_right_image, 1)
    day_right_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        local index = day_reward_page_view:getCurrentPageIndex()
        if index >= (get_table_length(self.today_reward_info_vector)-1) then
            index = -1
        end
        day_reward_page_view:scrollToPage(index+1)
    end)

    local function today_pageViewEvent(sender, eventType)
        if eventType == ccui.PageViewEventType.turning then
            local index = day_reward_page_view:getCurrentPageIndex()
            self.conch_once_num_text:setString(self.today_reward_info_vector[index+1].target_num)
        end
        
    end
    day_reward_page_view:addEventListener(today_pageViewEvent)
    
    local panel_reward = day_reward_page_view:getChildByName("panel_reward")
    day_reward_page_view:removeAllPages()

    self.conch_once_num_text:setString(self.today_reward_info_vector[1].target_num)

    self.today_button_get_image = {}
    local count_vector = get_table_length(self.today_reward_info_vector)
    for i=1,count_vector do
        day_reward_page_view:addPage(panel_reward:clone())
    end
    local is_all_get = true
    for i=1,count_vector do
        local page_item = day_reward_page_view:getItem(i-1)
        local image_reward = page_item:getChildByName("image_reward")
        local reward_num = image_reward:getChildByName("text_num")
        local image_get = image_reward:getChildByName("image_get")
        image_get:setVisible(false)
        local button_get_image = image_reward:getChildByName("button_get")
        self.today_button_get_image[i] = button_get_image
        button_get_image:setVisible(false)
        local today_reward_info = self.today_reward_info_vector[i]
        local get_Btn = ButtonWrap:create()
        get_Btn:wrap(button_get_image, 1)
        get_Btn:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
            local item_id = today_reward_info.reward_item.item_id
            local item_num = today_reward_info.reward_item.item_num
            player:add_player_bag_info_by_id(item_id,item_num)

            if item_id == "item_01" or item_id == "item_02" or item_id == "item_03" then
                if item_id == "item_03" then
                    TalkingDataManager:getInstance():sendOnReward(item_num,"周赛每日奖励")
                end
            else
                local bag_info = Bag_Data:getInstance():get_bag_info_by_id(item_id)
                self.bag_reward:setItemTexture(bag_info.res_key)
                self.bag_reward:setTouchEnabled(true)
                self.bag_reward:runAction()
            end

            get_Btn:disable()
            image_get:setVisible(true)
            button_get_image:setVisible(false)

            player:set_player_weekly_challenge_today_reward_info_is_get_by_id(today_reward_info.today_reward_id, true)

            --滚动奖励表
            for j=1,#self.today_reward_info_vector do
                local next_index = j + 1
                if j >= #self.today_reward_info_vector then
                    next_index = 1
                end
                local is_get = player:get_player_weekly_challenge_today_reward_info_is_get_by_id(self.today_reward_info_vector[next_index].today_reward_id)
                if self.conch_once_num >= self.today_reward_info_vector[next_index].target_num and is_get == false then
                    day_reward_page_view:scrollToPage(next_index-1)
                    break
                elseif self.conch_once_num < self.today_reward_info_vector[next_index].target_num and is_get == false then
                    local is_have_not_get = false
                    for k=1,count_vector do
                        if self.conch_once_num >= self.today_reward_info_vector[k].target_num and self.player_today_reward_info[k].is_get == false then
                            is_have_not_get = true
                            break
                        end
                    end
                    if not is_have_not_get then
                        day_reward_page_view:scrollToPage(next_index-1)
                        break
                    end
                end
            end

        end)
        
        reward_num:setString(string.format("x%d", today_reward_info.reward_item.item_num))
        local reward_pic = ccui.ImageView:create(today_reward_info.res_key)
        image_reward:addChild(reward_pic,-1)

        if self.player_today_reward_info[i].is_get then
            get_Btn:disable()
            image_get:setVisible(true)
            button_get_image:setVisible(false)
            --滚动奖励表
            local next_index = i + 1
            if i == count_vector then
                next_index = 1
            end
            local is_get = player:get_player_weekly_challenge_today_reward_info_is_get_by_id(self.today_reward_info_vector[next_index].today_reward_id)
            if self.conch_once_num >= self.today_reward_info_vector[next_index].target_num and is_get == false then
                performWithDelay(self, function() day_reward_page_view:scrollToPage(next_index-1) end, 0.016)
            elseif self.conch_once_num < self.today_reward_info_vector[next_index].target_num and is_get == false then
                local is_have_not_get = false
                for j=1,count_vector do
                    if self.conch_once_num >= self.today_reward_info_vector[j].target_num and self.player_today_reward_info[j].is_get == false then
                        is_have_not_get = true
                        break
                    end
                end
                if not is_have_not_get then
                    performWithDelay(self, function() day_reward_page_view:scrollToPage(next_index-1) end, 0.016)
                end
            end
        else
            is_all_get = false
        end
    end
    if is_all_get then
        performWithDelay(self, function() day_reward_page_view:scrollToPage(count_vector-1) end, 0.016)
    end
end

function WeeklyChallengeLayer:init_week_reward()
    local player = Player:getInstance()
    local panel_week_reward = self.image_dia:getChildByName("panel_week_reward")
    self.conch_count_text = panel_week_reward:getChildByName("text_num")

    local week_reward_page_view = panel_week_reward:getChildByName("reward")
    week_reward_page_view:setBounceEnabled(true)
    week_reward_page_view:setCurrentPageIndex(0)

    local box_des = panel_week_reward:getChildByName("box_des")
    self.box_des = box_des
    box_des:setVisible(false)
    -- for i=1,3 do
    --     local panel_ = box_des:getChildByName(string.format("panel_%d",i))
    --     panel_:setVisible(false)
    -- end

    local week_left_image = panel_week_reward:getChildByName("button_left")
    local week_left_Btn = ButtonWrap:create()
    week_left_Btn:wrap(week_left_image, 1)
    week_left_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        local index = week_reward_page_view:getCurrentPageIndex()
        if index <= 0 then
            index = get_table_length(self.week_reward_info_vector)
        end
        -- box_des:setVisible(false)
        week_reward_page_view:scrollToPage(index-1)
    end)

    local week_right_image = panel_week_reward:getChildByName("button_right")
    local week_right_Btn = ButtonWrap:create()
    week_right_Btn:wrap(week_right_image, 1)
    week_right_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        local index = week_reward_page_view:getCurrentPageIndex()
        if index >= (get_table_length(self.week_reward_info_vector)-1) then
            index = -1
        end
        -- box_des:setVisible(false)
        week_reward_page_view:scrollToPage(index+1)
    end)
    
    self.conch_count_text:setString(self.week_reward_info_vector[1].target_num)    

    local function week_pageViewEvent(sender, eventType)
        if eventType == ccui.PageViewEventType.turning then
            local index = week_reward_page_view:getCurrentPageIndex()
            self.conch_count_text:setString(self.week_reward_info_vector[index+1].target_num)
            -- if sender:getTouchBeganPosition().x ~= sender:getTouchEndPosition().x then
            --     self.box_des:setVisible(false)
            -- end
        end
    end
    week_reward_page_view:addEventListener(week_pageViewEvent)
    
    local panel_reward = week_reward_page_view:getChildByName("panel_reward")
    week_reward_page_view:removeAllPages()

    self.week_button_get_image = {}
    local count_vector = get_table_length(self.week_reward_info_vector)
    for i=1,count_vector do
        week_reward_page_view:addPage(panel_reward:clone())
    end
    local is_all_get = true
    for i=1,count_vector do
        local page_item = week_reward_page_view:getItem(i-1)
        local image_reward = page_item:getChildByName("image_reward")
        local image_get = image_reward:getChildByName("image_get")
        image_get:setVisible(false)

        local week_reward_info = self.week_reward_info_vector[i]

        local reward_pic = ccui.ImageView:create(week_reward_info.res_key)
        image_reward:addChild(reward_pic,-1)
        local reward_Btn = ButtonWrap:create()
        reward_Btn:wrap(reward_pic, 1)
        reward_Btn:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)

            local commonDialogLayer = CommonDialogLayer:create()
            commonDialogLayer:show_on_left(week_reward_info.chest_item)
            commonDialogLayer:setPosition(reward_pic:getParent():convertToWorldSpace(cc.p(reward_pic:getPosition())))
            
            -- box_des:setVisible(true)
            -- for j=1,3 do
            --     local panel_ = box_des:getChildByName(string.format("panel_%d",j))
            --     panel_:setVisible(false)
            -- end
            -- local count = get_table_length(week_reward_info.chest_item)
            -- local chest_panel = box_des:getChildByName(string.format("panel_%d",count-1))
            -- chest_panel:setVisible(true)
            -- for j=1,count do
            --     local item = chest_panel:getChildByName(string.format("item_%d",j))
            --     local text_num = item:getChildByName("text_num")
            --     text_num:setString(string.format("x%d",week_reward_info.chest_item[j].item_num))
            --     local bag_info = Bag_Data:getInstance():get_bag_info_by_id(week_reward_info.chest_item[j].item_id)
                
            --     if week_reward_info.chest_item[j].item_id == "item_02" then
            --         item:loadTexture("images/item/item_8.png")
            --     elseif week_reward_info.chest_item[j].item_id == "item_03" then
            --         item:loadTexture("images/item/item_9.png")
            --     else
            --         item:loadTexture(bag_info.res_key)
            --     end
            -- end
        end)

        local button_get_image = image_reward:getChildByName("button_get")
        self.week_button_get_image[i] = button_get_image
        button_get_image:setVisible(false)
        local get_Btn = ButtonWrap:create()
        get_Btn:wrap(button_get_image, 1)
        get_Btn:addTouchEndCallback(function()
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
            
            local count = get_table_length(week_reward_info.chest_item)
            for i=1,count do
                local item_id = week_reward_info.chest_item[i].item_id
                local item_num = week_reward_info.chest_item[i].item_num
                player:add_player_bag_info_by_id(item_id, item_num)

                if item_id == "item_01" or item_id == "item_02" or item_id == "item_03" then
                    if item_id == "item_03" then
                        TalkingDataManager:getInstance():sendOnReward(item_num,"周赛每周奖励")
                    end
                else
                    local bag_info = Bag_Data:getInstance():get_bag_info_by_id(item_id)
                    self.bag_reward:setItemTexture(bag_info.res_key)
                    self.bag_reward:setTouchEnabled(true)
                    self.bag_reward:runAction()
                end
            end

            reward_Btn:disable()
            image_get:setVisible(true)
            button_get_image:setVisible(false)

            player:set_player_weekly_challenge_week_reward_info_is_get_by_id(week_reward_info.week_reward_id, true)

            --滚动奖励表
            for j=1,#self.week_reward_info_vector do
                local next_index = j + 1
                if j >= #self.week_reward_info_vector then
                    next_index = 1
                end
                local is_get = player:get_player_weekly_challenge_week_reward_info_is_get_by_id(self.week_reward_info_vector[next_index].week_reward_id)
                if self.conch_count >= self.week_reward_info_vector[next_index].target_num and is_get == false then
                    week_reward_page_view:scrollToPage(next_index-1)
                    break
                elseif self.conch_count < self.week_reward_info_vector[next_index].target_num and is_get == false then
                    local is_have_not_get = false
                    for k=1,count_vector do
                        if self.conch_count >= self.week_reward_info_vector[k].target_num and self.player_week_reward_info[k].is_get == false then
                            is_have_not_get = true
                            break
                        end
                    end
                    if not is_have_not_get then
                        week_reward_page_view:scrollToPage(next_index-1)
                        break
                    end
                end
            end
        end)

        if self.player_week_reward_info[i].is_get then
            reward_Btn:disable()
            image_get:setVisible(true)
            button_get_image:setVisible(false)
            --滚动奖励表
            local next_index = i + 1
            if i == count_vector then
                next_index = 1
            end
            local is_get = player:get_player_weekly_challenge_week_reward_info_is_get_by_id(self.week_reward_info_vector[next_index].week_reward_id)
            if self.conch_count >= self.week_reward_info_vector[next_index].target_num and is_get == false then
                performWithDelay(self, function() week_reward_page_view:scrollToPage(next_index-1) end, 0.016)
            elseif self.conch_count < self.week_reward_info_vector[next_index].target_num and is_get == false then
                local is_have_not_get = false
                for j=1,count_vector do
                    if self.conch_count >= self.week_reward_info_vector[j].target_num and self.player_week_reward_info[j].is_get == false then
                        is_have_not_get = true
                        break
                    end
                end
                if not is_have_not_get then
                    performWithDelay(self, function() week_reward_page_view:scrollToPage(next_index-1) end, 0.016)
                end
            end
        else
            is_all_get = false
        end
    end
    if is_all_get then
        performWithDelay(self, function() week_reward_page_view:scrollToPage(count_vector-1) end, 0.016)
    end
end

function WeeklyChallengeLayer:init_rank()
    local image_rank = self.node:getChildByName("image_rank")
    self.my_conch_count_text = image_rank:getChildByName("text_collect")
    --self.my_conch_count_text:setString("我的收集：999个")

    local node = image_rank:getChildByName("node")
    local rank_friends = node:getChildByName("rank_friends")
    local rank_world = node:getChildByName("rank_world")
    rank_friends:setTouchEnabled(false)
    rank_world:setTouchEnabled(false)

    local touchBeginPosY = nil
    local function touchEventCallback(sender,eventType)
        if eventType == ccui.TouchEventType.began then
            touchBeginPosY = sender:getTouchBeganPosition().y
            return true
        elseif eventType == ccui.TouchEventType.moved then
            local moveY = sender:getPositionY() + sender:getTouchMovePosition().y - touchBeginPosY
            --moveY ∈ (204, 685)
            if moveY > 685 then
                moveY = 685
            elseif moveY < 204 then
                moveY = 204
            end
            sender:setPositionY(moveY)
            touchBeginPosY = sender:getTouchMovePosition().y
        elseif eventType == ccui.TouchEventType.ended then
            local endY = sender:getTouchEndPosition().y - sender:getTouchBeganPosition().y
            if endY > 30 or (endY < 0 and endY > -30) then
                sender:runAction(cc.MoveTo:create(0.2, cc.p(sender:getPositionX(), 685)))
                rank_friends:setTouchEnabled(true)
                rank_world:setTouchEnabled(true)
            elseif endY < -30 or (endY > 0 and endY < 30) then
                sender:runAction(cc.MoveTo:create(0.2, cc.p(sender:getPositionX(), 204)))
                rank_friends:setTouchEnabled(false)
                rank_world:setTouchEnabled(false)
            end
        end
    end
    image_rank:addTouchEventListener(touchEventCallback)
    

    local friends_image = image_rank:getChildByName("button_friends")
    local text_friends = friends_image:getChildByName("text_friends")
    local world_image = image_rank:getChildByName("button_world")
    local text_world = world_image:getChildByName("text_world")

    rank_world:setVisible(false)
    friends_image:loadTexture("images/weekgame/btn_checked.png")
    text_friends:setTextColor(cc.c4b(255,255,255,255))
    text_friends:enableOutline(cc.c4b(162,61,11,255))
    world_image:loadTexture("images/weekgame/btn_unchecked.png")
    text_world:setTextColor(cc.c4b(74,203,164,255))
    text_world:enableOutline(cc.c4b(4,100,71,255))

    local friends_Btn = ButtonWrap:create()
    friends_Btn:wrap(friends_image, 1)
    friends_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        rank_friends:setVisible(true)
        rank_world:setVisible(false)

        friends_image:loadTexture("images/weekgame/btn_checked.png")
        text_friends:setTextColor(cc.c4b(255,255,255,255))
        text_friends:enableOutline(cc.c4b(162,61,11,255))
        world_image:loadTexture("images/weekgame/btn_unchecked.png")
        text_world:setTextColor(cc.c4b(74,203,164,255))
        text_world:enableOutline(cc.c4b(4,100,71,255))
    end)

    local world_Btn = ButtonWrap:create()
    world_Btn:wrap(world_image, 1)
    world_Btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)
        rank_friends:setVisible(false)
        rank_world:setVisible(true)

        world_image:loadTexture("images/weekgame/btn_checked.png")
        text_world:setTextColor(cc.c4b(255,255,255,255))
        text_world:enableOutline(cc.c4b(162,61,11,255))
        friends_image:loadTexture("images/weekgame/btn_unchecked.png")
        text_friends:setTextColor(cc.c4b(74,203,164,255))
        text_friends:enableOutline(cc.c4b(4,100,71,255))

        if not self.is_request_ranklist then
            self.is_request_ranklist = true
            self:request_getRanklist(self.world_async_callback)
        end
    end)
    self.is_request_ranklist = false

    local panel_1 = rank_friends:getChildByName("panel_1")
    --rank_friends:setScrollBarEnabled(false)
    rank_friends:setItemModel(panel_1:clone())
    rank_friends:removeAllItems()
    local function friend_async_callback(list)
        local friendRanklist = list
        local count = get_table_length(friendRanklist)
        for i=1,count do
            rank_friends:pushBackDefaultItem()
        end
        for i=1,count do
            local item = rank_friends:getItem(i -1)
            local image_head = item:getChildByName("image_id")
            local text_name = item:getChildByName("text_name")
            local text_num = item:getChildByName("text_num")
            local text_rank_num = item:getChildByName("text_rank_num")

            local friendRank_info = friendRanklist[i]
            text_rank_num:setString(friendRank_info.rank)
            text_name:setString(friendRank_info.name)
            text_num:setString(friendRank_info.score)

            if friendRank_info.sex == "0" and friendRank_info.headUrl and friendRank_info.headUrl ~= "" then
                
                local function on_create_finish()
                    local headPath = FileManipulator:getInstance():getPath().."head_cache/"..friendRank_info.guid.."head_00"
                    if cc.FileUtils:getInstance():isFileExist(headPath) then
                        image_head:loadTexture(headPath)
                    end
                end
                global_create_social_head(friendRank_info.guid, friendRank_info.headUrl, on_create_finish)

            else
                local head_id = string.format("head_%02d", friendRank_info.sex+1)
                local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                image_head:loadTexture(head_info.res_key)
            end

            -- local head_id = string.format("head_%02d", friendRank_info.sex+1)
            -- local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
            -- image_head:loadTexture(head_info.res_key)

            -- if friendRank_info.guid == PLAYER_ID then
            --     local player_head_id = Player:getInstance():get_player_head_id()
            --     if player_head_id == "head_00" then
            --         local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
            --         image_head:loadTexture(headPath)
            --     end
            -- end
        end

        performWithDelay(self,function() 
            local size = rank_friends:getInnerContainerSize()
            rank_friends:setInnerContainerSize(cc.size(size.width,size.height+100))
            end, 0.016)
    end

    self:request_getfriendRanklist(friend_async_callback)

    local rank_bar = rank_world:getChildByName("rank_bar")
    local bar = rank_bar:getChildByName("bar")
    local panel_1 = rank_world:getChildByName("panel_1")
    --rank_world:setScrollBarEnabled(false)
    rank_world:setItemModel(panel_1:clone())
    rank_world:removeLastItem()
    -- bar:setVisible(false)
    for i=1,3 do
        local bar_panel = bar:getChildByName(string.format("panel_%d",i))
        bar_panel:setVisible(false)
    end

    local function world_async_callback(list)
        local worldRanklist = list
        -- if gezt_table_length(worldRanklist) >= 3 then
            local min = math.min(get_table_length(worldRanklist),3)
            for i=1,min do
                -- bar:setVisible(true)
                local bar_panel = bar:getChildByName(string.format("panel_%d",i))
                bar_panel:setVisible(true)
                local image_head = bar_panel:getChildByName("image_id")
                local text_name = bar_panel:getChildByName("text_name")
                local text_num = bar_panel:getChildByName("text_num")

                local worldRank_info = worldRanklist[i]
                text_name:setString(worldRank_info.name)
                text_num:setString(worldRank_info.score)

                if worldRank_info.sex == "0" and worldRank_info.headUrl and worldRank_info.headUrl ~= "" then
                
                    local function on_create_finish()
                        local headPath = FileManipulator:getInstance():getPath().."head_cache/"..worldRank_info.guid.."head_00"
                        if cc.FileUtils:getInstance():isFileExist(headPath) then
                            image_head:loadTexture(headPath)
                        end
                    end
                    global_create_social_head(worldRank_info.guid, worldRank_info.headUrl, on_create_finish)

                else
                    local head_id = string.format("head_%02d", worldRank_info.sex+1)
                    local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                    image_head:loadTexture(head_info.res_key)
                end

                -- local head_id = string.format("head_%02d", worldRank_info.sex+1)
                -- local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                -- image_head:loadTexture(head_info.res_key)

                -- if worldRank_info.guid == PLAYER_ID then
                --     local player_head_id = Player:getInstance():get_player_head_id()
                --     if player_head_id == "head_00" then
                --         local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
                --         image_head:loadTexture(headPath)
                --     end
                -- end
            end
        -- end

        local count = get_table_length(worldRanklist) - 3
        for i=1,count do
            rank_world:pushBackDefaultItem()
        end

        for i=1,count do
            local item = rank_world:getItem(i)
            local image_head = item:getChildByName("image_id")
            local text_name = item:getChildByName("text_name")
            local text_num = item:getChildByName("text_num")
            local text_rank_num = item:getChildByName("text_rank_num")

            local worldRank_info = worldRanklist[i+3]

            text_rank_num:setString(worldRank_info.rank)
            text_name:setString(worldRank_info.name)
            text_num:setString(worldRank_info.score)

            if worldRank_info.sex == "0" and worldRank_info.headUrl and worldRank_info.headUrl ~= "" then
                
                local function on_create_finish()
                    local headPath = FileManipulator:getInstance():getPath().."head_cache/"..worldRank_info.guid.."head_00"
                    if cc.FileUtils:getInstance():isFileExist(headPath) then
                        image_head:loadTexture(headPath)
                    end
                end
                global_create_social_head(worldRank_info.guid, worldRank_info.headUrl, on_create_finish)

            else
                local head_id = string.format("head_%02d", worldRank_info.sex+1)
                local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
                image_head:loadTexture(head_info.res_key)
            end

            -- local head_id = string.format("head_%02d", worldRank_info.sex+1)
            -- local head_info = Head_Data:getInstance():get_head_info_by_id(head_id)
            -- image_head:loadTexture(head_info.res_key)

            -- if worldRank_info.guid == PLAYER_ID then
            --     local player_head_id = Player:getInstance():get_player_head_id()
            --     if player_head_id == "head_00" then
            --         local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
            --         image_head:loadTexture(headPath)
            --     end
            -- end
        end

        --占坑位
        for i=1,2 do
            local index = get_table_length(worldRanklist)
            if index>=3 then
                rank_world:pushBackDefaultItem()
                rank_world:getItem(i+index-3):setVisible(false)
            end
        end
    end
    
    self.world_async_callback = world_async_callback
    -- self:request_getRanklist(world_async_callback)
end

function WeeklyChallengeLayer:update_date()
    local player = Player:getInstance()
    self.today_reward_info_vector = require "script/today_reward_info.lua"
    self.week_reward_info_vector = require "script/week_reward_info.lua"

    local lase_challenge_day = player:get_player_weekly_challenge_lase_challenge_day()
    local lase_challenge_week = player:get_player_weekly_challenge_lase_challenge_week()

    local tm = os.date("*t", self.timeStamp)
    local week = os.date("%W", self.timeStamp)

    --每天更新
    local diff_day = tm.yday - lase_challenge_day
    if tm.yday == 1 and lase_challenge_day ~= 1 then
        diff_day = 1
    end
    if diff_day > 0 then
        player:set_player_weekly_challenge_lase_challenge_day(tm.yday)
        -- player:set_player_weekly_challenge_today_challenge_num(2)
        player:set_player_weekly_challenge_go_to_play_times(0)
        player:set_player_weekly_challenge_add_friend_times(0)
        player:set_player_weekly_challenge_conch_once_num(0)
        
        local today_reward_info_vector = self.today_reward_info_vector
        local today_reward_info = {}
        for i=1,#today_reward_info_vector do
            today_reward_info[i] = { 
                today_reward_id = today_reward_info_vector[i].today_reward_id,
                is_get = false,
            }
        end
        player:set_player_weekly_challenge_today_reward_info(today_reward_info)        
    end

    --每周更新
    week = week - 0
    local diff_week = week - lase_challenge_week
    if week == 1 and lase_challenge_week ~= 1 then
        diff_week = 1
    end
    if diff_week > 0 then
        player:set_player_weekly_challenge_lase_challenge_week(week)
        player:set_player_weekly_challenge_today_challenge_num(2)
        player:set_player_weekly_challenge_conch_count(0)
        
        local week_reward_info_vector = self.week_reward_info_vector
        local week_reward_info = {}
        for i=1,#week_reward_info_vector do
            week_reward_info[i] = {
                week_reward_id = week_reward_info_vector[i].week_reward_id,
                is_get = false,
            }
        end
        player:set_player_weekly_challenge_week_reward_info(week_reward_info)
    end

    self.today_challenge_num = player:get_player_weekly_challenge_today_challenge_num()
    self.conch_count = player:get_player_weekly_challenge_conch_count()
    self.conch_once_num = player:get_player_weekly_challenge_conch_once_num()
    self.player_today_reward_info = player:get_player_weekly_challenge_today_reward_info()
    self.player_week_reward_info = player:get_player_weekly_challenge_week_reward_info()
end

function WeeklyChallengeLayer:update_reward()
	local player = Player:getInstance()

    --判断self.conch_once_num、self.conch_count 数量
    for i=1,#self.today_reward_info_vector do
        if self.conch_once_num >= self.today_reward_info_vector[i].target_num then
            if self.player_today_reward_info[i].is_get == false then
                self.today_button_get_image[i]:setVisible(true)
            end            
        end
    end
    for i=1,#self.week_reward_info_vector do
        if self.conch_count >= self.week_reward_info_vector[i].target_num then
            if self.player_week_reward_info[i].is_get == false then
                self.week_button_get_image[i]:setVisible(true)
            end
        end
    end

    self.my_conch_count_text:setString(string.format("我的收集：%d个",self.conch_count))
    if self.today_challenge_num > 0 then
        self.today_challenge_num_text:setString(self.today_challenge_num)
    else
        self.text_start:setString("获得机会")
        self.image_mask:setVisible(false)
    end    

end

function WeeklyChallengeLayer:add_challenge_num_callback()
    self.today_challenge_num = Player:getInstance():get_player_weekly_challenge_today_challenge_num()
    if self.today_challenge_num > 0 then
        self.today_challenge_num_text:setString(self.today_challenge_num)
        self.text_start:setString("马上挑战")
        self.image_mask:setVisible(true)
    else
        self.text_start:setString("获得机会")
        self.image_mask:setVisible(false)
    end
end

function WeeklyChallengeLayer:get_electronic_clock()
    local tm = os.date("*t")
    local wday = tm.wday - 1
    if wday==0 then
        wday = 7
    end
    local day =7-wday

    local diff = tm.hour*3600+tm.min*60+tm.sec
    local hour = math.floor((24*3600-diff)/3600)
    local min = math.floor(((24*3600-diff)%3600)/60)
    local sec = math.floor(((24*3600-diff)%3600)%60)

	return string.format("%d天%02d小时%02d分钟%02d秒后结束", day, hour, min, sec), string.format("%02d:%02d:%02d", hour, min, sec)
end

function WeeklyChallengeLayer:set_close_callback(callback)
    self.close_callback = callback
end

function WeeklyChallengeLayer:close_self()
    if self.close_callback then
        self.close_callback()
    end
    self:addChild(MaskLayer:create(), 5)
    LayerManager:getInstance():remove_top_layer()

    self:removeFromParent()
end

function WeeklyChallengeLayer:request_getfriendRanklist(async_callback)
    --request getfriendRanklist 好友排行
    local rpc_param = {

        PLAYER_ID,

    }

    RPC:call_bind("SHIN/seaVenture/getfriendRanklist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            -- dump(response)
            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function WeeklyChallengeLayer:request_getRanklist(async_callback)
    --request getRanklist 世界排行
    local rpc_param = {

    }

    RPC:call_bind("SHIN/seaVentureMgr/showRanklist", rpc_param, self, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            if response == nil then
                response = {}
            end
            async_callback(response)

        else

            print("request error")

        end

    end)
end

function WeeklyChallengeLayer:onEnter()
    RPC:register_target(self)
end

function WeeklyChallengeLayer:onExit()
    if self.scheduleScriptEntryID then
        cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.scheduleScriptEntryID)
    end
    RPC:unregister_target(self)
end

return WeeklyChallengeLayer