require "core.LayerManager"
require "UI.Widget.MaskLayer"
require "core.TalkingDataManager"
require "core.Wheel_Reward_Data"

WheelLayerEx = class("WheelLayerEx", function() return cc.Layer:create() end)

function WheelLayerEx:ctor(timeStamp)

    self:append_studio_node()
    self:install_light()
    self:install_stick()
    self:wrap_button()
    self:set_slot_number(8)
    self:set_overload_window(11)
    self:change_button_mode(WheelLayerEx.BUTTON_MODE.FREE)
    self:play_button_animation()

    self.timeStamp = timeStamp
    self:start()
    self:init_data()

end

function WheelLayerEx:append_studio_node()

    local widget = cc.CSLoader:createNode("luckyroller.csb")
    self:addChild(widget, 1);
    self.widget = widget:getChildByName("node")

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    --商店动画
    self.action = cc.CSLoader:createTimeline("luckyroller.csb")
    widget:runAction(self.action)

    self.action:setCurrentFrame(0)
    self.action:play("luckyroller_in", false)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "luckyroller_in_end" then

        elseif str == "luckyroller_out_end" then
            self:removeFromParent()

        end
    end
    self.action:setFrameEventCallFunc(onFrameEvent)

    self.roulette = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("image_rotate")
    self.pointer = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("image_pointer")
    self.pointer:setLocalZOrder(5)
    self.pointer:setAnchorPoint(cc.p(0.5, 0.69))

    local px, py = self.pointer:getPosition()
    self.pointer:setPosition(cc.p(px, py - 20))

    local  listenner = cc.EventListenerTouchOneByOne:create()
    listenner:setSwallowTouches(true)
    listenner:registerScriptHandler(function(touch, event)
        return true
    end, cc.Handler.EVENT_TOUCH_BEGAN )
    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listenner, self)


end

function WheelLayerEx:init_data()
    self.luck_free_callback = nil

    local wheel_reward_info_vector = Wheel_Reward_Data:getInstance():get_wheel_reward_info_vector()
    self.wheel_reward_info_vector = wheel_reward_info_vector
    local count = get_table_length(wheel_reward_info_vector)
    -- for i=1,count do
    --     print(wheel_reward_info_vector[i].odds)
    -- end

    local item_table = {}
    for i=1,count do
        table.insert(item_table, {img = wheel_reward_info_vector[i].img,num = wheel_reward_info_vector[i].num})
    end

    self:set_item(item_table)

    local player = Player:getInstance()
    local last_wheel_day = player:get_last_wheel_day()

    local tm = os.date("*t", self.timeStamp)

    local diff = tm.yday - last_wheel_day
    if tm.yday == 1 and last_wheel_day ~= 1 then
        diff = 1
    end
    if diff>0 then
        player:set_today_is_wheel(false)
        player:set_num_wheel(0)
    end
    local today_is_wheel = player:get_today_is_wheel()
    local num_wheel = player:get_num_wheel()

    local wheel_need_jewel_info = require "script/wheel_need_jewel_info.lua"
    self.wheel_need_jewel_info = wheel_need_jewel_info
    local vector_count = get_table_length(wheel_need_jewel_info)

    if today_is_wheel then
        --10.8 不限次数
        -- if num_wheel >= 15 then
        --     num_wheel = 15
        -- end

        if num_wheel >= vector_count then
            self:change_button_mode(WheelLayerEx.BUTTON_MODE.DISABLE)
        else
            
            self:change_button_mode(WheelLayerEx.BUTTON_MODE.PAY)
            self:set_rotate_price(wheel_need_jewel_info[num_wheel], "LUCKY_JEWEL")
        end
    end

    --bag_get_reward
    self.__c_mask_layer = MaskLayer:create()
    self.__c_mask_layer:init_background(cc.c4b(0, 0, 0, 120))
    self.__c_mask_layer:setEnabled(false)
    self:addChild(self.__c_mask_layer, 2)
    self.__c_mask_layer:setVisible(false)

    self.bag_reward = BagRewardWidget:create()
    self:addChild(self.bag_reward, 5)

    local text = self.widget:getChildByName("image_dia"):getChildByName("image_dia_2"):getChildByName("text_title")
    text:setString(ResourceManager:getInstance():getTextStringByKey("WheelLayerEx_1"))
end

function WheelLayerEx:set_luck_free_callback(callback)
    self.luck_free_callback = callback
end

function WheelLayerEx:is_currency_enought()
    local player = Player:getInstance()
    local num_wheel = player:get_num_wheel()
    if player:get_player_jewel() >= self.wheel_need_jewel_info[num_wheel] then
        player:add_player_jewel(-self.wheel_need_jewel_info[num_wheel])
        if self.wheel_need_jewel_info[num_wheel] > 0 then
            TalkingDataManager:getInstance():sendOnPurchase("幸运大转盘购买次数",1,self.wheel_need_jewel_info[num_wheel])
            TalkingDataManager:getInstance():send("大转盘", {["玩家累计使用钻石转转盘次数"] = "次数"})
        end
        TalkingDataManager:getInstance():send("大转盘", {["玩家累计总转转盘次数"] = "总数"})

        return true
    else
        -- local shopLayer = ShopLayer:create()
        -- shopLayer:show_goods_by_type(ShopLayer.SHOP_TYPE.JEWEL)
        -- LayerManager:getInstance():add_layer_to_scene(shopLayer)
        --113
        local SpecialGiftLayer = SpecialGiftLayer:create(SpecialGiftLayer.Gift_Type.Jewel)
        LayerManager:getInstance():add_layer_to_scene(SpecialGiftLayer)
        
        local warningLayer = WarningLayer:create()
        warningLayer:set_text_string(ResourceManager:getInstance():getTextStringByKey("needJewel"))
        LayerManager:getInstance():add_layer_to_scene(warningLayer)
        return false
    end

end

function WheelLayerEx:get_random_num()

    math.randomseed(os.time())
    local random_num = math.random()
    print(random_num)

    local index = nil
    local count = get_table_length(self.wheel_reward_info_vector)
    local item_id = nil
    for i=1,count do
        if random_num <= self.wheel_reward_info_vector[i].odds then                
            index = i
            break
        end
    end
    return index
end

function WheelLayerEx:install_light()

    local light_frame = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1")
    local origin = cc.p(456, 458)
    local radius = 385

    self.light_number = 48
    self.rot_interval = 360 / self.light_number
    self.lights = {}

    for i = 1, self.light_number do

        local light_bg = ResourceManager:getInstance():getSpriteByKey("LUCKY_LIGHT_BG")
        light_bg:setScale(0.8)
        light_frame:addChild(light_bg, 2)

        local light = ResourceManager:getInstance():getSpineByKey("SPINE_ROULETTE_LIGHT", 0.8)
        light:setAnimation(0, "quick_yellow", false)
        light_frame:addChild(light, 2)

        local step = 360 / self.light_number
        local rotation = step * (i)
        local offset_x = math.cos(math.rad(rotation)) * radius
        local offset_y = math.sin(math.rad(rotation)) * radius
        light:setPosition(cc.p(origin.x + offset_x, origin.y + offset_y))
        light_bg:setPosition(cc.p(origin.x + offset_x, origin.y + offset_y))

        self.lights[i] = light

    end

end

function WheelLayerEx:install_stick()

    for i = 1, 8 do

        local light_bg = ResourceManager:getInstance():getSpriteByKey("LUCKY_LIGHT_STICK")
        local rotation = -45 * (i) + 22.5
        local origin = cc.p(370, 370)
        local offset_x = math.cos(math.rad(rotation)) * 361
        local offset_y = math.sin(math.rad(rotation)) * 361

        light_bg:setPosition(cc.p(origin.x + offset_x, origin.y + offset_y))
        light_bg:setRotation(45 * i + 90 - 22.5)

        self.roulette:addChild(light_bg, 2)

    end

end



function WheelLayerEx:wrap_button()

    local button_ok = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("button_rotate")
    local button_close = self.widget:getChildByName("image_dia"):getChildByName("button_close")

    local ok_wrap = ButtonWrap:create()
    ok_wrap:wrap(button_ok, 1)
    ok_wrap:addTouchEndCallback(function()
        if self:is_currency_enought() then

            --每日任务事件发送
            local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
            event.param = {task_id = "task_05", finish_count = 1, func = function() end }
            cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

            self.wrap_close:setDarkButton(false)

            button_ok:getChildByName("panel_free"):setVisible(false)
            button_ok:getChildByName("panel_price"):setVisible(false)
        
            MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click2_Audio"), false)

            local index = self:get_random_num()
            local item_id = self.wheel_reward_info_vector[index].item_id
            local num = self.wheel_reward_info_vector[index].num

            local player = Player:getInstance()
            local tm = os.date("*t", self.timeStamp)
            local num_wheel = player:get_num_wheel()
            num_wheel = num_wheel + 1

            --10.8 不限次数
            -- if num_wheel >= 15 then
            --     num_wheel = 15
            -- end

            player:set_last_wheel_day(tm.yday)
            player:set_today_is_wheel(true)
            player:set_num_wheel(num_wheel)

            TalkingDataManager:getInstance():send("大转盘", {["各奖励获取的次数"] = self.wheel_reward_info_vector[index].reward_text})

            if item_id == "item_01" or item_id == "item_02" or item_id == "item_03" then
                if item_id == "item_03" then
                    player:add_player_jewel_without_change_top_bar(num)
                    TalkingDataManager:getInstance():sendOnReward(num,"幸运大转盘获得")
                elseif item_id == "item_02" then 
                    player:add_player_gold_without_change_top_bar(num)
                end
            else
                player:add_player_bag_info_by_id(item_id, num)
            end
            
            print("index::"..index)
            self:rotate(index, function()
                self.wrap_close:setDarkButton(true)

                if item_id == "item_01" or item_id == "item_02" or item_id == "item_03" then
                    --动画并刷新top_bar
                    player:add_player_bag_info_by_id(item_id, 0)
                else
                    --player:add_player_bag_info_by_id(item_id, num)
                    self.__c_mask_layer:setVisible(true)
                    local bag_info = Bag_Data:getInstance():get_bag_info_by_id(item_id)                    
                    self.bag_reward:setItemTexture(bag_info.res_key)
                    self.bag_reward:setTouchEnabled(true)
                    self.bag_reward:runAction()

                    self:runAction(cc.Sequence:create(cc.DelayTime:create(34 * 0.033), cc.CallFunc:create(function()
                        self.__c_mask_layer:setVisible(false)
                    end)))
                end

                -- 
                local vector_count = get_table_length(self.wheel_need_jewel_info)
                if num_wheel >= vector_count then
                    self:change_button_mode(WheelLayerEx.BUTTON_MODE.DISABLE)
                else
                    self:change_button_mode(WheelLayerEx.BUTTON_MODE.PAY)
                    self:set_rotate_price(self.wheel_need_jewel_info[num_wheel], "LUCKY_JEWEL")
                end

                --隐藏可以免费转动的红点提示
                if self.luck_free_callback then
                    self.luck_free_callback()
                    self.luck_free_callback = nil
                end

            end)

            

            self.ok_callback()
        end

    end)

    local close_wrap = ButtonWrap:create()
    close_wrap:wrap(button_close, 1)
    close_wrap:addTouchEndCallback(function()
        
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        self:close_self()
        
    end)

    self.wrap_ok = ok_wrap
    self.wrap_close = close_wrap
    self.ok_callback = function() end

end

function WheelLayerEx:close_self()
    self:addChild(MaskLayer:create(), 10)
    LayerManager:getInstance():remove_top_layer()

    self.action:play("luckyroller_out", false)

    --package.loaded["script.wheel_reward_info"] = nil

    -- self:runAction(cc.Sequence:create(cc.DelayTime:create(10 * 0.033), cc.CallFunc:create(function()

    --     self:removeFromParent()

    -- end)))

end

function WheelLayerEx:play_button_animation()

    self:stop_button_animation()
    elastic_action(self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("button_rotate"))

end

function WheelLayerEx:stop_button_animation()

    self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("button_rotate"):stopActionByTag(ELASTIC_ACTION_TAG)

end

function WheelLayerEx:get_index()

    return math.floor((self.roulette:getRotation() - self.__slot_step / 2 + 2.5) / self.__slot_step)

end

function WheelLayerEx:get_overload()

    return (self.roulette:getRotation() - self.__slot_step / 2 + 2.5) % self.__slot_step

end

function WheelLayerEx:get_pointer_overload()

    return -self.pointer:getRotation() / 3 > 0 and -self.pointer:getRotation() / 3 or 0

end

function WheelLayerEx:update_pointer(dt)

    local index = self:get_index() 
    local overload = self:get_overload()
    local pointer_overload = self:get_pointer_overload() < self.__overload_window and self:get_pointer_overload() or self.__overload_window

    if not (index == self.last_index) then

        self.last_overload = 0

    end

    if index >= self.last_index and self.last_overload <= self.last_pointer_overload and overload >= pointer_overload then

        if overload <= self.__overload_window then

            self.pointer:stopAllActions()
            self.pointer:setRotation(-overload * 3)

        else

            self.pointer:setRotation(-self.__overload_window * 3)
            
            if self.pointer:getActionByTag(WheelLayerEx.BACKWARD_ACTION_TAG) then

                self.pointer:stopAllActions()

                local rotate_1 = (cc.RotateTo:create(0.033, -35))
                local rotate_2 = (cc.RotateTo:create(0.033, -30))
                local sequence = cc.Sequence:create(rotate_1, rotate_2)

                self.pointer:runAction(sequence) 

            else

                local rotate_2 = cc.EaseSineInOut:create(cc.RotateBy:create(0.3, self.__overload_window * 3 + 5))
                local rotate_3 = cc.EaseSineInOut:create(cc.RotateBy:create(0.1, -7))
                local rotate_4 = cc.RotateBy:create(0.066, 2)
                local sequence = cc.Sequence:create(rotate_2, rotate_3, rotate_4)
                sequence:setTag(WheelLayerEx.BACKWARD_ACTION_TAG)

                self.pointer:runAction(sequence)   

            end           

        end

    elseif overload < self.__overload_window and overload < self.last_overload and pointer_overload > 0 then

        self.pointer:setRotation(-overload * 3)        

    end

    self.last_index = index
    self.last_overload = overload
    self.last_pointer_overload = self:get_pointer_overload()

end

function WheelLayerEx:rotate_scheduler(dt)

    self:update_pointer()

end

function WheelLayerEx:light_scheduler(dt)

    WheelLayerEx.LIGHT_CONTROL[self.__current_light_mode].update(self, dt)

end

function WheelLayerEx:reset_roulette()

    self.roulette:setRotation(360)
    self:reset_pointer()

end

function WheelLayerEx:reset_pointer()

    self.last_index = self:get_index()
    self.last_overload = self:get_overload()

    if self.last_overload < self.__overload_window then

        self.pointer:setRotation(-self.last_overload * 3)
        self.last_pointer_overload = self.last_overload

    else

        self.pointer:setRotation(0)
        self.last_pointer_overload = 0

    end    

end

function WheelLayerEx:change_light_mode(light_mode)

    self.__current_light_mode = self.__current_light_mode or WheelLayerEx.LIGHT_MODE.TERMINATE

    WheelLayerEx.LIGHT_CONTROL[self.__current_light_mode].stop(self)

    self.__current_light_mode = light_mode

    WheelLayerEx.LIGHT_CONTROL[self.__current_light_mode].start(self)

end

function WheelLayerEx:set_slot_number(slot_number)

    self.__slot_number = slot_number
    self.__slot_step = 360 / self.__slot_number

end

function WheelLayerEx:set_overload_window(window)

    self.__overload_window = window

end

function WheelLayerEx:reverse_overload(finish_callback)

    if self.last_overload and self.last_overload <= self.__overload_window then

        local overload_percent = self.last_overload / self.__overload_window
        local reverse = cc.EaseSineInOut:create(cc.RotateBy:create(6 * overload_percent, - self.last_overload - 5 * overload_percent))
        local call = cc.CallFunc:create(function() self:rotate_complete() end)
        self.roulette:runAction(cc.Sequence:create(reverse, call))

    else

        self:rotate_complete()

    end

end

function WheelLayerEx:rotate_complete()

    self:enable_button()
    self:change_light_mode(WheelLayerEx.LIGHT_MODE.HIT)
    self.finish_callback()

end

function WheelLayerEx:start()

    self:reset_roulette()
    self:change_light_mode(WheelLayerEx.LIGHT_MODE.IDLE)

    local function _scheduler(dt)

        self:rotate_scheduler(dt)
        self:light_scheduler(dt)

    end

    self:scheduleUpdateWithPriorityLua(_scheduler, 0)

end

function WheelLayerEx:stop()

    self:reset_roulette()
    self:change_light_mode(WheelLayerEx.LIGHT_MODE.TERMINATE)

    self:unscheduleUpdate()

end

function WheelLayerEx:rotate(target_index, finish_callback)

    math.randomseed(os.time())

    local target = (target_index - 1) * 45 + 8.5 + 45 * math.random() + self.__slot_step / 2
    local current = self.roulette:getRotation() % 360
    local offset = target > current and target - current or 360 - current + target 

    self:change_light_mode(WheelLayerEx.LIGHT_MODE.FOLLOW)

    local rotation_1 = cc.EaseCubicActionInOut:create(cc.RotateBy:create(9, offset + 360 * 5))
    local sequence = cc.Sequence:create(rotation_1)
    local call = cc.CallFunc:create(function() self:reverse_overload() end)

    self.roulette:runAction(cc.Sequence:create(sequence, call))
    self.finish_callback = finish_callback or function() end

    self:disable_button()

end

function WheelLayerEx:change_button_mode(mode)

    local button = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("button_rotate")

    if mode == WheelLayerEx.BUTTON_MODE.FREE then

        button:getChildByName("panel_free"):setVisible(true)
        button:getChildByName("panel_price"):setVisible(false)
        self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("image_lib"):setVisible(false)

    elseif mode == WheelLayerEx.BUTTON_MODE.PAY then

        button:getChildByName("panel_free"):setVisible(false)
        button:getChildByName("panel_price"):setVisible(true)
        self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("image_lib"):setVisible(false)

    elseif mode == WheelLayerEx.BUTTON_MODE.DISABLE then

        button:getChildByName("panel_free"):setVisible(false)
        button:getChildByName("panel_price"):setVisible(false)
        self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("image_lib"):setVisible(true)

        self:disable_ok_button()
    end

end

function WheelLayerEx:set_rotate_price(price, currency_img)

    local widget = self.widget:getChildByName("image_dia"):getChildByName("image_dia_1"):getChildByName("button_rotate"):getChildByName("panel_price")
    widget:getChildByName("image_price"):loadTexture(ResourceManager:getInstance():getPicturePathByKey(currency_img))
    widget:getChildByName("text_price"):setString(string.format("%d", price))

end

function WheelLayerEx:set_button_callback(callback)

    self.ok_callback = callback

end

function WheelLayerEx:set_item(item_list)

    if #item_list == self.__slot_number then

        for i = 1, #item_list do

            local item = self.roulette:getChildByName(string.format("%d", 9 - i))
            local img = item:getChildByName("image_item")
            local text = item:getChildByName("text_num")

            img:loadTexture(item_list[i].img)--ResourceManager:getInstance():getPicturePathByKey(item_list[i].img))
            if item_list[i].num >= 1000 then
                local num = math.floor(item_list[i].num/1000)
                text:setString(string.format("x%dk",num))
            else
                text:setString(string.format("x%d",item_list[i].num))
            end
            

        end

    end

end

function WheelLayerEx:enable_button()

    self.wrap_ok:enable()
    self.wrap_close:enable()

    self:play_button_animation()

end

function WheelLayerEx:disable_button()

    self.wrap_ok:disable()
    self.wrap_close:disable()

    self:stop_button_animation()

end

function WheelLayerEx:disable_ok_button()

    self.wrap_ok:disable()

    self:stop_button_animation()

end

WheelLayerEx.BUTTON_MODE = {
    
    ["FREE"] = 1,
    ["PAY"] = 2,
    ["DISABLE"] = 3,

}

WheelLayerEx.BACKWARD_ACTION_TAG = 1

WheelLayerEx.LIGHT_MODE = {
    
    ["FOLLOW"] = 1,
    ["IDLE"] = 2,
    ["HIT"] = 3,
    ["TERMINATE"] = 4,

}

WheelLayerEx.LIGHT_CONTROL_IDLE = {
    
    tick = 0,
    counter = 0,
    color = { "yellow", "blue" },

    start = function(self)

        WheelLayerEx.LIGHT_CONTROL_IDLE.tick = 0
        WheelLayerEx.LIGHT_CONTROL_IDLE.counter = 0

    end,

    stop = function(self)

        for i = 1, self.light_number do

            if self.lights[i].on then

                self.lights[i]:setAnimation(0, string.format("normal_%s", self.lights[i].color), false)

            end

        end

    end,

    update = function(self, dt)

        WheelLayerEx.LIGHT_CONTROL_IDLE.counter = WheelLayerEx.LIGHT_CONTROL_IDLE.counter + dt

        if WheelLayerEx.LIGHT_CONTROL_IDLE.counter > 0.8 then

            for i = 1, self.light_number, 3 do

                local index_off = (WheelLayerEx.LIGHT_CONTROL_IDLE.tick + i) % self.light_number + 1
                local index_on = (WheelLayerEx.LIGHT_CONTROL_IDLE.tick + 1 + i) % self.light_number + 1

                self.lights[index_off]:setAnimation(0, string.format("quick_%s", WheelLayerEx.LIGHT_CONTROL_IDLE.color[WheelLayerEx.LIGHT_CONTROL_IDLE.tick % 2 + 1]), false)
                self.lights[index_on]:setAnimation(0, string.format("%s", WheelLayerEx.LIGHT_CONTROL_IDLE.color[(WheelLayerEx.LIGHT_CONTROL_IDLE.tick + 1) % 2 + 1]), false)

                self.lights[index_on].color = WheelLayerEx.LIGHT_CONTROL_IDLE.color[WheelLayerEx.LIGHT_CONTROL_IDLE.tick % 2 + 1]
                self.lights[index_on].on = true
                self.lights[index_off].on = false

            end

            WheelLayerEx.LIGHT_CONTROL_IDLE.tick = WheelLayerEx.LIGHT_CONTROL_IDLE.tick + 1
            WheelLayerEx.LIGHT_CONTROL_IDLE.tick = WheelLayerEx.LIGHT_CONTROL_IDLE.tick % self.light_number            

            WheelLayerEx.LIGHT_CONTROL_IDLE.counter = WheelLayerEx.LIGHT_CONTROL_IDLE.counter - 0.8

        end

    end,

}

WheelLayerEx.LIGHT_CONTROL_FOLLOW = {
    
    tick = 0,
    delta = 0,
    last_rot = 0,

    start = function(self)

        WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick = math.floor(self.roulette:getRotation() / self.rot_interval)
        WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta = 0
        WheelLayerEx.LIGHT_CONTROL_FOLLOW.last_rot = self.roulette:getRotation()

    end,

    stop = function(self)

        for i = 1, self.light_number do

            if self.lights[i].on then

                self.lights[i]:setAnimation(0, string.format("normal_%s", self.lights[i].color), false)

            end

        end

    end,

    update = function(self, dt)

        local current_rot = self.roulette:getRotation()

        WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta = WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta + current_rot - WheelLayerEx.LIGHT_CONTROL_FOLLOW.last_rot

        while WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta > self.rot_interval  do

            for i = 1, self.light_number, 24 do

                local index_off = (WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick + i) % self.light_number + 1
                local index_on = (WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick + 1 + i) % self.light_number + 1

                self.lights[index_off]:setAnimation(0, "slow_yellow", false)
                self.lights[index_on]:setAnimation(0, "yellow", false)

                self.lights[index_on].color = "yellow"
                self.lights[index_on].on = true
                self.lights[index_off].on = false

            end

            WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick = WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick + 1
            WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick = WheelLayerEx.LIGHT_CONTROL_FOLLOW.tick % self.light_number  

            WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta = WheelLayerEx.LIGHT_CONTROL_FOLLOW.delta - self.rot_interval

            WheelLayerEx.LIGHT_CONTROL_FOLLOW.last_rot = current_rot     

        end

    end,

}

WheelLayerEx.LIGHT_CONTROL_HIT= {
    
    tick = 0,
    counter = 0,
    accu = 0,
    color = { "yellow", "blue" },

    start = function(self)

        WheelLayerEx.LIGHT_CONTROL_HIT.tick = 0
        WheelLayerEx.LIGHT_CONTROL_HIT.counter = 0
        WheelLayerEx.LIGHT_CONTROL_HIT.accu = 0

    end,

    stop = function(self)

        for i = 1, self.light_number do

            if self.lights[i].on then

                self.lights[i]:setAnimation(0, string.format("normal_%s", self.lights[i].color), false)

            end

        end

    end,

    update = function(self, dt)

        WheelLayerEx.LIGHT_CONTROL_HIT.counter = WheelLayerEx.LIGHT_CONTROL_HIT.counter + dt
        WheelLayerEx.LIGHT_CONTROL_HIT.accu = WheelLayerEx.LIGHT_CONTROL_HIT.accu + dt

        if WheelLayerEx.LIGHT_CONTROL_HIT.counter > 0.2 then

            for i = 1, self.light_number, 2 do

                local index_off = (WheelLayerEx.LIGHT_CONTROL_HIT.tick + i) % self.light_number + 1
                local index_on = (WheelLayerEx.LIGHT_CONTROL_HIT.tick + 1 + i) % self.light_number + 1

                self.lights[index_off]:setAnimation(0, string.format("normal_%s", WheelLayerEx.LIGHT_CONTROL_HIT.color[WheelLayerEx.LIGHT_CONTROL_HIT.tick % 2 + 1]), false)
                self.lights[index_on]:setAnimation(0, string.format("%s", WheelLayerEx.LIGHT_CONTROL_HIT.color[(WheelLayerEx.LIGHT_CONTROL_HIT.tick + 1) % 2 + 1]), false)

                self.lights[index_on].color = WheelLayerEx.LIGHT_CONTROL_HIT.color[WheelLayerEx.LIGHT_CONTROL_HIT.tick % 2 + 1]
                self.lights[index_on].on = true
                self.lights[index_off].on = false

            end

            WheelLayerEx.LIGHT_CONTROL_HIT.tick = WheelLayerEx.LIGHT_CONTROL_HIT.tick + 1
            WheelLayerEx.LIGHT_CONTROL_HIT.tick = WheelLayerEx.LIGHT_CONTROL_HIT.tick % self.light_number            

            WheelLayerEx.LIGHT_CONTROL_HIT.counter = WheelLayerEx.LIGHT_CONTROL_HIT.counter - 0.2

        end

        if WheelLayerEx.LIGHT_CONTROL_HIT.accu > 15 then

            self:change_light_mode(WheelLayerEx.LIGHT_MODE.IDLE)

        end

    end,

}

WheelLayerEx.LIGHT_CONTROL_TERMINATE = {
    
    start = function(self)

        for i = 1, self.light_number do

            if self.lights[i].on then

                self.lights[i]:setAnimation(0, string.format("normal_%s", self.lights[i].color), false)

            end

        end

    end,

    stop = function(self)


    end,

    update = function(self, dt)



    end,

}

WheelLayerEx.LIGHT_CONTROL = {
    
    [WheelLayerEx.LIGHT_MODE.FOLLOW] = WheelLayerEx.LIGHT_CONTROL_FOLLOW,
    [WheelLayerEx.LIGHT_MODE.IDLE] = WheelLayerEx.LIGHT_CONTROL_IDLE,
    [WheelLayerEx.LIGHT_MODE.HIT] = WheelLayerEx.LIGHT_CONTROL_HIT,
    [WheelLayerEx.LIGHT_MODE.TERMINATE] = WheelLayerEx.LIGHT_CONTROL_TERMINATE,

}

return WheelLayerEx

