require "UI.Layer.TopBarLayer"
require "UI.Layer.SectionBottomLayer"
require "UI.Layer.LevelInfoLayer"
require "UI.Layer.MapLayer"
require "UI.Layer.SectionLayer"
require "core.LayerManager"
require "UI.Widget.MaskLayer"

ExternalScene = class("ExternalScene", function() return cc.Scene:create() end)

ExternalScene.CURRENT_LAYER = {
    SECTION_MAP_LAYER = 1,
    MAP_LAYER = 2,
}

function ExternalScene:ctor()

	self:initUI()

end

function ExternalScene:getInstance()

end

function ExternalScene:initUI()
    cc.Director:getInstance():getRunningScene():addChild(MaskLayer:create(), 5)
    self.layer_vector = {}
	local visible_size = cc.Director:getInstance():getVisibleSize()

    -- local text = ccui.Text:create(LOGIN_PLATFROM, "images/font/zh_CN.TTF", 60)
    -- text:setPosition(visible_size.width/2,visible_size.height*0.92)
    -- self:addChild(text,999)

    -- 顶部栏
    local topBarLayer = TopBarLayer:create()
    self:addChild(topBarLayer, 1)

    -- 底部按钮
    local sectionBottomLayer = SectionBottomLayer:create()
    self.sectionBottomLayer = sectionBottomLayer
    self:addChild(sectionBottomLayer, 1)


    local function onKeyReleased(keyCode, event)
        if keyCode == cc.KeyCode.KEY_BACK then
            local top_layer = LayerManager:getInstance():get_top_layer()
            if top_layer then
                if top_layer.close_self then
                    top_layer:close_self()
                end
                return
            end
            self:back_scene()
        end
    end

    local listener = cc.EventListenerKeyboard:create()
    listener:registerScriptHandler(onKeyReleased, cc.Handler.EVENT_KEYBOARD_RELEASED )

    local eventDispatcher = self:getEventDispatcher()
    eventDispatcher:addEventListenerWithSceneGraphPriority(listener, self)

end

function ExternalScene:add_mask_layer()
    local mask_layer = MaskLayer:create()
    mask_layer:setTag(123)
    self:addChild(mask_layer, 5)
end

function ExternalScene:remove_mask_layer()
    self:removeChildByTag(123)
end


function ExternalScene:add_section_map()
    -- 地图部分
    local sectionLayer = SectionLayer:create()
    self.layer_vector[#self.layer_vector + 1] = sectionLayer
    self:addChild(sectionLayer)

    --sectionLayer:setScrollFinishCallback(function()

        self.sectionBottomLayer:show_setting()

    --end)

end

function ExternalScene:move_section_map()
    -- 地图部分
    local sectionLayer = SectionLayer:create()
    self.layer_vector[#self.layer_vector + 1] = sectionLayer
    self:addChild(sectionLayer)    

    sectionLayer:setScrollFinishCallback(function()

        self.sectionBottomLayer:show_setting()

    end)
    sectionLayer:move_to_current_section()

end

function ExternalScene:move_and_enter_section_map(section_id, mission_id)
    --7.26
    local mapLayer = MapLayer:create(section_id-1)
    -- mapLayer.show_level = mission_id + 1
    mapLayer:init_map(section_id - 1)
    mapLayer:scroll_to_level(Player:getInstance():get_player_current_level())
    self.layer_vector[#self.layer_vector + 1] = mapLayer
    self:addChild(mapLayer)
    
    self:runAction(cc.Sequence:create(cc.DelayTime:create(1), cc.CallFunc:create(function()

        local show_section = function() 

            local section_map = self.layer_vector[#self.layer_vector]
            if not(section_map == nil) and section_map:getTag() == KSectionTag  then
                table.remove(self.layer_vector, #self.layer_vector)
                section_map:removeFromParent()
            end
            --7.26
            local sectionLayer = SectionLayer:create()
            self.layer_vector[#self.layer_vector + 1] = sectionLayer
            self:addChild(sectionLayer)
            sectionLayer:move_and_enter_section(section_id, mission_id)

            sectionLayer:setScrollFinishCallback(function()
                self.sectionBottomLayer:show_setting()
            end)            

        end

        mapLayer:show_finish_dialog(mission_id - 1, show_section)

    end)))

end

function ExternalScene:add_map(mission_section)
    local section_map = self.layer_vector[#self.layer_vector]
    if not(section_map == nil) and section_map:getTag() == KSectionTag  then
        table.remove(self.layer_vector, #self.layer_vector)
        section_map:removeFromParent()
    end

    -- 关卡部分
    local mapLayer = MapLayer:create(mission_section)
    mapLayer:init_map(mission_section)
    mapLayer:scroll_to_level(Player:getInstance():get_player_current_level())
    self.layer_vector[#self.layer_vector + 1] = mapLayer
    self:addChild(mapLayer)
    self.sectionBottomLayer:show_back()
end

function ExternalScene:add_WeeklyChallengeLayer()
    local weeklyChallengeLayer = WeeklyChallengeLayer:create()
    self:addChild(weeklyChallengeLayer,5)
    --LayerManager:getInstance():add_layer_to_scene(weeklyChallengeLayer)
end

function ExternalScene:move_map(mission_section)
    local section_map = self.layer_vector[#self.layer_vector]
    if not(section_map == nil) and section_map:getTag() == KSectionTag  then
        table.remove(self.layer_vector, #self.layer_vector)
        section_map:removeFromParent()
    end

    -- 关卡部分
    local mapLayer = MapLayer:create(mission_section)
    mapLayer:init_map(mission_section)
    self.layer_vector[#self.layer_vector + 1] = mapLayer
    self:addChild(mapLayer)
    mapLayer:move_to_newest_level()
    self.sectionBottomLayer:show_back()
end

function ExternalScene:switch_to_level_info(mission_section, mission_level)
    local section_map = self.layer_vector[#self.layer_vector]
    if not(section_map == nil) and section_map:getTag() == KSectionTag  then
        table.remove(self.layer_vector, #self.layer_vector)
        section_map:removeFromParent()
    end

    -- 关卡部分
    local mapLayer = MapLayer:create(mission_section)
    mapLayer.show_level_info = true
    mapLayer.show_section = mission_section
    mapLayer.show_level = mission_level
    mapLayer:init_map(mission_section)

    self.layer_vector[#self.layer_vector + 1] = mapLayer
    self:addChild(mapLayer)

    --判断该关卡是否已通关
    local mission_info = Mission_Data:getInstance():get_mission_info_by_level(mission_level)
    local player_mission_info = Player:getInstance():get_player_mission_info_by_id(mission_info.mission_id)
    if not player_mission_info.is_move_action then
        mapLayer.show_level_info = false
        mapLayer:add_level_info_layer(mission_section, mission_level)
    end
    
    -- mapLayer:add_level_info_layer(mission_section, mission_level)
    self.sectionBottomLayer:show_back()
end

function ExternalScene:switch_to_level_info_without_show(mission_section, mission_level)
    print("switch without show")
    local section_map = self.layer_vector[#self.layer_vector]
    if not(section_map == nil) and section_map:getTag() == KSectionTag  then
        table.remove(self.layer_vector, #self.layer_vector)
        section_map:removeFromParent()
    end

    -- 关卡部分
    local mapLayer = MapLayer:create(mission_section)
    mapLayer:init_map(mission_section)
    self.layer_vector[#self.layer_vector + 1] = mapLayer
    self:addChild(mapLayer)
    -- mapLayer:add_level_info_layer(mission_section, mission_level)
    self.sectionBottomLayer:show_back()
end

function ExternalScene:get_current_layer_index()
    return self.current_layer_index
end

function ExternalScene:add_role_layer()
    local roleInfoLayer = RoleInfoLayer:create()
    LayerManager:getInstance():add_layer_to_scene(roleInfoLayer)
end

function ExternalScene:delay_add_role_layer()
    self:runAction(cc.Sequence:create(cc.DelayTime:create(0.1), cc.CallFunc:create(function()
        local roleInfoLayer = RoleInfoLayer:create()
        LayerManager:getInstance():add_layer_to_scene(roleInfoLayer)
    end)))
end

function ExternalScene:back_scene()
    while self.layer_vector[#self.layer_vector] == nil do
        table.remove(self.layer_vector, #self.layer_vector)
    end
    local cur_layer = self.layer_vector[#self.layer_vector]
    if cur_layer:getTag() == KMapTag  then
        table.remove(self.layer_vector, #self.layer_vector)
        cur_layer:removeFromParent()

        self:add_section_map()
        return
    end

    if cur_layer:getTag() == KSectionTag  then
        
        -- local commonPopupLayer = CommonPopupLayer:create()
        -- local str1 = ResourceManager:getInstance():getTextStringByKey("HomeScene_2")
        -- local str2 = ResourceManager:getInstance():getTextStringByKey("HomeScene_3")
        -- local str3 = ResourceManager:getInstance():getTextStringByKey("HomeScene_4")
        -- commonPopupLayer:showMessage(str1, str2, str3, function()
        --     MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Ui_Cancelitem_Audio"), false)
        --     cc.Director:getInstance():endToLua()
        -- end, nil)
        luaj = require "cocos.cocos2d.luaj"
        luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_exit", {}                  
            ,"()V")

    end
end

function ExternalScene:play_bg_music()
    MusicManager:getInstance().play_bg_music(ResourceManager:getInstance():getAudioPathByKey("Main_Scene_Music"), true)
    MusicManager:getInstance().set_music_volume(0.15)
end

return ExternalScene