-- 当前游戏的准备流程：检查更新->自动登录->进入游戏(自动登录失败->选择登录模式)

local CommonPopupLayer = require "UI.Layer.CommonPopupLayer"
local AccountManager = require "core.AccountManager"
local AccountDriverTourist = require "core.AccountDriverTourist"
local AccountDriverWeiXin = require "core.AccountDriverWeiXin"
local AccountDriverQQ = require "core.AccountDriverQQ"
local AccountDriverAuto = require "core.AccountDriverAuto"

require "core.MusicManager"
require "core.ResourceManager"
require "UI.Widget.ButtonWrap"
require "GameEx.Foundations.Utils.InternalUtils"

HomeScene = class("HomeScene", function() return cc.Scene:create() end)

function HomeScene:ctor()
    
    self:setName("HomeScene")

    self:play_logo_animation()

    local function onNodeEvent(event)

        if "enter" == event then

            self:register_update_event_listener()

        elseif "exit" == event then

            self:unregister_update_event_listener()

        end

    end

    self:registerScriptHandler(onNodeEvent)

end

function HomeScene:play_logo_animation()

    local widget = cc.CSLoader:createNode("startlogo.csb")
    self:addChild(widget)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)
    --播放动画，动画有多个play
    local action = cc.CSLoader:createTimeline("startlogo.csb")  
    action:play("start", false)
    widget:runAction(action)

    local function onFrameEvent(frame)
        if nil == frame then
            return
        end
        local str = frame:getEvent()
        if str == "start_end" then
            self:init_home()
            widget:removeFromParent()
        end
    end
    --设置动画的回调事件
    action:setFrameEventCallFunc(onFrameEvent)

end

function HomeScene:init_home()

    -- self:init_game()
    MusicManager:getInstance().play_bg_music(ResourceManager:getInstance():getAudioPathByKey("Main_Scene_Music"), true)
    MusicManager:getInstance().set_music_volume(0.15)

    self.platform = 0

    local screen_size = cc.Director:getInstance():getVisibleSize()
    --管理资源
    local title_spine = ResourceManager:getInstance():getSpineByKey("SPINE_TITLE", 1)
    title_spine:setPosition(cc.p(screen_size.width / 2, screen_size.height / 2))
    title_spine:setAnimation(0, "title", false)
    title_spine:addAnimation(0, "title_cycle", true)

    self:addChild(title_spine)

    local widget = cc.CSLoader:createNode("home.csb")
    self:addChild(widget, 1);

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

    self.action = cc.CSLoader:createTimeline("home.csb")
    widget:runAction(self.action)
    self.action:setCurrentFrame(0)

    local node = widget:getChildByName("node")
    self.node = node
    local image_loading = node:getChildByName("image_loading")
    self.image_loading = image_loading
    self.loadingBar = image_loading:getChildByName("loadingbar")
    self.loadingBar:setPercent(0)
    self.image_loading:setVisible(false)

    -- self.node:setVisible(false)

    local qq_image = node:getChildByName("button_qq")
    qq_image:setVisible(false)
    local qq_btn = ButtonWrap:create()
    qq_btn:wrap(qq_image, 1)
    qq_btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        
        self.platform = 2
        self:login()
    end)

    local wechat_image = node:getChildByName("button_wechat")
     wechat_image:setVisible(false)

    local wechat_btn = ButtonWrap:create()
    wechat_btn:wrap(wechat_image, 1)
    wechat_btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        
        self.platform = 1
        self:login()
    end)

    local guest_image = node:getChildByName("button_guest")
    local text_guest = guest_image:getChildByName("text_guest")
    text_guest:setString("开始游戏")
    local guest_btn = ButtonWrap:create()
    guest_btn:wrap(guest_image, 1)
    guest_btn:addTouchEndCallback(function()
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("Click1_Audio"), false)
        MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey("VOICE_mx"), false)

        self.platform = 0
        self:login()

    end)    

    self:init_indication()

    self:start_update()


    -- local number = SecuredNumber:create(10)
    -- print(number:tonumber())
    -- number = nil

    -- local deadline = DeadLineLayer:create()
    -- deadline:init_with_time_stamp(1486624152, 10)
    -- deadline:show()
    -- self:addChild(deadline, 9999)

    --test
    -- self.node:setVisible(false)
    -- self.indication_node:setVisible(true)
    -- self:init_game()
    -- self:auto_login()
    
end

function HomeScene:init_game()
    --读取一些数据？
    require "GameInit"
    GameInit.load_game_module()
    GameInit.init_game()

end

function HomeScene:enter_game()

    GameInit.init_account()

    local function on_load_finish()

        local function enter_game_callback()
            cc.UserDefault:getInstance():setBoolForKey("home_scene_in", true)

            local externalScene = ExternalScene:create()
            LayerManager:getInstance():clear_layer()
            externalScene:move_section_map()
            
            if LOGIN_PLATFROM ~= 0 then
                local warningLayer = WarningLayer:create()
                warningLayer:set_text_string("授权成功")
                externalScene:addChild(warningLayer)
            end
            
            cc.Director:getInstance():replaceScene(externalScene)
        end

        local event_config = {
        
            {["name"] = "HOME_SCENE_START"}

        }

        local call_info = {

            func = enter_game_callback,
            target = self,

        }

        HOOKABLE_CALL(event_config, call_info)
    end
    -- on_load_finish()
    RemoteArchiveManager:getInstance():load(on_load_finish)

end

function HomeScene:be_tourist()
    -- 加载游客账号代理对象
    local driver = AccountDriverTourist:create()
    AccountManager:getInstance():load_driver(driver)
end

function HomeScene:be_weixin()
    -- 加载微信账号代理对象
    local driver = AccountDriverWeiXin:create()
    AccountManager:getInstance():load_driver(driver)
end

function HomeScene:be_QQ()
    -- 加载QQ账号代理对象
    local driver = AccountDriverQQ:create()
    AccountManager:getInstance():load_driver(driver)
end

function HomeScene:login()

    if self.platform == 1 then
        self:be_weixin()
    elseif self.platform == 2 then
        self:be_QQ()
    elseif self.platform == 0 then
        self:be_tourist()
        TradeManager:getInstance():noSessionLogin()
    end
    
    --加屏蔽层
    self:add_mask_layer()
    self.indicate_label:setString("账号登录...")
    self.node:setVisible(false)
    self.indication_node:setVisible(true)

    function login_success()
print("login success")
        --提示 "授权成功"

        PLAYER_ID = AccountManager:getInstance():get_server_account()
        self:enter_game()
        
    end

    function login_failed()
print("login failed")
        --去屏蔽层
        self:remove_mask_layer()
        self.node:setVisible(true)
        self.indication_node:setVisible(false)
        
        --提示 "登录授权失败"
        local warningLayer = WarningLayer:create()
        warningLayer:set_text_string("登录授权失败")
        LayerManager:getInstance():add_layer_to_scene(warningLayer)
    end

    AccountManager:getInstance():login(login_success, login_failed)

end

function HomeScene:add_mask_layer()
    local mask_layer = MaskLayer:create()
    mask_layer:setTag(123)
    self:addChild(mask_layer, 5)
end

function HomeScene:remove_mask_layer()
    self:removeChildByTag(123)
end

function HomeScene:init_indication()

    local visible_size = cc.Director:getInstance():getVisibleSize()
    local image_position = cc.p(visible_size.width/2, 200)
    
    self.indication_node = cc.DrawNode:create()
    self.indication_node:drawSolidRect(cc.p(0, image_position.y - 30), cc.p(visible_size.width, image_position.y + 30), cc.c4f(0, 0, 0, 0.6))
    self:addChild(self.indication_node, 2) 

    self.indicate_label = cc.Label:createWithTTF("...", "font/zh_CN.TTF", 36)
    self.indicate_label:setAnchorPoint(cc.p(0.5, 0.5))
    self.indicate_label:setPosition(image_position)
    self.indication_node:addChild(self.indicate_label, 2)
    self.indication_node:setVisible(false)    

end

function HomeScene:start_update()

    local UpdateManager = require "core.UpdateManager"

    UpdateManager:get_instance():init("http://static.qn.dayugame.com/package/baymax/hotUpdate/tencent/",
        "http://static.qn.dayugame.com/package/baymax/hotUpdate/tencent/",
        FileManipulator:getInstance():getPath())

    UpdateManager:get_instance():start_update_ex()

    self.node:setVisible(false)
    self.indication_node:setVisible(true)
    self.indicate_label:setString("检查新版本...")

end

function HomeScene:register_update_event_listener()

    self.confirmed_listener = cc.EventListenerCustom:create("REQUEST_UPDATE_CONFIRM", function()

        self.indicate_label:setString("下载更新内容...")
        cc.Director:getInstance():getEventDispatcher():dispatchEvent(cc.EventCustom:new("UPDATE_CONFIRMED"))

    end)

    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.confirmed_listener, cc.Director:getInstance():getNotificationNode())
    
    self.success_listener = cc.EventListenerCustom:create("UPDATE_SUCCEEDED", function()

        self:init_game()
        self.node:setVisible(true)
        self.indication_node:setVisible(false)

    end)

    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.success_listener, cc.Director:getInstance():getNotificationNode())

    self.failure_listener = cc.EventListenerCustom:create("UPDATE_FAILED", function()

        local update_popup = CommonPopupLayer:create()

        function retry()

            self:start_update()
            update_popup:close_self()

        end

        update_popup:disable_close()
        update_popup:showMessage("更新失败", "更新失败，请检查网络状态，确保足够的存储空间。", "重试", retry, nil)

    end)

    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.failure_listener, cc.Director:getInstance():getNotificationNode())    

end

function HomeScene:unregister_update_event_listener()

    cc.Director:getInstance():getEventDispatcher():removeEventListener(self.confirmed_listener)
    cc.Director:getInstance():getEventDispatcher():removeEventListener(self.success_listener)
    cc.Director:getInstance():getEventDispatcher():removeEventListener(self.failure_listener)

end

return HomeScene