local ResourceManager = require "GameEx.Foundations.ResourceManager"
local ResourceManager1 = require "core.ResourceManager"
local LevelBuilder = require "GameEx.Foundations.LevelBuilder"
local LevelBuilderTournament = require "GameEx.Foundations.LevelBuilderTournament"
local SceneBuilder = require "GameEx.SceneBuilder"
local SceneBuilderTournament = require "GameEx.SceneBuilderTournament"
require "core.TalkingDataManager"

LoadingScene = class("LoadingScene", function() return cc.Scene:create() end)

function LoadingScene:ctor(loading_type)
	self.finish_callback = nil
	local visible_size = cc.Director:getInstance():getVisibleSize()
	-- loading ui
	local widget =  nil 
    self.image_tips = nil
    math.randomseed(os.time())
    if loading_type == 1 then
        widget = cc.CSLoader:createNode("loading.csb")

        self.action = cc.CSLoader:createTimeline("loading.csb")
        widget:runAction(self.action)
        self.action:play("shin_move", true)

        --7.25
        local node = widget:getChildByName("node")
        local text_tips = node:getChildByName("text_tips")

        local tips_string_vector = require "script.tips_string"
        text_tips:setString(ResourceManager1:getInstance():getTipsStringByKey(math.random(1, #tips_string_vector)))
        --7.25
    elseif loading_type == 2 then
        widget = cc.CSLoader:createNode("loading_0.csb")
        self.image_tips = widget:getChildByName("node"):getChildByName("image_tips")
        local tips_map = {
            [1] = "images/guide/guide_1.png",
            [2] = "images/guide/guide_2.png",
            [3] = "images/guide/guide_3.png",
            [4] = "images/guide/guide_4.png",
        }
        self.image_tips:setTexture(tips_map[math.random(1, 4)])
    end

    self:addChild(widget, 1)

    widget:setContentSize(cc.Director:getInstance():getVisibleSize())
    ccui.Helper:doLayout(widget)

	-- 进度条
	local node = widget:getChildByName("node")
	local image_loading = node:getChildByName("image_loading")
	self.loadingBar = image_loading:getChildByName("loadingbar")
    self.loadingBar:setPercent(0)
end

function LoadingScene:load_game_scene_resource(scene_start_param)

    local function perform_load()

        self:remove_unuse_resource()
        -- 
        local scene = SceneBuilder:build(scene_start_param)
        local function update(delta)
            local percent = LevelBuilder:get_instance():get_preload_progress()
            self.loadingBar:setPercent(percent)
        end
        self:scheduleUpdateWithPriorityLua(update, 0)

        LoadingScene.__n_game_scene_loaded = true

        
        local player = Player:getInstance()
        local level_index = player:get_player_current_level()
        local current_role_id = player:get_player_cur_role_id()
        local current_skin_id = player:get_player_cur_skin_id()
        
        TalkingDataManager:getInstance():send("每个角色皮肤的使用次数统计", {[current_role_id] = current_skin_id})
        TalkingDataManager:getInstance():send("每关进入次数统计", {["进入关卡id"] = level_index})
        local str_level = "关卡"..level_index
        TalkingDataManager:getInstance():send("每关每个角色的使用次数统计" ,{[str_level] = current_role_id})
        TalkingDataManager:getInstance():sendMissionInfo("begin", str_level)


    end

    local call = cc.CallFunc:create(perform_load)
    local delay = cc.DelayTime:create(0.1)
    local sequence = cc.Sequence:create(delay, call)

    self:runAction(sequence)

end

function LoadingScene:load_game_scene_tournament_resource(scene_start_param)

    local function perform_load()

        self:remove_unuse_resource()
        -- 
        local scene = SceneBuilder:build_tournament(scene_start_param)
        local function update(delta)
            local percent = LevelBuilderTournament:get_instance():get_preload_progress()
            self.loadingBar:setPercent(percent)
        end
        self:scheduleUpdateWithPriorityLua(update, 0)

        LoadingScene.__n_game_scene_loaded = true

    end

    local call = cc.CallFunc:create(perform_load)
    local delay = cc.DelayTime:create(0.1)
    local sequence = cc.Sequence:create(delay, call)

    self:runAction(sequence)

end

function LoadingScene:load_extern_scene_resource()

    local function perform_load()

        self:remove_unuse_resource()

        local rm = ResourceManager:get_instance()
        rm:add_textures_to_preload_list("images/skin/bo/bo_100.png")
        rm:add_textures_to_preload_list("images/skin/bo/bo_101.png")
        rm:add_textures_to_preload_list("images/skin/bo/bo_102.png")
        --rm:add_textures_to_preload_list("images/skin/bo/bo_103.png")
        rm:add_textures_to_preload_list("images/skin/kazama/kazama_100.png")
        rm:add_textures_to_preload_list("images/skin/kazama/kazama_101.png")
        --rm:add_textures_to_preload_list("images/skin/kazama/kazama_102.png")
        rm:add_textures_to_preload_list("images/skin/masao/masao_100.png")
        rm:add_textures_to_preload_list("images/skin/masao/masao_101.png")
        rm:add_textures_to_preload_list("images/skin/masao/masao_102.png")
        --rm:add_textures_to_preload_list("images/skin/masao/masao_103.png")
        rm:add_textures_to_preload_list("images/skin/nene/nene_100.png")
        rm:add_textures_to_preload_list("images/skin/nene/nene_101.png")
        rm:add_textures_to_preload_list("images/skin/nene/nene_102.png")
        --rm:add_textures_to_preload_list("images/skin/nene/nene_103.png")
        rm:add_textures_to_preload_list("images/skin/shin/shin_100.png")
        rm:add_textures_to_preload_list("images/skin/shin/shin_101.png")
        rm:add_textures_to_preload_list("images/skin/shin/shin_102.png")
        rm:add_textures_to_preload_list("images/skin/shin/shin_103.png")
        rm:add_textures_to_preload_list("images/skin/shin/shin_104.png")

        rm:add_textures_to_preload_list("images/stage_map/101_0.jpg")
        rm:add_textures_to_preload_list("images/stage_map/101_1.jpg")
        --rm:add_textures_to_preload_list("images/stage_map/101_2.png")
        --rm:add_textures_to_preload_list("images/stage_map/101_button_stage_gray.png")
        rm:add_textures_to_preload_list("images/stage_map/101_button_stage.png")
        rm:add_textures_to_preload_list("images/stage_map/guang.png")
        rm:add_textures_to_preload_list("images/stage_map/line.png")
        rm:add_textures_to_preload_list("images/stage_map/stage_dobber.png")

        rm:set_finish_preload_callback(function()
            self:unscheduleUpdate()
            self.loadingBar:setPercent(100)
            if self.finish_callback then
                self.finish_callback()
            end
        end)
        rm:start_preload()

        local function update(delta)
            local percent =  rm:get_preload_progress()
            self.loadingBar:setPercent(percent)
        end
        self:scheduleUpdateWithPriorityLua(update, 0)

    end

    local call = cc.CallFunc:create(perform_load)
    local delay = cc.DelayTime:create(0.1)
    local sequence = cc.Sequence:create(delay, call)

    self:runAction(sequence)

end

function LoadingScene:set_load_finish_callback(finish_callback)
	self.finish_callback = finish_callback
end

function LoadingScene:remove_unuse_resource()

    if LoadingScene.__n_game_scene_loaded then

        LoadingScene.__n_game_scene_loaded = false

        SceneBuilder:clearGameScene()

    end

	cc.SpriteFrameCache:getInstance():removeUnusedSpriteFrames()
	cc.Director:getInstance():getTextureCache():removeUnusedTextures()

    for i = 0, 1 do

        collectgarbage("collect")

    end

    print(collectgarbage("count"))

end

return LoadingScene

