require "UI.Widget.SweepLightLayer"
ButtonWrap = class("ButtonWrap")

ButtonWrap.Action_Type = {
    NORMAL = 0,
    EASEIN = 1,
}

function ButtonWrap:ctor()
    self.touchBeginCallback = {}
    self.touchMoveCallback = {}
    self.touchEndCallback = {}
    self.touchCancelCallback = {}
end

function ButtonWrap:wrap(widget, action_type)

    local sprite;

    if widget.getVirtualRenderer and widget:getVirtualRenderer().getSprite and widget:getVirtualRenderer():getSprite() then
        sprite = cc.Sprite:createWithTexture(widget:getVirtualRenderer():getSprite():getTexture())
    else
        sprite = cc.Sprite:create()
    end

    local children = widget:getChildren()
    for k,v in pairs(children) do     
        if v:getDescription() == "Label" or v:getDescription() == "ImageView" then

        else
            if v.getTexture then
                local child = cc.Sprite:createWithTexture(v:getTexture())
                self:darkNode(child)
                child:setPosition(cc.p(v:getPosition()))
                sprite:addChild(child)
            end
        end
    end

    local size = widget:getContentSize()
    sprite:setPosition(cc.p(size.width/2, size.height/2))
    widget:addChild(sprite)
    self:darkNode(sprite)
    self.sprite = sprite

    self.sprite:setVisible(false)
    self.sprite:setContentSize(size)
    
    -- 常用机制
    self.whiting = nil
    self.widget = widget
    self.cur_scale_x = self.widget:getScaleX()
    self.cur_scale_y = self.widget:getScaleY()
    widget:setTouchEnabled(true)

    local function touchEventCallback(sender,eventType)
        local touch_begin_pos = sender:getTouchBeganPosition()
        local conver_pos = self.sprite:convertToNodeSpace(touch_begin_pos)

        if eventType == ccui.TouchEventType.began then

            for i = #self.touchBeginCallback, 1, -1 do

                self.touchBeginCallback[i]()

            end

            widget:runAction(self:beginAction(action_type))
            if self.whitting then
                self.whitting:setVisible(true)
            end

        elseif eventType == ccui.TouchEventType.moved then

            for i = #self.touchMoveCallback, 1, -1 do

                self.touchMoveCallback[i]()

            end

        elseif eventType == ccui.TouchEventType.ended then

            if self.whitting then
                self.whitting:setVisible(false)
            end
            widget:setScale(0.9 * self.cur_scale_x, 0.9 * self.cur_scale_y)
            widget:runAction(self:endAction(action_type))

            for i = #self.touchEndCallback, 1, -1 do

                self.touchEndCallback[i]()

            end

        elseif eventType == ccui.TouchEventType.canceled then

            if self.whitting then
                self.whitting:setVisible(false)
            end
            widget:setScale(0.9 * self.cur_scale_x, 0.9 * self.cur_scale_y)
            widget:runAction(self:cancelAction(action_type))

            for i = #self.touchCancelCallback, 1, -1 do

                self.touchCancelCallback[i]()

            end

        end
    end

    widget:addTouchEventListener(touchEventCallback)

    widget.wrapper = self
end

function ButtonWrap:set_cur_scale(scalex, scaley)
    self.cur_scale_x = scalex
    self.cur_scale_y = scaley
end

function ButtonWrap:add_whiting()
    local whitting = SweepLightLayer:create()
    whitting:add_whiting(cc.Sprite:create(self.widget:getResourcePath()))
    whitting:setPosition(cc.p(self.widget:getContentSize().width/2, self.widget:getContentSize().height/2))
    self.widget:addChild(whitting)
    whitting:setVisible(false)
    self.whitting = whitting
end

function ButtonWrap:setDarkButton(flag)
    if flag == true then
        self.sprite:setVisible(false)
    else
        self.sprite:setVisible(true)
    end
end

function ButtonWrap:setDarkButtonWithText(flag, text_name)
    local text_widget = self.widget:getChildByName(text_name)
    text_widget:setLocalZOrder(1)
    if flag == true then
        self.sprite:setVisible(false)
        text_widget:setColor(cc.c3b(255, 255, 255))
        text_widget:setOpacity(255)
    else
        self.sprite:setVisible(true)
        text_widget:setColor(cc.c3b(0, 0, 0))
        text_widget:setOpacity(255 * 0.5)
    end
end

function ButtonWrap:isDarkButton()
    if self.sprite:isVisible() then
        return true
    else
        return false
    end
end

function ButtonWrap:isImageVisible()
    if self.widget:isVisible() then
        return true
    else
        return false
    end
end

function ButtonWrap:getImage()
    return self.widget
end

function ButtonWrap:triggerTouchEndCallback()
    for i = #self.touchEndCallback, 1, -1 do

        self.touchEndCallback[i]()

    end
end

function ButtonWrap:disable()
    self.widget:setTouchEnabled(false)
end

function ButtonWrap:enable()
    self.widget:setTouchEnabled(true)
end

function ButtonWrap:beginAction(action_type)
    if action_type == ButtonWrap.Action_Type.NORMAL then
        return cc.ScaleTo:create(0.17, self.cur_scale_x, self.cur_scale_y)
    elseif action_type == ButtonWrap.Action_Type.EASEIN then
        return cc.EaseElasticOut:create(cc.ScaleTo:create(0.17, 0.9 * self.cur_scale_x, 0.9 * self.cur_scale_y))
    end
end

function ButtonWrap:endAction(action_type)
    if action_type == ButtonWrap.Action_Type.NORMAL then
        return cc.ScaleTo:create(0.25, self.cur_scale_x, self.cur_scale_y)
    elseif action_type == ButtonWrap.Action_Type.EASEIN then
        return cc.EaseElasticOut:create(cc.ScaleTo:create(0.25, self.cur_scale_x, self.cur_scale_y))
    end
end

function ButtonWrap:cancelAction(action_type)
    if action_type == ButtonWrap.Action_Type.NORMAL then
        return cc.ScaleTo:create(0.25, self.cur_scale_x, self.cur_scale_y)
    elseif action_type == ButtonWrap.Action_Type.EASEIN then
        return cc.EaseElasticOut:create(cc.ScaleTo:create(0.25, self.cur_scale_x, self.cur_scale_y))
    end
end

function ButtonWrap:addTouchBeginCallback(callback)
    self.touchBeginCallback[#self.touchBeginCallback + 1] = callback
end

function ButtonWrap:addTouchMoveCallback(callback)
    self.touchMoveCallback[#self.touchMoveCallback + 1] = callback 
end

function ButtonWrap:addTouchEndCallback(callback)
    self.touchEndCallback[#self.touchEndCallback + 1] = callback
end

function ButtonWrap:addTouchCancelCallback(callback)
    self.touchCancelCallback[#self.touchCancelCallback + 1] = callback
end

function ButtonWrap:removeTouchBeginCallback(callback)
    for i = 1, #self.touchBeginCallback do
        if self.touchBeginCallback[i] == callback then self.touchBeginCallback[i] = nil end
        if not self.touchBeginCallback[i] then 
            self.touchBeginCallback[i] = self.touchBeginCallback[i + 1] 
            self.touchBeginCallback[i + 1] = nil 
        end
    end
end

function ButtonWrap:removeTouchMoveCallback(callback)
    for i = 1, #self.touchMoveCallback do
        if self.touchMoveCallback[i] == callback then self.touchMoveCallback[i] = nil end
        if not self.touchMoveCallback[i] then 
            self.touchMoveCallback[i] = self.touchMoveCallback[i + 1] 
            self.touchMoveCallback[i + 1] = nil 
        end
    end
end

function ButtonWrap:removeTouchEndCallback(callback)
    for i = 1, #self.touchEndCallback do
        if self.touchEndCallback[i] == callback then self.touchEndCallback[i] = nil end
        if not self.touchEndCallback[i] then 
            self.touchEndCallback[i] = self.touchEndCallback[i + 1] 
            self.touchEndCallback[i + 1] = nil 
        end
    end
end

function ButtonWrap:removeTouchCancelCallback(callback)
    for i = 1, #self.touchCancelCallback do
        if self.touchCancelCallback[i] == callback then self.touchCancelCallback[i] = nil end
        if not self.touchCancelCallback[i] then 
            self.touchCancelCallback[i] = self.touchCancelCallback[i + 1] 
            self.touchCancelCallback[i + 1] = nil 
        end
    end
end

function ButtonWrap:darkNode(node)

    ButtonWrap.s_gl_program = ButtonWrap.s_gl_program or ButtonWrap:createGLProgram()

    node:setGLProgram(ButtonWrap.s_gl_program)
    
end

function ButtonWrap:createGLProgram()

    local vertDefaultSource = "\n"..
    "attribute vec4 a_position; \n" ..
    "attribute vec2 a_texCoord; \n" ..
    "attribute vec4 a_color; \n"..                                                    
    "#ifdef GL_ES  \n"..
    "varying lowp vec4 v_fragmentColor;\n"..
    "varying mediump vec2 v_texCoord;\n"..
    "#else                      \n" ..
    "varying vec4 v_fragmentColor; \n" ..
    "varying vec2 v_texCoord;  \n"..
    "#endif    \n"..
    "void main() \n"..
    "{\n" ..
    "gl_Position = CC_PMatrix * a_position; \n"..
    "v_fragmentColor = a_color;\n"..
    "v_texCoord = a_texCoord;\n"..
    "}"
     
    local pszFragSource = "#ifdef GL_ES \n" ..
    "precision mediump float; \n" ..
    "#endif \n" ..
    "varying vec4 v_fragmentColor; \n" ..
    "varying vec2 v_texCoord; \n" ..
    "void main(void) \n" ..
    "{ \n" ..
    "vec4 c = texture2D(CC_Texture0, v_texCoord); \n" ..
    "gl_FragColor.xyz = vec3(0.4*c.r + 0.4*c.g +0.4*c.b); \n"..
    "gl_FragColor.w = c.w; \n"..
    "}"
 
    local pProgram = cc.GLProgram:createWithByteArrays(vertDefaultSource,pszFragSource)
     
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_POSITION,cc.VERTEX_ATTRIB_POSITION)
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_COLOR,cc.VERTEX_ATTRIB_COLOR)
    pProgram:bindAttribLocation(cc.ATTRIBUTE_NAME_TEX_COORD,cc.VERTEX_ATTRIB_FLAG_TEX_COORDS)
    pProgram:link()
    pProgram:updateUniforms()

    return pProgram

end

function ButtonWrap:set_fade_out()
    self.widget:runAction(cc.Sequence:create(cc.FadeOut:create(1.0/12),cc.CallFunc:create(function()self:setDarkButton(true)end)))
end

return ButtonWrap