
-- 返回电子表
function get_electronic_clock(second)
	local hour = math.floor(second / 3600)
	local min = math.floor((second - hour * 3600)/60)
	local sec = second - hour * 3600 - min * 60
	if hour == 0 then
		if min == 0 then
			return string.format("%02d", sec)
		else
			return string.format("%02d:%02d", min, sec)
		end
	else
		return string.format("%02d:%02d:%02d", hour, min, sec)
	end

end

function get_electronic_clock_m_s(second)
	local min = math.floor((second)/60)
	local sec = second - min * 60
	return string.format("%02d:%02d", min, sec)
end

function get_electronic_clock_h_m_s(second)
	local hour = math.floor(second / 3600)
	local min = math.floor((second - hour * 3600)/60)
	local sec = second - hour * 3600 - min * 60
	return string.format("%02d:%02d:%02d", hour, min, sec)
end


function get_node_width(node)
	return node:getContentSize().width
end

function get_node_height(node)
	return node:getContentSize().height
end

function get_node_center(node)
	return cc.p(get_node_width(node)/2, get_node_height(node)/2)
end

function get_table_length(table)
	local length = 0
	for k,v in pairs(table) do
		length = length + 1
	end
	return length
end

function get_guide_hand()
	local guide_hand = ResourceManager:getInstance():getSpineByKey("SPINE_GUIDE_HAND", 1.0)
	guide_hand:setAnimation(0, "animation", true)
	return guide_hand
end

function text_count_action(text_widget, star_count, end_count, time)
	local count_updater_entry = 0
    local cur_count = star_count
    local function count_updater()
        local add_count = 0.033 / time * (end_count - star_count)
        cur_count = cur_count + add_count
        if cur_count >= end_count then
            text_widget:setString(string.format("%d", end_count))
            cc.Director:getInstance():getScheduler():unscheduleScriptEntry(count_updater_entry)
        else
            text_widget:setString(string.format("%d", cur_count))
        end
    end
    count_updater_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(count_updater, 0.033, false)

    return count_updater_entry
end

function progress_count_action(progress_widget, star_percent, end_percent, time)
	local progress_updater_entry = 0

    local function progress_updater()
        local addPercent = 0.033 / time * 100
        local lastPercent = progress_widget:getPercent() + addPercent
        if lastPercent >= end_percent then
        	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(score_updater_entry)
        	progress_widget:setPercent(end_percent)
        else
        	progress_widget:setPercent(lastPercent)
        end
    end
    progress_updater_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(progress_updater, 0.033, false)
end

function move_cycle_action(widget)
	local per = 0.033
	local origin_scale_x = widget:getScaleX()
	local origin_scale_y = widget:getScaleY()

	local scale1 = cc.ScaleTo:create(per * 4, 0.9 * origin_scale_x, 1.1 * origin_scale_y)
	local scale2 = cc.ScaleTo:create(per * 3, 1.1 * origin_scale_x, 0.9 * origin_scale_y)
	local scale3 = cc.ScaleTo:create(per * 3, 0.9 * origin_scale_x, 1.05 * origin_scale_y)
	local scale4 = cc.ScaleTo:create(per * 3, 1.05 * origin_scale_x, 0.97 * origin_scale_y)
	local scale5 = cc.ScaleTo:create(per * 3, 0.98 * origin_scale_x, 1.0 * origin_scale_y)
	local scale6 = cc.ScaleTo:create(per * 2, 1.0 * origin_scale_x, 1.0 * origin_scale_y)

	local delay = cc.DelayTime:create(0.5)

	widget:runAction(cc.RepeatForever:create(cc.Sequence:create(scale1, scale2, scale3, scale4, scale5, scale6, delay)))
end

ELASTIC_ACTION_TAG = 10001

function elastic_action(widget)

	local per = 0.066
	local origin_scale_x = widget:getScaleX()
	local origin_scale_y = widget:getScaleY()

	local scale1 = cc.ScaleTo:create(per * 4, 0.9 * origin_scale_x, 1.1 * origin_scale_y)
	local scale2 = cc.ScaleTo:create(per * 3, 1.1 * origin_scale_x, 0.9 * origin_scale_y)
	local scale3 = cc.ScaleTo:create(per * 3, 0.9 * origin_scale_x, 1.05 * origin_scale_y)
	local scale4 = cc.ScaleTo:create(per * 3, 1.05 * origin_scale_x, 0.97 * origin_scale_y)
	local scale5 = cc.ScaleTo:create(per * 3, 0.98 * origin_scale_x, 1.0 * origin_scale_y)
	local scale6 = cc.ScaleTo:create(per * 2, 1.0 * origin_scale_x, 1.0 * origin_scale_y)

	local delay = cc.DelayTime:create(3.0)
	local rep = cc.RepeatForever:create(cc.Sequence:create(scale1, scale2, scale3, scale4, scale5, scale6, delay))
	rep:setTag(ELASTIC_ACTION_TAG)

	widget:runAction(rep)
	
end

function origin_cycle_action(widget)
	local per = 0.033
	local origin_scale_x = 1.0
	local origin_scale_y = 1.0

	local scale1 = cc.ScaleTo:create(per * 4, 0.9 * origin_scale_x, 1.1 * origin_scale_y)
	local scale2 = cc.ScaleTo:create(per * 3, 1.1 * origin_scale_x, 0.9 * origin_scale_y)
	local scale3 = cc.ScaleTo:create(per * 3, 0.9 * origin_scale_x, 1.05 * origin_scale_y)
	local scale4 = cc.ScaleTo:create(per * 3, 1.05 * origin_scale_x, 0.97 * origin_scale_y)
	local scale5 = cc.ScaleTo:create(per * 3, 0.98 * origin_scale_x, 1.0 * origin_scale_y)
	local scale6 = cc.ScaleTo:create(per * 2, 1.0 * origin_scale_x, 1.0 * origin_scale_y)

	local delay = cc.DelayTime:create(0.5)

	widget:runAction(cc.RepeatForever:create(cc.Sequence:create(scale1, scale2, scale3, scale4, scale5, scale6, delay)))
end

function springsale_action(widget)

	local per = 0.033

	local rotate1 = cc.RotateTo:create(per * 3, 5)
	local skew1 = cc.SkewTo:create(per * 3, 5, 5)
	local spawn1 = cc.Spawn:create(rotate1, skew1)

	local rotate2 = cc.RotateTo:create(per * 3, -5)
	local skew2 = cc.SkewTo:create(per * 3, -5, -5)
	local spawn2 = cc.Spawn:create(rotate2, skew2)

	local rotate3 = cc.RotateTo:create(per * 3, 0)
	local skew3 = cc.SkewTo:create(per * 3, 0, 0)
	local spawn3 = cc.Spawn:create(rotate3, skew3)

	local delay = cc.DelayTime:create(1.0)
	local rep = cc.RepeatForever:create(cc.Sequence:create(rotate1, rotate2, rotate1, rotate3, delay))

	widget:runAction(rep)
	
end

function tips_label_action(string,parent)
	local label = cc.LabelTTF:create(string,"Arial",64)
	local visibleSize = cc.Director:getInstance():getVisibleSize()
	label:setPosition(cc.p(visibleSize.width*0.5, visibleSize.height*0.4))
	label:setColor(cc.RED)
	label:setOpacity(0)
	parent:addChild(label,5)

	local moveTo = cc.MoveTo:create(1,cc.p(visibleSize.width*0.5, visibleSize.height*0.5))
	local fadeIn = cc.FadeIn:create(1)
	local fadeOut = cc.FadeOut:create(1)
	local callfunc = cc.CallFunc:create(function ()	
					label:removeFromParent()
					end)
	local spawn = cc.Spawn:create(moveTo, fadeIn)
	local sequence = cc.Sequence:create(spawn, fadeOut, callfunc)

	label:runAction(sequence)
end

function set_list_fade_out(list)
	list:runAction(cc.Sequence:create(cc.FadeOut:create(1.0/12),cc.CallFunc:create(function()list:setVisible(false) end)))
end

function global_get_random_name()
	local random_name_info = require "script/random_name_info"
	local str_prefix = random_name_info[1]
	local str_postfix = random_name_info[2]

	math.randomseed(os.time())
	local num_1 = math.random(1,#str_prefix)
	local num_2 = math.random(1,#str_postfix)
	local name = string.format("%s%s", str_prefix[num_1], str_postfix[num_2])

	return name
end

function global_get_screenshot_path()

	if device.platform == "android" and MSDKJava then
		return "/mnt/sdcard/android/data/com.tencent.tmgp.lbxxtgsj/"
	else
		return FileManipulator:getInstance():getPath()
	end
	
end

function global_create_share_pic(target, share_bg, save_name)
	local sp = cc.Node:create()
    sp:setPosition(cc.p(0, 0))
    sp:setAnchorPoint(0,0)
    sp:retain()

    local widget = cc.CSLoader:createNode("share_picture.csb")
	sp:addChild(widget)
	local node = widget:getChildByName("node")
    local image_game_pic = node:getChildByName("image_game_pic")
    local image_dia = node:getChildByName("image_dia")
    local image_id = image_dia:getChildByName("image_id")
    local text_name = image_dia:getChildByName("text_name")
    local image_qrcode = image_dia:getChildByName("image_qrcode")
    local text_stage_num = image_dia:getChildByName("text_stage_num")

    image_game_pic:setVisible(false)
    text_stage_num:setVisible(false)
    image_dia:loadTexture(string.format("res/images/common/share_picture/%s.jpg", share_bg))
    image_qrcode:loadTexture("images/common/share_picture/test.png")

    local player = Player:getInstance()
    text_name:setString(player:get_player_nick_name())
    local player_head_id = player:get_player_head_id()

    if player_head_id == "head_00" then
        local headPath = FileManipulator:getInstance():getPath().."head_cache/"..PLAYER_ID..player_head_id
        image_id:loadTexture(headPath)
    else
        local head_info = Head_Data:getInstance():get_head_info_by_id(player_head_id)
        image_id:loadTexture(head_info.res_key)
    end

    local size = widget:getContentSize()
    local render = cc.RenderTexture:create(size.width, size.height)
    render:begin()
    sp:visit()
    render:endToLua()
    render:retain()
    
    performWithDelay(target,function()
        local image = render:newImage()
        local fileName = global_get_screenshot_path()..save_name
        image:saveToFile(fileName)
        render:release()
        sp:release()

    end, 0.016)
end

function global_create_social_head(guid, pictureLargeUrl, on_create_finish)

	if pictureLargeUrl and pictureLargeUrl ~= "" then
		local headPath = FileManipulator:getInstance():getPath().."head_cache/"..guid.."head_00"
		if not cc.FileUtils:getInstance():isFileExist(headPath) then

		    local xhr = cc.XMLHttpRequest:new()

		    xhr.responseType = cc.XMLHTTPREQUEST_RESPONSE_STRING
		    xhr:open("GET", pictureLargeUrl)
		    local function onDownloadImage()
		        if xhr.readyState == 4 and (xhr.status >= 200 and xhr.status < 207) then
		            local fileData = xhr.response
		            local fullFileName = headPath
		            local file = io.open(fullFileName,"wb")
		            file:write(fileData)
		            file:close()

		            -- if TalkingData then
		            --     TalkingData.converToPng(fullFileName)
		            -- end 

		            if on_create_finish then
		            	on_create_finish(1)
		            end
		        end
		    end
		    -- ERROR_ASSERT(false, "string")
		    xhr:registerScriptHandler(onDownloadImage)
		    xhr:send()
		else
			if on_create_finish then
            	on_create_finish()
            end
		end
	end
end

function global_set_user_social_head()
	global_create_head_cache()

	-- local data_path = FileManipulator:getInstance():getPath()..PLAYER_ID.."player_base_data.lua"
	-- local is_data_file_exist = cc.FileUtils:getInstance():isFileExist(data_path)

    performWithDelay(cc.Director:getInstance():getNotificationNode(),function()
		local nickName,pictureLargeUrl = MSDKJava.getUserInfoRocord()
		JavaUtils.log("com.dayu", nickName..","..pictureLargeUrl)

		local function update_head_url()

			RPC:call("SHIN/user/updateHeadUrl", {PLAYER_ID, pictureLargeUrl}, function(status, response)
		        if status == RPC.RPC_CALL_SUCCESS then
		            
		        else
		            print("request error")
		        end
		    end)

		end

		RPC:call("SHIN/user/getUserData", {PLAYER_ID,}, function(status, response)

	        if status == RPC.RPC_CALL_SUCCESS then

	            local headUrl = response.headUrl
	            if not headUrl or headUrl == "" then
	            	update_head_url()
	            end
	        else
	            print("request error")
	        end

	    end)

		local function on_create_finish(result)

			if result == 1 then
	            JavaUtils.log("com.dayu", "create head success")
	            
                -- update_head_url()

                local function click_enter()
				    Player:getInstance():set_player_head_id("head_00")
		            Player:getInstance():set_player_nick_name(nickName)

				    RPC:call("SHIN/account/modifyName", {PLAYER_ID, nickName, 0}, function(status, response)

				        if status == RPC.RPC_CALL_SUCCESS then
				            
				        else
				            print("request error")
				        end

				    end)
				end

				if not cc.UserDefault:getInstance():getBoolForKey("first_login", true) then
					local commonPopupLayer = CommonPopupLayer:create()
				    commonPopupLayer:showMessage("提醒", "要更换到QQ/微信\n的名字和头像吗", "确定", function()
				    	
				    	click_enter()
				        commonPopupLayer:close_self()
				    end, nil)
				else
					click_enter()
				end
			    
	        end
	    end
        global_create_social_head(PLAYER_ID, pictureLargeUrl, on_create_finish)
	    
    end, 1)

end

function global_check_remote_archive(on_check_finish)

	local function on_load_finish(result)
		if result == 3 then

			local externalScene = ExternalScene:create()
	        LayerManager:getInstance():clear_layer()
	        externalScene:move_section_map()

	        local warningLayer = WarningLayer:create()
            warningLayer:set_text_string("下载存档成功")
            externalScene:addChild(warningLayer)

	        cc.Director:getInstance():replaceScene(externalScene)
	    else
	    	on_check_finish()
		end
	end

	RemoteArchiveManager:getInstance():load(on_load_finish)

end

function global_create_head_cache()

	local path = FileManipulator:getInstance():getPath().."head_cache"
    if not cc.FileUtils:getInstance():isFileExist(path) then
        os.execute("mkdir -p "..path)
    end

end

function global_remove_file_from_cache()

	local function remove_module(path)
		os.execute("rm -rf "..FileManipulator:getInstance():getPath()..path)
	end
	remove_module("src")
	remove_module("res")
	remove_module("current_version.lua")

	remove_module("version_manifest.lua")

end
