Dialog = class("Dialog", function() return cc.Node:create() end)

function Dialog:ctor()

	self.__c_role_sprite = cc.Sprite:create()
	self.__c_role_sprite:setScale(0.7)
	self.__c_role_sprite:setAnchorPoint(cc.p(0.5, 0))
	self.__c_role_sprite:setPosition(cc.p(0, -150))
	self:addChild(self.__c_role_sprite, 1)

	self.__c_frame_sprite = ccui.Scale9Sprite:create({x = 50, y = 50, width = 13, height = 35}, ResourceManager:getInstance():getPicturePathByKey("DIALOG_FRAME"))
	self.__c_frame_sprite:setContentSize(cc.size(0, 0))
	self:addChild(self.__c_frame_sprite, 2)

	self.__c_decorate_sprite = cc.Sprite:create(ResourceManager:getInstance():getPicturePathByKey("DIALOG_DECORATE"))
	-- self.__c_frame_sprite:addChild(self.__c_decorate_sprite, 3)

	self.__c_text_label = cc.Label:createWithTTF("", "font/zh_CN.TTF", 36)
	self.__c_text_label:setMaxLineWidth(360)
	self.__c_text_label:setOverflow(cc.LabelOverflow.RESIZE_HEIGHT)
	self.__c_text_label:setDimensions(0, 0)
	self.__c_text_label:setColor(cc.c3b(117, 38, 7))
	self.__c_frame_sprite:addChild(self.__c_text_label, 4)

	self.__t_pop_scenario = {}

	self.__n_max_width = 500
	self.__n_max_height = 500

	self.__n_showed = false 
    
end

function Dialog:set_config(config)

	self:set_text(config.text)
	self:set_text_size(config.font_size)
	self:set_text_max_width(config.max_width)

	self:set_voice(config.voice)
	self:set_animation(config.animation)

	if not self.__n_showed then

		self:set_role(config.role)
		self:set_side(config.side)
		self:setPosition(config.position)

	end

end

function Dialog:set_text_size(size)

	local ttf_config = self.__c_text_label:getTTFConfig()
	ttf_config.fontSize = size
	self.__c_text_label:setTTFConfig(ttf_config)

end

function Dialog:set_text(text)

	self.__c_text_label:setString(text)

end

function Dialog:set_text_max_width(width)

	self.__c_text_label:setMaxLineWidth(width)

end

function Dialog:set_voice(voice)

	self.__t_pop_scenario.voice = voice or ""

end

function Dialog:set_role(role)

	if role then

		self.__c_role_sprite:setTexture(ResourceManager:getInstance():getPicturePathByKey(role))

	end

end

function Dialog:set_animation(animation)

	animation = type(animation) == "table" and animation or {}

	for i = 1, #animation do

		if animation[i] == "DIRECT_LEAVE" then

			self.__n_direct_leave = true

		end

	end

	self.__t_pop_scenario.animation = self:get_animation_by_type(animation)

end

function Dialog:set_side(side)

	self.__t_pop_scenario.side = side

end

function Dialog:append_to(target)

	if not (self:getParent() == target) then

		self:removeFromParent()
		target:addChild(self)

	end

end

function Dialog:get_animation_by_type(type)

	return cc.DelayTime:create(0)

end

function Dialog:pop()

	self:play_scenario()

	if not self.__n_showed then

		self.__n_showed = true
		self:play_show_animation()

	end

end


function Dialog:play_show_animation()

	local direction = self.__t_pop_scenario.side == "RIGHT" and -1 or 1

	self.__c_frame_sprite:setOpacity(0)
	self.__c_text_label:setOpacity(0)
	self.__c_role_sprite:setOpacity(0)
	self.__c_role_sprite:setPosition(cc.p(-120 * direction, -150))
	self.__c_frame_sprite:setScale(0.1)

	local move_1 = cc.MoveTo:create(0.25, cc.p(5 * direction, -150))
	local move_2 = cc.MoveTo:create(0.066, cc.p(-5 * direction, -150))
	local ease = cc.EaseExponentialOut:create(move_1)
	local sequence = cc.Sequence:create(ease, move_2)
	self.__c_role_sprite:runAction(sequence)

	local rotate_1 = cc.RotateBy:create(0.25, 4 * direction)
	local rotate_2 = cc.RotateBy:create(0.066, -4 * direction)
	local ease = cc.EaseExponentialOut:create(rotate_1)
	local sequence = cc.Sequence:create(ease, rotate_2, rotate_3)
	self.__c_role_sprite:runAction(sequence)

	local fade_in = cc.FadeTo:create(0.066, 255)
	self.__c_role_sprite:runAction(fade_in)

	local function show_frame()

		local fade_in = cc.FadeTo:create(0.1, 255)
		self.__c_frame_sprite:runAction(fade_in)

		local fade_in = cc.FadeTo:create(0.1, 255)
		self.__c_text_label:runAction(fade_in)

		local zoom_1 = cc.ScaleTo:create(0.25, 1.1)
		local zoom_2 = cc.ScaleTo:create(0.066, 1.0)
		local ease = cc.EaseExponentialOut:create(zoom_1)
		local sequence = cc.Sequence:create(ease, zoom_2)

		self.__c_frame_sprite:runAction(sequence)		

	end

	local delay = cc.DelayTime:create(0.45)
	local sequence = cc.Sequence:create(delay, cc.CallFunc:create(show_frame))
	self:runAction(sequence)

end

function Dialog:play_scenario()

	self:resize_frame()
	self:do_layout()
	self:play_voice()
	self:play_change_animation()

end

function Dialog:play_change_animation()

	local direction = self.__t_pop_scenario.side == "RIGHT" and -1 or 1

	self.__c_frame_sprite:setScale(0.1)
	local zoom_1 = cc.ScaleTo:create(0.25, 1.1)
	local zoom_2 = cc.ScaleTo:create(0.066, 1.0)
	local ease = cc.EaseExponentialOut:create(zoom_1)
	local sequence = cc.Sequence:create(ease, zoom_2)
	self.__c_frame_sprite:runAction(sequence)

	self.__c_frame_sprite:setRotation(10 * direction)
	local rotate_1 = cc.RotateBy:create(0.25, -12 * direction)
	local rotate_2 = cc.RotateBy:create(0.055, 2 * direction)
	local ease = cc.EaseExponentialOut:create(rotate_1)
	local sequence = cc.Sequence:create(ease, rotate_2)
	self.__c_frame_sprite:runAction(sequence)

end

function Dialog:play_voice()

	MusicManager:getInstance().play_effect(ResourceManager:getInstance():getAudioPathByKey(self.__t_pop_scenario.voice) or "", false)

end

function Dialog:resize_frame()

	local text_size = self.__c_text_label:getContentSize()
	local frame_size = cc.size(text_size.width + 100, text_size.height + 100)

	self.__c_frame_sprite:setContentSize(frame_size)
	self.__c_text_label:setPosition(cc.p(frame_size.width / 2, frame_size.height / 2))

end

function Dialog:do_layout()

	if self.__t_pop_scenario.side == "LEFT" then

		self.__c_role_sprite:setFlippedX(false)
		self.__c_frame_sprite:setAnchorPoint(cc.p(0, 0))
		self.__c_frame_sprite:setPosition(cc.p(100, 50))
		-- self.__c_decorate_sprite:setPosition(cc.p(-5, 70))

	elseif self.__t_pop_scenario.side == "RIGHT" then

		self.__c_role_sprite:setFlippedX(true)
		self.__c_frame_sprite:setAnchorPoint(cc.p(1.0, 0.0))
		self.__c_frame_sprite:setPosition(cc.p(-100, 50))
		-- self.__c_decorate_sprite:setPosition(cc.p(self.__c_frame_sprite:getContentSize().width + 5, 70))

	end

end

function Dialog:with_draw()

	if self.__n_direct_leave then

		self:removeFromParent()

	else

		self:play_with_draw_animation()

	end

end

function Dialog:play_with_draw_animation()

	local direction = self.__t_pop_scenario.side == "RIGHT" and 1 or -1

	local fade_in = cc.FadeTo:create(0.066, 0)
	self.__c_frame_sprite:runAction(cc.Sequence:create(cc.DelayTime:create(0.2), fade_in))

	local fade_in = cc.FadeTo:create(0.066, 0)
	self.__c_text_label:runAction(cc.Sequence:create(cc.DelayTime:create(0.2), fade_in))

	local zoom_1 = cc.ScaleTo:create(0.25, 0.1)
	local zoom_2 = cc.ScaleTo:create(0.066, 1.2)
	local ease = cc.EaseExponentialIn:create(zoom_1)
	local sequence = cc.Sequence:create(zoom_2, ease)

	self.__c_frame_sprite:runAction(sequence)	


	local function hide_role()

		local move_1 = cc.MoveTo:create(0.25, cc.p(120 * direction, -150))
		local move_2 = cc.MoveTo:create(0.15, cc.p(-15 * direction, -150))
		local ease_1 = cc.EaseExponentialIn:create(move_1)
		local sequence = cc.Sequence:create(move_2, ease_1)
		self.__c_role_sprite:runAction(sequence)

		local rotate_1 = cc.RotateBy:create(0.15, -4 * direction)
		local rotate_2 = cc.RotateBy:create(0.25, 8 * direction)
		local ease = cc.EaseExponentialIn:create(rotate_2)
		local sequence = cc.Sequence:create(rotate_1, rotate_2)
		self.__c_role_sprite:runAction(sequence)

		local fade_in = cc.FadeTo:create(0.066, 0)
		self.__c_role_sprite:runAction(cc.Sequence:create(cc.DelayTime:create(0.34), fade_in))

	end

	local delay = cc.DelayTime:create(0.25)
	local sequence = cc.Sequence:create(delay, cc.CallFunc:create(hide_role))
	self:runAction(sequence)

	self:runAction(cc.Sequence:create(cc.DelayTime:create(0.6), cc.RemoveSelf:create()))

end

return Dialog