-- 游客登录的代理实现（关于代理，参看AccountManager.lua文件当中的注释）

AccountDriverTourist = class("AccountDriverTourist")

function AccountDriverTourist:ctor()



end

-- 挂载时初始化操作

function AccountDriverTourist:on_load()

	self:load_account_data()

	if not self.account_data then

		self:init_account_data()

	end

	self.wakeup_info = {}
	self.wakeup_info.awakened = false
	self.wakeup_info.platform = MSDK_Interface.ePlatform.ePlatform_None
	self.wakeup_info.callback_flag = MSDK_Interface.CallbackFlag.eFlag_Succ

	self.touristApi = MSDK_Interface:create()
	self.touristApi:setWakeupCallback(function(platform, ret_code, info) self:on_msdk_wakeup(platform,ret_code,info) end)

	self.wakeup_check_handler = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:check_wake_up() end, 0.033, false)


end

function AccountDriverTourist:check_wake_up()

	if self.wakeup_info.awakened then

		self.wakeup_info.awakened = false

		if self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_AccountRefresh or 
			self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_Succ then

			--不存在异账号，MSDK已通过刷新接口将本地账号票据刷新	读取LoginRet结构体中的票据进行游戏授权流程
			if self.wakeup_info.platform == MSDK_Interface.ePlatform.ePlatform_Weixin then
				
			elseif self.wakeup_info.platform == MSDK_Interface.ePlatform.ePlatform_QQ then
				
			end

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_UrlLogin then

			--不存在异账号，游戏通过拉起账号快速登录成功	读取LoginRet结构体中的票据进行游戏授权流程
			self.touristApi:url_login(self.wakeup_info.platform)

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_NeedSelectAccount then

			--游戏本地账号和拉起账号存在异账号	弹出对话框让用户选择登录的账号
			self.touristApi:need_select_account(self.wakeup_info.platform)

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_NeedLogin then
			
			--游戏本地账号和拉起账号均无法登录	返回登录界面，引导玩家重新登录授权
			self.touristApi:need_login(self.wakeup_info.platform)

		end

	end

end

function AccountDriverTourist:on_msdk_wakeup(platform, ret_code, info)

	self.wakeup_info.awakened = true
	self.wakeup_info.platform = platform
	self.wakeup_info.callback_flag = ret_code

end

function AccountDriverTourist:on_unload()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.wakeup_check_handler)

end

-- 登录操作

function AccountDriverTourist:login(on_success, on_failure)
	LOGIN_PLATFROM = 0
print("tourist login")
	if self.account_data.tourist_server_account ~= "" then -- 如果已经成功登录过，则跳过连线登录过程，直接返回成功

print("already login")
		if on_success then on_success() end

	else -- 如果未成功登录过，则链接服务器进行登录
		math.randomseed(os.time())
    	local sex = math.random(0,15)
    	local user_name = global_get_random_name()

		local rpc_param = {

			self.account_data.fast_account,
			0,
			sex,
			user_name,

		}

		RPC:call("SHIN/account/guestLogin", rpc_param, function(status, response)

			if status == RPC.RPC_CALL_SUCCESS then -- 登录成功后，将服务器的账号token记录在本地

				self.account_data.tourist_server_account = response.guid
				PLAYER_ID = self:get_server_account()

				if response.loginType == 0 then
					Player:getInstance():set_springsale_start_time(os.time())

					performWithDelay(cc.Director:getInstance():getNotificationNode(),function() 
						Player:getInstance():set_player_head_id(string.format("head_%02d", sex+1))
						Player:getInstance():set_player_nick_name(user_name)
				    end, 1)

				    global_create_head_cache()

				end

				self:save_account_data()

				if on_success then on_success() end

			else

				if on_failure then on_failure() end

			end

		end)

	end

end

function AccountDriverTourist:logout()



end

function AccountDriverTourist:get_server_account()

	return self.account_data.tourist_server_account

end

-- 初始化账号信息

function AccountDriverTourist:init_account_data()

	local initial_data = {}

	initial_data.tourist_server_account = ""
	initial_data.QQ_server_account = ""
	initial_data.WX_server_account = ""
	initial_data.login_type = 0

	initial_data.fast_account = self:get_device_id() --账户ID，格式为 IMEI + 游戏首次启动时间 + 随机数，无法获取IMEI时第一项为空 （唯一）
	initial_data.device_imei = self:get_device_imei()

	self.account_data = initial_data

	self:save_account_data()

end

-- 存储账号信息

function AccountDriverTourist:save_account_data()

	FileManipulator:getInstance():writeToFile("account_data", "account_data", self.account_data)

end

-- 读取账号信息

function AccountDriverTourist:load_account_data()

	self.account_data = SAFE_REQUIRE_NIL("account_data")

end

-- 获取设备识别码

function AccountDriverTourist:get_device_id()

	if JavaUtils then

		return JavaUtils.get_device_id()

	end

	return string.format("-%d-%d", os.time(), math.random(1, 1000000))

end

function AccountDriverTourist:get_device_imei()

	if JavaUtils then

		return JavaUtils.get_imei()

	end

	return string.format("%d%d", os.time(), math.random(10000, 100000))

end

return AccountDriverTourist