-- 微信登录的代理实现（关于代理，参看AccountManager.lua文件当中的注释）

local MSDK_Platform_WeiXin = require "SDK.MSDK.MSDK_Platform_WeiXin"

AccountDriverWeiXin = class("AccountDriverWeiXin")

function AccountDriverWeiXin:ctor()


end

-- 挂载时初始化操作

function AccountDriverWeiXin:on_load()

	self:load_account_data()

	if not self.account_data then

		self:init_account_data()

	end

	self.login_info = {}
	self.login_info.awakened = false
	self.login_info.platform = MSDK_Interface.ePlatform.ePlatform_None
	self.login_info.callback_flag = MSDK_Interface.CallbackFlag.eFlag_Succ

	self.wakeup_info = {}
	self.wakeup_info.awakened = false
	self.wakeup_info.platform = MSDK_Interface.ePlatform.ePlatform_None
	self.wakeup_info.callback_flag = MSDK_Interface.CallbackFlag.eFlag_Succ

	self.iMSDKApi = MSDK_Platform_WeiXin:create()

	self.iMSDKApi:setLoginCallback(function(platform, ret_code, info) self:on_msdk_login(platform,ret_code,info) end)
	self.iMSDKApi:setWakeupCallback(function(platform, ret_code, info) self:on_msdk_wakeup(platform,ret_code,info) end)

	-- self.server_account = nil

	self.wakeup_check_handler = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:check_wake_up() end, 0.033, false)

end

function AccountDriverWeiXin:on_unload()

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.wakeup_check_handler)

end

-- 登录操作

function AccountDriverWeiXin:login(on_success, on_failure)

	-- MSDK 登录操作

	local _on_success = on_success or function() print("MSDK WeiXin Login Success") end
	local _on_failure = on_failure or function() print("MSDK WeiXin Login Failure") end

	local function link_success_callback()
		if MSDKJava then
			self.iMSDKApi:login()

			local handler = 0
			local overtime = 0
			local function msdk_login_check()

				if self.login_info.awakened then

					self.login_info.awakened = false

					if self.login_info.platform == MSDK_Interface.ePlatform.ePlatform_Weixin then

						if self.login_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_Succ then

							-- 服务器登录操作

							local open_id = self.iMSDKApi:get_open_id()
							local user_name = global_get_random_name()
							local sex = math.random(0,15)
							LOGIN_PLATFROM = 1
							-- todo: 以微信open_id作为账号填入参数，作为账号名来注册或登录

							local rpc_param = {

								open_id,
								1,
								sex,
								user_name,

							}

							-- todo: 修改guestLogin 为对应的注册或登录接口

							RPC:call("SHIN/account/guestLogin", rpc_param, function(status, response)

								if status == RPC.RPC_CALL_SUCCESS then

									self.account_data.WX_server_account = response.guid
									self.account_data.login_type = 1

									PLAYER_ID = self:get_server_account()

									if response.loginType == 0 then
										Player:getInstance():set_springsale_start_time(os.time())
									end
									
									global_set_user_social_head()

								    self:save_account_data()

									_on_success()
									handler = self.login_check_handler
								else

									_on_failure()
									handler = self.login_check_handler
								end

							end)				

						else
							_on_failure()
							handler = self.login_check_handler
						end

					else
						_on_failure()
						handler = self.login_check_handler
					end

				end

				overtime = overtime + 0.1
				if overtime >= 3 then
					_on_failure()
					handler = self.login_check_handler
				end

				cc.Director:getInstance():getScheduler():unscheduleScriptEntry(handler)

			end

			self.login_check_handler = cc.Director:getInstance():getScheduler():scheduleScriptFunc(msdk_login_check, 0.1, false)
		else
			_on_failure()
		end
	end

	RPC:call("SHIN/message/getTimestamp", {}, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
            link_success_callback()

        elseif self.account_data.WX_server_account ~= "" then
        	_on_success()

        else
        	_on_failure()

        end

    end)
end

function AccountDriverWeiXin:logout()

	self.iMSDKApi:logout()

end

function AccountDriverWeiXin:get_server_account()

	return self.account_data.WX_server_account

end

-- 初始化账号信息

function AccountDriverWeiXin:init_account_data()

	local initial_data = {}
	initial_data.tourist_server_account = ""
	initial_data.QQ_server_account = ""
	initial_data.WX_server_account = ""
	initial_data.login_type = 0

	initial_data.fast_account = self:get_device_id() --账户ID，格式为 IMEI + 游戏首次启动时间 + 随机数，无法获取IMEI时第一项为空 （唯一）
	initial_data.device_imei = self:get_device_imei()
	
	self.account_data = initial_data

	self:save_account_data()

end

-- 存储账号信息

function AccountDriverWeiXin:save_account_data()

	FileManipulator:getInstance():writeToFile("account_data", "account_data", self.account_data)

end

-- 读取账号信息

function AccountDriverWeiXin:load_account_data()

	self.account_data = SAFE_REQUIRE_NIL("account_data")

end

-- 获取设备识别码

function AccountDriverWeiXin:get_device_id()

	if JavaUtils then

		return JavaUtils.get_device_id()

	end

	return string.format("-%d-%d", os.time(), math.random(1, 1000000))

end

function AccountDriverWeiXin:get_device_imei()

	if JavaUtils then

		return JavaUtils.get_imei()

	end

	return string.format("%d%d", os.time(), math.random(10000, 100000))

end

function AccountDriverWeiXin:check_wake_up()

	if self.wakeup_info.awakened then

		self.wakeup_info.awakened = false

		if self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_AccountRefresh or 
			self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_Succ then

			--不存在异账号，MSDK已通过刷新接口将本地账号票据刷新	读取LoginRet结构体中的票据进行游戏授权流程
			if self.wakeup_info.platform == MSDK_Interface.ePlatform.ePlatform_Weixin then
				
			elseif self.wakeup_info.platform == MSDK_Interface.ePlatform.ePlatform_QQ then
				
			end

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_UrlLogin then

			--不存在异账号，游戏通过拉起账号快速登录成功	读取LoginRet结构体中的票据进行游戏授权流程
			self.iMSDKApi:url_login(self.wakeup_info.platform)

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_NeedSelectAccount then

			--游戏本地账号和拉起账号存在异账号	弹出对话框让用户选择登录的账号
			self.iMSDKApi:need_select_account(self.wakeup_info.platform)

		elseif self.wakeup_info.callback_flag == MSDK_Interface.CallbackFlag.eFlag_NeedLogin then
			
			--游戏本地账号和拉起账号均无法登录	返回登录界面，引导玩家重新登录授权
			self.iMSDKApi:need_login(self.wakeup_info.platform)

		end

	end

end

function AccountDriverWeiXin:on_msdk_login(platform, ret_code, info)

	self.login_info.awakened = true
	self.login_info.platform = platform
	self.login_info.callback_flag = ret_code

end

function AccountDriverWeiXin:on_msdk_wakeup(platform, ret_code, info)

	self.wakeup_info.awakened = true
	self.wakeup_info.platform = platform
	self.wakeup_info.callback_flag = ret_code

end

return AccountDriverWeiXin