-- AccountManager封装了与账号系统相关的操作，
--
-- AccountManager本身不提相关供操作的实现，而是通过一个driver的代理对象来执行
-- 在调用AccountManager的账号操作接口前需要先调用load_driver加载一个合适的
-- 代理对象。

-- 代理对象需要实现的几个基本接口目前包括：
-- 【on_load】: 当代理对象被AccountManager挂载时调用，一般用于执行初始化操作
-- 【login】: 执行账号登录操作
-- 【get_server_account】: 返回登录成功后服务器返回的账号token

-- 代理对象的实现方式可以参看 AccountDriverTourist.lua 游客登录代理

AccountManager = class("AccountManager")

AccountManager.DefaultDriver = {}
AccountManager.DefaultDriver.on_load = function() end
AccountManager.DefaultDriver.login = function(on_success, on_failure) on_failure() end
AccountManager.DefaultDriver.get_server_account = function() end


function AccountManager:ctor()

	self._driver = AccountManager.DefaultDriver

end

-- 加载代理对象

function AccountManager:load_driver(account_driver)

	if self._driver then

		self._driver:on_unload()

	end

	self._driver = account_driver
	self._driver:on_load()

end

-- 执行账号登录

function AccountManager:login(on_success, on_failure)

	self._driver:login(on_success, on_failure)

end

function AccountManager:logout()

	self._driver:on_unload()
	self._driver:logout()

end

function AccountManager:get_server_account()

	return self._driver:get_server_account()

end

function AccountManager:getInstance()

	if not AccountManager.instance then

		AccountManager.instance = AccountManager:create()

	end

	return AccountManager

end

return AccountManager