local DialogLayer = require "UI.Layer.DialogLayer"

DialogManager = class("DialogManager")

function DialogManager:ctor()

	self.__t_progress = {}
	self.__t_dialog_config = {}

end

function DialogManager:load_dialog_list()

	self.__t_dialog_config = require "GameEx.Resources.DialogConfigs.DialogConfigList"

end

function DialogManager:load_dialog(file_name)

	return require(string.format("GameEx.Resource.DialogConfigs.%s", file_name))

end

function DialogManager:load_progress()

	if not cc.UserDefault:getInstance():getBoolForKey("dialog_progress_saved", false) then self:save_progress() end

	package.loaded["dialog_progress"] = nil

	self.__t_progress = SAFE_REQUIRE "dialog_progress"

	for key, value in pairs(self.__t_dialog_config) do

		value.finished = false

	end

	for key, value in pairs(self.__t_progress) do

		if self.__t_dialog_config[key] then self.__t_dialog_config[key].finished = value end

	end

end

function DialogManager:save_progress()

	cc.UserDefault:getInstance():setBoolForKey("dialog_progress_saved", true)
	FileManipulator:getInstance():writeToFile("dialog_progress", "dialog_progress", self.__t_progress)

end

function DialogManager:clear_progress()

	cc.UserDefault:getInstance():setBoolForKey("dialog_progress_saved", false)
	self.__t_progress = {}
	self:load_progress()

end

function DialogManager:register_event_listener()

	function on_start(event)

		self:on_event("LEVEL_START", event.param)

	end 

	self.__c_start_listener = cc.EventListenerCustom:create("LEVEL_START->DIALOG", on_start)
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__c_start_listener, cc.Director:getInstance():getNotificationNode())	

	function on_finish(event)

		self:on_event("LEVEL_FINISH", event.param)

	end

	self.__c_finish_listener = cc.EventListenerCustom:create("LEVEL_FINISH->DIALOG", on_finish)
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__c_finish_listener, cc.Director:getInstance():getNotificationNode())	

end

function DialogManager:on_event(event_type, event_param)

	for key, config in ipairs(self.__t_dialog_config) do

		if not config.finished and 
			config.toggle_condition.type == event_type and 
			config.toggle_condition.constraint == event_param.constraint then

			event_param.show = false
			config.finished = true
			self.__t_progress[key] = true
			self:save_progress()
			self:play_dialog(config, event_param.func)

			break

		end

	end

end

function DialogManager:lazy_init()

	self:load_dialog_list()
	self:load_progress()
	self:register_event_listener()

end

function DialogManager:play_dialog(config, finish_callback)

	local layer_config = nil
	pcall(function() layer_config = require(string.format("GameEx.Resources.DialogConfigs.%s", config.config_file)) end)
	layer_config = layer_config or config.config
dump(layer_config)
	local layer = DialogLayer:create()
	layer:init_with_config(layer_config)
	layer:set_finish_callback(finish_callback)
	layer:start()

	cc.Director:getInstance():getRunningScene():addChild(layer, 999)

end

function DialogManager:get_instance()

	if not DialogManager.instance then

		DialogManager.instance = DialogManager:create()
		DialogManager.instance:lazy_init()

	end

	return DialogManager.instance

end

function DialogManager:start()



end

return DialogManager