-- 下载器实现
--
-- 下载器采用了cocos的原生实现，由于cocos本身未提供下载器的lua层接口，这里实现了一个简单
-- 的 C to Lua 的转接器，转接器的源码位于项目source/main/DownloaderConnector.cpp文件中
-- DownloaderConnector在lua虚拟机中注册了一个名为CDownloader的函数库。

local Downloader = class("Downloader")

Downloader._cobjs = {} --全局静态表，记录使用中的Downloader对象

Downloader._on_c_progress = function(c_obj, bytes_expected, bytes_recieved, task_id) --提供给C语言层的回调
print("progress")
	local downloader = Downloader._cobjs[c_obj]
	local task_info = downloader:get_task_info_by_id(task_id)
	task_info.on_progress(bytes_expected, bytes_recieved, task_id)

end

Downloader._on_c_failure = function(c_obj, err_code, err_str, task_id) --提供给C语言层的lua回调
print("fail")
	local downloader = Downloader._cobjs[c_obj]
	local task_info = downloader:get_task_info_by_id(task_id)
	task_info.on_failure(err_code, err_str, task_id)

end

Downloader._on_c_success = function(c_obj, task_id) --提供给C语言层的lua回调
print("success")
	local downloader = Downloader._cobjs[c_obj]
	local task_info = downloader:get_task_info_by_id(task_id)
	task_info.on_success(task_id)

end

LUA_DOWNLOADER = Downloader -- C语言层使用的别名

function Downloader:ctor()
	
	self._tasks = {}
	self._cobj = CDownloader.create_instance()
	Downloader._cobjs[self._cobj] = self

end

function Downloader:gen_task_id()

	self.__n_task_id_seed = self.__n_task_id_seed or 0

	self.__n_task_id_seed = self.__n_task_id_seed + 1 

	return tostring(self.__n_task_id_seed)

end

function Downloader:get_task_info_by_id(task_id)

	return self._tasks[task_id]

end

function Downloader:run_task(task_param)

	local task_info = task_param
	self._tasks[task_info.task_id] = task_info

	CDownloader.create_task(self._cobj, task_info.url, task_info.storage_path, task_info.task_id)

end

function Downloader:destory() -- 使用完的Downloader对象需要主动销毁

	Downloader._cobjs[self._cobj] = nil
	CDownloader.destroy_instance(self._cobj)

end

return Downloader 