GameKernel = class("GameKernel")

function GameKernel:getInstance()
	GameKernel.instance = GameKernel.instance or GameKernel:create()
	return GameKernel.instance 
end

function GameKernel:ctor()
	self.init_player_data = require "script/init_player_data"
	self.map_effect_config = nil
	self.scheduleScriptEntryID = nil
	self:init_gift_visible()
end

--地图特效数据
function GameKernel:get_map_effect_config()
	if self.map_effect_config then
		return self.map_effect_config 
	else
		self.map_effect_config  = require "script/map_effect_config.lua"
	end
	return self.map_effect_config
end

-- 玩家数据
function GameKernel:getPlayer()
	return Player:getInstance()
end

function GameKernel:get_init_player_data()
	return self.init_player_data
end

function GameKernel:readPlayerData()
	local player = self:getPlayer()
	player:init_player_data()
	local init_player_data = cc.UserDefault:getInstance():getBoolForKey("init_player_data", true)
	if init_player_data then
		-- 初始玩家数据，并存储起来
		self:writePlayerAllData()
		cc.UserDefault:getInstance():setBoolForKey("init_player_data", false)

		--文件生成时间戳
		player:get_player_base_data().create_timestamp = os.time()
	else
		self:load_player_data()
	end

end

function GameKernel:clear_player_data()
	-- local player = GameKernel:getInstance():getPlayer()
	-- player:init_player_data()
	-- self:writePlayerAllData()

	cc.UserDefault:getInstance():setBoolForKey("init_player_data", true)
	self:readPlayerData()
end

function GameKernel:writePlayerAllData()
	self:write_player_base_data()
	self:write_player_bag_data()
	self:write_player_role_data()
	self:write_player_skin_data()
	self:write_player_book_data()
	self:write_player_mission_data()
	self:write_player_section_data()
	self:write_player_star_reward_data()

	self:write_player_weekly_challenge_data()
	self:write_player_daily_task_data()
	self:write_player_achv_data()
	self:write_player_invite_reward_data()
end

function GameKernel:write_player_base_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_base_data", "player_base_data", player:get_player_base_data())
	cc.UserDefault:getInstance():setBoolForKey("is_player_base_data", true)
end

function GameKernel:write_player_bag_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_bag_data", "player_bag_data", player:get_player_bag_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_bag_data", true)
end

function GameKernel:write_player_role_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_role_data", "player_role_data", player:get_player_role_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_role_data", true)
end

function GameKernel:write_player_skin_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_skin_data", "player_skin_data", player:get_player_skin_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_skin_data", true)
end

function GameKernel:write_player_book_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_book_data", "player_book_data", player:get_player_book_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_book_data", true)
end

function GameKernel:write_player_mission_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_mission_data", "player_mission_data", player:get_player_mission_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_mission_data", true)
end

function GameKernel:write_player_section_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_section_data", "player_section_data", player:get_player_section_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_section_data", true)
end

function GameKernel:write_player_star_reward_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_star_reward_data", "player_star_reward_data", player:get_player_star_reward_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_star_reward_data", true)
end

function GameKernel:write_player_weekly_challenge_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_weekly_challenge_data", "player_weekly_challenge_data", player:get_player_weekly_challenge_info())
	cc.UserDefault:getInstance():setBoolForKey("is_player_weekly_challenge_data", true)
end

function GameKernel:write_player_daily_task_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_daily_task_data", "player_daily_task_data", player:get_player_daily_task_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_daily_task_data", true)
end

function GameKernel:write_player_achv_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_achv_data", "player_achv_data", player:get_player_achv_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_achv_data", true)
end

function GameKernel:write_player_invite_reward_data()
	local player = self:getPlayer()
	FileManipulator:getInstance():writeToFileWithID("player_invite_reward_data", "player_invite_reward_data", player:get_player_invite_reward_info_vector())
	cc.UserDefault:getInstance():setBoolForKey("is_player_invite_reward_data", true)
end


function GameKernel:load_player_data()

	local player = self:getPlayer()

	local is_player_base_data = cc.UserDefault:getInstance():getBoolForKey("is_player_base_data", false)
	if is_player_base_data then
		local player_base_data = SAFE_REQUIRE_WITH_ID "player_base_data"
-- dump(player_base_data)
		-- 读取玩家数据，当不存在该数据是自动补全
		-- 基础数据
		if not(player_base_data.player_gold == nil) then
			player:init_player_gold(player_base_data.player_gold)
		end

		if not(player_base_data.player_jewel == nil) then
			player:init_player_jewel(player_base_data.player_jewel)
		end

		if not(player_base_data.player_power == nil) then
			player:init_player_power(player_base_data.player_power)
		end

		if not(player_base_data.player_left_recover_time == nil) then
			player:init_player_left_recover_time(player_base_data.player_left_recover_time)
		end

		if not(player_base_data.player_cur_role_id == nil) then
			player:init_player_cur_role_id(player_base_data.player_cur_role_id)
		end

		if not(player_base_data.player_cur_skin_id == nil) then
			player:init_player_cur_skin_id(player_base_data.player_cur_skin_id)
		end

		if not(player_base_data.player_head_id == nil) then
			player:init_player_head_id(player_base_data.player_head_id)
		end

		if not(player_base_data.player_nick_name == nil) then
			player:init_player_nick_name(player_base_data.player_nick_name)
		end

		if not(player_base_data.player_card_unsee == nil) then
			player:init_player_card_unsee(player_base_data.player_card_unsee)
		end

		if not(player_base_data.player_addition_max_power == nil) then
			player:init_player_addition_max_power(player_base_data.player_addition_max_power)
		end

		--8.22
		if not(player_base_data.today_is_sign == nil) then
			player:init_today_is_sign(player_base_data.today_is_sign)
		end
		if not(player_base_data.num_sign == nil) then
			player:init_num_sign(player_base_data.num_sign)
		end
		if not(player_base_data.last_sign_day == nil) then
			player:init_last_sign_day(player_base_data.last_sign_day)
		end
		--8.22

		--9.2
		if not(player_base_data.today_is_wheel == nil) then
			player:init_today_is_wheel(player_base_data.today_is_wheel)
		end
		if not(player_base_data.num_wheel == nil) then
			player:init_num_wheel(player_base_data.num_wheel)
		end
		if not(player_base_data.last_wheel_day == nil) then
			player:init_last_wheel_day(player_base_data.last_wheel_day)
		end

		if not(player_base_data.num_view == nil) then
			player:init_num_view(player_base_data.num_view)
		end
		if not(player_base_data.last_view_time == nil) then
			player:init_last_view_time(player_base_data.last_view_time)
		end
		--9.2
		if not(player_base_data.is_buy_fish_gift == nil) then
			player:init_is_buy_fish_gift(player_base_data.is_buy_fish_gift)
		end

		if not(player_base_data.is_buy_springsale == nil) then
			player:init_is_buy_springsale(player_base_data.is_buy_springsale)
		end

		if not(player_base_data.is_buy_limit_gift == nil) then
			player:init_is_buy_limit_gift(player_base_data.is_buy_limit_gift)
		end

		if not(player_base_data.last_daily_task_day == nil) then
			player:init_last_daily_task_day(player_base_data.last_daily_task_day)
		end

		if not(player_base_data.is_create_lucky_gift == nil) then
			player:init_is_create_lucky_gift(player_base_data.is_create_lucky_gift)
		end

		if not(player_base_data.continue_pass_stage == nil) then
			player:init_continue_pass_stage(player_base_data.continue_pass_stage)
		end

		if not(player_base_data.springsale_start_time == nil) then
			player:init_springsale_start_time(player_base_data.springsale_start_time)
		end

		if not(player_base_data.continue_pass_stage == nil) then
			player:init_continue_pass_stage(player_base_data.continue_pass_stage)
		end

		--
		if not(player_base_data.last_luckygift_day == nil) then
			player:init_last_luckygift_day(player_base_data.last_luckygift_day)
		end
		if not(player_base_data.num_luckygift_buy == nil) then
			player:init_num_luckygift_buy(player_base_data.num_luckygift_buy)
		end
		if not(player_base_data.num_luckygift_again == nil) then
			player:init_num_luckygift_again(player_base_data.num_luckygift_again)
		end

		player.player_base_data.already_opened = player_base_data.already_opened
		player.player_base_data.fast_account = player_base_data.fast_account
		player.player_base_data.user_name = player_base_data.user_name

		player.player_base_data.create_timestamp = player_base_data.create_timestamp

		player.player_base_data.num_share = player_base_data.num_share or 1
		player.player_base_data.last_share_day = player_base_data.last_share_day or 0

	end

	-- 玩家背包数据
	local is_player_bag_data = cc.UserDefault:getInstance():getBoolForKey("is_player_bag_data", false)
	if is_player_bag_data then
		local player_bag_data = SAFE_REQUIRE_WITH_ID "player_bag_data"
		for i=1,#player_bag_data do
			local player_bag_info = player:get_player_bag_info_by_id(player_bag_data[i].item_id)
			if not(player_bag_info == nil) then
				if not(player_bag_data[i].item_id == nil) then
					player_bag_info.item_id = player_bag_data[i].item_id
				end
				if not(player_bag_data[i].item_num == nil) then
					player_bag_info.item_num = player_bag_data[i].item_num
				end		
				if not(player_bag_data[i].limited_time_item_vector == nil) then
					player_bag_info.limited_time_item_vector = player_bag_data[i].limited_time_item_vector
				end	

				player:set_player_bag_info_by_id(player_bag_info.item_id,player_bag_info)
			end
		end
	end

	-- 玩家角色数据
	local is_player_role_data = cc.UserDefault:getInstance():getBoolForKey("is_player_role_data", false)
	if is_player_role_data then
		local player_role_data = SAFE_REQUIRE_WITH_ID "player_role_data"
		for i=1,#player_role_data do
			local player_role_info = player:get_player_role_info_by_id(player_role_data[i].role_id)
			if not(player_role_info == nil) then
				if not(player_role_data[i].role_id == nil) then
					player_role_info.role_id = player_role_data[i].role_id
				end
				if not(player_role_data[i].skin_id == nil) then
					player_role_info.skin_id = player_role_data[i].skin_id
				end
				if not(player_role_data[i].role_level == nil) then
					player_role_info.role_level = player_role_data[i].role_level
				end
				if not(player_role_data[i].is_unlock == nil) then
					player_role_info.is_unlock = player_role_data[i].is_unlock
				end
				player:set_player_role_info_by_id(player_role_info.role_id, player_role_info)
			end
		end
	end

	-- 皮肤数据
	local is_player_skin_data = cc.UserDefault:getInstance():getBoolForKey("is_player_skin_data", false)
	if is_player_skin_data then
		local player_skin_data = SAFE_REQUIRE_WITH_ID "player_skin_data"
		for i=1,#player_skin_data do
			local player_skin_info = player:get_player_skin_info_by_id(player_skin_data[i].skin_id)
			if not(player_skin_info == nil) then
				if not(player_skin_data[i].skin_id == nil) then
					player_skin_info.skin_id = player_skin_data[i].skin_id
				end
				if not(player_skin_data[i].is_unlock == nil) then
					player_skin_info.is_unlock = player_skin_data[i].is_unlock
				end		

				player:set_player_skin_info_by_id(player_skin_info.skin_id, player_skin_info)
			end
		end
	end

	-- 纪念册数据
	local is_player_book_data = cc.UserDefault:getInstance():getBoolForKey("is_player_book_data", false)
	if is_player_book_data then
		local player_book_data = SAFE_REQUIRE_WITH_ID "player_book_data"
		for i=1,#player_book_data do
			local player_book_info = player:get_player_book_info_by_id(player_book_data[i].book_id)
			if not(player_book_info == nil) then
				if not(player_book_data[i].book_id == nil) then
					player_book_info.book_id = player_book_data[i].book_id
				end
				if not(player_book_data[i].is_get == nil) then
					player_book_info.is_get = player_book_data[i].is_get
				end		
				if not(player_book_data[i].is_see == nil) then
					player_book_info.is_see = player_book_data[i].is_see
				end	

				player:set_player_book_info_by_id(player_book_info.book_id, player_book_info)
			end
		end
	end

	-- 关卡数据
	local is_player_mission_data = cc.UserDefault:getInstance():getBoolForKey("is_player_mission_data", false)
	if is_player_mission_data then
		local player_mission_data = SAFE_REQUIRE_WITH_ID "player_mission_data"
		for i=1,#player_mission_data do
			local player_mission_info = player:get_player_mission_info_by_id(player_mission_data[i].mission_id)
			if not(player_mission_info == nil) then
				if not(player_mission_data[i].mission_id == nil) then
					player_mission_info.mission_id = player_mission_data[i].mission_id
				end
				if not(player_mission_data[i].is_enable == nil) then
					player_mission_info.is_enable = player_mission_data[i].is_enable
				end	
				if not(player_mission_data[i].is_pass == nil) then
					player_mission_info.is_pass = player_mission_data[i].is_pass
				end	
				if not(player_mission_data[i].is_max_star == nil) then
					player_mission_info.is_max_star = player_mission_data[i].is_max_star
				end	
				if not(player_mission_data[i].star_count == nil) then
					player_mission_info.star_count = player_mission_data[i].star_count
				end	
				if not(player_mission_data[i].max_score == nil) then
					player_mission_info.max_score = player_mission_data[i].max_score
				end	

				if not(player_mission_data[i].pass_count == nil) then
					player_mission_info.pass_count = player_mission_data[i].pass_count
				end
				if not(player_mission_data[i].is_move_action == nil) then
					player_mission_info.is_move_action = player_mission_data[i].is_move_action
				end	

				if not(player_mission_data[i].fail_times == nil) then
					player_mission_info.fail_times = player_mission_data[i].fail_times
				end
				if not(player_mission_data[i].is_last_level_get == nil) then
					player_mission_info.is_last_level_get = player_mission_data[i].is_last_level_get
				end
				
				player:set_player_mission_info_by_id(player_mission_info.mission_id, player_mission_info)
			end
		end
	end

	-- 章节数据
	local is_player_section_data = cc.UserDefault:getInstance():getBoolForKey("is_player_section_data", false)
	if is_player_section_data then
		local player_section_data = SAFE_REQUIRE_WITH_ID "player_section_data"
		for i=1,#player_section_data do
			local player_section_info = player:get_player_section_info_by_id(player_section_data[i].section_id)
			if not(player_section_info == nil) then
				if not(player_section_data[i].section_id == nil) then
					player_section_info.section_id = player_section_data[i].section_id
				end
				if not(player_section_data[i].is_enable == nil) then
					player_section_info.is_enable = player_section_data[i].is_enable
				end	
				if not(player_section_data[i].is_animation == nil) then
					player_section_info.is_animation = player_section_data[i].is_animation
				end			
				player:set_player_section_info_by_id(player_section_info.section_id, player_section_info)
			end
		end
	end

	-- 星星奖励数据
	local is_player_star_reward_data = cc.UserDefault:getInstance():getBoolForKey("is_player_star_reward_data", false)
	if is_player_star_reward_data then
		local player_star_reward_data = SAFE_REQUIRE_WITH_ID "player_star_reward_data"
		for i=1,#player_star_reward_data do
			local player_star_reward_info = player:get_player_star_reward_info_by_id(player_star_reward_data[i].reward_id)
			if not(player_star_reward_info == nil) then
				if not(player_star_reward_data[i].reward_id == nil) then
					player_star_reward_info.reward_id = player_star_reward_data[i].reward_id
				end
				if not(player_star_reward_data[i].is_get == nil) then
					player_star_reward_info.is_get = player_star_reward_data[i].is_get
				end			
				player:set_player_star_reward_info_by_id(player_star_reward_info.reward_id, player_star_reward_info)
			end
		end
	end

	-- 周赛数据
	local is_player_weekly_challenge_data = cc.UserDefault:getInstance():getBoolForKey("is_player_weekly_challenge_data", false)
	if is_player_weekly_challenge_data then
		local player_weekly_challenge_data = SAFE_REQUIRE_WITH_ID "player_weekly_challenge_data"
		local player_weekly_challenge_info = player:get_player_weekly_challenge_info()
		if not(player_weekly_challenge_info == nil) then
			if not(player_weekly_challenge_data.today_challenge_num == nil) then
				player_weekly_challenge_info.today_challenge_num = player_weekly_challenge_data.today_challenge_num
			end
			if not(player_weekly_challenge_data.go_to_play_times == nil) then
				player_weekly_challenge_info.go_to_play_times = player_weekly_challenge_data.go_to_play_times
			end
			if not(player_weekly_challenge_data.add_friend_times == nil) then
				player_weekly_challenge_info.add_friend_times = player_weekly_challenge_data.add_friend_times
			end
			
			if not(player_weekly_challenge_data.conch_count == nil) then
				player_weekly_challenge_info.conch_count = player_weekly_challenge_data.conch_count
			end
			if not(player_weekly_challenge_data.conch_once_num == nil) then
				player_weekly_challenge_info.conch_once_num = player_weekly_challenge_data.conch_once_num
			end

			if not(player_weekly_challenge_data.lase_challenge_day == nil) then
				player_weekly_challenge_info.lase_challenge_day = player_weekly_challenge_data.lase_challenge_day
			end
			if not(player_weekly_challenge_data.lase_challenge_week == nil) then
				player_weekly_challenge_info.lase_challenge_week = player_weekly_challenge_data.lase_challenge_week
			end
			
			if not(player_weekly_challenge_data.today_reward_info == nil) then
				player_weekly_challenge_info.today_reward_info = player_weekly_challenge_data.today_reward_info
			end
			if not(player_weekly_challenge_data.week_reward_info == nil) then
				player_weekly_challenge_info.week_reward_info = player_weekly_challenge_data.week_reward_info
			end

			player:set_player_weekly_challenge_info(player_weekly_challenge_info)
		end
	end

	-- 每日任务数据
	local is_player_daily_task_data = cc.UserDefault:getInstance():getBoolForKey("is_player_daily_task_data", false)
	if is_player_daily_task_data then
		local player_daily_task_data = SAFE_REQUIRE_WITH_ID "player_daily_task_data"
		for i=1,#player_daily_task_data do
			local player_daily_task_info = player:get_player_daily_task_info_by_id(player_daily_task_data[i].task_id)
			if not(player_daily_task_info == nil) then
				if not(player_daily_task_data[i].task_id == nil) then
					player_daily_task_info.task_id = player_daily_task_data[i].task_id
				end
				if not(player_daily_task_data[i].finish_count == nil) then
					player_daily_task_info.finish_count = player_daily_task_data[i].finish_count
				end	
				if not(player_daily_task_data[i].is_get == nil) then
					player_daily_task_info.is_get = player_daily_task_data[i].is_get
				end
				if not(player_daily_task_data[i].is_choose == nil) then
					player_daily_task_info.is_choose = player_daily_task_data[i].is_choose
				end
				if not(player_daily_task_data[i].target_type == nil) then
					player_daily_task_info.target_type = player_daily_task_data[i].target_type
				end

				player:set_player_daily_task_info_by_id(player_daily_task_info.task_id, player_daily_task_info)
			end
		end
	end

	-- 成就任务数据
	local is_player_achv_data = cc.UserDefault:getInstance():getBoolForKey("is_player_achv_data", false)
	if is_player_achv_data then
		local player_achv_data = SAFE_REQUIRE_WITH_ID "player_achv_data"
		for i=1,#player_achv_data do
			local player_achv_info = player:get_player_achv_info_by_id(player_achv_data[i].achv_id)
			if not(player_achv_info == nil) then
				if not(player_achv_data[i].achv_id == nil) then
					player_achv_info.achv_id = player_achv_data[i].achv_id
				end

				if not(player_achv_data[i].finish_count == nil) then
					player_achv_info.finish_count = player_achv_data[i].finish_count
				end	
				if not(player_achv_data[i].stage_info_vector == nil) then
					player_achv_info.stage_info_vector = player_achv_data[i].stage_info_vector
				end

				player:set_player_achv_info_by_id(player_achv_info.achv_id, player_achv_info)
			end
		end
	end

	-- 邀请奖励数据
	local is_player_invite_reward_data = cc.UserDefault:getInstance():getBoolForKey("is_player_invite_reward_data", false)
	if is_player_invite_reward_data then
		local player_invite_reward_data = SAFE_REQUIRE_WITH_ID "player_invite_reward_data"
		for i=1,#player_invite_reward_data do
			local player_invite_reward_info = player:get_player_invite_reward_info_by_id(player_invite_reward_data[i].invite_id)
			if not(player_invite_reward_info == nil) then
				if not(player_invite_reward_data[i].invite_id == nil) then
					player_invite_reward_info.invite_id = player_invite_reward_data[i].invite_id
				end
				if not(player_invite_reward_data[i].is_get == nil) then
					player_invite_reward_info.is_get = player_invite_reward_data[i].is_get
				end
				player:set_player_invite_reward_info_by_id(player_invite_reward_info.invite_id, player_invite_reward_info)
			end
		end
	end

end

function GameKernel:loadResourceKey()
	ResourceManager:getInstance()
end

function GameKernel:run_time()
	if self.scheduleScriptEntryID then
		return
	end
	local scheduler = cc.Director:getInstance():getScheduler()
	self.scheduleScriptEntryID = scheduler:scheduleScriptFunc(update_power_time, 1.0, false)
end

function update_power_time()
	local player = GameKernel:getInstance():getPlayer()

	--判断限时道具消失
	local current_time = os.time()
	local bag_info_limited_time_vector = player:get_player_bag_info_limited_time_vector()
	local items_count = table.getn(bag_info_limited_time_vector)
	for i=1,items_count do
		local get_time = bag_info_limited_time_vector[i].limited_time_item_vector[1].get_time
		local diff = current_time - get_time
		if diff >= 86400 then 	--24*3600
			table.remove(bag_info_limited_time_vector[i].limited_time_item_vector,1)
			bag_info_limited_time_vector[i].item_num = 0
			for k,v in pairs(bag_info_limited_time_vector[i].limited_time_item_vector) do
				bag_info_limited_time_vector[i].item_num = bag_info_limited_time_vector[i].item_num + v.num
			end
			GameKernel:getInstance():write_player_bag_data()
			--分发限时道具消失事件
			cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.remove_limited_time_item_from_game)
			cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.remove_limited_time_item_from_bag)
		end
	end

	local is_Unlimit_Power = cc.UserDefault:getInstance():getBoolForKey("is_Unlimit_Power", false)
	if player:is_full_power() and is_Unlimit_Power == false then
	else
		local last_time = player:get_player_last_time()
		local now_time = os.time()
		local sub_time = now_time - last_time
		if sub_time >= 0 then
			local add_power = math.floor((sub_time/player:get_player_recover_time()))
			local left_time = sub_time - player:get_player_recover_time() * add_power
			local left_recover_time = player:get_player_left_recover_time()
			if left_time >= left_recover_time then
				add_power = add_power + 1

				player:add_player_left_cover_time(-left_time)
			else
				player:add_player_left_cover_time(-left_time)
			end

			--7.21
			if is_Unlimit_Power then
				--当前的时间-无限体力开启的时间，判断差值>=3600秒
				local tm_current = os.time()
				local tm_first = cc.UserDefault:getInstance():getIntegerForKey("tm_first", 1)
				local tm_value = tm_current - tm_first

				local time_limit2 = cc.UserDefault:getInstance():getIntegerForKey("tm_limit2") --2表示两个小时 113
				if time_limit2 then
					time_limit2 = 3600 * 2
				else
					time_limit2 = 3600
				end
				-- print(time_limit2)
				if tm_value >= time_limit2 then					
					cc.UserDefault:getInstance():setBoolForKey("is_Unlimit_Power", false)
					player:set_player_power(player:get_player_max_power())
					Player:getInstance():set_player_left_cover_time(Player:getInstance():get_player_recover_time())
				end	
			end
			--7.21

			if player:get_player_power() < player:get_player_max_power() then
				if player:get_player_power() + add_power >= player:get_player_max_power() then
					player:add_player_power(player:get_player_max_power() - player:get_player_power())
				else
					player:add_player_power(add_power)
				end
			else
				player:add_player_power(0)
			end
		else
			cc.UserDefault:getInstance():setBoolForKey("is_Unlimit_Power", false)
			Player:getInstance():set_player_left_cover_time(Player:getInstance():get_player_recover_time())
			player:set_player_power(player:get_player_power())
		end
		player:set_player_last_time(os.time())
	end
end

function GameKernel:getNetworkTime(async_callback)
	local timeStamp = os.time()
	local is_link = false

	RPC:call("SHIN/message/getTimestamp", {}, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

            timeStamp = response
            is_link = true
            if async_callback then
            	async_callback(timeStamp)
            end
            return timeStamp, is_link
        else

            print("request error")
            return timeStamp, is_link
        end

    end)
    
end

function GameKernel:get_is_link_net(link_callback)
	
	self.__c_mask_layer = CommunicatingMask:create()
	cc.Director:getInstance():getRunningScene():addChild(self.__c_mask_layer, 99999)
	
	local is_link = false

	RPC:call("SHIN/message/getTimestamp", {}, function(status, response)
		if self.__c_mask_layer then 
			self.__c_mask_layer:removeFromParent()
			self.__c_mask_layer = nil
		end
        
        if status == RPC.RPC_CALL_SUCCESS then

        	global_check_remote_archive(function()end)
            -- dump(response)
            is_link = true
            if link_callback then
            	link_callback(response)
            end
            return is_link
        else

            print("request error")
            local warningLayer = WarningLayer:create()
	        warningLayer:set_text_string("请检查你的网络连接！")
	        LayerManager:getInstance():add_layer_to_scene(warningLayer)
            return is_link
        end

    end)

end

function GameKernel:init_gift_visible()
	self.is_gift_visible = false

	local rpc_param = {
        1,

    }
	RPC:call("SHIN/channel/gift_visible", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
        	if response == "1" then
        		self.is_gift_visible = true
        	end

        else

            print("request error")
        end

    end)
    
end

function GameKernel:get_gift_visible()

	return self.is_gift_visible

end

return GameKernel
