require "core.Book_Data"
Mission_Data = class("Mission_Data")

function Mission_Data:ctor()
	self.mission_info_vector = require "script/mission_info.lua"
	self.mission_info_vector_id = {}

	for k,v in pairs(self.mission_info_vector) do
		self.mission_info_vector_id[v.mission_id] = v                   -- 关卡id表
	end
end

function Mission_Data:get_mission_info_vector()
	return self.mission_info_vector
end

function Mission_Data:get_mission_info_by_id(mission_id)
	if self.mission_info_vector_id[mission_id] then
		return self.mission_info_vector_id[mission_id]
	else
		return self:create_new_mission_by_id(mission_id)
	end
end

function Mission_Data:get_section_mission_count(section)
	local mission_count = 0
	for i=1,#self.mission_info_vector do
		if self.mission_info_vector[i].mission_section == section then
			mission_count = mission_count + 1
		end
	end
	return mission_count
end

-- function Mission_Data:get_mission_info_by_level(level)
-- 	if level <= 0 then
-- 		for k,v in pairs(self.mission_info_vector) do
-- 			if v.mission_level == level then
-- 				return v
-- 			end
-- 		end
-- 	end
-- 	for i=1,#self.mission_info_vector do
-- 		if self.mission_info_vector[i].mission_level == level then
-- 			return self.mission_info_vector[i]
-- 		end
-- 	end
-- 	--return self:create_new_mission_by_level(level)
-- end

function Mission_Data:get_mission_info_by_level(level)
	if level <= 0 then
		for k,v in pairs(self.mission_info_vector) do
			if v.mission_level == level then
				return v
			end
		end
	end
	for i=1,#self.mission_info_vector do
		if self.mission_info_vector[i].mission_level == level then
			return self.mission_info_vector[i]
		end
	end
	--return self:create_new_mission_by_level(level)
end
function Mission_Data:create_new_mission_by_id(mission_id)
	local str = mission_id
    local index = string.find(str, '_')
    local str1 = string.sub(str, index + 1, -1)
    local level = tonumber(str1)
	local mission_info = {
		mission_id = mission_id,
		mission_level = level,
		mission_section = math.ceil(level/20.0),
		star_goal = self.mission_info_vector[1].star_goal,
		reward_rate = self.mission_info_vector[1].reward_rate,
		reward_gold = self.mission_info_vector[1].reward_gold,
		second_gold = self.mission_info_vector[1].second_gold,
		third_gold = self.mission_info_vector[1].third_gold,
	}
	return mission_info
end

function Mission_Data:create_new_mission_by_level(mission_level)
	local mission_id = string.format("mission_%03d", mission_level)
	local mission_info = {
		mission_id = mission_id,
		mission_level = mission_level,
		mission_section = math.ceil(mission_level/20.0),
		star_goal = self.mission_info_vector[1].star_goal,
		reward_rate = self.mission_info_vector[1].reward_rate,
		reward_gold = self.mission_info_vector[1].reward_gold,
		second_gold = self.mission_info_vector[1].second_gold,
		third_gold = self.mission_info_vector[1].third_gold,
	}
	return mission_info
end

-- 通过关卡获取下一关
function Mission_Data:get_next_mission_info_by_level(level)
	for i=1,#self.mission_info_vector do
		if self.mission_info_vector[i].mission_level == level then
			if self.mission_info_vector[i+1] then
				return self.mission_info_vector[i+1]
			else
				return self.mission_info_vector[i]
			end
		end
	end
end

-- 通过关卡和章节获取当前章节的第几关
function Mission_Data:get_mission_cur_level(level, section)
	local cur_level = level
	for i=1,section do
		if i < section then
			cur_level = cur_level - self:get_section_mission_count(i)
		end
	end
	return cur_level
end

-- 通过当前章节的第几关和章节获取关卡
function Mission_Data:get_mission_level(level, section)
	local mission_level = 0
	for i=1,section do
		if i < section then
			mission_level = mission_level + self:get_section_mission_count(i)
		end
	end
	mission_level = mission_level + level
	return mission_level
end

function Mission_Data:is_cur_section_last_level(level, section)
	local cur_level = self:get_mission_cur_level(level, section)
	if cur_level == self:get_section_mission_count(section) then
		return true
	end
	return false
end

function Mission_Data:is_last_level(level)
	for i=1,#self.mission_info_vector do
		if self.mission_info_vector[i].mission_level == level then
			if self.mission_info_vector[i+1] then
				return false
			else
				return true
			end
		end
	end
end

function Mission_Data:get_generate_card_by_mission(mission_id)
	local book_info_vetor = Book_Data:getInstance():get_book_info_vector()
	for i=1,#book_info_vetor do
		if book_info_vetor[i].generate_mission_id == mission_id then
			return book_info_vetor[i]
		end
	end
	return nil
end

function Mission_Data:getInstance()
	Mission_Data.instance = Mission_Data.instance or Mission_Data:create()
	return Mission_Data.instance
end

return Mission_Data 