local engine = cc.SimpleAudioEngine:getInstance()

-- 默认音量大小
local DEFAULT_BG_VOLUME = 50
local DEFAULT_AUDIO_VOLUME = 80
local DEFAULT_EFFECT_VOLUME = 50

local bg_enable = cc.UserDefault:getInstance():getBoolForKey("music_enable", true)
local effect_enable = cc.UserDefault:getInstance():getBoolForKey("effect_enable", true)
local curMusicStr = ""

MusicManager = class("MusicManager")

function MusicManager:ctor()

	self.__t_play_tags = {}

	self.__c_schedule_node = cc.Node:create()

	self.__f_stop_duplicate = function(dt)

		for key, value in pairs(self.__t_play_tags) do

			self.__t_play_tags[key] = self.__t_play_tags[key] - dt

			if self.__t_play_tags[key] <= 0 then

				self.__t_play_tags[key] = nil

			end
 
		end

	end

	self.__c_schedule_node:scheduleUpdateWithPriorityLua(self.__f_stop_duplicate, 0)

	cc.Director:getInstance():getNotificationNode():addChild(self.__c_schedule_node)	

end

function MusicManager:getInstance()
	MusicManager.instance = MusicManager.instance or MusicManager:create()
	return MusicManager.instance
end

-- 背景音乐
function MusicManager.set_music_volume(volume)
	engine:setMusicVolume(volume)
end

function MusicManager.getBgMusicEnable()
	return bg_enable
end

function MusicManager.setBgMusicEnable(flag)
	bg_enable = flag 
	cc.UserDefault:getInstance():setBoolForKey("music_enable", bg_enable)
	engine:playMusic(curMusicStr, true)
	if bg_enable then
		engine:resumeMusic()
	else
		engine:pauseMusic()
	end
end

function MusicManager.play_bg_music(file_path, is_loop)
	engine:stopMusic(true)
	curMusicStr = file_path
	engine:playMusic(file_path, is_loop)
	if bg_enable then
		engine:resumeMusic()
	else
		engine:pauseMusic();
	end
end

function MusicManager.stop_bg_music()	
	engine:stopMusic(true)
end

function MusicManager.resume_bg_music()
	engine:resumeMusic()
end

function MusicManager.preload_music(filename)
	engine:preloadMusic(filename)
end

-- 音效
function MusicManager.getEffectEnable()
	return effect_enable
end


function MusicManager.setEffectEnable(flag)
	effect_enable = flag 
	cc.UserDefault:getInstance():setBoolForKey("effect_enable", effect_enable)
	engine:stopAllEffects()
end

function MusicManager.play_effect(file_path, is_loop)
	if effect_enable then
		-- local effectId = ccexp.AudioEngine:play2d(file_path, is_loop, 1.0)
		-- return effectId
		audio.playSound(file_path, is_loop)
	end
end

function MusicManager.stop_effect(handle)
	engine:stopEffect(handle)
end

function MusicManager.stop_all_efeec()
	engine:stopAllEffects()
end

function MusicManager.preloadEffect(pszFilePath)
	audio.preloadSound(pszFilePath)
end

function MusicManager.play_2d(filePath)
--print(filePath)
	--ccexp.AudioEngine:play2d(filePath, false, 1.0)
	audio.playSound(filePath, false)

end

function MusicManager:play_sound(sound_key, conflict_window)

	if not self:check_conflict(sound_key) then

		self.__t_play_tags[sound_key] = conflict_window or 0.1

		self.play_effect(ResourceManager:getInstance():getAudioPathByKey(sound_key), false)

	end

end

function MusicManager:check_conflict(sound_key)

	if sound_key and MusicManager.CONFLICT_LIST[sound_key] then

		for _, key in ipairs(MusicManager.CONFLICT_LIST[sound_key]) do

			if self.__t_play_tags[key] then

				return true

			end

		end

		return false

	else

		return false

	end

end

MusicManager.CONFLICT_LIST = {
	
	["VOICE_good"] = {

		"VOICE_cross",

	},

	["VOICE_lightning"] = {

		"VOICE_good",
		"VOICE_great",
		"VOICE_perfect",
		"VOICE_excellent",

	},

	["Combo_Voice_Audio"] = {

		"Combo_Voice_Audio",

	},

	["Goal_Audio"] = {

		"Goal_Audio",

	},	

}