local JSON = require "core.JSON"

RPC = class("RPC")

RPC.RPC_CALL_SUCCESS = 0;
RPC.RPC_CALL_FAILURE = 1;
RPC.RPC_CALL_RUNNING = 2;

RPC.REMOTES = {}
RPC.MODULES = {}
RPC.targets = {}

RPC.sessions = {}
RPC.default_driver = require "core.RPCHttp_buildin"
RPC.c_driver = nil

function RPC:ctor()


end

function RPC:get_instance()
    
    if not RPC.instance then

        RPC.instance = RPC:create()

    end

    return RPC.instance

end

function RPC:init()

    self:init_config()
    self:init_driver()
    self:init_scheduler()
    self.targets = {}

end

function RPC:init_driver()

    RPC.c_driver = RPCHttp_Jni or RPC.default_driver

end

function RPC:init_scheduler()

    cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:check_session() end, 0.1, false)

end

function RPC:check_session()

    for id, session in pairs(RPC.sessions) do

        if session.state ~= RPC.RPC_CALL_RUNNING then
print(string.format("session: %s completed !!!", id))
print(string.format("session status: %d", session.state))

            local response_content = nil
            local state = session.state

            if state == RPC.RPC_CALL_SUCCESS then

                local success, response = pcall(function()
print("result json: " .. session.response)
                    -- local commonPopupLayer = CommonPopupLayer:create()
                    -- commonPopupLayer:showMessage("log", session.response, "enter", function()
                    --     commonPopupLayer:close_self()
                    -- end, nil)
                    return JSON:decode(session.response)

                end)

                if success then

                    if response.errCode == 0 then

                        response_content = response.result

                    else

                        state = RPC.RPC_CALL_FAILURE

                    end

                else
                   
                   state = RPC.RPC_CALL_FAILURE 

                end

            end

            if session.callback then

                if session.target then

                    if RPC.targets[session.target] then

                        session.callback(state, response_content, id)

                    end

                else

                    session.callback(state, response_content, id)

                end

            end

            RPC.sessions[id] = nil

        end

    end

end

function RPC:call(route, param, on_return)

    local rpc_session_id = self:gen_session_id()
    local url = self:parse_route(route)
    local serialized_param = self:serialize_param(param)
    local session = {

        id = rpc_session_id,
        callback = on_return,
        state = RPC.RPC_CALL_RUNNING,
        response = nil,

    }
    
    -- local commonPopupLayer = CommonPopupLayer:create()
    -- commonPopupLayer:showLog("param", serialized_param, "enter", function()
    --     commonPopupLayer:close_self()
    -- end, nil)

print(string.format("session_id: %s", rpc_session_id)) print(string.format("url: %s", url)) print(string.format("param: %s", serialized_param))
    RPC.sessions[rpc_session_id] = session

    RPC.c_driver.call(rpc_session_id, url, serialized_param)

    return rpc_session_id

end

function RPC:call_bind(route, param, target, on_return)

    local rpc_session_id = self:gen_session_id()
    local url = self:parse_route(route)
    local serialized_param = self:serialize_param(param)
    local session = {

        id = rpc_session_id,
        callback = on_return,
        state = RPC.RPC_CALL_RUNNING,
        response = nil,
        target = target,

    }
print(string.format("session_id: %s", rpc_session_id)) print(string.format("url: %s", url)) print(string.format("param: %s", serialized_param))
    RPC.sessions[rpc_session_id] = session

    RPC.c_driver.call(rpc_session_id, url, serialized_param)

    return rpc_session_id

end

function RPC:call_event(route, param, event_name)

    local rpc_session_id = self:gen_session_id()
    local url = self:parse_route(route)
    local serialized_param = self:serialize_param(param)
    local on_return = function(status, response, session_id)

        local event = cc.EventCustom:new( RPC.sessions[session_id].event_name or "RPC_RETURN" )
        event.status = status
        event.response = response
        event.session_id = session_id

        cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

    end

    local session = {

        id = rpc_session_id,
        callback = on_return,
        state = RPC.RPC_CALL_RUNNING,
        response = nil,
        event_name = event_name,

    }

    RPC.sessions[rpc_session_id] = session

    RPC.c_driver.call(rpc_session_id, url, serialized_param)

    return rpc_session_id    

end

function RPC:register_target(target)

    if target then

        RPC.targets[target] = true

    end

end

function RPC:unregister_target(target)

    if target then

        RPC.targets[target] = nil

    end

end

function RPC:gen_session_id()

    RPC.session_id_seed = RPC.session_id_seed or 0
    RPC.session_id_seed = RPC.session_id_seed + 1

    return tostring(RPC.session_id_seed)

end

function RPC:serialize_param(param)

    local _param = {}

    for index, value in ipairs(param) do

        _param[tostring(index)] = value

    end

    return JSON:encode(_param)

end

function RPC:parse_route(route)

    local route_breakups = SPLIT_STRING(route, "/")

    local remote = RPC.REMOTES[route_breakups[1]]
    local _module = RPC.MODULES[route_breakups[2]]
    local func = _module[route_breakups[3]]

    local url = string.format("%s?handler=%s&findex=%d", remote, route_breakups[2], func)

    return url

end

function RPC.return_to_lua(rpc_session_id, status, response)

    local session = RPC.sessions[rpc_session_id]

    if session then

        session.state = status
        session.response = response

    end

end

function RPC.init_config()

    RPC.REMOTES = {

        -- ["SHIN"] = "http://192.168.1.80:8089/interface.php",  --测试服
        ["SHIN"] = "http://xiaoxin.dayugame.com/interface.php",  --正式服
    }

    RPC.MODULES = {

        ["account"] = {
            ["charinfo"] = 2005,
            ["guestLogin"] = 2007,
            ["archive"] = 2008,
            ["loadArchive"] = 2009,
            ["modifyName"] = 2010,
        },

        ["message"] = {

            ["newMsg"] = 2011,
            ["getMsgList"] = 2012,
            ["handleMsg"] = 2015,
            ["getTimestamp"] = 2016,
            ["getMailList"] = 2017,
            ["readMail"] = 2018,
            ["createRewardMail"] = 2019,
            ["createNotifyMail"] = 2020,

        },

        ["friend"] = {

            ["unfriend"] = 2023,
            ["recommendFriends"] = 2026,
            ["searchFriendById"] = 2027,
            ["getFriendlist"] = 2028,
            ["send_invite_msg"] = 2029,
            ["getPersonalInviteState"] = 2030,
        },

        ["chapter"] = {
            ["updateChapter"] = 2042,
            ["getfriendRanklist"] = 2043,
        },
        ["chapterMgr"] = {
            ["getRanklist"] = 2072,
        },

        ["seaVentureMgr"] = {
            ["loadRanklist"] = 2051,
            ["showRanklist"] = 2052,
            ["getRanklist"] = 2053,
            ["ranklistSort"] = 2054,
        },
        ["seaVenture"] = {
            ["doVenture"] = 2061,
            ["getfriendRanklist"] = 2062,
        },
        ["sns"] = {
            ["createOrder"] = 2073,
            ["refreshOrder"] = 2074,
        },
        ["channel"] = {
            ["gift_visible"] = 2075,
        },
        ["code"] = {
            ["redeem"] = 2077,
            ["redeem_2"] = 2078,
        },

        ["user"] = {
            ["getUserData"] = 2079,
            ["updateDiamond"] = 2080,
            ["updateCoin"] = 2081,
            ["updateHeadUrl"] = 2082,
            ["updateMdata"] = 2083,
        },
        ["share"] = {
            ["statsShareCount"] = 2085,
        },

    }

end
