-- RemoveArchiveManager 在线存档功能管理器
--
-- 

RemoteArchiveManager = class("RemoteArchiveManager")

-- 将存档分成几个独立的部分，方便分开写入
RemoteArchiveManager.SECTION = {
	
	["BASE"] = 1,
	["ACHIVEMENT"] = 2,
	["CARD"] = 3,
	["MISSION"] = 4,
	["STAR_REWARD"] = 5,
	["ROLE"] = 6,
	["BAG"] = 7,
	["SKIN"] = 8,
	["WEEK_GAME"] = 9,

}

RemoteArchiveManager.DESERIALIZE_TYPE = {
	
	["STRING"] = 0,
	["NUMBER"] = 1,

}

RemoteArchiveManager.RESULT_CODE = {
	
	["ALREADY_SYNC"] = 0,
	["SUCCESS"] = 1,
	["FAILURE"] = 2,
	["COVER_LOCAL"] = 3,

}

function RemoteArchiveManager:ctor()

	self.cache = {}
	self.in_sync = false --本地存档是否与云端存档完成同步
	self.loaded = false --是否已经成功获取云端存档
	self.cloud_device_imei = nil

end

function RemoteArchiveManager:clear()

	self.cache = {}
	self.in_sync = false
	self.loaded = false
	self.cloud_device_imei = nil

end

function RemoteArchiveManager:getInstance() 

	if not RemoteArchiveManager.instance then

		RemoteArchiveManager.instance = RemoteArchiveManager:create()

	end

	return RemoteArchiveManager.instance

end

-- 获取云端存档数据

function RemoteArchiveManager:load(on_load_finish)

	-- 若已经与云端存档完成同步则直接返回成功
	if self.in_sync == true then
		JavaUtils.log("com.dayu", "已与云端完成同步存档")
		on_load_finish(RemoteArchiveManager.RESULT_CODE.ALREADY_SYNC)
	else
		--菊花等待
		local mask_layer = CommunicatingMask:create()
		cc.Director:getInstance():getRunningScene():addChild(mask_layer, 99999)

	    RPC:call("SHIN/account/loadArchive", {PLAYER_ID,}, function(status, response)

	        if status == RPC.RPC_CALL_SUCCESS then
				-- dump(response)
	        	-- 若不存在云端存档（首次游玩）则自动将本地存档上传云端并返回成功
	        	if not response or response == "" then

	        		self.in_sync = true
	        		self.loaded = true
	        		JavaUtils.log("com.dayu", "不存在云端存档自动将本地存档上传云端")
	        		self:save(function()end)
	        		on_load_finish(RemoteArchiveManager.RESULT_CODE.SUCCESS)

	        	else
		        	local parts = RemoteArchiveManager.deserialize(response, "|", RemoteArchiveManager.DESERIALIZE_TYPE.STRING)

		        	for index, data_str in ipairs(parts) do
		        		if index == 1 then
		        			
		        			local array = SPLIT_STRING(data_str, ",")
		        			self.cloud_device_imei = array[11]

		        		end

		        		self.cache[index] = RemoteArchiveManager.deserialize(data_str, ",", RemoteArchiveManager.DESERIALIZE_TYPE.NUMBER)

		        	end

		        	self.loaded = true
		        	RemoteArchiveManager.CACHE = self.cache

		        	local function on_contrast_finish(result)

		        		if result == 3 then
		        			on_load_finish(RemoteArchiveManager.RESULT_CODE.COVER_LOCAL)
	        			else
	        				on_load_finish(RemoteArchiveManager.RESULT_CODE.SUCCESS)
		        		end

		        	end
		        	self:cloud_contrast_local(on_contrast_finish)

	        	end

	        	mask_layer:removeFromParent()

	        else
	        	JavaUtils.log("com.dayu", "下载存档失败")
	        	mask_layer:removeFromParent()
	        	on_load_finish(RemoteArchiveManager.RESULT_CODE.FAILURE)

	        end

	    end)
	end

end


-- 对比云端数据和本地数据的创建时间
function RemoteArchiveManager:cloud_contrast_local(on_contrast_finish)

	if self.loaded then

		local local_create_timestamp = RemoteArchiveManager.PARSER[RemoteArchiveManager.SECTION.BASE]()[1]
		local cloud_create_timestamp = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.BASE][1] or os.time()

		local account_data = SAFE_REQUIRE_NIL("account_data")
		local local_device_imei = account_data and account_data.device_imei or string.format("%d%d", os.time(), math.random(10000, 100000))
		local cloud_device_imei = self.cloud_device_imei or string.format("%d%d", os.time(), math.random(10000, 100000))
		
		JavaUtils.log("com.dayu", cloud_create_timestamp..","..local_create_timestamp)
		JavaUtils.log("com.dayu", cloud_device_imei..","..local_device_imei)

		if local_create_timestamp < cloud_create_timestamp then
			
			JavaUtils.log("com.dayu", "本地存档上传更新到服务器存档")
			self.in_sync = true
			self:save(function()end)
			on_contrast_finish(RemoteArchiveManager.RESULT_CODE.SUCCESS)
		elseif local_create_timestamp > cloud_create_timestamp then
			
			JavaUtils.log("com.dayu", "服务器存档覆盖本地存档")
			self:apply_to_local()
			on_contrast_finish(RemoteArchiveManager.RESULT_CODE.COVER_LOCAL)
		else
			
			if local_device_imei == cloud_device_imei then
				JavaUtils.log("com.dayu", "device_imei本地存档上传更新到服务器存档")
				self.in_sync = true
				self:save(function()end)
				on_contrast_finish(RemoteArchiveManager.RESULT_CODE.SUCCESS)

			else
				JavaUtils.log("com.dayu", "device_imei服务器存档覆盖本地存档")
				self:apply_to_local()
				on_contrast_finish(RemoteArchiveManager.RESULT_CODE.COVER_LOCAL)

			end

		end

	end

end

-- 将本地存档上传到云端
--
-- 存档管理器为即将上传的存档数据设置了一个缓存（cache）所有需要上传的数据
-- 都会先进入cache当中，在离线状态下调用save函数，虽然数据不会被上传到云端
-- 但是会覆盖缓存中的上一份数据。

function RemoteArchiveManager:save(on_save_finish)

	-- 为了保证云端数据不会被随意篡改，这里只有在确定云端和本地数据完成同步后才会开启上传
	if self.in_sync then

		-- 获取本地存档数据
		self:retrieve_from_local()

		local parts = {}

		for index, archive in ipairs(self.cache) do

			parts[index] = RemoteArchiveManager.serialize(archive, ",")

		end

		local serialized = RemoteArchiveManager.serialize(parts, "|")
		serialized = string.sub(serialized, 0, string.len(serialized)-1)

		RPC:call("SHIN/account/archive", {PLAYER_ID, serialized}, function(status, response)

			if status == RPC.RPC_CALL_SUCCESS then

				on_save_finish(RemoteArchiveManager.RESULT_CODE.SUCCESS)

			else

				JavaUtils.log("com.dayu", "上传失败")
				
			end

	    end)	

	else
		JavaUtils.log("com.dayu", "没完成同步导致上传失败")
		on_save_finish(RemoteArchiveManager.RESULT_CODE.FAILURE)

	end

end

-- 获取被顶存档数据
function RemoteArchiveManager:retrieve_from_local()

	-- 调用该函数后，存档管理器会认为用户已经同意将本地存档文件上传至云端（完成同步）
	self.in_sync = true

	-- 依次将各个数据部分写入上传缓存当中
	for key, index in pairs(RemoteArchiveManager.SECTION) do

		self.cache[index] = RemoteArchiveManager.PARSER[index]()

	end

end

-- 将云端数据应用到本地
--
-- ！！云端数据应用到本地后需要重启游戏
--

function RemoteArchiveManager:apply_to_local()

	-- 只有成功获取云端存档数据后才允覆盖本地存档
	if self.loaded then

		-- 调用该函数后，存档管理器会认为用户已经同意将本地存档文件上传至云端（完成同步）
		self.in_sync = true

		-- 调用REPLACER将数据写入本地存档
		for key, index in pairs(RemoteArchiveManager.SECTION) do
			
			RemoteArchiveManager.REPLACER[index]()

		end
		GameKernel:getInstance():write_player_base_data()
		GameKernel:getInstance():write_player_achv_data()
		GameKernel:getInstance():write_player_book_data()
		GameKernel:getInstance():write_player_mission_data()
		GameKernel:getInstance():write_player_star_reward_data()
		GameKernel:getInstance():write_player_role_data()
		GameKernel:getInstance():write_player_bag_data()
		GameKernel:getInstance():write_player_skin_data()
		GameKernel:getInstance():write_player_weekly_challenge_data()
		GameKernel:getInstance():write_player_section_data()

	end

	-- ！！
	-- ！！这里应该将数据写入本地文件，请添加相应实现
	-- ！！

end

-- 将数组中的数据序列化为字符串的形式，数组中的数据类型必须是字符串或数字，通过delimiter参数指定分割符
RemoteArchiveManager.serialize = function(data_array, delimiter)

	local str = ""

	for _, value in ipairs(data_array) do

		str = string.format("%s%s%s", str, tostring(value), delimiter or ",")

	end

	return str

end

-- 将字符串中的数组反序列化为一个lua数组，通过delimiter参数指定分隔符，data_type指定最终的数据类型
RemoteArchiveManager.deserialize = function(data_string, delimiter, data_type)

	local array = {}

	array = SPLIT_STRING(data_string, delimiter)

	if data_type == RemoteArchiveManager.DESERIALIZE_TYPE.NUMBER then

		for index, data in ipairs(array) do

			array[index] = tonumber(data)

		end

 	end

	return array

end

-- PARSER当中包含了一系列的函数，用于简化本地存档数据，输出为一个数组。
RemoteArchiveManager.PARSER = {

	[RemoteArchiveManager.SECTION.BASE] = function()

		local player_base_data = Player:getInstance():get_player_base_data()

		local parse_result = {}

		if not player_base_data.create_timestamp then
			player_base_data.create_timestamp = os.time()
		end
		parse_result[1] = player_base_data.create_timestamp
		parse_result[2] = player_base_data.player_addition_max_power
		parse_result[3] = player_base_data.player_jewel
		parse_result[4] = player_base_data.player_gold
		parse_result[5] = player_base_data.num_sign
		parse_result[6] = player_base_data.last_sign_day
		parse_result[7] = player_base_data.num_wheel
		parse_result[8] = player_base_data.last_wheel_day
		if player_base_data.today_is_sign == true then
			parse_result[9] = 1
		else
			parse_result[9] = 0
		end
		if player_base_data.today_is_wheel == true then
			parse_result[10] = 1
		else
			parse_result[10] = 0
		end

		local account_data = SAFE_REQUIRE_NIL("account_data")
		if account_data then
			parse_result[11] = account_data.device_imei
		else
			parse_result[11] = 0
		end
		

		return parse_result
		--基本数据
	end, 

	[RemoteArchiveManager.SECTION.ACHIVEMENT] = function()

		local achivement_data = Player:getInstance():get_player_achv_info_vector()
		local parse_result = {}

		for index, achv_info in ipairs(achivement_data) do

			local aquire_index = 0

			for _index, stage_info in ipairs(achv_info.stage_info_vector) do

				if stage_info.is_get then

					aquire_index = _index

				else

					break

				end

			end

			local finish_count = achv_info.finish_count

			parse_result[index * 2 - 1] = aquire_index
			parse_result[index * 2] = finish_count

		end

		return parse_result
		--成就数据 2个为一个成就数据(已领取的阶段(aquire_index前面都为true),完成数量)
	end, 

	[RemoteArchiveManager.SECTION.CARD] = function()

		local card_data = Player:getInstance():get_player_book_info_vector()
		local parse_result = {}

		for index, card_info in ipairs(card_data) do

			if card_info.is_get then

				parse_result[index] = 1

			else

				parse_result[index] = 0

			end

		end

		return parse_result
		--卡片数据 1为true已获取
	end, 

	[RemoteArchiveManager.SECTION.MISSION] = function()

		local mission_data = Player:getInstance():get_player_mission_info_vector()
		local parse_result = {}

		for index, mission_info in ipairs(mission_data) do

			parse_result[index * 3 - 2] = mission_info.max_score
			parse_result[index * 3 - 1] = mission_info.pass_count
			parse_result[index * 3] = mission_info.star_count
		end

		return parse_result
		--关卡数据 3个为一关的数据(分数,通关数,星星数)
	end, 

	[RemoteArchiveManager.SECTION.STAR_REWARD] = function()

		local star_reward_data = Player:getInstance():get_player_star_reward_info_vector()
		local parse_result = {}
		local reward_get_index = 0

		for index, reward_info in ipairs(star_reward_data) do

			if reward_info.is_get then

				reward_get_index = index

			else

				break

			end

		end

		parse_result[1] = reward_get_index

		return parse_result
		--星星奖励 reward_get_index前面(包括自己)都为true
	end, 

	[RemoteArchiveManager.SECTION.ROLE] = function()

		local role_data = Player:getInstance():get_player_role_info_vector()
		local parse_result = {}

		for index, role_info in ipairs(role_data) do

			if not role_info.is_unlock then

				parse_result[index] = 0

			else

				parse_result[index] = role_info.role_level

			end

		end

		return parse_result
		--角色数据 2个为一个角色(已解锁,等级)
	end, 

	[RemoteArchiveManager.SECTION.BAG] = function()

		local bag_data = Player:getInstance():get_player_bag_info_vector()
		local parse_result = {}

		for index = 5, #bag_data do

			parse_result[index - 4] = bag_data[index].item_num

		end

		return parse_result	
		--背包数据 从5(非限时道具)开始,数量
	end, 

	[RemoteArchiveManager.SECTION.SKIN] = function()

		local skin_data = Player:getInstance():get_player_skin_info_vector()
		local parse_result = {}

		for index, skin_info in ipairs(skin_data) do

			if skin_info.is_unlock then

				parse_result[index] = 1

			else

				parse_result[index] = 0

			end

		end

		return parse_result
		--皮肤数据 1为true已解锁
	end, 

	[RemoteArchiveManager.SECTION.WEEK_GAME] = function()

		local weekly_challenge_data = Player:getInstance():get_player_weekly_challenge_info()
		local parse_result = {}

		local today_aquire_index = 0
		local week_aquire_index = 0

		for index, today_reward_info in ipairs(weekly_challenge_data.today_reward_info) do

			if today_reward_info.is_get then

				today_aquire_index = index

			else

				break

			end

		end
		for index, week_reward_info in ipairs(weekly_challenge_data.week_reward_info) do

			if week_reward_info.is_get then

				week_aquire_index = index

			else

				break

			end

		end

		parse_result[1] = today_aquire_index
		parse_result[2] = week_aquire_index
		parse_result[3] = weekly_challenge_data.conch_count
		parse_result[4] = weekly_challenge_data.conch_once_num
		parse_result[5] = weekly_challenge_data.today_challenge_num
		parse_result[6] = weekly_challenge_data.lase_challenge_day
		parse_result[7] = weekly_challenge_data.lase_challenge_week

		return parse_result
		--周赛数据 
	end, 

}

-- REPLACER当中包含了一系列的函数，用于将简化后的云端数据写回本地存档当中。

RemoteArchiveManager.REPLACER = {

	[RemoteArchiveManager.SECTION.BASE] = function()

		local base_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.BASE]
		
		local base_data = Player:getInstance():get_player_base_data()

		base_data.create_timestamp = base_archive[1]
		base_data.player_addition_max_power = base_archive[2]
		base_data.player_jewel = base_archive[3]
		base_data.player_gold = base_archive[4]
		base_data.num_sign = base_archive[5]
		base_data.last_sign_day = base_archive[6]
		base_data.num_wheel = base_archive[7]
		base_data.last_wheel_day = base_archive[8]

		if base_archive[9] == 1 then
			base_data.today_is_sign = true
		else
			base_data.today_is_sign = false
		end

		if base_archive[10] == 1 then
			base_data.today_is_wheel = true
		else
			base_data.today_is_wheel = false
		end

	end, 

	[RemoteArchiveManager.SECTION.ACHIVEMENT] = function()

		local achivement_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.ACHIVEMENT]
		
		local achivement_data = Player:getInstance():get_player_achv_info_vector()

		for index, achivement_info in ipairs(achivement_data) do

			achivement_info.finish_count = achivement_archive[index * 2]

			for i = 1, achivement_archive[index * 2 - 1] do

				achivement_info.stage_info_vector[i].is_get = true

			end

		end	

	end, 

	[RemoteArchiveManager.SECTION.CARD] = function()

		local card_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.CARD]
		
		local card_data = Player:getInstance():get_player_book_info_vector()

		for index, card_info in ipairs(card_data) do

			if card_archive[index] == 1 then

				card_info.is_get = true

			else

				card_info.is_get = false

			end

		end		

	end, 

	[RemoteArchiveManager.SECTION.MISSION] = function()

		local mission_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.MISSION]
		
		local mission_data = Player:getInstance():get_player_mission_info_vector()

		local last_passed = 0

		for index, mission_info in ipairs(mission_data) do

			local max_score = mission_archive[index * 3 - 2]
			local pass_count = mission_archive[index * 3 - 1]
			local star_count = mission_archive[index * 3]

			if pass_count == 0 then

				mission_info.is_enable = false
				mission_info.is_move_action = true
				mission_info.max_score = 0
				mission_info.pass_count = 0
				mission_info.is_pass = false
				mission_info.star_count = 0

			else

				mission_info.is_enable = true
				mission_info.is_move_action = false
				mission_info.max_score = max_score
				mission_info.pass_count = pass_count
				mission_info.star_count = star_count
				mission_info.is_pass = true

				if star_count == 3 then

					mission_info.is_max_star = true

				end

				last_passed = last_passed + 1
			end

		end

		if last_passed < #mission_data and mission_data[last_passed + 1] then

			mission_data[last_passed + 1].is_enable = true

		end

		local current_level = last_passed
    	local mission_info =  Mission_Data:getInstance():get_mission_info_by_level(current_level)
    	if mission_info then
			Player:getInstance():set_player_mission_is_pass(mission_info.mission_id)

			local section_id = mission_info.mission_section
			local player_section_info_vector = Player:getInstance():get_player_section_info_vector()
			for i=1, section_id do
				player_section_info_vector[i].is_enable = true
			end

		end

	end, 

	[RemoteArchiveManager.SECTION.STAR_REWARD] = function()

		local reward_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.STAR_REWARD]
		
		local reward_data = Player:getInstance():get_player_star_reward_info_vector()

		for index = 1, reward_archive[1] do

			reward_data[index].is_get = true

		end

	end, 

	[RemoteArchiveManager.SECTION.ROLE] = function()

		local role_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.ROLE]
		
		local role_data = Player:getInstance():get_player_role_info_vector()

		for index, role_info in ipairs(role_data) do

			if role_archive[index] > 0 then

				role_info.is_unlock = true
				role_info.role_level = role_archive[index]

			else

				role_info.is_unlock = false
				role_info.role_level = 1				

			end

		end

	end, 

	[RemoteArchiveManager.SECTION.BAG] = function()

		local bag_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.BAG]
		
		local bag_data = Player:getInstance():get_player_bag_info_vector()

		for index = 5, #bag_data do

			bag_data[index].item_num = bag_archive[index - 4]

		end

	end, 

	[RemoteArchiveManager.SECTION.SKIN] = function()

		local skin_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.SKIN]
		
		local skin_data = Player:getInstance():get_player_skin_info_vector()

		for index, skin_info in ipairs(skin_data) do

			if skin_archive[index] == 1 then

				skin_info.is_unlock = true

			else

				skin_info.is_unlock = false

			end

		end		

	end, 

	[RemoteArchiveManager.SECTION.WEEK_GAME] = function()

		local weekly_challenge_archive = RemoteArchiveManager.CACHE[RemoteArchiveManager.SECTION.WEEK_GAME]
		
		local weekly_challenge_data = Player:getInstance():get_player_weekly_challenge_info()

		for i = 1, weekly_challenge_archive[1] do

			weekly_challenge_data.today_reward_info[i].is_get = true

		end
		for i = 1, weekly_challenge_archive[2] do

			weekly_challenge_data.week_reward_info[i].is_get = true

		end
		weekly_challenge_data.conch_count = weekly_challenge_archive[3]
		weekly_challenge_data.conch_once_num = weekly_challenge_archive[4]
		weekly_challenge_data.today_challenge_num = weekly_challenge_archive[5]
		weekly_challenge_data.lase_challenge_day = weekly_challenge_archive[6]
		weekly_challenge_data.lase_challenge_week = weekly_challenge_archive[7]

	end, 

}