ResourceManager = class("ResourceManager")

function ResourceManager:ctor()
	self.sprite_res_key = require "script.sprite_res_key"
	self.spine_res_key = require "script.spine_res_key"
	self.particle_res_key = require "script.particle_res_key"
	self.text_string = require "script.text_string"
	self.font_res_key = require "script.font_res_key"
	self.audio_res_key = require "script.audio_res_key"
	self.tips_string = require "script.tips_string"
end

function ResourceManager:getInstance() 
	ResourceManager.instance = ResourceManager.instance or ResourceManager:create()
	return ResourceManager.instance
end

function ResourceManager:getSpriteByKey(sprite_key)

	if self.sprite_res_key[sprite_key] == nil then
		print(string.format("%s not found", sprite_key))
		return nil
	else
		local sprite = cc.Sprite:create(self.sprite_res_key[sprite_key])
		return sprite
	end
end

function ResourceManager:getSpineByKey(spine_key, scale_rate)
	if self.spine_res_key[spine_key] == nil then
		print(string.format("%s not found", spine_key))
		return nil
	else
		local res_path = self.spine_res_key[spine_key]
		local spine = sp.SkeletonAnimation:create(res_path[1], res_path[2], scale_rate)
		return spine
	end
end

function ResourceManager:getParticleByKey(particle_key)
	if self.particle_res_key[particle_key] == nil then
		print(string.format("%s not found", particle_key))
		return nil
	else
		local particle = cc.ParticleSystemQuad:create(self.particle_res_key[particle_key])
		return particle
	end
end

function ResourceManager:getPicturePathByKey(sprite_key)
	if self.sprite_res_key[sprite_key] == nil then
		print(string.format("%s not found", sprite_key))
		return 
	else
		return self.sprite_res_key[sprite_key]
	end
end

function ResourceManager:getTextStringByKey(string_key)
	return self.text_string[string_key]
end

function ResourceManager:getFontPathByKey(font_key)
	return self.font_res_key[font_key]
end

function ResourceManager:getAudioPathByKey(audio_key)
	return self.audio_res_key[audio_key]
end

function ResourceManager:getTipsStringByKey(string_key)
	return self.tips_string[string_key]
end


return ResourceManager