require "core.Daily_Task_Data"

TaskManager = class("TaskManager")

function TaskManager:ctor()
    self:start()
    print("TaskManager:create")

end

function TaskManager:start()
    self:add_daily_task_event()
    self:add_achv_event()
    --self:update_date()
end

function TaskManager:add_daily_task_event()
    local player = Player:getInstance()
    local function handle_daily_task_event(event)
        local param = event.param
        if param.task_id == "task_03" or param.task_id == "task_04" then
            local player_daily_task_info = player:get_player_daily_task_info_by_id(param.task_id)
            if player_daily_task_info.target_type == param.target_type then
                local finish_count = player:get_player_daily_task_finish_count_by_id(param.task_id) + param.finish_count
                player:set_player_daily_task_finish_count_by_id(param.task_id, finish_count)
            end
        else
            local finish_count = player:get_player_daily_task_finish_count_by_id(param.task_id) + param.finish_count
            player:set_player_daily_task_finish_count_by_id(param.task_id, finish_count)
        end
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mission_image_mark_event)
    end 

    local listener = cc.EventListenerCustom:create(NotificationUtil.EventType.daily_task_event, handle_daily_task_event)
    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, cc.Director:getInstance():getNotificationNode())


    -- local event = cc.EventCustom:new(NotificationUtil.EventType.daily_task_event)
    -- event.param = {task_id = "task_01", finish_count = 1, func = function() end }    --target_type = red,green,blue,yellow,pink, bomb,ten,lightning
    -- cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
end

function TaskManager:add_achv_event()
    local player = Player:getInstance()

    local function handle_achv_event(event)
        local param = event.param
        if param.achv_id == "achv_01" then
            player:set_player_achv_finish_count_by_id(param.achv_id, param.finish_count)
        else
            local finish_count = player:get_player_achv_finish_count_by_id(param.achv_id) + param.finish_count
            player:set_player_achv_finish_count_by_id(param.achv_id, finish_count)
        end
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.mission_image_mark_event)
    end 

    local listener = cc.EventListenerCustom:create(NotificationUtil.EventType.achv_event, handle_achv_event)
    cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(listener, cc.Director:getInstance():getNotificationNode())


    -- local event = cc.EventCustom:new(NotificationUtil.EventType.achv_event)
    -- event.param = {achv_id = "achv_01", finish_count = 1, func = function() end }
    -- cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)
end

function TaskManager:update_date()
    local function async_callback(timestamp)
        local player = Player:getInstance()
        local last_daily_task_day = player:get_last_daily_task_day()

        local tm = os.date("*t", timestamp)
        local diff_day = tm.yday - last_daily_task_day
        if tm.yday == 1 and last_daily_task_day ~= 1 then
            diff_day = 1
        end
        if diff_day > 0 then
            player:set_last_daily_task_day(tm.yday)
            --init player_daily_task
            local daily_task_info_vector = Daily_Task_Data:getInstance():get_daily_task_info_vector()
            local player_daily_task_info_vector = {}
            for i=1,#daily_task_info_vector do
                local player_daily_task_info = {
                    task_id = daily_task_info_vector[i].task_id,
                    finish_count = 0,
                    is_get = false,
                    is_choose = false,
                }
                if daily_task_info_vector[i].target_type_info then
                    local target_type_info = daily_task_info_vector[i].target_type_info
                    local index = self:get_random_num(target_type_info)
                    if daily_task_info_vector[i].task_id == "task_12" then
                        if player:get_player_role_info_by_id(target_type_info[index].target_type).is_unlock == false then
                            player_daily_task_info.target_type = "role_101"
                        else
                            player_daily_task_info.target_type = target_type_info[index].target_type
                        end
                    else
                        player_daily_task_info.target_type = target_type_info[index].target_type
                    end
                end
                if i == 1 then
                    local index = self:get_random_num(daily_task_info_vector[1].reward_info)
                    player_daily_task_info.target_type = index
                end
                player_daily_task_info_vector[i] = player_daily_task_info
            end
            player:set_player_daily_task_info_vector(player_daily_task_info_vector)

            --随机选择任务 1,2,3,4必出,{5,6,7,11,12}选2
            local a,b = self:selectNumber(2,{5,6,7,11,12,9,10})
            
            for i=1,4 do
                player:set_player_daily_task_reward_info_is_choose_by_id(daily_task_info_vector[i].task_id, true)
            end
            player:set_player_daily_task_reward_info_is_choose_by_id(daily_task_info_vector[a].task_id, true)
            player:set_player_daily_task_reward_info_is_choose_by_id(daily_task_info_vector[b].task_id, true)

        end
    end
    GameKernel:getInstance():getNetworkTime(async_callback)

end

function TaskManager:get_random_num(table)
    math.randomseed(os.time())
    local random_num = math.random()

    local index = nil
    local count = get_table_length(table)
    for i=1,count do
        if random_num <= table[i].odds then
            index = i
            break
        end
    end
    return index
end

function TaskManager:selectNumber(count,arg)
    local selected={};
    math.random(0,#arg);
    math.randomseed(os.time());
    if #arg<=count then return unpack(arg) end
    while #selected < count do
        math.random(#arg);
        table.insert(selected,table.remove(arg,math.random(#arg)));
    end
    table.sort(selected)
    return unpack(selected);
end

function TaskManager:getInstance()
	TaskManager.instance = TaskManager.instance or TaskManager:create()
	return TaskManager.instance
end

return TaskManager