require "UI.Layer.WarningLayer"
local JSON = require "core.JSON"

TradeManager = class("TradeManager")
 
function TradeManager:ctor()

	TradeManager.__pay = MidasPayment and MidasPayment.pay or TradeManager.__pay

	self.__login_expired_listener = cc.EventListenerCustom:create("LOGIN_EXPIRED", function() self:handle_login_expired() end)
	self.__pay_provide_fail_listener = cc.EventListenerCustom:create("PAY_PROVIDE_FAILURE", function() self:handle_provide_fail() end)
	self.__pay_failure_listener = cc.EventListenerCustom:create("PAY_FAILURE", function() self:handle_failure() end)
	self.__pay_success_listener = cc.EventListenerCustom:create("PAY_SUCCESS", function() self:handle_success() end)
	
	self.__s_pay_result = "NONE"
	self.order_billno = ""

	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__login_expired_listener, cc.Director:getInstance():getNotificationNode())
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__pay_provide_fail_listener, cc.Director:getInstance():getNotificationNode())
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__pay_failure_listener, cc.Director:getInstance():getNotificationNode())
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__pay_success_listener, cc.Director:getInstance():getNotificationNode())

	self.__f_empty_func = function() end
	self.__f_success_callback = self.__f_empty_func
	self.__f_failure_callback = self.__f_empty_func

	self.charge_table = {
		["60钻石"] = "60jewel",
		["180钻石"] = "180jewel",
		["300钻石"] = "300jewel",
		["幸运大礼包"] = "LuckyGift",
		["限时特惠礼包"] = "Springsale",
		["限购礼包"] = "LimitGift",
		["新手礼包"] = "FirstGift",
		["道具礼包"] = "ItemGift",
		["体力礼包"] = "EnergyGift",
		["15000金币"] = "15000gold",
		["45000金币"] = "45000gold",
		["75000金币"] = "75000gold",
		["金币特惠礼包"] = "Special_Gold",
		["钻石特惠礼包"] = "Special_Jewel",
		["复活"] = "Revive",
		["宝藏"] = "Treasure",
		["体力特惠"] = "Special_Energy",
	}

	self.payChannel_table = {
		["-1"] = "未知渠道",
		["0"] = "Q点渠道",
		["1"] = "财付通",
		["2"] = "银行卡快捷支付",
		["3"] = "银行卡快捷支付",
		["4"] = "Q卡渠道",
		["5"] = "手机充值卡渠道",
		["7"] = "元宝渠道",
		["8"] = "微信支付渠道",
		["9"] = "话费渠道",
		["10"] = "金券渠道",
		["11"] = "Q币渠道",
	}

	self.index_table = {
		["60钻石"] = "1",
		["180钻石"] = "2",
		["300钻石"] = "3",	
		["15000金币"] = "4",
		["45000金币"] = "5",
		["75000金币"] = "6",
		["幸运大礼包"] = "7",
		["限时特惠礼包"] = "8",
		["限购礼包"] = "9",
		["新手礼包"] = "10",
		["道具礼包"] = "11",
		["体力礼包"] = "12",
		["金币特惠礼包"] = "13",
		["钻石特惠礼包"] = "14",
		["宝藏礼包"] = "15",
		["复活"] = "16",
		["体力特惠礼包"] = "17",
		["25金币"] = "18",
	}
end

function TradeManager:getInstance()

	if not TradeManager.instance then

		TradeManager.instance = TradeManager:create()

	end

	return TradeManager.instance

end

function TradeManager:noSessionLogin()
	if MidasPayment then
		MidasPayment.noSessionLogin()
	end
end

function TradeManager:showFloatingAdView()
	if MidasPayment then
		MidasPayment.showFloatingAdView()
	end
end

function TradeManager:hideFloatingAdView()
	if MidasPayment then
		MidasPayment.hideFloatingAdView()
	end
end

function TradeManager:showStartAdView()
	if MidasPayment then
		MidasPayment.showStartAdView()
	end
end

function TradeManager:showPauseAdView()
	if MidasPayment then
		MidasPayment.showPauseAdView()
	end
end

-- function TradeManager:raw_pay(description, price, callback_success, callback_failure)
-- 	callback_failure = 	function()
--         local commonPopupLayer = CommonPopupLayer:create()
--         local visible_size = cc.Director:getInstance():getVisibleSize()

--         commonPopupLayer:disable_close()
--         local str1 = "支付失败"
--         local str2 = "支付失败,请重试"
--         local str3 = "确定"
--         commonPopupLayer:showMessage(str1, str2, str3, function()
--             commonPopupLayer:close_self()
--         end, nil)
-- 	end

-- 	return self:start_pay(description, price, callback_success, callback_failure)

-- end

function TradeManager:pay_buy_currency(description, price, callback_success, callback_failure)
	self.re_callback_failure = callback_failure
	callback_failure = 	function()
        local commonPopupLayer = CommonPopupLayer:create()
        local visible_size = cc.Director:getInstance():getVisibleSize()
        cc.Director:getInstance():getEventDispatcher():dispatchCustomEvent(NotificationUtil.EventType.turn_on_touch)
        commonPopupLayer:disable_close()
        local str1 = "支付失败"
        local str2 = "支付失败,请重试"
        local str3 = "确定"
        commonPopupLayer:showMessage(str1, str2, str3, function()
            commonPopupLayer:close_self()
            if self.re_callback_failure then
            	self.re_callback_failure()
        	end
        end, nil)
	end

	return self:start_pay(description, price, callback_success, callback_failure)

end

function TradeManager:__pay(price)
	local event = cc.EventCustom:new("PAY_SUCCESS")--"PAY_FAILURE")
	cc.Director:getInstance():getEventDispatcher():dispatchEvent(event)

end

function TradeManager:handle_login_expired()

	self.__s_pay_result = "LOGIN_EXPIRED"

end

function TradeManager:handle_provide_fail()

	self.__s_pay_result = "PAY_PROVIDE_FAILURE"

end

function TradeManager:handle_failure()

	self.__s_pay_result = "PAY_FAILURE"

end

function TradeManager:handle_success()

	self.__s_pay_result = "PAY_SUCCESS"

end


function TradeManager:add_mask()

	self.__c_mask = MaskLayer:create()
	self.__c_mask:init_background(cc.c4b(0, 0, 0, 120))
	self.__c_mask:setEnabled(true)

	cc.Director:getInstance():getRunningScene():addChild(self.__c_mask, 99999)

end

function TradeManager:remove_mask()

	self.__c_mask:removeFromParent()

end

function TradeManager:start_pay(description, price, callback_success, callback_failure)
	
	if not self.__n_trading and self:check_param(description, price) then

		self:add_mask()
		self.__f_success_callback = callback_success or self.__f_empty_func
		self.__f_failure_callback = callback_failure or self.__f_empty_func
		self.__n_schedule_entry = cc.Director:getInstance():getScheduler():scheduleScriptFunc(function() self:check_result() end, 0.1, false)


		local charge_id = self.charge_table[description]; --平台的计费代码


		local function async_callback()

			self.__pay(price)

		end
		-- luaj = require "cocos.cocos2d.luaj"
		-- -- luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay",tostring(self.charge_table[description]),{tostring(price)}             		
  -- --               	,"(Ljava/lang/String;Ljava/lang/String;)V")
  -- 		luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay",tostring(price),{tostring(price)}             		
  --               	,"(Ljava/lang/String;Ljava/lang/String;)V")
  		self:request_createOrder(description, price, async_callback)
  		-- async_callback()
		return false

	else

		return true

	end

end

function TradeManager:request_createOrder(description, price, async_callback)
    --request 生成订单
    local platform = 1 	-- 1米大师
    local pay_sign = self:get_Md5()		--支付签名

    local is_sandbox = 1 				--支付环境 1沙盒 0正式

 --    if CONFIG_CHANNEL_ID == 0 or CONFIG_CHANNEL_ID == 1 then
 -- 		local charge_tab = "60jewel"	
 -- 		local price = price
	-- else
	-- 	local charge_tab = self.charge_table[description]	
	-- 	local price = price
	-- end
    local rpc_param = {

        PLAYER_ID,
       	self.charge_table[description],
        price,
        price*10,
        platform,
        pay_sign,
        is_sandbox,
    }

    RPC:call("SHIN/sns/createOrder", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then

        	local function on_check_finish()
        		luaj = require "cocos.cocos2d.luaj"
	            self.order_billno = response
	            -- async_callback()
	            luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay", {tostring(self.index_table[description]),tostring("asd")}             		
                	,"(Ljava/lang/String;Ljava/lang/String;)V")

	            -- TalkingDataManager:getInstance():sendOnChargeRequest(self.order_billno, self.charge_table[description], price, 
	            -- 	"CNY", 0, "Midas")
        	end

        	global_check_remote_archive(on_check_finish)

        else --特殊处理 不然报错
        	luaj = require "cocos.cocos2d.luaj"
        	luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay", {tostring(self.index_table[description]),tostring("asd")}             		
         		,"(Ljava/lang/String;Ljava/lang/String;)V")
     --     		if CONFIG_CHANNEL_ID == 0 or CONFIG_CHANNEL_ID == 1 then
     --     			 luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay", {tostring(self.index_table[description]),tostring("asd")}             		
     --            	,"(Ljava/lang/String;Ljava/lang/String;)V")
     --     		else
	    --      		self.__f_failure_callback()
					-- self:end_pay()
					-- print("request error")
     --     		end
         		

        end

    end)
end
-- function TradeManager:request_createOrder(description, price, async_callback)
--     --request 生成订单
-- 	-- local function on_check_finish()
-- 		luaj = require "cocos.cocos2d.luaj"
--         -- self.order_billno = response
--         -- async_callback()
--         luaj.callStaticMethod("org.cocos2dx.lua.AppActivity","game_pay", {tostring(self.index_table[description]),tostring("asd")}             		
--         	,"(Ljava/lang/String;Ljava/lang/String;)V")

--         -- TalkingDataManager:getInstance():sendOnChargeRequest(self.order_billno, self.charge_table[description], price, 
--         -- 	"CNY", 0, "Midas")
-- 	-- end

        
-- end

function TradeManager:get_Md5()
	if MidasPayment then
		return MidasPayment.getMd5()
	end
	return "066075ebc8b946ad4a9474a6c1ee8dbb"	--"066075ebc8b946ad4a9474a6c1ee8dbb"
end

function TradeManager:get_pay_state_info()
	if MidasPayment then
		local info = JSON:decode(MidasPayment.getPayStateInfo())

		if info then

			return info

		end
	end
	return {resultMsg="", resultCode="0", payState="0", provideState="0", realSaveNum="60", extendInfo="0", payChannel="-1"}

end

function TradeManager:request_refreshOrder()

	local info = self:get_pay_state_info()
	local platform = 1 	-- 1米大师

	local pay_sign = self:get_Md5()
	local is_sandbox = 1


    --request 更新订单
    local rpc_param = {

        PLAYER_ID,
        self.order_billno,
        info.payState,
        platform,
        info.payChannel,
        self.payChannel_table[info.payChannel],
        pay_sign,
        is_sandbox,
    }

    RPC:call("SHIN/sns/refreshOrder", rpc_param, function(status, response)

        if status == RPC.RPC_CALL_SUCCESS then
        	-- 订单更新

        else
            print("request error")
        end

    end)
end

function TradeManager:check_result()

	if self.__s_pay_result == "PAY_FAILURE" then

		self.__f_failure_callback()
		self:end_pay()

	elseif self.__s_pay_result == "PAY_SUCCESS" or self.__s_pay_result == "PAY_PROVIDE_FAILURE" then
		--更新订单状态
		-- self:request_refreshOrder()
		TalkingDataManager:getInstance():sendOnChargeSuccess(self.order_billno)

		self.__f_success_callback()
		self:end_pay()

		performWithDelay(cc.Director:getInstance():getRunningScene(),function() 
			RemoteArchiveManager:getInstance():save(function()end)
	    end, 1)

 --  	elseif self.__s_pay_result == "LOGIN_EXPIRED" then
 --  		self.__f_failure_callback()
	-- 	self:end_pay()
 --  		--返回登录界面
 --  		AccountManager:getInstance():logout()
 --        local homeScene = HomeScene:create(1)
 --        cc.Director:getInstance():replaceScene(homeScene)

	 end

end

function TradeManager:check_param(description, price)

	if description and price and type(description) == "string" and type(price) == "number" then

		return true

	else

		return false

	end

end

function TradeManager:end_pay()

	self:remove_mask()

	self.__f_success_callback = self.__f_empty_func
	self.__f_failure_callback = self.__f_empty_func

	self.__n_trading = false

	self.__s_pay_result = "NONE"
	self.order_billno = ""

	cc.Director:getInstance():getScheduler():unscheduleScriptEntry(self.__n_schedule_entry)

end

return TradeManager 