-- 该实现尚未启用。

TradeManagerEx = class("TradeManagerEx")

function TradeManagerEx:ctor()

	self:load_items()
	self.session_id = 0
	self.current_session = 0
	self.trade_session = {}

end

function TradeManagerEx:register_event_listener()

	self.trade_event_listener = cc.EventListenerCustom:create("SIGNAL_SESSION", function(event) self:on_signal_session(event.session, event.signal) end)
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.trade_event_listener, cc.Director:getInstance():getNotificationNode())

end

function TradeManagerEx:load_items()

	self.__t_item_data = require "script.commodity_item"

	for _, item_info in ipairs(self.__t_item_data) do

		self.__t_item_data[item_info.id] = item_info

	end

end

function TradeManagerEx:get_item_info(item_id)

	if item_id then return self.__t_item_data[item_id] end

end

function TradeManagerEx:get_session_id()

	self.session_id = self.session_id + 1

	return 1

end

function TradeManagerEx:get_instance()

	if not TradeManagerEx.instance then


		TradeManagerEx.instance = TradeManagerEx:create()

	end

	return TradeManagerEx.instance

end

function TradeManagerEx:open_session(func)

	local session_id = self:get_session_id()
	local session_coroutine = coroutine.create(func)

	self.trade_session[session_id] = {coroutine = session_coroutine, suspend_on = "START", result = nil}

	return session_id

end

function TradeManagerEx:async_call(func, ...)

	local result, async, event = func(self.current_session, ...)

	if async then

		return coroutine.yield(event)

	else

		return result

	end

end

function TradeManagerEx:resume_session(session_id, signal)

	if not session_id or not self.trade_session[session_id] then

		return "INVALID_SESSION_ID"

	end

	if not coroutine.status(self.trade_session[session_id].coroutine) == "suspended" then

		return "COULD_NOT_RESUME"

	end

	if self.trade_session[session_id].suspend_on ~= "ANY" and ( not signal or self.trade_session[session_id].suspend_on ~= signal.name ) then

		return "SIGNAL_NOT_MATCH"

	end

	self.current_session = session_id

	local result = coroutine.resume(self.trade_session[session_id].coroutine, signal.param)
	local status = coroutine.status(self.trade_session[session_id].coroutine)

	self.current_session = 0

	if status == "dead" then

		self.trade_session[session_id].result = result
		return "SESSION_COMPLETE"

	elseif status == "suspended"

		self.trade_session[session_id].suspend_on = result
		return "SESSION_SUSPENDED"

	end

end

function TradeManagerEx:close_session(session_id)

	if session_id then

		self.trade_session[session_id] = nil

	end

end

function TradeManagerEx:pay_for_item(pay_config)

	local new_session = self:open_session(function() return self:pay_flow(pay_config) end)
	self:resume_session(new_session)

end

function TradeManagerEx:pay_flow(pay_config)

	local item_id = pay_config.item_id
	local item_info = self:get_item_info(item_id)
	local amount = pay_config.amount
	local success_callback = pay_config.success_config
	local failure_callback = pay_config.failure_callback
	local need_confirm = pay_config.need_confirm

	if not self:do_confirm(pay_config) then

		failure_callback(pay_config, "")
		return

	end

	if not self:do_sufficient_check(item_info.cost) then

		failure_callback(pay_config, "")
		return

	end

	if not self:do_transaction(item_info.cost) then

		failure_callback(pay_config, "")
		return

	else

		self:do_dispatch(item_info.content)
		success_callback(pay_config)

		return

	end

end

function TradeManagerEx:do_confirm(pay_config)

	return self:async_call(function() return self:show_confirm(pay_config) end)

end

function TradeManagerEx:do_sufficient_check(cost_info)

	for _, cost in ipairs(cost_info) do

		local strategy = TradeManagerEx.TRADE_STATEGY_KEY_MAP[cost.type]
		local total = cost.price * cost.discount / 100 * amount

		if not self:async_call(function() return strategy.sufficient_check(total) end) then

			return false		

		end

	end

	return true

end

function TradeManagerEx:do_transaction(cost_info)

	for _, cost in ipairs(cost_info) do

		local strategy = TradeManagerEx.TRADE_STATEGY_KEY_MAP[cost.type]
		local total = cost.price * cost.discount / 100 * amount

		if not self:async_call(function() strategy.cost(total) end) then

			return false

		else

			transaction_progress = transaction_progress + 1

		end

	end

	return true

end

function TradeManagerEx:do_dispatch(content_info)

	for _, content in ipairs(content_info) do

		local strategy = TradeManagerEx.DISPATCH_STATEGY_KEY_MAP[content.type]

		strategy.dispatch(content)

	end

end


function TradeManagerEx:show_confirm(pay_config)



end

function TradeManagerEx:on_signal_session(session, signal)

	self:resume_session(session, signal)

end

-- ## ERROR CODE ## --

TradeManagerEx.ERROR_CODE = {
	
	["INVALID_ITEM_ID"] = 1,
	["NOT_ENOUGH"] = 2,

},




-- ## TRADE STRATEGY ## --

TradeManagerEx.TRADE_STATEGY_KEY_MAP = {
	
	["gold"] = TradeManagerEx.TRADE_STRATEGY_GOLD,
	["diamond"] = TradeManagerEx.TRADE_STRATEGY_DIAMOND,
	["money"] = TradeManagerEx.TRADE_STRATEGY_MONEY,

},

TradeManagerEx.TRADE_STRATEGY_GOLD = {
	
	["sufficient_check"] = function(session, amount) end
	["cost"] = function(session, amount) end

},

TradeManagerEx.TRADE_STRATEGY_DIAMOND = {
	
	["sufficient_check"] = function(session, amount) end
	["cost"] = function(session, amount) end

},

TradeManagerEx.TRADE_STRATEGY_MONEY = {
	
	["sufficient_check"] = function(session, amount) end
	["cost"] = function(session, amount) end
	
},




-- ## DISPATCH STRATEGY ## --

TradeManagerEx.DISPATCH_STATEGY_KEY_MAP = {
	
	["entity"] = TradeManagerEx.DISPATCH_STATEGY_ENTITY,
	["event"] = TradeManagerEx.DISPATCH_STATEGY_EVENT,

},

TradeManagerEx.DISPATCH_STATEGY_ENTITY = {
	
	["dispatch"] = function(content) end

},

TradeManagerEx.DISPATCH_STATEGY_EVENT = {
	
	["dispatch"] = function(content) end

},

return TradeManagerEx 