local TutorialInterface = require "UI.Layer.TutorialInterface"

-- 1. 激发开始触发器的事件会进行递归传递，结束触发器也会在递归传递之后进行激发 （方案B：触发开始触发器的事件不会递归传递，结束触发器在递归传递之前激发）

-- 2. revoke采用自底向上的方式进行，而同一级的revoke次序与invoke的次序可能不同 （方案B：同一级的revoke严格按照invoke的顺序进行）

-- 3. enter, exit, 以及nested分组由其宿主引导项的revoke模式进行统一控制，其中enter以及exit会在complete时触发一次revoke，
--    nested会在其子项全部complete以及父项complete时进行revoke，父项成功执行revoke其子项会触发强制revoke (方案B：三个事件组都拥有独立的revoke模式）


local TutorialManagerEx = class("TutorialManager")

TutorialManager = TutorialManagerEx

function TutorialManagerEx:ctor()

	self.__t_tutorial_progress = {}
	self.__t_tutorial_config = {}
	self.__t_tutorial_script = {}

end

function TutorialManagerEx:load_config()

	self.__t_tutorial_config = require "GameEx.Resources.Tutorial.TutorialConfig"
	self.__t_tutorial_script = require "GameEx.Resources.Tutorial.TutorialScript"

	self:preprocess(self.__t_tutorial_config)
print("[Tutorial] -- config loaded !!")
end

function TutorialManagerEx:load_progress()

	if not cc.UserDefault:getInstance():getBoolForKey("tutorial_progress_saved", false) then self:save_progress() end

	self.__t_tutorial_progress = SAFE_REQUIRE "tutorial_progress"

print("[Tutorial] -- progress loaded !!")

end

function TutorialManagerEx:reset()

	self.__t_tutorial_progress = {}
	cc.UserDefault:getInstance():setBoolForKey("tutorial_progress_saved", false)

	package.loaded["tutorial_progress"] = nil
	package.loaded["GameEx.Resources.Tutorial.TutorialConfig"] = nil
	package.loaded["GameEx.Resources.Tutorial.TutorialScript"] = nil

end

function TutorialManagerEx:preprocess(accepter_list, key_chain)
print("[Tutorial] -- preprocess")
	for index, accepter in pairs(accepter_list) do

		local __key_chain = string.format("%s/%s", key_chain or "", index)
print(string.format("[Tutorial] -- key *%s* ", __key_chain))
		accepter.state = "ready"
		accepter.nested = accepter.nested or {}
		accepter.id =  __key_chain

		if accepter.interface_config and accepter.interface_config.indication then

			for index, info in ipairs(accepter.interface_config.indication) do

				info.routine = SPLIT_STRING(info.routine, "/")

			end

		end

		self:preprocess(accepter.nested, __key_chain)

	end

end

function TutorialManagerEx:split_routine(routine_str)

	return SPLIT_STRING(routine_str, "/")

end

function TutorialManagerEx:split_id(id_str)

	return SPLIT_STRING(routine_str, "..")

end

function TutorialManagerEx:strip_completed_tutorial()

	for key, value in pairs(self.__t_tutorial_progress) do

		if value and self.__t_tutorial_config[key] and not (self.__t_tutorial_config[key].state == "started") then

			self.__t_tutorial_config[key] = nil
-- print(string.format("[Tutorial] -- key *%d* stripped !!", key))
		end

	end

end

function TutorialManagerEx:save_progress()

	cc.UserDefault:getInstance():setBoolForKey("tutorial_progress_saved", true)
	FileManipulator:getInstance():writeToFile("tutorial_progress", "tutorial_progress", self.__t_tutorial_progress)	
print("[Tutorial] -- progress saved !!")
end

function TutorialManagerEx:handle_wrapper_event(wrapper_event)
print(string.format("[Tutorial] ----- event *%s* dispatching: ", wrapper_event._event.name))
		self:dispatch_event(wrapper_event._event.name or "INVALID_EVENT", wrapper_event._event.param or {}, self.__t_tutorial_config)
		self:tag_checkpoint()
		self:strip_completed_tutorial()

end

function TutorialManagerEx:register_event_listener()

	function dispatch_event(wrapper_event)

		self:handle_wrapper_event(wrapper_event)

	end 

	self.__c_root_listener = cc.EventListenerCustom:create("TUTORIAL_EVENT", dispatch_event)
	cc.Director:getInstance():getEventDispatcher():addEventListenerWithSceneGraphPriority(self.__c_root_listener, cc.Director:getInstance():getNotificationNode())	
print("[Tutorial] -- event listener registered !!")
end

function TutorialManagerEx:dispatch_event(event, param, accepter_list)

	if param.stopped then return end

	if accepter_list then

		for key, accepter in pairs(accepter_list) do

			if accepter.state == "started" then

				self:dispatch_event(event, param, accepter.nested)

				-- run double check on state (new event could be triggered during recursive dispatching, which might in turn change the accepter's state)

				if accepter.state == "started" and self:evaluate_event(event, param, accepter.event.exit) then

					param.stopped = true
					self:complete_tutorial(accepter)
					print("readyher")
				end

			elseif accepter.state == "ready" then

				if self:evaluate_event(event, param, accepter.event.enter) then

					param.stopped = true
					self:add_hitcher(accepter, param.hitcher_info)
					self:start_tutorial(accepter)

					if accepter.state == "started" then

						self:dispatch_event(event, param, accepter.nested)
						print("finisher")
					end

				end

			end

			if param.stopped then return end

		end

	end

end


function TutorialManagerEx:evaluate_event(event, param, event_config)

	if not (event_config.name == event) then 

		return false 

	end

	for key, value in pairs(event_config.constraint or {}) do

		if key == "tutorial" then

			return self:check_finish(value)

		elseif param[key] ~= value then

			return false

		end

	end

	return true

end

function TutorialManagerEx:check_finish(path_str)

	local path = SPLIT_STRING(path_str, "/")

	local top = 2
	local nested = self.__t_tutorial_config

	while top < #path do

		nested = nested[path[top]] and nested[path[top]].nested or nil

		if not nested then

			return false

		else

			top = top + 1

		end

	end

	local accepter = nested[path[top]]
print(top) print(accepter) print(accepter.state)
	if accepter and (accepter.state == "finished" or accepter.state == "revoked") then

		return true

	else

		return false

	end

end

function TutorialManagerEx:play_voice(voice_key)

	if not voice_key then return end

	MusicManager:getInstance():play_sound(voice_key, 0.3)

end

function TutorialManagerEx:start_tutorial(accepter)
print(string.format("[Tutorial] -- tutorial *%s* started !!", accepter.id or "??"))
	accepter.state = "started"
	self:create_interface(accepter)
	self:call(accepter.event.enter.invoke, accepter)
	self:trigger_event({name = "TUTORIAL_START", param = {id = accepter.id or "??"}})

	self:play_voice(accepter.event.enter.voice)

end

function TutorialManagerEx:complete_tutorial(accepter)
print(string.format("[Tutorial] -- tutorial *%s* finished !!", accepter.id or "??"))
	accepter.state = "finished"
	self:clear_interface(accepter)

	-- it is strongly suggested not to change the order of execution below  
	self:interrupt_dispatch(accepter.nested)
	self:revoke(accepter)
	self:call(accepter.event.exit.invoke, accepter)
	self:trigger_event({name = "TUTORIAL_COMPLETE", param = {id = accepter.id or "??"}})

	self:play_voice(accepter.event.exit.voice)

end

function TutorialManagerEx:trigger_event(event)

	self:dispatch_event(event.name or "INVALID_EVENT", event.param or {}, self.__t_tutorial_config)
	self:tag_checkpoint()
	self:strip_completed_tutorial()

end

function TutorialManagerEx:interrupt_dispatch(accepter_list)

	for index, accepter in pairs(accepter_list) do

		if accepter.state == "started" then

			accepter.state = "interrupted"

		end

		self:interrupt_dispatch(accepter.nested)

	end

end

function TutorialManagerEx:add_hitcher(accepter, hitcher_info)

	if hitcher_info then

		hitcher_info.added = true
		accepter.hitcher_info = hitcher_info

	end

end

function TutorialManagerEx:tag_checkpoint()

	local will_save = false

	for key, accepter in pairs(self.__t_tutorial_config) do

		if (accepter.checkpoint == "CHECKPOINT_START" and accepter.state ~= "ready") or 
			accepter.state == "revoked" then

			if not self.__t_tutorial_progress[key] then
print(string.format("[Tutorial] -- tutorial *%s* reached checkpoint !!", accepter.id))
				self.__t_tutorial_progress[key] = true
				will_save = true

			end

		end

	end

	if will_save then self:save_progress() end

end

function TutorialManagerEx:skip_tutorial(accepter)

	self:revoke(accepter, true)

end

function TutorialManagerEx:revoke(accepter, force_revoke)

	if (accepter.revoke_type ~= "REVOKE_DELAYED" or force_revoke) and accepter.state ~= "ready" and accepter.state ~= "revoked" then
print(string.format("[Tutorial] -- tutorial *%s* enter revoke !!", accepter.id or "??"))
		self:call(accepter.event.enter.revoke, accepter)

		for key, accepter in pairs(accepter.nested) do

			self:revoke(accepter, true)

		end

		if accepter.state == "finished" then self:call(accepter.event.exit.revoke, accepter) end

		accepter.state = "revoked"

	end 

end

function TutorialManagerEx:call(call_info, accepter)

	if call_info and call_info.type == "script" then
print(string.format("[Tutorial] -- call script %s !!", call_info.path))
		func = self.__t_tutorial_script[call_info.path] or function() end

		func(accepter)

	elseif call_info and call_info.type == "hitcher" then
print(string.format("[Tutorial] -- call hitcher !!"))
		if accepter.hitcher_info then

			CALL(accepter.hitcher_info)

		end

	end

end

function TutorialManagerEx:create_interface(accepter)

	if accepter.interface_config then
print(string.format("[Tutorial] -- create interface!!"))
	    local interface = TutorialInterface:create()
	    interface:init_with_config(accepter.interface_config)
	    interface:start()

	    accepter.interface = interface

	end

end

function TutorialManagerEx:clear_interface(accepter)

	if accepter.interface then
print(string.format("[Tutorial] -- clear interface!!"))
		accepter.interface:finish()
		accepter.interface = nil

	end

end


function TutorialManagerEx:start()

	self:load_progress()
	self:load_config()
	self:strip_completed_tutorial()
	-- self:register_event_listener()

end

function TutorialManagerEx:get_instance()

	if not TutorialManagerEx.instance then

		TutorialManagerEx.instance = TutorialManagerEx:create()
		
	end

	return TutorialManagerEx.instance

end

return TutorialManagerEx